@@ -21,10 +21,8 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <math.h>
-#include <dirent.h>
 
-#define GRETLCLI
+#include <dirent.h>
 
 #include "libgretl.h"
 #include "monte_carlo.h"
@@ -35,6 +33,7 @@
 #include "libset.h"
 #include "forecast.h"
 #include "cmd_private.h"
+#include "flow_control.h"
 #include "objstack.h"
 #include "gretl_xml.h"
 #include "gretl_panel.h"
@@ -51,11 +50,6 @@
 # include <unistd.h>
 #endif 
 
-enum {
-    ERRFATAL_AUTO,
-    ERRFATAL_FORCE
-};
-
 #ifdef HAVE_READLINE
 # include <readline/readline.h>
 /* readline functions from complete.c */
@@ -63,14 +57,14 @@ extern char *rl_gets (char **line_read, const char *prompt);
 extern void initialize_readline (void);
 #endif /* HAVE_READLINE */
 
+char datafile[MAXLEN];
 char cmdfile[MAXLEN];
 char outfile[MAXLEN];
 char hdrfile[MAXLEN];
 char syscmd[MAXLEN];
-PATHS paths;                  /* useful paths */
 PRN *cmdprn;
 FILE *fb;
-int errfatal, batch;
+int batch;
 int runit;
 int data_status;
 char linebak[MAXLINE];      /* for storing comments */
@@ -85,7 +79,7 @@ static int saved_object_action (const char *line, double ***pZ,
 
 static void usage (int err)
 {
-    logo();
+    logo(0);
 
     printf(_("\nYou may supply the name of a data file on the command line.\n"
 	     "Options:\n"
@@ -94,7 +88,7 @@ static void usage (int err)
 	     " -h or --help      Print this info and exit.\n"
 	     " -v or --version   Print version info and exit.\n"
 	     " -e or --english   Force use of English rather than translation.\n"
-	     " -q or --basque    Force use of Basque translation.\n"
+	     " -q or --quiet     Print less verbose program information.\n"
 	     "Example of batch mode usage:\n"
 	     " gretlcli -b myfile.inp >myfile.out\n"
 	     "Example of run mode usage:\n"
@@ -107,11 +101,6 @@ static void usage (int err)
     }
 }
 
-static int cli_workdir_callback (const char *s)
-{
-    return set_gretl_work_dir(s, &paths);
-}
-
 static void gretl_abort (char *line)
 {
     fprintf(stderr, _("\ngretlcli: error executing script: halting\n"));
@@ -127,12 +116,16 @@ static void noalloc (void)
 
 static int file_get_line (char *line, CMD *cmd)
 {
+    int len;
+
     clear(line, MAXLINE);
     fgets(line, MAXLINE, fb);
 
+    len = strlen(line);
+
     if (*line == '\0') {
 	strcpy(line, "quit");
-    } else if (line[strlen(line)-1] != '\n') {
+    } else if (len == MAXLINE - 1 && line[len-1] != '\n') {
 	return E_TOOLONG;
     } else {
 	*linebak = 0;
@@ -152,10 +145,9 @@ static int file_get_line (char *line, CMD *cmd)
     return 0;
 }
 
-#ifdef ENABLE_NLS
-
 static void nls_init (void)
 {
+#ifdef ENABLE_NLS
 # ifdef WIN32
     char gretldir[MAXLEN], LOCALEDIR[MAXLEN];
 
@@ -173,16 +165,15 @@ static void nls_init (void)
     putenv("LC_NUMERIC=");
     setlocale(LC_NUMERIC, "");
     reset_local_decpoint();
-}
-
 #endif /* ENABLE_NLS */
+}
 
 static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
 			   MODEL **models)
 {
     int err = 0;
 
-    *paths.datfile = 0;
+    *datafile = 0;
 
     if (pZ != NULL && *pZ != NULL) {
 	err = restore_full_sample(pZ, pdinfo, NULL); 
@@ -216,11 +207,50 @@ static void get_a_filename (char *fname)
     fgets(fname, MAXLEN - 1, stdin);
 }
 
-static int 
-cli_get_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
+static const char *get_prompt (ExecState *s)
+{
+    if (s->flags & DEBUG_EXEC) {
+	return "$ ";
+    } else if (gretl_compiling_function() ||
+	       gretl_compiling_loop()) {
+	return "> ";
+    } else {
+	return "? ";
+    }
+}
+
+/* this function is set up as it is to make it available for debugging
+   purposes */
+
+static int get_interactive_line (void *p)
+{
+    ExecState *s = (ExecState *) p;
+    const char *prompt = get_prompt(s);
+    int err = 0;
+
+#ifdef HAVE_READLINE
+    rl_gets(&line_read, prompt);
+
+    if (line_read == NULL) {
+	strcpy(s->line, "quit");
+    } else if (strlen(line_read) > MAXLINE - 2) {
+	err = E_TOOLONG;
+    } else {
+	*s->line = '\0';
+	strncat(s->line, line_read, MAXLINE - 2);
+	strcat(s->line, "\n");
+    }
+#else
+    printf("%s", prompt);
+    fflush(stdout);
+    file_get_line(s->line, s->cmd); /* note: "file" = stdin here */
+#endif
+
+    return err;
+}
+
+static int cli_get_input_line (ExecState *s)
 {
-    int coding = gretl_compiling_function() || 
-	gretl_compiling_loop();
     int err = 0;
 
     if (runit || batch) {
@@ -228,22 +258,7 @@ cli_get_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	err = file_get_line(s->line, s->cmd);
     } else {
 	/* interactive use */
-#ifdef HAVE_READLINE
-	rl_gets(&line_read, (coding)? "> " : "? ");
-	if (line_read == NULL) {
-	    strcpy(s->line, "quit");
-	} else if (strlen(line_read) > MAXLINE - 2) {
-	    err = E_TOOLONG;
-	} else {
-	    *s->line = '\0';
-	    strncat(s->line, line_read, MAXLINE - 2);
-	    strcat(s->line, "\n");
-	}
-#else
-	printf("%s", (coding)? "> " : "? ");
-	fflush(stdout);
-	file_get_line(s->line, s->cmd); /* note: "file" = stdin here */
-#endif
+	err = get_interactive_line(s);
     }
 
     return err;
@@ -291,27 +306,6 @@ static int maybe_get_input_line_continuation (char *line)
     return err;
 }
 
-static void set_errfatal (int code)
-{
-    static int hoe = -1;
-
-    if (hoe < 0) {
-	hoe = libset_get_bool(HALT_ON_ERR);
-    }
-
-    if (code == ERRFATAL_FORCE) {
-	/* for contexts where continuation on error is
-	   bound to make a big mess */
-	errfatal = 1;
-    } else if (hoe == 0) {
-	/* we've been explicitly told to keep going on error */
-	errfatal = 0;
-    } else {
-	/* default: errors are fatal in batch mode only */
-	errfatal = batch;
-    }
-}
-
 static int xout;
 
 #ifdef HAVE_READLINE
@@ -326,21 +320,23 @@ static int ctrl_x (int count, int key)
 
 int main (int argc, char *argv[])
 {
+#ifdef WIN32
+    char *callname = argv[0];
+#endif
     double **Z = NULL;
     DATAINFO *datainfo = NULL;
     MODEL **models = NULL;
     ExecState state;
     char *line = NULL;
-    int cli_get_data = 0;
+    int quiet = 0;
+    int load_datafile = 1;
     char filearg[MAXLEN];
     char runfile[MAXLEN];
     CMD cmd;
     PRN *prn;
     int err = 0;
 
-#ifdef ENABLE_NLS
     nls_init();
-#endif
 
 #ifdef HAVE_READLINE
     rl_bind_key(0x18, ctrl_x);
@@ -350,56 +346,58 @@ int main (int argc, char *argv[])
     if (datainfo == NULL) {
 	noalloc();
     }
-    
-    if (argc > 1) {
-	int force_lang = 0;
-	int opt = parseopt(&argc, &argv, filearg, &force_lang);
 
-	if (opt & OPT_ERROR) {
-	    usage(1);
-	}
-
-	switch (opt) {
-	case OPT_BATCH:
-	    batch = 1;
-	    if (*filearg == '\0') usage(1);
-	    strcpy(runfile, filearg);
-	    cli_get_data = 1;
-	    break;
-	case OPT_HELP:
-	    usage(0);
-	    break;
-	case OPT_DBOPEN:
-	case OPT_WEBDB:
-	    usage(1);
-	    break;
-	case OPT_VERSION:
-	    logo();
-	    exit(EXIT_SUCCESS);
-	    break;
-	case OPT_RUNIT:
-	    runit = 1;
-	    if (*filearg == '\0') {
-		usage(1);
-	    }
-	    strcpy(runfile, filearg); 
-	    cli_get_data = 1;
-	    break;
-	default:
-	    break;
-	}
-
-#ifdef ENABLE_NLS
-	if (force_lang) {
-	    force_language(force_lang);
-	    if (argc == 2) {
-		cli_get_data = 1;
-	    }
-	}
-#endif
+    if (argc < 2) {
+	load_datafile = 0;
     } else {
-	cli_get_data = 1;
-    }
+	gretlopt opt;
+
+	err = parseopt(&argc, &argv, &opt, filearg);
+
+	if (!err && (opt & (OPT_DBOPEN | OPT_WEBDB))) {
+	    /* catch GUI-only options */
+	    err = E_BADOPT;
+	}
+
+	if (err) {
+	    /* bad option */
+	    usage(1);
+	} else if (opt & (OPT_HELP | OPT_VERSION)) {
+	    /* we'll exit in these cases */
+	    if (opt & OPT_HELP) {
+		usage(0);
+	    } else {
+		logo(0);
+		exit(EXIT_SUCCESS);
+	    }
+	}
+	    
+	if (opt & (OPT_BATCH | OPT_RUNIT)) {
+	    if (*filearg == '\0') {
+		/* we're missing a filename argument */
+		usage(1);
+	    } else {
+		/* record argument (not a datafile) */
+		strcpy(runfile, filearg);
+		load_datafile = 0;
+		if (opt & OPT_BATCH) {
+		    batch = 1;
+		} else {
+		    runit = 1;
+		}
+	    }
+	} else if (*filearg == '\0') {
+	    load_datafile = 0;
+	}
+
+	if (opt & OPT_QUIET) {
+	    quiet = 1;
+	}
+
+	if (opt & OPT_ENGLISH) {
+	    force_language(LANG_C);
+	}
+    } 
 
 #ifdef WIN32
     if (!batch) {
@@ -408,7 +406,8 @@ int main (int argc, char *argv[])
 #endif
 
     libgretl_init();
-    logo(); 
+
+    logo(quiet); 
     session_time(NULL);
 
     prn = gretl_print_new(GRETL_PRINT_STDOUT, &err);
@@ -421,18 +420,14 @@ int main (int argc, char *argv[])
 	noalloc();
     } 
 
-    gretl_set_paths(&paths, OPT_D); /* defaults, not gui */
-
 #ifdef WIN32
-    cli_read_registry(argv[0], &paths);
+    cli_read_registry(callname);
 #else
-    cli_read_rc(&paths);
+    cli_read_rc();
 #endif /* WIN32 */
 
-    set_workdir_callback(cli_workdir_callback);
-
     if (!batch) {
-	strcpy(cmdfile, gretl_work_dir());
+	strcpy(cmdfile, gretl_workdir());
 	strcat(cmdfile, "session.inp");
 	cmdprn = gretl_print_new_with_filename(cmdfile, &err);
 	if (err) {
@@ -441,14 +436,14 @@ int main (int argc, char *argv[])
 	}
     }
 
-    if (!cli_get_data) {
+    if (load_datafile) {
 	char given_file[MAXLEN];
 	int ftype;
 
 	strcpy(given_file, filearg);
-	strcpy(paths.datfile, filearg);
+	strcpy(datafile, filearg);
 
-	ftype = detect_filetype(paths.datfile, &paths);
+	ftype = detect_filetype(datafile);
 
 	switch (ftype) {
 	case GRETL_UNRECOGNIZED:
@@ -457,45 +452,44 @@ int main (int argc, char *argv[])
 	    exit(EXIT_FAILURE);
 	    break;
 	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(paths.datfile, &paths, &Z, datainfo,
-				 OPT_NONE, prn);
+	    err = gretl_get_data(datafile, &Z, datainfo, OPT_NONE, prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(paths.datfile, &paths, &Z, datainfo, 
-				 OPT_NONE, prn);
+	    err = gretl_read_gdt(datafile, &Z, datainfo, OPT_NONE, prn);
 	    break;
 	case GRETL_CSV:
-	    err = import_csv(paths.datfile, &Z, datainfo, OPT_NONE, prn);
+	    err = import_csv(datafile, &Z, datainfo, OPT_NONE, prn);
 	    break;
 	case GRETL_XLS:
 	case GRETL_GNUMERIC:
 	case GRETL_ODS:
-	    err = import_spreadsheet(paths.datfile, ftype, NULL, NULL,
+	    err = import_spreadsheet(datafile, ftype, NULL, NULL,
 				     &Z, datainfo, OPT_NONE, prn);
 	    break;
 	case GRETL_DTA:
 	case GRETL_SAV:
+	case GRETL_SAS:
 	case GRETL_JMULTI:
 	case GRETL_OCTAVE:
 	case GRETL_WF1:
-	    err = import_other(paths.datfile, ftype, &Z, datainfo, 
+	    err = import_other(datafile, ftype, &Z, datainfo, 
 			       OPT_NONE, prn);
 	    break;
 	case GRETL_SCRIPT:
 	    runit = 1;
-	    strcpy(runfile, paths.datfile); 
-	    clear(paths.datfile, MAXLEN);
-	    cli_get_data = 1;
+	    strcpy(runfile, datafile); 
+	    clear(datafile, MAXLEN);
+	    load_datafile = 0;
 	    break;
 	default:
 	    break;
 	}
 
-	if (!cli_get_data) {
+	if (load_datafile) {
 	    if (err) {
 		errmsg(err, prn);
 		if (err == E_FOPEN) {
-		    show_paths(&paths);
+		    show_paths();
 		}
 		return EXIT_FAILURE;
 	    }
@@ -514,6 +508,7 @@ int main (int argc, char *argv[])
 
     gretl_cmd_init(&cmd);
     gretl_exec_state_init(&state, 0, line, &cmd, models, prn);
+    set_debug_read_func(get_interactive_line);
 
     /* print list of variables */
     if (data_status) {
@@ -522,15 +517,15 @@ int main (int argc, char *argv[])
 
     /* check for help file */
     if (!batch) {
-	FILE *fp = fopen(paths.helpfile, "r");
+	const char *hpath = helpfile_path(GRETL_HELPFILE);
+	FILE *fp = fopen(hpath, "r");
 
 	if (fp != NULL) { 
 	    printf(_("\n\"help\" gives a list of commands\n"));
 	    fclose(fp);
 	} else {
-	    printf(_("help file %s is not accessible\n"), 
-		   paths.helpfile);
-	    show_paths(&paths);
+	    printf(_("help file %s is not accessible\n"), hpath);
+	    show_paths();
 	}
     } 
 
@@ -544,7 +539,8 @@ int main (int argc, char *argv[])
     }
 
 #ifdef HAVE_READLINE
-    if (!batch) initialize_readline();
+    /* note: this was conditional on !batch */
+    initialize_readline();
 #endif
 
     if (batch || runit) {
@@ -554,14 +550,11 @@ int main (int argc, char *argv[])
 	exec_line(&state, &Z, datainfo);
     }
 
-    /* should we stop immediately on error, in batch mode? */
-    set_errfatal(ERRFATAL_AUTO);
-
     /* main command loop */
     while (cmd.ci != QUIT && fb != NULL && !xout) {
 	char linecopy[MAXLINE];
 
-	if (err && errfatal) {
+	if (err && gretl_error_is_fatal()) {
 	    gretl_abort(linecopy);
 	}
 
@@ -569,9 +562,8 @@ int main (int argc, char *argv[])
 	    if (gretl_loop_exec(&state, &Z, datainfo)) {
 		return 1;
 	    }
-	    set_errfatal(ERRFATAL_AUTO);
 	} else {
-	    err = cli_get_input_line(&state, &Z, datainfo);
+	    err = cli_get_input_line(&state);
 	    if (err) {
 		errmsg(err, prn);
 		break;
@@ -634,20 +626,24 @@ static void cli_exec_callback (ExecState *s, double ***pZ,
 	maybe_stack_var(s->var, s->cmd);
     } else if (ci == END && !strcmp(s->cmd->param, "restrict")) {
 	maybe_stack_var(s->var, s->cmd);
+    } else if (ci == MODELTAB || ci == GRAPHPG) {
+	pprintf(s->prn, _("%s: command not available\n"), s->cmd->word);
     } 
+
+    /* otherwise, no-op */
 }
 
 static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 			    DATAINFO *pdinfo, MODEL **models,
 			    PRN *prn)
 {
-    char datfile[MAXLEN] = {0};
+    char newfile[MAXLEN] = {0};
     char response[3];
     int ftype, dbdata = 0;
     int err = 0;
 
     if (!(cmd->opt & OPT_O)) {
-	err = getopenfile(line, datfile, &paths, (cmd->opt & OPT_W)?
+	err = getopenfile(line, newfile, (cmd->opt & OPT_W)?
 			  OPT_W : OPT_NONE);
 	if (err) {
 	    errmsg(err, prn);
@@ -660,7 +656,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     } else if (cmd->opt & OPT_O) {
 	ftype = GRETL_ODBC;
     } else {
-	ftype = detect_filetype(datfile, &paths);
+	ftype = detect_filetype(newfile);
     }
 
     dbdata = (ftype == GRETL_NATIVE_DB || ftype == GRETL_NATIVE_DB_WWW ||
@@ -668,7 +664,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 	      ftype == GRETL_ODBC);
 
     if (data_status && !batch && !dbdata && cmd->ci != APPEND &&
-	strcmp(datfile, paths.datfile)) {
+	strcmp(newfile, datafile)) {
 	fprintf(stderr, _("Opening a new data file closes the "
 			  "present one.  Proceed? (y/n) "));
 	fgets(response, sizeof response, stdin);
@@ -683,22 +679,20 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     } 
 
     if (ftype == GRETL_CSV) {
-	err = import_csv(datfile, pZ, pdinfo, cmd->opt, prn);
+	err = import_csv(newfile, pZ, pdinfo, cmd->opt, prn);
     } else if (SPREADSHEET_IMPORT(ftype)) {
-	err = import_spreadsheet(datfile, ftype, cmd->list, cmd->extra,
+	err = import_spreadsheet(newfile, ftype, cmd->list, cmd->extra,
 				 pZ, pdinfo, cmd->opt, prn);
     } else if (OTHER_IMPORT(ftype)) {
-	err = import_other(datfile, ftype, pZ, pdinfo, cmd->opt, prn);
+	err = import_other(newfile, ftype, pZ, pdinfo, cmd->opt, prn);
     } else if (ftype == GRETL_XML_DATA) {
-	err = gretl_read_gdt(datfile, &paths, pZ, pdinfo, 
-			     cmd->opt, prn);
+	err = gretl_read_gdt(newfile, pZ, pdinfo, cmd->opt, prn);
     } else if (ftype == GRETL_ODBC) {
 	err = set_odbc_dsn(line, prn);
     } else if (dbdata) {
-	err = set_db_name(datfile, ftype, &paths, prn);
+	err = set_db_name(newfile, ftype, prn);
     } else {
-	err = gretl_get_data(datfile, &paths, pZ, pdinfo,  
-			     cmd->opt, prn);
+	err = gretl_get_data(newfile, pZ, pdinfo, cmd->opt, prn);
     }
 
     if (err) {
@@ -707,7 +701,7 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     }
 
     if (!dbdata && cmd->ci != APPEND) {
-	strncpy(paths.datfile, datfile, MAXLEN - 1);
+	strncpy(datafile, newfile, MAXLEN - 1);
     }
 
     data_status = 1;
@@ -776,7 +770,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     /* if we're stacking commands for a loop, parse "lightly" */
     if (gretl_compiling_loop()) {
-	err = get_command_index(line, cmd, pdinfo);
+	err = get_command_index(line, cmd);
     } else {
 	err = parse_command_line(line, cmd, pZ, pdinfo);
     }
@@ -828,7 +822,6 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    }
 	    err = gretl_loop_append_line(s, pZ, pdinfo);
 	    if (err) {
-		set_errfatal(ERRFATAL_FORCE);
 		errmsg(err, prn);
 	    } else if (!batch && !runit) {
 		/* echo to record */
@@ -902,7 +895,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	break;
 
     case HELP:
-	cli_help(cmd->param, &paths, cmd->opt, prn);
+	cli_help(cmd->param, cmd->opt, prn);
 	break;
 
     case OPEN:
@@ -954,13 +947,12 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
 	if (*outfile != 0 && *outfile != '\n' && *outfile != '\r' 
 	    && strcmp(outfile, "q")) {
-	    const char *udir = gretl_work_dir();
+	    const char *udir = gretl_workdir();
 
 	    printf(_("writing session output to %s%s\n"), udir, outfile);
 #ifdef WIN32
-	    sprintf(syscmd, "\"%s\\gretlcli\" -b \"%s\" > \"%s%s\"", 
-		    paths.gretldir, cmdfile, udir, outfile);
-	    /* WinExec(syscmd, SW_SHOWMINIMIZED); */
+	    sprintf(syscmd, "\"%sgretlcli\" -b \"%s\" > \"%s%s\"", 
+		    gretl_home(), cmdfile, udir, outfile);
 	    system(syscmd);
 #else
 	    sprintf(syscmd, "gretlcli -b \"%s\" > \"%s%s\"", 
@@ -973,7 +965,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     case RUN:
     case INCLUDE:
-	err = getopenfile(line, runfile, &paths, OPT_S);
+	err = getopenfile(line, runfile, OPT_S);
 	if (err) { 
 	    pputs(prn, _("Command is malformed\n"));
 	    break;
@@ -1002,6 +994,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    fprintf(stderr, _("Couldn't open script \"%s\"\n"), runfile);
 	    fb = pop_input_file();
 	} else {
+	    gretl_set_current_dir(runfile);
 	    strcpy(s->runfile, runfile);
 	    if (libset_get_bool(VERBOSE_INCLUDE)) {
 		pprintf(prn, _("%s opened OK\n"), runfile);
