@@ -62,7 +62,9 @@ struct test_t_ {
 };
 
 struct dist_t_ {
+    int flags;
     GtkWidget *entry[NDISTENTRY];
+    GtkWidget *check;
 };
 
 enum {
@@ -93,6 +95,8 @@ enum {
     NP_RUNS
 };
 
+static void plot_cdf (GtkWidget *parent);
+
 /* functions relating to distribution graphics */
 
 int use_alt_form (int d, double *parms)
@@ -100,8 +104,8 @@ int use_alt_form (int d, double *parms)
     if (d == CHISQ_DIST && parms[0] > 69) {
 	return 1;
     } else if (d == BINOMIAL_DIST) {
-	int n = parms[1];
 	double p = parms[0];
+	int n = parms[1];
 
 	return (n*p > 8 && n*(1-p) > 8);
     } else {
@@ -250,15 +254,16 @@ enum {
 #define F_STDN   "normal(x)=1/(sqrt(2*pi))*exp(-(x)**2/2)"
 #define F_NORM   "normal(x,mu,s)=1/(s*sqrt(2*pi))*exp(-(x-mu)**2/(2*s*s))"
 #define F_BINV   "Binv(p,q)=exp(lgamma(p+q)-lgamma(p)-lgamma(q))"
-#define F_CHI    "chi(x,m)=x**(0.5*m-1.0)*exp(-0.5*x)/gamma(0.5*m)/2**(0.5*m)"
-#define F_ALTCHI "bigchi(x,m)=exp((0.5*m-1.0)*log(x)-0.5*x-lgamma(0.5*m)-df1*0.5*log(2.0))"
-#define F_F      "f(x,m,n)=Binv(0.5*m,0.5*n)*(m/n)**(0.5*m)*" \
+#define F_CHI    "chi(x,m)=x<0?0.0/0.0:x**(0.5*m-1.0)*exp(-0.5*x)/gamma(0.5*m)/2**(0.5*m)"
+#define F_ALTCHI "bigchi(x,m)=x<0?0.0/0.0:exp((0.5*m-1.0)*log(x)-0.5*x-lgamma(0.5*m)-df1*0.5*log(2.0))"
+#define F_F      "f(x,m,n)=x<0?0.0/0.0:Binv(0.5*m,0.5*n)*(m/n)**(0.5*m)*" \
                  "x**(0.5*m-1.0)/(1.0+m/n*x)**(0.5*(m+n))"
 #define F_STUD   "stud(x,m)=Binv(0.5*m,0.5)/sqrt(m)*(1.0+(x*x)/m)**(-0.5*(m+1.0))"
-#define F_POIS   "poisson(z,k)=exp(-z)*(z**k)/(int(k))!"
-#define F_WEIB   "weibull(x,shp,scl)=(shp/scl)*(x/scl)**(shp-1.0)*exp(-(x/scl)**shp)"
+#define F_POIS   "poisson(z,k)=k<0?0.0/0.0:exp(-z)*(z**k)/(int(k))!"
+#define F_WEIB   "weibull(x,shp,scl)=x<0?0.0/0.0:(shp/scl)*(x/scl)**(shp-1.0)*exp(-(x/scl)**shp)"
 #define F_BINOM  "binom(k,n,p)=k<0||k>n?0.0:exp(lgamma(n+1)-lgamma(n-k+1)-lgamma(k+1)" \
                  "+k*log(p)+(n-k)*log(1.0-p))"
+#define F_ALTBIN "bigbin(x,mu,s)=x<0?0.0/0.0:1/(s*sqrt(2*pi))*exp(-(x-mu)**2/(2*s*s))"
 
 static void 
 dist_xmin_xmax (int d, double *parms, double *xmin, double *xmax, int alt)
@@ -410,7 +415,7 @@ range_from_test_stat (int d, double x, double *parms, double *spike,
 	*spike = .25;
 	fprintf(fp, "set xrange [%.3f:%.3f]\n", -x1, x1);
 	fprintf(fp, "set yrange [0:.50]\n");
-	fprintf(fp, "set xlabel '%s'\n", _("Standard errors"));
+	fprintf(fp, "set xlabel \"%s\"\n", _("Standard errors"));
     } else {
 	x1 = dist_xmax(d, parms);
 	if (x > x1) {
@@ -428,6 +433,8 @@ range_from_dist (int d, double *parms, int alt, FILE *fp)
 
     dist_xmin_xmax(d, parms, &tmin, &tmax, alt);
 
+    fprintf(stderr, "range_from_dist: got %g:%g\n", tmin, tmax);
+
     fprintf(fp, "set trange [%g:%g]\n", tmin, tmax);
 
     if (d == NORMAL_DIST) {
@@ -436,15 +443,14 @@ range_from_dist (int d, double *parms, int alt, FILE *fp)
     } else if (d == T_DIST) {
 	fputs("set yrange [0:.50]\n", fp);
     } else if (d == BINOMIAL_DIST || d == POISSON_DIST) {  
-	fprintf(fp, "set samples %d\n", (int) (tmax - tmin + 1));
 	fprintf(fp, "set xtics %d\n", tic_step(tmax - tmin));
     }	
 }
 
 static void
-spec_range_from_dist (int d, double *parms, int alt, GPT_SPEC *spec)
+adjust_range_from_dist (int d, double *parms, int alt, GPT_SPEC *spec)
 {
-    double *t_range = spec->range[3];
+    double *t_range = spec->range[GP_T_RANGE];
     double tmin, tmax;
 
     dist_xmin_xmax(d, parms, &tmin, &tmax, alt);
@@ -457,21 +463,25 @@ spec_range_from_dist (int d, double *parms, int alt, GPT_SPEC *spec)
 
     if (tmax > t_range[1]) {
 	t_range[1] = tmax;
-    }   
+    } 
+
+    if (d == NORMAL_DIST && !na(spec->range[GP_Y_RANGE][1])) {
+	double ymax = normal_pdf_height(parms[1]);
+
+	if (spec->range[GP_Y_RANGE][1] < ymax) {
+	    spec->range[GP_Y_RANGE][1] = 1.05 * ymax;
+	}
+    }
+
+    if (d != NORMAL_DIST && d != T_DIST) {
+	spec->range[GP_Y_RANGE][0] = spec->range[GP_Y_RANGE][1] = NADBL;
+    }
 
     if (d == BINOMIAL_DIST && alt) { 
 	sprintf(spec->xtics, "%d", tic_step(t_range[1] - t_range[0]));
     }
 
-    if (d == BINOMIAL_DIST || d == POISSON_DIST) {
-	int ns = t_range[1] - t_range[0] + 1;
-
-	if (ns > spec->samples) {
-	    spec->samples = ns;
-	}
-    } else if (spec->samples != 0) {
-	spec->samples = 100;
-    }
+    spec->samples = 200;
 }
 
 static void 
@@ -501,15 +511,15 @@ make_plot_line (char *targ, int d, int alt, const int *ids)
 	k = ids[ID_N] + 1;
 	j = ids[ID_P] + 1;
 	if (alt) {
-	    sprintf(targ, "int(t),normal(t+.5,n%d*p%d,sqrt(n%d*p%d*(1-p%d)))", 
+	    sprintf(targ, "int(t),bigbin(int(t)+.5,n%d*p%d,sqrt(n%d*p%d*(1-p%d)))", 
 		    k, j, k, j, j);
 	} else {
-	    sprintf(targ, "int(t),binom(t,n%d,p%d)", k, j);
+	    sprintf(targ, "int(t),binom(int(t),n%d,p%d)", k, j);
 	}
 	break;
     case POISSON_DIST:
 	k = ids[ID_L] + 1;
-	sprintf(targ, "int(t),poisson(lambda%d,t)", k);
+	sprintf(targ, "int(t),poisson(lambda%d,int(t))", k);
 	break;
     case WEIBULL_DIST:
 	k = ids[ID_SHP] + 1;
@@ -584,8 +594,8 @@ static void htest_graph (int d, double x, double *parms)
     }
 
     title = htest_graph_title(d, x, parms);
-    fprintf(fp, " title '%s' w lines , \\\n", title);
-    fprintf(fp, "'-' using 1:2 title '%s' w impulses\n",
+    fprintf(fp, " title \"%s\" w lines , \\\n", title);
+    fprintf(fp, "'-' using 1:2 title \"%s\" w impulses\n",
 	    _("test statistic"));
     fprintf(fp, "%g %g\n", x, spike);
     fputs("e\n", fp);
@@ -654,11 +664,7 @@ static void dist_graph (int d, double *parms)
     case BINOMIAL_DIST:
 	fprintf(fp, "n1=%d\n", (int) parms[1]);
 	fprintf(fp, "p1=%g\n", parms[0]);
-	if (alt) {
-	    fprintf(fp, "%s\n", F_NORM);
-	} else {
-	    fprintf(fp, "%s\n", F_BINOM);
-	}
+	fprintf(fp, "%s\n", (alt)? F_ALTBIN : F_BINOM);
 	break;
     case POISSON_DIST:
 	fprintf(fp, "lambda1=%g\n", parms[0]);
@@ -677,9 +683,9 @@ static void dist_graph (int d, double *parms)
     make_plot_line(pline, d, alt, ids);
 
     if (d == BINOMIAL_DIST || d == POISSON_DIST) {
-	fprintf(fp, "%s title '%s' w linespoints\n", pline, title);
+	fprintf(fp, "%s title \"%s\" w linespoints\n", pline, title);
     } else {
-	fprintf(fp, "%s title '%s' w lines\n", pline, title);
+	fprintf(fp, "%s title \"%s\" w lines\n", pline, title);
     }
 
     g_free(title);
@@ -696,6 +702,7 @@ static void dist_graph (int d, double *parms)
 }
 
 #define ALT_CHI MAX_DIST
+#define ALT_BIN (ALT_CHI + 1)
 
 static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 {
@@ -704,7 +711,7 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
     const char *f1 = NULL, *f2 = NULL;
     char v1[32] = {0};
     char v2[32] = {0};
-    char got[MAX_DIST+1] = {0};
+    char got[MAX_DIST+2] = {0};
     int ids[ID_MAX] = {0};
     double x[2] = {0};
     int i, k, alt = 0;
@@ -724,10 +731,10 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 		    /* no-op: line is already present */
 		    return;
 		}
-		if (prevd == CHISQ_DIST && x[0] > 69) {
+		if (prevd == CHISQ_DIST && use_alt_form(prevd, x)) {
 		    got[ALT_CHI] = 1;
-		} else if (prevd == BINOMIAL_DIST && x[0] > 150) { /* ?? */
-		    got[NORMAL_DIST] = 1;
+		} else if (prevd == BINOMIAL_DIST && use_alt_form(prevd, x)) { 
+		    got[ALT_BIN] = 1;
 		} else {
 		    got[prevd] = 1;
 		}
@@ -770,8 +777,8 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
     /* alternate forms for some plots */
     alt = use_alt_form(d, parms);
 
-    /* maybe adjust t-range */
-    spec_range_from_dist(d, parms, alt, spec);
+    /* maybe adjust plot range */
+    adjust_range_from_dist(d, parms, alt, spec);
 
     /* add a comment line for the new plot */
     title = dist_comment_line(d, parms);
@@ -857,8 +864,8 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 	if (!got[F_DIST]) f2 = F_F;
 	break;
     case BINOMIAL_DIST:
-	if (alt && !got[NORMAL_DIST]) {
-	    f1 = F_NORM;
+	if (alt && !got[ALT_BIN]) {
+	    f1 = F_ALTBIN;
 	} else if (!alt && !got[BINOMIAL_DIST]) {
 	    f1 = F_BINOM;
 	}
@@ -963,6 +970,7 @@ double entry_get_numeric_value (GtkWidget *w, EntryValType t)
     strncat(s, text, 31);
     tailstrip(s);
 
+    /* try for constants: e, pi */
     x = get_real_const(s, t);
     if (!na(x)) {
 	return x;
@@ -1074,36 +1082,6 @@ static void dw_lookup_call (dist_t *tab)
     }
 }
 
-static int page_from_dist (int code, int dist)
-{
-    if (code == CALC_RAND) {
-	return dist;
-    } else if (code == CALC_PVAL) {
-	return dist - 1;
-    } else {
-	switch (dist) {
-	case NORMAL_DIST:
-	    return 0;
-	case T_DIST:
-	    return 1;
-	case CHISQ_DIST:
-	    return 2;
-	case F_DIST:
-	    return 3;
-	case BINOMIAL_DIST:
-	    return 4;
-	case POISSON_DIST:
-	    return 5;
-	case WEIBULL_DIST:
-	    return 6;
-	case DW_DIST:
-	    return 7;
-	}
-    }
-
-    return 0;
-}
-
 static int dist_from_page (int code, int page)
 {
     if (code == CALC_RAND) {
@@ -1371,6 +1349,7 @@ static void get_random (GtkWidget *w, CalcChild *child)
     }
 
     vname = gtk_entry_get_text(GTK_ENTRY(tabs[i]->entry[j]));
+
     if (vname == NULL || *vname == '\0') {
 	warnbox(_("You must give a name for the variable"));
 	return;
@@ -1383,6 +1362,8 @@ static void get_random (GtkWidget *w, CalcChild *child)
 	return;
     }
 
+    gretl_push_c_numeric_locale();
+
     switch (d) {
     case UNIFORM_DIST:
 	gretl_command_sprintf("genr %s = randgen(u,%g,%g)", vname,
@@ -1420,6 +1401,8 @@ static void get_random (GtkWidget *w, CalcChild *child)
 	break;
     }
 
+    gretl_pop_c_numeric_locale();
+
     if (check_and_record_command()) {
 	return;
     }
@@ -1467,28 +1450,25 @@ static void np_test (GtkWidget *w, test_t *test)
     if (test->code == NP_DIFF) {
 	int list[3] = { 2, v1, v2 };
 
-	if (test->extra != NULL &&
-	    GTK_TOGGLE_BUTTON(test->extra)->active) {
+	if (test->extra != NULL && button_is_active(test->extra)) {
 	    opt |= OPT_V;
 	}
 
-	if (GTK_TOGGLE_BUTTON(test->radio[0])->active) {
+	if (button_is_active(test->radio[0])) {
 	    opt |= OPT_G;
-	} else if (GTK_TOGGLE_BUTTON(test->radio[1])->active) {
+	} else if (button_is_active(test->radio[1])) {
 	    opt |= OPT_R;
-	} else if (GTK_TOGGLE_BUTTON(test->radio[2])->active) {
+	} else if (button_is_active(test->radio[2])) {
 	    opt |= OPT_I;
 	}
 
 	err = diff_test(list, (const double **) Z, datainfo, 
 			opt, prn);
     } else if (test->code == NP_RUNS) {
-	if (test->extra != NULL &&
-	    GTK_TOGGLE_BUTTON(test->extra)->active) {
+	if (test->extra != NULL && button_is_active(test->extra)) {
 	    opt |= OPT_D;
 	}
-	if (test->check != NULL &&
-	    GTK_TOGGLE_BUTTON(test->check)->active) {
+	if (test->check != NULL && button_is_active(test->check)) {
 	    opt |= OPT_E;
 	}	
 	err = runs_test(v1, (const double **) Z, datainfo, 
@@ -1516,7 +1496,7 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 	return;
     }
 
-    grf = GTK_TOGGLE_BUTTON(test->extra)->active;
+    grf = button_is_active(test->extra);
 
     switch (test->code) {
 
@@ -1529,7 +1509,7 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 	pprintf(prn, _("Sample mean = %g, std. deviation = %g\n"), 
 		x[0], x[1]);
 
-	if (GTK_TOGGLE_BUTTON(test->check)->active) {
+	if (button_is_active(test->check)) {
 	    pprintf(prn, _("Test statistic: z = (%g - %g)/%g = %g\n"), 
 		    x[0], x[2], se, ts);
 	    pv = normal_pvalue_2(ts);
@@ -1615,7 +1595,7 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 		x[2] - z, x[2] + z);
 	pputc(prn, '\n');
 
-	if (GTK_TOGGLE_BUTTON(test->check)->active) {
+	if (button_is_active(test->check)) {
 	    /* the user specified a common variance */
 	    j = 1;
 	} else if (n1 < 30 || n2 < 30) {
@@ -1860,6 +1840,11 @@ static void dist_graph_callback (GtkWidget *w, CalcChild *child)
     i = gtk_notebook_get_current_page(GTK_NOTEBOOK(child->book));
     d = dist_from_page(child->code, i);
 
+    if (d == NORMAL_DIST && tabs[i]->flags) {
+	plot_cdf(child->dlg);
+	return;
+    }
+
     if (get_dist_entry_vector(child->code, tabs[i], d, x, NULL)) {
 	return;
     }
@@ -1871,6 +1856,43 @@ static void dist_graph_callback (GtkWidget *w, CalcChild *child)
     } 
 }
 
+static void toggle_dist_flag (GtkToggleButton *b, 
+			      dist_t *dist)
+{
+    int i;
+
+    dist->flags = gtk_toggle_button_get_active(b);
+
+    for (i=0; i<NDISTENTRY; i++) {
+	if (dist->entry[i] == NULL) {
+	    break;
+	}
+	gtk_widget_set_sensitive(dist->entry[i], 
+				 dist->flags == 0);
+    }
+}
+
+static void 
+calc_checkbox (GtkWidget *tbl, gint *rows, const gchar *label,
+	       CalcChild *child, int i)
+{
+    dist_t **dist = child->calcp;
+    GtkWidget *w;
+
+    *rows += 1;
+
+    gtk_table_resize(GTK_TABLE(tbl), *rows, 2);
+    w = gtk_check_button_new_with_label(_(label));
+    gtk_table_attach_defaults(GTK_TABLE(tbl), 
+			      w, 0, 1, *rows - 1, *rows);
+    gtk_widget_show(w);
+    dist[i]->check = w;
+    
+    g_signal_connect(G_OBJECT(w), "toggled", 
+		     G_CALLBACK(toggle_dist_flag),
+		     dist[i]);    
+}
+
 static void 
 calc_entry_with_default (GtkWidget *tbl, gint *rows, 
 			 const gchar *label, CalcChild *child,
@@ -1998,6 +2020,10 @@ static void make_dist_tab (CalcChild *child, int i)
 	calc_entry(tbl, &rows, N_("value"), child, i);
     } else if (child->code == CALC_RAND) {
 	calc_entry(tbl, &rows, N_("name"), child, i);
+    } else if (child->code == CALC_GRAPH) {
+	if (d == NORMAL_DIST) {
+	    calc_checkbox(tbl, &rows, N_("CDF instead of density"), child, i);
+	}
     }
 }
 
@@ -2559,7 +2585,7 @@ static void make_nptest_tab (CalcChild *child, int idx)
 
     if (idx == NP_DIFF) {
 	gtk_widget_set_sensitive(tmp, FALSE);
-	desensitize_widget_from_check(test->radio[0], tmp);
+	desensitize_conditional_on(tmp, test->radio[0]);
     }
 }
 
@@ -2721,39 +2747,76 @@ static void gretl_child_destroy (GtkWidget *w, CalcChild *child)
     free(child);
 }
 
+static test_t *test_holder_new (int i)
+{
+    test_t *test = mymalloc(sizeof *test);
+
+    if (test != NULL) {
+	int j;
+
+	test->code = i;
+	test->check = NULL;
+	test->extra = NULL;
+
+	test->combo[0] = test->combo[1] = NULL;
+	test->radio[0] = test->radio[1] = test->radio[2] = NULL;
+
+	for (j=0; j<NTESTENTRY; j++) {
+	    test->entry[j] = NULL;
+	}
+    }
+
+    return test;
+}
+
+static dist_t *dist_holder_new (void)
+{
+    dist_t *dist = mymalloc(sizeof *dist);
+
+    if (dist != NULL) {
+	int j;
+
+	dist->flags = 0;
+	dist->check = NULL;
+
+	for (j=0; j<NDISTENTRY; j++) {
+	    dist->entry[j] = NULL;
+	}
+    }
+
+    return dist;
+}
+
 static int child_allocate_calcp (CalcChild *child)
 {
     int c = child->code;
+    int n = child->n_pages;
     int i, err = 0;
 
     child->calcp = NULL;
 
     if (c == CALC_TEST || c == CALC_NPTEST) {
-	test_t **test;
+	test_t **test = mymalloc(n * sizeof *test);
 
-	test = mymalloc(child->n_pages * sizeof *test);
 	if (test != NULL) {
 	    child->calcp = test;
-	    for (i=0; i<child->n_pages && !err; i++) {
-		test[i] = mymalloc(sizeof **test);
+	    for (i=0; i<n && !err; i++) {
+		test[i] = test_holder_new(i);
 		if (test[i] == NULL) {
 		    err = E_ALLOC;
-		} else {
-		    test[i]->code = i;
-		}
+		} 
 	    }
 	} 
     } else {
-	dist_t **dist;
+	dist_t **dist = mymalloc(n * sizeof *dist);
 
-	dist = mymalloc(child->n_pages * sizeof *dist);
 	if (dist != NULL) {
 	    child->calcp = dist;
-	    for (i=0; i<child->n_pages && !err; i++) {
-		dist[i] = mymalloc(sizeof **dist);
+	    for (i=0; i<n && !err; i++) {
+		dist[i] = dist_holder_new();
 		if (dist[i] == NULL) {
 		    err = E_ALLOC;
-		}
+		} 
 	    }
 	} 
     }
@@ -2867,6 +2930,7 @@ static void switch_child_role (GtkWidget *win, png_plot *plot)
 {
     CalcChild *child;
     GtkNotebook *book;
+    dist_t **dists, *dist;
     int i, d;
 
     child = g_object_get_data(G_OBJECT(win), "gchild");
@@ -2881,12 +2945,12 @@ static void switch_child_role (GtkWidget *win, png_plot *plot)
 
     i = gtk_notebook_get_current_page(book);
     d = dist_from_page(child->code, i);
+    dists = child->calcp;
+    dist = dists[i];
 
-    if (d == T_DIST || d == CHISQ_DIST || d == POISSON_DIST) {
-	dist_t *dist, **dists = child->calcp;
-	int p = page_from_dist(CALC_GRAPH_ADD, d);
-
-	dist = dists[p];
+    if (d == NORMAL_DIST) {
+	gtk_widget_set_sensitive(dist->check, FALSE);
+    } else if (d == T_DIST || d == CHISQ_DIST || d == POISSON_DIST) {
 	gtk_editable_select_region(GTK_EDITABLE(dist->entry[0]), 0, -1);
 	gtk_widget_grab_focus(dist->entry[0]);
     }
@@ -2910,7 +2974,7 @@ static int calc_help_code (int c)
 static void real_stats_calculator (int code, gpointer data) 
 {
     GtkWidget *tmp = NULL;
-    static GtkWidget *winptr[CALC_MAX];
+    static GtkWidget *winptr[CALC_RAND + 1];
     GtkWidget *oldwin;
     CalcChild *child;
     const char *calc_titles[] = {
@@ -2925,6 +2989,7 @@ static void real_stats_calculator (int code, gpointer data)
     int i, hcode, nv = 0;
 
     oldwin = winptr[code];
+
     if (oldwin != NULL) {
 	gtk_window_present(GTK_WINDOW(oldwin));
  	return;
@@ -2983,14 +3048,12 @@ static void real_stats_calculator (int code, gpointer data)
     g_signal_connect(G_OBJECT(tmp), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     child->dlg);
-    gtk_widget_show(tmp);
 
     /* OK button */
     tmp = gtk_button_new_from_stock(GTK_STOCK_OK);
     GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
     gtk_container_add(GTK_CONTAINER(child->bbox), tmp);
     g_signal_connect(G_OBJECT(tmp), "clicked", child->callback, child);
-    gtk_widget_show(tmp);
 
     /* Help button? */
     hcode = calc_help_code(code);
@@ -3003,10 +3066,10 @@ static void real_stats_calculator (int code, gpointer data)
 	g_signal_connect(G_OBJECT(tmp), "clicked", 
 			 G_CALLBACK(context_help), 
 			 GINT_TO_POINTER(hcode));
-	gtk_widget_show(tmp);
     }
 
-    gtk_widget_show(child->dlg);
+    gtk_widget_show_all(child->dlg);
+    add_window_list_item(child->dlg, STAT_TABLE);
 }
 
 /* for gnuplot: convert '^' to '**' for exonentiation */
@@ -3115,7 +3178,7 @@ static void plot_curve (void)
 	plotter.xrange = 10;
     }
 
-    vbox = GTK_DIALOG(dialog)->vbox;
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
 
     /* gnuplot formula entry box */
     hbox = gtk_hbox_new(FALSE, 5);
@@ -3149,18 +3212,148 @@ static void plot_curve (void)
 		     G_CALLBACK(set_double_from_spinner), &plotter.xrange);
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 0); 
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
     
     /* "Cancel" button */
-    cancel_delete_button(GTK_DIALOG(dialog)->action_area, dialog, NULL);
+    cancel_delete_button(hbox, dialog, NULL);
 
     /* "OK" button */
-    button = ok_button(GTK_DIALOG(dialog)->action_area);
+    button = ok_button(hbox);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(do_plot_curve), &plotter);
     gtk_widget_grab_default(button);
 
     /* Help button */
-    context_help_button(GTK_DIALOG(dialog)->action_area, GPT_CURVE);
+    context_help_button(hbox, GPT_CURVE);
+
+    gtk_widget_show_all(dialog);
+}
+
+static void do_plot_cdf (GtkWidget *w, GtkWidget *dlg)
+{
+    const char *formulae[] = {
+	"normcdf(x)=0.5+0.5*erf(x/sqrt(2.0))",
+	"logcdf(x)=1.0/(1+exp(-x))"
+    };
+    const char *titles[] = {
+	N_("normal CDF"),
+	N_("logistic CDF")
+    };
+    FILE *fp = NULL;
+    double xmax = 4.0;
+    int opt, err = 0;
+
+    if (gnuplot_init(PLOT_CURVE, &fp)) { 
+	return;
+    }
+
+    opt = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(dlg), "opt"));
+
+    print_keypos_string(GP_KEY_LEFT_TOP, fp);
+
+    if (opt > 0) {
+	xmax = 6.0;
+    }     
+    
+    gretl_push_c_numeric_locale();
+
+    if (opt == 0 || opt == 1) {
+	fputs("# literal lines = 2\n", fp);
+	fprintf(fp, "%s\n", formulae[opt]);
+	fputs("set zeroaxis\n", fp);
+    } else {
+	fputs("# literal lines = 3\n", fp);
+	fprintf(fp, "%s\n", formulae[0]);
+	fprintf(fp, "%s\n", formulae[1]);
+	fputs("set zeroaxis\n", fp);
+    }
+
+    fprintf(fp, "set xrange [%g:%g]\n", -xmax, xmax);
+    fputs("plot \\\n", fp);
+
+    if (opt == 0) {
+	fprintf(fp, "normcdf(x) title \"%s\" w lines\n", _(titles[opt]));
+    } else if (opt == 1) {
+	fprintf(fp, "logcdf(x) title \"%s\" w lines\n", _(titles[opt]));
+    } else {
+	fprintf(fp, "normcdf(x) title \"%s\" w lines , \\\n", _(titles[0]));
+	fprintf(fp, "logcdf(x) title \"%s\" w lines\n", _(titles[1]));
+    }
+
+    gretl_pop_c_numeric_locale();
+
+    fclose(fp);
+
+    err = gnuplot_make_graph();
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	register_graph();
+    }
+
+    gtk_widget_destroy(dlg);
+}
+
+static void set_cdf_opt (GtkWidget *button, GtkWidget *dlg)
+{
+    gpointer p = g_object_get_data(G_OBJECT(button), "opt");
+
+    g_object_set_data(G_OBJECT(dlg), "opt", p);
+}
+
+static void plot_cdf (GtkWidget *parent)
+{
+    static GtkWidget *dialog;
+    GtkWidget *hbox, *vbox;
+    GtkWidget *button;
+    GSList *group;
+
+    if (dialog) {
+	gtk_window_present(GTK_WINDOW(dialog));
+	return;
+    }
+
+    dialog = gretl_dialog_new(_("gretl: plot CDF"), parent, 0);
+
+    g_signal_connect(G_OBJECT(dialog), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed), &dialog);
+
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+
+    /* normal */
+    button = gtk_radio_button_new_with_label(NULL, _("standard normal"));
+    pack_in_hbox(button, vbox, 0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (button), TRUE);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(set_cdf_opt), dialog);
+
+    /* logistic */
+    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
+    button = gtk_radio_button_new_with_label(group, _("logistic"));
+    g_object_set_data(G_OBJECT(button), "opt", GINT_TO_POINTER(1));
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(set_cdf_opt), dialog);
+    pack_in_hbox(button, vbox, 0);
+
+    /* both */
+    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
+    button = gtk_radio_button_new_with_label(group, _("both CDFs"));
+    g_object_set_data(G_OBJECT(button), "opt", GINT_TO_POINTER(2));
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(set_cdf_opt), dialog);
+    pack_in_hbox(button, vbox, 0);
+
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
+    
+    /* "Cancel" button */
+    cancel_delete_button(hbox, dialog, NULL);
+
+    /* "OK" button */
+    button = ok_button(hbox);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(do_plot_cdf), dialog);
+    gtk_widget_grab_default(button);
 
     gtk_widget_show_all(dialog);
 }
