@@ -34,11 +34,6 @@
 
 #include "boxplots.h"
 
-/* these live in dialogs.c */
-extern GtkWidget *active_edit_id;
-extern GtkWidget *active_edit_name;
-extern GtkWidget *active_edit_text;
-
 static void doubleclick_action (windata_t *vwin)
 {
     switch (vwin->role) {
@@ -66,7 +61,7 @@ static void doubleclick_action (windata_t *vwin)
     case RATS_SERIES:
     case PCGIVE_SERIES:
     case REMOTE_SERIES:
-	db_series_callback(NULL, vwin);
+	display_db_series(vwin);
 	break;
     default:
 	break;
@@ -160,8 +155,6 @@ struct open_data_code {
 struct open_data_code open_data_codes[] = {
     { OPEN_DATA,       "OpenGdt" },
     { APPEND_DATA,     "AppendGdt" },
-    { OPEN_ASCII,      "OpenASCII" },
-    { APPEND_ASCII,    "AppendASCII" },
     { OPEN_CSV,        "OpenCSV" },
     { APPEND_CSV,      "AppendCSV" },
     { OPEN_OCTAVE,     "OpenOctave" },
@@ -176,6 +169,8 @@ struct open_data_code open_data_codes[] = {
     { APPEND_DTA,      "AppendDTA" },
     { OPEN_SAV,        "OpenSAV" },
     { APPEND_SAV,      "AppendSAV" },
+    { OPEN_SAS,        "OpenSAS" },
+    { APPEND_SAS,      "AppendSAS" },
     { OPEN_JMULTI,     "OpenJMulTi" },
     { APPEND_JMULTI,   "AppendJMulTi" },
     { OPEN_ODS,        "OpenODS" },
@@ -209,120 +204,39 @@ void open_data (GtkAction *action)
 
     code = open_data_code(gtk_action_get_name(action));
 
-    switch (code) {
-    case OPEN_DATA:
-    case APPEND_DATA:
-    case OPEN_ASCII:
-    case APPEND_ASCII:
-	file_selector(_("Open data file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_CSV:
-    case APPEND_CSV:
-	if (delimiter_dialog(NULL)) {
+    if (code == OPEN_CSV || code == APPEND_CSV) {
+	if (csv_options_dialog(NULL)) {
 	    return;
 	}
-	file_selector(_("Open CSV file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_OCTAVE:
-    case APPEND_OCTAVE:
-	file_selector(_("Open Octave file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_GNUMERIC:
-    case APPEND_GNUMERIC:
-	file_selector(_("Open Gnumeric file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_XLS:
-    case APPEND_XLS:
-	file_selector(_("Open Excel file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_WF1:
-    case APPEND_WF1:
-	file_selector(_("Open Eviews workfile"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_DTA:
-    case APPEND_DTA:
-	file_selector(_("Open Stata file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_SAV:
-    case APPEND_SAV:
-	file_selector(_("Open SPSS file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_JMULTI:
-    case APPEND_JMULTI:
-	file_selector(_("Open JMulTi file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_ODS:
-    case APPEND_ODS:
-	file_selector(_("Open ODS file"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_MARKERS:
-	file_selector(_("gretl: add markers"), code, FSEL_DATA_NONE, NULL);
-	break;
-    case OPEN_RATS_DB:
-    case OPEN_PCGIVE_DB:
-	file_selector(_("gretl: open database"), code, FSEL_DATA_NONE, NULL);
-	break;
-    default:
-	fprintf(stderr, "open_data: unrecognized action '%s'\n",
-		gtk_action_get_name(action));
-	break;
     }
-}
 
-void open_script (GtkAction *action)
-{
-    const gchar *s = gtk_action_get_name(action);
-
-    if (!strcmp(s, "OpenScript")) {
-	file_selector(_("Open script file"), OPEN_SCRIPT, FSEL_DATA_NONE, NULL);
-    } else if (!strcmp(s, "OpenSession")) {
-	file_selector(_("Open session file"), OPEN_SESSION, FSEL_DATA_NONE, NULL);
-    }
+    file_selector(code, FSEL_DATA_NONE, NULL);
 }
 
 void file_save (windata_t *vwin, int ci)
 {
-    gretlopt opt = OPT_NONE;
-    gpointer p = NULL;
-
     switch (ci) {
     case SAVE_OUTPUT:
-	file_selector(_("Save output file"), ci, FSEL_DATA_VWIN, vwin);
-	break;
     case SAVE_CONSOLE:
-	file_selector(_("Save console output"), ci, FSEL_DATA_VWIN, vwin);
-	break;
     case SAVE_SCRIPT:
-	file_selector(_("Save command script"), ci, FSEL_DATA_VWIN, vwin);
+    case SAVE_GP_CMDS:
+    case SAVE_R_CMDS:
+	file_selector(ci, FSEL_DATA_VWIN, vwin);
 	break;
     case SAVE_DATA:
     case SAVE_DATA_AS:
     case SAVE_DBDATA:
     case SAVE_FUNCTIONS:	
-	data_save_selection_wrapper(ci, NULL);
-	break;
     case EXPORT_CSV:
-	if (delimiter_dialog(&opt)) {
-	    return;
-	}
-	p = GINT_TO_POINTER(opt);
     case EXPORT_R:
     case EXPORT_OCTAVE:
     case EXPORT_DAT:
     case EXPORT_JM:
-	data_save_selection_wrapper(ci, p);
+	data_save_selection_wrapper(ci);
 	break;
     case SAVE_TEX:
-	file_selector(_("Save LaTeX file"), ci, FSEL_DATA_MISC, vwin->data);
-	break;
     case SAVE_TEXT:
-	file_selector(_("Save text"), ci, FSEL_DATA_MISC, vwin->data);
-	break;
-    case SAVE_GP_CMDS:
-	file_selector(_("Save gnuplot commands"), ci, FSEL_DATA_VWIN, vwin);
-	break;
-    case SAVE_R_CMDS:
-	file_selector(_("Save R commands"), ci, FSEL_DATA_VWIN, vwin);
+	file_selector(ci, FSEL_DATA_MISC, vwin->data);
 	break;
     default:
 	dummy_call();
@@ -332,7 +246,7 @@ void file_save (windata_t *vwin, int ci)
 static int fsave_code (const gchar *s)
 {
     if (!strcmp(s, "SaveAsGdt"))
-	return SAVE_DATA;
+	return SAVE_DATA_AS;
     if (!strcmp(s, "SaveAsDb"))
 	return SAVE_DBDATA;
     if (!strcmp(s, "ExportCSV"))
@@ -364,28 +278,6 @@ void dummy_call (void)
     errbox(_("Sorry, this item not yet implemented!"));
 }
 
-void print_report (GtkAction *action)
-{
-    PRN *prn;
-
-    if (bufopen(&prn)) return;
-
-    data_report(datainfo, &paths, prn);
-
-    view_buffer(prn, 77, 400, _("gretl: data summary"), 
-		DATA_REPORT, NULL);
-}
-
-void edit_header (GtkAction *action)
-{
-    if (data_status & BOOK_DATA) {
-	errbox(_("You don't have permission to do this"));
-    } else { 
-	edit_buffer(&datainfo->descrip, 80, 400, _("gretl: edit data info"),
-		    EDIT_HEADER);
-    }
-}
-
 static int model_action_code (GtkAction *action)
 {
     const gchar *s = gtk_action_get_name(action);
@@ -443,7 +335,7 @@ void fit_resid_callback (GtkAction *action, gpointer data)
 	code = M_AHAT;
     }
 
-    add_fit_resid(pmod, code, 0);
+    save_fit_resid(pmod, code);
 }
 
 /* callback for adding a scalar from a model */
@@ -632,6 +524,7 @@ void gretl_callback (GtkAction *action, gpointer data)
     const char *title = NULL;
     const char *query = NULL;
     const char *defstr = NULL;
+    gchar *dynquery = NULL;
     void (*okfunc)() = NULL;
     guint varclick = VARCLICK_NONE;
     int cmd, cancel = 0;
@@ -639,12 +532,6 @@ void gretl_callback (GtkAction *action, gpointer data)
     cmd = gretl_callback_code(gtk_action_get_name(action));
 
     switch (cmd) {
-    case SMPLBOOL:
-	title = N_("gretl: restrict sample");
-	query = N_("Enter boolean condition for selecting cases:");
-	okfunc = do_samplebool;
-	varclick = VARCLICK_INSERT_NAME;
-	break;
     case GENR:
 	title = N_("gretl: add var");
 	query = N_("Enter formula for new variable\n"
@@ -654,7 +541,9 @@ void gretl_callback (GtkAction *action, gpointer data)
 	break;
     case VSETMISS:
 	title = N_("gretl: missing code");
-	query = N_("Enter value to be read as \"missing\":");
+	dynquery = g_strdup_printf(_("Enter value to be read as \"missing\"\n"
+				     "for the variable \"%s\""), 
+				   datainfo->varname[mdata->active_var]);
 	okfunc = do_variable_setmiss;
 	break;
     case GSETMISS:
@@ -714,8 +603,14 @@ void gretl_callback (GtkAction *action, gpointer data)
 	return;
     }
 
-    edit_dialog(_(title), _(query), defstr, okfunc, data, 
-		cmd, varclick, &cancel);   
+    if (dynquery != NULL) {
+	edit_dialog(_(title), dynquery, defstr, okfunc, data, 
+		    cmd, varclick, &cancel);
+	g_free(dynquery);
+    } else {
+	edit_dialog(_(title), _(query), defstr, okfunc, data, 
+		    cmd, varclick, &cancel);
+    }
 }
 
 void revise_nl_model (MODEL *pmod)
@@ -845,8 +740,8 @@ void do_nistcheck (GtkAction *action)
 	return;
     }
 
-    datadir = g_strdup_printf("%sdata%s", paths.gretldir, SLASHSTR);
-    fname = g_strdup_printf("%snist.out", paths.dotdir);
+    datadir = g_strdup_printf("%sdata%s", gretl_home(), SLASHSTR);
+    fname = g_strdup_printf("%snist.out", gretl_dotdir());
 
     (*run_nist_tests)(datadir, fname, nist_verbosity(action));
 
@@ -870,7 +765,7 @@ void send_file (char *fullname)
         return;
     }
     
-    email_file(fullname, paths.dotdir);
+    email_file(fullname, gretl_dotdir());
     close_plugin(handle);
 }
 
