@@ -20,31 +20,31 @@
 #include "gretl.h"
 #include "textutil.h"
 
+#define CLIPDEBUG 0
+
 static gchar *clipboard_buf; 
 
-GtkTargetEntry basic_targets[] = {
+GtkTargetEntry text_targets[] = {
     { "UTF8_STRING",     0, TARGET_UTF8_STRING },
     { "STRING",          0, TARGET_STRING },
     { "TEXT",            0, TARGET_TEXT }, 
     { "COMPOUND_TEXT",   0, TARGET_COMPOUND_TEXT }
 };
 
-GtkTargetEntry full_targets[] = {
-    { "UTF8_STRING",     0, TARGET_UTF8_STRING },
-    { "STRING",          0, TARGET_STRING },
-    { "TEXT",            0, TARGET_TEXT }, 
-    { "COMPOUND_TEXT",   0, TARGET_COMPOUND_TEXT },
-    { "text/rtf",        0, TARGET_RTF },
-    { "application/rtf", 0, TARGET_RTF }
+GtkTargetEntry rtf_targets[] = {
+    { "application/rtf",   0, TARGET_RTF },
+    { "application/x-rtf", 0, TARGET_RTF },
+    { "text/rtf",          0, TARGET_RTF },
+    { "text/richtext",     0, TARGET_RTF },
+    { "STRING",            0, TARGET_STRING },
+    { "TEXT",              0, TARGET_TEXT }
 };
 
-static int n_basic = sizeof basic_targets / sizeof basic_targets[0];
-static int n_full = sizeof full_targets / sizeof full_targets[0];
+static int n_text = sizeof text_targets / sizeof text_targets[0];
+static int n_rtf = sizeof rtf_targets / sizeof rtf_targets[0];
 
 static void gretl_clipboard_set (int copycode);
 
-#define CLIPDEBUG 0
-
 static void gretl_clipboard_free (void)
 {
     free(clipboard_buf);
@@ -70,44 +70,27 @@ int buf_to_clipboard (const char *buf)
     return err;
 }
 
-#define UTF_MINUS(u,i) (u[i] == 0xE2 && u[i+1] == 0x88 && u[i+2] == 0x92)
+#if CLIPDEBUG
 
-static gchar *minus_check (gchar *s)
+static const char *fmt_label (int f)
 {
-    guchar *u = (guchar *) s;
-    gchar *ret = s;
-    int i, n = strlen(s);
-    int got_minus = 0;
-
-    for (i=0; i<n-3; i++) {
-	if (UTF_MINUS(u, i)) {
-	    got_minus = 1;
-	    break;
-	}
+    if (f == TARGET_UTF8_STRING) {
+	return "TARGET_UTF8_STRING";
+    } else if (f == TARGET_STRING) {
+	return "TARGET_STRING";
+    } else if (f == TARGET_TEXT) {
+	return "TARGET_STRING";
+    } else if (f == TARGET_COMPOUND_TEXT) {
+	return "TARGET_COMPOUND_STRING";
+    } else if (f == TARGET_RTF) {
+	return "TARGET_RTF";
+    } else {
+	return "unknown";
     }
-
-    if (got_minus) {
-	/* convert U+2212 into plain ASCII dash */
-	int j = 0;
-
-	ret = calloc(n, 1);
-	if (ret != NULL) {
-	    for (i=0; i<n; i++) {
-		if (i < n - 3 && UTF_MINUS(u, i)) {
-		    ret[j++] = '-';
-		    i += 2;
-		} else {
-		    ret[j++] = u[i];
-		}
-	    }
-	} else {
-	    ret = s;
-	}
-    } 
-
-    return ret;
 }
 
+#endif  
+
 static void gretl_clipboard_get (GtkClipboard *clip,
 				 GtkSelectionData *selection_data,
 				 guint info,
@@ -116,18 +99,8 @@ static void gretl_clipboard_get (GtkClipboard *clip,
     gchar *str = clipboard_buf; /* global */
 
 #if CLIPDEBUG
-    fprintf(stderr, "info = %d\n", (int) info);
-    if (info == TARGET_UTF8_STRING) {
-	fprintf(stderr, " = TARGET_UTF8_STRING\n");
-    } else if (info == TARGET_STRING) {
-	fprintf(stderr, " = TARGET_STRING\n");
-    } else if (info == TARGET_TEXT) {
-	fprintf(stderr, " = TARGET_STRING\n");
-    } else if (info == TARGET_COMPOUND_TEXT) {
-	fprintf(stderr, " = TARGET_COMPOUND_STRING\n");
-    } else if (info == TARGET_RTF) {
-	fprintf(stderr, " = TARGET_RTF\n");
-    }
+    fprintf(stderr, "gretl_clipboard_get: info = %d (%s)\n", 
+	    (int) info, fmt_label(info));
 #endif   
 
     if (str == NULL || *str == '\0') {
@@ -136,22 +109,17 @@ static void gretl_clipboard_get (GtkClipboard *clip,
 
     if (info != TARGET_UTF8_STRING) {
 	/* need to remove any UTF-8 minuses? */
-	str = minus_check(str);
+	str = strip_utf_minus(str);
     }
 
     if (info == TARGET_RTF) {
 	gtk_selection_data_set(selection_data,
 			       GDK_SELECTION_TYPE_STRING,
-			       8 * sizeof(gchar), 
-			       (guchar *) str, 
+			       8, (guchar *) str, 
 			       strlen(str));
     } else {
 	gtk_selection_data_set_text(selection_data, str, -1);
     }
-
-    if (str != clipboard_buf) {
-	g_free(str);
-    }
 }
 
 static void gretl_clipboard_clear (GtkClipboard *clip, gpointer p)
@@ -165,18 +133,34 @@ static void gretl_clipboard_set (int fmt)
     GtkTargetEntry *targs;
     gint n_targs;
 
+#ifdef OSX_BUILD
+    if (fmt == GRETL_FORMAT_RTF || fmt == GRETL_FORMAT_RTF_TXT) {
+	FILE *fp = popen("/usr/bin/pbcopy", "w");
+
+	if (fp != NULL) {
+	    fputs(clipboard_buf, fp);
+	    pclose(fp);
+	    return;
+	}
+    }
+#endif
+
     if (clip == NULL) {
 	clip = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
     }
 
     if (fmt == GRETL_FORMAT_RTF || fmt == GRETL_FORMAT_RTF_TXT) {
-	targs = full_targets;
-	n_targs = n_full;
+	targs = rtf_targets;
+	n_targs = n_rtf;
     } else {
-	targs = basic_targets;
-	n_targs = n_basic;
+	targs = text_targets;
+	n_targs = n_text;
     }
 
+#if CLIPDEBUG
+    fprintf(stderr, "gretl_clipboard_set: fmt = %d\n", fmt);
+#endif
+
     if (!gtk_clipboard_set_with_owner(clip, targs, n_targs,
 				      gretl_clipboard_get,
 				      gretl_clipboard_clear,
@@ -196,7 +180,6 @@ int prn_to_clipboard (PRN *prn, int fmt)
 
     gretl_clipboard_free();
 
-#ifdef ENABLE_NLS
     if (fmt == GRETL_FORMAT_TXT) {
 	/* recode (only) if needed */
 	clipboard_buf = my_locale_from_utf8(buf);
@@ -222,13 +205,6 @@ int prn_to_clipboard (PRN *prn, int fmt)
 	/* TeX, CSV */
 	clipboard_buf = gretl_strdup(buf);
     }
-#else
-    if (fmt == GRETL_FORMAT_RTF_TXT) { 
-	clipboard_buf = dosify_buffer(buf, fmt);
-    } else { 
-	clipboard_buf = gretl_strdup(buf);
-    }
-#endif
 
     if (clipboard_buf == NULL) {
 	err = 1;
