@@ -28,6 +28,7 @@
 #include "menustate.h"
 #include "treeutils.h"
 #include "winstack.h"
+#include "toolbar.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -108,22 +109,6 @@ static void set_time_series (DATAINFO *pdinfo)
     }
 }
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-typedef struct {
-    long frac;
-    short exp;
-} netfloat;
-
-float retrieve_float (netfloat nf)
-{
-    short exp = ntohs(nf.exp);
-    long frac = ntohl(nf.frac);
-    double receive = frac / 10e6;
-    
-    return ldexp(receive, exp);
-}
-#endif
-
 void show_network_error (windata_t *vwin)
 {
     const char *msg = gretl_errmsg_get();
@@ -432,17 +417,17 @@ add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
     }
 }
 
-static void gui_display_series (GtkWidget *w, windata_t *vwin)
+static void db_display_series (GtkWidget *w, windata_t *vwin)
 {
     gui_get_db_series(vwin, DB_DISPLAY);
 }
 
-static void gui_graph_series (GtkWidget *w, windata_t *vwin)
+static void db_graph_series (GtkWidget *w, windata_t *vwin)
 {
     gui_get_db_series(vwin, DB_GRAPH);
 }
 
-static void gui_import_series (GtkWidget *w, windata_t *vwin)
+static void db_import_series (GtkWidget *w, windata_t *vwin)
 {
     gui_get_db_series(vwin, DB_IMPORT);
 }
@@ -452,31 +437,35 @@ void sync_db_windows (void)
     const char *dname = get_db_name();
 
     if (*dname != '\0') {
-	GtkWidget *w = match_window_by_filename(dname);
-	windata_t *vwin = NULL;
+	GtkWidget *w = match_db_window_by_filename(dname);
 
 	if (w != NULL) {
-	    vwin = g_object_get_data(G_OBJECT(w), "object");
-	}
+	    windata_t *vwin = g_object_get_data(G_OBJECT(w), "vwin");
 
-	if (vwin != NULL) {
-	    add_local_db_series_list(vwin);
+	    if (vwin != NULL) {
+		add_local_db_series_list(vwin);
+	    }
 	}
     }
 }
 
-static void gui_delete_series (GtkWidget *w, windata_t *vwin)
+static void db_delete_callback (GtkWidget *w, windata_t *vwin)
 {
     int *list = db_get_selection_list(vwin);
+    gchar *query;
     int resp, err = 0;
 
     if (list == NULL) {
 	return;
     }
 
-    resp = yes_no_dialog ("gretl", 
-			  _("Really delete the selected series?"),
-			  0);
+    query = g_strdup_printf(_("Really delete the selected series\n"
+			      "from the database '%s'?"), 
+			    gtk_window_get_title(GTK_WINDOW(vwin->main)));
+
+    resp = yes_no_dialog ("gretl", query, 0);
+
+    g_free(query);
 
     if (resp == GRETL_YES) { 	
 	err = db_delete_series_by_number(list, vwin->fname);
@@ -662,27 +651,11 @@ static void gui_get_db_series (windata_t *vwin, int cmd)
     dbwrapper_destroy(dw);
 }
 
-static int get_db_command (GtkAction *action)
-{
-    if (action == NULL) {
-	return DB_DISPLAY;
-    } else {
-	const gchar *s = gtk_action_get_name(action);
+/* double-click callback */
 
-	if (!strcmp(s, "SeriesDisplay"))
-	    return DB_DISPLAY;
-	else if (!strcmp(s, "SeriesGraph"))
-	    return DB_GRAPH;
-	if (!strcmp(s, "SeriesImport"))
-	    return DB_IMPORT;
-	else
-	    return DB_DISPLAY;
-    }
-}
-
-void db_series_callback (GtkAction *action, gpointer data)
+void display_db_series (windata_t *vwin)
 {
-    gui_get_db_series((windata_t *) data, get_db_command(action));
+    gui_get_db_series(vwin, DB_DISPLAY);
 } 
 
 static void db_view_codebook (GtkWidget *w, windata_t *vwin)
@@ -695,12 +668,6 @@ static void db_view_codebook (GtkWidget *w, windata_t *vwin)
     view_file(cbname, 0, 0, 78, 350, VIEW_CODEBOOK);
 }
 
-static void 
-book_callback_wrapper (GtkAction *action, gpointer p)
-{
-    db_view_codebook(NULL, p);
-}
-
 static void build_db_popup (windata_t *vwin, int cb, int del)
 {
     if (vwin->popup != NULL) {
@@ -710,24 +677,20 @@ static void build_db_popup (windata_t *vwin, int cb, int del)
     vwin->popup = gtk_menu_new();
 
     add_popup_item(_("Display"), vwin->popup, 
-		   G_CALLBACK(gui_display_series), 
+		   G_CALLBACK(db_display_series), 
 		   vwin);
     add_popup_item(_("Graph"), vwin->popup, 
-		   G_CALLBACK(gui_graph_series), 
+		   G_CALLBACK(db_graph_series), 
 		   vwin);
     add_popup_item(_("Import"), vwin->popup, 
-		   G_CALLBACK(gui_import_series), 
+		   G_CALLBACK(db_import_series), 
 		   vwin);
     if (del) {
 	add_popup_item(_("Delete"), vwin->popup, 
-		       G_CALLBACK(gui_delete_series), 
+		       G_CALLBACK(db_delete_callback), 
 		       vwin);
     }	
 
-    add_popup_item(_("Find..."), vwin->popup, 
-		   G_CALLBACK(listbox_find), 
-		   vwin);
-
     if (cb) {
 	add_popup_item(_("Codebook"), vwin->popup, 
 		       G_CALLBACK(db_view_codebook), 
@@ -735,61 +698,51 @@ static void build_db_popup (windata_t *vwin, int cb, int del)
     }
 }
 
-static void delete_series_callback (GtkAction *action, windata_t *vwin)
-{
-    gui_delete_series(NULL, vwin);
-}
-
-static void close_db_callback (GtkAction *action, windata_t *vwin)
+static void db_close (GtkWidget *w, windata_t *vwin)
 {
     gtk_widget_destroy(vwin->main);
 }
 
-static void set_up_db_menu (windata_t *vwin, int cb, int del)
+enum {
+    DEL_BTN = 1,
+    CB_BTN
+};
+
+static GretlToolItem db_items[] = {
+    { N_("Display values"), GTK_STOCK_OPEN,   G_CALLBACK(db_display_series), 0 },
+    { N_("Graph"),          GRETL_STOCK_TS,   G_CALLBACK(db_graph_series), 0 },
+    { N_("Add to dataset"), GTK_STOCK_ADD,    G_CALLBACK(db_import_series), 0 },
+    { N_("Delete"),         GTK_STOCK_DELETE, G_CALLBACK(db_delete_callback), DEL_BTN },
+    { N_("Codebook"),       GRETL_STOCK_BOOK, G_CALLBACK(db_view_codebook), CB_BTN },
+    { N_("Close"),          GTK_STOCK_CLOSE,  G_CALLBACK(db_close), 0 }
+};
+
+static int n_db_items = G_N_ELEMENTS(db_items);
+
+static void make_db_toolbar (windata_t *vwin, int cb, int del)
 {
-    GtkActionEntry db_items[] = {
-	{ "File", NULL, N_("_File"), NULL, NULL, NULL },
-	{ "FileClose", GTK_STOCK_CLOSE, N_("_Close"), NULL, NULL, G_CALLBACK(close_db_callback) },
-	{ "Series", NULL, N_("_Series"), NULL, NULL, NULL },
-	{ "SeriesDisplay", NULL, N_("_Display"), NULL, NULL, G_CALLBACK(db_series_callback) },
-	{ "SeriesGraph", NULL, N_("_Graph"), NULL, NULL, G_CALLBACK(db_series_callback) },
-	{ "SeriesImport", NULL, N_("_Import"), NULL, NULL, G_CALLBACK(db_series_callback) },
-	{ "SeriesDelete", NULL, N_("_Delete"), NULL, NULL, G_CALLBACK(delete_series_callback) },
-	{ "Find", NULL, N_("_Find"), NULL, NULL, NULL },
-	{ "WindowFind", GTK_STOCK_FIND, N_("_Find in window"), NULL, NULL, G_CALLBACK(listbox_find) },
-    };
-    GtkActionEntry cb_items[] = {
-	{ "Codebook", NULL, N_("_Codebook"), NULL, NULL, NULL },
-	{ "CodebookOpen", GTK_STOCK_OPEN, N_("_Open"), NULL, NULL, G_CALLBACK(book_callback_wrapper) },
-    };
-    const gchar *db_ui = 
-	"<ui>"
-	"  <menubar name='MenuBar'>"
-	"    <menu action='File'>"
-	"      <menuitem action='FileClose'/>"
-	"    </menu>"
-	"    <menu action='Series'>"
-	"      <menuitem action='SeriesDisplay'/>"
-	"      <menuitem action='SeriesGraph'/>"
-	"      <menuitem action='SeriesImport'/>"
-	"      <menuitem action='SeriesDelete'/>"
-	"    </menu>"
-	"    <menu action='Find'>"
-	"      <menuitem action='WindowFind'/>"
-	"    </menu>"
-	"  </menubar>"
-	"</ui>";
+    GtkWidget *hbox;
+    GretlToolItem *item;
+    int i;
 
-    vwin_add_ui(vwin, db_items, G_N_ELEMENTS(db_items), db_ui);
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
 
-    if (cb) {
-	vwin_menu_add_menu(vwin, "/MenuBar", &cb_items[0]);
-	vwin_menu_add_item(vwin, "/MenuBar/Codebook", &cb_items[1]);
+    vwin->mbar = gretl_toolbar_new();
+
+    for (i=0; i<n_db_items; i++) {
+	item = &db_items[i];
+	if (!del && item->flag == DEL_BTN) {
+	    continue;
+	} if (!cb && item->flag == CB_BTN) {
+	    continue;
+	}
+	gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, -1);
     }
 
-    if (!del) {
-	flip(vwin->ui, "/MenuBar/Series/SeriesDelete", FALSE);
-    }
+    gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+    vwin_add_finder(vwin);
+    gtk_widget_show_all(hbox);
 }
 
 static void destroy_db_win (GtkWidget *w, windata_t *vwin)
@@ -876,31 +829,18 @@ maybe_adjust_descrip_column (windata_t *vwin)
     }
 }
 
-static void db_select_first_series (windata_t *vwin)
-{
-    GtkTreeView *view = GTK_TREE_VIEW(vwin->listbox);
-    GtkTreeSelection *selection;
-    GtkTreeIter iter;
-    GtkListStore *store;
-
-    store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-    selection = gtk_tree_view_get_selection(view);
-    gtk_tree_selection_select_iter(selection, &iter);
-}
-
 static int 
 make_db_series_window (int action, char *fname, char *buf)
 {
-    GtkWidget *listbox;
-    GtkWidget *w, *main_vbox;
+    GtkWidget *w, *listbox;
     gchar *title;
     windata_t *vwin;
     int db_width = 700, db_height = 420;
     int cb = 0, del = 0;
     int record, err = 0;
 
-    w = match_window_by_filename(fname);
+    w = match_db_window_by_filename(fname);
+
     if (w != NULL) {
 	gtk_window_present(GTK_WINDOW(w));
 	return 0;
@@ -937,29 +877,26 @@ make_db_series_window (int action, char *fname, char *buf)
     }
 
     strcpy(vwin->fname, fname);
-    
-    /* set up grids */
-    main_vbox = gtk_vbox_new(FALSE, 5);
-    gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 10);
-    gtk_container_add(GTK_CONTAINER(vwin->main), main_vbox);
+
+    vwin->vbox = gtk_vbox_new(FALSE, 1);
+    gtk_box_set_spacing(GTK_BOX(vwin->vbox), 4);
+    gtk_container_set_border_width(GTK_CONTAINER(vwin->vbox), 4);
+    gtk_container_add(GTK_CONTAINER(vwin->main), vwin->vbox);
 
     cb = db_has_codebook(fname);
     del = db_is_writable(action, fname);
 
-    set_up_db_menu(vwin, cb, del);
+    make_db_toolbar(vwin, cb, del);
     build_db_popup(vwin, cb, del);
 
-    gtk_box_pack_start(GTK_BOX(main_vbox), vwin->mbar, FALSE, TRUE, 0);
-    gtk_widget_show(vwin->mbar);
-
     listbox = database_window(vwin);
-    gtk_box_pack_start(GTK_BOX(main_vbox), listbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), listbox, TRUE, TRUE, 0);
 
     if (action == REMOTE_SERIES) {
 	GtkWidget *hbox; 
 
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
 	vwin->status = gtk_label_new(_("Network status: OK"));
 	gtk_label_set_justify(GTK_LABEL(vwin->status), GTK_JUSTIFY_LEFT);
 	gtk_box_pack_start(GTK_BOX(hbox), vwin->status, FALSE, FALSE, 0);
@@ -980,7 +917,7 @@ make_db_series_window (int action, char *fname, char *buf)
     } else {
 	gtk_widget_show_all(vwin->main); 
 	maybe_adjust_descrip_column(vwin);
-	db_select_first_series(vwin);
+	listbox_select_first(vwin);
     }
 
     return err;
@@ -1580,8 +1517,6 @@ void open_named_remote_db_index (char *dbname)
     free(getbuf);
 }
 
-#define KEEP_BROWSER_OPEN 1
-
 void open_db_index (GtkWidget *w, gpointer data)
 {
     gchar *fname = NULL, *dbdir = NULL;
@@ -1605,12 +1540,6 @@ void open_db_index (GtkWidget *w, gpointer data)
     g_free(dbdir);
 
     make_db_series_window(action, dbfile, NULL); 
-
-#ifndef KEEP_BROWSER_OPEN
-    if (vwin != NULL && vwin->main != NULL && GTK_IS_WIDGET(vwin->main)) {
-	gtk_widget_destroy(GTK_WIDGET(vwin->main));
-    }
-#endif
 }
 
 void open_remote_db_index (GtkWidget *w, gpointer data)
@@ -1839,6 +1768,11 @@ enum {
     TMP_INSTALL
 };
 
+/* try to find a suitable path, for which the user has write
+   permission, for installing a database, collection of
+   data files, or function package 
+*/
+
 static char *get_writable_target (int code, int op, char *objname)
 {
     const char *ext;
@@ -1860,15 +1794,15 @@ static char *get_writable_target (int code, int op, char *objname)
     }
 
     if (code == REMOTE_DB) {
-	build_path(targ, paths.binbase, objname, ext);
+	build_path(targ, gretl_binbase(), objname, ext);
     } else if (code == REMOTE_DATA_PKGS) {
-	    char pkgdir[MAXLEN];
+	char pkgdir[MAXLEN];
 
-	    get_default_dir(pkgdir, SAVE_DATA_PKG);
-	    build_path(targ, pkgdir, objname, ext);
+	get_default_dir(pkgdir, SAVE_DATA_PKG);
+	build_path(targ, pkgdir, objname, ext);
     } else {
 	if (op == TMP_INSTALL) {
-	    build_path(targ, paths.dotdir, "dltmp", NULL);
+	    build_path(targ, gretl_dotdir(), "dltmp", NULL);
 	    err = gretl_tempname(targ);
 	} else {
 	    char pkgdir[MAXLEN];
@@ -1888,7 +1822,7 @@ static char *get_writable_target (int code, int op, char *objname)
 	if (fp == NULL) {
 	    if (errno == EACCES && code != REMOTE_FUNC_FILES) { 
 		/* write to user dir instead? */
-		build_path(targ, paths.workdir, objname, ext);
+		build_path(targ, gretl_workdir(), objname, ext);
 	    } else {
 		file_write_errbox(targ);
 		free(targ);
@@ -1938,10 +1872,10 @@ static int unpack_book_data (const char *fname)
    that is being installed onto the local machine.
 */
 
-static int real_install_file_from_server (windata_t *vwin, int op)
+void install_file_from_server (GtkWidget *w, windata_t *vwin)
 {
     gchar *objname = NULL;
-    char *target = NULL;
+    char *path = NULL;
     int err = 0;
 
     if (vwin->role == REMOTE_DB) {
@@ -1961,24 +1895,24 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 
     if (objname == NULL || *objname == '\0') {
 	g_free(objname);
-	return 1;
+	return;
     }    
 
-    target = get_writable_target(vwin->role, op, objname);
-    if (target == NULL) {
+    path = get_writable_target(vwin->role, REAL_INSTALL, objname);
+    if (path == NULL) {
 	g_free(objname);
-	return 1;
+	return;
     }
 
     if (vwin->role == REMOTE_FUNC_FILES) {
-	err = retrieve_remote_function_package(objname, target);
+	err = retrieve_remote_function_package(objname, path);
     } else if (vwin->role == REMOTE_DATA_PKGS) {
-	err = retrieve_remote_datafiles_package(objname, target);
+	err = retrieve_remote_datafiles_package(objname, path);
     } else {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-	err = retrieve_remote_db(objname, target, GRAB_NBO_DATA);
+	err = retrieve_remote_db(objname, path, GRAB_NBO_DATA);
 #else
-	err = retrieve_remote_db(objname, target, GRAB_DATA);
+	err = retrieve_remote_db(objname, path, GRAB_DATA);
 #endif
     }
 
@@ -1988,28 +1922,24 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	windata_t *local = get_local_viewer(vwin->role);
 
 	if (vwin->role == REMOTE_FUNC_FILES) {
-	    if (op == REAL_INSTALL) {
-		list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
-				      vwin->active_var, 3,
-				      _("Up to date"));
-		if (local != NULL) {
-		    populate_filelist(local, NULL);
-		} 
-	    } else {
-		gui_show_function_info(target, VIEW_FUNC_INFO);
-	    }
+	    list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
+				  vwin->active_var, 3,
+				  _("Up to date"));
+	    if (local != NULL) {
+		populate_filelist(local, NULL);
+	    } 
 	} else if (vwin->role == REMOTE_DATA_PKGS) {
-	    fprintf(stderr, "downloaded '%s'\n", target);
-	    err = unpack_book_data(target);
-	    remove(target);
+	    fprintf(stderr, "downloaded '%s'\n", path);
+	    err = unpack_book_data(path);
+	    remove(path);
 	    if (err) {
 		errbox(_("Error unzipping compressed data"));
 	    } else {
 		infobox("Restart gretl to access this database");
 	    }
 	} else {
-	    fprintf(stderr, "downloaded '%s'\n", target);
-	    err = ggz_extract(target);
+	    fprintf(stderr, "downloaded '%s'\n", path);
+	    err = ggz_extract(path);
 	    if (err) {
 		if (err != E_FOPEN) {
 		    errbox(_("Error unzipping compressed data"));
@@ -2021,30 +1951,41 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 		if (local != NULL) {
 		    populate_filelist(local, NULL);
 		} else {
-		    offer_db_open(target);
+		    offer_db_open(path);
 		}
 	    }
 	}
     }
 
     g_free(objname);
-    free(target);
-
-    return err;
+    free(path);
 }
 
-void install_file_from_server (GtkWidget *w, gpointer data)
+void pkg_info_from_server (GtkWidget *w, windata_t *vwin)
 {
-    windata_t *vwin = (windata_t *) data;
+    gchar *objname = NULL;
+    char *path = NULL;
+    int err = 0;
 
-    real_install_file_from_server(vwin, REAL_INSTALL);
-}
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
+			 vwin->active_var, 0, &objname);
 
-void file_info_from_server (GtkWidget *w, gpointer data)
-{
-    windata_t *vwin = (windata_t *) data;
+    path = get_writable_target(vwin->role, TMP_INSTALL, objname);
+    if (path == NULL) {
+	g_free(objname);
+	return;
+    }
 
-    real_install_file_from_server(vwin, TMP_INSTALL);
+    err = retrieve_remote_function_package(objname, path);
+
+    if (err) {
+	show_network_error(NULL);
+    } else {
+	display_function_package_data(objname, path, VIEW_PKG_INFO);
+    }
+
+    g_free(objname);
+    free(path);
 }
 
 static gchar *
@@ -2186,12 +2127,12 @@ static void get_local_object_status (const char *fname, int role,
     int err;
 
     if (role == REMOTE_DB) {
-	build_path(fullname, paths.binbase, fname, NULL);
+	build_path(fullname, gretl_binbase(), fname, NULL);
     } else if (role == REMOTE_DATA_PKGS) {
-	build_path(filedir, paths.gretldir, "data", NULL);
+	build_path(filedir, gretl_home(), "data", NULL);
 	build_path(fullname, filedir, fname, NULL);
     } else {
-	build_path(filedir, paths.gretldir, "functions", NULL);
+	build_path(filedir, gretl_home(), "functions", NULL);
 	build_path(fullname, filedir, fname, NULL);
     }
 
@@ -2202,18 +2143,18 @@ static void get_local_object_status (const char *fname, int role,
 #else
 	    /* try user dir too, if not on Windows */
 	    if (role == REMOTE_DB) {
-		build_path(fullname, paths.workdir, fname, NULL);
+		build_path(fullname, gretl_workdir(), fname, NULL);
 	    } else if (role == REMOTE_DATA_PKGS) {
-		build_path(fullname, paths.workdir, fname, NULL);
+		build_path(fullname, gretl_workdir(), fname, NULL);
 	    } else {
-		build_path(filedir, paths.workdir, "functions", NULL);
+		build_path(filedir, gretl_workdir(), "functions", NULL);
 		build_path(fullname, filedir, fname, NULL);
 	    }
 	    err = gretl_stat(fullname, &fbuf);
 	    if (err == -1) {
 		if (role == REMOTE_FUNC_FILES || role == REMOTE_DATA_PKGS) {
 		    /* try default working dir */
-		    char *tmp = gretl_default_workdir(&paths);
+		    char *tmp = gretl_default_workdir();
 
 		    if (tmp != NULL) {
 			if (role == REMOTE_FUNC_FILES) {
@@ -2305,7 +2246,7 @@ static int read_remote_filetime (char *line, char *fname, time_t *date,
     return 0;
 }
 
-/* below: mechanism for tucking individual databases under a twisty,
+/* below: mechanism for tucking individual databases under a 'twisty',
    when there's a large number of databases from a single source.
 */
 
@@ -2677,7 +2618,7 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
     return err;
 }
 
-static DIR *open_gretl_db_dir (char *dbdir)
+static DIR *open_gretl_db_dir (const char *dbdir)
 {
     DIR *dir = NULL;
     int tries = 0;
@@ -2690,7 +2631,7 @@ static DIR *open_gretl_db_dir (char *dbdir)
 #endif
 	if (dir == NULL) {
 	    errbox(_("Can't open folder %s"), dbdir);
-	    if (++tries == 2 || options_dialog(TAB_DBS, "binbase")) {
+	    if (++tries == 2 || options_dialog(TAB_DBS, "binbase", mdata->main)) {
 		/* canceled */
 		return NULL;
 	    }
@@ -2704,7 +2645,7 @@ gint populate_dbfilelist (windata_t *vwin)
 {
     GtkListStore *store;
     GtkTreeIter iter;
-    char *dbdir;
+    const char *dbdir;
     DIR *dir = NULL;
     int ndb = 0;
 
@@ -2713,7 +2654,7 @@ gint populate_dbfilelist (windata_t *vwin)
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     /* pick up any databases in the shared gretl dir */
-    dbdir = paths.binbase;
+    dbdir = gretl_binbase();
     dir = open_gretl_db_dir(dbdir);
     if (dir != NULL) {
 	ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
@@ -2721,7 +2662,7 @@ gint populate_dbfilelist (windata_t *vwin)
     }
 
     /* pick up any databases in the user's personal dir */
-    dbdir = paths.workdir;
+    dbdir = gretl_workdir();
 #ifdef G_OS_WIN32 
     dir = win32_opendir(dbdir);
 #else
@@ -2732,6 +2673,20 @@ gint populate_dbfilelist (windata_t *vwin)
 	closedir(dir);
     }
 
+    /* and in the default working dir? */
+    dbdir = gretl_default_workdir();
+    if (dbdir != NULL) {
+#ifdef G_OS_WIN32 
+	dir = win32_opendir(dbdir);
+#else
+	dir = opendir(dbdir);
+#endif
+	if (dir != NULL) {
+	    ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
+	    closedir(dir);
+	}
+    }
+
     if (ndb == 0) {
 	errbox(_("No database files found"));
     }
@@ -2739,6 +2694,26 @@ gint populate_dbfilelist (windata_t *vwin)
     return (ndb == 0);
 }
 
+void set_db_dir_callback (windata_t *vwin, char *path)
+{
+    DIR *dir = opendir(path);
+    int ndb = 0;
+
+    if (dir != NULL) {
+	GtkListStore *store;
+	GtkTreeIter iter;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	ndb = read_db_files_in_dir(dir, vwin->role, path, store, &iter);
+    }
+
+    if (ndb == 0) {
+	warnbox(_("No database files found"));
+    }
+}
+
 static void set_compact_info_from_default (int method)
 {
     int i;
