@@ -56,9 +56,7 @@ void menu_exit_check (void);
 
 double gui_double_from_string (const char *str, int *err);
 
-int delimiter_dialog (gretlopt *optp);
-
-void varinfo_dialog (int varnum, int full);
+int csv_options_dialog (gretlopt *optp);
 
 void rand_seed_dialog (void);
 
@@ -73,6 +71,8 @@ int select_var_from_list (const int *list, const char *query);
 
 void sample_range_dialog (GtkAction *action, gpointer p);
 
+void sample_restrict_dialog (GtkAction *action, gpointer p);
+
 void panel_structure_dialog (DATAINFO *pdinfo);
 
 void data_compact_dialog (GtkWidget *w, int spd, int *target_pd, 
@@ -108,6 +108,10 @@ int checks_dialog (const char *title, const char *blurb,
 		   int *spinvar, const char *spintxt, 
 		   int spinmin, int spinmax, int helpcode);
 
+int checks_only_dialog (const char *title, const char *blurb,
+			const char **opts, int nopts,
+			int *active, int hcode);
+
 int spin_dialog (const char *title, const char *blurb,
 		 int *spinvar, const char *spintxt, 
 		 int spinmin, int spinmax, int helpcode);
@@ -129,7 +133,10 @@ void dialog_add_confidence_selector (GtkWidget *dlg, double *conf);
 
 int freq_dialog (const char *title, const char *blurb,
 		 int *nbins, int nbmax, double *f0, double *fwid,
-		 double xmin, double xmax, int *dist, int plot);
+		 double xmin, double xmax, int *dist, int *plot);
+
+int model_table_dialog (int *colhead_opt, int *se_opt, int *pv_opt,
+			int *ast_opt, int *figs);
 
 void bootstrap_dialog (windata_t *vwin, int *pp, int *pB,
 		       gretlopt *popt, int *cancelled);
@@ -138,6 +145,9 @@ void lmax_dialog (double *lmax, double ymax);
 
 int chow_dialog (int tmin, int tmax, int *t, int *dumv);
 
+void iter_control_dialog (int *optim, int *pmaxit, double *ptol, 
+			  int *plmem, int *cancel, GtkWidget *parent);
+
 void tex_format_dialog (void);
 
 int gretl_all_done (void);
