@@ -24,7 +24,9 @@
 #include "menustate.h"
 #include "dlgutils.h"
 
+#include "libset.h"
 #include "system.h"
+#include "gretl_bfgs.h"
 
 void set_window_busy (windata_t *vwin)
 {
@@ -77,25 +79,33 @@ void vbox_add_hsep (GtkWidget *vbox)
     gtk_widget_show(h);
 }
 
+void pack_in_hbox (GtkWidget *w, GtkWidget *vbox, int vspace)
+{
+    GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
+
+    gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, vspace);
+}
+
 /* Various buttons, usable in several sorts of dialogs */
 
 GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode)
 {
-    GtkWidget *w;
+    GtkWidget *button;
 
-    w = gtk_button_new_from_stock(GTK_STOCK_HELP);
-    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
-    gtk_container_add(GTK_CONTAINER(hbox), w);
+    button = gtk_button_new_from_stock(GTK_STOCK_HELP);
+    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_container_add(GTK_CONTAINER(hbox), button);
     gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
-				       w, TRUE);
+				       button, TRUE);
+
     if (cmdcode >= 0) {
-	g_signal_connect(G_OBJECT(w), "clicked", 
+	g_signal_connect(G_OBJECT(button), "clicked", 
 			 G_CALLBACK(context_help), 
 			 GINT_TO_POINTER(cmdcode));
     }
-    gtk_widget_show(w);
 
-    return w;
+    return button;
 }
 
 static void set_canceled (GtkWidget *w, int *c)
@@ -121,6 +131,7 @@ GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
     button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+
     if (canceled != NULL) {
 	g_signal_connect(G_OBJECT(button), "clicked", 
 			 G_CALLBACK(set_canceled), 
@@ -129,12 +140,11 @@ GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
 			 G_CALLBACK(maybe_set_canceled), 
 			 canceled);
     }
+
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     targ);
 	
-    gtk_widget_show(button);
-
     return button;
 }
 
@@ -155,8 +165,9 @@ static void maybe_invalidate_opt (GtkDialog *d, int resp, int *opt)
 GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 				  int *opt)
 {
-    GtkWidget *button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    GtkWidget *button;
 
+    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 
@@ -168,10 +179,10 @@ GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 			 G_CALLBACK(maybe_invalidate_opt), 
 			 opt);
     }
+
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     targ);
-    gtk_widget_show(button);
 
     return button;
 }
@@ -209,6 +220,17 @@ GtkWidget *cancel_button (GtkWidget *hbox)
     return w;
 }
 
+GtkWidget *close_button (GtkWidget *hbox)
+{
+    GtkWidget *w;
+
+    w = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+    GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
+    gtk_container_add(GTK_CONTAINER(hbox), w);
+
+    return w;
+}
+
 GtkWidget *next_button (GtkWidget *hbox)
 {
     GtkWidget *w;
@@ -233,13 +255,13 @@ GtkWidget *back_button (GtkWidget *hbox)
 
 static void set_dialog_border_widths (GtkWidget *dlg)
 {
+    GtkWidget *box = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
     int w1 = 10, w2 = 5;
 
-    gtk_container_set_border_width(GTK_CONTAINER 
-				   (GTK_DIALOG(dlg)->vbox), w1);
-    gtk_container_set_border_width(GTK_CONTAINER 
-				   (GTK_DIALOG(dlg)->action_area), w2);
-    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dlg)->vbox), w2);
+    gtk_container_set_border_width(GTK_CONTAINER(box), w1);
+    gtk_box_set_spacing(GTK_BOX(box), w2);
+    box = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
+    gtk_container_set_border_width(GTK_CONTAINER(box), w2);
 }
 
 static void gretl_dialog_set_resizeable (GtkWidget *w, gboolean s)
@@ -331,28 +353,38 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
     return d;
 }
 
-/* make the sensitivity of widget @w positively dependent on the state
-   of check button @check */
-
-void sensitize_widget_from_check (GtkWidget *check, GtkWidget *w)
+static void sensitize_widget (GtkWidget *b, GtkWidget *w)
 {
-    gboolean s;
+    gboolean s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b));
 
-    s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
     gtk_widget_set_sensitive(w, s);
 }
 
-/* make the sensitivity of widget @w negatively dependent on the state
-   of check button @check */
+/* make the sensitivity of widget @w positively dependent on the state
+   of button @b */
 
-void desensitize_widget_from_check (GtkWidget *check, GtkWidget *w)
+void sensitize_conditional_on (GtkWidget *w, GtkWidget *b)
 {
-    gboolean s;
+    g_signal_connect(G_OBJECT(b), "toggled",
+		     G_CALLBACK(sensitize_widget), w);	
+}
+
+static void desensitize_widget (GtkWidget *b, GtkWidget *w)
+{
+    gboolean s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b));
 
-    s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
     gtk_widget_set_sensitive(w, !s);
 }
 
+/* make the sensitivity of widget @w negatively dependent on the state
+   of button @b */
+
+void desensitize_conditional_on (GtkWidget *w, GtkWidget *b)
+{
+    g_signal_connect(G_OBJECT(b), "toggled",
+		     G_CALLBACK(desensitize_widget), w);	
+}
+
 void set_double_from_spinner (GtkSpinButton *b, double *x)
 {
     *x = gtk_spin_button_get_value(b);
@@ -425,9 +457,29 @@ GtkWidget *gretl_option_check_button_switched (const char *label,
 
 /* "edit dialog" apparatus */
 
-GtkWidget *active_edit_id;
-GtkWidget *active_edit_name;
-GtkWidget *active_edit_text;
+static GtkWidget *active_edit_id;
+static GtkWidget *active_edit_name;
+static GtkWidget *active_edit_text;
+
+GtkWidget *get_active_edit_id (void)
+{
+    return active_edit_id;
+}
+
+GtkWidget *get_active_edit_name (void)
+{
+    return active_edit_name;
+}
+
+GtkWidget *get_active_edit_text (void)
+{
+    return active_edit_text;
+}
+
+void set_active_edit_name (GtkWidget *w)
+{
+    active_edit_name = w;
+}
 
 struct dialog_t_ {
     GtkWidget *dialog;
@@ -639,12 +691,12 @@ gpointer edit_dialog_get_data (dialog_t *dlg)
 
 static void dialog_table_setup (dialog_t *dlg, int hsize)
 {
-    GtkWidget *sw;
+    GtkWidget *vbox, *sw;
 
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_set_size_request(sw, hsize, 200);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->dialog)->vbox), 
-		       sw, TRUE, TRUE, FALSE);
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg->dialog));
+    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, FALSE);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (sw),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
@@ -793,9 +845,7 @@ static GtkWidget *build_edit_popup (dialog_t *d)
 static gboolean 
 edit_dialog_popup_handler (GtkWidget *w, GdkEventButton *event, dialog_t *d)
 {
-    GdkModifierType mods;
-
-    gdk_window_get_pointer(w->window, NULL, NULL, &mods);
+    GdkModifierType mods = widget_get_pointer_mask(w);
 
     if (mods & GDK_BUTTON3_MASK) {
 	if (d->popup != NULL) {
@@ -818,40 +868,6 @@ edit_dialog_popup_handler (GtkWidget *w, GdkEventButton *event, dialog_t *d)
     return FALSE;
 }
 
-static void set_replace_restrictions (GtkWidget *w, gpointer p)
-{
-    dialog_t *d = (dialog_t *) p;
-    
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
-	d->opt = OPT_P;
-    } else {
-	d->opt = OPT_NONE;
-    }
-}
-
-static void sample_replace_buttons (GtkWidget *box, gpointer data)
-{
-    GtkWidget *tmp;
-    GSList *group;
-
-    /* add to current sample restriction */
-    tmp = gtk_radio_button_new_with_label(NULL, _("add to current restriction"));
-    gtk_box_pack_start(GTK_BOX(box), tmp, TRUE, TRUE, 0);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
-    gtk_widget_show(tmp);
-
-    /* replace current sample restriction */
-    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(tmp));
-    tmp = gtk_radio_button_new_with_label(group, _("replace current restriction"));
-    gtk_box_pack_start(GTK_BOX(box), tmp, TRUE, TRUE, 0);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), FALSE);
-
-    g_signal_connect(G_OBJECT(tmp), "clicked",
-		     G_CALLBACK(set_replace_restrictions), data);
-
-    gtk_widget_show(tmp);
-}
-
 static void set_sys_method (GtkComboBox *box, dialog_t *d)
 {
     gchar *str = gtk_combo_box_get_active_text(box);
@@ -975,9 +991,78 @@ GtkWidget *gretl_opts_combo (combo_opts *opts, int deflt)
     return gretl_opts_combo_full(opts, deflt, NULL, NULL);
 }
 
+void depopulate_combo_box (GtkComboBox *box)
+{
+    GtkTreeModel *model = gtk_combo_box_get_model(box);
+    GtkTreeIter iter;
+
+    while (gtk_tree_model_get_iter_first(model, &iter)) {
+	gtk_combo_box_remove_text(box, 0);
+    }
+}
+
+static void mle_gmm_iters_dialog (GtkWidget *w, dialog_t *d)
+{
+    int maxit, lmem = 0, optim = BFGS_MAX;
+    double tol;
+    int cancel = 0;
+
+    BFGS_defaults(&maxit, &tol, d->code);
+    lmem = libset_get_int(LBFGS_MEM);
+
+    if (maxit <= 0) {
+	maxit = 1000;
+    }  
+
+    if ((d->opt & OPT_L) || libset_get_bool(USE_LBFGS)) {
+	optim = LBFGS_MAX;
+    }
+
+    iter_control_dialog(&optim, &maxit, &tol, &lmem, &cancel,
+			d->dialog);
+
+    if (!cancel) {
+	int err;
+
+	err = libset_set_int(BFGS_MAXITER, maxit);
+	err += libset_set_double(BFGS_TOLER, tol);
+
+	if (optim == LBFGS_MAX) {
+	    d->opt |= OPT_L;
+	    libset_set_int(LBFGS_MEM, lmem);
+	} else {
+	    d->opt &= ~OPT_L;
+	}
+
+	if (err) {
+	    errbox("Error setting values");
+	}
+    }
+}
+
+static void iter_control_button (GtkWidget *vbox, dialog_t *d, MODEL *pmod)
+{
+    GtkWidget *hbox, *button;
+
+    if (pmod != NULL && pmod->ci == MLE) {
+	if (pmod->opt & OPT_L) {
+	    d->opt |= OPT_L;
+	}
+    }
+
+    hbox = gtk_hbox_new(FALSE, 5);
+
+    button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(mle_gmm_iters_dialog), d);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    gtk_widget_show_all(hbox);   
+}
+
 static void build_gmm_combo (GtkWidget *vbox, dialog_t *d, MODEL *pmod)
 {
-    GtkWidget *combo, *hbox;
+    GtkWidget *combo, *hbox, *button;
     static const char *strs[] = {
 	N_("One-step estimation"),
 	N_("Two-step estimation"),
@@ -1002,16 +1087,24 @@ static void build_gmm_combo (GtkWidget *vbox, dialog_t *d, MODEL *pmod)
 	} else if (pmod->opt & OPT_I) {
 	    deflt = 2;
 	}
+	if (pmod->opt & OPT_L) {
+	    d->opt |= OPT_L;
+	}
     }
 
     combo = gretl_opts_combo(&gmm_opts, deflt);
 
     hbox = gtk_hbox_new(FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0); /* 5 ? */
-    gtk_widget_show(combo);
+    gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 5);
+
+    /* BFGS controls */
+    button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
+    g_signal_connect(G_OBJECT(button), "clicked",
+		     G_CALLBACK(mle_gmm_iters_dialog), d);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-    gtk_widget_show(hbox);      
+    gtk_widget_show_all(hbox);      
 }
 
 static void system_estimator_list (GtkWidget *vbox, dialog_t *d,
@@ -1066,10 +1159,12 @@ static void dlg_display_sys (dialog_t *d)
 
     if (d->code == ESTIMATE) {
 	/* estimating an existing system, not editing */
+	GtkWidget *vbox;
+
 	w = gtk_label_new(sys->name);
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d->dialog)->vbox), 
-			   w, TRUE, TRUE, 10);
+	vbox = gtk_dialog_get_content_area(GTK_DIALOG(d->dialog));
+	gtk_box_pack_start(GTK_BOX(vbox), w, TRUE, TRUE, 10);
 	gtk_widget_show(w);
     }
 
@@ -1079,13 +1174,13 @@ static void dlg_display_sys (dialog_t *d)
     gretl_dialog_set_resizeable(d->dialog, TRUE);
 }
 
-static void raise_and_focus_dialog (GtkEditable *editable, gpointer p)
+void raise_and_focus_dialog (GtkEditable *entry, 
+			     GtkWidget *parent)
 {
-    dialog_t *d = (dialog_t *) p;
+    gtk_window_present(GTK_WINDOW(parent));
 
-    gdk_window_raise(d->dialog->window);
-    if (!GTK_WIDGET_HAS_FOCUS(d->edit)) {
-	gtk_widget_grab_focus(d->edit);
+    if (!GTK_WIDGET_HAS_FOCUS(GTK_WIDGET(entry))) {
+	gtk_widget_grab_focus(GTK_WIDGET(entry));
     }
 }
 
@@ -1186,8 +1281,8 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     gretl_dialog_set_resizeable(d->dialog, FALSE);
 
     /* convenience pointers */
-    top_vbox = GTK_DIALOG(d->dialog)->vbox;
-    button_box = GTK_DIALOG(d->dialog)->action_area;
+    top_vbox = gtk_dialog_get_content_area(GTK_DIALOG(d->dialog));
+    button_box = gtk_dialog_get_action_area(GTK_DIALOG(d->dialog));
     gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), 
 			      GTK_BUTTONBOX_END);
 
@@ -1252,12 +1347,10 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	gtk_widget_show(d->edit);
 
 	g_signal_connect(G_OBJECT(GTK_EDITABLE(d->edit)), "changed", 
-			 G_CALLBACK(raise_and_focus_dialog), d);
+			 G_CALLBACK(raise_and_focus_dialog), d->dialog);
     }
 
-    if (ci == SMPLBOOL && dataset_is_restricted()) {
-	sample_replace_buttons(top_vbox, d);
-    } else if (ci == SYSTEM || ci == ESTIMATE) {
+    if (ci == SYSTEM || ci == ESTIMATE) {
 	GtkWidget *bt, *bv;
 
 	bt = dialog_option_switch(top_vbox, d, OPT_T, NULL);
@@ -1266,6 +1359,9 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     } else if (ci == NLS || ci == MLE) {
 	dialog_option_switch(top_vbox, d, OPT_V, pmod);
 	dialog_option_switch(top_vbox, d, OPT_R, pmod);
+	if (ci == MLE) {
+	    iter_control_button(top_vbox, d, pmod);
+	}
     } else if (ci == GMM) {
 	dialog_option_switch(top_vbox, d, OPT_V, pmod);
 	build_gmm_combo(top_vbox, d, pmod);
@@ -1285,6 +1381,12 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	modal = 1;
     }
 
+    hlpcode = edit_dialog_help_code(ci, okptr);
+
+    if (varclick == VARCLICK_NONE && !hlpcode) {
+	gtk_window_set_keep_above(GTK_WINDOW(d->dialog), TRUE);
+    }
+
     gtk_widget_grab_focus(d->edit);
 
     /* "Clear" button? */
@@ -1293,7 +1395,6 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	gtk_container_add(GTK_CONTAINER(button_box), w);
 	g_signal_connect(G_OBJECT(w), "clicked", 
 			 G_CALLBACK(clear_dlg_previous), d);
-	gtk_widget_show(w);  
     }    
 
     /* "Cancel" button */
@@ -1308,17 +1409,16 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	g_signal_connect(G_OBJECT(w), "clicked", 
 			 G_CALLBACK(edit_dialog_ok), d);
     }
-	
     gtk_widget_grab_default(w);
-    gtk_widget_show(w);    
 
     /* "Help" button, if wanted */
-    hlpcode = edit_dialog_help_code(ci, okptr);
     if (hlpcode > 0) {
 	context_help_button(button_box, hlpcode);
 	modal = 0;
     }
 
+    gtk_widget_show_all(button_box); 
+
     if (ci == MODEL_GENR || ci == RESTRICT) {
 	windata_t *vwin = (windata_t *) okptr;
 
@@ -1411,3 +1511,86 @@ gchar *gtk_combo_box_get_active_text (GtkComboBox *box)
 }
 
 #endif
+
+GdkModifierType widget_get_pointer_mask (GtkWidget *w)
+{
+    GdkModifierType mods = 0;
+    GdkWindow *window;
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)    
+    window = w->window;
+#else
+    window = gtk_widget_get_window(w);
+#endif
+    if (window != NULL) {
+	gdk_window_get_pointer(window, NULL, NULL, &mods);
+    }
+
+    return mods;
+}
+
+GdkModifierType parent_get_pointer_mask (GtkWidget *w)
+{
+    GdkWindow *window = gtk_widget_get_parent_window(w);
+    GdkModifierType mods = 0;
+
+    if (window != NULL) {
+	gdk_window_get_pointer(window, NULL, NULL, &mods);
+    }
+
+    return mods;
+}
+
+gboolean widget_get_pointer_info (GtkWidget *w, gint *x, gint *y,
+				  GdkModifierType *mask)
+{
+    GdkWindow *window;
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)    
+    window = w->window;
+#else
+    window = gtk_widget_get_window(w);
+#endif
+
+    if (window == NULL) {
+	return FALSE;
+    } else {
+	gdk_window_get_pointer(window, x, y, mask);
+	return TRUE;
+    }
+}
+
+/* set-up for a top-level window, @dlg, which emulates a GtkDialog
+   structure: *pvbox is like the vbox member of a GtkDialog, and
+   *pbbox like the action_area member.
+*/
+
+void gretl_emulated_dialog_add_structure (GtkWidget *dlg,
+					  GtkWidget **pvbox,
+					  GtkWidget **pbbox)
+{
+    GtkWidget *base;
+
+    g_signal_connect(G_OBJECT(dlg), "key-press-event", 
+		     G_CALLBACK(esc_kills_window), NULL);
+
+    base = gtk_vbox_new(FALSE, 5);
+    gtk_container_add(GTK_CONTAINER(dlg), base);
+
+    *pvbox = gtk_vbox_new(FALSE, 0);
+
+    /* make (upper) vbox expansible */
+    gtk_box_pack_start(GTK_BOX(base), *pvbox, TRUE, TRUE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(*pvbox), 5);
+    gtk_box_set_spacing(GTK_BOX(*pvbox), 5);
+
+    vbox_add_hsep(base);
+
+    *pbbox = gtk_hbutton_box_new();
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(*pbbox), 
+			      GTK_BUTTONBOX_END);
+    gtk_box_set_spacing(GTK_BOX(*pbbox), 10);
+    gtk_box_pack_start(GTK_BOX(base), *pbbox,
+		       FALSE, FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(*pbbox), 5);
+}
