@@ -38,6 +38,17 @@ struct combo_opts_ {
     const char **strs;
 };
 
+GtkWidget *get_active_edit_id (void);
+
+GtkWidget *get_active_edit_name (void);
+
+GtkWidget *get_active_edit_text (void);
+
+void set_active_edit_name (GtkWidget *w);
+
+void raise_and_focus_dialog (GtkEditable *entry, 
+			     GtkWidget *parent);
+
 typedef struct combo_opts_ combo_opts;
 
 void set_window_busy (windata_t *vwin);
@@ -51,6 +62,8 @@ int maybe_raise_dialog (void);
 
 void vbox_add_hsep (GtkWidget *vbox);
 
+void pack_in_hbox (GtkWidget *w, GtkWidget *vbox, int vspace);
+
 GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode);
 
 GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
@@ -61,6 +74,8 @@ GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
 
 GtkWidget *cancel_button (GtkWidget *hbox);
 
+GtkWidget *close_button (GtkWidget *hbox);
+
 GtkWidget *ok_button (GtkWidget *hbox);
 
 GtkWidget *apply_button (GtkWidget *hbox);
@@ -72,9 +87,9 @@ GtkWidget *back_button (GtkWidget *hbox);
 gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key, 
 			   gpointer unused);
 
-void sensitize_widget_from_check (GtkWidget *check, GtkWidget *w);
+void sensitize_conditional_on (GtkWidget *w, GtkWidget *b);
 
-void desensitize_widget_from_check (GtkWidget *check, GtkWidget *w);
+void desensitize_conditional_on (GtkWidget *w, GtkWidget *b);
 
 void set_double_from_spinner (GtkSpinButton *b, double *x);
 
@@ -124,11 +139,30 @@ void set_combo_box_strings_from_list (GtkComboBox *box, GList *list);
 
 void set_combo_box_default_text (GtkComboBox *box, const char *s);
 
+void depopulate_combo_box (GtkComboBox *box);
+
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6)
 
 gchar *gtk_combo_box_get_active_text (GtkComboBox *box);
 
 #endif
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)
+# define gtk_dialog_get_content_area(d) (d->vbox)
+# define gtk_dialog_get_action_area(d) (d->action_area)
+#endif
+
+GdkModifierType widget_get_pointer_mask (GtkWidget *w);
+
+GdkModifierType parent_get_pointer_mask (GtkWidget *w);
+
+gboolean widget_get_pointer_info (GtkWidget *w, gint *x, gint *y,
+				  GdkModifierType *mask);
+
+void gretl_emulated_dialog_add_structure (GtkWidget *dlg,
+					  GtkWidget **pvbox,
+					  GtkWidget **pbbox);
+
+#define button_is_active(b) (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b)))
 
 #endif /* DLGUTILS_H */
