@@ -27,6 +27,7 @@
 #include "fnsave.h"
 #include "database.h"
 #include "datafiles.h"
+#include "graphics.h"
 #include "bootstrap.h"
 
 #include <sys/stat.h>
@@ -42,13 +43,13 @@
 
 #define OPEN_DATA_ACTION(i)  (i == OPEN_DATA || \
                               i == OPEN_CSV || \
-                              i == OPEN_ASCII || \
                               i == OPEN_OCTAVE || \
                               i == OPEN_GNUMERIC || \
 	                      i == OPEN_XLS || \
                               i == OPEN_WF1 || \
                               i == OPEN_DTA || \
 	                      i == OPEN_SAV || \
+			      i == OPEN_SAS || \
                               i == OPEN_JMULTI || \
                               i == OPEN_ODS)
 
@@ -57,15 +58,13 @@
                                i == APPEND_OCTAVE || \
                                i == APPEND_GNUMERIC || \
                                i == APPEND_XLS || \
-                               i == APPEND_ASCII || \
                                i == APPEND_WF1 || \
                                i == APPEND_DTA || \
                                i == APPEND_SAV || \
+			       i == APPEND_SAS || \
                                i == APPEND_JMULTI || \
                                i == APPEND_ODS)
 
-#define SAVE_GRAPH_ACTION(i) (i == SAVE_GNUPLOT)
-
 #define EXPORT_ACTION(a,s) ((a == EXPORT_OCTAVE || \
                              a == EXPORT_R || \
                              a == EXPORT_CSV || \
@@ -78,7 +77,8 @@
                        i == OPEN_DATA || \
                        i == APPEND_DATA)
 
-#define SET_DIR_ACTION(i) (i == SET_DIR || i == SET_WDIR || i == SET_FDIR)
+#define SET_DIR_ACTION(i) (i == SET_DIR || i == SET_WDIR || \
+                           i == SET_FDIR || i == SET_DBDIR)
 
 struct extmap {
     int action;
@@ -89,11 +89,14 @@ static struct extmap action_map[] = {
     { SAVE_DBDATA,       ".bin" },
     { SAVE_SCRIPT,       ".inp" },
     { SAVE_FUNCTIONS_AS, ".inp" },
-    { SAVE_CONSOLE,      ".inp" },
+    { SAVE_CMD_LOG,      ".inp" },
+    { SAVE_CONSOLE,      ".txt" },
     { SAVE_SESSION,      ".gretl" },
     { SAVE_GP_CMDS,      ".plt" },
     { SAVE_R_CMDS,       ".R" },
+    { SAVE_OX_CMDS,      ".ox" },
     { SAVE_FUNCTIONS,    ".gfn" },
+    { SAVE_MARKERS,      ".txt" },
     { EXPORT_CSV,        ".csv" },
     { EXPORT_R,          ".R" },
     { OPEN_OCTAVE,       ".m" },
@@ -108,8 +111,6 @@ static struct extmap action_map[] = {
     { OPEN_SESSION,      ".gretl" },
     { OPEN_CSV,          ".csv" },
     { APPEND_CSV,        ".csv" },
-    { OPEN_ASCII,        ".txt" },
-    { APPEND_ASCII,      ".txt" },
     { OPEN_GNUMERIC,     ".gnumeric" },
     { APPEND_GNUMERIC,   ".gnumeric" },
     { OPEN_XLS,          ".xls" },
@@ -120,12 +121,15 @@ static struct extmap action_map[] = {
     { APPEND_DTA,        ".dta" },
     { OPEN_SAV,          ".sav" },
     { APPEND_SAV,        ".sav" },
+    { OPEN_SAS,          ".xpt" },
+    { APPEND_SAS,        ".xpt" },
     { OPEN_JMULTI,       ".dat" },
     { APPEND_JMULTI,     ".dat" },
     { OPEN_ODS,          ".ods" },
     { APPEND_ODS,        ".ods" },
     { OPEN_RATS_DB,      ".rat" },
     { OPEN_PCGIVE_DB,    ".bn7" },
+    { OPEN_GFN,          ".gfn" },
     { FILE_OP_MAX,       NULL }
 };
 
@@ -146,6 +150,10 @@ static gretlopt save_action_to_opt (int action, gpointer p)
     default: break;
     }
 
+    if (action == SAVE_DATA_AS && session_file_is_open()) {
+	opt |= OPT_X; /* "exporting" to GDT */
+    }
+
     if (p != NULL) {
 	opt |= GPOINTER_TO_INT(p);
     }
@@ -166,36 +174,14 @@ static const char *get_gp_ext (int ttype)
     else return "*";
 }
 
-static int dat_ext (const char *str, int showerr)
-{
-    const char *suff;
-    int err = 0;
-
-    if (str == NULL) {
-	return 0;
-    }
-
-    suff = strrchr(str, '.');
-
-    if (suff != NULL && !strcmp(suff, ".gdt")) {
-	if (showerr) {
-	    errbox(_("The suffix you selected should be used\n"
-		   "only for gretl datafiles"));
-	}
-	err = 1;
-    }
-
-    return err;
-}
-
 static const char *get_ext (int action, gpointer data)
 {
     const char *s = NULL;
 
     if (GDT_ACTION(action)) {
 	return ".gdt";
-    } else if (action == SAVE_GNUPLOT) {
-	int ttype = gp_term_code(data);
+    } else if (action == SAVE_GNUPLOT || action == SAVE_GRAPHIC) {
+	int ttype = gp_term_code(data, action);
 
 	s = get_gp_ext(ttype);
     } else {
@@ -261,9 +247,12 @@ static void script_window_update (windata_t *vwin, const char *fname)
     strcpy(vwin->fname, fname);
 
     if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT ||
-	vwin->role == VIEW_FUNC_CODE) {
+	vwin->role == VIEW_PKG_CODE) {
 	/* change role of window for editing */
 	vwin->role = EDIT_SCRIPT;
+    } else if (vwin->role == EDIT_GP) {
+	/* plot file no longer under session control */
+	vwin->flags &= ~VWIN_SESSION_GRAPH;
     }
 
     mark_vwin_content_saved(vwin);
@@ -279,9 +268,7 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 {
     FILE *fp;
     gchar *buf;
-#ifdef ENABLE_NLS
     gchar *trbuf;
-#endif
 
     buf = textview_get_text(vwin->text);
     if (buf == NULL) {
@@ -296,15 +283,12 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	return;
     }
 
-#ifdef ENABLE_NLS
     trbuf = my_locale_from_utf8(buf);
     if (trbuf != NULL) {
+	/* UTF minuses? */
 	system_print_buf(trbuf, fp);
 	g_free(trbuf);
     }
-#else
-    system_print_buf(buf, fp);
-#endif
 
     g_free(buf);
     fclose(fp);
@@ -313,7 +297,9 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	strcpy(scriptfile, fname);
 	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
 	script_window_update(vwin, fname);
-    } else if (action == SAVE_GP_CMDS || action == SAVE_R_CMDS) {
+    } else if (action == SAVE_GP_CMDS || 
+	       action == SAVE_R_CMDS ||
+	       action == SAVE_OX_CMDS) {
 	script_window_update(vwin, fname);
     } 
 }
@@ -340,13 +326,15 @@ static void filesel_open_script (const char *fname, windata_t *vwin)
     } else if (has_suffix(fname, ".R")) {
 	view_file(fname, 1, 0, 78, 370, EDIT_R);
     } else if (has_suffix(fname, ".plt")) {
-	view_file(fname, 1, 0, 78, 370, EDIT_GP);	
+	view_file(fname, 1, 0, 78, 370, EDIT_GP);
+    } else if (ox_support && has_suffix(fname, ".ox")) {
+	view_file(fname, 1, 0, 78, 370, EDIT_OX);
     } else {
 	strcpy(tryfile, fname);
 	if (view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
 	    strcpy(scriptfile, tryfile);
 	    mkfilelist(FILE_LIST_SCRIPT, scriptfile);
-	    set_currdir_from_filename(scriptfile);
+	    gretl_set_current_dir(scriptfile);
 	}
     }
 }
@@ -355,12 +343,13 @@ static void filesel_open_session (const char *fname)
 {
     strcpy(tryfile, fname);
 
-    if (is_session_file(fname)) {
+    if (gretl_is_pkzip_file(fname)) {
 	verify_open_session();
     } else {
+	/* old script-style session file? */
 	windata_t *vwin;
 
-	if (has_system_prefix(tryfile, &paths, SCRIPT_SEARCH)) {
+	if (has_system_prefix(tryfile, SCRIPT_SEARCH)) {
 	    vwin = view_file(tryfile, 0, 0, 78, 370, VIEW_SCRIPT);
 	} else {
 	    vwin = view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT);
@@ -446,20 +435,30 @@ static void bootstrap_save_callback (const char *fname)
     } 
 }
 
+static void maybe_set_fsel_status (FselDataSrc src, gpointer p, int val)
+{
+    if (src == FSEL_DATA_STATUS && p != NULL) {
+	* (int *) p = val;
+    }
+}
+
 static void
 file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 			      gpointer data)
 {
     char fname[FILENAME_MAX];
+    int err = 0;
 
     *fname = 0;
     strncat(fname, in_fname, FILENAME_MAX - 1);
 
     if (action < END_OPEN) {
+	/* opening a file: check that the file is accessible */
 	FILE *fp = gretl_fopen(fname, "r");
 
 	if (fp == NULL) {
 	    file_read_errbox(fname);
+	    maybe_set_fsel_status(src, data, E_FOPEN);
 	    return;
 	} else {
 	    fclose(fp);
@@ -473,9 +472,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	strcpy(tryfile, fname);
 	do_open_data(NULL, action);
     } else if (action == OPEN_SCRIPT) {
-	if (src == FSEL_DATA_FNPKG) {
-	    fnsave_set_script(fname, data);
-	} else if (src == FSEL_DATA_VWIN) {
+	if (src == FSEL_DATA_VWIN) {
 	    filesel_open_script(fname, data);
 	} else {
 	    filesel_open_script(fname, NULL);
@@ -484,6 +481,8 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	filesel_open_session(fname);
     } else if (action == OPEN_MARKERS) {
 	do_add_markers(fname);
+    } else if (action == OPEN_GFN) {
+	edit_function_package(fname);
     } else if (action == OPEN_RATS_DB) {
 	open_rats_window(fname);
     } else if (action == OPEN_PCGIVE_DB) {
@@ -496,10 +495,6 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 
     /* now for the save/export options */
 
-    if (action > SAVE_DBDATA && action != EXPORT_DAT && dat_ext(fname, 1)) { 
-	return;
-    }
-
     if (check_maybe_add_ext(fname, action, data)) {
 	return;
     }
@@ -513,30 +508,39 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     } else if (src == FSEL_DATA_PRN) {
 	filesel_save_prn_buffer((PRN *) data, fname);
     } else if (SAVE_DATA_ACTION(action)) {
-	do_store(fname, save_action_to_opt(action, data));
+	err = do_store(fname, save_action_to_opt(action, data));
     } else if (action == SAVE_GNUPLOT) {
 	save_graph_to_file(data, fname);
+    } else if (action == SAVE_GRAPHIC) {
+	save_graphic_to_file(data, fname);
     } else if (action == SAVE_SESSION) {
-	save_session(fname);
+	err = save_session(fname);
+    } else if (action == SAVE_CMD_LOG) {
+	err = save_session_commands(fname);
     } else if (action == SAVE_FUNCTIONS) {
-	save_user_functions(fname, data);
+	err = save_function_package(fname, data);
     } else if (action == SAVE_FUNCTIONS_AS) {
-	save_user_functions_as_script(fname, data);
+	err = save_function_package_as_script(fname, data);
     } else if (action == SAVE_BOOT_DATA) {
 	bootstrap_save_callback(fname);
+    } else if (action == SAVE_MARKERS) {
+	err = do_save_markers(fname);
     } else if (action == SET_PROG || action == SET_DIR) {
 	set_path_callback(data, fname);
     } else if (action == SET_WDIR) {
 	set_working_dir_callback(data, fname);
     } else if (action == SET_FDIR) {
 	set_funcs_dir_callback(data, fname);
+    } else if (action == SET_DBDIR) {
+	set_db_dir_callback(data, fname);
     } else {
 	windata_t *vwin = (windata_t *) data;
 
 	save_editable_content(action, fname, vwin);
     }
-}
 
+    maybe_set_fsel_status(src, data, err);
+}
 
 static char *get_filter_suffix (int action, gpointer data, char *suffix)
 {
@@ -581,7 +585,7 @@ static gchar *inplace_windows_filename (gchar *s)
     return s;
 }
 
-static void correct_default_dir (char *s)
+static char *win32_correct_path (char *s)
 {
     if (!g_utf8_validate(s, -1, NULL)) {
 	gchar *tmp = my_locale_to_utf8(s);
@@ -591,121 +595,166 @@ static void correct_default_dir (char *s)
 	    g_free(tmp);
 	}
     }
+
+    return s;
 }
 
 #endif
 
-static void gtk_file_selector (const char *msg, int action, FselDataSrc src, 
+static void set_default_progs_path (GtkFileChooser *fsel)
+{
+#ifdef G_OS_WIN32
+    char *progs = program_files_path();
+
+    if (progs != NULL) {
+	gchar *path = my_filename_to_utf8(progs);
+
+	gtk_file_chooser_set_current_folder(fsel, path);
+	g_free(path);
+	free(progs);
+    }
+#else
+    gtk_file_chooser_set_current_folder(fsel, "/usr/bin");
+#endif
+}
+
+static void filesel_maybe_set_current_name (GtkFileChooser *filesel,
+					    int action, FselDataSrc src,
+					    gpointer data)
+{
+    gchar *currname;
+
+    if (action == SAVE_DATA && *datafile != '\0') {
+	currname = suggested_savename(datafile);
+	gtk_file_chooser_set_current_name(filesel, currname);
+	g_free(currname);
+    } else if (EXPORT_ACTION(action, src) && *datafile != '\0') {
+	currname = suggested_exportname(datafile, action);
+	gtk_file_chooser_set_current_name(filesel, currname);
+	g_free(currname);
+    } else if (action == SAVE_CMD_LOG) {
+	gtk_file_chooser_set_current_name(filesel, "session.inp");
+    } else if (action == SET_PROG || action == SET_DIR) {
+	currname = (gchar *) data;
+	if (currname != NULL && g_path_is_absolute(currname)) {
+	    gtk_file_chooser_set_filename(filesel, currname);
+	} else if (action == SET_PROG) {
+	    set_default_progs_path(filesel);
+	}	    
+    } else if (action == SAVE_FUNCTIONS || action == SAVE_FUNCTIONS_AS) {
+	char fname[MAXLEN];
+
+	*fname = '\0';
+	get_default_package_name(fname, data, action);
+	gtk_file_chooser_set_current_name(filesel, fname);
+    } else if (action == SAVE_MARKERS) {
+	gtk_file_chooser_set_current_name(filesel, "markers.txt");
+    }
+}
+
+static void filesel_add_filter (GtkWidget *filesel,
+				const char *desc, 
+				const char *pat)
+{
+    GtkFileFilter *filt = gtk_file_filter_new();
+
+    gtk_file_filter_set_name(filt, desc);
+    gtk_file_filter_add_pattern(filt, pat);
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filt);
+}
+
+static void filesel_set_filters (GtkWidget *filesel, int action,
+				 FselDataSrc src, gpointer data)
+{
+    if (action == OPEN_CSV || action == APPEND_CSV) {
+	filesel_add_filter(filesel, _("CSV files (*.csv)"), "*.csv");
+	filesel_add_filter(filesel, _("ASCII files (*.txt)"), "*.txt");
+	filesel_add_filter(filesel, _("all files (*.*)"), "*");
+    } else if (action == OPEN_SCRIPT) {
+	filesel_add_filter(filesel, _("gretl script files (*.inp)"), "*.inp");
+	if (src != FSEL_DATA_FNPKG) {
+	    filesel_add_filter(filesel, _("GNU R files (*.R)"), "*.R");
+	    filesel_add_filter(filesel, _("gnuplot files (*.plt)"), "*.plt");
+#ifdef USE_OX
+	    if (ox_support) {
+		filesel_add_filter(filesel, _("Ox files (*.ox)"), "*.ox");
+	    }
+#endif
+	}
+    } else {
+	GtkFileFilter *filter = get_file_filter(action, data);
+
+	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
+    }
+}
+
+static void remember_folder (GtkFileChooser *chooser, char *savedir)
+{
+    gchar *folder = gtk_file_chooser_get_current_folder(chooser);
+
+    strcpy(savedir, folder);
+    g_free(folder);
+}
+
+static void gtk_file_selector (int action, FselDataSrc src, 
 			       gpointer data, GtkWidget *parent) 
 {
-    GtkWidget *filesel;
+    static char savedir[MAXLEN];
     char startdir[MAXLEN];
-    GtkFileFilter *filter;
+    GtkWidget *filesel;
     GtkFileChooserAction fa;
     const gchar *okstr;
+    int remember = get_keep_folder();
     gint response;
 
-    get_default_dir(startdir, action);
-
-#ifdef G_OS_WIN32
-    correct_default_dir(startdir);
-#endif
-
     if (SET_DIR_ACTION(action)) {
 	fa = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 	okstr = GTK_STOCK_OK;
+	remember = 0;
     } else if (action == SET_PROG) {
 	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
 	okstr = GTK_STOCK_OK;
+	remember = 0;
     } else if (action < END_OPEN) {
 	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
 	okstr = GTK_STOCK_OPEN;
     } else {
 	fa = GTK_FILE_CHOOSER_ACTION_SAVE;
 	okstr = GTK_STOCK_SAVE;
+	if (action == SAVE_FUNCTIONS || action == SAVE_DATA_PKG) {
+	    remember = 0;
+	}
     }
 
+    if (remember && *savedir != '\0') {
+	strcpy(startdir, savedir);
+    } else {
+	get_default_dir(startdir, action);
+    }
+
+#ifdef G_OS_WIN32
+    win32_correct_path(startdir);
+#endif
+
     if (parent == NULL) {
 	parent = mdata->main;
     }
 
-    filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(parent), fa,
+    filesel = gtk_file_chooser_dialog_new(NULL, GTK_WINDOW(parent), fa,
 					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					  okstr, GTK_RESPONSE_ACCEPT,
 					  NULL);
 
     gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
-    filter = get_file_filter(action, data);
-
-    if (action == OPEN_ASCII || action == APPEND_ASCII) {
-	gtk_file_filter_set_name(filter, _("ASCII files (*.txt)"));
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    } else if (action == OPEN_SCRIPT) {
-	gtk_file_filter_set_name(filter, _("gretl script files (*.inp)"));
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    }	
-
-    gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
-
-    if (action == OPEN_ASCII || action == APPEND_ASCII) {
-	filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(filter, _("all files (*.*)"));
-	gtk_file_filter_add_pattern(filter, "*");
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    } else if (action == OPEN_SCRIPT && src != FSEL_DATA_FNPKG) {
-	filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(filter, _("GNU R files (*.R)"));
-	gtk_file_filter_add_pattern(filter, "*.R");
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-	filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(filter, _("gnuplot files (*.plt)"));
-	gtk_file_filter_add_pattern(filter, "*.plt");
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    } 
-
-    /* FIXME session dir */
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
-
-    /* special cases */
-
-    if (action == SAVE_DATA && *paths.datfile != '\0') {
-	char *savename = suggested_savename(paths.datfile);
-
-	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
-					  savename);
-	g_free(savename);
-    } else if (EXPORT_ACTION(action, src) && *paths.datfile != '\0') {
-	char *savename = suggested_exportname(paths.datfile, action);
-
-	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
-					  savename);
-	g_free(savename);
-    } else if (action == SET_PROG) {
-	char *strvar = (char *) data;
-
-	if (strvar != NULL && g_path_is_absolute(strvar)) {
-	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
-					  strvar);
-	} else {
-	    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), 
-						"/usr/bin");
-	}	    
-    } else if (action == SET_DIR) {
-	char *strvar = (char *) data;
-
-	if (strvar != NULL && g_path_is_absolute(strvar)) {
-	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
-					  strvar);
-	} 
-    } else if (action == SAVE_FUNCTIONS || action == SAVE_FUNCTIONS_AS) {
-	char fname[MAXLEN];
-
-	*fname = '\0';
-	get_default_package_name(fname, data, action);
-	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
-					  fname);
-    } 
+    if (SET_DIR_ACTION(action)) {
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), startdir);
+    } else {
+	filesel_set_filters(filesel, action, src, data);
+	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
+	filesel_maybe_set_current_name(GTK_FILE_CHOOSER(filesel), action,
+				       src, data);
+    }
 
     response = gtk_dialog_run(GTK_DIALOG(filesel));
 
@@ -713,6 +762,11 @@ static void gtk_file_selector (const char *msg, int action, FselDataSrc src,
 	gchar *fname;
 	
 	fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
+	if (remember) {
+	    remember_folder(GTK_FILE_CHOOSER(filesel), savedir);
+	}
+	gtk_widget_destroy(filesel);
+	filesel = NULL;
 
 	if (fname != NULL) {
 #ifdef G_OS_WIN32
@@ -721,12 +775,16 @@ static void gtk_file_selector (const char *msg, int action, FselDataSrc src,
 	    file_selector_process_result(fname, action, src, data);
 	    g_free(fname);
 	}
-    } 
+    } else {
+	maybe_set_fsel_status(src, data, GRETL_CANCEL);
+    }
 
-    gtk_widget_destroy(filesel);
+    if (filesel != NULL) {
+	gtk_widget_destroy(filesel);
+    }
 }
 
-void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
+void file_selector (int action, FselDataSrc src, gpointer data)
 {
     GtkWidget *w = NULL;
 
@@ -736,11 +794,11 @@ void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
 	w = vwin->main;
     }
 
-    gtk_file_selector(msg, action, src, data, w);
+    gtk_file_selector(action, src, data, w);
 }
 
-void file_selector_with_parent (const char *msg, int action, FselDataSrc src, 
+void file_selector_with_parent (int action, FselDataSrc src, 
 				gpointer data, GtkWidget *w)
 {
-    gtk_file_selector(msg, action, src, data, w);
+    gtk_file_selector(action, src, data, w);
 }
