@@ -29,7 +29,8 @@ enum {
     FILTER_SMA = 1,
     FILTER_EMA,
     FILTER_HP,
-    FILTER_BK
+    FILTER_BK,
+    FILTER_FD
 };
 
 enum {
@@ -49,6 +50,7 @@ typedef struct filter_info_ filter_info;
 struct filter_info_ {
     int ftype;
     int vnum;
+    const char *vname;
     int t1;
     int t2;
     int nobs;
@@ -77,6 +79,8 @@ static const char *filter_get_title (int ftype)
 	return N_("Hodrick-Prescott filter");
     } else if (ftype == FILTER_BK) {
 	return N_("Baxter-King Band-pass filter");
+    } else if (ftype == FILTER_FD) {
+	return N_("Fractional difference");
     } else {
 	return "";
     }
@@ -87,6 +91,7 @@ static void filter_info_init (filter_info *finfo, int ftype, int v,
 {
     finfo->ftype = ftype;
     finfo->vnum = v;
+    finfo->vname = datainfo->varname[v];
     finfo->t1 = t1;
     finfo->t2 = t2;
     finfo->nobs = 0;
@@ -129,6 +134,8 @@ static void filter_info_init (filter_info *finfo, int ftype, int v,
 	if (finfo->bku <= finfo->bkl) {
 	    finfo->bku = finfo->bkl + 1;
 	}
+    } else if (ftype == FILTER_FD) {
+	finfo->lambda = 0.5;
     }
 }
 
@@ -145,10 +152,12 @@ static void filter_make_savename (filter_info *finfo, int i)
 	strcpy(targ, (i == 0)? "hpt_" : "hp_");
     } else if (finfo->ftype == FILTER_BK) {
 	strcpy(targ, "bk_");
+    } else if (finfo->ftype == FILTER_FD) {
+	strcpy(targ, "fd_");
     }
 
     len = strlen(targ);
-    strncat(targ, datainfo->varname[finfo->vnum], VNAMELEN - len - 1);
+    strncat(targ, finfo->vname, VNAMELEN - len - 1);
 }
 
 static void filter_make_varlabel (filter_info *finfo, int v, int i)
@@ -159,34 +168,35 @@ static void filter_make_varlabel (filter_info *finfo, int v, int i)
 	if (i == FILTER_SAVE_TREND) {
 	    if (finfo->center) {
 		sprintf(targ, _("Centered %d-period moving average of %s"), 
-			finfo->nobs, datainfo->varname[finfo->vnum]);
+			finfo->nobs, finfo->vname);
 	    } else {
 		sprintf(targ, _("%d-period moving average of %s"), 
-			finfo->nobs, datainfo->varname[finfo->vnum]);
+			finfo->nobs, finfo->vname);
 	    }
 	} else {
 	    sprintf(targ, _("Residual from %d-period MA of %s"), 
-		    finfo->nobs, datainfo->varname[finfo->vnum]);
+		    finfo->nobs, finfo->vname);
 	}
     } else if (finfo->ftype == FILTER_EMA) {
 	if (i == FILTER_SAVE_TREND) {
 	    sprintf(targ, _("Exponential moving average of %s (current weight %g)"),
-		    datainfo->varname[finfo->vnum], 1.0 - finfo->lambda);
+		    finfo->vname, 1.0 - finfo->lambda);
 	} else {
 	    sprintf(targ, _("Residual from EMA of %s (current weight %g)"),
-		    datainfo->varname[finfo->vnum], 1.0 - finfo->lambda);
+		    finfo->vname, 1.0 - finfo->lambda);
 	}	    
     } else if (finfo->ftype == FILTER_HP) {
 	if (i == FILTER_SAVE_TREND) {
 	    sprintf(targ, _("Filtered %s: Hodrick-Prescott trend (lambda = %g)"), 
-		    datainfo->varname[finfo->vnum], finfo->lambda);
+		    finfo->vname, finfo->lambda);
 	} else {
 	    sprintf(targ, _("Filtered %s: Hodrick-Prescott cycle (lambda = %g)"), 
-		    datainfo->varname[finfo->vnum], finfo->lambda);
+		    finfo->vname, finfo->lambda);
 	}	    
     } else if (finfo->ftype == FILTER_BK) {
-	sprintf(targ, _("Filtered %s: Baxter-King cycle"), 
-		datainfo->varname[finfo->vnum]);
+	sprintf(targ, _("Filtered %s: Baxter-King cycle"), finfo->vname);
+    } else if (finfo->ftype == FILTER_FD) {
+	sprintf(targ, "fracdiff(%s, %g)", finfo->vname, finfo->lambda);
     }
 }
 
@@ -269,34 +279,45 @@ static void filter_dialog_ok (GtkWidget *w, filter_info *finfo)
 
 static void filter_dialog_hsep (GtkWidget *dlg)
 {
+    GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
     GtkWidget *hs = gtk_hseparator_new();
     
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hs, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hs, TRUE, TRUE, 0);
 }
 
 static void filter_graph_check_button (GtkWidget *dlg, filter_info *finfo,
 				       const char *txt, gretlopt g)
 {
+    GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
     GtkWidget *hbox, *w;
 
     hbox = gtk_hbox_new(FALSE, 5);
     w = gretl_option_check_button(txt, &finfo->graph_opt, g);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), finfo->graph_opt & g);
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
 static void filter_save_check_buttons (GtkWidget *dlg, filter_info *finfo)
 {
+    GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
     GtkWidget *hbox, *tab, *w;
-    int i;
+    int i, imax = 2;
+
+    if (finfo->ftype == FILTER_FD) {
+	imax = 1;
+    }
 
     hbox = gtk_hbox_new(FALSE, 5);
     tab = gtk_table_new(2, 2, FALSE);
     gtk_table_set_col_spacing(GTK_TABLE(tab), 0, 5);
 
-    for (i=0; i<2; i++) {
-	if (i == 0) {
+    for (i=0; i<imax; i++) {
+	if (finfo->ftype == FILTER_FD) {
+	    w = gretl_option_check_button(_("Save output as"),
+					  &finfo->save_opt, 
+					  FILTER_SAVE_TREND);
+	} else if (i == 0) {
 	    w = gretl_option_check_button(_("Save smoothed series as"),
 					  &finfo->save_opt, 
 					  FILTER_SAVE_TREND);
@@ -312,6 +333,7 @@ static void filter_save_check_buttons (GtkWidget *dlg, filter_info *finfo)
 	gtk_entry_set_width_chars(GTK_ENTRY(w), VNAMELEN + 3);
 	filter_make_savename(finfo, i);
 	gtk_entry_set_text(GTK_ENTRY(w), (i == 0)? finfo->save1 : finfo->save2);
+	gtk_entry_set_activates_default(GTK_ENTRY(w), TRUE);	
 	gtk_table_attach_defaults(GTK_TABLE(tab), w, 1, 2, i, i+1);
 	if (i == 0) {
 	    finfo->entry1 = w;
@@ -321,11 +343,12 @@ static void filter_save_check_buttons (GtkWidget *dlg, filter_info *finfo)
     }
 
     gtk_box_pack_start(GTK_BOX(hbox), tab, TRUE, TRUE, 5);  
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
 static void bkbp_frequencies_table (GtkWidget *dlg, filter_info *finfo)
 {
+    GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
     GtkWidget *tab, *hbox, *w;
     GtkWidget *s1, *s2;
 
@@ -359,16 +382,19 @@ static void bkbp_frequencies_table (GtkWidget *dlg, filter_info *finfo)
 
     /* pack everything */
     gtk_box_pack_start(GTK_BOX(hbox), tab, TRUE, TRUE, 5);  
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
 static void use_submean (GtkWidget *w, GtkWidget *s)
 {
-    gtk_widget_set_sensitive(s, GTK_TOGGLE_BUTTON(w)->active);
+    gboolean t = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    gtk_widget_set_sensitive(s, t);
 }
 
 static void ema_obs_radios (GtkWidget *dlg, filter_info *finfo)
 {
+    GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));    
     GtkWidget *tab, *hbox, *w;
     GtkWidget *r1, *r2;
     GSList *group;
@@ -376,7 +402,7 @@ static void ema_obs_radios (GtkWidget *dlg, filter_info *finfo)
     hbox = gtk_hbox_new(FALSE, 5);
     w = gtk_label_new(_("The first EMA value is"));
     gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
     hbox = gtk_hbox_new(FALSE, 5);
     tab = gtk_table_new(2, 2, FALSE);
@@ -400,12 +426,13 @@ static void ema_obs_radios (GtkWidget *dlg, filter_info *finfo)
 
     /* pack everything */
     gtk_box_pack_start(GTK_BOX(hbox), tab, TRUE, TRUE, 10);  
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
 static int filter_dialog (filter_info *finfo)
 {
     GtkWidget *dlg;
+    GtkWidget *vbox;
     GtkWidget *hbox;
     GtkWidget *w;
     int ret = 0;
@@ -417,8 +444,9 @@ static int filter_dialog (filter_info *finfo)
     /* box title */
     hbox = gtk_hbox_new(FALSE, 5);
     w = gtk_label_new(_(filter_get_title(finfo->ftype)));
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);  
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 
     if (finfo->ftype == FILTER_SMA) {
 	GtkWidget *nspin;
@@ -434,7 +462,7 @@ static int filter_dialog (filter_info *finfo)
 	g_signal_connect(G_OBJECT(nspin), "value-changed",
 			 G_CALLBACK(set_int_from_spinner), &finfo->nobs);
 	gtk_box_pack_start(GTK_BOX(hbox), nspin, TRUE, TRUE, 5);    
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
 	/* select centered or not */
 	hbox = gtk_hbox_new(FALSE, 5);
@@ -445,7 +473,7 @@ static int filter_dialog (filter_info *finfo)
 	g_signal_connect(G_OBJECT(w), "toggled", 
 			 G_CALLBACK(sma_center_callback), &finfo->center);
 	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     } else if (finfo->ftype == FILTER_EMA) {
 	/* set weight on current observation */
 	hbox = gtk_hbox_new(FALSE, 5);
@@ -456,7 +484,7 @@ static int filter_dialog (filter_info *finfo)
 	g_signal_connect(G_OBJECT(w), "value-changed",
 			 G_CALLBACK(set_double_from_spinner), &finfo->lambda);
 	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
 	/* set calculation of initial EMA value */
 	ema_obs_radios(dlg, finfo);
@@ -470,7 +498,7 @@ static int filter_dialog (filter_info *finfo)
 	g_signal_connect(G_OBJECT(w), "value-changed",
 			 G_CALLBACK(set_double_from_spinner), &finfo->lambda);
 	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     } else if (finfo->ftype == FILTER_BK) {
 	/* set "k" */
 	hbox = gtk_hbox_new(FALSE, 5);
@@ -481,11 +509,22 @@ static int filter_dialog (filter_info *finfo)
 	g_signal_connect(G_OBJECT(w), "value-changed",
 			 G_CALLBACK(set_int_from_spinner), &finfo->k);
 	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
 	/* set periods */
 	bkbp_frequencies_table(dlg, finfo);
-    }
+    } else if (finfo->ftype == FILTER_FD) {
+	/* set "d" */
+	hbox = gtk_hbox_new(FALSE, 5);
+	w = gtk_label_new(_("differencing parameter:"));
+	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+	w = gtk_spin_button_new_with_range(0.01, 10, 0.001);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), finfo->lambda);
+	g_signal_connect(G_OBJECT(w), "value-changed",
+			 G_CALLBACK(set_double_from_spinner), &finfo->lambda);
+	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);    
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    }	
 
     filter_dialog_hsep(dlg);
 
@@ -505,7 +544,13 @@ static int filter_dialog (filter_info *finfo)
 	gtk_entry_set_text(GTK_ENTRY(w), finfo->save2);
 	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
 	finfo->entry2 = w;
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    } else if (finfo->ftype == FILTER_FD) {
+	/* graphical output? */
+	filter_graph_check_button(dlg, finfo, _("Graph differenced series"),
+				  FILTER_GRAPH_TREND);
+	/* save to dataset? */
+	filter_save_check_buttons(dlg, finfo);
     } else {
 	/* graphical output? */
 	filter_graph_check_button(dlg, finfo, _("Graph original and smoothed series"),
@@ -518,10 +563,10 @@ static int filter_dialog (filter_info *finfo)
     }
 
     /* convenience pointer for button box */
-    hbox = GTK_DIALOG(dlg)->action_area;
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
 
     /* Cancel button */
-    w = cancel_delete_button(hbox, dlg, &ret);
+    cancel_delete_button(hbox, dlg, &ret);
 
     /* "OK" button */
     w = ok_button(hbox);
@@ -568,12 +613,16 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
     if ((finfo->graph_opt & FILTER_GRAPH_TREND) &&
 	(finfo->graph_opt & FILTER_GRAPH_CYCLE)) {
 	twoplot = 1;
-    }
+    } 
 
     if (gnuplot_init((twoplot)? PLOT_TRI_GRAPH : PLOT_REGULAR, &fp)) { 
 	return E_FOPEN;
     }
 
+    if (!twoplot) {
+	fprintf(fp, "# timeseries %d\n", datainfo->pd);
+    }
+
     if (datainfo->pd == 4) {
 	if ((finfo->t2 - finfo->t1) / 4 < 8) {
 	    fputs("set xtics nomirror 0,1\n", fp); 
@@ -603,8 +652,8 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	if (zkeypos == 'L') {
 	    print_keypos_string(GP_KEY_LEFT_TOP, fp);
 	}
-	sprintf(xtitle, _("%s (original data)"), datainfo->varname[v]);
-	sprintf(ztitle, _("%s (smoothed)"), datainfo->varname[v]);
+	sprintf(xtitle, _("%s (original data)"), finfo->vname);
+	sprintf(ztitle, _("%s (smoothed)"), finfo->vname);
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines, \\\n"
 		" '-' using 1:2 title '%s' w lines\n", xtitle, ztitle);
 	print_gp_data(finfo, obs, Z[v], fp);
@@ -615,17 +664,22 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	fputs("set size 1.0,0.38\n", fp);
 	fputs("set origin 0.0,0.0\n", fp);
 	fputs("set xzeroaxis\n", fp);
-	sprintf(title, _("Cyclical component of %s"), datainfo->varname[v]);
+	sprintf(title, _("Cyclical component of %s"), finfo->vname);
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines\n", title);
 	print_gp_data(finfo, obs, u, fp);
 	fputs("e\n", fp);
 	fputs("set nomultiplot\n", fp);
-    } else if (finfo->graph_opt & FILTER_GRAPH_TREND) {
+    } else if (finfo->ftype == FILTER_FD) {
+	sprintf(ztitle, "fracdiff(%s, %g)", finfo->vname, finfo->lambda);
+	fprintf(fp, "plot '-' using 1:2 title '%s' w lines\n", ztitle);
+	print_gp_data(finfo, obs, fx, fp);
+	fputs("e\n", fp);
+     } else if (finfo->graph_opt & FILTER_GRAPH_TREND) {
 	if (zkeypos == 'L') {
 	    print_keypos_string(GP_KEY_LEFT_TOP, fp);
 	}
-	sprintf(xtitle, _("%s (original data)"), datainfo->varname[v]);
-	sprintf(ztitle, _("%s (smoothed)"), datainfo->varname[v]);
+	sprintf(xtitle, _("%s (original data)"), finfo->vname);
+	sprintf(ztitle, _("%s (smoothed)"), finfo->vname);
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines, \\\n"
 		" '-' using 1:2 title '%s' w lines\n", xtitle, ztitle);
 	print_gp_data(finfo, obs, Z[v], fp);
@@ -635,9 +689,9 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
     } else if (finfo->graph_opt & FILTER_GRAPH_CYCLE) {
 	if (finfo->ftype == FILTER_BK) {
 	    sprintf(title, _("Baxter-King component of %s at frequency %d to %d"), 
-		    datainfo->varname[v], finfo->bkl, finfo->bku);
+		    finfo->vname, finfo->bkl, finfo->bku);
 	} else {
-	    sprintf(title, _("Cyclical component of %s"), datainfo->varname[v]);
+	    sprintf(title, _("Cyclical component of %s"), finfo->vname);
 	}
 	fprintf(fp, "set title '%s'\n", title); 
 	fputs("set xzeroaxis\n", fp);
@@ -789,6 +843,12 @@ static int calculate_filter (filter_info *finfo)
 	for (t=t1; t<=finfo->t2; t++) {
 	    fx[t] = finfo->lambda * x[t] + (1.0 - finfo->lambda) * fx[t-1];
 	}
+	for (t=finfo->t2; t>=t1; t--) {
+	    fx[t] = fx[t-1];
+	}
+	for (t=t1-1; t>=0; t--) {
+	    fx[t] = NADBL;
+	}
     } else if (finfo->ftype == FILTER_HP) {
 	double l0 = libset_get_double(HP_LAMBDA);
 
@@ -801,6 +861,8 @@ static int calculate_filter (filter_info *finfo)
 	err = bkbp_filter(x, u, datainfo);
 	unset_bkbp_k();
 	unset_bkbp_periods();
+    } else if (finfo->ftype == FILTER_FD) {
+	err = fracdiff_series(x, fx, finfo->lambda, 1, -1, datainfo);
     }
 	
     if (!err && fx != NULL && u != NULL) {
@@ -850,6 +912,8 @@ static int filter_code (GtkAction *action)
 	return FILTER_HP;
     else if (!strcmp(s, "FilterBK")) 
 	return FILTER_BK;
+    else if (!strcmp(s, "FilterFD")) 
+	return FILTER_FD;
     else
 	return FILTER_SMA;
 }
@@ -863,10 +927,6 @@ void filter_callback (GtkAction *action)
     int code, cancel = 0;
     int err = 0;
 
-    if (reject_scalar(v)) {
-	return;
-    }
-
     code = filter_code(action);
 
     err = array_adjust_t1t2(Z[v], &t1, &t2);
