@@ -40,7 +40,8 @@ struct call_info_ {
     GList *lsels; /* list arg selectors */
     GList *msels; /* matrix arg selectors */
     GList *ssels; /* string arg selectors */
-    int iface;
+    int *publist; /* list of public interfaces */
+    int iface;    /* selected interface */
     int extracol;
     const ufunc *func;
     int n_params;
@@ -54,6 +55,10 @@ struct call_info_ {
 #define series_arg(t) (t == GRETL_TYPE_SERIES || t == GRETL_TYPE_SERIES_REF)
 #define matrix_arg(t) (t == GRETL_TYPE_MATRIX || t == GRETL_TYPE_MATRIX_REF)
 
+static GtkWidget *open_fncall_dlg;
+
+static void fncall_OK_callback (GtkWidget *w, call_info *cinfo);
+
 static call_info *cinfo_new (void)
 {
     call_info *cinfo = mymalloc(sizeof *cinfo);
@@ -62,6 +67,7 @@ static call_info *cinfo_new (void)
 	return NULL;
     }
 
+    cinfo->publist = NULL;
     cinfo->iface = -1;
 
     cinfo->lsels = NULL;
@@ -112,7 +118,8 @@ static void cinfo_free (call_info *cinfo)
     if (cinfo->msels != NULL) {
 	g_list_free(cinfo->msels);
     }
-
+    
+    free(cinfo->publist);
     free(cinfo);
 }
 
@@ -121,17 +128,15 @@ static const char *arg_type_string (int t)
     if (t == GRETL_TYPE_BOOL)   return "boolean";
     if (t == GRETL_TYPE_INT)    return "int";
     if (t == GRETL_TYPE_LIST)   return "list";
-
     if (t == GRETL_TYPE_DOUBLE) return "scalar";
     if (t == GRETL_TYPE_SERIES) return "series";
     if (t == GRETL_TYPE_MATRIX) return "matrix";
+    if (t == GRETL_TYPE_STRING) return "string";
     
     if (t == GRETL_TYPE_SCALAR_REF) return "scalar *";
     if (t == GRETL_TYPE_SERIES_REF) return "series *";
     if (t == GRETL_TYPE_MATRIX_REF) return "matrix *";
 
-    if (t == GRETL_TYPE_STRING) return "string";
-
     return "";
 }
 
@@ -154,13 +159,14 @@ static int check_args (call_info *cinfo)
     return 0;
 }
 
-static void fncall_finalize (GtkWidget *w, call_info *cinfo)
+static void fncall_dialog_destruction (GtkWidget *w, call_info *cinfo)
 {
-    if (check_args(cinfo)) {
-	return;
+    if (!cinfo->ok) {
+	/* don't free cinfo if we're going to use it */
+	cinfo_free(cinfo);
     }
-    cinfo->ok = 1;
-    gtk_widget_destroy(cinfo->dlg);
+
+    open_fncall_dlg = NULL;
 }
 
 static void fncall_cancel (GtkWidget *w, call_info *cinfo)
@@ -168,12 +174,13 @@ static void fncall_cancel (GtkWidget *w, call_info *cinfo)
     gtk_widget_destroy(cinfo->dlg);
 }
 
-static GtkWidget *label_hbox (GtkWidget *w, const char *txt, int center)
+static GtkWidget *label_hbox (GtkWidget *w, const char *txt, 
+			      int vspace, int center)
 {
     GtkWidget *hbox, *label;
 
     hbox = gtk_hbox_new(FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(w), hbox, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(w), hbox, FALSE, FALSE, vspace);
 
     label = gtk_label_new(txt);
     if (center) {
@@ -189,8 +196,7 @@ static GtkWidget *label_hbox (GtkWidget *w, const char *txt, int center)
 static gboolean update_int_arg (GtkWidget *w, call_info *cinfo)
 {
     int val = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(w));
-    int i = 
-	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "argnum"));
+    int i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "argnum"));
 
     free(cinfo->args[i]);
     cinfo->args[i] = g_strdup_printf("%d", val);
@@ -200,8 +206,7 @@ static gboolean update_int_arg (GtkWidget *w, call_info *cinfo)
 
 static gboolean update_bool_arg (GtkWidget *w, call_info *cinfo)
 {
-    int i = 
-	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "argnum"));
+    int i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "argnum"));
 
     free(cinfo->args[i]);
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
@@ -242,8 +247,7 @@ static char *combo_box_get_trimmed_text (GtkComboBox *combo)
 static gboolean update_arg (GtkComboBox *combo, 
 			    call_info *cinfo)
 {
-    int i = 
-	GPOINTER_TO_INT(g_object_get_data(G_OBJECT(combo), "argnum"));
+    int i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(combo), "argnum"));
     char *s;
 
     free(cinfo->args[i]);
@@ -274,8 +278,6 @@ static GList *get_selection_list (call_info *cinfo, int i, int type,
     const char *name;
     int n, optional = 0;
 
-    /* FIXME int, bool arguments */
-
     if (i >= 0) {
 	optional = fn_param_optional(cinfo->func, i);
     }
@@ -325,8 +327,7 @@ static GList *get_selection_list (call_info *cinfo, int i, int type,
 
 static void fncall_help (GtkWidget *w, call_info *cinfo)
 {
-    const char *fnname = 
-	user_function_name_by_index(cinfo->iface);
+    const char *fnname = user_function_name_by_index(cinfo->iface);
     PRN *prn;
     int err;
 
@@ -339,7 +340,7 @@ static void fncall_help (GtkWidget *w, call_info *cinfo)
 	gretl_print_destroy(prn);
 	dummy_call();
     } else {
-	view_buffer(prn, 80, 400, fnname, VIEW_FUNC_INFO, NULL);
+	view_buffer(prn, 80, 400, fnname, VIEW_PKG_INFO, NULL);
     }
 }
 
@@ -377,6 +378,7 @@ static void update_matrix_selectors (call_info *cinfo)
     while (slist != NULL) {
 	sel = GTK_COMBO_BOX(slist->data);
 	saved = gtk_combo_box_get_active_text(sel);
+	depopulate_combo_box(sel);
 	set_combo_box_strings_from_list(sel, mlist);
 	if (saved != NULL) {
 	    old = combo_list_index(saved, mlist);
@@ -389,34 +391,54 @@ static void update_matrix_selectors (call_info *cinfo)
     g_list_free(mlist);
 }
 
+static int combo_accepts_null (GtkComboBox *c)
+{
+    gpointer p = g_object_get_data(G_OBJECT(c), "null_OK");
+
+    return (p != NULL);
+}
+
 static void update_list_selectors (call_info *cinfo)
 {
     GList *slist = cinfo->lsels;
-    GList *llist = NULL;
+    GList *llist1 = NULL;
+    GList *llist2 = NULL;
     GtkComboBox *sel;
     const char *lname;
     gchar *saved;
     int nl = n_saved_lists();
+    int null_OK;
     int i, old;
 
+    llist2 = g_list_append(llist2, "null");
+
     for (i=0; i<nl; i++) {
 	lname = get_list_name_by_index(i);
-	llist = g_list_append(llist, (gpointer) lname);
+	llist1 = g_list_append(llist1, (gpointer) lname);
+	llist2 = g_list_append(llist2, (gpointer) lname);
     }
 
     while (slist != NULL) {
+	GList *llist;
+
 	sel = GTK_COMBO_BOX(slist->data);
+	null_OK = combo_accepts_null(sel);
+	llist = (null_OK)? llist2 : llist1; 
 	saved = gtk_combo_box_get_active_text(sel);
+	depopulate_combo_box(sel);
 	set_combo_box_strings_from_list(sel, llist);
 	if (saved != NULL) {
 	    old = combo_list_index(saved, llist);
 	    gtk_combo_box_set_active(sel, (old >= 0)? old : 0);
 	    g_free(saved);
+	} else if (null_OK) {
+	    gtk_combo_box_set_active(sel, 0);
 	}
 	slist = slist->next;
     }
 
-    g_list_free(llist);
+    g_list_free(llist1);
+    g_list_free(llist2);
 }
 
 int do_make_list (selector *sr)
@@ -479,8 +501,13 @@ int do_make_list (selector *sr)
 
 static void launch_list_maker (GtkWidget *w, GtkWidget *entry)
 {
-    simple_selection(_("Define list"), do_make_list, DEFINE_LIST, 
-		     entry);
+    selector *sr;
+    GtkWidget *dlg;
+
+    sr = simple_selection(_("Define list"), do_make_list, DEFINE_LIST, 
+			  entry);
+    dlg = selector_get_window(sr);
+    gtk_window_set_keep_above(GTK_WINDOW(dlg), TRUE);
 }
 
 static void launch_matrix_maker (GtkWidget *w, call_info *cinfo)
@@ -493,7 +520,7 @@ static void launch_matrix_maker (GtkWidget *w, call_info *cinfo)
 	update_matrix_selectors(cinfo);
     }
 
-    gdk_window_raise(cinfo->dlg->window);
+    gtk_window_present(GTK_WINDOW(cinfo->dlg));
 }
 
 static int spinner_arg (call_info *cinfo, int i)
@@ -557,6 +584,10 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
 		     G_CALLBACK(update_arg), cinfo);
     gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
 
+    if (i >= 0 && fn_param_optional(cinfo->func, i)) {
+	g_object_set_data(G_OBJECT(combo), "null_OK", GINT_TO_POINTER(1));
+    }
+
     list = get_selection_list(cinfo, i, ptype, 1);
     if (list != NULL) {
 	set_combo_box_strings_from_list(GTK_COMBO_BOX(combo), list);
@@ -580,14 +611,50 @@ static GtkWidget *combo_arg_selector (call_info *cinfo, int ptype, int i)
     return combo;
 }
 
+static void add_table_hsep (GtkWidget *tbl, int cols, int r0)
+{
+    GtkWidget *hsep = gtk_hseparator_new();
+
+    gtk_table_attach(GTK_TABLE(tbl), hsep, 0, cols, r0, r0 + 1,
+		     GTK_FILL, GTK_FILL, 5, 5);
+}
+
+static void add_table_header (GtkWidget *tbl, gchar *txt,
+			      int cols, int r0)
+{
+    GtkWidget *label = gtk_label_new(txt);
+    GtkWidget *align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
+
+    gtk_container_add(GTK_CONTAINER(align), label);
+    gtk_table_attach(GTK_TABLE(tbl), align, 0, cols, r0, r0 + 1,
+		     GTK_FILL, GTK_FILL, 5, 5);
+}
+
+static void add_table_cell (GtkWidget *tbl, GtkWidget *w,
+			    int c0, int c1, int r0)
+{
+    gtk_table_attach(GTK_TABLE(tbl), w, c0, c1, r0, r0 + 1,
+		     GTK_FILL, GTK_FILL, 5, 3);
+}
+
+#define cinfo_has_return(c) (c->rettype != GRETL_TYPE_NONE && \
+			     c->rettype != GRETL_TYPE_VOID)
+
 static void function_call_dialog (call_info *cinfo)
 {
     GtkWidget *button, *label;
-    GtkWidget *tbl, *hbox;
-    GtkWidget *sel;
+    GtkWidget *sel, *tbl = NULL;
+    GtkWidget *vbox, *hbox, *bbox;
     gchar *txt;
     const char *fnname;
-    int i, err;
+    int trows = 0, tcols = 0;
+    int i, row = 0;
+    int err;
+
+    if (open_fncall_dlg != NULL) {
+	gtk_window_present(GTK_WINDOW(open_fncall_dlg));
+	return;
+    }
 
     err = cinfo_args_init(cinfo);
     if (err) {
@@ -595,30 +662,36 @@ static void function_call_dialog (call_info *cinfo)
 	return;
     }
 
-    cinfo->dlg = gretl_dialog_new(_("gretl: function call"), NULL, 
-				  GRETL_DLG_BLOCK | GRETL_DLG_RESIZE);
+    cinfo->dlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_title(GTK_WINDOW(cinfo->dlg), ("gretl: function call"));
+    gretl_emulated_dialog_add_structure(cinfo->dlg, &vbox, &bbox);
+    open_fncall_dlg = cinfo->dlg;
+    g_signal_connect(G_OBJECT(cinfo->dlg), "destroy",
+		     G_CALLBACK(fncall_dialog_destruction), cinfo);
 
     fnname = user_function_name_by_index(cinfo->iface);
     txt = g_strdup_printf(_("Call to function %s"), fnname);
-    hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, txt, 1);
-    gtk_widget_show(hbox);
-
-    /* function argument selection */
+    hbox = label_hbox(vbox, txt, 5, 1);
+    g_free(txt);
 
     if (cinfo->n_params > 0) {
-	int tcols = (cinfo->extracol)? 4 : 3;
+	tcols = (cinfo->extracol)? 3 : 2;
+	trows = cinfo->n_params + 1;
+	if (cinfo_has_return(cinfo)) { 
+	    trows += 4;
+	}
+    } else if (cinfo_has_return(cinfo)) {
+	tcols = 2;
+	trows = 3;
+    }
 
-	hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, _("Arguments:"), 0);
-	gtk_widget_show(hbox);
+    if (trows > 0 && tcols > 0) {
+	tbl = gtk_table_new(trows, tcols, FALSE);
+    }
 
-	tbl = gtk_table_new(cinfo->n_params + 1, tcols, FALSE);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cinfo->dlg)->vbox),
-			   tbl, FALSE, FALSE, 5);
+    if (cinfo->n_params > 0) {
 
-	label = gtk_label_new(_("selection"));
-	gtk_table_attach(GTK_TABLE(tbl), label, 2, 3, 0, 1,
-			 GTK_EXPAND, GTK_FILL, 5, 5);
-	gtk_widget_show(label);
+	add_table_header(tbl, _("Select arguments:"), tcols, row);
 
 	for (i=0; i<cinfo->n_params; i++) {
 	    const char *desc = fn_param_descrip(cinfo->func, i);
@@ -627,16 +700,16 @@ static void function_call_dialog (call_info *cinfo)
 	    if (desc != NULL) {
 		label = gtk_label_new(desc);
 	    } else {
-		gchar *tmp = g_strdup_printf("%s (%s)",
-					     fn_param_name(cinfo->func, i), 
-					     arg_type_string(ptype));
-		label = gtk_label_new(tmp);
-		g_free(tmp);			     
+		txt = g_strdup_printf("%s (%s)",
+				      fn_param_name(cinfo->func, i), 
+				      arg_type_string(ptype));
+		label = gtk_label_new(txt);
+		g_free(txt);			     
 	    }
 
-	    gtk_table_attach(GTK_TABLE(tbl), label, 0, 2, i+1, i+2,
-			     GTK_EXPAND, GTK_FILL, 5, 5);
-	    gtk_widget_show(label);
+	    row++;
+	    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+	    add_table_cell(tbl, label, 0, 1, row);
 
 	    if (ptype == GRETL_TYPE_BOOL) {
 		sel = bool_arg_selector(cinfo, i);
@@ -646,70 +719,45 @@ static void function_call_dialog (call_info *cinfo)
 		sel = combo_arg_selector(cinfo, ptype, i);
 	    }
 
-	    gtk_table_attach(GTK_TABLE(tbl), sel, 2, 3, i+1, i+2,
-			     GTK_EXPAND, GTK_FILL, 5, 5);
-	    gtk_widget_show(sel);
+	    add_table_cell(tbl, sel, 1, 2, row);
 
 	    if (ptype == GRETL_TYPE_LIST) {
 		cinfo->lsels = g_list_append(cinfo->lsels, sel);
 		button = gtk_button_new_with_label(_("More..."));
-		gtk_table_attach(GTK_TABLE(tbl), button, 3, 4, i+1, i+2,
-				 GTK_EXPAND, GTK_FILL, 5, 5);
+		add_table_cell(tbl, button, 2, 3, row);
 		g_signal_connect(G_OBJECT(button), "clicked", 
 				 G_CALLBACK(launch_list_maker),
 				 gtk_bin_get_child(GTK_BIN(sel)));
-		gtk_widget_show(button);
 	    } else if (ptype == GRETL_TYPE_MATRIX) {
 		cinfo->msels = g_list_append(cinfo->msels, sel);
 		button = gtk_button_new_with_label(_("New..."));
-		gtk_table_attach(GTK_TABLE(tbl), button, 3, 4, i+1, i+2,
-				 GTK_EXPAND, GTK_FILL, 5, 5);
+		add_table_cell(tbl, button, 2, 3, row);
 		g_signal_connect(G_OBJECT(button), "clicked", 
 				 G_CALLBACK(launch_matrix_maker), 
 				 cinfo);
-		gtk_widget_show(button);
 	    } 
 	}
 
-	gtk_widget_show(tbl);
+	if (cinfo_has_return(cinfo)) {
+	    row++;
+	    add_table_hsep(tbl, tcols, row++);
+	}
     }
-
-    if (cinfo->n_params > 0 && cinfo->rettype != GRETL_TYPE_NONE) {
-	GtkWidget *hsep = gtk_hseparator_new();
-
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cinfo->dlg)->vbox),
-			   hsep, FALSE, FALSE, 5);
-	gtk_widget_show(hsep);
-    }
-
-    /* function return(s) assignment */
-
-    if (cinfo->rettype != GRETL_TYPE_NONE) {
+	
+    if (cinfo_has_return(cinfo)) {
+	GtkWidget *child;
 	GList *list = NULL;
 
-	hbox = label_hbox(GTK_DIALOG(cinfo->dlg)->vbox, 
-			  _("Assign return value (optional):"), 0);
-	gtk_widget_show(hbox);
-
-	tbl = gtk_table_new(1, 2, FALSE);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cinfo->dlg)->vbox),
-			   tbl, FALSE, FALSE, 5);
-
-	label = gtk_label_new("type");
-	gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, 0, 1,
-			 GTK_EXPAND, GTK_FILL, 5, 5);
-	gtk_widget_show(label);
+	add_table_header(tbl, _("Assign return value (optional):"), tcols, row);
+	row++;
 
 	label = gtk_label_new(_("selection (or new variable)"));
-	gtk_table_attach(GTK_TABLE(tbl), label, 1, 2, 0, 1,
-			 GTK_EXPAND, GTK_FILL, 5, 5);
-	gtk_widget_show(label);
+	add_table_cell(tbl, label, 1, 2, row);
+	row++;
 
 	label = gtk_label_new(arg_type_string(cinfo->rettype));
-	gtk_misc_set_alignment(GTK_MISC(label), 0.75, 0.5);
-	gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, 1, 2,
-			 GTK_EXPAND, GTK_FILL, 5, 5);
-	gtk_widget_show(label);
+	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+	add_table_cell(tbl, label, 0, 1, row);
 
 	sel = gtk_combo_box_entry_new_text();
 	g_signal_connect(G_OBJECT(sel), "changed",
@@ -719,35 +767,34 @@ static void function_call_dialog (call_info *cinfo)
 	    set_combo_box_strings_from_list(GTK_COMBO_BOX(sel), list);
 	    g_list_free(list);
 	}
-	gtk_table_attach(GTK_TABLE(tbl), sel, 1, 2, 1, 2,
-			 GTK_EXPAND, GTK_FILL, 5, 5);
-	gtk_widget_show(sel);
+	child = gtk_bin_get_child(GTK_BIN(sel));
+	gtk_entry_set_activates_default(GTK_ENTRY(child), TRUE);
+	add_table_cell(tbl, sel, 1, 2, row);
+    }
 
-	gtk_widget_show(tbl);
+    if (tbl != NULL) {
+	gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0);
     }
 
     /* Cancel button */
-    button = cancel_button(GTK_DIALOG(cinfo->dlg)->action_area);
+    button = cancel_button(bbox);
     g_signal_connect(G_OBJECT (button), "clicked", 
 		     G_CALLBACK(fncall_cancel), cinfo);
-    gtk_widget_show(button);
 
     /* "OK" button */
-    button = ok_button(GTK_DIALOG(cinfo->dlg)->action_area);
+    button = ok_button(bbox);
     g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(fncall_finalize), cinfo);
+		     G_CALLBACK(fncall_OK_callback), cinfo);
     gtk_widget_grab_default(button);
-    gtk_widget_show(button);
 
     /* Help button? */
     if (cinfo->n_params > 0 || cinfo->rettype != GRETL_TYPE_NONE) {
-	button = context_help_button(GTK_DIALOG(cinfo->dlg)->action_area, -1);
+	button = context_help_button(bbox, -1);
 	g_signal_connect(G_OBJECT(button), "clicked", 
 			 G_CALLBACK(fncall_help), cinfo);
-	gtk_widget_show(button);
     }  
 
-    gtk_widget_show(cinfo->dlg);
+    gtk_widget_show_all(cinfo->dlg);
 }
 
 static int function_data_check (call_info *cinfo)
@@ -783,35 +830,6 @@ static int function_data_check (call_info *cinfo)
     return err;
 }
 
-static int temp_install_remote_fnpkg (const char *fname, char *target)
-{
-    int err = 0;
-
-    build_path(target, paths.dotdir, "dltmp", NULL);
-    err = gretl_tempname(target);
-    if (err) {
-	return err;
-    }
-
-    err = retrieve_remote_function_package(fname, target);
-    if (err) {
-	show_network_error(NULL);
-	return err;
-    } 
-
-    err = load_user_function_file(target);
-    if (err) {
-	fprintf(stderr, "load_user_function_file: failed on %s\n", target);
-	file_read_errbox(target);
-    }
-
-    if (err) {
-	gretl_remove(target);
-    }
-
-    return err;
-}
-
 /* detect the case where we need a "pointer" variable but
    have been given a scalar or matrix constant 
 */
@@ -909,116 +927,15 @@ static int pre_process_args (call_info *cinfo, PRN *prn)
     return err;
 }
 
-void call_function_package (const char *fname, GtkWidget *w,
-			    int *loaderr)
+static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 {
     ExecState state;
-    char tmpfile[FILENAME_MAX];
     char fnline[MAXLINE];
-    const char *fnname;
-    FuncDataReq dreq;
-    int minver;
-    PRN *prn = NULL;
-    call_info *cinfo = NULL;
-    int orig_v;
+    const char *funname;
+    int orig_v = datainfo->v;
     int i, err = 0;
 
-    *tmpfile = 0;
-
-    if (strstr(fname, ".gfn") == NULL) {
-	/* not a full filename -> a function package on server */
-	err = temp_install_remote_fnpkg(fname, tmpfile);
-    } else {
-	if (!function_package_is_loaded(fname)) {
-	    err = load_user_function_file(fname);
-	    if (err) {
-		file_read_errbox(fname);
-		*loaderr = 1;
-	    }
-	}
-    }
-
-    if (err) {
-	return;
-    }
-
-    cinfo = cinfo_new();
-    if (cinfo == NULL) {
-	return;
-    }
-
-    /* get interface for package */
-    err = function_package_get_info((*tmpfile)? tmpfile : fname,
-				    NULL,
-				    &cinfo->iface,
-				    NULL,
-				    NULL,
-				    NULL,
-				    NULL,
-				    NULL,
-				    NULL,
-				    &dreq,
-				    &minver);
-
-    if (*tmpfile) {
-	gretl_remove(tmpfile);
-    }
-
-    if (cinfo->iface < 0) {
-	errbox(_("Function package is broken"));
-	goto bailout;
-    }
-
-    err = check_function_needs(datainfo, dreq, minver);
-    if (err) {
-	gui_errmsg(err);
-	goto bailout;
-    }
-
-    cinfo->func = get_user_function_by_index(cinfo->iface);
-
-    if (cinfo->func == NULL) {
-	fprintf(stderr, "get_user_function_by_index: got NULL for idx = %d\n", 
-		cinfo->iface);
-	errbox(_("Couldn't get function package information"));
-	goto bailout;
-    }
-
-    cinfo->n_params = fn_n_params(cinfo->func);
-
-    if (function_data_check(cinfo)) {
-	goto bailout;
-    }
-
-    cinfo->rettype = user_func_get_return_type(cinfo->func);
-
-    if (err) {
-	fprintf(stderr, "user_func_get_return_types: failed for idx = %d\n",
-		cinfo->iface);
-	errbox(_("Couldn't get function package information"));
-	goto bailout;
-    }
-
-    function_call_dialog(cinfo);
-
-    if (!cinfo->ok || check_args(cinfo) || bufopen(&prn)) {
-	goto bailout;
-    }
-
-#if 0
-    /* kill the launcher window? */
-    gtk_widget_destroy(w);
-#endif
-
-    if (cinfo->args != NULL) {
-	err = pre_process_args(cinfo, prn);
-	if (err) {
-	    gui_errmsg(err);
-	    goto bailout;
-	}
-    }
-
-    fnname = user_function_name_by_index(cinfo->iface);
+    funname = user_function_name_by_index(cinfo->iface);
     *fnline = 0;
 
     if (cinfo->ret != NULL) {
@@ -1026,9 +943,7 @@ void call_function_package (const char *fname, GtkWidget *w,
 	strcat(fnline, " = ");
     }    
 
-    strcat(fnline, fnname);
-    orig_v = datainfo->v;
-
+    strcat(fnline, funname);
     strcat(fnline, "(");
 
     if (cinfo->args != NULL) {
@@ -1038,18 +953,21 @@ void call_function_package (const char *fname, GtkWidget *w,
 		strcat(fnline, ", ");
 	    }
 	}
-    } 
+    }
 
     /* destroy any "ARG" vars or matrices that were created? */
 
     strcat(fnline, ")");
     pprintf(prn, "? %s\n", fnline);
 
-    gretl_exec_state_init(&state, SCRIPT_EXEC, fnline, get_lib_cmd(),
+    gretl_exec_state_init(&state, SCRIPT_EXEC, NULL, get_lib_cmd(),
 			  models, prn);
 
+    /* note: gretl_exec_state_init zeros the first byte of the
+       supplied 'line' */
+    state.line = fnline;
     err = gui_exec_line(&state, &Z, datainfo);
-    view_buffer(prn, 80, 400, fnname, PRINT, NULL);
+    view_buffer(prn, 80, 400, funname, PRINT, NULL);
 
     if (err) {
 	gui_errmsg(err);
@@ -1060,12 +978,188 @@ void call_function_package (const char *fname, GtkWidget *w,
 	populate_varlist();
     }
 
-    cinfo_free(cinfo);
+    return err;
+}
 
-    return;
+/* In case a function package offers more than one public
+   interface, put up a radio-button selector */
 
- bailout:
+static void select_interface (call_info *cinfo)
+{
+    const char *funname;
+    char **opts = NULL;
+    int nopts = 0;
+    int i, err = 0;
 
-    cinfo_free(cinfo);
-    gretl_print_destroy(prn);
+    for (i=1; i<=cinfo->publist[0] && !err; i++) {
+	funname = user_function_name_by_index(cinfo->publist[i]);
+	if (funname == NULL) {
+	    err = E_DATA;
+	} else {
+	    err = strings_array_add(&opts, &nopts, funname);
+	}
+    }
+
+    if (err) {
+	cinfo->iface = -1;
+	gui_errmsg(err);
+    } else {
+	int resp = radio_dialog("gretl", "select function", 
+				(const char **) opts, 
+				nopts, 0, 0);
+
+	if (resp >= 0) {
+	    cinfo->iface = cinfo->publist[resp+1];
+	} else {
+	    cinfo->iface = -1;
+	}
+    }
+
+    free_strings_array(opts, nopts);
+}
+
+/* Callback from "OK" button in function call GUI: if there's a
+   problem with the argument selection just return so the dialog stays
+   in place and the user can correct matters; otherwise close the
+   dialog and execute the function, then clean up.  (Note that we
+   need to set cinfo->ok so that the cinfo structure won't get
+   destroyed along with the dialog.)
+*/
+
+static void fncall_OK_callback (GtkWidget *w, call_info *cinfo)
+{
+    if (check_args(cinfo)) {
+	return;
+    } else {
+	PRN *prn = NULL;
+	int err;
+
+	cinfo->ok = 1; /* flag preservation of cinfo */
+	gtk_widget_destroy(cinfo->dlg);
+
+	err = bufopen(&prn);
+
+	if (!err && cinfo->args != NULL) {
+	    err = pre_process_args(cinfo, prn);
+	    if (err) {
+		gui_errmsg(err);
+	    }
+	}
+
+	if (!err) {
+	    err = real_GUI_function_call(cinfo, prn);
+	} else {
+	    gretl_print_destroy(prn);
+	}
+
+	cinfo_free(cinfo);
+    }
+}
+
+/* call to execute a function from the specified package: we do
+   this only for locally installed packages */
+
+void call_function_package (const char *fname, GtkWidget *w,
+			    int *loaderr)
+{
+    FuncDataReq dreq = 0;
+    int minver = 0;
+    call_info *cinfo;
+    fnpkg *pkg;
+    int err = 0;
+
+    pkg = get_function_package_by_filename(fname);
+
+    if (pkg == NULL) {
+	/* not already loaded */
+	err = load_function_package_from_file(fname);
+	if (err) {
+	    file_read_errbox(fname);
+	    *loaderr = 1;
+	} else {
+	    /* should be OK now */
+	    pkg = get_function_package_by_filename(fname);
+	}
+    }
+
+    if (err || pkg == NULL) {
+	return;
+    }
+
+    cinfo = cinfo_new();
+    if (cinfo == NULL) {
+	return;
+    }
+
+    /* get the interface list and other info for package */
+
+    err = function_package_get_properties(pkg,
+					  "publist", &cinfo->publist,
+					  "data-requirement", &dreq,
+					  "min-version", &minver,
+					  NULL);
+
+    if (err) {
+	gui_errmsg(err);
+    } else if (cinfo->publist == NULL) {
+	/* no available interfaces */
+	err = E_DATA;
+	errbox(_("Function package is broken"));
+    }
+
+    if (!err) {
+	/* do we have suitable data in place? */
+	err = check_function_needs(datainfo, dreq, minver);
+	if (err) {
+	    gui_errmsg(err);
+	}
+    }
+
+    if (!err) {
+	if (cinfo->publist[0] > 1) {
+	    select_interface(cinfo);
+	    if (cinfo->iface < 0) {
+		/* failed, or cancelled */
+		cinfo_free(cinfo);
+		return; /* note: handled */
+	    }
+	} else {
+	    /* only one interface available */
+	    cinfo->iface = cinfo->publist[1];
+	}
+    }
+
+    if (!err) {
+	cinfo->func = get_user_function_by_index(cinfo->iface);
+	if (cinfo->func == NULL) {
+	    fprintf(stderr, "get_user_function_by_index: failed\n");
+	    errbox(_("Couldn't get function package information"));
+	}
+    }	
+    
+    if (!err) {
+	cinfo->n_params = fn_n_params(cinfo->func);
+	err = function_data_check(cinfo);
+    }
+
+    if (!err) {
+	cinfo->rettype = user_func_get_return_type(cinfo->func);
+	if (err) {
+	    fprintf(stderr, "user_func_get_return_type: failed\n");
+	    errbox(_("Couldn't get function package information"));
+	}
+    }
+
+    if (!err) {
+	function_call_dialog(cinfo);
+    } else {
+	cinfo_free(cinfo);
+    }
+}
+
+void function_call_cleanup (void)
+{
+    if (open_fncall_dlg != NULL) {
+	gtk_widget_destroy(open_fncall_dlg);
+    }
 }
