@@ -26,9 +26,13 @@ typedef struct png_plot_t png_plot;
 
 int remove_png_term_from_plotfile_by_name (const char *fname);
 
-void display_session_graph_png (const char *pltname);
+void display_session_graph (const char *path, const char *title);
 
-int gnuplot_show_png_by_name (const char *fname);
+int display_graph_file (const char *fname);
+
+void saver_preview_graph (GPT_SPEC *spec, char *termstr);
+
+int saver_save_graph (GPT_SPEC *spec, char *termstr, const char *fname);
 
 void plot_label_position_click (GtkWidget *w, png_plot *plot);
 
@@ -36,6 +40,8 @@ int redisplay_edited_plot (png_plot *plot);
 
 void set_plot_has_y2_axis (png_plot *plot, gboolean s);
 
+int plot_is_saved (const png_plot *plot);
+
 int plot_is_mouseable (const png_plot *plot);
 
 GtkWidget *plot_get_shell (png_plot *plot);
@@ -48,7 +54,7 @@ GPT_SPEC *plot_get_spec (png_plot *plot);
 
 void revise_distribution_plotspec (png_plot *plot, int d, int df1, int df2);
 
-int gp_term_code (gpointer p);
+int gp_term_code (gpointer p, int action);
 
 void save_graph_to_file (gpointer p, const char *fname);
 
@@ -60,4 +66,6 @@ void run_gp_script (gchar *buf);
 int dump_plot_buffer (const char *buf, const char *fname,
 		      int addpause);
 
+void close_plot_windows (void);
+
 #endif /* GPT_CONTROL_H */
