@@ -22,6 +22,7 @@
 #include "gretl.h"
 #include "plotspec.h"
 #include "gpt_control.h"
+#include "graphics.h"
 #include "session.h"
 #include "dlgutils.h"
 #include "fileselect.h"
@@ -94,6 +95,7 @@ struct plot_editor_ {
     GtkWidget *y2_check;
     GtkWidget *ttfcombo;
     GtkWidget *ttfspin;
+    GtkWidget *fontcheck;
 
     int gui_nlines;
     int gui_nlabels;
@@ -327,16 +329,22 @@ static GtkWidget *color_patch_button (int i)
     return button;
 }
 
-static int style_from_line_number (GPT_SPEC *spec, int i)
+static int style_from_line_number (GPT_SPEC *spec, int i,
+				   int *dotted)
 {
     int j, lt = spec->lines[i].type;
 
     if (lt > 0) {
 	j = lt - 1;
-    } else if (lt == LT_NONE) {
+    } else if (lt == LT_AUTO) {
+	j = i;
+    } else if (lt == 0) {
+	if (dotted != NULL) {
+	    *dotted = 1;
+	}
 	j = i;
     } else {
-	j = LT_NONE;
+	j = LT_AUTO;
     }
 
     return j;
@@ -345,9 +353,11 @@ static int style_from_line_number (GPT_SPEC *spec, int i)
 static GtkWidget *line_color_button (GPT_SPEC *spec, int i)
 {
     GtkWidget *image, *button;
-    int j = style_from_line_number(spec, i);
+    int dotted = 0;
+    int j = style_from_line_number(spec, i, &dotted);
 
-    if (j == LT_NONE) {
+    if (j < 0) {
+	/* shouldn't happen */
 	return NULL;
     }
 
@@ -371,7 +381,11 @@ static GtkWidget *line_color_button (GPT_SPEC *spec, int i)
 	g_signal_connect(G_OBJECT(button), "clicked", 
 			 G_CALLBACK(graph_color_selector), 
 			 GINT_TO_POINTER(j));
-    }	
+    }
+
+    if (dotted) {
+	gtk_widget_set_sensitive(button, FALSE);
+    }
 
     return button;
 }
@@ -385,12 +399,12 @@ static void maybe_apply_line_color (GtkWidget *w, GPT_SPEC *spec,
 
     cb = g_object_get_data(G_OBJECT(w), "colorsel");
 
-    if (cb != NULL) {
+    if (cb != NULL && GTK_WIDGET_SENSITIVE(cb)) {
 	rgb = g_object_get_data(G_OBJECT(cb), "rgb");
     }
 
     if (rgb != NULL) {
-	j = style_from_line_number(spec, i);
+	j = style_from_line_number(spec, i, NULL);
 	print_rgb_hash(spec->lines[j].rgb, rgb);
     }
 }
@@ -431,7 +445,7 @@ static GdkPixbuf *get_pixbuf_for_line (int dots)
 static void flip_manual_range (GtkWidget *widget, plot_editor *ed)
 {
     gint i = widget_get_int(widget, "axis");
-    gboolean s = GTK_TOGGLE_BUTTON(ed->axis_range[i].isauto)->active;
+    gboolean s = button_is_active(ed->axis_range[i].isauto);
 
     gtk_widget_set_sensitive(ed->axis_range[i].min, !s);
     gtk_widget_set_sensitive(ed->axis_range[i].max, !s);
@@ -500,11 +514,18 @@ static gboolean fit_type_changed (GtkComboBox *box, plot_editor *ed)
     gchar *title = NULL;
     FitType f = PLOT_FIT_NONE;
 
+    f = gtk_combo_box_get_active(box);
+
+    if ((spec->flags & GPT_TS) && f != PLOT_FIT_NONE && ed->keycombo != NULL) {
+	if (!GTK_WIDGET_SENSITIVE(ed->keycombo)) {
+	    gtk_widget_set_sensitive(ed->keycombo, TRUE);
+	    gtk_combo_box_set_active(GTK_COMBO_BOX(ed->keycombo), 0);
+	}
+    }	
+
     if (*s1 == '\0' || *s2 == '\0') {
 	return FALSE;
-    }
-
-    f = gtk_combo_box_get_active(box);
+    }    
 
     if (f == PLOT_FIT_OLS) {
 	title = g_strdup_printf(_("%s versus %s (with least squares fit)"),
@@ -518,6 +539,8 @@ static gboolean fit_type_changed (GtkComboBox *box, plot_editor *ed)
     } else if (f == PLOT_FIT_LOESS) {
 	title = g_strdup_printf(_("%s versus %s (with loess fit)"),
 		s1, s2);
+    } else {
+	title = g_strdup("");
     }
 
     if (title != NULL) {
@@ -525,8 +548,10 @@ static gboolean fit_type_changed (GtkComboBox *box, plot_editor *ed)
 	g_free(title);
     }
 
-    /* also re-jig the "Lines" tab entries for the fitted
-       line */
+    /* FIXME: when changing from no fit case to showing a fit,
+       adjust the key position option, which may be disabled */
+
+    /* also re-jig the "Lines" tab entries for the fitted line */
 
     if (ed->fitformula != NULL && ed->fitlegend != NULL) {
 	fittype_from_combo(box, spec);
@@ -540,17 +565,19 @@ static gboolean fit_type_changed (GtkComboBox *box, plot_editor *ed)
 static void dot_callback (GtkComboBox *box, GPT_SPEC *spec)
 {
     int i = widget_get_int(GTK_WIDGET(box), "linenum");
-    int d = gtk_combo_box_get_active(box);
+    int solid = (gtk_combo_box_get_active(box) == 0);
     GtkWidget *w;
 
-    /* flip between colored solid line and black dotted */
-
-    spec->lines[i].type = (d > 0)? 0 : LT_NONE;
+    /* flip between colored solid line (the first option)
+       and a dotted black line (second option) 
+    */
+    spec->lines[i].type = solid ? LT_AUTO : 0;
 
+    /* the color selector is effective only for solid lines */
     w = g_object_get_data(G_OBJECT(box), "colorsel");
-    gtk_widget_set_sensitive(w, d == 0);
+    gtk_widget_set_sensitive(w, solid);
     w = g_object_get_data(G_OBJECT(box), "color-label");
-    gtk_widget_set_sensitive(w, d == 0);
+    gtk_widget_set_sensitive(w, solid);
 }
 
 /* take a double (which might be NA) and format it for
@@ -580,7 +607,6 @@ static void entry_to_gp_double (GtkWidget *w, double *val)
     if (w != NULL && GTK_IS_ENTRY(w)) {
 	const gchar *s = gtk_entry_get_text(GTK_ENTRY(w));
 
-#ifdef ENABLE_NLS
 	if (s != NULL && *s != '\0') {
 	    gchar *tmp = g_strdup(s);
 
@@ -594,15 +620,6 @@ static void entry_to_gp_double (GtkWidget *w, double *val)
 	    gretl_pop_c_numeric_locale();
 	    g_free(tmp);
 	}
-#else
-	if (s != NULL && *s != '\0') {
-	    if (check_atof(s)) {
-		errbox(gretl_errmsg_get());
-	    } else {
-		x = atof(s);
-	    }
-	}
-#endif
     }
 
     if (!na(x)) {
@@ -680,7 +697,7 @@ static void maybe_set_point_type (GPT_LINE *line, GtkWidget *w, int i)
 
     if (ptsel != NULL && GTK_WIDGET_SENSITIVE(ptsel)) {
 	int pt = gtk_combo_box_get_active(GTK_COMBO_BOX(ptsel));
-	int ptdef = (line->type == LT_NONE)? i : line->type - 1;
+	int ptdef = (line->type == LT_AUTO)? i : line->type - 1;
 
 	if (pt != ptdef) {
 	    /* point-type is not just the default */
@@ -714,8 +731,10 @@ static void set_graph_font_from_widgets (plot_editor *ed)
 	} else {
 	    *pngfont = '\0';
 	}
-	set_gretl_png_font(pngfont, &paths);
+
+	set_gretl_png_font(pngfont);
     }
+
     g_free(tmp);
 }
 
@@ -745,7 +764,7 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 
     spec->flags &= ~GPT_Y2AXIS;
 
-    if (ed->y2_check != NULL && GTK_TOGGLE_BUTTON(ed->y2_check)->active) {
+    if (ed->y2_check != NULL && button_is_active(ed->y2_check)) {
 	suppress_y2 = 1;
     } 
 
@@ -764,11 +783,11 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 	}
     }
 
-    if (spec->code == PLOT_REGULAR) {
+    if (spec->code == PLOT_REGULAR || spec->code == PLOT_CURVE) {
 	k = (spec->flags & GPT_Y2AXIS)? 3 : 2;
 	for (i=0; i<k; i++) {
 	    if (ed->axis_range[i].isauto != NULL) {
-		if (GTK_TOGGLE_BUTTON(ed->axis_range[i].isauto)->active) {
+		if (button_is_active(ed->axis_range[i].isauto)) {
 		    spec->range[i][0] = NADBL;
 		    spec->range[i][1] = NADBL;
 		} else {
@@ -777,7 +796,7 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 		    err = validate_range(spec->range[i]);
 		}
 	    }
-	    if (ed->axis_range[i].lbase != NULL) {
+	    if (spec->code == PLOT_REGULAR && ed->axis_range[i].lbase != NULL) {
 		if (GTK_WIDGET_SENSITIVE(ed->axis_range[i].lbase)) {
 		    err = set_logscale_from_entry(spec, i, ed->axis_range[i].lbase);
 		} else {
@@ -841,7 +860,7 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
     } 
 
     if (!err && ed->border_check != NULL) {
-	if (GTK_TOGGLE_BUTTON(ed->border_check)->active) {
+	if (button_is_active(ed->border_check)) {
 	    /* full border */
 	    spec->border = GP_BORDER_DEFAULT;
 	} else {
@@ -852,10 +871,13 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 
     if (!err) {
 	if (ed->fontname != NULL) {
-	    set_gretl_png_font(ed->fontname, &paths);
+	    set_gretl_png_font(ed->fontname);
 	} else if (ed->ttfcombo != NULL && ed->ttfspin != NULL) {
 	    set_graph_font_from_widgets(ed);
 	}
+	if (ed->fontcheck != NULL && button_is_active(ed->fontcheck)) {
+	    update_persistent_graph_font();
+	}
     }
 
     if (!err && ed->fitcombo != NULL && GTK_WIDGET_IS_SENSITIVE(ed->fitcombo)) {
@@ -867,7 +889,9 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 
 	set_plot_has_y2_axis(plot, spec->flags & GPT_Y2AXIS);
 	redisplay_edited_plot(plot);
-	mark_session_changed();
+	if (plot_is_saved(plot)) {
+	    mark_session_changed();
+	}
     }
 
     plot_editor_sync(ed);
@@ -924,13 +948,26 @@ static const char *get_font_filename (const char *showname)
     return NULL;
 }
 
+static void plot_editor_set_fontname (plot_editor *ed, gchar *name)
+{
+    g_free(ed->fontname);
+    ed->fontname = name;
+}
+
 #ifdef G_OS_WIN32
 
-static void graph_font_selector (GtkButton *button, plot_editor *ed)
+static void real_graph_font_selector (GtkButton *button, gpointer p, int type)
 {
+    const char *default_font;
     char fontname[128];
 
-    strcpy(fontname, gretl_png_font());
+    if (type == 1) {
+	default_font = pdf_saver_current_font(p);
+    } else {
+	default_font = gretl_png_font();
+    }
+
+    strcpy(fontname, default_font);
     win32_font_selector(fontname, APP_FONT_SELECTION);
 
     if (*fontname != '\0') {
@@ -938,8 +975,11 @@ static void graph_font_selector (GtkButton *button, plot_editor *ed)
 
 	gtk_button_set_label(button, title);
 	g_free(title);
-	g_free(ed->fontname);
-	ed->fontname = g_strdup(fontname);
+	if (type == 1) {
+	    pdf_saver_set_fontname(p, fontname);
+	} else {
+	    plot_editor_set_fontname(p, g_strdup(fontname));
+	}
     }
 }
 
@@ -958,23 +998,28 @@ static void graph_font_selection_ok (GtkWidget *w, GtkFontSelectionDialog *fs)
 
     if (fontname != NULL && *fontname != '\0') {
 	GtkWidget *b = g_object_get_data(G_OBJECT(fs), "launcher");
-	plot_editor *ed = g_object_get_data(G_OBJECT(fs), "editor");
+	gpointer p = g_object_get_data(G_OBJECT(fs), "editor");
+	gint etype = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(fs), "editor-type"));
 	gchar *title = g_strdup_printf(_("font: %s"), fontname);
 
 	gtk_button_set_label(GTK_BUTTON(b), title);
 	g_free(title);
-	g_free(ed->fontname);
-	ed->fontname = fontname;
-	fontname = NULL;
+	if (etype == 1) {
+	    pdf_saver_set_fontname(p, fontname);
+	} else {
+	    plot_editor_set_fontname(p, fontname);
+	    fontname = NULL;
+	}
     }
 
     g_free(fontname);
     gtk_widget_destroy(GTK_WIDGET(fs));
 }
 
-static void graph_font_selector (GtkButton *button, plot_editor *ed)
+static void real_graph_font_selector (GtkButton *button, gpointer p, int type)
 {
     static GtkWidget *fontsel = NULL;
+    const char *default_font;
     GtkWidget *b;
 
     if (fontsel != NULL) {
@@ -982,11 +1027,18 @@ static void graph_font_selector (GtkButton *button, plot_editor *ed)
         return;
     }
 
+    if (type == 1) {
+	default_font = pdf_saver_current_font(p);
+    } else {
+	default_font = gretl_png_font();
+    }
+
     fontsel = gtk_font_selection_dialog_new(_("Font for graphs"));
     gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(fontsel), 
-					    gretl_png_font());
+					    default_font);
     g_object_set_data(G_OBJECT(fontsel), "launcher", button);
-    g_object_set_data(G_OBJECT(fontsel), "editor", ed);
+    g_object_set_data(G_OBJECT(fontsel), "editor", p);
+    g_object_set_data(G_OBJECT(fontsel), "editor-type", GINT_TO_POINTER(type));
 
     gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
 
@@ -1031,6 +1083,16 @@ static int font_is_ok (const char *fname)
 
 #endif
 
+static void graph_font_selector (GtkButton *button, gpointer p)
+{
+    real_graph_font_selector(button, p, 0);
+}
+
+void pdf_font_selector (GtkButton *button, gpointer p)
+{
+    real_graph_font_selector(button, p, 1);
+}
+
 static struct font_info *get_gnuplot_ttf_list (int *nf)
 {
     static struct font_info *retlist = NULL;
@@ -1097,7 +1159,7 @@ static void strip_lr (gchar *txt)
 
 static void toggle_axis_selection (GtkWidget *w, plot_editor *ed)
 {
-    int no_y2 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+    int no_y2 = button_is_active(w);
     int i;
 
     for (i=0; i<ed->gui_nlines; i++) {
@@ -1210,7 +1272,7 @@ static void add_color_selector (int i, GtkWidget *tbl, int cols,
 
 static void set_aa_status (GtkWidget *w, int *ok)
 {
-    *ok = GTK_TOGGLE_BUTTON(w)->active;
+    *ok = button_is_active(w);
 
     gnuplot_png_set_use_aa(*ok);
 }
@@ -1393,10 +1455,25 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	ed->fitcombo = gtk_combo_box_new_text();
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  ed->fitcombo, 1, TAB_MAIN_COLS, rows-1, rows);
-	for (i=0; fittype_strings[i] != NULL; i++) {
-	    gtk_combo_box_append_text(GTK_COMBO_BOX(ed->fitcombo), 
-				      _(fittype_strings[i]));
+
+	if (spec->flags & GPT_TS) {
+	    char tmp[128];
+
+	    for (i=0; fittype_strings[i] != NULL; i++) {
+		if (!strncmp(fittype_strings[i], "inver", 5)) {
+		    continue;
+		}
+		strcpy(tmp, _(fittype_strings[i]));
+		charsub(tmp, 'x', 't');
+		gtk_combo_box_append_text(GTK_COMBO_BOX(ed->fitcombo), tmp);
+	    }	    
+	} else {
+	    for (i=0; fittype_strings[i] != NULL; i++) {
+		gtk_combo_box_append_text(GTK_COMBO_BOX(ed->fitcombo), 
+					  _(fittype_strings[i]));
+	    }
 	}
+
 	gtk_combo_box_set_active(GTK_COMBO_BOX(ed->fitcombo), spec->fit);
 	widget_set_int(ed->fitcombo, "oldfit", spec->fit);
 	g_signal_connect(G_OBJECT(ed->fitcombo), "changed",
@@ -1451,7 +1528,7 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 
     /* setting of graph font, if gnuplot uses freetype */
     if (pngcairo || gnuplot_has_ttf(0)) {
-	GtkWidget *hsep;
+	GtkWidget *hsep, *b;
 
 	/* first a separator */
 	table_add_row(tbl, &rows, TAB_MAIN_COLS);	
@@ -1462,8 +1539,8 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 
 	if (pngcairo) {
 	    gchar *title = g_strdup_printf(_("font: %s"), gretl_png_font());
-	    GtkWidget *b = gtk_button_new_with_label(title);
 
+	    b = gtk_button_new_with_label(title);
 	    table_add_row(tbl, &rows, TAB_MAIN_COLS);
 	    gtk_table_attach_defaults(GTK_TABLE(tbl), b, 0, TAB_MAIN_COLS, 
 				      rows - 1, rows); 
@@ -1475,6 +1552,14 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	} else {
 	    add_gd_font_selector(ed, tbl, &rows);
 	}
+
+	/* set font as default button */
+	table_add_row(tbl, &rows, TAB_MAIN_COLS);
+	b = gtk_check_button_new_with_label(_("Set as default"));
+	gtk_table_attach_defaults(GTK_TABLE(tbl), b, 0, TAB_MAIN_COLS, 
+				  rows - 1, rows); 
+	gtk_widget_show(b);
+	ed->fontcheck = b;
     } 
 
     if (gnuplot_png_terminal() != GP_PNG_OLD && frequency_plot_code(spec->code)) {
@@ -1510,7 +1595,7 @@ static void gpt_tab_new_line (plot_editor *ed, new_line_info *nlinfo)
     int tbl_len, tbl_num, tbl_col;
     char label_text[32];
 
-    vbox = GTK_DIALOG(nlinfo->dlg)->vbox;
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(nlinfo->dlg));
     hbox = gtk_hbox_new(FALSE, 5);
 
     tbl_len = 1;
@@ -1606,9 +1691,7 @@ static int allocate_label (plot_editor *ed)
 
 static int allocate_line (plot_editor *ed)
 {
-    int err;
-
-    err = make_lines_backup(ed);
+    int err = make_lines_backup(ed);
 
     if (!err) {
 	err = plotspec_add_line(ed->spec);
@@ -1644,11 +1727,34 @@ static void add_label_callback (GtkWidget *w, plot_editor *ed)
     }
 }
 
+static void set_gp_formula (char *targ, const char *s)
+{
+    int decom = (get_local_decpoint() == ',');
+    int rem = GP_MAXFORMULA - strlen(s);
+    int i = 0;
+
+    while (*s) {
+	if (decom && *s == ',') {
+	    targ[i++] = '.';
+	} else if (*s == '^' && rem > 1) {
+	    targ[i++] = '*';
+	    targ[i++] = '*';
+	    rem--;
+	} else {
+	    targ[i++] = *s;
+	}
+	s++;
+    }
+
+    targ[i] = '\0';
+}
+
 static void real_add_line (GtkWidget *w, new_line_info *nlinfo)
 {
     plot_editor *ed = nlinfo->editor;
     GPT_SPEC *spec = ed->spec;
     GPT_LINE *line;
+    char tmp[GP_MAXFORMULA];
     const gchar *s;
     gint pgnum;
     int err = 0;
@@ -1659,6 +1765,9 @@ static void real_add_line (GtkWidget *w, new_line_info *nlinfo)
 	return;
     }
 
+    *tmp = '\0';
+    strncat(tmp, s, GP_MAXFORMULA - 1);
+
     err = allocate_line(ed);
     if (err) {
 	gtk_widget_destroy(nlinfo->dlg);
@@ -1666,8 +1775,7 @@ static void real_add_line (GtkWidget *w, new_line_info *nlinfo)
     }
 
     line = &spec->lines[spec->n_lines - 1];
-
-    entry_to_gp_string(nlinfo->formula_entry, line->formula, GP_MAXFORMULA);
+    set_gp_formula(line->formula, tmp);
 
     line->style = GP_STYLE_LINES;
     line->type = spec->n_lines; /* assign next line style */
@@ -1707,22 +1815,20 @@ static void add_line_callback (GtkWidget *w, plot_editor *ed)
 
     gpt_tab_new_line(ed, nlinfo);
 
-    hbox = GTK_DIALOG(nlinfo->dlg)->action_area;
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(nlinfo->dlg));
 
     button = cancel_button(hbox);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(delete_widget), nlinfo->dlg);
-    gtk_widget_show(button);
 
     button = ok_button(hbox);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(real_add_line), nlinfo);
     gtk_widget_grab_default(button);
-    gtk_widget_show(button);
 
     context_help_button(hbox, GPT_ADDLINE);
 
-    gtk_widget_show(nlinfo->dlg);
+    gtk_widget_show_all(nlinfo->dlg);
 
     free(nlinfo);
 }
@@ -1830,9 +1936,8 @@ static void print_label_label (GtkWidget *tbl, int row, GPT_SPEC *spec,
 static void print_field_label (GtkWidget *tbl, int row,
 			       const gchar *text)
 {
-    GtkWidget *label; 
+    GtkWidget *label = gtk_label_new(text);
 
-    label = gtk_label_new(text);
     gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
 			      label, 1, 2, row - 1, row);
@@ -1899,7 +2004,7 @@ static int line_get_point_style (GPT_LINE *line, int i)
 	/* a specific point-style has been selected: convert
 	   to zero-based */
 	return line->ptype - 1;
-    } else if (line->type == LT_NONE) {
+    } else if (line->type == LT_AUTO) {
 	/* line type is set by placement of line in plot */
 	return i;
     } else {
@@ -2241,8 +2346,10 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 	if (line->flags & GP_LINE_USER) {
 	    /* dotted option */
 	    GtkWidget *dotcombo = dash_types_combo();
+	    int dotted = (line->type == 0);
 
-	    gtk_combo_box_set_active(GTK_COMBO_BOX(dotcombo), 0); 
+	    gtk_combo_box_set_active(GTK_COMBO_BOX(dotcombo), 
+				     dotted ? 1 : 0);
 	    gtk_widget_show(dotcombo);
 	    widget_set_int(dotcombo, "linenum", i);
 	    g_object_set_data(G_OBJECT(dotcombo), "colorsel", button);
@@ -2477,7 +2584,7 @@ static void gpt_tab_XY (plot_editor *ed, GPT_SPEC *spec, gint axis)
 	}
     } 
 
-    if (spec->code != PLOT_REGULAR) {
+    if (spec->code != PLOT_REGULAR && spec->code != PLOT_CURVE) {
 	return;
     }
 
@@ -2557,6 +2664,10 @@ static void gpt_tab_XY (plot_editor *ed, GPT_SPEC *spec, gint axis)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2), TRUE);
     }
 
+    if (spec->code != PLOT_REGULAR) {
+	return;
+    }
+
     /* axis scale: linear vs log? */
 
     if ((axis == 0 && plot_get_xmin(plot) >= 0.0) ||
@@ -2610,8 +2721,7 @@ static void gpt_tab_XY (plot_editor *ed, GPT_SPEC *spec, gint axis)
 	    gtk_widget_set_sensitive(entry, FALSE);
 	}
 
-	desensitize_widget_from_check(b1, entry);
-	sensitize_widget_from_check(b2, entry);
+	sensitize_conditional_on(entry, b2);
     }
 }
 
@@ -2669,11 +2779,9 @@ static void plot_editor_destroy (plot_editor *ed)
 
 static GtkWidget **widget_array_new (int n, int *err)
 {
-    GtkWidget **pw;
+    GtkWidget **pw = malloc(n * sizeof *pw);
     int i;
 
-    pw = malloc(n * sizeof *pw);
-
     if (pw != NULL) {
 	for (i=0; i<n; i++) {
 	    pw[i] = NULL;
@@ -2709,12 +2817,12 @@ static int add_line_widget (plot_editor *ed)
     int n = ed->gui_nlines + 1;
     int err = 0;
 
-    ed->linetitle = widget_array_expand(&ed->linetitle, n, &err);
+    ed->linetitle   = widget_array_expand(&ed->linetitle, n, &err);
     ed->lineformula = widget_array_expand(&ed->lineformula, n, &err);
-    ed->stylecombo = widget_array_expand(&ed->stylecombo, n, &err);
-    ed->yaxiscombo = widget_array_expand(&ed->yaxiscombo, n, &err);
-    ed->linescale = widget_array_expand(&ed->linescale, n, &err);
-    ed->linewidth = widget_array_expand(&ed->linewidth, n, &err);
+    ed->stylecombo  = widget_array_expand(&ed->stylecombo, n, &err);
+    ed->yaxiscombo  = widget_array_expand(&ed->yaxiscombo, n, &err);
+    ed->linescale   = widget_array_expand(&ed->linescale, n, &err);
+    ed->linewidth   = widget_array_expand(&ed->linewidth, n, &err);
     
     if (!err) {
 	ed->gui_nlines = n;
@@ -2728,12 +2836,12 @@ static int allocate_line_widgets (plot_editor *ed, int n)
     int err = 0;
 
     if (n > 0) {
-	ed->linetitle = widget_array_new(n, &err);
+	ed->linetitle   = widget_array_new(n, &err);
 	ed->lineformula = widget_array_new(n, &err);
-	ed->stylecombo = widget_array_new(n, &err);
-	ed->yaxiscombo = widget_array_new(n, &err);
-	ed->linescale = widget_array_new(n, &err);
-	ed->linewidth = widget_array_new(n, &err);
+	ed->stylecombo  = widget_array_new(n, &err);
+	ed->yaxiscombo  = widget_array_new(n, &err);
+	ed->linescale   = widget_array_new(n, &err);
+	ed->linewidth   = widget_array_new(n, &err);
 	if (!err) {
 	    ed->gui_nlines = n;
 	}
@@ -2748,7 +2856,7 @@ static int add_label_widget (plot_editor *ed)
     int err = 0;
 
     ed->labeltext = widget_array_expand(&ed->labeltext, n, &err);
-    ed->labelpos = widget_array_expand(&ed->labelpos, n, &err);
+    ed->labelpos  = widget_array_expand(&ed->labelpos, n, &err);
     ed->labeljust = widget_array_expand(&ed->labeljust, n, &err);
     
     if (!err) {
@@ -2764,7 +2872,7 @@ static int allocate_label_widgets (plot_editor *ed, int n)
 
     if (n > 0) {
 	ed->labeltext = widget_array_new(n, &err);
-	ed->labelpos = widget_array_new(n, &err);
+	ed->labelpos  = widget_array_new(n, &err);
 	ed->labeljust = widget_array_new(n, &err);
 	if (!err) {
 	    ed->gui_nlabels = n;
@@ -2812,6 +2920,7 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
     ed->y2_check = NULL;
     ed->ttfcombo = NULL;
     ed->ttfspin = NULL;
+    ed->fontcheck = NULL;
 
     ed->gui_nlines = ed->gui_nlabels = 0;
 
@@ -2848,7 +2957,7 @@ GtkWidget *plot_add_editor (png_plot *plot)
     GPT_SPEC *spec = plot_get_spec(plot);
     GtkWidget *plotshell = plot_get_shell(plot);
     plot_editor *editor;
-    GtkWidget *dialog, *hbox;
+    GtkWidget *dialog, *vbox, *hbox;
     GtkWidget *button, *notebook;
 
     editor = plot_editor_new(spec);
@@ -2859,7 +2968,8 @@ GtkWidget *plot_add_editor (png_plot *plot)
 
     dialog = gretl_dialog_new(_("gretl plot controls"), plotshell, 
 			      GRETL_DLG_RESIZE);
-    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 2);
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+    gtk_box_set_spacing(GTK_BOX(vbox), 2);
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     
     if (plot != NULL) {
@@ -2874,8 +2984,7 @@ GtkWidget *plot_add_editor (png_plot *plot)
 			     editor);
    
     editor->notebook = notebook = gtk_notebook_new();
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-		       notebook, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
     gtk_widget_show(notebook);
 
     gpt_tab_main(editor, spec);
@@ -2896,14 +3005,13 @@ GtkWidget *plot_add_editor (png_plot *plot)
 	gpt_tab_palette(notebook);
     }
 
-    hbox = GTK_DIALOG(dialog)->action_area;
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
 
     /* "Apply" button */
     button = apply_button(hbox);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(apply_gpt_changes), editor);
     gtk_widget_grab_default(button);
-    gtk_widget_show(button);
 
     /* "OK" button (apply and close) */
     button = ok_button(hbox);
@@ -2912,7 +3020,6 @@ GtkWidget *plot_add_editor (png_plot *plot)
     g_signal_connect_swapped(G_OBJECT(button), "clicked",
 			     G_CALLBACK(gtk_widget_destroy), 
 			     dialog);
-    gtk_widget_show(button);
 
     /* Close button (do not apply changes) */
     button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
@@ -2921,13 +3028,13 @@ GtkWidget *plot_add_editor (png_plot *plot)
     g_signal_connect_swapped(G_OBJECT(button), "clicked",
 			     G_CALLBACK(gtk_widget_destroy), 
 			     dialog);
-    gtk_widget_show(button);
 
     /* Help button */
     context_help_button(hbox, GR_PLOT);
 
     set_keyspec_sensitivity(editor);
 
+    gtk_widget_show_all(hbox);
     gtk_widget_show(dialog);
 
     return dialog;
