@@ -48,7 +48,7 @@ static char gpage_tex_base[FILENAME_MAX];
 static void gpage_filenames_init (const char *base)
 {
     if (base == NULL) {
-	strcpy(gpage_base, paths.dotdir);
+	strcpy(gpage_base, gretl_dotdir());
 	strcat(gpage_base, "gretl_graphpage");
 	strcpy(gpage_tex_base, "gretl_graphpage");
     } else {
@@ -300,15 +300,30 @@ int in_graph_page (const char *fname)
     return 0;
 }
 
+static int graph_page_add_last_graph (void)
+{
+    const char *fname = last_session_graph_name();
+    int err = 0;
+
+    if (fname != NULL && *fname != '\0') {
+	if (!in_graph_page(fname)) {
+	    err = graph_page_add_file(fname);
+	}
+    }
+
+    return err;
+}
+
 static int gnuplot_compile (const char *fname)
 {
+    const char *gnuplot = gretl_gnuplot_path();
     char plotcmd[MAXLEN];
     int err = 0;
 
 # ifdef WIN32
-    sprintf(plotcmd, "\"%s\" \"%s\"", paths.gnuplot, fname);
+    sprintf(plotcmd, "\"%s\" \"%s\"", gnuplot, fname);
 # else
-    sprintf(plotcmd, "%s \"%s\"", paths.gnuplot, fname);
+    sprintf(plotcmd, "%s \"%s\"", gnuplot, fname);
 # endif
 
     err = gretl_spawn(plotcmd);
@@ -344,12 +359,14 @@ static int gp_make_outfile (const char *gfname, int i, double scale)
 	pdfterm = gnuplot_pdf_terminal();
     } 
 
-#ifdef ENABLE_NLS
     if (gpage.term == GP_TERM_EPS || pdfterm != GP_PDF_CAIRO) {
 	latin = iso_latin_version();
+	if (latin == 9 && !gnuplot_has_latin5()) {
+	    /* what we gonna do? */
+	    latin = 1;
+	}
 	fprintf(fq, "set encoding iso_8859_%d\n", latin);
     }
-#endif
 
     gretl_push_c_numeric_locale();
     
@@ -429,7 +446,7 @@ static int spawn_dvips (char *texsrc)
 
     signal(SIGCHLD, SIG_DFL);
 
-    ok = g_spawn_sync (paths.dotdir, /* working dir */
+    ok = g_spawn_sync (gretl_dotdir(),
 		       argv,
 		       NULL,    /* envp */
 		       G_SPAWN_SEARCH_PATH,
@@ -477,7 +494,7 @@ int dvips_compile (char *texshort)
     }
 
     sprintf(tmp, "\"%s\" -o %s.ps %s", dvips_path, texshort, texshort);
-    if (winfork(tmp, paths.dotdir, SW_SHOWMINIMIZED, CREATE_NEW_CONSOLE)) {
+    if (winfork(tmp, gretl_dotdir(), SW_SHOWMINIMIZED, CREATE_NEW_CONSOLE)) {
 	return 1;
     }
 #else
@@ -660,10 +677,14 @@ int display_graph_page (void)
     return err;
 }
 
-void clear_graph_page (void)
+void clear_graph_page (int on_exit)
 {
     int i;
 
+    if (!on_exit && gpage.ngraphs > 0) {
+	mark_session_changed();
+    }
+
     for (i=0; i<gpage.ngraphs; i++) {
 	free(gpage.fnames[i]);
     }
@@ -715,3 +736,25 @@ int save_graph_page (const char *fname)
 
     return err;
 }
+
+int graph_page_parse_line (const char *line)
+{
+    char cmdword[9];
+    int err = 0;
+
+    if (sscanf(line, "%*s %8s", cmdword) != 1) {
+	return E_PARSE;
+    }
+
+    if (!strcmp(cmdword, "add")) {
+	err = graph_page_add_last_graph();
+    } else if (!strcmp(cmdword, "show")) {
+	err = display_graph_page();
+    } else if (!strcmp(cmdword, "free")) {
+	if (gpage.ngraphs > 0) {
+	    clear_graph_page(0);
+	}
+    }
+
+    return err;
+}
