@@ -26,6 +26,7 @@ enum extra_cmds {
     GR_XYZ,
     GR_BOX,
     GR_NBOX,
+    GR_QQ,
     GR_3D,
     COMPACT,
     EXPAND, 
@@ -42,8 +43,8 @@ enum extra_cmds {
     VIEW_SCRIPT,
     VIEW_CODEBOOK,
     VIEW_MODELTABLE,
-    VIEW_FUNC_INFO,
-    VIEW_FUNC_CODE,
+    VIEW_PKG_INFO,
+    VIEW_PKG_CODE,
     VIEW_FILE,
     DATA_REPORT,
     SCRIPT_OUT,
@@ -51,16 +52,16 @@ enum extra_cmds {
     EDIT_HEADER,
     EDIT_SCRIPT,
     EDIT_NOTES,
-    EDIT_FUNC_CODE,
+    EDIT_PKG_CODE,
+    EDIT_PKG_SAMPLE,
     EDIT_GP,
     EDIT_R,
+    EDIT_OX,
     CLI_HELP,
     GUI_HELP,
     CLI_HELP_EN,
     GUI_HELP_EN,
     FUNCS_HELP,
-    MODELTABLE,
-    GRAPHPAGE,
     KERNEL_DENSITY,
     CREATE_DATASET,
     HCCME,
@@ -102,38 +103,42 @@ enum extra_cmds {
     OPROBIT,
     IV_LIML,
     IV_GMM,
+    BFGS_CONFIG,
     GUI_CMD_MAX
 };
 
+#define HELP_ROLE(r) (r >= CLI_HELP && r <= FUNCS_HELP)
+
 enum file_ops {
     OPEN_DATA = GUI_CMD_MAX + 1, /* don't collide with extra_cmds */
+    OPEN_CSV,
+    OPEN_OCTAVE,
+    OPEN_GNUMERIC,
+    OPEN_XLS,
+    OPEN_ODS,
+    OPEN_WF1,
+    OPEN_DTA,
+    OPEN_SAV,
+    OPEN_SAS,
+    OPEN_JMULTI,     /* end of regular "open data" entries */
     OPEN_RATS_DB,
     OPEN_PCGIVE_DB,
     OPEN_SCRIPT,
-    OPEN_CSV,
     APPEND_DATA,
     APPEND_CSV,
-    OPEN_ASCII,
-    APPEND_ASCII,
-    OPEN_OCTAVE,
     APPEND_OCTAVE,
-    OPEN_GNUMERIC,
     APPEND_GNUMERIC,
-    OPEN_XLS,
     APPEND_XLS,
-    OPEN_WF1,
-    APPEND_WF1,
-    OPEN_DTA,
-    APPEND_DTA,
-    OPEN_SAV,
-    APPEND_SAV,
-    OPEN_JMULTI,
-    APPEND_JMULTI,
-    OPEN_ODS,
     APPEND_ODS,
+    APPEND_WF1,
+    APPEND_DTA,
+    APPEND_SAV,
+    APPEND_SAS,
+    APPEND_JMULTI,   /* end of "data append" entries */
     OPEN_SESSION,
     OPEN_MARKERS,
-    END_OPEN,      /* marker for end of file open section */
+    OPEN_GFN,
+    END_OPEN,        /* marker for end of file open section */
     SAVE_DATA,
     SAVE_DATA_AS,
     SAVE_DBDATA,
@@ -151,16 +156,22 @@ enum file_ops {
     SAVE_OUTPUT,
     SAVE_SESSION,
     SAVE_GNUPLOT,
+    SAVE_GRAPHIC,
     SAVE_GP_CMDS,
     SAVE_R_CMDS,
+    SAVE_OX_CMDS,
     SAVE_CONSOLE,
+    SAVE_CMD_LOG,
     SAVE_FUNCTIONS,
     SAVE_FUNCTIONS_AS,
     SAVE_BOOT_DATA,
+    SAVE_MARKERS,
+    EDIT_FUNCTIONS,
     SET_PROG,
     SET_DIR,
     SET_WDIR,
     SET_FDIR,
+    SET_DBDIR,
     SAVE_DATA_PKG,
     FILE_OP_MAX
 };
@@ -182,6 +193,8 @@ enum browser_codes {
     MAINWIN
 };
 
+#define BROWSER_ROLE(r) (r >= TEXTBOOK_DATA && r < MAINWIN)
+
 enum pref_tabs {
     TAB_NONE = 0,
     TAB_MAIN,
@@ -207,7 +220,8 @@ enum data_status_flags {
     IMPORT_DATA   = 1 << 3,
     GUI_DATA      = 1 << 4,
     MODIFIED_DATA = 1 << 5,
-    GZIPPED_DATA  = 1 << 6
+    GZIPPED_DATA  = 1 << 6,
+    SESSION_DATA  = 1 << 7
 };
 
 enum register_data_flags {
