@@ -89,10 +89,11 @@ static int ws_startup (void)
 	fprintf(stderr, I_("Couldn't find usable socket driver\n"));
 	WSACleanup();
 	return 1;
+    } else {
+	/* arrange for cleanup on exit */
+	atexit(ws_cleanup);
     }
 
-    atexit(ws_cleanup);
-
     return 0;
 }
 
@@ -144,18 +145,12 @@ int create_child_process (char *prog)
 
 static int Rgui_path_from_registry (void)
 {
-    char tmp[MAX_PATH] = {0}; 
+    char tmp[MAX_PATH];
     int err;
 
-    err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", tmp);
-
-    if (err) {
-	err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", tmp);
-    }
+    err = R_path_from_registry(tmp, RGUI);
 
     if (!err) {
-	strcat(tmp, "\\bin\\");
-	strcat(tmp, "Rgui.exe");
 	*Rcommand = '\0';
 	strncat(Rcommand, tmp, MAXSTR - 1);
     }
@@ -311,7 +306,7 @@ static void try_to_get_windows_font (void)
     }
 }
 
-int use_wimp;
+int use_wimp; /* note: published via gretlwin32.h */
 
 static int wimp_init (void)
 {
@@ -331,12 +326,13 @@ static int wimp_init (void)
 void set_up_windows_look (void)
 {
     if (use_wimp) { 
-	size_t n = strlen(paths.gretldir);
-	int needslash = (paths.gretldir[n-1] != SLASH);
+	const char *gretldir = gretl_home();
+	size_t n = strlen(gretldir);
+	int needslash = (gretldir[n-1] != SLASH);
 	gchar *wimprc;
 
 	wimprc = g_strdup_printf("%s%sshare\\themes\\MS-Windows\\gtk-2.0\\gtkrc", 
-				 paths.gretldir, (needslash)? "\\" : "");
+				 gretldir, (needslash)? "\\" : "");
 	gtk_rc_parse(wimprc);
 	g_free(wimprc);
     } else {
@@ -344,33 +340,8 @@ void set_up_windows_look (void)
     }
 }
 
-static char inifile[FILENAME_MAX];
-
-const char *get_network_cfg_filename (void)
-{
-    return inifile;
-}
-
-static int set_network_cfg_filename (const char *prog)
-{
-    const char *p;
-    int n;
-
-    *inifile = '\0';
-    
-    n = strlen(prog);
-    p = prog + n - 1;
-    while (p - prog >= 0) {
-	if (*p == '\\' || *p == '/') {
-	    strncpy(inifile, prog, n - strlen(p));
-	    strcat(inifile, "\\gretlnet.txt");
-	    break;
-	}
-	p--;
-    }
-
-    return 0;
-}
+/* carry out some Windows-specific start-up tasks, and
+   call read_rc to get configuration info */
 
 void gretl_win32_init (const char *progname, int debug)
 {
@@ -378,10 +349,10 @@ void gretl_win32_init (const char *progname, int debug)
         redirect_io_to_console();
     }
 
-    set_network_cfg_filename(progname);
+    set_gretlnet_filename(progname);
 
     wimp_init();
-    read_rc(debug); /* get config info from registry */
+    read_win32_config(debug);
     set_gretl_startdir();
     hush_warnings();
     ws_startup(); 
