@@ -35,19 +35,23 @@
 
 #include "gretl.h" /* this must provide standard and GTK headers and
 		      deal with gettext */
+#include "dlgutils.h"
 #include "gtkfontselhack.h"
 
-#define GTK_TYPE_FONTSEL_HACK              (gtk_fontsel_hack_get_type ())
-#define GTK_FONTSEL_HACK(obj)              (GTK_CHECK_CAST ((obj), GTK_TYPE_FONTSEL_HACK, GtkFontselHack))
-#define GTK_FONTSEL_HACK_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_FONTSEL_HACK, GtkFontselHackClass))
-#define GTK_IS_FONTSEL_HACK(obj)           (GTK_CHECK_TYPE ((obj), GTK_TYPE_FONTSEL_HACK))
-#define GTK_IS_FONTSEL_HACK_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_FONTSEL_HACK))
-#define GTK_FONTSEL_HACK_GET_CLASS(obj)    (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_FONTSEL_HACK, GtkFontselHackClass))
+#define GTK_TYPE_FNTHACK              (gtk_fontsel_hack_get_type ())
+#define GTK_FNTHACK(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_FNTHACK, GtkFontselHack))
+#define GTK_FNTHACK_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_FNTHACK, GtkFontselHackClass))
+#define GTK_IS_FNTHACK(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_FNTHACK))
+#define GTK_IS_FNTHACK_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_FNTHACK))
+#define GTK_FNTHACK_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_FNTHACK, GtkFontselHackClass))
 
-#define GTK_FONTSEL_HACK_DIALOG_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_FONTSEL_HACK_DIALOG, GtkFontselHackDialogClass))
-#define GTK_IS_FONTSEL_HACK_DIALOG(obj)           (GTK_CHECK_TYPE ((obj), GTK_TYPE_FONTSEL_HACK_DIALOG))
-#define GTK_IS_FONTSEL_HACK_DIALOG_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_FONTSEL_HACK_DIALOG))
-#define GTK_FONTSEL_HACK_DIALOG_GET_CLASS(obj)    (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_FONTSEL_HACK_DIALOG, GtkFontselHackDialogClass))
+
+#define GTK_TYPE_FNTHACK_DIALOG              (gtk_fontsel_hack_dialog_get_type ())
+#define GTK_FNTHACK_DIALOG(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_FNTHACK_DIALOG, GtkFontselHackDialog))
+#define GTK_FNTHACK_DIALOG_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_FNTHACK_DIALOG, GtkFontselHackDialogClass))
+#define GTK_IS_FNTHACK_DIALOG(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_FNTHACK_DIALOG))
+#define GTK_IS_FNTHACK_DIALOG_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_FNTHACK_DIALOG))
+#define GTK_FNTHACK_DIALOG_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_FNTHACK_DIALOG, GtkFontselHackDialogClass))
 
 #define GTK_TYPE_FONT_FILTER (gtk_font_filter_get_type())
 
@@ -277,7 +281,7 @@ gtk_fontsel_hack_set_property (GObject *object,
 {
     GtkFontselHack *fontsel;
 
-    fontsel = GTK_FONTSEL_HACK(object);
+    fontsel = GTK_FNTHACK(object);
 
     switch (prop_id) {
     case PROP_FONT_NAME:
@@ -302,7 +306,7 @@ static void gtk_fontsel_hack_get_property (GObject *object,
 {
     GtkFontselHack *fontsel;
 
-    fontsel = GTK_FONTSEL_HACK(object);
+    fontsel = GTK_FNTHACK(object);
 
     switch (prop_id) {
     case PROP_FONT_NAME:
@@ -543,7 +547,7 @@ gtk_fontsel_hack_new (void)
 {
     GtkFontselHack *fontsel;
   
-    fontsel = g_object_new(GTK_TYPE_FONTSEL_HACK, NULL);
+    fontsel = g_object_new(GTK_TYPE_FNTHACK, NULL);
   
     return GTK_WIDGET(fontsel);
 }
@@ -553,9 +557,9 @@ gtk_fontsel_hack_finalize (GObject *object)
 {
     GtkFontselHack *fontsel;
   
-    g_return_if_fail(GTK_IS_FONTSEL_HACK(object));
+    g_return_if_fail(GTK_IS_FNTHACK(object));
   
-    fontsel = GTK_FONTSEL_HACK(object);
+    fontsel = GTK_FNTHACK(object);
 
     (* G_OBJECT_CLASS (gtk_fontsel_hack_parent_class)->finalize) (object);
 }
@@ -564,7 +568,7 @@ static void
 gtk_fontsel_hack_screen_changed (GtkWidget *widget,
 				 GdkScreen *previous_screen)
 {
-    GtkFontselHack *fontsel = GTK_FONTSEL_HACK(widget);
+    GtkFontselHack *fontsel = GTK_FNTHACK(widget);
 
     if (gtk_widget_has_screen (GTK_WIDGET (fontsel))) {
 	gtk_fontsel_hack_show_available_fonts (fontsel, HACK_INIT);
@@ -617,7 +621,7 @@ gtk_fontsel_hack_scroll_on_map (GtkWidget *widget,
 {
     GtkFontselHack *fontsel;
   
-    fontsel = GTK_FONTSEL_HACK (data);
+    fontsel = GTK_FNTHACK (data);
   
     /* Try to scroll the font family list to the selected item */
     scroll_to_selection (GTK_TREE_VIEW (fontsel->family_list));
@@ -639,7 +643,7 @@ gtk_fontsel_hack_select_font (GtkTreeSelection *selection,
     GtkTreeIter iter;
     const gchar *family_name;
   
-    fontsel = GTK_FONTSEL_HACK (data);
+    fontsel = GTK_FNTHACK (data);
 
     if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
 	PangoFontFamily *family;
@@ -671,7 +675,6 @@ cmp_families (const void *a, const void *b)
 
 static GtkWidget *font_test_widget;
 static PangoContext *font_test_context;
-static PangoLayout *font_test_layout;
 static PangoLanguage *font_test_lang;
 
 static int create_font_test_rig (void)
@@ -684,13 +687,6 @@ static int create_font_test_rig (void)
 	return 1;
     }    
 
-    font_test_layout = pango_layout_new(font_test_context); 
-    if (font_test_layout == NULL) {
-	g_object_unref(G_OBJECT(font_test_context));
-	gtk_widget_destroy(font_test_widget);
-	return 1;
-    }	
-
     font_test_lang = pango_language_from_string("eng");
 
     return 0;
@@ -698,7 +694,6 @@ static int create_font_test_rig (void)
 
 static void destroy_font_test_rig (void)
 {
-    g_object_unref(G_OBJECT(font_test_layout));
     g_object_unref(G_OBJECT(font_test_context));
     gtk_widget_destroy(font_test_widget);    
 }
@@ -715,7 +710,8 @@ enum {
    by checking whether or not the letters 'i' and 'W' come out the
    same width. */
 
-static int get_font_characteristics (PangoFontDescription *desc)
+static int get_font_characteristics (PangoFontFamily *family,
+				     PangoFontDescription *desc)
 {
     PangoCoverage *coverage = NULL;
     PangoFont *pfont = NULL;
@@ -733,20 +729,9 @@ static int get_font_characteristics (PangoFontDescription *desc)
 
     if (pango_coverage_get(coverage, 'i') == PANGO_COVERAGE_EXACT &&
 	pango_coverage_get(coverage, 'W') == PANGO_COVERAGE_EXACT) {
-	int x1, x2;
-
 	ret = HACK_LATIN_FONT;
-	
-	if (font_test_context != NULL && font_test_layout != NULL) {
-	    pango_context_set_font_description(font_test_context, desc);
-	    pango_layout_set_text(font_test_layout, "i", 1);
-	    pango_layout_get_pixel_size(font_test_layout, &x1, NULL);
-	    pango_layout_set_text(font_test_layout, "W", 1);
-	    pango_layout_get_pixel_size(font_test_layout, &x2, NULL);
-
-	    if (x1 == x2) {
-		ret |= HACK_MONO_FONT;
-	    }
+	if (pango_font_family_is_monospace(family)) {
+	    ret |= HACK_MONO_FONT;
 	}
     }
 
@@ -799,7 +784,8 @@ static void font_progress_bar (int i, int nf)
    in respect of the criterion in filter.
 */
 
-static int validate_font_family (const gchar *famname, 
+static int validate_font_family (PangoFontFamily *family,
+				 const gchar *famname, 
 				 gint i, gint nf,
 				 gint filter, gint *err)
 {
@@ -831,7 +817,7 @@ static int validate_font_family (const gchar *famname,
 	    PangoFontDescription *desc = pango_font_description_from_string(font);
 
 	    if (desc != NULL) {
-		fcache[i] = get_font_characteristics(desc);
+		fcache[i] = get_font_characteristics(family, desc);
 		pango_font_description_free(desc);
 	    }
 	    g_free(font);
@@ -883,7 +869,7 @@ gtk_fontsel_hack_show_available_fonts (GtkFontselHack *fontsel,
 	fprintf(stderr, "Examining font family '%s'\n", famname);
 #endif
 
-	if (!validate_font_family(famname, i, nf, fontsel->filter, &err)) { 
+	if (!validate_font_family(families[i], famname, i, nf, fontsel->filter, &err)) { 
 	    continue;
 	}
 
@@ -1065,7 +1051,7 @@ static void
 gtk_fontsel_hack_select_style (GtkTreeSelection *selection,
 			       gpointer          data)
 {
-    GtkFontselHack *fontsel = GTK_FONTSEL_HACK(data);
+    GtkFontselHack *fontsel = GTK_FNTHACK(data);
     GtkTreeModel *model;
     GtkTreeIter iter;
   
@@ -1084,7 +1070,7 @@ gtk_fontsel_hack_select_style (GtkTreeSelection *selection,
 
 static void
 gtk_fontsel_hack_show_available_sizes (GtkFontselHack *fontsel,
-				       gboolean          first_time)
+				       gboolean first_time)
 {
     size_t i;
     GtkListStore *model;
@@ -1099,16 +1085,15 @@ gtk_fontsel_hack_show_available_sizes (GtkFontselHack *fontsel,
     if (first_time) {
 	gtk_list_store_clear (model);
 
-	for (i = 0; i < G_N_ELEMENTS (font_sizes); i++)
-	    {
-		GtkTreeIter iter;
+	for (i = 0; i < G_N_ELEMENTS (font_sizes); i++) {
+	    GtkTreeIter iter;
 	  
-		gtk_list_store_append (model, &iter);
-		gtk_list_store_set (model, &iter, SIZE_COLUMN, font_sizes[i], -1);
+	    gtk_list_store_append (model, &iter);
+	    gtk_list_store_set (model, &iter, SIZE_COLUMN, font_sizes[i], -1);
 	  
-		if (font_sizes[i] * PANGO_SCALE == fontsel->size)
-		    set_cursor_to_iter (GTK_TREE_VIEW (fontsel->size_list), &iter);
-	    }
+	    if (font_sizes[i] * PANGO_SCALE == fontsel->size)
+		set_cursor_to_iter (GTK_TREE_VIEW (fontsel->size_list), &iter);
+	}
     } else {
 	GtkTreeIter iter;
 	gboolean found = FALSE;
@@ -1176,7 +1161,7 @@ gtk_fontsel_hack_size_activate (GtkWidget *w,
     gint new_size;
     const gchar *text;
   
-    fontsel = GTK_FONTSEL_HACK(data);
+    fontsel = GTK_FNTHACK(data);
 
     text = gtk_entry_get_text(GTK_ENTRY(fontsel->size_entry));
     new_size = MAX(0.1, atof(text) * PANGO_SCALE + 0.5);
@@ -1205,7 +1190,7 @@ gtk_fontsel_hack_select_size (GtkTreeSelection *selection,
     GtkTreeIter iter;
     gint new_size;
   
-    fontsel = GTK_FONTSEL_HACK (data);
+    fontsel = GTK_FNTHACK (data);
   
     if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
 	gtk_tree_model_get (model, &iter, SIZE_COLUMN, &new_size, -1);
@@ -1297,7 +1282,7 @@ gtk_fontsel_hack_get_font_name (GtkFontselHack *fontsel)
 
 gboolean
 gtk_fontsel_hack_set_font_name (GtkFontselHack *fontsel,
-				const gchar      *fontname)
+				const gchar *fontname)
 {
     PangoFontFamily *new_family = NULL;
     PangoFontFace *new_face = NULL;
@@ -1308,7 +1293,7 @@ gtk_fontsel_hack_set_font_name (GtkFontselHack *fontsel,
     GtkTreeIter match_iter;
     gboolean valid;
   
-    g_return_val_if_fail (GTK_IS_FONTSEL_HACK (fontsel), FALSE);
+    g_return_val_if_fail (GTK_IS_FNTHACK (fontsel), FALSE);
   
     new_desc = pango_font_description_from_string (fontname);
 
@@ -1475,13 +1460,13 @@ gtk_fontsel_hack_dialog_buildable_get_internal_child (GtkBuildable *buildable,
 						      const gchar  *childname)
 {
     if (strcmp(childname, "ok_button") == 0)
-	return G_OBJECT (GTK_FONTSEL_HACK_DIALOG(buildable)->ok_button);
+	return G_OBJECT (GTK_FNTHACK_DIALOG(buildable)->ok_button);
     else if (strcmp(childname, "cancel_button") == 0)
-	return G_OBJECT (GTK_FONTSEL_HACK_DIALOG (buildable)->cancel_button);
+	return G_OBJECT (GTK_FNTHACK_DIALOG (buildable)->cancel_button);
     else if (strcmp(childname, "apply_button") == 0)
-	return G_OBJECT (GTK_FONTSEL_HACK_DIALOG(buildable)->apply_button);
+	return G_OBJECT (GTK_FNTHACK_DIALOG(buildable)->apply_button);
     else if (strcmp(childname, "font_selection") == 0)
-	return G_OBJECT (GTK_FONTSEL_HACK_DIALOG(buildable)->fontsel);
+	return G_OBJECT (GTK_FNTHACK_DIALOG(buildable)->fontsel);
 
     return parent_buildable_iface->get_internal_child (buildable, builder, childname);
 }
@@ -1513,7 +1498,7 @@ gtk_fontsel_hack_dialog_init (GtkFontselHackDialog *fontseldiag)
     gtk_container_set_border_width (GTK_CONTAINER (fontseldiag), 4);
     gtk_window_set_resizable (GTK_WINDOW (fontseldiag), FALSE);
   
-    fontseldiag->main_vbox = dialog->vbox;
+    fontseldiag->main_vbox = gtk_dialog_get_content_area (dialog);
   
     fontseldiag->fontsel = gtk_fontsel_hack_new ();
     gtk_container_set_border_width (GTK_CONTAINER (fontseldiag->fontsel), 4);
@@ -1522,7 +1507,7 @@ gtk_fontsel_hack_dialog_init (GtkFontselHackDialog *fontseldiag)
 			fontseldiag->fontsel, TRUE, TRUE, 0);
   
     /* Create the action area */
-    fontseldiag->action_area = dialog->action_area;
+    fontseldiag->action_area = gtk_dialog_get_action_area (dialog);
 
     fontseldiag->cancel_button = gtk_dialog_add_button (dialog,
 							GTK_STOCK_CANCEL,
@@ -1550,7 +1535,7 @@ gtk_fontsel_hack_dialog_new (const gchar *title)
 {
     GtkFontselHackDialog *fontseldiag;
   
-    fontseldiag = g_object_new (GTK_TYPE_FONTSEL_HACK_DIALOG, NULL);
+    fontseldiag = g_object_new (GTK_TYPE_FNTHACK_DIALOG, NULL);
 
     if (title)
 	gtk_window_set_title (GTK_WINDOW (fontseldiag), title);
@@ -1561,50 +1546,50 @@ gtk_fontsel_hack_dialog_new (const gchar *title)
 gchar*
 gtk_fontsel_hack_dialog_get_font_name (GtkFontselHackDialog *fsd)
 {
-    return gtk_fontsel_hack_get_font_name(GTK_FONTSEL_HACK(fsd->fontsel));
+    return gtk_fontsel_hack_get_font_name(GTK_FNTHACK(fsd->fontsel));
 }
 
 gboolean
 gtk_fontsel_hack_dialog_set_font_name (GtkFontselHackDialog *fsd,
 				       const gchar *fontname)
 {
-    return gtk_fontsel_hack_set_font_name(GTK_FONTSEL_HACK(fsd->fontsel), fontname);
+    return gtk_fontsel_hack_set_font_name(GTK_FNTHACK(fsd->fontsel), fontname);
 }
 
 gint
 gtk_fontsel_hack_dialog_get_filter (GtkFontselHackDialog *fsd)
 {
-    return gtk_fontsel_hack_get_filter (GTK_FONTSEL_HACK (fsd->fontsel));
+    return gtk_fontsel_hack_get_filter (GTK_FNTHACK (fsd->fontsel));
 }
 
 void
 gtk_fontsel_hack_dialog_set_filter (GtkFontselHackDialog *fsd,
 				    GtkFontFilterType	  filter)
 {
-    gtk_fontsel_hack_set_filter(GTK_FONTSEL_HACK(fsd->fontsel), filter);
+    gtk_fontsel_hack_set_filter(GTK_FNTHACK(fsd->fontsel), filter);
 }
 
 G_CONST_RETURN gchar*
 gtk_fontsel_hack_dialog_get_preview_text (GtkFontselHackDialog *fsd)
 {
-    return gtk_fontsel_hack_get_preview_text(GTK_FONTSEL_HACK(fsd->fontsel));
+    return gtk_fontsel_hack_get_preview_text(GTK_FNTHACK(fsd->fontsel));
 }
 
 void
 gtk_fontsel_hack_dialog_set_preview_text (GtkFontselHackDialog *fsd,
 					  const gchar *text)
 {
-    gtk_fontsel_hack_set_preview_text(GTK_FONTSEL_HACK(fsd->fontsel), text);
+    gtk_fontsel_hack_set_preview_text(GTK_FNTHACK(fsd->fontsel), text);
 }
 
 GtkWidget *gtk_fontsel_hack_dialog_ok_button (GtkWidget *fsd)
 {
-    return GTK_FONTSEL_HACK_DIALOG(fsd)->ok_button;
+    return GTK_FNTHACK_DIALOG(fsd)->ok_button;
 }
 
 GtkWidget *gtk_fontsel_hack_dialog_cancel_button (GtkWidget *fsd)
 {
-    return GTK_FONTSEL_HACK_DIALOG(fsd)->cancel_button;
+    return GTK_FNTHACK_DIALOG(fsd)->cancel_button;
 }
 
 /* This turns auto-shrink off if the user resizes the width of the dialog.
