@@ -20,6 +20,18 @@
 #ifndef GUI_UTILS_H
 #define GUI_UTILS_H
 
+#define vwin_editing_script(r) (r == EDIT_SCRIPT || \
+				r == EDIT_GP ||	    \
+				r == EDIT_R ||	    \
+				r == EDIT_OX)
+
+#define vwin_editing_buffer(r) (r == EDIT_HEADER || \
+	                        r == EDIT_NOTES)
+
+#define vwin_content_changed(v) (v->flags & VWIN_CONTENT_CHANGED)
+
+int vwin_is_editing (windata_t *vwin);
+
 int copyfile (const char *src, const char *dest);
 
 FILE *gretl_tempfile_open (char *fname);
@@ -58,22 +70,31 @@ int latex_is_ok (void);
 
 gint query_save_text (GtkWidget *w, GdkEvent *event, windata_t *vwin);
 
-void auto_save_plot (windata_t *vwin);
+void vwin_save_callback (GtkWidget *w, windata_t *vwin);
 
-void view_window_save (GtkWidget *widget, windata_t *vwin);
+gboolean vwin_copy_callback (GtkWidget *w, windata_t *vwin);
 
-void buf_edit_save (GtkWidget *widget, windata_t *vwin);
+void vwin_add_child (windata_t *parent, windata_t *child);
 
 windata_t *view_buffer (PRN *prn, int hsize, int vsize, 
 			const char *title, int role,
 			gpointer data);
 
+windata_t *view_buffer_with_parent (windata_t *parent, PRN *prn, 
+				    int hsize, int vsize, 
+				    const char *title, int role, 
+				    gpointer data);
+
 windata_t *view_file (const char *filename, int editable, int del_file, 
 		      int hsize, int vsize, int role);
 
-windata_t *
-view_help_file (const char *filename, int role, GtkActionEntry *menu_items,
-		const gchar *ui_info);
+gchar *gretl_window_title (const char *s1, const char *s2);
+
+gchar *title_from_filename (const char *fname);
+
+windata_t *console_window (int hsize, int vsize);
+
+windata_t *view_help_file (const char *filename, int role);
 
 windata_t *edit_buffer (char **pbuf, int hsize, int vsize, 
 			char *title, int role);
@@ -87,6 +108,8 @@ void view_window_set_editable (windata_t *vwin);
 
 int gui_validate_varname (const char *varname, GretlType t);
 
+int gui_validate_varname_strict (const char *name, GretlType t);
+
 gretlopt get_tex_eqn_opt (void);
 
 gint popup_menu_handler (GtkWidget *widget, GdkEvent *event,
@@ -95,6 +118,9 @@ gint popup_menu_handler (GtkWidget *widget, GdkEvent *event,
 void add_popup_item (const gchar *label, GtkWidget *menu,
 		     GCallback callback, gpointer data);
 
+gboolean text_popup_handler (GtkWidget *w, GdkEventButton *event, 
+			     gpointer p);
+
 void *gui_get_plugin_function (const char *funcname, 
 			       void **phandle);
 
@@ -108,6 +134,8 @@ int gretl_file_get_contents (const gchar *fname, gchar **contents);
 
 void start_R (const char *buf, int send_data, int interactive);
 
+void run_ox_script (gchar *buf);
+
 const char *print_today (void);
 
 #ifndef G_OS_WIN32
