@@ -40,41 +40,40 @@
 
 /* printing is enabled, either via Windows or GTK+ */
 
-static gchar *user_string (void)
+static const gchar *user_string (void)
 {
-    const gchar *username, *realname;
-    gchar *ret = NULL;
+    const gchar *ret = g_get_real_name();
 
-    username = g_get_user_name();
-    realname = g_get_real_name();
-
-    if (realname != NULL && *realname != '\0') {
-	ret = g_strdup_printf("%s %s", _("for"), realname);
-    } else if (username != NULL && *username != '\0') {
-	ret = g_strdup_printf("%s %s", _("for"), username);
+    if (ret == NULL || *ret == '\0') {
+	ret = g_get_user_name();
     }
 
     return ret;
 }
 
-static char *header_string (void)
+static char *header_string (const char *fname)
 {
-    gchar *hdr, *ustr;
-# if defined(ENABLE_NLS) && !defined(G_OS_WIN32)
+    gchar *hdr;
+# ifndef G_OS_WIN32
     gchar *trans;
 # endif
-    time_t prntime = time(NULL);
+    char tstr[48];
 
-    ustr = user_string();
-    if (ustr != NULL) {
-	hdr = g_strdup_printf("%s %s %s", _("gretl output"), ustr,
-			      print_time(&prntime));
-	g_free(ustr);
+    print_time(tstr);
+
+    if (fname != NULL && *fname != '\0' && !strstr(fname, "tmp")) {
+	hdr = g_strdup_printf("%s %s", fname, tstr);
     } else {
-	hdr = g_strdup_printf("%s %s", _("gretl output"), print_time(&prntime));
+	const gchar *ustr = user_string();
+
+	if (ustr != NULL && *ustr != '\0') {
+	    hdr = g_strdup_printf("%s %s %s %s", _("gretl output"), _("for"), ustr, tstr);
+	} else {
+	    hdr = g_strdup_printf("%s %s", _("gretl output"), tstr);
+	}
     }
 
-# if defined(ENABLE_NLS) && !defined(G_OS_WIN32)
+# ifndef G_OS_WIN32
     trans = my_locale_to_utf8(hdr);
     if (trans != NULL) {
 	g_free(hdr);
@@ -91,7 +90,8 @@ static char *header_string (void)
 
 #if defined(G_OS_WIN32)
 
-void print_window_content (char *fullbuf, char *selbuf)
+void print_window_content (char *fullbuf, char *selbuf, 
+			   const char *fname)
 {
     HDC dc;
     PRINTDLG pdlg;
@@ -166,19 +166,11 @@ void print_window_content (char *fullbuf, char *selbuf)
     
     printok = StartDoc(dc, &di);
 
-# ifdef ENABLE_NLS
     if (selbuf != NULL && (pdlg.Flags & PD_SELECTION)) {
 	printbuf = my_locale_from_utf8(selbuf);
     } else {
 	printbuf = my_locale_from_utf8(fullbuf);
     }
-# else
-    if (selbuf != NULL && (pdlg.Flags & PD_SELECTION)) {
-	printbuf = selbuf;
-    } else {
-	printbuf = fullbuf;
-    }
-# endif
 
     if (printbuf == NULL) {
 	printok = 0;
@@ -187,7 +179,7 @@ void print_window_content (char *fullbuf, char *selbuf)
 
     page = 1;
     x = px / 2; /* attempt at left margin */
-    hdrstart = header_string();
+    hdrstart = header_string(fname);
     while (*printbuf && printok) { /* pages loop */
 	StartPage(dc);
 	SelectObject(dc, fixed_font);
@@ -228,9 +220,7 @@ void print_window_content (char *fullbuf, char *selbuf)
     GlobalFree(pdlg.hDevMode);
     GlobalFree(pdlg.hDevNames);
 
-# ifdef ENABLE_NLS
     g_free(printbuf);
-# endif
 
     free(fullbuf); /* was allocated by gtk_editable_get_chars() */
     if (selbuf) {
@@ -458,7 +448,8 @@ static void job_set_n_pages (GtkPrintOperation *op, struct print_info *pinfo)
 
 static GtkPrintSettings *settings = NULL;
 
-void print_window_content (char *fullbuf, char *selbuf)
+void print_window_content (char *fullbuf, char *selbuf, 
+			   const char *fname)
 {
     GtkPrintOperation *op;
     GtkPrintOperationResult res;
@@ -477,7 +468,7 @@ void print_window_content (char *fullbuf, char *selbuf)
 
     pinfo.buf = (selbuf != NULL)? selbuf : fullbuf;
     pinfo.p = pinfo.buf;
-    pinfo.hdr = header_string();
+    pinfo.hdr = header_string(fname);
     pinfo.layout = NULL;
     pinfo.pagelines = 54; /* FIXME */
 
@@ -515,8 +506,9 @@ static int make_png_file (const char *fname,
 {
     FILE *fsrc, *ftmp;
     char cmd[MAXLEN], temp[MAXLEN], fline[MAXLEN];
+    const char *dotdir = gretl_dotdir();
 
-    sprintf(temp, "%sgpttmp", paths.dotdir);
+    sprintf(temp, "%sgpttmp", dotdir);
 
     ftmp = gretl_tempfile_open(temp);
     if (ftmp == NULL) {
@@ -530,7 +522,7 @@ static int make_png_file (const char *fname,
 	return 1;
     }
 
-    build_path(pngname, paths.dotdir, GRETL_PNG_TMP, NULL);
+    build_path(pngname, dotdir, GRETL_PNG_TMP, NULL);
 
     while (fgets(fline, MAXLEN-1, fsrc)) {
 	if (!strncmp(fline, "set output", 10)) {
@@ -544,7 +536,8 @@ static int make_png_file (const char *fname,
     fclose(ftmp);
 
     /* run gnuplot on the temp plotfile */
-    sprintf(cmd, "\"%s\" \"%s\"", paths.gnuplot, temp);
+    sprintf(cmd, "\"%s\" \"%s\"", gretl_gnuplot_path(), 
+	    temp);
     if (system(cmd)) {
 	gretl_remove(temp);
 	return 1;
@@ -771,6 +764,115 @@ rtfprint_summary (const Summary *summ, const DATAINFO *pdinfo, PRN *prn)
     pputs(prn, "}}\n");
 }
 
+int text_equation_ok (const MODEL *pmod) 
+{
+    if (pmod->ci == OLS || pmod->ci == WLS ||
+	pmod->ci == HSK || pmod->ci == AR1 ||
+	pmod->ci == ARCH || pmod->ci == IVREG ||
+	pmod->ci == LAD || pmod->ci == PANEL ||
+	pmod->ci == LOGIT || pmod->ci == PROBIT ||
+	pmod->ci == TOBIT) {
+	return pmod->ncoeff <= 6;
+    } else {
+	return 0;
+    }
+}
+
+static char *eqn_numstr (double x, char *s)
+{
+    sprintf(s, "%#.3g", x);
+    return gretl_fix_exponent(s);
+}
+
+int text_print_equation (const MODEL *pmod, const DATAINFO *pdinfo, 
+			 gretlopt opt, PRN *prn)
+{
+    double x;
+    char vname[32], xstr[12];
+    int pad, c, cols[6] = {0};
+    int i, nc = pmod->ncoeff;
+
+    /* dependent variable */
+    pputc(prn, '\n');
+    c = pprintf(prn, "^%s = ", gretl_model_get_depvar_name(pmod, pdinfo));
+
+    /* coefficients times indep vars */
+
+    for (i=0; i<nc; i++) {
+	cols[i] = (i == 0)? (c - 1) : (c + 2);
+	if (i == 0 && pmod->ifc) {
+	    eqn_numstr(pmod->coeff[i], xstr);
+	    c += pputs(prn, xstr);
+	} else {
+	    eqn_numstr(fabs(pmod->coeff[i]), xstr);
+	    c += pprintf(prn, " %c %s", (pmod->coeff[i] < 0.0)? '-' : '+',
+			 xstr);
+	    gretl_model_get_param_name(pmod, pdinfo, i, vname);
+	    c += pprintf(prn, "*%s", vname);
+	}
+    }
+    pputc(prn, '\n');
+
+    /* standard errors or t-stats */
+
+    c = cols[0];
+    bufspace(cols[0], prn);
+
+    for (i=0; i<nc; i++) {
+	if (i > 0) {
+	    pad = cols[i] - c;
+	    if (pad > 0) {
+		bufspace(pad, prn);
+		c += pad;
+	    }
+	}
+	if (na(pmod->sderr[i])) {
+	    c += pprintf(prn, "(NA)");
+	} else if (opt & OPT_T) {
+	    x = pmod->coeff[i] / pmod->sderr[i];
+	    eqn_numstr(x, xstr);
+	    c += pprintf(prn, "(%s)", xstr);
+	} else {
+	    eqn_numstr(pmod->sderr[i], xstr);
+	    c += pprintf(prn, "(%s)", xstr);
+	}
+    }
+
+    pputs(prn, "\n\n");
+
+    if (dataset_is_time_series(pdinfo)) {
+	pprintf(prn, "T = %d", pmod->nobs);
+    } else {
+	pprintf(prn, "n = %d", pmod->nobs);
+    }
+
+    /* additional info (R^2 etc) */
+    if (pmod->ci == LAD) { 
+	x = gretl_model_get_double(pmod, "ladsum");
+	if (!na(x)) {
+	    eqn_numstr(x, xstr);
+	    pprintf(prn, ", sum of abs. residuals = %s ", xstr);
+	}
+    } else {
+	if (!na(pmod->adjrsq)) {
+	    pprintf(prn, ", R-squared = %.3f ", pmod->rsq);
+	} else if (!na(pmod->lnL)) {
+	    eqn_numstr(pmod->lnL, xstr);
+	    pprintf(prn, ", loglikelihood = %s ", xstr);
+	}
+	x = gretl_model_get_double(pmod, "rho_in");
+	if (!na(x)) {
+	    pprintf(prn, ", rho = %.3f", x);
+	}
+    }
+
+    pprintf(prn, "\n(%s)\n",
+	    (opt & OPT_T)? I_("t-ratios in parentheses") :
+	    I_("standard errors in parentheses"));
+
+    return 0;
+}
+
 /* print value in (non-correlation) matrix */
 
 static void tex_matnum (double x, PRN *prn)
@@ -1138,6 +1240,65 @@ void special_print_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo,
     }
 }
 
+static int texprint_fcast_stats (const FITRESID *fr, 
+				 gretlopt opt,
+				 PRN *prn)
+{
+    const char *strs[] = {
+	N_("Mean Error"),
+	N_("Mean Squared Error"),
+	N_("Root Mean Squared Error"),
+	N_("Mean Absolute Error"),
+	N_("Mean Percentage Error"),
+	N_("Mean Absolute Percentage Error"),
+	N_("Theil's $U$"),
+	N_("Bias proportion, $U^M$"),
+	N_("Regression proportion, $U^R$"),
+	N_("Disturbance proportion, $U^D$")
+    };
+    gretl_matrix *m;
+    double x;
+    int i, j, t1, t2;
+    int len, err = 0;
+
+    fcast_get_continuous_range(fr, &t1, &t2);
+
+    if (t2 - t1 + 1 <= 0) {
+	return E_MISSDATA;
+    }
+
+    m = forecast_stats(fr->actual, fr->fitted, t1, t2, opt, &err);
+    if (err) {
+	return err;
+    }
+
+    len = gretl_vector_get_length(m);
+
+    pputs(prn, _("Forecast evaluation statistics"));
+    pputs(prn, "\\\\[1ex]\n\n");
+
+    pputs(prn, "\\begin{tabular}{ll}\n");
+
+    j = 0;
+    for (i=0; i<len; i++) {
+	x = gretl_vector_get(m, i);
+	if (!isnan(x)) {
+	    pprintf(prn, "%s & %s%.5g \\\\\n", I_(strs[j]), (x < 0)? "$-$" : "", 
+		    fabs(x));
+	    if (i == 1) {
+		pprintf(prn, "%s & %.5g \\\\\n", I_(strs[j+1]), sqrt(x));	
+	    }
+	}
+	j += (i == 1)? 2 : 1;
+    }
+
+    pputs(prn, "\\end{tabular}\n");
+    
+    gretl_matrix_free(m);
+
+    return err;
+}
+
 static 
 void tex_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo, 
 			 PRN *prn)
@@ -1235,12 +1396,16 @@ static void texprint_fit_resid (const FITRESID *fr,
 	pputs(prn, " \\\\\n");
     }
 
-    pputs(prn, "\\end{longtable}\n\\end{center}\n\n");
+    pputs(prn, "\\end{longtable}\n\n");
 
     if (anyast) {
 	pputs(prn, I_("\\textit{Note}: * denotes a residual "
 		      "in excess of 2.5 standard errors\n\n"));
     }
+
+    texprint_fcast_stats(fr, OPT_NONE, prn);
+
+    pputs(prn, "\\end{center}\n\n");
 }
 
 #define FR_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
@@ -1351,7 +1516,9 @@ static void texprint_fcast_without_errs (const FITRESID *fr,
 		actual, fitted);
     }
 
-    pputs(prn, "\\end{longtable}\n\\end{center}\n\n");
+    pputs(prn, "\\end{longtable}\n\n");
+    texprint_fcast_stats(fr, OPT_D, prn);
+    pputs(prn, "\\end{center}\n\n");
 }
 
 static void texprint_fcast_with_errs (const FITRESID *fr, 
@@ -1429,7 +1596,10 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 		actual, fitted, sderr, lo, hi);
     }
 
-    pputs(prn, "\\end{longtable}\n\\end{center}\n\n");
+    pputs(prn, "\\end{longtable}\n\n");
+    texprint_fcast_stats(fr, OPT_D, prn);
+    pputs(prn, "\\end{center}\n\n");
+    
 }
 
 #define FC_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
@@ -1729,38 +1899,13 @@ static int real_csv_to_clipboard (const int *list)
     return err;
 }
 
-int csv_to_clipboard (void)
-{
-    gretlopt opt = OPT_NONE;
-    int *list = NULL;
-    int err = 0;
-
-    if (delimiter_dialog(&opt)) {
-	return 0;
-    }
-
-    data_save_selection_wrapper(COPY_CSV, GINT_TO_POINTER(opt));
-
-    if (storelist != NULL && *storelist != '\0') {
-	list = gretl_list_from_string(storelist, &err);	
-	if (!err && list != NULL) {
-	    err = real_csv_to_clipboard(list);
-	    free(list);
-	}
-	free(storelist);
-	storelist = NULL;
-    }
-
-    return err;
-}
-
 int csv_selected_to_clipboard (void)
 {
     int *list = main_window_selection_as_list();
     int err = 0;
 
     if (list != NULL) {
-	if (delimiter_dialog(NULL)) {
+	if (csv_options_dialog(NULL)) {
 	    return 0;
 	}
 	err = real_csv_to_clipboard(list);
@@ -1770,6 +1915,32 @@ int csv_selected_to_clipboard (void)
     return err;
 }
 
+/* called from session.c: copy entire dataset to clipboard */
+
+int csv_to_clipboard (void)
+{
+    int cancel, err = 0;
+
+    data_save_selection_wrapper(COPY_CSV);
+
+    if (storelist != NULL && *storelist != '\0') {
+	int *list = gretl_list_from_string(storelist, &err);	
+
+	if (list != NULL) {
+	    cancel = csv_options_dialog(NULL);
+	    if (!cancel) {
+		err = real_csv_to_clipboard(list);
+	    }
+	    free(list);
+	}
+
+	free(storelist);
+	storelist = NULL;
+    }
+
+    return err;
+}
+
 int scalars_to_clipboard_as_csv (void)
 {
     PRN *prn = NULL;
@@ -1780,7 +1951,7 @@ int scalars_to_clipboard_as_csv (void)
 	return 0;
     }
 
-    if (delimiter_dialog(NULL)) {
+    if (csv_options_dialog(NULL)) {
 	return 0;
     }
 
@@ -1846,9 +2017,9 @@ int copy_vars_formatted (windata_t *vwin, int fmt, int action)
 	    if (action == W_COPY) {
 		err = prn_to_clipboard(prn, fmt);
 	    } else if (fmt == GRETL_FORMAT_RTF) {
-		file_selector(_("Save"), SAVE_RTF, FSEL_DATA_PRN, prn);
+		file_selector(SAVE_RTF, FSEL_DATA_PRN, prn);
 	    } else {
-		file_selector(_("Save data"), EXPORT_CSV, FSEL_DATA_PRN, prn);
+		file_selector(EXPORT_CSV, FSEL_DATA_PRN, prn);
 	    }
 	}
 
@@ -1876,7 +2047,7 @@ int font_has_minus (PangoFontDescription *desc)
 	return -1;
     }
 
-    widget = gtk_label_new(NULL);  
+    widget = gtk_label_new("");  
     context = gtk_widget_get_pango_context(widget); 
 
     if (context == NULL) {
@@ -1894,14 +2065,15 @@ int font_has_minus (PangoFontDescription *desc)
 	    if (coverage != NULL) {
 		/* U+2212 = minus sign */
 		ret = (pango_coverage_get(coverage, 0x2212) == PANGO_COVERAGE_EXACT);
+		pango_coverage_unref(coverage);
 	    }
+	    g_object_unref(G_OBJECT(pfont));
 	}
     } 
 
-    pango_coverage_unref(coverage);
     g_object_unref(G_OBJECT(layout));
     g_object_unref(G_OBJECT(context));
-    gtk_widget_destroy(widget);    
+    gtk_widget_destroy(widget);
 
     return ret;
 }
