@@ -23,7 +23,8 @@
 #define GUIPRINT_H
 
 #ifdef NATIVE_PRINTING
-void print_window_content (char *fullbuf, char *selbuf);
+void print_window_content (char *fullbuf, char *selbuf, 
+			   const char *fname);
 # ifdef G_OS_WIN32
 int win32_print_graph (char *emfname);
 # else
@@ -50,6 +51,11 @@ void special_print_forecast (const FITRESID *fr,
 void special_print_confints (const CoeffIntervals *cf, 
 			     PRN *prn);
 
+int text_print_equation (const MODEL *pmod, const DATAINFO *pdinfo, 
+			 gretlopt opt, PRN *prn);
+
+int text_equation_ok (const MODEL *pmod);
+
 int csv_to_clipboard (void);
 
 int csv_selected_to_clipboard (void);
