@@ -67,8 +67,6 @@ void menu_op_action (GtkAction *action, gpointer data);
 
 void do_menu_op (int ci, const char *liststr, gretlopt opt);
 
-void set_currdir_from_filename (const char *fname);
-
 void do_run_script (GtkWidget *w, windata_t *vwin);
 
 void run_script_fragment (windata_t *vwin, gchar *buf);
@@ -81,8 +79,6 @@ void register_graph (void);
 
 int bool_subsample (gretlopt opt);
 
-void do_samplebool (GtkWidget *w, dialog_t *dlg);
-
 int do_set_sample (void);
 
 void drop_all_missing (void);
@@ -91,7 +87,9 @@ void count_missing (void);
 
 void do_add_markers (const char *fname);
 
-void do_remove_markers (void);
+int do_save_markers (const char *fname);
+
+void markers_callback (void);
 
 int dataset_is_restricted (void);
 
@@ -161,6 +159,8 @@ void do_graph_model (const int *list, int fit);
 
 void do_gini (void);
 
+void do_qqplot (void);
+
 void do_kernel (void);
 
 void do_vif (GtkAction *action, gpointer p);
@@ -179,8 +179,6 @@ int out_of_sample_info (int add_ok, int *t2);
 
 /* variable-related functions */
 
-int reject_scalar (int vnum);
-
 void do_minibuf (GtkWidget *w, dialog_t *dlg);
 
 void do_genr (GtkWidget *w, dialog_t *dlg);
@@ -199,9 +197,7 @@ int record_varlabel_change (int v);
 
 void do_resid_freq (GtkAction *action, gpointer p);
 
-void do_freq_dist (int plot);
-
-void freq_callback (GtkAction *action);
+void do_freq_dist (void);
 
 void do_corrgm (void);
 
@@ -211,6 +207,8 @@ void do_pergm (GtkAction *action);
 
 void residual_periodogram (GtkAction *action, gpointer p);
 
+void residual_qq_plot (GtkAction *action, gpointer p);
+
 #if defined (HAVE_TRAMO) || defined (HAVE_X12A)
 void do_tramo_x12a (GtkAction *action, gpointer p);
 #endif
@@ -237,7 +235,7 @@ void logs_etc_callback (GtkAction *action);
 
 void add_system_resid (GtkAction *action, gpointer p);
 
-int add_fit_resid (MODEL *pmod, int code, int undo);
+int save_fit_resid (MODEL *pmod, int code);
 
 void add_model_stat (MODEL *pmod, int which);
 
@@ -283,7 +281,7 @@ void display_var (void);
 
 void do_open_script (int action);
 
-void open_info (void);
+void dataset_info (void);
 
 void do_new_script (int code);
 
