@@ -24,6 +24,8 @@
 #include "guiprint.h"
 #include "ssheet.h"
 #include "selector.h"
+#include "varinfo.h"
+#include "treeutils.h"
 #include "menustate.h"
 
 void refresh_data (void)
@@ -56,17 +58,6 @@ void flip (GtkUIManager *ui, const char *path, gboolean s)
     }
 }
 
-void edit_info_state (gboolean s)
-{
-    flip(mdata->ui, "/MenuBar/Data/EditInfo", s);
-}
-
-void add_remove_markers_state (gboolean s)
-{
-    flip(mdata->ui, "/MenuBar/Data/AddMarkers", !s);
-    flip(mdata->ui, "/MenuBar/Data/RemoveMarkers", s);
-}
-
 /* by using gretl_set_window_modal() we make the main
    window visibly insensitive */
 
@@ -113,40 +104,81 @@ void variable_menu_state (gboolean s)
 {
     if (mdata == NULL || mdata->ui == NULL) return;
 
-    flip(mdata->ui, "/MenuBar/Variable", s);
-    flip(mdata->ui, "/MenuBar/View/xcorrgm",  
+    flip(mdata->ui, "/menubar/Variable", s);
+    flip(mdata->ui, "/menubar/View/xcorrgm",  
 	 dataset_is_time_series(datainfo));
 }
 
+static void view_items_state (gboolean s)
+{
+    const char *viewpaths[] = {
+	"IconView",
+	"EditScalars",
+	"GraphVars",
+	"MultiPlots",
+	"summary",
+	"corr",
+	"xtab",
+	"pca",
+	"mahal",
+	"xcorrgm",
+	NULL
+    };
+    char fullpath[32];
+    int i;
+
+    for (i=0; viewpaths[i] != NULL; i++) {
+	sprintf(fullpath, "/menubar/View/%s", viewpaths[i]);
+	flip(mdata->ui, fullpath, s);
+    }
+}
+
 void main_menubar_state (gboolean s)
 {
     if (mdata == NULL || mdata->ui == NULL) return;
 
-    flip(mdata->ui, "/MenuBar/File/AppendData", s);
-    flip(mdata->ui, "/MenuBar/File/ClearData", s);
-    flip(mdata->ui, "/MenuBar/File/SaveData", s);
-    flip(mdata->ui, "/MenuBar/File/SaveDataAs", s);
-    flip(mdata->ui, "/MenuBar/File/ExportData", s);
-    flip(mdata->ui, "/MenuBar/File/MailData", s);
-    flip(mdata->ui, "/MenuBar/Data", s);
-    flip(mdata->ui, "/MenuBar/View", s);
-    flip(mdata->ui, "/MenuBar/Add", s);
-    flip(mdata->ui, "/MenuBar/Sample", s);
-    flip(mdata->ui, "/MenuBar/Variable", s);
-    flip(mdata->ui, "/MenuBar/Model", s);
-    flip(mdata->ui, "/MenuBar/Tools/SortVars", s);
+    flip(mdata->ui, "/menubar/File/AppendData", s);
+    flip(mdata->ui, "/menubar/File/ClearData", s);
+    flip(mdata->ui, "/menubar/File/SaveData", s);
+    flip(mdata->ui, "/menubar/File/SaveDataAs", s);
+    flip(mdata->ui, "/menubar/File/ExportData", s);
+    flip(mdata->ui, "/menubar/File/MailData", s);
+    flip(mdata->ui, "/menubar/Data", s);
+    flip(mdata->ui, "/menubar/Add", s);
+    flip(mdata->ui, "/menubar/Sample", s);
+    flip(mdata->ui, "/menubar/Variable", s);
+    flip(mdata->ui, "/menubar/Model", s);
 
-    flip(mdata->ui, "/MenuBar/File/NewData", !s);
+    view_items_state(s);
 
+    if (s || get_n_listed_windows() == 0) {
+	flip(mdata->ui, "/menubar/View", s);
+    }
+
+    flip(mdata->ui, "/menubar/File/NewData", !s);
+
+    set_main_colheads_clickable(s);
+}
+
+void window_list_state (gboolean s)
+{
     if (s) {
-	edit_info_state(!(data_status & BOOK_DATA));
-	add_remove_markers_state(datainfo->S != NULL);
+	GtkAction *a = gtk_ui_manager_get_action(mdata->ui, "/menubar/View");
+
+	if (a != NULL && !gtk_action_get_sensitive(a)) {
+	    gtk_action_set_sensitive(a, TRUE);
+	    view_items_state(FALSE);
+	}
+	flip(mdata->ui, "/menubar/View/Windows", s);
+    } else if (!data_status) {
+	flip(mdata->ui, "/menubar/View", s);
     }
 }
 
 #define COMPACTABLE(d) (d->structure == TIME_SERIES && \
                         (d->pd == 4 || d->pd == 12 || \
-                         d->pd == 5 || d->pd == 7))
+                         d->pd == 5 || d->pd == 6 || \
+                         d->pd == 7 || d->pd == 24))
 
 #define EXPANSIBLE(d) (d->structure == TIME_SERIES && (d->pd == 1 || d->pd == 4))
 
@@ -157,12 +189,6 @@ void main_menubar_state (gboolean s)
 			(d)->structure == SPECIAL_TIME_SERIES || \
 			(d)->structure == STACKED_TIME_SERIES)
 
-#define tramo_ts(d) ((d)->structure == TIME_SERIES && \
-                     (d->pd == 1 || d->pd == 4 || d->pd == 12))
-
-#define x12_ts(d) ((d)->structure == TIME_SERIES && \
-                   (d->pd == 4 || d->pd == 12))
-
 void time_series_menu_state (gboolean s)
 {
     gboolean sx = extended_ts(datainfo);
@@ -172,41 +198,44 @@ void time_series_menu_state (gboolean s)
     }
 
     /* File menu */
-    flip(mdata->ui, "/MenuBar/File/SaveDataAs/SaveAsDb", DATASET_DB_OK(datainfo));
+    flip(mdata->ui, "/menubar/File/SaveDataAs/SaveAsDb", DATASET_DB_OK(datainfo));
 
     /* Plots */
-    flip(mdata->ui, "/MenuBar/View/GraphVars/TSPlot", sx);
-    flip(mdata->ui, "/MenuBar/View/MultiPlots/MultiTS", sx);
-    flip(mdata->ui, "/MenuBar/Variable/VarTSPlot", sx);
+    flip(mdata->ui, "/menubar/View/GraphVars/TSPlot", sx);
+    flip(mdata->ui, "/menubar/View/MultiPlots/MultiTS", sx);
+    flip(mdata->ui, "/menubar/Variable/VarTSPlot", sx);
 
     /* Variable menu */
-    flip(mdata->ui, "/MenuBar/Variable/corrgm", s);
-    flip(mdata->ui, "/MenuBar/Variable/Spectrum", s);
-    flip(mdata->ui, "/MenuBar/Variable/ADF", s);
-    flip(mdata->ui, "/MenuBar/Variable/DFGLS", s);
-    flip(mdata->ui, "/MenuBar/Variable/KPSS", s);
-    flip(mdata->ui, "/MenuBar/Variable/Filter", s);
+    flip(mdata->ui, "/menubar/Variable/corrgm", s);
+    flip(mdata->ui, "/menubar/Variable/Spectrum", s);
+    flip(mdata->ui, "/menubar/Variable/ADF", s);
+    flip(mdata->ui, "/menubar/Variable/DFGLS", s);
+    flip(mdata->ui, "/menubar/Variable/KPSS", s);
+    flip(mdata->ui, "/menubar/Variable/Filter", s);
 #ifdef HAVE_X12A
-    flip(mdata->ui, "/MenuBar/Variable/X12A", x12_ts(datainfo));
+    flip(mdata->ui, "/menubar/Variable/X12A", get_x12a_ok());
 #endif
 #ifdef HAVE_TRAMO
-    flip(mdata->ui, "/MenuBar/Variable/Tramo", tramo_ts(datainfo));
+    flip(mdata->ui, "/menubar/Variable/Tramo", get_tramo_ok());
 #endif
-    flip(mdata->ui, "/MenuBar/Variable/Hurst", s);
+    flip(mdata->ui, "/menubar/Variable/Hurst", s);
     /* Model menu */
-    flip(mdata->ui, "/MenuBar/Model/TSModels", s);
+    flip(mdata->ui, "/menubar/Model/TSModels", s);
     /* Sample menu */
-    flip(mdata->ui, "/MenuBar/Data/DataCompact", 
+    flip(mdata->ui, "/menubar/Data/DataCompact", 
 	 s && (COMPACTABLE(datainfo) || dated_weekly_data(datainfo)));
-    flip(mdata->ui, "/MenuBar/Data/DataExpand", s && EXPANSIBLE(datainfo));
+    flip(mdata->ui, "/menubar/Data/DataExpand", s && EXPANSIBLE(datainfo));
 }
 
 void panel_menu_state (gboolean s)
 {
     if (mdata->ui != NULL) {
-	flip(mdata->ui, "/MenuBar/Add/UnitDums", s);
-	flip(mdata->ui, "/MenuBar/Add/TimeDums", s);
-	flip(mdata->ui, "/MenuBar/Model/PanelModels", s);
+	flip(mdata->ui, "/menubar/Add/UnitDums", s);
+	flip(mdata->ui, "/menubar/Add/TimeDums", s);
+	flip(mdata->ui, "/menubar/Model/PanelModels", s);
+	if (s && datainfo->pd <= 2) {
+	    flip(mdata->ui, "/menubar/Model/PanelModels/arbond", 0);
+	}
     }
 }
 
@@ -214,24 +243,24 @@ void ts_or_panel_menu_state (gboolean s)
 {
     if (mdata->ui == NULL) return;
 
-    flip(mdata->ui, "/MenuBar/Data/DataSort", !s);
+    flip(mdata->ui, "/menubar/Data/DataSort", !s);
 
-    flip(mdata->ui, "/MenuBar/Add/AddTime", s);
-    flip(mdata->ui, "/MenuBar/Add/lags", s);
-    flip(mdata->ui, "/MenuBar/Add/diff", s);
-    flip(mdata->ui, "/MenuBar/Add/ldiff", s);
+    flip(mdata->ui, "/menubar/Add/AddTime", s);
+    flip(mdata->ui, "/menubar/Add/lags", s);
+    flip(mdata->ui, "/menubar/Add/diff", s);
+    flip(mdata->ui, "/menubar/Add/ldiff", s);
 
     s = dataset_is_seasonal(datainfo);
-    flip(mdata->ui, "/MenuBar/Add/sdiff", s);
-    flip(mdata->ui, "/MenuBar/Add/PeriodDums", s);
+    flip(mdata->ui, "/menubar/Add/sdiff", s);
+    flip(mdata->ui, "/menubar/Add/PeriodDums", s);
 }
 
 void session_menu_state (gboolean s)
 {
     if (mdata->ui != NULL) {
-	flip(mdata->ui, "/MenuBar/View/IconView", s);
-	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", s);
-	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSessionAs", s);
+	flip(mdata->ui, "/menubar/View/IconView", s);
+	flip(mdata->ui, "/menubar/File/SessionFiles/SaveSession", s);
+	flip(mdata->ui, "/menubar/File/SessionFiles/SaveSessionAs", s);
     }
 
     if (!s && mdata->main != NULL) {
@@ -242,35 +271,35 @@ void session_menu_state (gboolean s)
 void restore_sample_state (gboolean s)
 {
     if (mdata->ui != NULL) {
-	flip(mdata->ui, "/MenuBar/Sample/FullRange", s);
+	flip(mdata->ui, "/menubar/Sample/FullRange", s);
     }
 }
 
 void drop_obs_state (gboolean s)
 {
     if (mdata->ui != NULL) {
-	flip(mdata->ui, "/MenuBar/Data/RemoveObs", s);
+	flip(mdata->ui, "/menubar/Data/RemoveObs", s);
     }
 }
 
 void compact_data_state (gboolean s)
 {
     if (mdata->ui != NULL) {
-	flip(mdata->ui, "/MenuBar/Data/DataCompact", s);
+	flip(mdata->ui, "/menubar/Data/DataCompact", s);
     }
 }
 
 void main_menus_enable (gboolean s)
 {
     if (mdata->ui != NULL) {
-	flip(mdata->ui, "/MenuBar/File", s);
-	flip(mdata->ui, "/MenuBar/Tools", s);
-	flip(mdata->ui, "/MenuBar/Data", s);
-	flip(mdata->ui, "/MenuBar/View", s);
-	flip(mdata->ui, "/MenuBar/Add", s);
-	flip(mdata->ui, "/MenuBar/Sample", s);
-	flip(mdata->ui, "/MenuBar/Variable", s);
-	flip(mdata->ui, "/MenuBar/Model", s);
+	flip(mdata->ui, "/menubar/File", s);
+	flip(mdata->ui, "/menubar/Tools", s);
+	flip(mdata->ui, "/menubar/Data", s);
+	flip(mdata->ui, "/menubar/View", s);
+	flip(mdata->ui, "/menubar/Add", s);
+	flip(mdata->ui, "/menubar/Sample", s);
+	flip(mdata->ui, "/menubar/Variable", s);
+	flip(mdata->ui, "/menubar/Model", s);
     }
 }
 
@@ -286,9 +315,7 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
     else if (!strcmp(item, _("Time series plot"))) 
 	do_graph_var(v);
     else if (!strcmp(item, _("Frequency distribution"))) 
-	do_freq_dist(0);
-    else if (!strcmp(item, _("Frequency plot"))) 
-	do_freq_dist(1);
+	do_freq_dist();
     else if (!strcmp(item, _("Boxplot")))
 	do_boxplot_var(v);
     else if (!strcmp(item, _("Gini coefficient")))
@@ -369,7 +396,7 @@ GtkWidget *build_var_popup (void)
 	N_("Display values"),
 	N_("Descriptive statistics"),
 	N_("Time series plot"),
-	N_("Frequency plot"),
+	N_("Frequency distribution"),
 	N_("Boxplot"),
 	N_("Correlogram"),
 	N_("Spectrum"),
@@ -483,21 +510,9 @@ void set_main_window_title (const char *name, gboolean modified)
     }
 }
 
-void set_sample_label (DATAINFO *pdinfo)
+static const char *get_pd_string (DATAINFO *pdinfo)
 {
-    GtkWidget *dlabel;
-    char stobs[OBSLEN], endobs[OBSLEN];
-    char labeltxt[128];
-    const char *pdstr;
-
-    if (mdata == NULL) {
-	return;
-    }
-
-    dlabel = g_object_get_data(G_OBJECT(mdata->main), "dlabel");
-
-    ntodate(stobs, 0, pdinfo);
-    ntodate(endobs, pdinfo->n - 1, pdinfo);
+    char *pdstr;
 
     if (custom_time_series(pdinfo)) {
 	pdstr = N_("Time series");
@@ -527,61 +542,94 @@ void set_sample_label (DATAINFO *pdinfo)
     } else {
 	pdstr = N_("Undated");
     }
+    
+    return pdstr;
+}
+
+void set_sample_label (DATAINFO *pdinfo)
+{
+    GtkWidget *dlabel;
+    char t1str[OBSLEN], t2str[OBSLEN];
+    char tmp[256];
+
+    if (mdata == NULL) {
+	return;
+    }
+
+    /* set the sensitivity of various menu items */
 
     time_series_menu_state(dataset_is_time_series(pdinfo));
     panel_menu_state(dataset_is_panel(pdinfo));
     ts_or_panel_menu_state(dataset_is_time_series(pdinfo) ||
 			   dataset_is_panel(pdinfo));
+    flip(mdata->ui, "/menubar/Data/DataTranspose", !dataset_is_panel(pdinfo));
 
-    flip(mdata->ui, "/MenuBar/Data/DataTranspose", !dataset_is_panel(pdinfo));
-
+    /* construct label showing summary of dataset/sample info
+       (this goes at the foot of the window) */
+    
     if (complex_subsampled() && pdinfo->t1 == 0 && 
 	pdinfo->t2 == pdinfo->n - 1 && 
 	datainfo->structure == CROSS_SECTION) {
-	sprintf(labeltxt, _("Undated: Full range n = %d; current sample"
-			    " n = %d"), get_full_length_n(), datainfo->n);
+	sprintf(tmp, _("Undated: Full range n = %d; current sample"
+		       " n = %d"), get_full_length_n(), datainfo->n);
     } else {
-	sprintf(labeltxt, _("%s: Full range %s - %s"), 
-		_(pdstr), stobs, endobs);
+	const char *pdstr;
+
+	ntodate(t1str, 0, pdinfo);
+	ntodate(t2str, pdinfo->n - 1, pdinfo);
+	pdstr = get_pd_string(pdinfo);
+	sprintf(tmp, _("%s: Full range %s - %s"), _(pdstr), 
+		t1str, t2str);
     }
 
     if (pdinfo->t1 > 0 || pdinfo->t2 < pdinfo->n - 1) {
-	char t1str[OBSLEN], t2str[OBSLEN];
-	char biglabel[160];
+	gchar *fulltext;
 
 	ntodate(t1str, pdinfo->t1, pdinfo);
 	ntodate(t2str, pdinfo->t2, pdinfo);
-	sprintf(biglabel, _("%s; sample %s - %s"), labeltxt, t1str, t2str);
-	gtk_label_set_text(GTK_LABEL(mdata->status), biglabel);
+	fulltext = g_strdup_printf(_("%s; sample %s - %s"), tmp, t1str, t2str);
+	gtk_label_set_text(GTK_LABEL(mdata->status), fulltext);
+	g_free(fulltext);
     } else {
-	gtk_label_set_text(GTK_LABEL(mdata->status), labeltxt);
+	gtk_label_set_text(GTK_LABEL(mdata->status), tmp);
     }
 
-    if (strlen(paths.datfile) > 2) {
-	/* data file open already */
-	const char *p = strrchr(paths.datfile, SLASH);
-	gchar *trfname;
+    /* construct label with datafile name (this goes above the
+       data series window) */
 
-	if (p != NULL) {
-	    trfname = my_filename_to_utf8(p + 1);
-	} else {
-	    trfname = my_filename_to_utf8(paths.datfile);
-	}
+    dlabel = g_object_get_data(G_OBJECT(mdata->main), "dlabel");
 
-	sprintf(labeltxt, " %s ", trfname);
-	g_free(trfname);
-	if (data_status & MODIFIED_DATA) { 
-	    strcat(labeltxt, "* ");
-	} 
-	if (dlabel != NULL) {
-	    gtk_label_set_text(GTK_LABEL(dlabel), labeltxt);
+    if (dlabel != NULL) {
+	if (strlen(datafile) > 2) {
+	    /* data file open already */
+	    const char *p = strrchr(datafile, SLASH);
+	    gchar *trfname;
+
+	    if (p != NULL) {
+		trfname = my_filename_to_utf8(p + 1);
+	    } else {
+		trfname = my_filename_to_utf8(datafile);
+	    }
+
+	    strcpy(tmp, " ");
+
+	    if (data_status & SESSION_DATA) {
+		sprintf(tmp + 1, "Imported %s", trfname);
+	    } else if (data_status & MODIFIED_DATA) {
+		sprintf(tmp + 1, "%s *", trfname);
+	    } else {
+		sprintf(tmp + 1, "%s", trfname);
+	    }
+
+	    gtk_label_set_text(GTK_LABEL(dlabel), tmp);
+	    g_free(trfname);
+	} else if (data_status & MODIFIED_DATA) {
+	    strcpy(tmp, _(" Unsaved data "));
+	    gtk_label_set_text(GTK_LABEL(dlabel), tmp);
 	}
-    } else if (data_status & MODIFIED_DATA) {
-	strcpy(labeltxt, _(" Unsaved data "));
-	gtk_label_set_text(GTK_LABEL(dlabel), labeltxt);
     }
 
-    if (complex_subsampled() || pdinfo->t1 > 0 ||
+    if (complex_subsampled() || pdinfo->t1 > 0 || 
 	pdinfo->t2 < pdinfo->n - 1) {
 	restore_sample_state(TRUE);
     }
@@ -604,9 +652,7 @@ int vwin_add_ui (windata_t *vwin, GtkActionEntry *entries,
     GError *err = NULL;
 
     actions = gtk_action_group_new("MyActions");
-#ifdef ENABLE_NLS
     gtk_action_group_set_translation_domain(actions, "gretl");
-#endif
 
     gtk_action_group_add_actions(actions, entries, n_entries, vwin);
 
@@ -623,7 +669,7 @@ int vwin_add_ui (windata_t *vwin, GtkActionEntry *entries,
 	g_error_free(err);
     }
 
-    vwin->mbar = gtk_ui_manager_get_widget(vwin->ui, "/MenuBar");
+    vwin->mbar = gtk_ui_manager_get_widget(vwin->ui, "/menubar");
 
     return 0;
 }
@@ -635,9 +681,7 @@ int vwin_menu_add_item (windata_t *vwin, const gchar *path,
     GtkActionGroup *actions;
 
     actions = gtk_action_group_new("AdHoc");
-#ifdef ENABLE_NLS
     gtk_action_group_set_translation_domain(actions, "gretl");
-#endif
 
     gtk_action_group_add_actions(actions, entry, 1, vwin);
     gtk_ui_manager_add_ui(vwin->ui, id, path, entry->name, entry->name,
@@ -656,9 +700,7 @@ int vwin_menu_add_items (windata_t *vwin, const gchar *path,
     int i;
 
     actions = gtk_action_group_new("AdHoc");
-#ifdef ENABLE_NLS
     gtk_action_group_set_translation_domain(actions, "gretl");
-#endif
 
     gtk_action_group_add_actions(actions, entries, n, vwin);
     for (i=0; i<n; i++) {
@@ -681,9 +723,7 @@ int vwin_menu_add_radios (windata_t *vwin, const gchar *path,
     int i;
 
     actions = gtk_action_group_new("Radios");
-#ifdef ENABLE_NLS
     gtk_action_group_set_translation_domain(actions, "gretl");
-#endif
 
     gtk_action_group_add_radio_actions(actions, entries, n,
 				       deflt, callback,
@@ -706,9 +746,7 @@ int vwin_menu_add_menu (windata_t *vwin, const gchar *path,
     GtkActionGroup *actions;
 
     actions = gtk_action_group_new("NewMenu");
-#ifdef ENABLE_NLS
     gtk_action_group_set_translation_domain(actions, "gretl");
-#endif
 
     gtk_action_group_add_actions(actions, entry, 1, vwin);
     gtk_ui_manager_add_ui(vwin->ui, id, path, entry->name, entry->name,
@@ -726,8 +764,3 @@ void vwin_menu_add_separator (windata_t *vwin, const gchar *path)
     gtk_ui_manager_add_ui(vwin->ui, id, path, NULL, NULL,
 			  GTK_UI_MANAGER_SEPARATOR, FALSE);
 }
-
-
-
-
-
