@@ -8,8 +8,6 @@ void gretl_set_window_quasi_modal (GtkWidget *w);
 
 void flip (GtkUIManager *ui, const char *path, gboolean s);
 
-void edit_info_state (gboolean s);
-void add_remove_markers_state (gboolean s);
 void variable_menu_state (gboolean s);
 void main_menubar_state (gboolean s);
 void time_series_menu_state (gboolean s);
@@ -19,6 +17,7 @@ void session_menu_state (gboolean s);
 void restore_sample_state (gboolean s);
 void compact_data_state (gboolean s);
 void drop_obs_state (gboolean s);
+void window_list_state (gboolean s);
 
 void main_menus_enable (gboolean s);
 
