@@ -23,6 +23,7 @@
 #include "model_table.h"
 #include "session.h"
 #include "textutil.h"
+#include "textbuf.h"
 #include "texprint.h"
 
 static MODEL **table_models;
@@ -30,14 +31,25 @@ static int n_models;
 static char **pnames;
 static int n_params;
 static int use_tstats;
+static int do_pvals;
+static int colheads;
 static int depvarnum;
-static int colwidth;
+
+static int do_asts = 1;
+static int mt_figs = 4;
 
 static void print_rtf_row_spec (PRN *prn, int tall);
 
 #define MAX_PORTRAIT_MODELS 6
 #define MAX_TABLE_MODELS 12
 
+enum {
+    COLHEAD_ARABIC,
+    COLHEAD_ROMAN,
+    COLHEAD_ALPHA,
+    COLHEAD_NAMES
+};
+
 static void mtable_errmsg (char *msg, int gui)
 {
     if (gui) {
@@ -81,6 +93,9 @@ int in_model_table (const MODEL *pmod)
     }
 
     for (i=0; i<n_models; i++) {
+	if (table_models[i] == NULL) {
+	    continue;
+	}
 	if (pmod == table_models[i] || pmod->ID == table_models[i]->ID) {
 	    return 1;
 	}
@@ -108,10 +123,27 @@ MODEL *model_table_model_by_index (int i)
     }
 }
 
-void clear_model_table (PRN *prn)
+int model_table_position (const MODEL *pmod)
 {
     int i;
 
+    for (i=0; i<n_models; i++) {
+	if (pmod == table_models[i]) {
+	    return i + 1;
+	}
+    }
+
+    return 0;
+}
+
+void clear_model_table (int on_exit, PRN *prn)
+{
+    int i;
+
+    if (!on_exit && n_models > 0) {
+	mark_session_changed();
+    }
+
     for (i=0; i<n_models; i++) {
 	/* reduce refcount on the model pointer */
 	if (table_models[i] != NULL) {
@@ -195,29 +227,43 @@ static int model_table_precheck (MODEL *pmod, int add_mode)
     return 0;
 }
 
-static int real_add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
+/* @pos will usually be 0, which means: add to end of model table array.
+   But when reconstituting a session, @pos may be a 1-based index of
+   the position within the array that this model should occupy.
+*/
+
+static int real_add_to_model_table (MODEL *pmod, int add_mode, int pos, PRN *prn)
 {
+    int i, n = (pos == 0)? n_models + 1 : pos;
+
     /* is the list started or not? */
     if (n_models == 0) {
-	table_models = mymalloc(sizeof *table_models);
+	table_models = mymalloc(n * sizeof *table_models);
 	if (table_models == NULL) {
 	    return 1;
 	}
-	n_models = 1;
-    } else {
+	for (i=0; i<n; i++) {
+	    table_models[i] = NULL;
+	}
+	n_models = n;
+    } else if (pos == 0 || pos > n_models) {
 	MODEL **mods;
 
-	n_models++;
-	mods = myrealloc(table_models, n_models * sizeof *mods);
+	mods = myrealloc(table_models, n * sizeof *mods);
 	if (mods == NULL) {
-	    clear_model_table(NULL);
+	    clear_model_table(0, NULL);
 	    return 1;
 	}
 
+	for (i=n_models; i<n; i++) {
+	    mods[i] = NULL;
+	}	
+
 	table_models = mods;
+	n_models = n;
     }
 
-    table_models[n_models - 1] = pmod;
+    table_models[n-1] = pmod;
 
     /* augment refcount so model won't get deleted */
     gretl_object_ref(pmod, GRETL_OBJ_EQN);
@@ -234,14 +280,14 @@ static int real_add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
     return 0;
 }
 
-int add_to_model_table (MODEL *pmod, int add_mode, PRN *prn)
+int add_to_model_table (MODEL *pmod, int add_mode, int pos, PRN *prn)
 {
     if (model_table_precheck(pmod, add_mode)) {
 	fprintf(stderr, "add_to_model_table: precheck failed\n");
 	return 1;
     }
 
-    return real_add_to_model_table(pmod, add_mode, prn);
+    return real_add_to_model_table(pmod, add_mode, pos, prn);
 }
 
 static int on_param_list (const char *pname)
@@ -397,19 +443,48 @@ static const char *get_pre_asts (double pval)
     return (pval >= 0.1)? "" : (pval >= 0.05)? "$\\,$" : "$\\,\\,$";
 }
 
-static void print_model_table_coeffs (int namewidth, PRN *prn)
+static void terminate_coeff_row (int namewidth, PRN *prn)
+{
+    if (tex_format(prn)) {
+	pputs(prn, "\\\\\n");
+    } else if (rtf_format(prn)) {
+	pputs(prn, "\\intbl \\row\n");
+	print_rtf_row_spec(prn, 1);
+	pputs(prn, "\\intbl ");
+    } else {
+	pputc(prn, '\n');
+	bufspace(namewidth + 2, prn);
+    }
+}
+
+static double modtab_get_pval (const MODEL *pmod, int k)
+{
+    double x = pmod->coeff[k];
+    double s = pmod->sderr[k];
+    double pval = NADBL;
+
+    if (!na(x) && !na(s)) {
+	pval = coeff_pval(pmod->ci, x / s, pmod->dfd);
+    }
+
+    return pval;
+}
+
+static void print_model_table_coeffs (int namewidth, int colwidth, PRN *prn)
 {
-    int i, j, k;
     const MODEL *pmod;
-    char tmp[32];
+    char numstr[32], tmp[32];
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
+    int i, j, k;
 
     /* loop across all variables that appear in any model */
 
     for (i=0; i<n_params; i++) {
 	char *pname = pnames[i];
-	int first = 1;
+	int first_coeff = 1;
+	int first_se = 1;
+	int first_pval = 1;
 
 	if (tex) {
 	    tex_escape(tmp, pname);
@@ -422,6 +497,7 @@ static void print_model_table_coeffs (int namewidth, PRN *prn)
 	}
 
 	/* print the coefficient estimates across a row */
+
 	for (j=0; j<n_models; j++) {
 	    pmod = table_models[j];
 	    if (pmod == NULL) {
@@ -429,39 +505,43 @@ static void print_model_table_coeffs (int namewidth, PRN *prn)
 	    }
 	    if ((k = gretl_model_get_param_number(pmod, datainfo, pname)) >= 0) {
 		double x = screen_zero(pmod->coeff[k]);
-		double s = screen_zero(pmod->sderr[k]);
-		double pval;
-		char numstr[32];
 
-		if (floateq(s, 0.0)) {
-		    if (floateq(x, 0.0)) {
-			pval = 1.0;
-		    } else {
-			pval = 0.0001;
-		    }
-		} else {
-		    pval = coeff_pval(pmod->ci, x / s, pmod->dfd);
-		}
-
-		sprintf(numstr, "%#.4g", x);
+		sprintf(numstr, "%#.*g", mt_figs, x);
 		gretl_fix_exponent(numstr);
 
-		if (tex) {
-		    if (x < 0) {
-			pprintf(prn, "& %s$-$%s%s ", get_pre_asts(pval),
-				numstr + 1, tex_get_asts(pval));
+		if (do_asts) {
+		    double pval = modtab_get_pval(pmod, k);
+
+		    if (tex) {
+			if (x < 0) {
+			    pprintf(prn, "& %s$-$%s%s ", get_pre_asts(pval),
+				    numstr + 1, tex_get_asts(pval));
+			} else {
+			    pprintf(prn, "& %s%s%s ", get_pre_asts(pval), 
+				    numstr, tex_get_asts(pval));
+			}
+		    } else if (rtf) {
+			pprintf(prn, "\\qc %s%s\\cell ", numstr, get_asts(pval));
 		    } else {
-			pprintf(prn, "& %s%s%s ", get_pre_asts(pval), 
-				numstr, tex_get_asts(pval));
+			/* note: strlen(asts) = 2 */
+			pprintf(prn, "%*s%s", (first_coeff)? colwidth : colwidth - 2,
+				numstr, get_asts(pval));
 		    }
-		} else if (rtf) {
-		    pprintf(prn, "\\qc %s%s\\cell ", numstr, get_asts(pval));
 		} else {
-		    /* note: strlen(asts) = 2 */
-		    pprintf(prn, "%*s%s", (first)? colwidth : colwidth - 2,
-			    numstr, get_asts(pval));
+		    /* not showing asterisks */
+		    if (tex) {
+			if (x < 0) {
+			    pprintf(prn, "& $-$%s ", numstr + 1);
+			} else {
+			    pprintf(prn, "& %s ", numstr);
+			}
+		    } else if (rtf) {
+			pprintf(prn, "\\qc %s\\cell ", numstr);
+		    } else {
+			pprintf(prn, "%*s", colwidth, numstr);
+		    }
 		}
-		first = 0;
+		first_coeff = 0;
 	    } else {
 		/* variable not present in this column */
 		if (tex) {
@@ -474,28 +554,16 @@ static void print_model_table_coeffs (int namewidth, PRN *prn)
 	    }
 	}
 
-	/* terminate the coefficient row and start the next one,
-	   which holds standard errors */
-	if (tex) {
-	    pputs(prn, "\\\\\n");
-	} else if (rtf) {
-	    pputs(prn, "\\intbl \\row\n");
-	    print_rtf_row_spec(prn, 1);
-	    pputs(prn, "\\intbl ");
-	} else {
-	    pputc(prn, '\n');
-	    bufspace(namewidth + 2, prn);
-	}
+	terminate_coeff_row(namewidth, prn);
 
 	/* print the t-stats or standard errors across a row */
-	first = 1;
+
 	for (j=0; j<n_models; j++) {
 	    pmod = table_models[j];
 	    if (pmod == NULL) {
 		continue;
 	    }
 	    if ((k = gretl_model_get_param_number(pmod, datainfo, pname)) >= 0) {
-		char numstr[32];
 		double val;
 
 		if (use_tstats) {
@@ -504,25 +572,25 @@ static void print_model_table_coeffs (int namewidth, PRN *prn)
 		    val = pmod->sderr[k];
 		}
 
-		sprintf(numstr, "%#.4g", val);
+		sprintf(numstr, "%#.*g", mt_figs, val);
 		gretl_fix_exponent(numstr);
 
 		if (tex) {
 		    if (val < 0) {
-			pprintf(prn, "& \\footnotesize{($-$%s)} ", numstr + 1);
+			pprintf(prn, "& \\subsize{($-$%s)} ", numstr + 1);
 		    } else {
-			pprintf(prn, "& \\footnotesize{(%s)} ", numstr);
+			pprintf(prn, "& \\subsize{(%s)} ", numstr);
 		    }
 		} else if (rtf) {
-		    if (first) {
+		    if (first_se) {
 			pputs(prn, "\\qc \\cell ");
 		    }
 		    pprintf(prn, "\\qc (%s)\\cell ", numstr);
-		    first = 0;
 		} else {
 		    sprintf(tmp, "(%s)", numstr);
 		    pprintf(prn, "%*s", colwidth, tmp);
 		}
+		first_se = 0;
 	    } else {
 		/* variable not present in this column */
 		if (tex) {
@@ -535,6 +603,47 @@ static void print_model_table_coeffs (int namewidth, PRN *prn)
 	    }
 	}
 
+	if (do_pvals) {
+	    terminate_coeff_row(namewidth, prn);
+	    for (j=0; j<n_models; j++) {
+		pmod = table_models[j];
+		if (pmod == NULL) {
+		    continue;
+		}
+		if ((k = gretl_model_get_param_number(pmod, datainfo, pname)) >= 0) {
+		    double pval = modtab_get_pval(pmod, k);
+
+		    if (na(pval)) {
+			strcpy(numstr, "NA");
+		    } else {
+			sprintf(numstr, "%.*f", mt_figs, pval);
+		    }
+
+		    if (tex) {
+			pprintf(prn, "& \\subsize{[%s]} ", numstr);
+		    } else if (rtf) {
+			if (first_pval) {
+			    pputs(prn, "\\qc \\cell ");
+			}
+			pprintf(prn, "\\qc [%s]\\cell ", numstr);
+		    } else {
+			sprintf(tmp, "[%s]", numstr);
+			pprintf(prn, "%*s", colwidth, tmp);
+		    }
+		    first_pval = 0;
+		} else {
+		    /* variable not present in this column */
+		    if (tex) {
+			pputs(prn, "& ");
+		    } else if (rtf) {
+			pputs(prn, "\\qc \\cell ");
+		    } else {
+			bufspace(colwidth, prn);
+		    }
+		}
+	    }
+	}
+
 	if (tex) {
 	    pputs(prn, "\\\\ [4pt] \n");
 	} else if (rtf) {
@@ -570,7 +679,15 @@ static int any_stat (int s)
     return 0;
 }
 
-static void print_equation_stats (int width0, PRN *prn, int *binary)
+static int catch_bad_point (char *s)
+{
+    int c = s[strlen(s) - 1];
+
+    return (c == '.' || c == ',');
+}
+
+static void print_equation_stats (int width0, int colwidth, PRN *prn, 
+				  int *binary)
 {
     const MODEL *pmod;
     int same_df, any_R2, any_ll;
@@ -649,19 +766,19 @@ static void print_equation_stats (int width0, PRN *prn, int *binary)
 		*binary = 1;
 		/* McFadden */
 		if (tex) {
-		    pprintf(prn, "& %.4f ", pmod->rsq);
+		    pprintf(prn, "& %.*f ", mt_figs, pmod->rsq);
 		} else if (rtf) {
-		    pprintf(prn, "\\qc %.4f\\cell ", pmod->rsq);
+		    pprintf(prn, "\\qc %.*f\\cell ", mt_figs, pmod->rsq);
 		} else {
-		    pprintf(prn, "%#*.4g", colwidth, pmod->rsq);
+		    pprintf(prn, "%*.*f", colwidth, mt_figs, pmod->rsq);
 		}
 	    } else {
 		if (tex) {
-		    pprintf(prn, "& %.4f ", rsq);
+		    pprintf(prn, "& %.*f ", mt_figs, rsq);
 		} else if (rtf) {
-		    pprintf(prn, "\\qc %.4f\\cell ", rsq);
+		    pprintf(prn, "\\qc %.*f\\cell ", mt_figs, rsq);
 		} else {
-		    pprintf(prn, "%#*.4g", colwidth, rsq);
+		    pprintf(prn, "%*.*f", colwidth, mt_figs, rsq);
 		}
 	    }
 	}
@@ -680,6 +797,8 @@ static void print_equation_stats (int width0, PRN *prn, int *binary)
 
     if (any_ll) {
 	/* print log-likelihoods */
+	char numstr[16];
+
 	if (tex) {
 	    pputs(prn, "$\\ell$");
 	} else if (rtf) {
@@ -702,14 +821,25 @@ static void print_equation_stats (int width0, PRN *prn, int *binary)
 	    } else {
 		if (tex) {
 		    if (pmod->lnL > 0) {
-			pprintf(prn, "& %.2f ", pmod->lnL);
+			sprintf(numstr, "%.*g", mt_figs, pmod->lnL);
+			catch_bad_point(numstr);
+			pprintf(prn, "& %s ", numstr);
 		    } else {
-			pprintf(prn, "& $-$%.2f ", -pmod->lnL);
+			sprintf(numstr, "%.*g", mt_figs, -pmod->lnL);
+			catch_bad_point(numstr);
+			pprintf(prn, "& $-$%s ", numstr);
 		    }
 		} else if (rtf) {
-		    pprintf(prn, "\\qc %.3f\\cell ", pmod->lnL);
+		    sprintf(numstr, "%.*g", mt_figs, pmod->lnL);
+		    catch_bad_point(numstr);
+		    pprintf(prn, "\\qc %s\\cell ", numstr);
 		} else {
-		    pprintf(prn, "%#*.6g", colwidth, pmod->lnL);
+		    sprintf(numstr, "%#*.*g", colwidth, mt_figs, pmod->lnL);
+		    if (catch_bad_point(numstr)) {
+			sprintf(numstr, "%#*.*g", colwidth + 1, mt_figs, pmod->lnL);
+			numstr[strlen(numstr) - 1] = '\0';
+		    }
+		    pputs(prn, numstr);
 		}
 	    }
 	}
@@ -738,32 +868,134 @@ static int max_namelen (void)
     return maxlen;
 }
 
-int display_model_table (int gui)
+static int get_colwidth (void)
 {
-    int j, ci;
+    int maxlen = 0;
+    int cw = 13;
+
+    if (colheads == COLHEAD_NAMES) {
+	const MODEL *pmod;
+	int i, len;
+
+	for (i=0; i<n_models; i++) {
+	    pmod = table_models[i];
+	    if (pmod != NULL) {
+		if (pmod->name != NULL) {
+		    len = strlen(pmod->name);
+		} else {
+		    char tmp[32];
+
+		    sprintf(tmp, _("Model %d"), pmod->ID);
+		    len = strlen(tmp);
+		}
+		if (len > maxlen) {
+		    maxlen = (len > 31)? 31 : len;
+		}
+	    }
+	}
+    }
+
+    if (maxlen < 11 && mt_figs < 4) {
+	cw -= 4 - mt_figs;
+    }
+
+    return (cw < maxlen + 2)? maxlen + 2 : cw;
+}
+
+static void print_estimator_strings (int colwidth, PRN *prn)
+{
+    const char *s;
+    char est[32];
+    int i;
+
+    pputc(prn, '\n');
+
+    for (i=0; i<n_models; i++) {
+	if (table_models[i] != NULL) {
+	    s = short_estimator_string(table_models[i], prn);
+	    if (tex_format(prn)) {
+		strcpy(est, I_(s));
+		pprintf(prn, " & %s ", est);
+	    } else if (rtf_format(prn)) {
+		strcpy(est, I_(s));
+		pprintf(prn, "\\qc %s\\cell ", est);
+	    } else {
+		strcpy(est, _(s));
+		print_centered(est, colwidth, prn);
+	    }		
+	}
+    }
+}
+
+static void print_model_head (const MODEL *pmod, int j, int colwidth,
+			      PRN *prn)
+{
+    char targ[48];
+
+    if (colheads == COLHEAD_ARABIC) {
+	sprintf(targ, "(%d)", j + 1);
+    } else if (colheads == COLHEAD_ROMAN) {
+	const char *R[] = {
+	    "I", "II", "III", "IV", "V", "VI", 
+	    "VII", "VIII", "IX", "X", "XI", "XII"
+	};
+
+	sprintf(targ, "%s", R[j]);
+    } else if (colheads == COLHEAD_ALPHA) {
+	sprintf(targ, "%c", 'A' + j);
+    } else if (tex_format(prn)) {
+	if (pmod->name != NULL) {
+	    char tmp[32];
+
+	    *targ = '\0';
+	    strncat(targ, pmod->name, 16);
+	    tex_escape(tmp, targ);
+	    strcpy(targ, tmp);
+	} else {
+	    sprintf(targ, I_("Model %d"), pmod->ID);
+	}
+    } else if (rtf_format(prn)) {
+	if (pmod->name != NULL) {
+	    *targ = '\0';
+	    strncat(targ, pmod->name, 31);
+	} else {
+	    sprintf(targ, I_("Model %d"), pmod->ID);
+	}	
+    } else {
+	if (pmod->name != NULL) {
+	    *targ = '\0';
+	    strncat(targ, pmod->name, 31);
+	} else {
+	    sprintf(targ, _("Model %d"), pmod->ID);
+	}
+    }
+
+    if (tex_format(prn)) {
+	pprintf(prn, " & %s ", targ);
+    } else if (rtf_format(prn)) {
+	pprintf(prn, "\\qc %s\\cell ", targ);
+    } else {
+	print_centered(targ, colwidth, prn);
+    }
+}
+
+static void print_column_heads (int colwidth, PRN *prn)
+{
+    int i, j = 0;
+
+    for (i=0; i<n_models; i++) {
+	if (table_models[i] != NULL) {
+	    print_model_head(table_models[i], j++, colwidth, prn);
+	}
+    }
+}
+
+static void plain_print_model_table (PRN *prn)
+{
+    int namelen = max_namelen();
+    int colwidth = get_colwidth();
+    int ci = common_estimator();
     int binary = 0;
-    int winwidth = 78;
-    int namelen;
-    PRN *prn;
-
-    if (model_table_is_empty()) {
-	mtable_errmsg(_("The model table is empty"), gui);
-	return 1;
-    }
-
-    if (make_full_param_list()) {
-	return 1;
-    }
-
-    namelen = max_namelen();
-    colwidth = 13;
-
-    if (bufopen(&prn)) {
-	clear_model_table(NULL);
-	return 1;
-    }
-
-    ci = common_estimator();
 
     if (ci > 0) {
 	/* all models use same estimation procedure */
@@ -776,41 +1008,19 @@ int display_model_table (int gui)
 
     pputc(prn, '\n');
     bufspace(namelen + 4, prn);
-
-    for (j=0; j<n_models; j++) {
-	char modhd[32];
-
-	if (table_models[j] == NULL) {
-	    continue;
-	}
-	if (table_models[j]->name != NULL) {
-	    *modhd = '\0';
-	    strncat(modhd, table_models[j]->name, 31);
-	} else {
-	    sprintf(modhd, _("Model %d"), table_models[j]->ID);
-	}
-	print_centered(modhd, colwidth, prn);
-    }
+    print_column_heads(colwidth, prn);
     pputc(prn, '\n');
     
     if (ci == 0) {
-	char est[32];	
-
 	bufspace(namelen + 4, prn);
-	for (j=0; j<n_models; j++) {
-	    if (table_models[j] != NULL) {
-		strcpy(est, 
-		       _(short_estimator_string(table_models[j], prn)));
-		print_centered(est, colwidth, prn);
-	    }
-	}
+	print_estimator_strings(colwidth, prn);
 	pputc(prn, '\n');
     }
 
     pputc(prn, '\n'); 
 
-    print_model_table_coeffs(namelen, prn);
-    print_equation_stats(namelen + 1, prn, &binary);
+    print_model_table_coeffs(namelen, colwidth, prn);
+    print_equation_stats(namelen + 1, colwidth, prn, &binary);
 
     if (use_tstats) {
 	pprintf(prn, "%s\n", _("t-statistics in parentheses"));
@@ -818,13 +1028,41 @@ int display_model_table (int gui)
 	pprintf(prn, "%s\n", _("Standard errors in parentheses"));
     }
 
-    pprintf(prn, "%s\n", _("* indicates significance at the 10 percent level"));
-    pprintf(prn, "%s\n", _("** indicates significance at the 5 percent level"));
+    if (do_pvals) {
+	pprintf(prn, "%s\n", _("p-values in brackets"));
+    }
+
+    if (do_asts) {
+	pprintf(prn, "%s\n", _("* indicates significance at the 10 percent level"));
+	pprintf(prn, "%s\n", _("** indicates significance at the 5 percent level"));
+    }
    
     if (binary) {
 	pprintf(prn, "%s\n", _("For logit and probit, R-squared is "
 			       "McFadden's pseudo-R-squared"));
     }
+}
+
+int display_model_table (int gui)
+{
+    int winwidth = 78;
+    PRN *prn;
+
+    if (model_table_is_empty()) {
+	mtable_errmsg(_("The model table is empty"), gui);
+	return 1;
+    }
+
+    if (make_full_param_list()) {
+	return 1;
+    }
+
+    if (bufopen(&prn)) {
+	clear_model_table(0, NULL);
+	return 1;
+    }
+
+    plain_print_model_table(prn);
 
     if (real_table_n_models() > 5) {
 	winwidth = 90;
@@ -838,9 +1076,9 @@ int display_model_table (int gui)
 
 static int tex_print_model_table (PRN *prn)
 {
-    int j, ci;
     int binary = 0;
     char tmp[32];
+    int i, ci;
 
     if (model_table_is_empty()) {
 	mtable_errmsg(_("The model table is empty"), 1);
@@ -851,10 +1089,11 @@ static int tex_print_model_table (PRN *prn)
 	return 1;
     }
 
-    ci = common_estimator();
-
+    pputs(prn, "\n\\newcommand{\\subsize}[1]{\\footnotesize{#1}}\n\n");
     pputs(prn, "\\begin{center}\n");
 
+    ci = common_estimator();
+
     if (ci > 0) {
 	/* all models use same estimation procedure */
 	pprintf(prn, I_("%s estimates"), 
@@ -867,49 +1106,26 @@ static int tex_print_model_table (PRN *prn)
 
     pputs(prn, "\\vspace{1em}\n\n");
     pputs(prn, "\\begin{longtable}{l");
-    for (j=0; j<n_models; j++) {
-	pputs(prn, "c");
+    for (i=0; i<n_models; i++) {
+	if (table_models[i] != NULL) {
+	    pputc(prn, 'c');
+	}
     }
     pputs(prn, "}\n");
 
-    for (j=0; j<n_models; j++) {
-	char modhd[48];
-
-	if (table_models[j] == NULL) {
-	    continue;
-	}
-	if (table_models[j]->name != NULL) {
-	    *modhd = '\0';
-	    strncat(modhd, table_models[j]->name, 16);
-	    tex_escape(tmp, modhd);
-	    strcpy(modhd, tmp);
-	} else {
-	    sprintf(modhd, I_("Model %d"), table_models[j]->ID);
-	}
-	pprintf(prn, " & %s ", modhd);
-    }
+    print_column_heads(0, prn);
     pputs(prn, "\\\\ ");
     
     if (ci == 0) {
-	char est[32];
-
 	pputc(prn, '\n');
-
-	for (j=0; j<n_models; j++) {
-	    if (table_models[j] == NULL) {
-		continue;
-	    }
-	    strcpy(est, 
-		   I_(short_estimator_string(table_models[j], prn)));
-	    pprintf(prn, " & %s ", est);
-	}
+	print_estimator_strings(0, prn);
 	pputs(prn, "\\\\ ");
     }
 
     pputs(prn, " [6pt] \n");   
 
-    print_model_table_coeffs(0, prn);
-    print_equation_stats(0, prn, &binary);
+    print_model_table_coeffs(0, 0, prn);
+    print_equation_stats(0, 0, prn, &binary);
 
     pputs(prn, "\\end{longtable}\n\n");
     pputs(prn, "\\vspace{1em}\n");
@@ -920,10 +1136,16 @@ static int tex_print_model_table (PRN *prn)
 	pprintf(prn, "%s\\\\\n", I_("Standard errors in parentheses"));
     }
 
-    pprintf(prn, "{}%s\\\\\n", 
-	    I_("* indicates significance at the 10 percent level"));
-    pprintf(prn, "{}%s\\\\\n", 
-	    I_("** indicates significance at the 5 percent level"));
+    if (do_pvals) {
+	pprintf(prn, "%s\\\\\n", I_("$p$-values in brackets"));
+    }
+
+    if (do_asts) {
+	pprintf(prn, "{}%s\\\\\n", 
+		I_("* indicates significance at the 10 percent level"));
+	pprintf(prn, "{}%s\\\\\n", 
+		I_("** indicates significance at the 5 percent level"));
+    }
 
     if (binary) {
 	pprintf(prn, "%s\\\\\n", I_("For logit and probit, $R^2$ is "
@@ -950,8 +1172,7 @@ static void print_rtf_row_spec (PRN *prn, int tall)
 
 static int rtf_print_model_table (PRN *prn)
 {
-    int j, ci;
-    int binary = 0;
+    int ci, binary = 0;
 
     if (model_table_is_empty()) {
 	mtable_errmsg(_("The model table is empty"), 1);
@@ -962,8 +1183,6 @@ static int rtf_print_model_table (PRN *prn)
 	return 1;
     }
 
-    gretl_print_set_format(prn, GRETL_FORMAT_RTF);
-
     ci = common_estimator();
 
     pputs(prn, "{\\rtf1\n");
@@ -979,42 +1198,19 @@ static int rtf_print_model_table (PRN *prn)
     pprintf(prn, "\\par \\qc %s: %s\n\\par\n\\par\n{", 
 	    I_("Dependent variable"), datainfo->varname[depvarnum]);
 
-    /* RTF row stuff */
     print_rtf_row_spec(prn, 1);
-
     pputs(prn, "\\intbl \\qc \\cell ");
-    for (j=0; j<n_models; j++) {
-	char modhd[32];
-
-	if (table_models[j] == NULL) {
-	    continue;
-	}
-	if (table_models[j]->name != NULL) {
-	    *modhd = '\0';
-	    strncat(modhd, table_models[j]->name, 31);
-	} else {
-	    sprintf(modhd, I_("Model %d"), table_models[j]->ID);
-	}
-	pprintf(prn, "\\qc %s\\cell ", modhd);
-    }
+    print_column_heads(0, prn);
     pputs(prn, "\\intbl \\row\n");
     
     if (ci == 0) {
-	char est[32];
-
 	pputs(prn, "\\intbl \\qc \\cell ");
-
-	for (j=0; j<n_models; j++) {
-	    if (table_models[j] == NULL) continue;
-	    strcpy(est, 
-		   I_(short_estimator_string(table_models[j], prn)));
-	    pprintf(prn, "\\qc %s\\cell ", est);
-	}
+	print_estimator_strings(0, prn);
 	pputs(prn, "\\intbl \\row\n");
     }
 
-    print_model_table_coeffs(0, prn);
-    print_equation_stats(0, prn, &binary);
+    print_model_table_coeffs(0, 0, prn);
+    print_equation_stats(0, 0, prn, &binary);
 
     pputs(prn, "}\n\n");
 
@@ -1024,10 +1220,16 @@ static int rtf_print_model_table (PRN *prn)
 	pprintf(prn, "\\par \\qc %s\n", I_("Standard errors in parentheses"));
     }
 
-    pprintf(prn, "\\par \\qc %s\n", 
-	    I_("* indicates significance at the 10 percent level"));
-    pprintf(prn, "\\par \\qc %s\n", 
-	    I_("** indicates significance at the 5 percent level"));
+    if (do_pvals) {
+	pprintf(prn, "\\par \\qc %s\n", I_("p-values in brackets"));
+    }
+
+    if (do_asts) {
+	pprintf(prn, "\\par \\qc %s\n", 
+		I_("* indicates significance at the 10 percent level"));
+	pprintf(prn, "\\par \\qc %s\n", 
+		I_("** indicates significance at the 5 percent level"));
+    }
 
     if (binary) {
 	pprintf(prn, "\\par \\qc %s\n", I_("For logit and probit, "
@@ -1081,7 +1283,7 @@ static int cli_modeltab_add (PRN *prn)
 	}
 
 	if (!err) {
-	    err = real_add_to_model_table(cpy, MODEL_ADD_BY_CMD, prn);
+	    err = real_add_to_model_table(cpy, MODEL_ADD_BY_CMD, 0, prn);
 	}
 
 	if (err && freeit) {
@@ -1106,29 +1308,55 @@ int modeltab_parse_line (const char *line, PRN *prn)
     } else if (!strcmp(cmdword, "show")) {
 	err = display_model_table(0);
     } else if (!strcmp(cmdword, "free")) {
-	if (model_table_is_empty()) {
-	    mtable_errmsg(_("The model table is empty"), 0);
-	    err = 1;
-	} else {
-	    clear_model_table(prn);
+	if (!model_table_is_empty()) {
+	    clear_model_table(0, prn);
 	}
     }
 
     return err;
 }
 
-void model_table_dialog (void)
+void format_model_table (windata_t *vwin)
 {
-    const char *opts[] = {
-	N_("standard errors in parentheses"),
-	N_("t-statistics in parentheses")
-    };
-    int opt;
+    int colhead_opt = colheads;
+    int se_opt = use_tstats;
+    int pv_opt = do_pvals;
+    int ast_opt = do_asts;
+    int figs = mt_figs;
+    int resp;
 
-    opt = radio_dialog(_("model table options"), NULL, opts, 2, use_tstats, 0);
+    resp = model_table_dialog(&colhead_opt, &se_opt, &pv_opt, &ast_opt,
+			      &figs);
 
-    if (opt >= 0) {
-	use_tstats = opt;
+    if (resp == GRETL_CANCEL) {
+	return;
+    }
+
+    if (colhead_opt == colheads && se_opt == use_tstats && 
+	pv_opt == do_pvals && ast_opt == do_asts && figs == mt_figs) {
+	/* no-op */
+	return;
+    } else {
+	GtkTextBuffer *buf;
+	const char *newtext;
+	PRN *prn;
+
+	colheads = colhead_opt;
+	use_tstats = se_opt;
+	do_pvals = pv_opt;
+	do_asts = ast_opt;
+	mt_figs = figs;
+
+	if (bufopen(&prn)) {
+	    return;
+	}
+
+	plain_print_model_table(prn);
+	newtext = gretl_print_get_buffer(prn);
+	buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
+	gtk_text_buffer_set_text(buf, "", -1);
+	textview_set_text(vwin->text, newtext);
+	gretl_print_destroy(prn); 
     }
 }
 
