@@ -45,11 +45,6 @@ static int gui_parse_object_request (const char *line,
 	return OBJ_ACTION_NONE;
     }
 
-    /* the model table is special, not handled here */
-    if (!strcmp(word, "modeltab")) {
-	return OBJ_ACTION_NONE;
-    }
-
     /* see if there's an object associated with the name */
     *pptr = get_session_object_by_name(word, type);
 
@@ -77,13 +72,7 @@ static int
 finalize_model_save (void *ptr, GretlObjType type, const char *name, 
 		     PRN *prn)
 {
-    int err;
-
-    err = gretl_stack_object_as(ptr, type, name);
-
-    if (!err) {
-	err = maybe_add_model_to_session(ptr, type);
-    }
+    int err = maybe_add_model_to_session(ptr, type, name);
 
     if (!err) {
 	pprintf(prn, _("%s saved\n"), name);
