@@ -30,10 +30,11 @@ struct iterinfo_t {
 
 void clear_selector (void);
 
-void selection_dialog (const char *title, int (*callback)(), guint cmdcode);
+selector *selection_dialog (const char *title, int (*callback)(), 
+			    guint cmdcode);
 
-void simple_selection (const char *title, int (*callback)(), guint cmdcode,
-		       gpointer p);
+selector *simple_selection (const char *title, int (*callback)(), 
+			    guint cmdcode, gpointer p);
 
 void modelspec_dialog (int ci);
 
@@ -43,7 +44,11 @@ void selector_from_model (void *ptr, int ci);
 
 char *main_window_selection_as_string (void);
 
-void data_save_selection_wrapper (int file_code, gpointer p);
+void data_save_selection_wrapper (int file_code);
+
+void add_remove_functions_dialog (char **pubnames, int npub,
+				  char **privnames, int npriv,
+				  void *p1, void *p2);
 
 int selector_code (const selector *sr);
 
@@ -65,4 +70,8 @@ int selector_error (const selector *sr);
 
 void maybe_clear_selector (const int *dlist);
 
+GtkWidget *selector_get_window (const selector *sr);
+
+void selector_cleanup (void);
+
 #endif /* SELECTOR_H */
