@@ -36,6 +36,7 @@
 #include "menustate.h"
 #include "toolbar.h"
 #include "winstack.h"
+#include "fncall.h"
 #include "lib_private.h"
 
 #include "var.h"
@@ -83,8 +84,7 @@ typedef struct gui_obj_ gui_obj;
 
 enum {
     SESSION_CHANGED  = 1 << 0,
-    SESSION_SAVED    = 1 << 1,
-    SESSION_EMBEDDED = 1 << 2
+    SESSION_SAVED    = 1 << 1
 };
 
 struct SESSION_ {
@@ -132,6 +132,7 @@ struct sample_info {
     int t1;
     int t2;
     char *mask;
+    char *restriction;
     unsigned int seed;
     int resample_n;
 };
@@ -157,7 +158,6 @@ static char *model_items[] = {
 static char *model_table_items[] = {
     N_("Display"),
     N_("Clear"),
-    N_("Options"),
     N_("Help")
 };
 
@@ -194,8 +194,7 @@ static char *scalars_items[] = {
 };
 
 static char *info_items[] = {
-    N_("View"),
-    N_("Edit"),
+    N_("View")
 };
 
 static char *matrix_items[] = {
@@ -254,6 +253,7 @@ static void add_user_matrix_callback (void);
 
 static int session_graph_count;
 static int session_bplot_count;
+static int commands_recorded;
 
 int session_is_modified (void)
 {
@@ -265,7 +265,7 @@ void mark_session_changed (void)
     session.status = SESSION_CHANGED;
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, TRUE);
-	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", TRUE);
+	flip(mdata->ui, "/menubar/File/SessionFiles/SaveSession", TRUE);
 	if (*session.name != '\0') {
 	    set_main_window_title(session.name, TRUE);
 	}
@@ -277,11 +277,21 @@ static void mark_session_saved (void)
     session.status = SESSION_SAVED;
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, FALSE);
-	flip(mdata->ui, "/MenuBar/File/SessionFiles/SaveSession", FALSE);
+	flip(mdata->ui, "/menubar/File/SessionFiles/SaveSession", FALSE);
 	set_main_window_title(session.name, FALSE);
     }
 }
 
+void set_commands_recorded (void)
+{
+    commands_recorded = 1;
+}
+
+int get_commands_recorded (void)
+{
+    return commands_recorded;
+}
+
 /* constructors and destructors for session data-objects */
 
 static void free_session_text (SESSION_TEXT *text)
@@ -424,16 +434,47 @@ session_append_graph (const char *grname,
     return graph;
 }
 
+const char *last_session_graph_name (void)
+{
+    int ng = session.ngraphs;
+
+    if (ng > 0) {
+	SESSION_GRAPH *graph = session.graphs[ng - 1];
+
+	return graph->fname;
+    } else {
+	return NULL;
+    }
+}
+
 static void session_switch_log_location (int code)
 {
     gchar *fullpath;
 
-    fullpath = g_strdup_printf("%s%s%c", paths.dotdir, 
+    fullpath = g_strdup_printf("%s%s%c", gretl_dotdir(), 
 			       session.dirname, SLASH);
     set_session_log(fullpath, code);
     g_free(fullpath);
 }
 
+char *session_graph_make_path (char *path, const char *fname)
+{
+    if (strstr(fname, session.dirname) != NULL) {
+	/* should be OK */
+	strcpy(path, fname);
+    } else {
+	char *p = strrchr(fname, SLASH);
+
+	if (p != NULL) {
+	    sprintf(path, "%s%s", session.dirname, p);
+	} else {
+	    sprintf(path, "%s%c%s", session.dirname, SLASH, fname);
+	}
+    }
+
+    return path;
+}
+
 /* first arg should be a MAXLEN string */
 
 static char *session_file_make_path (char *path, const char *fname)
@@ -472,8 +513,7 @@ static void edit_session_notes (void)
 			 G_CALLBACK(gtk_widget_destroyed),
 			 &notes_window);
     } else {
-	gdk_window_show(notes_window->window);
-	gdk_window_raise(notes_window->window);
+	gtk_window_present(GTK_WINDOW(notes_window));
     }
 }
 
@@ -729,7 +769,7 @@ static int session_dir_ok (void)
 	errno = 0;
 
 	sprintf(session.dirname, ".gretl-%d", (int) p);
-	if (gretl_chdir(paths.dotdir)) {
+	if (gretl_chdir(gretl_dotdir())) {
 	    perror("moving to user directory");
 	    ret = 0;
 	} else if (gretl_mkdir(session.dirname)) {
@@ -764,7 +804,7 @@ int add_graph_to_session (char *fname, char *fullname, int type)
 	return 1;
     }
 
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(gretl_dotdir());
 
     if (type == GRETL_OBJ_PLOT) {
 	sprintf(shortname, "plot.%d", session_bplot_count + 1);
@@ -805,7 +845,7 @@ int cli_add_graph_to_session (const char *fname, const char *gname,
 	return ADD_OBJECT_FAIL;
     }
 
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(gretl_dotdir());
 
     strcpy(name, gname);
     space_to_score(name);
@@ -815,6 +855,7 @@ int cli_add_graph_to_session (const char *fname, const char *gname,
 	errbox(_("Failed to copy graph file"));
 	return ADD_OBJECT_FAIL;
     } 
+
     gretl_remove(fname);
 
     return real_add_graph_to_session(name, gname, type);
@@ -861,22 +902,36 @@ static SESSION_MODEL *get_session_model_by_data (const void *ptr)
     return NULL;
 }
 
-int maybe_add_model_to_session (void *ptr, GretlObjType type)
+static void delete_icon_for_data (void *data)
+{
+    gui_obj *gobj = get_gui_obj_by_data(data);
+
+    if (gobj != NULL) {
+	/* icon is shown: delete it */
+	session_delete_icon(gobj);
+    }
+}
+
+int maybe_add_model_to_session (void *ptr, GretlObjType type,
+				const char *name)
 {
     SESSION_MODEL *oldmod;
-    const char *name;
 
     if (get_session_model_by_data(ptr)) {
 	/* model already present */
-	return 1;
+	return 0;
     }
 
     /* check to see if there's already a (different) model with the
        same name: if so, delete it
     */
-    name = gretl_object_get_name(ptr, type);
-    oldmod = get_session_model_by_name(name);
+    if (name == NULL) {
+	name = gretl_object_get_name(ptr, type);
+    }
+    oldmod = get_session_model_by_name(name); 
+
     if (oldmod != NULL) {
+	delete_icon_for_data(oldmod);
 	real_delete_model_from_session(oldmod);
     }
 
@@ -909,6 +964,7 @@ void model_add_as_icon (GtkAction *action, gpointer p)
     void *ptr = vwin->data;
     const char *name;
     int type;
+    int err = 0;
 
     if (ptr == NULL) {
 	return;
@@ -926,14 +982,26 @@ void model_add_as_icon (GtkAction *action, gpointer p)
     type = model_type_from_vwin(vwin);
     name = gretl_object_get_name(ptr, type);
 
-    if (real_add_model_to_session(ptr, name, type)) {
-	return;
+    if (name != NULL && *name == '\0') {
+	/* since we didn't get a match by data above, we
+	   need to ensure that this model is given a
+	   unique name */
+	gretl_object_compose_unique_name(ptr, type);
+	name = gretl_object_get_name(ptr, type);
     }
 
-    mark_session_changed();
+    if (name == NULL) {
+	nomem();
+	return;
+    }    
 
-    if (close_on_add(action) && !window_is_busy(vwin)) {
-	gtk_widget_destroy(vwin->main);
+    err = real_add_model_to_session(ptr, name, type);
+
+    if (!err) {
+	mark_session_changed();
+	if (close_on_add(action) && !window_is_busy(vwin)) {
+	    gtk_widget_destroy(vwin->main);
+	}
     } 	
 }
 
@@ -981,7 +1049,7 @@ static int unzip_session_file (const char *fname, char **zdirname)
 	return 1;
     }
     
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(gretl_dotdir());
 
     gretl_unzip_file = gui_get_plugin_function("gretl_unzip_file", 
 					       &handle);
@@ -1052,10 +1120,21 @@ static void sinfo_init (struct sample_info *sinfo)
     sinfo->t1 = 0;
     sinfo->t2 = 0;
     sinfo->mask = NULL;
+    sinfo->restriction = NULL;
     sinfo->seed = 0;
     sinfo->resample_n = 0;
 }
 
+static void sinfo_free_data (struct sample_info *sinfo)
+{
+    if (sinfo->mask != NULL) {
+	free(sinfo->mask);
+    }
+    if (sinfo->restriction != NULL) {
+	free(sinfo->restriction);
+    }
+}
+
 static int set_session_dirname (const char *zdirname)
 {
     char test[MAXLEN];
@@ -1106,7 +1185,7 @@ void do_open_session (void)
 
     fprintf(stderr, I_("\nReading session file %s\n"), tryfile);
 
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(gretl_dotdir());
     err = unzip_session_file(tryfile, &zdirname);
 
     if (err) {
@@ -1124,6 +1203,8 @@ void do_open_session (void)
 	fprintf(stderr, "Failed on set_session_dirname\n");
 	file_read_errbox("session.xml");
 	goto bailout;
+    } else {
+	fprintf(stderr, "set_session_dirname: '%s', OK\n", zdirname);
     }
 
     g_free(zdirname);
@@ -1139,21 +1220,22 @@ void do_open_session (void)
     }
 
     /* construct path to session data file */
-    session_file_make_path(paths.datfile, sinfo.datafile);
-    fp = gretl_fopen(paths.datfile, "r");
+    session_file_make_path(datafile, sinfo.datafile);
+    fp = gretl_fopen(datafile, "r");
 
     if (fp != NULL) {
 	/* OK, write good name into gdtname */
-	strcpy(gdtname, paths.datfile);
+	fprintf(stderr, "got datafile name '%s'\n", datafile);
+	strcpy(gdtname, datafile);
 	fclose(fp);
     } else {
 	/* try remedial action, transform filename? */
-	fprintf(stderr, "'%s' : not found, trying to fix\n", paths.datfile);
+	fprintf(stderr, "'%s' : not found, trying to fix\n", datafile);
 	err = get_session_dataname(gdtname, &sinfo);
     }
 
     if (!err) {
-	err = gretl_read_gdt(gdtname, NULL, &Z, datainfo, OPT_P, NULL);
+	err = gretl_read_gdt(gdtname, &Z, datainfo, OPT_B, NULL);
     }
 
     if (err) {
@@ -1186,11 +1268,19 @@ void do_open_session (void)
     session_file_make_path(fname, "lists.xml");
     err = maybe_read_lists_file(fname);
 
+    session_file_make_path(fname, "settings.inp");
+    err = maybe_read_settings_file(fname);
+
     if (sinfo.resample_n > 0) {
 	err = dataset_resample(sinfo.resample_n, sinfo.seed, &Z, 
 			       datainfo);
     } else if (sinfo.mask != NULL) {
-	err = restrict_sample_from_mask(sinfo.mask, &Z, datainfo);
+	err = restrict_sample_from_mask(sinfo.mask, &Z, datainfo,
+					OPT_NONE);
+	if (!err) {
+	    datainfo->restriction = sinfo.restriction;
+	    sinfo.restriction = NULL;
+	}
     }
 
     if (err) {
@@ -1205,9 +1295,10 @@ void do_open_session (void)
 
     if (sinfo.mask != NULL) {
 	set_sample_label(datainfo);
-	free(sinfo.mask);
     }
 
+    sinfo_free_data(&sinfo);
+
     set_main_window_title(session.name, FALSE);
 
  bailout:
@@ -1227,24 +1318,6 @@ void do_open_session (void)
     }
 }
 
-int is_session_file (const char *fname)
-{
-    int (*gretl_is_zipfile)(const char *);
-    void *handle;
-    int ret;
-
-    gretl_is_zipfile = gui_get_plugin_function("gretl_is_zipfile", &handle);
-    if (gretl_is_zipfile == NULL) {
-        return 0;
-    }   
-
-    ret = (*gretl_is_zipfile)(fname);
-
-    close_plugin(handle);
-
-    return ret;
-}
-
 void verify_clear_data (void)
 {
     if (dataset_locked()) {
@@ -1263,15 +1336,17 @@ void verify_clear_data (void)
 
 static void remove_session_dir (void)
 {
+    const char *dotdir = gretl_dotdir();
+
 #ifdef G_OS_WIN32
-    char *fullpath = g_strdup_printf("%s%s", paths.dotdir,
+    char *fullpath = g_strdup_printf("%s%s", dotdir,
 				     session.dirname);
 
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(dotdir);
     win32_delete_dir(fullpath);
     g_free(fullpath);
 #else
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(dotdir);
     gretl_deltree(session.dirname);
 #endif
 }
@@ -1388,7 +1463,7 @@ int session_file_is_open (void)
 
 void gui_clear_dataset (void)
 {
-    *paths.datfile = 0;
+    *datafile = 0;
 
     if (Z != NULL) {
 	free_Z(Z, datainfo);
@@ -1436,8 +1511,8 @@ void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
 
     free_session();
 
-    clear_model_table(NULL);
-    clear_graph_page();
+    clear_model_table(1, NULL);
+    clear_graph_page(1);
 
     session_menu_state(FALSE);
     *scriptfile = '\0';
@@ -1448,9 +1523,12 @@ void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
     }
 
     session.status = 0;
+    commands_recorded = 0;
 
     winstack_destroy();
-    clear_selector();
+    close_plot_windows();
+    selector_cleanup();
+    function_call_cleanup();
     edit_dialog_special_get_text(NULL);
 
     if (opt & OPT_P) {
@@ -1505,40 +1583,58 @@ static void relpath_from_fname (char *path, const char *fname)
     }
 }
 
-static int save_session_dataset (const char *dname)
+/* dump the current dataset into the session dir */
+
+static int real_save_session_dataset (const char *dname)
 {
     char tmpname[MAXLEN];
-    const double **dZ = NULL;
-    DATAINFO *dinfo = NULL;
-    int t1, t2;
+    char *mask = NULL;
+    int save_t1 = datainfo->t1;
+    int save_t2 = datainfo->t2;
+    int write_err = 0;
     int err = 0;
 
-    /* dump current dataset into session dir */
+    /* we need to retrieve and save the full version of the dataset */
 
     if (complex_subsampled()) {
-	/* save full version of dataset */
-	double ***fullZ = fetch_full_Z();
-
-	dZ = (const double **) *fullZ;
-	dinfo = fetch_full_datainfo();
-    } else {
-	dZ = (const double **) Z;
-	dinfo = datainfo;
+	mask = copy_datainfo_submask(datainfo, &err);
+	if (!err && dataset_is_resampled(datainfo)) {
+	    /* can't happen? */
+	    mask = NULL;
+	}
     }
 
-    t1 = dinfo->t1;
-    t2 = dinfo->t2;
-    dinfo->t1 = 0;
-    dinfo->t2 = dinfo->n - 1;
+    if (!err) {
+	err = restore_full_sample(&Z, datainfo, NULL);
+    }
 
-    session_file_make_path(tmpname, dname);
-    err = gretl_write_gdt(tmpname, NULL, dZ, dinfo, 0, NULL);
+    if (!err) {
+	session_file_make_path(tmpname, dname);
+	write_err = gretl_write_gdt(tmpname, NULL, (const double **) Z, 
+				    datainfo, OPT_NONE, 0);
+	fprintf(stderr, "Save session datafile as '%s', err = %d\n",
+		tmpname, write_err);
+    }
 
-    fprintf(stderr, "Save session datafile as '%s', err = %d\n",
-	    tmpname, err);
+    if (mask != NULL) {
+	/* reset the prior subsample */
+	if (!err) {
+	    err = restrict_sample_from_mask(mask, &Z, datainfo, OPT_NONE);
+	}
+	free(mask);
+    }
 
-    dinfo->t1 = t1;
-    dinfo->t2 = t2;
+    datainfo->t1 = save_t1;
+    datainfo->t2 = save_t2;
+
+    if (!err) {
+	err = write_err;
+    }
+
+    if (!err) {
+	/* flag the fact that the data are saved */
+	data_status &= ~MODIFIED_DATA;
+    }
     
     return err;
 }
@@ -1561,12 +1657,47 @@ static const char *unpath (const char *fname)
     return fname;
 }
 
+/* called in the context of a re-opened session */
+
+int save_session_dataset (void)
+{
+    int err = real_save_session_dataset(unpath(datafile));
+
+    if (err) {
+	gui_errmsg(err);
+    } 
+
+    return err;
+}
+
 static void make_session_dataname (char *datname)
 {
-    if (*paths.datfile != '\0') {
-	strcpy(datname, unpath(paths.datfile));
+    int changed = 0;
+
+    if (*datafile != '\0') {
+	char *p;
+
+	strcpy(datname, unpath(datafile));
+	p = strrchr(datname, '.');
+	if (p == NULL) {
+	    strcat(datname, ".gdt");
+	    changed = 1;
+	} else if (strcmp(p, ".gdt")) {
+	    strcpy(p, ".gdt");
+	    changed = 1;
+	}
     } else {
 	strcpy(datname, "data.gdt");
+	changed = 1;
+    }
+
+    if (changed) {
+	GtkWidget *dlabel;
+
+	dlabel = g_object_get_data(G_OBJECT(mdata->main), "dlabel");
+	if (dlabel != NULL) {
+	    gtk_label_set_text(GTK_LABEL(dlabel), datname);
+	}
     }
 }
 
@@ -1579,6 +1710,9 @@ int save_session (char *fname)
     int len, err = 0;
     GError *gerr = NULL;
 
+    fprintf(stderr, "save_session: fname='%s', sessionfile='%s'\n",
+	    fname, sessionfile);
+
     if (fname == NULL) {
 	/* saving session 'as is' */
 	fname = sessionfile;
@@ -1601,7 +1735,7 @@ int save_session (char *fname)
     }
 
     /* paths below are relative to this */
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(gretl_dotdir());
 
     if (strcmp(dirname, session.dirname)) {
 	/* rename session directory */
@@ -1611,7 +1745,7 @@ int save_session (char *fname)
 
     make_session_dataname(datname);
     write_session_xml(datname);
-    err = save_session_dataset(datname);
+    err = real_save_session_dataset(datname);
 
     if (!err) {
 	session_switch_log_location(LOG_SAVE);
@@ -1634,10 +1768,13 @@ int save_session (char *fname)
 	g_error_free(gerr);
     } else {
 	mkfilelist(FILE_LIST_SESSION, fname);
-	mark_session_saved();
 	if (fname != sessionfile) {
+	    session_name_from_session_file(session.name, fname);
 	    strcpy(sessionfile, fname);
+	    data_status |= SESSION_DATA;
+	    set_sample_label(datainfo);
 	}
+	mark_session_saved();
     }
 
     return err;
@@ -1658,10 +1795,33 @@ void save_session_callback (GtkAction *action)
     if (as_is) {
 	save_session(sessionfile);
     } else {
-	file_selector(_("Save session"), SAVE_SESSION, FSEL_DATA_NONE, NULL);
+	file_selector(SAVE_SESSION, FSEL_DATA_NONE, NULL);
     }
 }
 
+int save_session_commands (char *fname)
+{
+    FILE *fp = gretl_fopen(fname, "w");
+    int err = 0;
+
+    if (fp == NULL) {
+	file_write_errbox(fname);
+	err = E_FOPEN;
+    } else {
+	gchar *s = get_logfile_content(&err);
+
+	if (err) {
+	    gui_errmsg(err);
+	} else {
+	    fputs(s, fp);
+	}
+
+	fclose(fp);
+    }
+
+    return err;
+}
+
 static char *model_cmd_str (MODEL *pmod)
 {
     char *str = NULL;
@@ -1719,7 +1879,6 @@ static gchar *graph_str (SESSION_GRAPH *graph)
 	    } 
 	}
 
-#ifdef ENABLE_NLS
 	if (gottitle) {
 	    /* FIXME? encoding */
 	    buf = my_locale_to_utf8(title);
@@ -1731,13 +1890,6 @@ static gchar *graph_str (SESSION_GRAPH *graph)
 		free(s);
 	    }
 	}
-#else
-	if (gottitle) {
-	    buf = g_strdup(title);
-	} else if (gotxy == 2) {
-	    buf = g_strdup_printf("%s %s %s", ylabel, _("versus"), xlabel);
-	}
-#endif
 
 	fclose(fp);
     }
@@ -1804,12 +1956,7 @@ void session_model_callback (void *ptr, int action)
     if (action == OBJ_ACTION_SHOW) {
 	display_session_model(mod);
     } else if (action == OBJ_ACTION_FREE) {
-	gui_obj *gobj = get_gui_obj_by_data(mod);
-
-	if (gobj != NULL) {
-	    /* model icon is shown: delete it */
-	    session_delete_icon(gobj);
-	}	
+	delete_icon_for_data(mod);
 	real_delete_model_from_session(mod);
     } 
 }	
@@ -1907,7 +2054,7 @@ static void remove_session_graph_file (const char *gfname)
 {
     char fname[MAXLEN];
 
-    gretl_chdir(paths.dotdir);
+    gretl_chdir(gretl_dotdir());
     session_file_make_path(fname, gfname);
     gretl_remove(fname);
 }
@@ -1994,14 +2141,14 @@ static void maybe_delete_session_object (gui_obj *obj)
     g_free(msg);
 }
 
-static gui_obj *get_gui_obj_by_data (void *finddata)
+static gui_obj *get_gui_obj_by_data (void *targ)
 {
     GList *mylist = iconlist;
     gui_obj *obj = NULL;
 
     while (mylist != NULL) {
 	obj = (gui_obj *) mylist->data;
-	if (obj->data == finddata) {
+	if (obj->data == targ) {
 	    return obj;
 	}
 	mylist = mylist->next;
@@ -2467,7 +2614,7 @@ static void graph_page_save_wrapper (void)
     if (graph_page_get_n_graphs() == 0) {
 	warnbox(_("The graph page is empty"));
     } else {
-	file_selector(_("Save LaTeX file"), SAVE_TEX, FSEL_DATA_NONE, NULL);
+	file_selector(SAVE_TEX, FSEL_DATA_NONE, NULL);
     }
 }
 
@@ -2515,7 +2662,7 @@ static gboolean session_view_click (GtkWidget *widget,
 	    open_matrix(obj); 
 	    break;
 	case GRETL_OBJ_INFO:
-	    open_info(); 
+	    dataset_info(); 
 	    break;
 	case GRETL_OBJ_NOTES:
 	    edit_session_notes(); 
@@ -2536,7 +2683,7 @@ static gboolean session_view_click (GtkWidget *widget,
 	return TRUE;
     }
 
-    gdk_window_get_pointer(widget->window, NULL, NULL, &mods);
+    mods = widget_get_pointer_mask(widget);
 
     if (mods & GDK_BUTTON3_MASK) {
 	if (obj->sort == GRETL_OBJ_EQN  || obj->sort == GRETL_OBJ_GRAPH || 
@@ -2565,7 +2712,7 @@ static void global_popup_callback (GtkWidget *widget, gpointer data)
 		mark_session_saved();
 	    }
 	} else {
-	    file_selector(_("Save session"), SAVE_SESSION, FSEL_DATA_NONE, NULL);
+	    file_selector(SAVE_SESSION, FSEL_DATA_NONE, NULL);
 	}
     } else if (!strcmp(item, _("Arrange icons"))) {
 	rearrange_icons();
@@ -2576,13 +2723,7 @@ static void global_popup_callback (GtkWidget *widget, gpointer data)
 
 static void info_popup_callback (GtkWidget *widget, gpointer data)
 {
-    gchar *item = (gchar *) data;
-
-    if (!strcmp(item, _("View"))) {
-	open_info();
-    } else if (!strcmp(item, _("Edit"))) { 
-	edit_header(NULL);
-    }
+    dataset_info();
 }
 
 static void matrix_popup_callback (GtkWidget *widget, gpointer data)
@@ -2675,11 +2816,14 @@ static void object_popup_callback (GtkWidget *widget, gpointer data)
 	    char fullname[MAXLEN];
 	    windata_t *vwin;
 
-	    gretl_chdir(paths.dotdir);
+	    gretl_chdir(gretl_dotdir());
 	    session_file_make_path(fullname, graph->fname);
 	    remove_png_term_from_plotfile_by_name(fullname);
 	    vwin = view_file(fullname, 1, 0, 78, 400, EDIT_GP);
 	    object_set_window_title(vwin, obj);
+	    /* add flag so we can mark the session as modified
+	       if the plot file is changed */
+	    vwin->flags |= VWIN_SESSION_GRAPH;
 	}
     } else if (!strcmp(item, _("Rename"))) {
 	rename_object_dialog(obj);
@@ -2689,25 +2833,19 @@ static void object_popup_callback (GtkWidget *widget, gpointer data)
 	if (obj->sort == GRETL_OBJ_EQN) {
 	    SESSION_MODEL *mod = (SESSION_MODEL *) obj->data;
 
-	    add_to_model_table(mod->ptr, MODEL_ADD_FROM_MENU, NULL);
+	    add_to_model_table(mod->ptr, MODEL_ADD_FROM_MENU, 0, NULL);
 	}
     } else if (!strcmp(item, _("Clear"))) {
 	if (obj->sort == GRETL_OBJ_MODTAB) {
-	    clear_model_table(NULL);
+	    clear_model_table(0, NULL);
 	} else if (obj->sort == GRETL_OBJ_GPAGE) {
-	    clear_graph_page();
+	    clear_graph_page(0);
 	}
     } else if (!strcmp(item, _("Help"))) {
 	if (obj->sort == GRETL_OBJ_MODTAB) {
 	    context_help(NULL, GINT_TO_POINTER(MODELTAB));
 	} else if (obj->sort == GRETL_OBJ_GPAGE) {
-	    context_help(NULL, GINT_TO_POINTER(GRAPHPAGE));
-	}
-    } else if (!strcmp(item, _("Options"))) {
-	if (obj->sort == GRETL_OBJ_MODTAB) {
-	    model_table_dialog();
-	} else {
-	    dummy_call();
+	    context_help(NULL, GINT_TO_POINTER(GRAPHPG));
 	}
     } else if (!strcmp(item, _("Save as TeX..."))) {   
 	if (obj->sort == GRETL_OBJ_GPAGE) {
@@ -2748,7 +2886,7 @@ session_data_received (GtkWidget *widget,
 	MODEL **ppmod = (MODEL **) data->data;
 
 	if (ppmod != NULL) {
-	    add_to_model_table(*ppmod, MODEL_ADD_BY_DRAG, NULL);
+	    add_to_model_table(*ppmod, MODEL_ADD_BY_DRAG, 0, NULL);
 	}
     } else if (info == GRETL_GRAPH_FILE && data != NULL) {
 	gchar *fname = (gchar *) data->data;
@@ -2917,7 +3055,7 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
     }
 
     if (sort == GRETL_OBJ_DSET) {
-	obj->data = paths.datfile;
+	obj->data = datafile;
     }
 
     if (mode == ICON_ADD_SINGLE) {
@@ -2955,9 +3093,6 @@ static void session_build_popups (void)
     if (global_popup == NULL) {
 	global_popup = gtk_menu_new();
 	n = G_N_ELEMENTS(global_items);
-	if (session.status & SESSION_EMBEDDED) {
-	    n--;
-	}
 	for (i=0; i<n; i++) {
 	    create_pop_item(global_popup, _(global_items[i]), 
 			    global_popup_callback);
@@ -3086,26 +3221,17 @@ void view_session (GtkWidget *parent)
     gchar *title;
 
     if (iconview != NULL) {
-	if (parent == NULL) {
-	    gtk_window_present(GTK_WINDOW(iconview));
-	}
+	gtk_window_present(GTK_WINDOW(iconview));
 	return;
     }
 
     session_view_init();
 
-    if (parent == NULL) {
-	iconview = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	title = g_strdup_printf("gretl: %s", _("icon view"));
-	gtk_window_set_title(GTK_WINDOW(iconview), title);
-	g_free(title);
-	gtk_window_set_default_size(GTK_WINDOW(iconview), 400, 300);
-	session.status &= ~SESSION_EMBEDDED;
-    } else {
-	iconview = gtk_vbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(parent), iconview, TRUE, TRUE, 0);
-	session.status |= SESSION_EMBEDDED;
-    }
+    iconview = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    title = g_strdup_printf("gretl: %s", _("icon view"));
+    gtk_window_set_title(GTK_WINDOW(iconview), title);
+    g_free(title);
+    gtk_window_set_default_size(GTK_WINDOW(iconview), 400, 300);
 
     gtk_container_set_border_width(GTK_CONTAINER(iconview), 0);
     iconview_connect_signals(iconview);
@@ -3138,8 +3264,8 @@ void view_session (GtkWidget *parent)
     gtk_widget_show(iconview);
 
     gtk_container_foreach(GTK_CONTAINER(scroller), 
-			 (GtkCallback) white_bg_style, 
-			 NULL);
+			  (GtkCallback) white_bg_style, 
+			  NULL);
 
     GTK_WIDGET_SET_FLAGS(icon_table, GTK_CAN_FOCUS);
     gtk_widget_grab_focus(icon_table);
@@ -3157,7 +3283,7 @@ static void make_short_label_string (char *targ, const char *src)
 
 static void create_gobj_icon (gui_obj *obj, const char **xpm)
 {
-    gchar str[SHOWNAMELEN + 1];
+    gchar str[2*SHOWNAMELEN];
     GdkPixbuf *pbuf;
     GtkWidget *image;
 
@@ -3196,6 +3322,10 @@ static gui_obj *gui_object_new (gchar *name, int sort, gpointer data)
     char **xpm = NULL;
 
     obj = mymalloc(sizeof *obj);
+    if (obj == NULL) {
+	return NULL;
+    }
+
     obj->name = name; 
     obj->sort = sort;
     obj->data = data;
@@ -3228,22 +3358,22 @@ static gui_obj *gui_object_new (gchar *name, int sort, gpointer data)
     return obj;
 } 
 
-static void open_gui_graph (gui_obj *obj)
+static void real_open_session_graph (SESSION_GRAPH *graph)
 {
-    SESSION_GRAPH *graph = (SESSION_GRAPH *) obj->data;
     char tmp[MAXLEN];
 
     session_file_make_path(tmp, graph->fname);
-    display_session_graph_png(tmp);
+    display_session_graph(tmp, graph->name);
+}
+
+static void open_gui_graph (gui_obj *obj)
+{
+    real_open_session_graph((SESSION_GRAPH *) obj->data);
 }
 
 void display_session_graph_by_data (void *p)
 {
-    SESSION_GRAPH *graph = (SESSION_GRAPH *) p;
-    char tmp[MAXLEN];
-
-    session_file_make_path(tmp, graph->fname);
-    display_session_graph_png(tmp);
+    real_open_session_graph((SESSION_GRAPH *) p);
 }
 
 static int is_idempotent (const gretl_matrix *m,
@@ -3293,6 +3423,7 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
 {
     gretl_matrix *A = NULL;
     gretl_matrix *evals = NULL;
+    gchar *title;
     PRN *prn;
     int s, err = 0;
 
@@ -3341,13 +3472,16 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
 		pprintf(prn, "%s\n", _("Not positive definite"));
 	    }
 	    gretl_matrix_copy_values(A, m);
+	    err = 0;
 	    evals = gretl_symmetric_matrix_eigenvals(A, 0, &err);
 	}
     } 
 
     if (s > 0 && (s != GRETL_MATRIX_SYMMETRIC || evals == NULL)) {
+	gretl_matrix_free(A);
 	A = gretl_matrix_copy(m);
 	if (A != NULL) {
+	    err = 0;
 	    evals = gretl_general_matrix_eigenvals(A, 0, &err);
 	}
     }
@@ -3405,7 +3539,9 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
 
  done:
 
-    view_buffer(prn, 78, 400, name, PRINT, NULL);
+    title = gretl_window_title(name, ("properties"));
+    view_buffer(prn, 78, 400, title, PRINT, NULL);
+    g_free(title);
 }
 
 
