@@ -45,8 +45,16 @@ enum {
 
 int save_session (char *fname);
 
+int save_session_commands (char *fname);
+
+int save_session_dataset (void);
+
 int session_is_modified (void);
 
+void set_commands_recorded (void);
+
+int get_commands_recorded (void);
+
 void session_menu_state (gboolean s);
 
 const char *get_session_dirname (void);
@@ -60,9 +68,14 @@ int add_graph_to_session (char *fname, char *fullname, int type);
 int cli_add_graph_to_session (const char *fname, const char *gname,
 			      GretlObjType type);
 
+char *session_graph_make_path (char *path, const char *fname);
+
+const char *last_session_graph_name (void);
+
 void model_add_as_icon (GtkAction *action, gpointer p);
 
-int maybe_add_model_to_session (void *ptr, GretlObjType type);
+int maybe_add_model_to_session (void *ptr, GretlObjType type,
+				const char *name);
 
 void session_model_callback (void *ptr, int action);
 
@@ -91,8 +104,6 @@ void free_session (void);
 
 int highest_numbered_variable_in_session (void);
 
-int is_session_file (const char *fname);
-
 int is_session_model (void *p);
 
 void view_session (GtkWidget *parent);
