@@ -21,6 +21,7 @@
 #include "textbuf.h"
 #include "gretl_func.h"
 #include "toolbar.h"
+#include "dlgutils.h"
 
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcelanguage.h>
@@ -41,7 +42,7 @@ enum {
 };
 
 #define gui_help(r) (r == GUI_HELP || r == GUI_HELP_EN)
-#define foreign_script_role(r) (r == EDIT_GP || r == EDIT_R)
+#define foreign_script_role(r) (r == EDIT_GP || r == EDIT_R || r == EDIT_OX)
 
 /* globals accessed in settings.c */
 int tabwidth = 4;
@@ -141,6 +142,33 @@ gchar *textview_get_text (GtkWidget *view)
     return gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
 }
 
+gchar *textview_get_trimmed_text (GtkWidget *view)
+{
+    gchar *s = textview_get_text(view);
+    gchar *ret = NULL;
+
+    if (s != NULL && *s != '\0') {
+	while (isspace(*s)) s++;
+
+	if (*s != '\0') {
+	    int i, len = strlen(s);
+
+	    for (i=len-1; i>0; i--) {
+		if (!isspace(s[i])) break;
+		len--;
+	    }
+
+	    if (len > 0) {
+		ret = g_strndup(s, len);
+	    }
+	}
+    }
+
+    g_free(s);
+
+    return ret;
+}
+
 gchar *textview_get_selection_or_all (GtkWidget *view,
 				      int *sel)
 {
@@ -165,7 +193,8 @@ gchar *textview_get_selection_or_all (GtkWidget *view,
     return gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
 }
 
-int textview_set_text (GtkWidget *view, const gchar *text)
+int real_textview_set_text (GtkWidget *view, const gchar *text,
+			    int select)
 {
     GtkTextBuffer *tbuf;
 
@@ -174,7 +203,14 @@ int textview_set_text (GtkWidget *view, const gchar *text)
     tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
     g_return_val_if_fail(tbuf != NULL, 1);
 
-    if (text != NULL) {
+    if (text != NULL && select) {
+	GtkTextIter start, end;
+
+	gtk_text_buffer_set_text(tbuf, text, -1);
+	gtk_text_buffer_get_start_iter(tbuf, &start);
+	gtk_text_buffer_get_end_iter(tbuf, &end);
+	gtk_text_buffer_select_range(tbuf, &start, &end);
+    } else if (text != NULL) {
 	gtk_text_buffer_set_text(tbuf, text, -1);
     } else {
 	gtk_text_buffer_set_text(tbuf, "", -1);
@@ -183,6 +219,16 @@ int textview_set_text (GtkWidget *view, const gchar *text)
     return 0;
 }
 
+int textview_set_text (GtkWidget *view, const gchar *text)
+{
+    return real_textview_set_text(view, text, 0);
+}
+
+int textview_set_text_selected (GtkWidget *view, const gchar *text)
+{
+    return real_textview_set_text(view, text, 1);
+}
+
 int textview_set_cursor_at_line (GtkWidget *view, int line)
 {
     GtkTextBuffer *tbuf;
@@ -285,9 +331,7 @@ static int source_buffer_load_file (GtkSourceBuffer *sbuf,
     char fline[MAXSTR];
     gchar *chunk = NULL;
     GtkTextIter iter;
-#ifdef ENABLE_NLS
     int i = 0;
-#endif
 
     gtk_source_buffer_begin_not_undoable_action(sbuf);
 
@@ -297,7 +341,6 @@ static int source_buffer_load_file (GtkSourceBuffer *sbuf,
     memset(fline, 0, sizeof fline);
 
     while (fgets(fline, sizeof fline, fp)) {
-#ifdef ENABLE_NLS
 	if (!g_utf8_validate(fline, -1, NULL)) {
 	    if (i == 0) {
 		chunk = my_locale_to_utf8(fline);
@@ -311,9 +354,6 @@ static int source_buffer_load_file (GtkSourceBuffer *sbuf,
 	} else {
 	    chunk = fline;
 	}
-#else
-	chunk = fline;
-#endif /* ENABLE_NLS */
 
 	strip_CRLF(chunk);
 
@@ -377,11 +417,14 @@ static void sourceview_apply_language (windata_t *vwin)
 	id = "gnuplot";
     } else if (vwin->role == EDIT_R) {
 	id = "r";
+    } else if (vwin->role == EDIT_OX) {
+	id = "cpp";
     } else {
 	id = "gretl";
     }
 
     lang = gtk_source_language_manager_get_language(lm, id);
+
     if (lang == NULL) {
 	fprintf(stderr, "*** gtksourceview: lang is NULL for id='%s'\n", id);
     } else {
@@ -403,12 +446,16 @@ static void sourceview_apply_language (windata_t *vwin)
 	mtype = "application/x-gnuplot";
     } else if (vwin->role == EDIT_R) {
 	mtype = "text/x-R";
+    } else if (vwin->role == EDIT_OX) {
+	mtype = "text/x-c++src";
     } else {
 	mtype = "application/x-gretlscript";
     }
 
     lang = gtk_source_languages_manager_get_language_from_mime_type(lm, mtype);
+
     if (lang == NULL) {
+	fprintf(stderr, "*** gtksourceview: lang is NULL for mtype='%s'\n", mtype);
 	g_object_set(G_OBJECT(vwin->sbuf), "highlight", FALSE, NULL);
     } else {
 	g_object_set(G_OBJECT(vwin->sbuf), "highlight", TRUE, NULL);
@@ -473,7 +520,9 @@ static void set_source_tabs (GtkWidget *w, int cw)
 		   k == GDK_ISO_Left_Tab || \
 		   k == GDK_KP_Tab)
 
-#define script_editing(r) (r == EDIT_SCRIPT || r == EDIT_FUNC_CODE)
+#define script_editing(r) (r == EDIT_SCRIPT || \
+			   r == EDIT_PKG_CODE || \
+			   r == EDIT_PKG_SAMPLE)
 
 /* Special keystrokes in script window: Ctrl-Return sends the current
    line for execution; Ctrl-R sends the whole script for execution
@@ -483,11 +532,9 @@ static void set_source_tabs (GtkWidget *w, int cw)
 static gint 
 script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 {
-    GdkModifierType mods;
+    GdkModifierType mods = widget_get_pointer_mask(w);
     gboolean ret = FALSE;
 
-    gdk_window_get_pointer(w->window, NULL, NULL, &mods);
-
     if (mods & GDK_CONTROL_MASK) {
 	if (key->keyval == GDK_r)  {
 	    do_run_script(w, vwin);
@@ -521,7 +568,8 @@ script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 
 #define gretl_script_role(r) (r == EDIT_SCRIPT || \
 			      r == VIEW_SCRIPT || \
-			      r == EDIT_FUNC_CODE)
+			      r == EDIT_PKG_CODE || \
+			      r == EDIT_PKG_SAMPLE)
 
 void create_source (windata_t *vwin, int hsize, int vsize, 
 		    gboolean editable)
@@ -610,6 +658,8 @@ void text_zoom (GtkAction *action, gpointer data)
     gtk_widget_modify_font(vwin->text, hpf);
 }
 
+#define helpfont "sans"
+
 static GtkTextTagTable *gretl_tags_new (void)
 {
     GtkTextTagTable *table;
@@ -628,24 +678,24 @@ static GtkTextTagTable *gretl_tags_new (void)
     tag = gtk_text_tag_new("title");
     g_object_set(tag, "justification", GTK_JUSTIFY_CENTER,
 		 "pixels_above_lines", 15,
-		 "family", "sans",
+		 "family", helpfont,
 		 "size", 15 * PANGO_SCALE, NULL);
     gtk_text_tag_table_add(table, tag);
 
-    tag = gtk_text_tag_new("sansbold");
-    g_object_set(tag, "family", "sans", 
+    tag = gtk_text_tag_new("heading");
+    g_object_set(tag, "family", helpfont, 
 		 "weight", PANGO_WEIGHT_BOLD, 
 		 NULL);
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("italic");
-    g_object_set(tag, "family", "sans",
+    g_object_set(tag, "family", helpfont,
 		 "style", PANGO_STYLE_ITALIC, 
 		 NULL);
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("replaceable");
-    g_object_set(tag, "family", "sans",
+    g_object_set(tag, "family", helpfont,
 		 "style", PANGO_STYLE_ITALIC, 
 		 NULL);
 
@@ -671,7 +721,7 @@ static GtkTextTagTable *gretl_tags_new (void)
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("text");
-    g_object_set(tag, "family", "sans", NULL);
+    g_object_set(tag, "family", helpfont, NULL);
     gtk_text_tag_table_add(table, tag);
 
     tag = gtk_text_tag_new("indented");
@@ -707,7 +757,7 @@ real_textview_add_colorized (GtkWidget *view, const char *buf,
     GtkTextIter iter; 
     int nextcolor, thiscolor = PLAIN_TEXT;
     int in_comment = 0;
-    char readbuf[MAXSTR];
+    char readbuf[2048];
     int i = 0;
 
     g_return_if_fail(GTK_IS_TEXT_VIEW(view));
@@ -795,7 +845,6 @@ void textview_insert_file (windata_t *vwin, const char *fname)
     memset(fline, 0, sizeof fline);
 
     while (fgets(fline, sizeof fline, fp)) {
-#ifdef ENABLE_NLS
 	if (!g_utf8_validate(fline, -1, NULL)) {
 	    if (i == 0) {
 		chunk = my_locale_to_utf8(fline);
@@ -809,9 +858,6 @@ void textview_insert_file (windata_t *vwin, const char *fname)
 	} else {
 	    chunk = fline;
 	}
-#else
-	chunk = fline;
-#endif
 
 	nextcolor = PLAIN_TEXT;
 	
@@ -958,10 +1004,21 @@ static void insert_xlink (GtkTextBuffer *tbuf, GtkTextIter *iter,
 static void link_open_script (GtkTextTag *tag)
 {
     const char *fname = g_object_get_data(G_OBJECT(tag), "fname");
+    const char *gretldir = gretl_home();
     char fullname[MAXLEN];
+    FILE *fp;
 
-    sprintf(fullname, "%sscripts%cmisc%c%s", paths.gretldir, 
+    sprintf(fullname, "%sscripts%cmisc%c%s", gretldir, 
 	    SLASH, SLASH, fname);
+
+    fp = gretl_fopen(fullname, "r");
+    if (fp != NULL) {
+	fclose(fp);
+    } else {
+	sprintf(fullname, "%sscripts%c%s", gretldir, 
+		SLASH, fname);
+    }
+
     view_file(fullname, 0, 0, 78, 370, VIEW_SCRIPT);
 }
 
@@ -1127,7 +1184,7 @@ cmdref_motion_notify (GtkWidget *tview, GdkEventMotion *event)
 					  GTK_TEXT_WINDOW_WIDGET,
 					  event->x, event->y, &x, &y);
     set_cursor_if_appropriate(GTK_TEXT_VIEW(tview), x, y);
-    gdk_window_get_pointer(tview->window, NULL, NULL, NULL);
+    widget_get_pointer_mask(tview);
 
     return FALSE;
 }
@@ -1136,8 +1193,8 @@ static gboolean
 cmdref_visibility_notify (GtkWidget *tview,  GdkEventVisibility *e)
 {
     gint wx, wy, bx, by;
-  
-    gdk_window_get_pointer(tview->window, &wx, &wy, NULL);
+
+    widget_get_pointer_info(tview, &wx, &wy, NULL);
     gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(tview), 
 					  GTK_TEXT_WINDOW_WIDGET,
 					  wx, wy, &bx, &by);
@@ -1191,11 +1248,16 @@ static void maybe_set_help_tabs (windata_t *hwin)
     }
 }
 
-static void cmdref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
+/* construct the index page for the gretl command reference */
+
+static void cmdref_index_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
 {
     const char *header = N_("Gretl Command Reference");
+    const gchar *s = (const gchar *) hwin->data;
     GtkTextIter iter;
-    int i, j, k;
+    char word[10];
+    int llen, llen_max = 6;
+    int idx, j, n;
 
     gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
     gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
@@ -1203,26 +1265,25 @@ static void cmdref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
 					     "title", NULL);
     gtk_text_buffer_insert(tbuf, &iter, "\n\n", -1);
 
-    j = 1;
+    llen = 0;
 
-    for (i=1; i<NC; i++) {
-	const char *word;
-
-	if (HIDDEN_COMMAND(i)) {
-	    continue;
-	}
-
-	word = gretl_command_word(i);
-	insert_link(tbuf, &iter, gretl_command_word(i), i, NULL);
-	if (j++ % 8 == 0) {
-	    gtk_text_buffer_insert(tbuf, &iter, "\n", -1);
-	} else {
-	    int n = 10 - strlen(word);
-
-	    for (k=0; k<n; k++) {
-		gtk_text_buffer_insert(tbuf, &iter, " ", -1);
+    while (*s) {
+	if (*s == '\n' && *(s+1) == '#' && *(s+2) != '\0') {
+	    if (sscanf(s + 2, "%8s", word)) {
+		idx = command_help_index(word);
+		insert_link(tbuf, &iter, word, idx, NULL);
+		if (++llen == llen_max) {
+		    gtk_text_buffer_insert(tbuf, &iter, "\n", -1);
+		    llen = 0;
+		} else {
+		    n = 10 - strlen(word);
+		    for (j=0; j<n; j++) {
+			gtk_text_buffer_insert(tbuf, &iter, " ", -1);
+		    }
+		}
 	    }
 	}
+	s++;
     }
 
     gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->text), tbuf);
@@ -1231,13 +1292,15 @@ static void cmdref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
     maybe_set_help_tabs(hwin);
 }
 
-static void funcref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
+/* construct the index page for the gretl function reference */
+
+static void funcref_index_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
 {
     const char *header = N_("Gretl Function Reference");
-    const gchar *s;
+    const gchar *s = (const gchar *) hwin->data;
     GtkTextIter iter;
-    char funword[12];
-    int llen;
+    char word[12];
+    int llen, llen_max = 5;
     int i, j, n;
 
     gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
@@ -1246,7 +1309,6 @@ static void funcref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
 					     "title", NULL);
     gtk_text_buffer_insert(tbuf, &iter, "\n\n", -1);
 
-    s = (const gchar *) hwin->data;
     i = 1;
     llen = 0;
 
@@ -1256,20 +1318,20 @@ static void funcref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
 		/* category divider */
 		if (i > 1) {
 		    gtk_text_buffer_insert(tbuf, &iter, "\n", -1);
-		    if (llen < 7) {
+		    if (llen < llen_max) {
 			gtk_text_buffer_insert(tbuf, &iter, "\n", -1);
 			llen = 0;
 		    }
 		}
 		s += 2;
-	    } else if (sscanf(s + 2, "%10s", funword)) {
+	    } else if (sscanf(s + 2, "%10s", word)) {
 		/* function name */
-		insert_link(tbuf, &iter, funword, i, NULL);
-		if (++llen == 7) {
+		insert_link(tbuf, &iter, word, i, NULL);
+		if (++llen == llen_max) {
 		    gtk_text_buffer_insert(tbuf, &iter, "\n", -1);
 		    llen = 0;
 		} else {
-		    n = 12 - strlen(funword);
+		    n = 12 - strlen(word);
 		    for (j=0; j<n; j++) {
 			gtk_text_buffer_insert(tbuf, &iter, " ", -1);
 		    }
@@ -1286,6 +1348,22 @@ static void funcref_title_page (windata_t *hwin, GtkTextBuffer *tbuf, int en)
     maybe_set_help_tabs(hwin);
 }
 
+/* apparatus to support the 'Back' popup menu item */
+
+static void push_backpage (GtkWidget *w, int pg)
+{
+    gpointer p = GINT_TO_POINTER(pg);
+
+    g_object_set_data(G_OBJECT(w), "backpage", p);
+}
+
+static int pop_backpage (GtkWidget *w)
+{
+    gpointer p = g_object_get_data(G_OBJECT(w), "backpage");
+
+    return GPOINTER_TO_INT(p);
+}
+
 static gint help_popup_click (GtkWidget *w, gpointer p)
 {
     windata_t *hwin = (windata_t *) p;
@@ -1294,8 +1372,7 @@ static gint help_popup_click (GtkWidget *w, gpointer p)
     int page = 0;
 
     if (action == 2) {
-	page = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->text), 
-						 "backpage"));
+	page = pop_backpage(hwin->text); 
     }
 
     if (hwin->role == FUNCS_HELP) {
@@ -1315,9 +1392,19 @@ static GtkWidget *build_help_popup (windata_t *hwin)
     };
     GtkWidget *pmenu = gtk_menu_new();
     GtkWidget *item;
-    int i;
+    int i, imin = 0, imax = 2;
 
-    for (i=0; i<2; i++) {
+    if (hwin->active_var == 0) {
+	/* don't offer "Index" if we're in the index */
+	imin = 1;
+    }
+
+    if (pop_backpage(hwin->text) == 0) {
+	/* don't offer "Back" if we haven't been anywhere */
+	imax = 1;
+    }
+
+    for (i=imin; i<imax; i++) {
 	item = gtk_menu_item_new_with_label(_(items[i]));
 	g_object_set_data(G_OBJECT(item), "action", GINT_TO_POINTER(i+1));
 	g_signal_connect(G_OBJECT(item), "activate",
@@ -1333,14 +1420,12 @@ static GtkWidget *build_help_popup (windata_t *hwin)
 gboolean 
 help_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
 {
-    GdkModifierType mods;
-
-    gdk_window_get_pointer(w->window, NULL, NULL, &mods);
+    GdkModifierType mods = widget_get_pointer_mask(w);
 
     if (mods & GDK_BUTTON3_MASK) {
 	windata_t *hwin = (windata_t *) p;
 
-	if (hwin->active_var == 0) {
+	if (hwin->active_var == 0 && pop_backpage(w) == 0) {
 	    return TRUE;
 	}
 
@@ -2182,8 +2267,6 @@ static void line_numbers_cb (GtkWidget *w, windata_t *vwin)
     gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(vwin->text), !s);
 }
 
-#define editing_code(r) (r == EDIT_SCRIPT || r == EDIT_FUNC_CODE)
-
 static GtkWidget *
 build_script_popup (windata_t *vwin, struct textbit **ptb)
 {
@@ -2215,7 +2298,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 
     tb->commented = text_is_commented(tb->chunk);
 
-    if (tb->commented > 0 && !editing_code(vwin->role)) {
+    if (tb->commented > 0 && !script_editing(vwin->role)) {
 	g_free(tb->chunk);
 	free(tb);
 	*ptb = NULL;
@@ -2224,7 +2307,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 
     *ptb = tb;
 
-    if (tb->commented <= 0 && vwin->role != EDIT_FUNC_CODE) {
+    if (tb->commented <= 0 && vwin->role != EDIT_PKG_CODE) {
 	/* we have some uncommented material: allow exec option */
 	if (tb->selected) {
 	    item = gtk_menu_item_new_with_label(_("Execute region"));
@@ -2238,7 +2321,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
     }
 
-    if (editing_code(vwin->role) && tb->commented >= 0) {
+    if (script_editing(vwin->role) && tb->commented >= 0) {
 	/* material is either all commented or all uncommented:
 	   allow comment/uncomment option */
 	int i = (tb->selected && !tb->commented)? 2 : 
@@ -2253,7 +2336,7 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
     }
 
-    if (editing_code(vwin->role)) {
+    if (script_editing(vwin->role)) {
 	if (tb->selected) {
 	    item = gtk_menu_item_new_with_label(smarttab? 
 						_("Auto-indent region") :
@@ -2293,6 +2376,16 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
     }
 
+    item = gtk_separator_menu_item_new();
+    gtk_widget_show(item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
+
+    item = gtk_menu_item_new_with_label(_("Main window"));
+    g_signal_connect(G_OBJECT(item), "activate",
+		     G_CALLBACK(raise_main_window), NULL);
+    gtk_widget_show(item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
+
     return pmenu;
 }
 
@@ -2310,9 +2403,7 @@ static gboolean destroy_textbit (GtkWidget **pw, struct textbit *tc)
 static gboolean 
 script_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
 {
-    GdkModifierType mods;
-
-    gdk_window_get_pointer(w->window, NULL, NULL, &mods);
+    GdkModifierType mods = widget_get_pointer_mask(w);
 
     if (mods & GDK_BUTTON3_MASK) {
 	windata_t *vwin = (windata_t *) p;
@@ -2361,7 +2452,7 @@ static void insert_help_figure (GtkTextBuffer *tbuf, GtkTextIter *iter,
     char figfile[FILENAME_MAX];
     GdkPixbuf *pixbuf;
 
-    sprintf(figfile, "%shelpfigs%c%s.png", paths.gretldir,
+    sprintf(figfile, "%shelpfigs%c%s.png", gretl_home(),
 	    SLASH, fig);
 
     pixbuf = gdk_pixbuf_new_from_file(figfile, NULL);
@@ -2549,7 +2640,7 @@ insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
 
 static char *grab_topic_buffer (const char *s)
 {
-    const char *p = strstr(s, "\n# ");
+    const char *p = strstr(s, "\n#");
     char *buf;
 
     if (p != NULL) {
@@ -2561,10 +2652,12 @@ static char *grab_topic_buffer (const char *s)
     return buf;
 }
 
-/* pull the appropriate chunk of help text out of the buffer attached
-   to the help viewer and display it */
+/* Pull the appropriate chunk of help text out of the buffer attached
+   to the help viewer and display it, if possible.  Return >= 0
+   if we did OK, < 0 on failure.
+*/
 
-void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
+int set_help_topic_buffer (windata_t *hwin, int pos, int en)
 {
     GtkTextBuffer *textb;
     GtkTextIter iter;
@@ -2572,21 +2665,22 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
     gchar *hbuf;
     char *buf;
 
+    push_backpage(hwin->text, hwin->active_var);
+
     textb = gretl_text_buf_new();
 
     if (pos == 0) {
 	/* no topic selected */
 	if (hwin->role == FUNCS_HELP) {
-	    funcref_title_page(hwin, textb, en);
+	    funcref_index_page(hwin, textb, en);
 	} else {
-	    cmdref_title_page(hwin, textb, en);
+	    cmdref_index_page(hwin, textb, en);
 	}
 	cursor_to_top(hwin);
-	hwin->active_var = 0;
-	return;
+	return 0;
     }
 
-    /* OK, pos is non-zero */
+    /* OK, so pos is non-zero */
 
     maybe_connect_help_signals(hwin, en);
     maybe_set_help_tabs(hwin);
@@ -2600,7 +2694,7 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
     bufgets_finalize(hbuf);
 
     if (buf == NULL) {
-	return;
+	return -1;
     }
 
     tailstrip(line);
@@ -2611,7 +2705,7 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
 
 	gtk_text_buffer_insert_with_tags_by_name(textb, &iter,
 						 p, -1,
-						 "sansbold", NULL);
+						 "heading", NULL);
 	free(p);
     } else {
 	/* topic heading: plain command word */
@@ -2631,18 +2725,17 @@ void set_help_topic_buffer (windata_t *hwin, int hcode, int pos, int en)
 
     buf = grab_topic_buffer(hbuf + strlen(line) + 1);
     if (buf == NULL) {
-	return;
+	return -1;
     }
 
     insert_text_with_markup(textb, &iter, buf, hwin->role);
     free(buf);
 
     gtk_text_view_set_buffer(GTK_TEXT_VIEW(hwin->text), textb);
-    g_object_set_data(G_OBJECT(hwin->text), "backpage", 
-		      GINT_TO_POINTER(hwin->active_var));
     maybe_connect_help_signals(hwin, en);
     cursor_to_top(hwin);
-    hwin->active_var = hcode;
+
+    return 1;
 }
 
 static int get_screen_height (void)
@@ -2714,6 +2807,8 @@ void text_table_setup (GtkWidget *vbox, GtkWidget *w)
 
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, FALSE);
+    g_object_set_data(G_OBJECT(vbox), "sw", sw);
+
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
@@ -2724,3 +2819,97 @@ void text_table_setup (GtkWidget *vbox, GtkWidget *w)
     gtk_widget_show(sw);
 }
 
+static void set_pane_text_properties (GtkTextView *w)
+{
+    gtk_text_view_set_wrap_mode(w, 0);
+    gtk_text_view_set_left_margin(w, 4);
+    gtk_text_view_set_right_margin(w, 4);
+
+    gtk_widget_modify_font(GTK_WIDGET(w), fixed_font);
+
+    gtk_text_view_set_editable(w, FALSE);
+    gtk_text_view_set_cursor_visible(w, FALSE);
+}
+
+/* limited to script output window at present: divide the
+   window into two panes */
+
+void viewer_split_pane (windata_t *vwin, int vertical)
+{
+    GtkWidget *vbox = vwin->vbox;
+    GtkWidget *view1 = vwin->text;
+    GtkWidget *sw, *paned, *view2;
+    GtkTextBuffer *tbuf;
+    gint width, height;
+
+    sw = g_object_get_data(G_OBJECT(vbox), "sw");
+
+    gtk_window_get_size(GTK_WINDOW(vwin->main), &width, &height);
+
+    g_object_ref(G_OBJECT(sw));
+    gtk_container_remove(GTK_CONTAINER(vwin->vbox), sw);
+
+    if (vertical) {
+	paned = gtk_hpaned_new();
+    } else {
+	paned = gtk_vpaned_new();
+    }
+
+    gtk_container_set_border_width(GTK_CONTAINER(paned), 0);
+    gtk_container_add(GTK_CONTAINER(vbox), paned);
+
+    g_object_set_data(G_OBJECT(vwin->vbox), "paned", paned);
+    g_object_set_data(G_OBJECT(vwin->vbox), "sw", NULL);
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view1));
+    view2 = gtk_text_view_new_with_buffer(tbuf);
+    set_pane_text_properties(GTK_TEXT_VIEW(view2));
+
+    g_signal_connect(G_OBJECT(view2), "button-press-event", 
+		     G_CALLBACK(text_popup_handler), vwin);
+
+    gtk_paned_add1(GTK_PANED(paned), sw);
+    g_object_unref(G_OBJECT(sw));
+
+    sw = gtk_scrolled_window_new(NULL, NULL);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+				   GTK_POLICY_AUTOMATIC,
+				   GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
+					GTK_SHADOW_IN);
+    gtk_paned_add2(GTK_PANED(paned), sw);
+    gtk_container_add(GTK_CONTAINER(sw), view2);
+
+    if (vertical) {
+	gtk_paned_set_position(GTK_PANED(paned), width / 2 - 5);
+    } else {
+	gtk_paned_set_position(GTK_PANED(paned), height / 2 - 24);
+    }
+
+    gtk_widget_show_all(paned);
+}
+
+/* script output window: revert to a single pane */
+
+void viewer_close_pane (windata_t *vwin)
+{
+    GtkWidget *sw, *paned;
+
+    paned = g_object_get_data(G_OBJECT(vwin->vbox), "paned");
+
+    /* grab the first child and reference it */
+    sw = gtk_paned_get_child1(GTK_PANED(paned));
+    g_object_ref(G_OBJECT(sw));
+    gtk_container_remove(GTK_CONTAINER(paned), sw);
+
+    /* remove the "paned" widget */
+    gtk_widget_destroy(paned);
+    g_object_set_data(G_OBJECT(vwin->vbox), "paned", NULL);
+
+    /* and repack the scrolled window */
+    gtk_container_add(GTK_CONTAINER(vwin->vbox), sw);
+    g_object_set_data(G_OBJECT(vwin->vbox), "sw", sw);
+    gtk_widget_show(sw);
+    g_object_unref(G_OBJECT(sw));
+}
+
