@@ -18,6 +18,7 @@
  */
 
 #include "gretl.h"
+#include "dlgutils.h"
 #include "varprint.h"
 #include "textutil.h"
 #include "textbuf.h"
@@ -45,10 +46,8 @@ struct search_replace {
 static void replace_string_setup (GtkWidget *widget, 
 				  struct search_replace *s)
 {
-    s->find = 
-	gtk_editable_get_chars(GTK_EDITABLE(s->f_entry), 0, -1);
-    s->replace = 
-	gtk_editable_get_chars(GTK_EDITABLE(s->r_entry), 0, -1);
+    s->find = gtk_editable_get_chars(GTK_EDITABLE(s->f_entry), 0, -1);
+    s->replace = gtk_editable_get_chars(GTK_EDITABLE(s->r_entry), 0, -1);
     gtk_widget_destroy(s->w);
 }
 
@@ -62,7 +61,8 @@ static void trash_replace (GtkWidget *widget,
 
 static void replace_string_dialog (struct search_replace *s)
 {
-    GtkWidget *label, *button, *hbox;
+    GtkWidget *label, *button;
+    GtkWidget *vbox, *hbox, *abox;
 
     s->w = gtk_dialog_new();
 
@@ -74,7 +74,9 @@ static void replace_string_dialog (struct search_replace *s)
 		     G_CALLBACK(esc_kills_window), NULL);
 #endif
 
-    /* Find part */
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(s->w));
+
+    /* 'Find' part */
     hbox = gtk_hbox_new(TRUE, TRUE);
     label = gtk_label_new(_("Find:"));
     gtk_widget_show(label);
@@ -83,10 +85,9 @@ static void replace_string_dialog (struct search_replace *s)
     gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), s->f_entry, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->vbox), 
-		       hbox, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
 
-    /* Replace part */
+    /* 'Replace' part */
     hbox = gtk_hbox_new(TRUE, TRUE);
     label = gtk_label_new(_("Replace with:"));
     gtk_widget_show(label);
@@ -98,12 +99,12 @@ static void replace_string_dialog (struct search_replace *s)
     gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), s->r_entry, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->vbox), 
-		       hbox, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
 
-    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(s->w)->action_area), 15);
-    gtk_box_set_homogeneous(GTK_BOX 
-			    (GTK_DIALOG(s->w)->action_area), TRUE);
+    abox = gtk_dialog_get_action_area(GTK_DIALOG(s->w));
+
+    gtk_box_set_spacing(GTK_BOX(abox), 15);
+    gtk_box_set_homogeneous(GTK_BOX(abox), TRUE);
     gtk_window_set_position(GTK_WINDOW(s->w), GTK_WIN_POS_MOUSE);
 
     g_signal_connect(G_OBJECT(s->w), "destroy",
@@ -112,8 +113,7 @@ static void replace_string_dialog (struct search_replace *s)
     /* replace button -- make this the default */
     button = gtk_button_new_with_label(_("Replace all"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->action_area), 
-		       button, TRUE, TRUE, FALSE);
+    gtk_box_pack_start(GTK_BOX(abox), button, TRUE, TRUE, 0);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(replace_string_setup), s);
     gtk_widget_grab_default(button);
@@ -122,8 +122,7 @@ static void replace_string_dialog (struct search_replace *s)
     /* cancel button */
     button = gtk_button_new_with_label(_("Cancel"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->action_area), 
-		       button, TRUE, TRUE, FALSE);
+    gtk_box_pack_start(GTK_BOX(abox), button, TRUE, TRUE, 0);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(trash_replace), s);
     gtk_widget_show(button);
@@ -349,7 +348,7 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
 	} else if (what == W_SAVE) {
 	    int action = (fmt & GRETL_FORMAT_TEX)? SAVE_TEX : SAVE_RTF;
 
-	    file_selector_with_parent(_("Save"), action, FSEL_DATA_PRN, 
+	    file_selector_with_parent(action, FSEL_DATA_PRN, 
 				      prn, vwin->main);
 	}
     }
@@ -366,7 +365,17 @@ void window_tex_callback (GtkWidget *w, windata_t *vwin)
 	N_("Copy"),
 	N_("Save")
     };
-    int opt = radio_dialog("gretl: LaTeX", NULL, opts, 3, 0, 0);
+    int opt;
+
+    if (vwin->role == VAR_IRF || vwin->role == VAR_DECOMP) {
+	if (vwin->gretl_parent == NULL) {
+	    warnbox(_("Not available"));
+	    gtk_widget_set_sensitive(w, FALSE);
+	    return;
+	}
+    }
+
+    opt = radio_dialog("gretl: LaTeX", NULL, opts, 3, 0, 0);
 
     if (opt >= 0) {
 	int fmt = GRETL_FORMAT_TEX;
@@ -421,29 +430,25 @@ void model_tex_copy (GtkAction *action, gpointer data)
 
 static gchar *text_window_get_copy_buf (windata_t *vwin, int select)
 {
-    gchar *cpybuf = NULL;
+    gchar *cpy = NULL;
 
     if (select) {
-	GtkTextBuffer *textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
-	GtkTextIter selstart, selend;
+	GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
+	GtkTextIter start, end;
 
-	if (gtk_text_buffer_get_selection_bounds(textbuf, &selstart, &selend)) {
-	    cpybuf = gtk_text_buffer_get_text(textbuf, &selstart, &selend, FALSE);
+	if (gtk_text_buffer_get_selection_bounds(buf, &start, &end)) {
+	    cpy = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
 	}
     } else {	
-	cpybuf = textview_get_text(vwin->text); 
+	cpy = textview_get_text(vwin->text); 
     }
 
-    return cpybuf;
+    return cpy;
 }
 
 static gchar *maybe_amend_buffer (gchar *inbuf, int fmt)
 {
-#ifdef ENABLE_NLS
     gchar *outbuf = my_locale_from_utf8(inbuf);
-#else
-    gchar *outbuf = g_strdup(inbuf);
-#endif
 
     free(inbuf);
     inbuf = outbuf;
@@ -499,7 +504,7 @@ static void window_copy_or_save (windata_t *vwin, guint fmt, int action)
 	    int fcode = (fmt == GRETL_FORMAT_RTF_TXT)? 
 		SAVE_RTF : SAVE_OUTPUT;
 
-	    file_selector_with_parent(_("Save"), fcode, FSEL_DATA_PRN, 
+	    file_selector_with_parent(fcode, FSEL_DATA_PRN, 
 				      textprn, vwin->main);
 	}
 	gretl_print_destroy(textprn);
@@ -522,7 +527,8 @@ void window_save (windata_t *vwin, guint fmt)
 
 void window_print (GtkAction *action, windata_t *vwin) 
 {
-    char *buf, *selbuf = NULL;
+    gchar *buf, *selbuf = NULL;
+    const char *filename = NULL;
     GtkTextBuffer *tbuf;
     GtkTextIter start, end;
 
@@ -533,11 +539,63 @@ void window_print (GtkAction *action, windata_t *vwin)
 	selbuf = gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
     }
 
-    print_window_content(buf, selbuf);
+    if (vwin->role == EDIT_SCRIPT ||
+	vwin->role == VIEW_SCRIPT) {
+	const char *p = strrchr(vwin->fname, SLASH);
+	
+	if (p != NULL) {
+	    filename = p + 1;
+	} else {
+	    filename = vwin->fname;
+	}
+    }
+
+    print_window_content(buf, selbuf, filename);
 }
 
 #endif
 
+#define UTF_MINUS(u,i) (u[i] == 0xE2 && u[i+1] == 0x88 && u[i+2] == 0x92)
+
+/* check @s for UTF-8 minus signs (U+2212), and if any are found do an
+   in-place replacement with ASCII dashes
+*/
+
+char *strip_utf_minus (char *s)
+{
+    unsigned char *u = (unsigned char *) s;
+    int i, n = strlen(s);
+    int got_minus = 0;
+
+    for (i=0; i<n-3; i++) {
+	if (UTF_MINUS(u, i)) {
+	    got_minus = 1;
+	    break;
+	}
+    }
+
+    if (got_minus) {
+	char *tmp = calloc(n, 1);
+
+	if (tmp != NULL) {
+	    int j = 0;
+
+	    for (i=0; i<n; i++) {
+		if (i < n - 3 && UTF_MINUS(u, i)) {
+		    tmp[j++] = '-';
+		    i += 2;
+		} else {
+		    tmp[j++] = u[i];
+		}
+	    }
+	    strcpy(s, tmp);
+	    free(tmp);
+	} 
+    } 
+
+    return s;
+}
+
 /* print buf to file, trying to ensure it's not messed up */
 
 void system_print_buf (const gchar *buf, FILE *fp)
@@ -548,10 +606,10 @@ void system_print_buf (const gchar *buf, FILE *fp)
     while (*p) {
 	if (*p == '\r') {
 	    if (*(p+1) != '\n') {
-		putc('\n', fp);
+		fputc('\n', fp);
 	    } 
 	} else {
-	    putc(*p, fp);
+	    fputc(*p, fp);
 	}
 	cbak = *p;
 	p++;
@@ -559,7 +617,7 @@ void system_print_buf (const gchar *buf, FILE *fp)
 
     /* ensure file ends with newline */
     if (cbak != '\n') {
-	putc('\n', fp);
+	fputc('\n', fp);
     }
 }
 
