@@ -27,6 +27,7 @@
 #include "textbuf.h"
 #include "textutil.h"
 #include "series_view.h"
+#include "model_table.h"
 #include "cmdstack.h"
 #include "dlgutils.h"
 #include "fileselect.h"
@@ -47,6 +48,10 @@
 #include "../pixmaps/mini.manual.xpm"
 #include "../pixmaps/mini.pin.xpm"
 #include "../pixmaps/mini.alpha.xpm"
+#include "../pixmaps/mini.en.xpm"
+#include "../pixmaps/mini.split_h.xpm"
+#include "../pixmaps/mini.split_v.xpm"
+#include "../pixmaps/mini.compass.xpm"
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 8)
 # include "../pixmaps/info_24.xpm"
 #endif
@@ -85,7 +90,8 @@ enum {
     STICKIFY_ITEM,
     ALPHA_ITEM,
     REFRESH_ITEM,
-    OPEN_ITEM
+    OPEN_ITEM,
+    SPLIT_ITEM
 } viewbar_flags;
 
 static GtkIconFactory *gretl_stock_ifac;
@@ -113,7 +119,11 @@ void gretl_stock_icons_init (void)
 	mini_model_xpm,
 	mini_func_xpm,
 	mini_pin_xpm,
-	mini_alpha_xpm
+	mini_alpha_xpm,
+	mini_en_xpm,
+	mini_split_h_xpm,
+	mini_split_v_xpm,
+	mini_compass_xpm
     };
     const char *stocks[] = {
 #if NO_INFO_ICON
@@ -136,7 +146,11 @@ void gretl_stock_icons_init (void)
 	GRETL_STOCK_MODEL,
 	GRETL_STOCK_FUNC,
 	GRETL_STOCK_PIN,
-	GRETL_STOCK_ALPHA
+	GRETL_STOCK_ALPHA,
+	GRETL_STOCK_EN,
+	GRETL_STOCK_SPLIT_H,
+	GRETL_STOCK_SPLIT_V,
+	GRETL_STOCK_COMPASS
     };
     int n = G_N_ELEMENTS(stocks);
 
@@ -191,12 +205,14 @@ void save_as_callback (GtkWidget *w, windata_t *vwin)
     } else if (vwin->role == EDIT_SCRIPT ||
 	       vwin->role == VIEW_SCRIPT ||
 	       vwin->role == VIEW_LOG ||
-	       vwin->role == VIEW_FUNC_CODE) {
+	       vwin->role == VIEW_PKG_CODE) {
 	u = SAVE_SCRIPT;
     } else if (vwin->role == EDIT_GP) {
 	u = SAVE_GP_CMDS;
     } else if (vwin->role == EDIT_R) {
 	u = SAVE_R_CMDS;
+    } else if (vwin->role == EDIT_OX) {
+	u = SAVE_OX_CMDS;
     } else if (vwin->role == VIEW_FILE) {
 	u = SAVE_TEXT;
     } else {
@@ -207,40 +223,6 @@ void save_as_callback (GtkWidget *w, windata_t *vwin)
     file_save(vwin, u);
 }
 
-static int vwin_selection_present (gpointer p)
-{
-    windata_t *vwin = (windata_t *) p;
-    GtkTextBuffer *buf;
-    int ret = 0;
-
-    buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
-
-    if (gtk_text_buffer_get_selection_bounds(buf, NULL, NULL)) {
-	ret = 1;
-    }
-
-    return ret;
-}
-
-#define editor_role(r) (r == EDIT_SCRIPT || \
-                        r == EDIT_HEADER || \
-                        r == EDIT_NOTES || \
-                        r == EDIT_FUNC_CODE || \
-                        r == GR_PLOT)
-
-#define script_role(r) (r == VIEW_SCRIPT || r == VIEW_LOG)
-
-static void text_copy_callback (GtkWidget *w, windata_t *vwin)
-{
-    if (vwin_selection_present(vwin)) {
-	window_copy(vwin, GRETL_FORMAT_SELECTION);
-    } else if (!script_role(vwin->role) && !editor_role(vwin->role)) {
-	copy_format_dialog(vwin, W_COPY);
-    } else {
-	window_copy(vwin, GRETL_FORMAT_TXT);
-    }
-}
-
 static void mail_script_callback (GtkWidget *w, windata_t *vwin)
 {
     if (viewer_char_count(vwin) == 0) {
@@ -261,7 +243,7 @@ static void file_open_callback (GtkWidget *w, windata_t *vwin)
 	return;
     }
 
-    file_selector(_("Open script file"), OPEN_SCRIPT, FSEL_DATA_VWIN, vwin);
+    file_selector(OPEN_SCRIPT, FSEL_DATA_VWIN, vwin);
 
     if (vwin->flags & VWIN_CONTENT_CHANGED) {
 	mark_vwin_content_saved(vwin);
@@ -274,11 +256,6 @@ static void toolbar_new_callback (GtkWidget *w, windata_t *vwin)
     do_new_script(vwin->role);
 }
 
-static void save_plot_commands_callback (GtkWidget *w, windata_t *vwin)
-{
-    auto_save_plot(vwin);
-}
-
 #ifdef NATIVE_PRINTING
 static void window_print_callback (GtkWidget *w, windata_t *vwin)
 {
@@ -425,6 +402,41 @@ static void alpha_button_callback (GtkToggleButton *b, double *x)
     } 
 }
 
+static void reformat_callback (GtkWidget *w, windata_t *vwin)
+{
+    if (vwin->role == VIEW_MODELTABLE) {
+	format_model_table(vwin);
+    } else {
+	series_view_format_dialog(w, vwin);
+    }
+}
+
+static void split_pane_callback (GtkWidget *w, windata_t *vwin)
+{
+    GtkWidget *hb = g_object_get_data(G_OBJECT(w), "hpane");
+    GtkWidget *vb = g_object_get_data(G_OBJECT(w), "vpane");
+    int vertical = 0;
+
+    if (hb != NULL) {
+	vb = w;
+	vertical = 1;
+    } else {
+	hb = w;
+    }
+
+    if (g_object_get_data(G_OBJECT(vwin->vbox), "sw") != NULL) {
+	/* currently in single-view mode */
+	viewer_split_pane(vwin, vertical);
+	gtk_widget_set_sensitive(vb, vertical);
+	gtk_widget_set_sensitive(hb, !vertical);
+    } else if (g_object_get_data(G_OBJECT(vwin->vbox), "paned") != NULL) {
+	/* currently in split-view mode */
+	viewer_close_pane(vwin);
+	gtk_widget_set_sensitive(hb, TRUE);
+	gtk_widget_set_sensitive(vb, TRUE);
+    }
+}
+
 static void toggle_alpha_spin (GtkToggleButton *b, GtkWidget *w)
 {
     gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(b));
@@ -453,11 +465,12 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     hb2 = gtk_hbox_new(FALSE, 5);
     tmp = gtk_label_new(_("Confidence level"));
     gtk_box_pack_start(GTK_BOX(hb2), tmp, FALSE, FALSE, 0);
+
     tmp = gtk_label_new("1 - α :");
     gtk_box_pack_start(GTK_BOX(hb2), tmp, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), hb2, TRUE, TRUE, 10);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-		       hbox, FALSE, FALSE, 5);
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 
     hbox = gtk_hbox_new(FALSE, 5);
     vbox = gtk_vbox_new(FALSE, 5);
@@ -502,17 +515,20 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     gtk_box_pack_start(GTK_BOX(vbox), hb2, FALSE, FALSE, 0);
 
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 10);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-		       hbox, FALSE, FALSE, 0);
+
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
 
     /* Cancel button */
-    cancel_options_button(GTK_DIALOG(dialog)->action_area, dialog, NULL);
+    cancel_options_button(hbox, dialog, NULL);
 
     g_object_set_data(G_OBJECT(dialog), "vwin", vwin);
     g_object_set_data(G_OBJECT(dialog), "xptr", &x);
 
     /* "OK" button */
-    tmp = ok_button(GTK_DIALOG(dialog)->action_area);
+    tmp = ok_button(hbox);
     g_signal_connect(G_OBJECT(tmp), "clicked", 
 		     G_CALLBACK(real_coeffint_set_alpha), dialog);
     gtk_widget_grab_default(tmp);
@@ -539,6 +555,11 @@ static void set_output_sticky (GtkWidget *w, windata_t *vwin)
     }
 }
 
+static void compass_callback (GtkWidget *w, windata_t *vwin)
+{
+    window_list_popup(vwin->main);
+}
+
 static void activate_script_help (GtkWidget *widget, windata_t *vwin)
 {
     text_set_cursor(vwin->text, GDK_QUESTION_ARROW);
@@ -554,9 +575,7 @@ static void delete_file_viewer (GtkWidget *widget, windata_t *vwin)
 	return;
     }
 
-    if ((vwin->role == EDIT_SCRIPT || vwin->role == EDIT_HEADER ||
-	 vwin->role == EDIT_NOTES || vwin->role == GR_PLOT) &&
-	(vwin->flags & VWIN_CONTENT_CHANGED)) {
+    if (vwin_is_editing(vwin) && vwin_content_changed(vwin)) {
 	resp = query_save_text(NULL, NULL, vwin);
     }
 
@@ -586,13 +605,13 @@ static void set_plot_icon (GretlToolItem *item)
 static GretlToolItem viewbar_items[] = {
     { N_("New window"), GTK_STOCK_NEW, G_CALLBACK(toolbar_new_callback), OPEN_ITEM },
     { N_("Open..."), GTK_STOCK_OPEN, G_CALLBACK(file_open_callback), OPEN_ITEM },
-    { N_("Save"), GTK_STOCK_SAVE, G_CALLBACK(view_window_save), SAVE_ITEM },
+    { N_("Save"), GTK_STOCK_SAVE, G_CALLBACK(vwin_save_callback), SAVE_ITEM },
     { N_("Save as..."), GTK_STOCK_SAVE_AS, G_CALLBACK(save_as_callback), SAVE_AS_ITEM },
 #ifdef NATIVE_PRINTING
     { N_("Print..."), GTK_STOCK_PRINT, G_CALLBACK(window_print_callback), 0 },
 #endif
     { N_("Run"), GTK_STOCK_EXECUTE, G_CALLBACK(do_run_script), EXEC_ITEM },
-    { N_("Copy"), GTK_STOCK_COPY, G_CALLBACK(text_copy_callback), COPY_ITEM }, 
+    { N_("Copy"), GTK_STOCK_COPY, G_CALLBACK(vwin_copy_callback), COPY_ITEM }, 
     { N_("Paste"), GTK_STOCK_PASTE, G_CALLBACK(text_paste), EDIT_ITEM },
     { N_("Find..."), GTK_STOCK_FIND, G_CALLBACK(text_find), 0 },
     { N_("Replace..."), GTK_STOCK_FIND_AND_REPLACE, G_CALLBACK(text_replace), EDIT_ITEM },
@@ -606,38 +625,37 @@ static GretlToolItem viewbar_items[] = {
     { N_("Refresh"), GTK_STOCK_REFRESH, G_CALLBACK(cmd_log_refresh), REFRESH_ITEM },
     { N_("LaTeX"), GRETL_STOCK_TEX, G_CALLBACK(window_tex_callback), TEX_ITEM },
     { N_("Graph"), GRETL_STOCK_TS, G_CALLBACK(series_view_graph), PLOT_ITEM },
-    { N_("Reformat..."), GTK_STOCK_CONVERT, G_CALLBACK(series_view_format_dialog), FORMAT_ITEM },
+    { N_("Reformat..."), GTK_STOCK_CONVERT, G_CALLBACK(reformat_callback), FORMAT_ITEM },
     { N_("Add to dataset..."), GTK_STOCK_ADD, G_CALLBACK(add_data_callback), ADD_DATA_ITEM },
     { N_("Add as matrix..."), GTK_STOCK_ADD, G_CALLBACK(add_matrix_callback), ADD_MATRIX_ITEM },
     { N_("Stickiness..."), GRETL_STOCK_PIN, G_CALLBACK(set_output_sticky), STICKIFY_ITEM },
+    { N_("Toggle split pane"), GRETL_STOCK_SPLIT_H, G_CALLBACK(split_pane_callback), SPLIT_ITEM },
+    { N_("Toggle split pane"), GRETL_STOCK_SPLIT_V, G_CALLBACK(split_pane_callback), SPLIT_ITEM },
     { N_("Help on command"), GTK_STOCK_HELP, G_CALLBACK(activate_script_help), CMD_HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(window_help), HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_gnuplot_help), GP_HELP_ITEM },
+    { N_("Windows"), GRETL_STOCK_COMPASS, G_CALLBACK(compass_callback), 0 },
     { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_file_viewer), 0 }
 };
 
 static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 
-#define exec_ok(r) (r == EDIT_SCRIPT || \
-                    r == EDIT_GP || \
-                    r == EDIT_R || \
-	            r == VIEW_SCRIPT)
+#define exec_ok(r) (vwin_editing_script(r) || \
+		    r == VIEW_SCRIPT || \
+	            r == EDIT_PKG_SAMPLE)
 
-#define open_ok(r) (r == EDIT_SCRIPT || \
-                    r == EDIT_GP || \
-                    r == EDIT_R)
+#define open_ok(r) (vwin_editing_script(r))
 
-#define edit_ok(r) (r == EDIT_SCRIPT || \
-                    r == EDIT_HEADER || \
-                    r == EDIT_NOTES || \
-                    r == EDIT_FUNC_CODE || \
-	            r == EDIT_GP || \
-		    r == EDIT_R || \
-                    r == SCRIPT_OUT)
+#define edit_ok(r) (vwin_editing_script(r) || \
+		    vwin_editing_buffer(r) || \
+                    r == EDIT_PKG_CODE || \
+		    r == EDIT_PKG_SAMPLE)
 
 #define save_as_ok(r) (r != EDIT_HEADER && \
 	               r != EDIT_NOTES && \
-	               r != EDIT_FUNC_CODE)
+	               r != EDIT_PKG_CODE && \
+		       r != EDIT_PKG_SAMPLE && \
+	               r != CONSOLE)
 
 #define help_ok(r) (r == LEVERAGE || \
 		    r == COINT2 || \
@@ -646,16 +664,19 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 		    r == MAHAL)
 
 #define cmd_help_ok(r) (r == EDIT_SCRIPT || \
+	                r == EDIT_PKG_CODE || \
+                        r == EDIT_PKG_SAMPLE || \
 			r == VIEW_SCRIPT || \
 			r == VIEW_LOG)
 
 #define sort_ok(r)    (r == VIEW_SERIES)
-#define format_ok(r)  (r == VIEW_SERIES)
 #define plot_ok(r)    (r == VIEW_SERIES)
 
 #define add_data_ok(r) (r == PCA || r == LEVERAGE || \
                         r == MAHAL || r == FCAST)
 
+#define split_ok(r) (r == SCRIPT_OUT)
+
 /* Screen out unwanted menu items depending on the context; also
    adjust the callbacks associated with some items based on
    context.
@@ -663,7 +684,7 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 
 static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin, 
 				    int latex_ok, int sortby_ok,
-				    int format_ok)
+				    int format_ok, int save_ok)
 {
     GCallback func = item->func;
     int f = item->flag;
@@ -693,9 +714,12 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (!plot_ok(r) && f == PLOT_ITEM) {
 	return NULL;
+    } else if (!split_ok(r) && f == SPLIT_ITEM) {
+	return NULL;
     } else if (!format_ok && f == FORMAT_ITEM) {
 	return NULL;
-    } else if (r != EDIT_SCRIPT && r != EDIT_FUNC_CODE && f == EDIT_SCRIPT_ITEM) {
+    } else if (r != EDIT_SCRIPT && r != EDIT_PKG_CODE && 
+	       r != EDIT_PKG_SAMPLE && f == EDIT_SCRIPT_ITEM) {
 	return NULL;
     } else if (r != VIEW_SCRIPT && f == INDEX_ITEM) {
 	return NULL;
@@ -707,16 +731,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (r != EDIT_GP && f == GP_HELP_ITEM) {
 	return NULL;
-    } else if (f == SAVE_ITEM) { 
-	if (!edit_ok(r) || r == SCRIPT_OUT) {
-	    /* script output doesn't already have a filename */
-	    return NULL;
-	}
-	if (r == EDIT_HEADER || r == EDIT_NOTES) {
-	    func = G_CALLBACK(buf_edit_save);
-	} else if (r == GR_PLOT) {
-	    func = G_CALLBACK(save_plot_commands_callback);
-	}
+    } else if (f == SAVE_ITEM && !save_ok) {
+	return NULL;
     } else if (f == SAVE_AS_ITEM) {
 	if (!save_as_ok(r)) {
 	    return NULL;
@@ -788,11 +804,13 @@ GtkToolItem *gretl_toolbar_insert (GtkWidget *tbar,
     return button;
 }
 
-static void viewbar_add_items (windata_t *vwin)
+static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 {
     int sortby_ok = has_sortable_data(vwin);
     int format_ok = can_format_data(vwin);
     int latex_ok = latex_is_ok();
+    int save_ok = (flags & VIEWBAR_EDITABLE);
+    GtkWidget *hpane = NULL, *vpane = NULL;
     GtkToolItem *button;
     GretlToolItem *item;
     GCallback func;
@@ -802,7 +820,7 @@ static void viewbar_add_items (windata_t *vwin)
 	item = &viewbar_items[i];
 
 	func = item_get_callback(item, vwin, latex_ok, sortby_ok,
-				 format_ok);
+				 format_ok, save_ok);
 	if (func == NULL) {
 	    continue;
 	}
@@ -813,10 +831,20 @@ static void viewbar_add_items (windata_t *vwin)
 
 	button = gretl_toolbar_insert(vwin->mbar, item, func, vwin, -1);
 
+	if (func == (GCallback) split_pane_callback) {
+	    if (hpane == NULL) {
+		hpane = GTK_WIDGET(button);
+	    } else {
+		vpane = GTK_WIDGET(button);
+	    }
+	}
+
 	if (item->flag == SAVE_ITEM) { 
-	    g_object_set_data(G_OBJECT(vwin->mbar), "save_button", button); 
-	    /* nothing to save just yet */
-	    gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
+	    if (vwin->role != CONSOLE) {
+		/* nothing to save just yet */
+		g_object_set_data(G_OBJECT(vwin->mbar), "save_button", button); 
+		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
+	    }
 	} else if (item->flag == SAVE_AS_ITEM) {
 	    g_object_set_data(G_OBJECT(vwin->mbar), "save_as_button", button);
 	    if (strstr(vwin->fname, "script_tmp")) {
@@ -824,15 +852,17 @@ static void viewbar_add_items (windata_t *vwin)
 	    }
 	}
     }
+
+    if (hpane != NULL) {
+	g_object_set_data(G_OBJECT(hpane), "vpane", vpane);
+    }
+
+    if (vpane != NULL) {
+	g_object_set_data(G_OBJECT(vpane), "hpane", hpane);
+    }
 }
 
-/* "text_out": if this argument is non-zero, it means that vwin
-   carries text output which could appropriately be saved, either
-   to file or in the form of a chunk of text to be added as a
-   session icon.
-*/
-
-void vwin_add_viewbar (windata_t *vwin, int text_out)
+void vwin_add_viewbar (windata_t *vwin, ViewbarFlags flags)
 {
     GtkWidget *hbox;
 
@@ -840,7 +870,7 @@ void vwin_add_viewbar (windata_t *vwin, int text_out)
 	gretl_stock_icons_init();
     }
 
-    if (text_out || vwin->role == SCRIPT_OUT) {
+    if ((flags & VIEWBAR_HAS_TEXT) || vwin->role == SCRIPT_OUT) {
 	g_object_set_data(G_OBJECT(vwin->main), "text_out", GINT_TO_POINTER(1));
     }
 
@@ -849,7 +879,7 @@ void vwin_add_viewbar (windata_t *vwin, int text_out)
 
     vwin->mbar = gretl_toolbar_new();
 
-    viewbar_add_items(vwin);
+    viewbar_add_items(vwin, flags);
 
     gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
     gtk_widget_show_all(hbox);
@@ -864,20 +894,23 @@ void viewbar_add_edit_items (windata_t *vwin)
 {
     gtk_container_foreach(GTK_CONTAINER(vwin->mbar),
 			  (GtkCallback) remove_child, vwin->mbar);
-    viewbar_add_items(vwin);
+    viewbar_add_items(vwin, VIEWBAR_EDITABLE);
     gtk_widget_show_all(vwin->mbar);
 }
 
 GtkWidget *build_text_popup (windata_t *vwin)
 {
+    GtkWidget *pmenu = gtk_menu_new();
     GretlToolItem *item;
     GCallback func;
-    GtkWidget *pmenu = gtk_menu_new();
     GtkWidget *w;
     int i;
 
     for (i=0; i<n_viewbar_items; i++) {
 	item = &viewbar_items[i];
+	if (item->flag == SPLIT_ITEM) {
+	    continue;
+	}
 	if (vwin->role == EDIT_SCRIPT) {
 	    /* the script editor popup may have some special stuff
 	       added: don't clutter it up */
@@ -887,7 +920,7 @@ GtkWidget *build_text_popup (windata_t *vwin)
 		func = NULL;
 	    }
 	} else {
-	    func = item_get_callback(item, vwin, 0, 0, 0);
+	    func = item_get_callback(item, vwin, 0, 0, 0, 0);
 	}
 	if (func != G_CALLBACK(NULL)) {
 	    if (func == G_CALLBACK(text_paste)) {
@@ -906,6 +939,18 @@ GtkWidget *build_text_popup (windata_t *vwin)
 	}
     }
 
+    if (vwin->role != EDIT_SCRIPT) {
+	w = gtk_separator_menu_item_new();
+	gtk_widget_show(w);
+	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), w);
+
+	w = gtk_menu_item_new_with_label(_("Main window"));
+	g_signal_connect(G_OBJECT(w), "activate", 
+			 G_CALLBACK(raise_main_window), NULL);
+	gtk_widget_show(w);
+	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), w);
+    }
+
     return pmenu;
 }
 
@@ -989,7 +1034,7 @@ static GretlToolItem mainbar_items[] = {
 #else
     { N_("new script"),         GTK_STOCK_EDIT,      G_CALLBACK(tbar_new_script), 0 },
 #endif
-    { N_("open gretl console"), GRETL_STOCK_CONSOLE, G_CALLBACK(show_gretl_console), 0 },
+    { N_("open gretl console"), GRETL_STOCK_CONSOLE, G_CALLBACK(gretl_console), 0 },
     { N_("session icon view"),  GRETL_STOCK_ICONS,   G_CALLBACK(tbar_iconview), 0 },
     { N_("function packages"),  GRETL_STOCK_FUNC,    G_CALLBACK(tbar_show_funcs), 0 },
     { N_("user's guide"),       GRETL_STOCK_PDF,     G_CALLBACK(tbar_users_guide), 0 },
