@@ -22,50 +22,111 @@
 #include "gretl.h"
 #include "treeutils.h"
 #include "datafiles.h"
+#include "dlgutils.h"
 
-/* these live in dialogs.c */
-extern GtkWidget *active_edit_id; 
-extern GtkWidget *active_edit_name;
-extern GtkWidget *active_edit_text;
+/* special comparator which always preserves "const" (variable 0) in
+   the first position when sorting variables by name */
 
 static gint list_alpha_compare (GtkTreeModel *model, 
 				GtkTreeIter *a, GtkTreeIter *b,
 				gpointer p)
 {
-    gchar *vname_a, *vname_b;
-    gint ret;
+    gchar *str_a, *str_b;
+    gchar *id_a, *id_b;
+    GtkSortType order;
+    gint scol, ret;
 
-    gtk_tree_model_get(model, a, 1, &vname_a, -1);
-    gtk_tree_model_get(model, b, 1, &vname_b, -1);
+    gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(model),
+					 &scol, &order);
 
-    if (strcmp(vname_a, "const") == 0) {
-	ret = 0;
-    } else if (strcmp(vname_b, "const") == 0) {
-	ret = 1;
+    gtk_tree_model_get(model, a, 0, &id_a, scol, &str_a, -1);
+    gtk_tree_model_get(model, b, 0, &id_b, scol, &str_b, -1);
+
+    if (atoi(id_a) == 0) {
+	ret = (order == GTK_SORT_DESCENDING)? 1 : 0;
+    } else if (atoi(id_b) == 0) {
+	ret = (order == GTK_SORT_DESCENDING)? 0 : 1;
     } else {
-	ret = strcmp(vname_a, vname_b);
-    }
+	ret = strcmp(str_a, str_b);
+    }    
 
-    g_free(vname_a);
-    g_free(vname_b);
+    g_free(id_a);
+    g_free(id_b);
+
+    g_free(str_a);
+    g_free(str_b);
     
     return ret;
 }
 
+/* special comparator which preserves 0 in first position when sorting
+   variables by ID number */
+
 static gint list_id_compare (GtkTreeModel *model, 
 			     GtkTreeIter *a, GtkTreeIter *b,
 			     gpointer p)
 {
-    gchar *vnum_a, *vnum_b;
-    gint ret;
+    gchar *id_a, *id_b;
+    GtkSortType order;
+    gint ia, ib, scol, ret;
 
-    gtk_tree_model_get(model, a, 0, &vnum_a, -1);
-    gtk_tree_model_get(model, b, 0, &vnum_b, -1);
+    gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(model),
+					 &scol, &order);
+    
+    gtk_tree_model_get(model, a, 0, &id_a, -1);
+    gtk_tree_model_get(model, b, 0, &id_b, -1);
 
-    ret = strcmp(vnum_a, vnum_b);
+    ia = atoi(id_a);
+    ib = atoi(id_b);
 
-    g_free(vnum_a);
-    g_free(vnum_b);
+    if (ia == 0) {
+	ret = (order == GTK_SORT_DESCENDING)? 1 : 0;
+    } else if (ib == 0) {
+	ret = (order == GTK_SORT_DESCENDING)? 0 : 1;
+    } else {
+	ret = ia - ib;
+    }    
+
+    g_free(id_a);
+    g_free(id_b);
+    
+    return ret;
+}
+
+/* special comparator to put the series in a database window back into
+   "natural" order (by position in the database) in place of a
+   descending sort action in the first (series name) column
+*/
+
+static gint db_series_compare (GtkTreeModel *model, 
+			       GtkTreeIter *a, GtkTreeIter *b,
+			       GtkTreeViewColumn *col)
+{
+    GtkSortType order;
+    gint scol, ret;
+
+    gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(model),
+					 &scol, &order);
+
+    if (order == GTK_SORT_ASCENDING) {
+	/* alphabetize as usual */
+	gchar *sa, *sb;
+
+	gtk_tree_model_get(model, a, 0, &sa, -1);
+	gtk_tree_model_get(model, b, 0, &sb, -1);
+	ret = strcmp(sa, sb);
+	g_free(sa);
+	g_free(sb);
+    } else {
+	/* 'natural' order in place of descending sort:
+	   use the last (integer) column */
+	gint ia, ib;
+
+	gtk_tree_model_get(model, a, 3, &ia, -1);
+	gtk_tree_model_get(model, b, 3, &ib, -1);
+	ret = ib - ia;
+	gtk_tree_view_column_set_sort_indicator(col, FALSE);
+    }
     
     return ret;
 }
@@ -76,11 +137,7 @@ static gboolean no_select_row_zero (GtkTreeSelection *selection,
 				    gboolean path_currently_selected,
 				    gpointer data)
 {
-    if (tree_path_get_row_number(path) == 0) {
-	return FALSE;
-    }
-
-    return TRUE;
+    return tree_path_get_row_number(path) != 0;
 }
 
 static void get_selected_varnum (GtkTreeModel *model, GtkTreePath *path,
@@ -140,6 +197,9 @@ static void my_gtk_entry_append_text (GtkEntry *entry, gchar *add)
 
 static void update_dialogs_from_varclick (int active_var)
 {
+    GtkWidget *active_edit_id = get_active_edit_id();
+    GtkWidget *active_edit_name = get_active_edit_name();
+    GtkWidget *active_edit_text = get_active_edit_text();
     const gchar *edttext;
 
     if (active_edit_id != NULL) {
@@ -217,6 +277,7 @@ bool_col_toggled (GtkCellRendererToggle *cell, gchar *path_str, windata_t *vwin)
     gtk_tree_model_get(model, &iter, col, &val, -1);
 
     if (val) {
+	gtk_tree_path_free(path);
 	return;
     }
 
@@ -247,7 +308,7 @@ static gint catch_listbox_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	    return TRUE;
 	}
 
-	gdk_window_get_pointer(w->window, NULL, NULL, &mods); 
+	mods = widget_get_pointer_mask(w);
 	if (mods & GDK_CONTROL_MASK) {
 	    listbox_find(NULL, vwin);
 	    return TRUE;
@@ -307,6 +368,20 @@ static void check_db_series_selection (GtkTreeSelection *sel,
     }  
 }
 
+static void id_col_clicked (GtkTreeViewColumn *column, GtkWidget *view)
+{
+    GtkTreeModel *model;
+    GtkSortType order;
+    gint scol;
+
+    model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
+    gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(model),
+					 &scol, &order);
+    if (order == GTK_SORT_ASCENDING) {
+	gtk_tree_view_column_set_sort_indicator(column, FALSE);
+    }
+}
+
 #define db_series_window(v) (v->role == NATIVE_SERIES || \
                              v->role == RATS_SERIES || \
                              v->role == PCGIVE_SERIES || \
@@ -355,11 +430,9 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
 							      bool_renderer,
 							      "active", i,
 							      NULL);
-#if 1
 	    gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column),
 					    GTK_TREE_VIEW_COLUMN_FIXED);
 	    gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 50);
-#endif
 	    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
 	} else {
 	    column = gtk_tree_view_column_new_with_attributes(_(titles[i]),
@@ -367,9 +440,29 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
 							      "text", i, 
 							      NULL);
 	    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
+
+	    if ((vwin->role == FUNC_FILES || vwin->role == REMOTE_FUNC_FILES) && i == 1) {
+		; /* sorting functions by version number is not meaningful */
+	    } else {
+		gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), i);
+	    }
+
 	    if (vwin != mdata) {
 		g_object_set(G_OBJECT(column), "resizable", TRUE, NULL);
 	    }
+
+	    /* special actions on first column */
+	    if (i == 0) {
+		if (vwin == mdata) {
+		    g_signal_connect(G_OBJECT(column), "clicked",
+				     G_CALLBACK(id_col_clicked), view);
+		} else if (db_series_window(vwin)) {
+		    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(lstore), 0, 
+						    (GtkTreeIterCompareFunc) 
+						    db_series_compare,
+						    column, NULL);
+		}
+	    }
 	}	
     }
 
@@ -416,16 +509,19 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
 		     vwin);
 
     /* set sort properties on the tree model */
-    if (tstore != NULL) {
+    if (vwin == mdata && tstore != NULL) {
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(tstore), 0, 
 					(GtkTreeIterCompareFunc) 
 					list_id_compare,
 					NULL, NULL);
-
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(tstore), 1, 
 					(GtkTreeIterCompareFunc) 
 					list_alpha_compare,
 					NULL, NULL);
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(tstore), 2, 
+					(GtkTreeIterCompareFunc) 
+					list_alpha_compare,
+					NULL, NULL);
     } 
 
     scroller = gtk_scrolled_window_new(NULL, NULL);
@@ -442,6 +538,21 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
     gtk_widget_show(scroller);
 }
 
+void set_main_colheads_clickable (gboolean s)
+{
+    GtkTreeViewColumn *col;
+    int i;
+
+    for (i=0; i<3; i++) {
+	col = gtk_tree_view_get_column(GTK_TREE_VIEW(mdata->listbox), i);
+	gtk_tree_view_column_set_clickable(col, s);
+    }
+
+    if (!s) {
+	gtk_widget_grab_focus(mdata->listbox);
+    }
+}
+
 void tree_view_get_string (GtkTreeView *view, int row, int col, gchar **val)
 {
     GtkTreeModel *model;
@@ -501,6 +612,18 @@ int tree_path_get_row_number (GtkTreePath *path)
     return gtk_tree_path_get_indices(path)[0];
 }
 
+void tree_model_get_iter_last (GtkTreeModel *mod, GtkTreeIter *iter)
+{
+    GtkTreeIter first;
+    gboolean s;
+
+    s = gtk_tree_model_get_iter_first(mod, &first);
+    *iter = first;
+    while (s && gtk_tree_model_iter_next(mod, &first)) {
+	*iter = first;
+    }
+}
+
 static void add_to_selection_count (GtkTreeModel *model, GtkTreePath *path,
 				    GtkTreeIter *iter, int *count)
 {
