@@ -403,7 +403,9 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp, const char *fname,
 	fp = gretl_fopen(fname, "w");
     } else if (opt & OPT_B) {
 	/* batch mode: auto-named file */
-	fp = gnuplot_batch_init(&err);
+	const char *optname = get_optval_string(BXPLOT, OPT_U);
+
+	fp = gnuplot_batch_init(optname, &err);
 	if (!err) {
 	    *fmt = specified_gp_output_format();
 	    qtype = (*fmt)? 1 : 3;
@@ -767,13 +769,11 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 	if (bools[i-1] == NULL) {
 	    continue;
 	}
+
 	sprintf(formula, "bool_%d = %s", i-1, bools[i-1]);
 	err = generate(formula, pZ, pdinfo, OPT_P, NULL);
-	if (err) {
-	    gretl_errmsg_sprintf(_("boxplots: generation of dummy variable failed\n%s"),
-				 gretl_errmsg_get());
-	    err = 1;
-	} else {
+
+	if (!err) {
 	    for (t=0; t<n; t++) {
 		if ((*pZ)[k][t] == 1.0) {
 		    (*pZ)[k][t] = (*pZ)[list[i]][t];
