@@ -41,10 +41,10 @@ enum {
 };
 
 enum {
-    ADD_TEST,
-    OMIT_TEST,
-    OMIT_WALD,
-    ADD_WALD
+    COMPARE_ADD,
+    COMPARE_OMIT,
+    COMPARE_OMIT_WALD,
+    COMPARE_ADD_WALD
 };
 
 struct COMPARE {
@@ -355,10 +355,32 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
     } 
 
     if (verbosity > 0 && stat_ok) {
-	pputs(prn, _("\n  Null hypothesis: the regression parameters are "
-		     "zero for the variables\n\n"));
-	for (i=1; i<=diffvars[0]; i++) {
-	    pprintf(prn, "    %s\n", pdinfo->varname[diffvars[i]]);	
+	if (diffvars[0] == 1) {
+	    pputs(prn, "\n  ");
+	    pprintf(prn, _("Null hypothesis: the regression parameter is zero for %s"), 
+		    pdinfo->varname[diffvars[1]]);
+	    pputc(prn, '\n');
+	} else {
+	    const char *vname;
+	    int nc = 0;
+
+	    pputs(prn, _("\n  Null hypothesis: the regression parameters are "
+			 "zero for the variables\n"));
+	    pputs(prn, "    ");
+	    for (i=1; i<=diffvars[0]; i++) {
+		vname = pdinfo->varname[diffvars[i]];
+		nc += strlen(vname) + 2;
+		pprintf(prn, "%s", vname);
+		if (i < diffvars[0]) {
+		    if (nc > 60) {
+			pputs(prn, ",\n    ");
+			nc = 0;
+		    } else {
+			pputs(prn, ", ");
+		    }
+		}
+	    }
+	    pputc(prn, '\n');
 	}
     }
 
@@ -373,8 +395,11 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
     /* printout, if wanted */
 
     if (verbosity > 0) {
+	if (diffvars[0] > 1 || statcode == GRETL_STAT_WALD_CHISQ) {
+	    pputc(prn, '\n');
+	}
 	if (statcode == GRETL_STAT_WALD_CHISQ) {
-	    pprintf(prn, "\n  %s:\n    %s(%d) = %g, ",  _("Asymptotic test statistic"),
+	    pprintf(prn, "  %s:\n    %s(%d) = %g, ",  _("Asymptotic test statistic"),
 		    _("Wald chi-square"), cmp->dfn, testval);
 	    pprintf(prn, _("with p-value = %g\n"), pval);
 	    if (verbosity <= 1) {
@@ -389,14 +414,14 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 		pprintf(prn, _("with p-value = %g\n"), pval);
 	    }
 	} else if (statcode == GRETL_STAT_LR) {
-	    pprintf(prn, "\n  %s:\n    %s(%d) = %g, ",  _("Likelihood ratio test"),
+	    pprintf(prn, "  %s:\n    %s(%d) = %g, ",  _("Likelihood ratio test"),
 		    _("Chi-square"), cmp->dfn, cmp->LR);
 	    pprintf(prn, _("with p-value = %g\n"), pval);
 	    if (verbosity <= 1) {
 		pputc(prn, '\n');
 	    }
 	} else if (statcode == GRETL_STAT_F) {
-	    pprintf(prn, "\n  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
+	    pprintf(prn, "  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
 		    (cmp->robust)? _("Robust F") : "F",
 		    cmp->dfn, cmp->dfd, cmp->F);
 	    pprintf(prn, _("with p-value = %g\n"), pval);
@@ -436,14 +461,18 @@ gretl_make_compare (const struct COMPARE *cmp, const int *diffvars,
 	maybe_add_test_to_model(newmod, test);
     }
 
-    if (verbosity > 1 && cmp->score >= 0) {
-	pputc(prn, '\n');
-	pprintf(prn, _("  Of the %d model selection statistics, %d "), 
-		C_MAX, cmp->score);
-	if (cmp->score == 1) {
-	    pputs(prn, _("has improved.\n"));
+    if (verbosity > 1) {
+	if (cmp->score >= 0) {
+	    pprintf(prn, _("  Of the %d model selection statistics, %d "), 
+		    C_MAX, cmp->score);
+	    if (cmp->score == 1) {
+		pputs(prn, _("has improved.\n"));
+		pputc(prn, '\n');
+	    } else {
+		pputs(prn, _("have improved.\n\n"));
+	    }
 	} else {
-	    pputs(prn, _("have improved.\n\n"));
+	    pputc(prn, '\n');
 	}
     }
 }
@@ -458,7 +487,7 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 
     cmp.err = 0;
 
-    if (flag == ADD_TEST || flag == ADD_WALD) {
+    if (flag == COMPARE_ADD || flag == COMPARE_ADD_WALD) {
 	umod = pmodB;
 	rmod = pmodA;
 	cmp.cmd = ADD;
@@ -494,7 +523,7 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 
     cmp.dfd = umod->dfd;
 
-    if (flag == OMIT_WALD || flag == ADD_WALD) {
+    if (flag == COMPARE_OMIT_WALD || flag == COMPARE_ADD_WALD) {
 	cmp.err = wald_test(testvars, umod, &cmp.wald, &cmp.F);
     } else if (pmodA != NULL && (pmodA->opt & OPT_R)) {
 	cmp.F = wald_omit_F(testvars, umod);
@@ -507,7 +536,7 @@ add_or_omit_compare (MODEL *pmodA, MODEL *pmodB, int flag,
 	cmp.err = wald_test(testvars, umod, &cmp.wald, NULL);
     }
 
-    if (pmodA != NULL && pmodB != NULL && flag != ADD_WALD) {
+    if (pmodA != NULL && pmodB != NULL && flag != COMPARE_ADD_WALD) {
 	int miss = 0;
 
 	cmp.score = 0;
@@ -906,8 +935,8 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		       int aux_code, gretlopt opt, PRN *prn) 
 {
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int *tmplist = NULL;
     const int orig_nvar = pdinfo->v; 
     int err = 0;
@@ -955,8 +984,8 @@ int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     dataset_drop_last_variables(pdinfo->v - orig_nvar, pZ, pdinfo);
 
     /* put back into pdinfo what was there on input */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
 
     free(tmplist);
 
@@ -1014,8 +1043,8 @@ int add_test (const int *addvars, MODEL *orig, MODEL *pmod,
 	      gretlopt opt, PRN *prn)
 {
     gretlopt est_opt = opt;
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int *tmplist = NULL;
     const int orig_nvar = pdinfo->v; 
     int err = 0;
@@ -1077,7 +1106,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *pmod,
     }
 
     if (!err) {
-	int flag = ADD_WALD;
+	int flag = COMPARE_ADD_WALD;
 	struct COMPARE cmp;
 	MODEL *origmod = orig;
 	int *addlist;
@@ -1089,7 +1118,7 @@ int add_test (const int *addvars, MODEL *orig, MODEL *pmod,
 	}
 
 	if (orig->ci == OLS && pmod->nobs == orig->nobs) {
-	    flag = ADD_TEST;
+	    flag = COMPARE_ADD;
 	} else if (opt & OPT_B) {
 	    /* ivreg: adding as both regressor and instrument */
 	    origmod = NULL;
@@ -1107,8 +1136,8 @@ int add_test (const int *addvars, MODEL *orig, MODEL *pmod,
     dataset_drop_last_variables(pdinfo->v - orig_nvar, pZ, pdinfo);
 
     /* put back into pdinfo what was there on input */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
 
     free(tmplist);
 
@@ -1139,7 +1168,7 @@ static int wald_omit_test (const int *list, MODEL *pmod,
 
     free(test);
 
-    cmp = add_or_omit_compare(pmod, NULL, OMIT_WALD, list);
+    cmp = add_or_omit_compare(pmod, NULL, COMPARE_OMIT_WALD, list);
     if (cmp.err) {
 	return cmp.err;
     }
@@ -1149,14 +1178,47 @@ static int wald_omit_test (const int *list, MODEL *pmod,
     return err;
 }
 
-/* determine if a model contains a variable with p-value
-   greater than some cutoff alpha_max; and if so, remove
-   this variable from the regression list 
+/* Check whether coefficient @i corresponds to a variable
+   that is removable from the model: this is the case if either
+   (a) the @cands list is NULL, or (b) coefficient @i is the
+   coefficient on one of the variables in @cands.
 */
 
-static int auto_drop_var (const MODEL *pmod, int *list,
-			  DATAINFO *pdinfo, double alpha_max,
-			  int d0, PRN *prn, int *err)
+static int coeff_is_removable (const int *cands, const MODEL *pmod,
+			       DATAINFO *pdinfo, int i)
+{
+    int ret = 1;
+
+    if (cands != NULL) {
+	const char *vname;
+	int j, pj;
+
+	ret = 0; /* reverse the presumption */
+
+	for (j=1; j<=cands[0]; j++) {
+	    vname = pdinfo->varname[cands[j]];
+	    pj = gretl_model_get_param_number(pmod, pdinfo, vname);
+	    if (pj == i) {
+		ret = 1;
+		break;
+	    }
+	}
+    }
+
+    return ret;
+}
+
+/* Determine if a model contains a variable with p-value
+   greater than some cutoff alpha_max; and if so, remove
+   this variable from the regression list.  If the list
+   @cands is non-NULL then confine the search to candidate
+   variables in that list.
+*/
+
+static int auto_drop_var (const int *cands, const MODEL *pmod, 
+			  int *list, DATAINFO *pdinfo, 
+			  double alpha_max, int d0, 
+			  PRN *prn, int *err)
 {
     double tstat, pv = 0.0, tmin = 4.0;
     int i, k = -1;
@@ -1167,10 +1229,12 @@ static int auto_drop_var (const MODEL *pmod, int *list,
     }
     
     for (i=pmod->ifc; i<pmod->ncoeff; i++) {
-	tstat = fabs(pmod->coeff[i] / pmod->sderr[i]);
-	if (tstat < tmin) {
-	    tmin = tstat;
-	    k = i;
+	if (coeff_is_removable(cands, pmod, pdinfo, i)) {
+	    tstat = fabs(pmod->coeff[i] / pmod->sderr[i]);
+	    if (tstat < tmin) {
+		tmin = tstat;
+		k = i;
+	    }
 	}
     }
 
@@ -1209,10 +1273,12 @@ static void list_copy_values (int *targ, const int *src)
 /* run a loop in which the least significant variable is dropped
    from the regression list, provided its p-value exceeds some
    specified cutoff.  FIXME this probably still needs work for 
-   estimators other than OLS.
+   estimators other than OLS.  If @omitlist is non-NULL the
+   routine is confined to members of the list.
 */
 
-static int auto_omit (MODEL *orig, MODEL *pmod, 
+static int auto_omit (const int *omitlist, 
+		      MODEL *orig, MODEL *pmod, 
 		      double ***pZ, DATAINFO *pdinfo, 
 		      gretlopt est_opt, gretlopt opt,
 		      PRN *prn)
@@ -1231,7 +1297,7 @@ static int auto_omit (MODEL *orig, MODEL *pmod,
 	amax = 0.10;
     }
 
-    if (!auto_drop_var(orig, tmplist, pdinfo, amax, 1, prn, &err)) {
+    if (!auto_drop_var(omitlist, orig, tmplist, pdinfo, amax, 1, prn, &err)) {
 	free(tmplist);
 	return (err)? err : E_NOOMIT;
     }    
@@ -1247,7 +1313,8 @@ static int auto_omit (MODEL *orig, MODEL *pmod,
 	    fprintf(stderr, "auto_omit: error %d from replicate_estimator\n", err);
 	} else {
 	    list_copy_values(tmplist, pmod->list);
-	    if (auto_drop_var(pmod, tmplist, pdinfo, amax, 0, prn, &err)) {
+	    if (auto_drop_var(omitlist, pmod, tmplist, pdinfo, 
+			      amax, 0, prn, &err)) {
 		model_count_minus();
 		clear_model(pmod);
 	    } else {
@@ -1349,8 +1416,8 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *pmod,
 	       gretlopt opt, PRN *prn)
 {
     gretlopt est_opt = opt;
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int omitlast = 0;
     int *tmplist = NULL;
     int err = 0;
@@ -1395,7 +1462,11 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *pmod,
     remove_special_flags(&est_opt);
 
     if (opt & OPT_A) {
-	err = auto_omit(orig, pmod, pZ, pdinfo, est_opt, opt, prn);
+	if (omitvars != NULL && omitvars[0] > 0) {
+	    err = auto_omit(omitvars, orig, pmod, pZ, pdinfo, est_opt, opt, prn);
+	} else {
+	    err = auto_omit(NULL, orig, pmod, pZ, pdinfo, est_opt, opt, prn);
+	}
     } else {
 	*pmod = replicate_estimator(orig, &tmplist, pZ, pdinfo, est_opt, prn);
 	err = pmod->errcode;
@@ -1413,7 +1484,7 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *pmod,
 	}	
 
 	if (!omitlast) {
-	    int flag = OMIT_TEST;
+	    int flag = COMPARE_OMIT;
 	    MODEL *newmod = pmod;
 	    struct COMPARE cmp;
 	    int *omitlist;
@@ -1421,7 +1492,7 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *pmod,
 	    if (opt & OPT_B) {
 		/* ivreg: omitting both as regressor and instrument */
 		newmod = NULL;
-		flag = OMIT_WALD;
+		flag = COMPARE_OMIT_WALD;
 	    } 
 
 	    omitlist = gretl_list_diff_new(orig->list, pmod->list, 2);
@@ -1438,8 +1509,8 @@ int omit_test (const int *omitvars, MODEL *orig, MODEL *pmod,
     }
 
     /* put back into pdinfo what was there on input */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
 
     free(tmplist);
 
@@ -1450,8 +1521,8 @@ double get_dw_pvalue (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		      int *err)
 {
     MODEL dwmod;
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int *list = NULL;
     double pv = NADBL;
 
@@ -1489,8 +1560,8 @@ double get_dw_pvalue (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* put back into pdinfo what was there on input */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
     
     clear_model(&dwmod);
     free(list);
@@ -1516,9 +1587,11 @@ double get_dw_pvalue (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
 		gretlopt opt, PRN *prn)
 {
-    int *newlist;
+    int *newlist = NULL;
     MODEL aux;
     double RF;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int i, t, v = pdinfo->v; 
     int addcols;
     const char *mode;
@@ -1551,14 +1624,20 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	mode = N_("squares and cubes");
     }
 
+    impose_model_smpl(pmod, pdinfo);
+
     if (pmod->ncoeff + addcols >= pdinfo->t2 - pdinfo->t1) {
-	return E_DF;
+	err = E_DF;
     }
 
-    newlist = malloc((pmod->list[0] + addcols + 1) * sizeof *newlist);
-    if (newlist == NULL) {
-	err = E_ALLOC;
-    } else {
+    if (!err) {
+	newlist = malloc((pmod->list[0] + addcols + 1) * sizeof *newlist);
+	if (newlist == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (newlist != NULL) {
 	newlist[0] = pmod->list[0] + addcols;
 	for (i=1; i<=pmod->list[0]; i++) {
 	    newlist[i] = pmod->list[i];
@@ -1653,6 +1732,9 @@ int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     dataset_drop_last_variables(addcols, pZ, pdinfo); 
     clear_model(&aux); 
 
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
+
     return err;
 }
 
@@ -1802,7 +1884,7 @@ static int ivreg_autocorr_test (MODEL *pmod, int order,
 	    lnum = laggenr(v, i, pZ, pdinfo);
 
 	    if (lnum < 0) {
-		sprintf(gretl_errmsg, _("lagging uhat failed"));
+		gretl_errmsg_set(_("lagging uhat failed"));
 		err = E_LAGS;
 	    } else {
 		/* set the first entries to 0 for compatibility with Godfrey (1994)
@@ -1899,8 +1981,8 @@ int autocorr_test (MODEL *pmod, int order,
 		   double ***pZ, DATAINFO *pdinfo, 
 		   gretlopt opt, PRN *prn)
 {
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int *newlist = NULL;
     MODEL aux;
     double RSSxe, RSSx = pmod->ess;
@@ -1934,7 +2016,7 @@ int autocorr_test (MODEL *pmod, int order,
 	order = pdinfo->pd;
     }
 
-    if (pmod->ncoeff + order >= pdinfo->t2 - pdinfo->t1) {
+    if (pmod->ncoeff + order >= pmod->t2 - pmod->t1) {
 	return E_DF;
     }
 
@@ -2057,8 +2139,8 @@ int autocorr_test (MODEL *pmod, int order,
     clear_model(&aux); 
 
     /* reset sample as it was */
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
 
     return err;
 }
@@ -2351,8 +2433,8 @@ static int get_chow_dummy (const char *s, const double **Z,
 int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	       DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
-    int smpl_t1 = pdinfo->t1;
-    int smpl_t2 = pdinfo->t2;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int *chowlist = NULL;
     int origv = pdinfo->v;
     char chowparm[VNAMELEN];
@@ -2387,7 +2469,7 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	} else {
 	    split = dateton(chowparm, pdinfo);
 	    if (split <= 0 || split >= pdinfo->n) { 
-		strcpy(gretl_errmsg, _("Invalid sample split for Chow test"));
+		gretl_errmsg_set(_("Invalid sample split for Chow test"));
 		err = E_DATA;
 	    }
 	    smax = split;
@@ -2468,10 +2550,13 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 	if (chow_mod.errcode) {
 	    err = chow_mod.errcode;
 	    errmsg(err, prn);
+	} else if (chow_mod.ncoeff <= pmod->ncoeff) {
+	    err = chow_mod.errcode = E_DATA;
+	    errmsg(err, prn);
 	} else {
 	    int dfd = (robust)? 0 : chow_mod.dfd;
 	    int dfn = chow_mod.ncoeff - pmod->ncoeff;
-	    double test, pval;
+	    double test, pval = NADBL;
 
 	    if (!(opt & OPT_Q)) {
 		chow_mod.aux = AUX_CHOW;
@@ -2480,12 +2565,18 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 
 	    if (robust) {
 		test = robust_chow_test(&chow_mod, pmod->list, &err);
+		if (!na(test)) {
+		    pval = chisq_cdf_comp(dfn, test);
+		}
 	    } else {
 		test = (pmod->ess - chow_mod.ess) * dfd / 
 		    (chow_mod.ess * dfn);
+		if (!na(test)) {
+		    pval = snedecor_cdf_comp(dfn, dfd, test);
+		}
 	    }
 
-	    if (!na(test)) {
+	    if (!na(test) && !na(pval)) {
 		if (opt & OPT_Q) {
 		    pputc(prn, '\n');
 		}
@@ -2497,21 +2588,22 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
 			    chowparm);
 		} 
 		pputc(prn, '\n');
+
 		if (robust) {
-		    pval = chisq_cdf_comp(dfn, test);
 		    pprintf(prn, "  %s(%d) = %g %s %.4f\n", _("Chi-square"),
 			    dfn, test, _("with p-value"), pval);
 		    pprintf(prn, "  %s: F(%d, %d) = %g %s %.4f\n\n", _("F-form"), 
 			    dfn, chow_mod.dfd, test / dfn, _("with p-value"),
 			    snedecor_cdf_comp(dfn, chow_mod.dfd, test / dfn));
 		} else {
-		    pval = snedecor_cdf_comp(dfn, dfd, test);
 		    pprintf(prn, "  F(%d, %d) = %g %s %.4f\n\n", 
 			    dfn, dfd, test, _("with p-value"), pval);
 		}
+
 		if (opt & OPT_S) {
 		    save_chow_test(pmod, chowparm, dumv, test, pval, dfn, dfd);
 		}
+
 		record_test_result(test, pval, "Chow");
 	    } 
 	}
@@ -2524,8 +2616,8 @@ int chow_test (const char *line, MODEL *pmod, double ***pZ,
     dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
     free(chowlist);
 
-    pdinfo->t1 = smpl_t1;
-    pdinfo->t2 = smpl_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
 
     return err;
 }
@@ -2720,9 +2812,9 @@ static void cusum_harvey_collier (double wbar, double sigma, int m,
 int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
 		gretlopt opt, PRN *prn) 
 {
-    int orig_t1 = pdinfo->t1;
-    int orig_t2 = pdinfo->t2;
-    int T = pmod->t2 - pmod->t1 + 1;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
+    int T = pmod->nobs;
     int k = pmod->ncoeff;
     char cumdate[OBSLEN];
     double wbar = 0.0;
@@ -2848,8 +2940,8 @@ int cusum_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* restore original sample range */
-    pdinfo->t1 = orig_t1;
-    pdinfo->t2 = orig_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
     
     free(cresid);
     free(W);
@@ -2876,8 +2968,8 @@ int comfac_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 		 gretlopt opt, PRN *prn)
 {
     MODEL cmod;
-    int orig_t1 = pdinfo->t1;
-    int orig_t2 = pdinfo->t2;
+    int save_t1 = pdinfo->t1;
+    int save_t2 = pdinfo->t2;
     int v = pdinfo->v;
     int *biglist = NULL;
     int clearit = 0;
@@ -3017,8 +3109,8 @@ int comfac_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     dataset_drop_last_variables(nadd, pZ, pdinfo);   
     free(biglist);
 
-    pdinfo->t1 = orig_t1;
-    pdinfo->t2 = orig_t2;
+    pdinfo->t1 = save_t1;
+    pdinfo->t2 = save_t2;
 
     return err;
 }
@@ -3363,7 +3455,7 @@ int model_test_driver (const char *param,
     /* groupwise heteroskedasticity */
     if (!err && (opt & OPT_P)) {
 	if (type == GRETL_OBJ_EQN) {
-	    err = groupwise_hetero_test(ptr, pZ, pdinfo, prn);
+	    err = groupwise_hetero_test(ptr, pdinfo, testopt, prn);
 	} else {
 	    err = E_NOTIMP;
 	}
