@@ -310,15 +310,13 @@ static int ascii_plot (const int *list, const double **Z,
     int i, nc2, vy, vz, cntrline;
     int ix = 0, iy = 0, iz = 0, n = 0;
     int ncols = 70;
-    int ls, lineno, t, t1 = pdinfo->t1, t2 = pdinfo->t2;
+    int ls, t, t1 = pdinfo->t1, t2 = pdinfo->t2;
     char word[32], px[132];
     char s1[10], s2[10];
     double xmin, xmax, xrange, ymin, ymax, yrange, xymin, xymax; 
     double xyrange, xx, yy;
     double *x, *y;
 
-    int pause = gretl_get_text_pause();
-
     x = malloc(pdinfo->n * sizeof *x);
     y = malloc(pdinfo->n * sizeof *y);
 
@@ -361,16 +359,11 @@ static int ascii_plot (const int *list, const double **Z,
 
 	gretl_push_c_numeric_locale();
 
-	lineno = 1;
 	for (t=t1; t<=t2; ++t) {
 	    xx = Z[vy][t];
 	    if (na(xx)) {
 		continue;
 	    }
-	    if (pause && (lineno % PAGELINES == 0)) {
-		scroll_pause();
-		lineno = 1;
-	    }
 	    prntdate(t, n, pdinfo, prn);
 	    ix = (floatneq(xrange, 0.0))? ((xx-xmin)/xrange) * ncols : nc2;
 	    initpx(ncols, px);
@@ -384,7 +377,6 @@ static int ascii_plot (const int *list, const double **Z,
 	    if (ix == ncols) {
 		pputc(prn, '\n');
 	    }
-	    lineno++;
 	}
 
 	gretl_pop_c_numeric_locale();
@@ -414,7 +406,6 @@ static int ascii_plot (const int *list, const double **Z,
     /* print headings for the two variables */
     pprintf(prn, _("%17cNOTE: o stands for %s,   x stands for %s\n%17c+ means %s "
 	   "and %s are equal when scaled\n"), ' ', s1, s2, ' ', s1, s2);
-    lineno = 6;
 
     if (opt & OPT_O) {
 	pprintf(prn, _("%20c%s and %s are plotted on same scale\n\n%8c"),
@@ -470,13 +461,7 @@ static int ascii_plot (const int *list, const double **Z,
 
     gretl_push_c_numeric_locale();
 
-    lineno = 1;
-
     for (t=t1; t<=t2; ++t) {
-	if (pause && (lineno % PAGELINES == 0)) {
-	    scroll_pause();
-	    lineno = 1;
-	}
 
 	xx = Z[vy][t];
 	yy = Z[vz][t];
@@ -515,8 +500,6 @@ static int ascii_plot (const int *list, const double **Z,
 	if (ix == ncols || iy == ncols) {
 	    pputc(prn, '\n');
 	}
-
-	lineno++;
     }
 
     gretl_pop_c_numeric_locale();
@@ -549,7 +532,7 @@ static int ascii_plot (const int *list, const double **Z,
 int ascii_graph (const int *list, const double **Z, const DATAINFO *pdinfo, 
 		 gretlopt opt, PRN *prn)
 {
-    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int T = sample_size(pdinfo);
     int vx, vy1, vy2 = -1;
     double *x = NULL;
     double *y1 = NULL;
@@ -608,107 +591,3 @@ int ascii_graph (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     return err;
 }
-
-#undef RHODEBUG
-
-/**
- * rhodiff:
- * @param: please see the gretl help on rhodiff() for syntax.
- * @list: list of variables to process.
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- *
- * Generates and adds to the data set rho-differenced versions
- * of the variables given in @list.
- *
- * Returns: 0 on successful completion, error code on error.
- */
-
-int rhodiff (char *param, const int *list, double ***pZ, DATAINFO *pdinfo)
-{
-    int i, j, maxlag, p, t, t1;
-    int v = pdinfo->v, n = pdinfo->n;
-    char parmbit[VNAMELEN];
-    double *rhot;
-    int err = 0;
-
-#ifdef RHODEBUG
-    fprintf(stderr, "rhodiff: param = '%s'\n", param);
-#endif
-
-    maxlag = count_fields(param);
-    rhot = malloc(maxlag * sizeof *rhot);
-    if (rhot == NULL) {
-	return E_ALLOC;
-    }
-
-    if (maxlag > pdinfo->t1) {
-	t1 = maxlag;
-    } else {
-	t1 = pdinfo->t1;
-    }
-
-#ifdef RHODEBUG
-    fprintf(stderr, "rhodiff: maxlag = %d, t1 = %d\n", maxlag, t1);
-#endif
-
-    /* parse "param" string */
-    j = strlen(param);
-    p = 0;
-    for (i=0; i<j; i++) {
-	if ((i == 0 || param[i] == ' ') && i < (j - 1)) {
-	    sscanf(param + i + (i? 1 : 0), "%15s", parmbit); 
-#ifdef RHODEBUG
-	    fprintf(stderr, "rhodiff: parmbit = '%s'\n", parmbit);
-#endif
-	    rhot[p] = gretl_double_from_string(parmbit, &err);
-	    if (err) {
-		return err;
-	    }
-	    p++;
-	}
-    }
-
-    if (dataset_add_series(list[0], pZ, pdinfo)) {
-	return E_ALLOC;
-    }
-
-    for (i=1; i<=list[0]; i++) {
-	int vr = v + i - 1;
-	double xx;
-
-	j = list[i];
-
-	*pdinfo->varname[vr] = 0;
-	strncat(pdinfo->varname[vr], pdinfo->varname[j], VNAMELEN-3);
-	strcat(pdinfo->varname[vr], "_r");
-
-	sprintf(VARLABEL(pdinfo, vr), _("= rho-differenced %s"),
-		pdinfo->varname[j]);
-
-	for (t=0; t<n; t++) {
-	    if (t < t1 || t > pdinfo->t2) {
-		(*pZ)[vr][t] = NADBL;
-		continue;
-	    }
-	    xx = (*pZ)[j][t];
-	    if (na(xx)) {
-		(*pZ)[vr][t] = NADBL;
-		continue;
-	    }
-	    for (p=0; p<maxlag; p++) {
-		if (na((*pZ)[j][t-p-1])) {
-		    xx = NADBL;
-		    break;
-		} else {
-		    xx -= rhot[p] * (*pZ)[j][t-p-1];
-		}
-	    }
-	    (*pZ)[vr][t] = xx;
-	}
-    }
-
-    free(rhot);
-
-    return 0;
-}
