@@ -424,22 +424,22 @@ static int check_daily_dates (DATAINFO *pdinfo, int *pd, int *reversed, PRN *prn
 	    err = 1;
 	} else {
 	    nmiss = fulln - T;
-	    pprintf(prn, "Observations: %d; days in sample: %d\n", 
+	    pprintf(prn, M_("Observations: %d; days in sample: %d\n"), 
 		    T, fulln);
 	    if (nmiss > 300 * T) {
-		pprintf(prn, "Probably annual data\n");
+		pprintf(prn, M_("Probably annual data\n"));
 		*pd = 1;
 	    } else if (nmiss > 50 * T) {
-		pprintf(prn, "Probably quarterly data\n");
+		pprintf(prn, M_("Probably quarterly data\n"));
 		*pd = 4;
 	    } else if (nmiss > 20 * T) {
-		pprintf(prn, "Probably monthly data\n");
+		pprintf(prn, M_("Probably monthly data\n"));
 		*pd = 12;
 	    } else if (nmiss > 3 * T) {
-		pprintf(prn, "Probably weekly data\n");
+		pprintf(prn, M_("Probably weekly data\n"));
 		*pd = pdinfo->pd = 52;
 	    } else {
-		pprintf(prn, "Missing daily observations: %d\n", nmiss);
+		pprintf(prn, M_("Missing daily observations: %d\n"), nmiss);
 	    }
 	}
     }
@@ -730,7 +730,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, int *reversed,
     tryagain:
 
 	if (dorder == YYYYMMDD) {
-	    pputs(prn, "Trying date order YYYYMMDD\n");
+	    pputs(prn, M_("Trying date order YYYYMMDD\n"));
 	    yr1 = d1[0];
 	    mon1 = d1[1];
 	    day1 = d1[2];
@@ -738,7 +738,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, int *reversed,
 	    mon2 = d2[1];
 	    day2 = d2[2];
 	} else if (dorder == DDMMYYYY) {
-	    pputs(prn, "Trying date order DDMMYYYY\n");
+	    pputs(prn, M_("Trying date order DDMMYYYY\n"));
 	    day1 = d1[0];
 	    mon1 = d1[1];
 	    yr1 = d1[2];
@@ -746,7 +746,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, int *reversed,
 	    mon2 = d2[1];
 	    yr2 = d2[2];
 	} else {
-	    pputs(prn, "Trying date order MMDDYYYY\n");
+	    pputs(prn, M_("Trying date order MMDDYYYY\n"));
 	    mon1 = d1[0];
 	    day1 = d1[1];
 	    yr1 = d1[2];
@@ -765,7 +765,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, int *reversed,
 		    return -1;
 		}
 	    }
-	    pprintf(prn, "Could be %s - %s\n", lbl1, lbl2);
+	    pprintf(prn, M_("Could be %s - %s\n"), lbl1, lbl2);
 	    ret = check_daily_dates(pdinfo, &pd, reversed, prn);
 	    if (ret >= 0 && pd > 0) {
 		if (pd == 52) {
@@ -790,7 +790,7 @@ csv_daily_date_check (double ***pZ, DATAINFO *pdinfo, int *reversed,
 	    return ret;
 	}
     } else {
-	pprintf(prn, "'%s' and '%s': couldn't get dates\n", lbl1, lbl2);
+	pprintf(prn, M_("'%s' and '%s': couldn't get dates\n"), lbl1, lbl2);
     }
 
     return -1;
@@ -920,6 +920,85 @@ static int dates_maybe_reversed (const char *s1, const char *s2,
     return ret;
 }
 
+/* e.g. "M1 1957", "M12 2009", with spaces removed */
+
+static int fix_IFS_data_labels (DATAINFO *pdinfo)
+{
+    char *s1 = pdinfo->S[0];
+    char *s2 = pdinfo->S[pdinfo->n - 1];
+    int ret = 0;
+
+    if ((*s1 == 'M' || *s1 == 'Q') && *s2 == *s1) {
+	const char *dig = "0123456789";
+	int n1 = strlen(s1);
+	int n2 = strlen(s2);
+
+	if ((n1 == 6 || n1 == 7) && (n2 == 6 || n2 == 7) &&
+	    strspn(s1 + 1, dig) == n1 - 1 &&
+	    strspn(s2 + 1, dig) == n2 - 1) {
+	    char sp[3], tmp[8], *s;
+	    int pmax = (*s1 == 'M')? 12 : 4;
+	    int y, p, pbak = 0;
+	    int i, n, doit = 1;
+
+	    for (i=0; i<pdinfo->n; i++) {
+		s = pdinfo->S[i];
+		if (*s != *s1) {
+		    doit = 0;
+		    break;
+		}
+		n = strlen(s);
+		if (n != 6 && n != 7) {
+		    doit = 0;
+		    break;
+		}
+		if (strspn(s + 1, dig) != n - 1) {
+		    doit = 0;
+		    break;
+		}
+		y = atoi(s + n - 4);
+		*sp = '\0';
+		strncat(sp, s + 1, n - 5);
+		p = atoi(sp);
+		if (y < 1800 || y > 2500 || p <= 0 || p > pmax) {
+		    doit = 0;
+		    break;
+		} 
+		if (i > 0 && p != pbak + 1 && p != 1) {
+		    doit = 0;
+		    break;
+		}		    
+		pbak = p;
+	    }
+
+	    if (doit) {
+		for (i=0; i<pdinfo->n; i++) {
+		    s = pdinfo->S[i];
+		    n = strlen(s);
+		    y = atoi(s + n - 4);
+		    *sp = '\0';
+		    strncat(sp, s + 1, n - 5);
+		    p = atoi(sp);
+		    if (pmax == 12) {
+			sprintf(tmp, "%d:%02d", y, p);
+		    } else {
+			sprintf(tmp, "%d:%d", y, p);
+		    }
+		    if (strlen(tmp) > strlen(s)) {
+			free(s);
+			pdinfo->S[i] = gretl_strdup(tmp);
+		    } else {
+			strcpy(s, tmp);
+		    }		    
+		}
+		ret = 1;
+	    }
+	}
+    }
+
+    return ret;
+}
+
 /* Attempt to parse csv row labels as dates.  Return -1 if this
    doesn't work out, or 0 if the labels seem to be just integer
    observation numbers, else return the inferred data frequency.  The
@@ -951,6 +1030,12 @@ int test_markers_for_dates (double ***pZ, DATAINFO *pdinfo,
 	return 0;
     }
 
+    if (fix_IFS_data_labels(pdinfo)) {
+	lbl1 = pdinfo->S[0];
+	lbl2 = pdinfo->S[n - 1];
+	len1 = strlen(lbl1);
+    }
+
     /* labels are of different lengths? */
     if (len1 != strlen(lbl2)) {
 	pputs(prn, M_("   label strings can't be consistent dates\n"));
@@ -1265,7 +1350,7 @@ static int non_numeric_check (csvdata *c, PRN *prn)
 	    int v = list[i];
 	    int cv = is_codevar(c->dinfo->varname[v]);
 
-	    set_var_discrete(c->dinfo, v, 1);
+	    series_set_flag(c->dinfo, v, VAR_DISCRETE);
 
 	    if (!cv) {
 		double nnfrac;
@@ -1845,11 +1930,9 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	cols = get_optval_string(OPEN, OPT_F);
     }
 
-#ifdef ENABLE_NLS
     if (prn != NULL) {
 	check_for_console(prn);
     }
-#endif
 
     if (gretl_messages_on()) {
 	mprn = prn;
@@ -2068,6 +2151,10 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	c->descrip = NULL;
     }
 
+    if (!err) {
+	dataset_add_import_info(pdinfo, fname, GRETL_CSV);
+    }
+
  csv_bailout:
 
     if (fp != NULL) {
@@ -2080,9 +2167,7 @@ int import_csv (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	pputs(prn, M_("Out of memory\n"));
     }    
 
-#ifdef ENABLE_NLS
     console_off();
-#endif
 
     pdinfo->delim = save_delim;
 
