@@ -164,8 +164,8 @@ static int readdata (FILE *fp, const DATAINFO *pdinfo, double **Z,
 	for (i=1; i<pdinfo->v; i++) {
 	    for (t=0; t<n; t++) {
 		if (!fread(&x, sizeof x, 1, fp)) {
-		    sprintf(gretl_errmsg, _("WARNING: binary data read error at "
-			    "var %d"), i);
+		    gretl_errmsg_sprintf(_("WARNING: binary data read error at "
+					   "var %d"), i);
 		    return 1;
 		}
 		if (x == -999.0) {
@@ -182,8 +182,7 @@ static int readdata (FILE *fp, const DATAINFO *pdinfo, double **Z,
 	for (i=1; i<pdinfo->v; i++) {
 	    for (t=0; t<n; t++) {
 		if (!fread(&x, sizeof x, 1, fp)) {
-		    sprintf(gretl_errmsg, 
-			    _("WARNING: binary data read error at var %d"), i);
+		    gretl_errmsg_sprintf(_("WARNING: binary data read error at var %d"), i);
 		    return 1;
 		}
 		if (x == -999.0) {
@@ -196,18 +195,17 @@ static int readdata (FILE *fp, const DATAINFO *pdinfo, double **Z,
     } else if (old_byvar) {
 	/* ascii data by variable */
 	for (i=1; i<pdinfo->v; i++) {
-	   for (t=0; t<n && !err; t++) {
+	    for (t=0; t<n && !err; t++) {
 		if ((fscanf(fp, "%lf", &Z[i][t])) != 1) {
-		    sprintf(gretl_errmsg, 
-			    _("WARNING: ascii data read error at var %d, "
-			    "obs %d"), i, t + 1);
+		    gretl_errmsg_sprintf(_("WARNING: ascii data read error at var %d, "
+					   "obs %d"), i, t + 1);
 		    err = 1;
 		    break;
 		}
 		if (Z[i][t] == -999.0) {
 		    Z[i][t] = NADBL;
 		} 
-	   }
+	    }
 	}	       
     } else { 
 	/* ascii data by observation */
@@ -238,9 +236,8 @@ static int readdata (FILE *fp, const DATAINFO *pdinfo, double **Z,
 	    }
 	    for (i=1; i<pdinfo->v; i++) {
 		if ((fscanf(fp, "%lf", &Z[i][t])) != 1) {
-		    sprintf(gretl_errmsg, 
-			    _("WARNING: ascii data read error at var %d, "
-			    "obs %d"), i, t + 1);
+		    gretl_errmsg_sprintf(_("WARNING: ascii data read error at var %d, "
+					   "obs %d"), i, t + 1);
 		    err = 1;
 		    break;
 		}
@@ -271,8 +268,8 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 	for (i=1; i<pdinfo->v; i++) {
 	    for (t=0; t<n; t++) {
 		if (!gzread(fz, &xx, sizeof xx)) {
-		    sprintf(gretl_errmsg, _("WARNING: binary data read error at "
-			    "var %d"), i);
+		    gretl_errmsg_sprintf(_("WARNING: binary data read error at "
+					   "var %d"), i);
 		    return 1;
 		}
 		Z[i][t] = (double) xx;
@@ -282,8 +279,7 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 	/* double-precision binary data */
 	for (i=1; i<pdinfo->v; i++) {
 	    if (!gzread(fz, &Z[i][0], n * sizeof(double))) {
-		sprintf(gretl_errmsg, 
-			_("WARNING: binary data read error at var %d"), i);
+		gretl_errmsg_sprintf(_("WARNING: binary data read error at var %d"), i);
 		return 1;
 	    }
 	}
@@ -305,8 +301,8 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 	for (t=0; t<n; t++) {
 	    offset = 0L;
 	    if (!gzgets(fz, line, llen - 1)) {
-		sprintf(gretl_errmsg, _("WARNING: ascii data read error at "
-			"obs %d"), t + 1);
+		gretl_errmsg_sprintf(_("WARNING: ascii data read error at "
+				       "obs %d"), t + 1);
 		err = 1;
 		break;
 	    }
@@ -320,11 +316,10 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 
 	    if (pdinfo->markers) {
 		if (sscanf(line, sformat, pdinfo->S[t]) != 1) {
-		   sprintf(gretl_errmsg, 
-			   _("WARNING: failed to read case marker for "
-			   "obs %d"), t + 1);
-		   err = 1;
-		   break;
+		    gretl_errmsg_sprintf(_("WARNING: failed to read case marker for "
+					   "obs %d"), t + 1);
+		    err = 1;
+		    break;
 		}
 		pdinfo->S[t][OBSLEN-1] = 0;
 		offset += strlen(pdinfo->S[t]) + 1;
@@ -332,9 +327,8 @@ static int gz_readdata (gzFile fz, const DATAINFO *pdinfo, double **Z,
 
 	    for (i=1; i<pdinfo->v; i++) {
 		if (sscanf(line + offset, "%23s", numstr) != 1) {
-		    sprintf(gretl_errmsg, 
-			    _("WARNING: ascii data read error at var %d, "
-			    "obs %d"), i, t + 1);
+		    gretl_errmsg_sprintf(_("WARNING: ascii data read error at var %d, "
+					   "obs %d"), i, t + 1);
 		    err = 1;
 		    break;
 		}
@@ -396,23 +390,23 @@ int check_varname (const char *varname)
     if (testchar != 'a') {
 	if (isprint((unsigned char) testchar)) {
 	    if (ret == VARNAME_FIRSTCHAR) {
-		sprintf(gretl_errmsg, _("First char of varname ('%c') is bad\n"
-					"(first must be alphabetical)"), 
-			(unsigned char) testchar);
+		gretl_errmsg_sprintf(_("First char of varname ('%c') is bad\n"
+				       "(first must be alphabetical)"), 
+				     (unsigned char) testchar);
 	    } else {
-		sprintf(gretl_errmsg, _("Varname contains illegal character '%c'\n"
-					"Use only letters, digits and underscore"), 
-			(unsigned char) testchar);
+		gretl_errmsg_sprintf(_("Varname contains illegal character '%c'\n"
+				       "Use only letters, digits and underscore"), 
+				     (unsigned char) testchar);
 	    }
 	} else {
 	    if (ret == VARNAME_FIRSTCHAR) {
-		sprintf(gretl_errmsg, _("First char of varname (0x%x) is bad\n"
-					"(first must be alphabetical)"), 
-			(unsigned) testchar);
+		gretl_errmsg_sprintf(_("First char of varname (0x%x) is bad\n"
+				       "(first must be alphabetical)"), 
+				     (unsigned) testchar);
 	    } else {
-		sprintf(gretl_errmsg, _("Varname contains illegal character 0x%x\n"
-					"Use only letters, digits and underscore"), 
-			(unsigned) testchar);
+		gretl_errmsg_sprintf(_("Varname contains illegal character 0x%x\n"
+				       "Use only letters, digits and underscore"), 
+				     (unsigned) testchar);
 	    }
 	}
     }
@@ -431,7 +425,7 @@ static int readhdr (const char *hdrfile, DATAINFO *pdinfo,
 
     fp = gretl_fopen(hdrfile, "r");
     if (fp == NULL) {
-	sprintf(gretl_errmsg, _("Couldn't open file %s"),  hdrfile);
+	gretl_errmsg_sprintf(_("Couldn't open file %s"),  hdrfile);
 	return E_FOPEN;
     }
 
@@ -443,9 +437,10 @@ static int readhdr (const char *hdrfile, DATAINFO *pdinfo,
     while (1) {
         if (fscanf(fp, "%s", str) != 1) {
 	    fclose(fp);
-	    sprintf(gretl_errmsg, _("Opened header file %s\n"
-		    "Couldn't find list of variables (must "
-		    "be terminated with a semicolon)"), hdrfile);
+	    gretl_errmsg_sprintf(_("Opened header file %s\n"
+				   "Couldn't find list of variables (must "
+				   "be terminated with a semicolon)"), 
+				 hdrfile);
 	    return 1;
 	}
 	n = strlen(str);
@@ -567,7 +562,7 @@ static int readhdr (const char *hdrfile, DATAINFO *pdinfo,
 	    }
 	    free(dbuf);
 	} else if (lines < 0) {
-	    fprintf(stderr, I_("Failed to store data comments\n"));
+	    fputs(I_("Failed to store data comments\n"), stderr);
 	}
 	fclose(fp);
     } 
@@ -591,11 +586,9 @@ static int bad_date_string (const char *s)
     while (*s && !err) {
 	if (!isdigit((unsigned char) *s) && !IS_DATE_SEP(*s)) {
 	    if (isprint((unsigned char) *s)) {
-		sprintf(gretl_errmsg, 
-			_("Bad character '%c' in date string"), *s);
+		gretl_errmsg_sprintf(_("Bad character '%c' in date string"), *s);
 	    } else {
-		sprintf(gretl_errmsg, 
-			_("Bad character %d in date string"), *s);
+		gretl_errmsg_sprintf(_("Bad character %d in date string"), *s);
 	    }
 	    err = 1;
 	}
@@ -750,7 +743,7 @@ real_dateton (const char *date, const DATAINFO *pdinfo, int nolimit)
 	dotpos2 = get_dot_pos(pdinfo->stobs);
 
 	if ((dotpos1 && !dotpos2) || (dotpos2 && !dotpos1)) {
-	    sprintf(gretl_errmsg, _("Date strings inconsistent"));
+	    gretl_errmsg_set(_("Date strings inconsistent"));
 	} else if (!dotpos1 && !dotpos2) {
 	    n = atoi(date) - atoi(pdinfo->stobs);
 	} else {
@@ -825,8 +818,19 @@ out_of_range_panel_obs (char *s, int t, const DATAINFO *pdinfo)
     return s;
 }
 
-static char *
-real_ntodate (char *datestr, int t, const DATAINFO *pdinfo, int full)
+/**
+ * ntodate:
+ * @datestr: string to which date is to be printed.
+ * @t: an observation number (zero-based).
+ * @pdinfo: data information struct.
+ * 
+ * print to @datestr the calendar representation of observation
+ * number @t.
+ * 
+ * Returns: the observation string.
+ */
+
+char *ntodate (char *datestr, int t, const DATAINFO *pdinfo)
 {
     double x;
 
@@ -842,12 +846,6 @@ real_ntodate (char *datestr, int t, const DATAINFO *pdinfo, int full)
 	} else {
 	    calendar_date_string(datestr, t, pdinfo);
 	}
-	if (!full && strlen(datestr) > 8) {
-	    char tmp[12];
-
-	    strcpy(tmp, datestr);
-	    strcpy(datestr, tmp + 2);
-	}
 	return datestr;
     } else if (dataset_is_daily(pdinfo) || 
 	       dataset_is_weekly(pdinfo)) {
@@ -887,28 +885,6 @@ real_ntodate (char *datestr, int t, const DATAINFO *pdinfo, int full)
     return datestr;
 }
 
-/**
- * ntodate:
- * @datestr: string to which date is to be printed.
- * @t: an observation number (zero-based).
- * @pdinfo: data information struct.
- * 
- * print to @datestr the calendar representation of observation
- * number @t.
- * 
- * Returns: the observation string.
- */
-
-char *ntodate (char *datestr, int t, const DATAINFO *pdinfo)
-{
-    return real_ntodate(datestr, t, pdinfo, 0);
-}
-
-char *ntodate_full (char *datestr, int t, const DATAINFO *pdinfo)
-{
-    return real_ntodate(datestr, t, pdinfo, 1);
-}
-
 #define xround(x) (((x-floor(x))>.5)? ceil(x) : floor(x))
 
 /* for "seasonal" time series data (broad sense): given
@@ -1054,8 +1030,8 @@ static int writehdr (const char *hdrfile, const int *list,
 	binary = 2;
     }
 
-    ntodate_full(startdate, pdinfo->t1, pdinfo);
-    ntodate_full(enddate, pdinfo->t2, pdinfo);
+    ntodate(startdate, pdinfo->t1, pdinfo);
+    ntodate(enddate, pdinfo->t2, pdinfo);
 
     fp = gretl_fopen(hdrfile, "w");
     if (fp == NULL) {
@@ -1246,8 +1222,8 @@ static void date_maj_min (int t, const DATAINFO *pdinfo, int *maj, int *min)
  * @Z: data matrix.
  * @pdinfo: data information struct.
  * @opt: option flag indicating format in which to write the data.
- * @ppaths: pointer to paths information (should be NULL when not
- * called from gui).
+ * @progress: may be 1 when called from gui to display progress
+ * bar in case of a large data write; generally should be 0.
  * 
  * Write out a data file containing the values of the given set
  * of variables.
@@ -1257,7 +1233,7 @@ static void date_maj_min (int t, const DATAINFO *pdinfo, int *maj, int *min)
 
 int write_data (const char *fname, int *list, 
 		const double **Z, const DATAINFO *pdinfo, 
-		gretlopt opt, PATHS *ppaths)
+		gretlopt opt, int progress)
 {
     int i, t, v, l0;
     GretlDataFormat fmt;
@@ -1297,7 +1273,7 @@ int write_data (const char *fname, int *list,
     if (fmt == 0 || fmt == GRETL_FMT_GZIPPED) {
 	err = gretl_write_gdt(fname, list, Z, pdinfo, 
 			      (fmt == GRETL_FMT_GZIPPED)? OPT_Z : OPT_NONE,
-			      ppaths);
+			      progress);
 	goto write_exit;
     }
 
@@ -1308,8 +1284,8 @@ int write_data (const char *fname, int *list,
 
     if (fmt == GRETL_FMT_CSV && get_csv_delim(pdinfo) == ',' && 
 	',' == pdinfo->decpoint) {
-	sprintf(gretl_errmsg, _("You can't use the same character for "
-				"the column delimiter and the decimal point"));
+	gretl_errmsg_set(_("You can't use the same character for "
+			   "the column delimiter and the decimal point"));
 	err = E_DATA;
 	goto write_exit;
     }
@@ -1328,12 +1304,12 @@ int write_data (const char *fname, int *list,
 	    gz_switch_ext(lblfile, datfile, "lbl");
 	}
 	if (writehdr(hdrfile, list, pdinfo, fmt)) {
-	    fprintf(stderr, I_("Write of header file failed"));
+	    fputs(I_("Write of header file failed"), stderr);
 	    err = E_FOPEN;
 	    goto write_exit;
 	}
 	if (writelbl(lblfile, list, pdinfo)) {
-	    fprintf(stderr, I_("Write of labels file failed"));
+	    fputs(I_("Write of labels file failed"), stderr);
 	    err = E_FOPEN;
 	    goto write_exit;
 	}
@@ -1386,6 +1362,7 @@ int write_data (const char *fname, int *list,
 	}
     } else if (fmt == GRETL_FMT_CSV || fmt == GRETL_FMT_R) { 
 	/* export CSV or GNU R (dataframe) */
+	char na_string[8] = "NA";
 	int print_obs = 0;
 
 	if (fmt == GRETL_FMT_CSV) {
@@ -1396,6 +1373,7 @@ int write_data (const char *fname, int *list,
 	    if (!delim) {
 		delim = get_csv_delim(pdinfo);
 	    }
+	    strcpy(na_string, get_csv_na_string());
 	} else {
 	    print_obs = (pdinfo->S != NULL);
 	    delim = ' ';
@@ -1404,20 +1382,24 @@ int write_data (const char *fname, int *list,
 	if (fmt == GRETL_FMT_R && dataset_is_time_series(pdinfo)) {
 	    char datestr[OBSLEN];
 
-	    ntodate_full(datestr, pdinfo->t1, pdinfo);
+	    ntodate(datestr, pdinfo->t1, pdinfo);
 	    fprintf(fp, "# time-series data: start = %s, frequency = %d\n",
 		    datestr, pdinfo->pd);
 	}
 
-	/* variable names */
-	if (fmt == GRETL_FMT_CSV && print_obs && 
-	    (pdinfo->S != NULL || pdinfo->structure != CROSS_SECTION)) {
-	    fprintf(fp, "obs%c", delim);
+	if (fmt == GRETL_FMT_CSV && (opt & OPT_N)) {
+	    ;/* no header */
+	} else {
+	    /* header: variable names */
+	    if (fmt == GRETL_FMT_CSV && print_obs && 
+		(pdinfo->S != NULL || pdinfo->structure != CROSS_SECTION)) {
+		fprintf(fp, "obs%c", delim);
+	    }
+	    for (i=1; i<l0; i++) {
+		fprintf(fp, "%s%c", pdinfo->varname[list[i]], delim);
+	    }
+	    fprintf(fp, "%s\n", pdinfo->varname[list[l0]]);
 	}
-	for (i=1; i<l0; i++) {
-	    fprintf(fp, "%s%c", pdinfo->varname[list[i]], delim);
-	}
-	fprintf(fp, "%s\n", pdinfo->varname[list[l0]]);
 	
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    if (print_obs) {
@@ -1426,7 +1408,7 @@ int write_data (const char *fname, int *list,
 		} else {
 		    char tmp[OBSLEN];
 
-		    ntodate_full(tmp, t, pdinfo);
+		    ntodate(tmp, t, pdinfo);
 		    if (quarterly_or_monthly(pdinfo)) {
 			modify_date_for_csv(tmp, pdinfo->pd);
 		    }
@@ -1437,7 +1419,7 @@ int write_data (const char *fname, int *list,
 		v = list[i];
 		xx = Z[v][t];
 		if (na(xx)) {
-		    fprintf(fp, "NA");
+		    fputs(na_string, fp);
 		} else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
 		    fprintf(fp, "%.12g", xx);
 		} else {
@@ -1505,7 +1487,8 @@ int write_data (const char *fname, int *list,
 
 	fputs("/*\n", fp);
 	for (i=1; i<=list[0]; i++) {
-	    fprintf(fp, " %s: %s\n", pdinfo->varname[list[i]], VARLABEL(pdinfo, i));
+	    v = list[i];
+	    fprintf(fp, " %s: %s\n", pdinfo->varname[v], VARLABEL(pdinfo, v));
 	}
 	fputs("*/\n", fp);
 	date_maj_min(pdinfo->t1, pdinfo, &maj, &min);
@@ -1517,7 +1500,8 @@ int write_data (const char *fname, int *list,
 	    fputs("<1>\n", fp);
 	}
 	for (i=1; i<=list[0]; i++) {
-	    fprintf(fp, " %s", pdinfo->varname[list[i]]);
+	    v = list[i];
+	    fprintf(fp, " %s", pdinfo->varname[v]);
 	}
 	fputc('\n', fp);
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
@@ -1556,96 +1540,25 @@ int write_data (const char *fname, int *list,
     return err;
 }
 
-static void dataset_type_string (char *str, const DATAINFO *pdinfo)
+static int no_case_series_index (const DATAINFO *pdinfo,
+				 const char *vname)
 {
-    if (dataset_is_time_series(pdinfo)) {
-	strcpy(str, _("time series"));
-    } else if (dataset_is_panel(pdinfo)) {
-        strcpy(str, _("panel"));
-    } else {
-        strcpy(str, _("undated"));
-    }
-}
-
-static void pd_string (char *str, const DATAINFO *pdinfo)
-{
-    if (custom_time_series(pdinfo)) {
-	strcpy(str, _("special"));
-    } else {
-	switch (pdinfo->pd) {
-	case 1:
-	    strcpy(str, _("annual")); break;
-	case 4:
-	    strcpy(str, _("quarterly")); break;
-	case 12:
-	    strcpy(str, _("monthly")); break;
-	case 24:
-	    strcpy(str, _("hourly")); break;
-	case 52:
-	    strcpy(str, _("weekly")); break;
-	case 5:
-	case 6:
-	case 7:
-	    strcpy(str, _("daily")); break;
-	case 10:
-	    strcpy(str, _("decennial")); break;
-	default:
-	    strcpy(str, _("unknown")); break;
-	}
-    }
-}
-
-/**
- * data_report:
- * @pdinfo: data information struct.
- * @ppaths: path information struct.
- * @prn: gretl printing struct.
- * 
- * Write out a summary of the content of the current data set.
- * 
- * Returns: 0 on successful completion, non-zero on error.
- * 
- */
-
-int data_report (const DATAINFO *pdinfo, PATHS *ppaths, PRN *prn)
-{
-    char startdate[OBSLEN], enddate[OBSLEN], tmp[MAXLEN];
-    time_t prntime = time(NULL);
+    char s1[VNAMELEN], s2[VNAMELEN];
     int i;
 
-    ntodate_full(startdate, 0, pdinfo);
-    ntodate_full(enddate, pdinfo->n - 1, pdinfo);
-
-    sprintf(tmp, _("Data file %s\nas of"), 
-	    strlen(ppaths->datfile)? ppaths->datfile : _("(unsaved)"));
-
-    pprintf(prn, "%s %s\n\n", tmp, print_time(&prntime));
-
-    if (pdinfo->descrip != NULL && *pdinfo->descrip != '\0') {
-	pprintf(prn, "%s:\n\n", _("Description"));
-	pputs(prn, pdinfo->descrip);
-	pputs(prn, "\n\n");
-    }
-
-    dataset_type_string(tmp, pdinfo);
-    pprintf(prn, "%s: %s\n", _("Type of data"), tmp);
-    
-    if (dataset_is_time_series(pdinfo)) {
-	pd_string(tmp, pdinfo);
-	pprintf(prn, "%s: %s\n", _("Frequency"), tmp);
-    }	
-
-    pprintf(prn, "%s: %s - %s (n = %d)\n\n", _("Range"),
-	    startdate, enddate, pdinfo->n);
-
-    pprintf(prn, "%s:\n\n", _("Listing of variables"));
+    *s1 = '\0';
+    strncat(s1, vname, VNAMELEN - 1);
+    lower(s1);
 
     for (i=1; i<pdinfo->v; i++) {
-	pprintf(prn, "%*s  %s\n", VNAMELEN, pdinfo->varname[i], 
-		VARLABEL(pdinfo, i));
+	strcpy(s2, pdinfo->varname[i]);
+	lower(s2);
+	if (strcmp(s1, s2) == 0) {
+	    return i;
+	}
     }
 
-    return 0;
+    return -1;
 }
 
 /* read data "labels" from file */
@@ -1667,11 +1580,14 @@ static int readlbl (const char *lblfile, DATAINFO *pdinfo)
     while (fgets(line, MAXLEN, fp)) {
 	tailstrip(line);
         if (sscanf(line, "%s", varname) != 1) {
-	    sprintf(gretl_errmsg, _("Bad data label in %s"), lblfile); 
+	    gretl_errmsg_sprintf(_("Bad data label in %s"), lblfile); 
             break;
         }
 	v = series_index(pdinfo, varname);
-	if (v < pdinfo->v) {
+	if (v == pdinfo->v) {
+	    v = no_case_series_index(pdinfo, varname);
+	}
+	if (v > 0 && v < pdinfo->v) {
 	    p = line + strlen(varname);
 	    p += strspn(p, " \t");
 	    VARLABEL(pdinfo, v)[0] = '\0';
@@ -1802,7 +1718,6 @@ static void try_gdt (char *fname)
 /**
  * gretl_get_data:
  * @datfile: name of file to try.
- * @ppaths: path information struct.
  * @pZ: pointer to data set.
  * @pdinfo: pointer to data information struct.
  * @opt: for use with "append".
@@ -1814,8 +1729,7 @@ static void try_gdt (char *fname)
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
-int gretl_get_data (char *datfile, PATHS *ppaths,
-		    double ***pZ, DATAINFO *pdinfo, 
+int gretl_get_data (char *datfile, double ***pZ, DATAINFO *pdinfo, 
 		    gretlopt opt, PRN *prn) 
 {
     DATAINFO *tmpdinfo = NULL;
@@ -1823,7 +1737,6 @@ int gretl_get_data (char *datfile, PATHS *ppaths,
     FILE *dat = NULL;
     gzFile fz = NULL;
     char hdrfile[MAXLEN], lblfile[MAXLEN];
-    int newdata = (*pZ == NULL);
     int gdtsuff, gzsuff = 0;
     int binary = 0, old_byvar = 0;
     int err = 0;
@@ -1837,7 +1750,7 @@ int gretl_get_data (char *datfile, PATHS *ppaths,
 	gzsuff = has_suffix(datfile, ".gz");
     }
 
-    if (addpath(datfile, ppaths, 0) == NULL) { /* not found yet */
+    if (addpath(datfile, 0) == NULL) { /* not found yet */
 	char tryfile[MAXLEN];
 	int found = 0;
 
@@ -1846,23 +1759,25 @@ int gretl_get_data (char *datfile, PATHS *ppaths,
 	    *tryfile = '\0';
 	    strncat(tryfile, datfile, MAXLEN-1);
 	    try_gdt(tryfile); 
-	    found = (addpath(tryfile, ppaths, 0) != NULL);
+	    found = (addpath(tryfile, 0) != NULL);
 	    if (found) {
 		gdtsuff = 1;
 	    }
 	}
 
-	/* or maybe the file is gzipped but lacks a .gz extension? */
+	/* or maybe the file is gzipped but lacks a .gz extension?
+	   (backward compatibility) 
+	*/
 	if (!found && !gzsuff) { 
 	    sprintf(tryfile, "%s.gz", datfile);
-	    if (addpath(tryfile, ppaths, 0) != NULL) {
+	    if (addpath(tryfile, 0) != NULL) {
 		gzsuff = 1;
 		found = 1;
 	    }
 	}
 
 	if (!found) {
-	    sprintf(gretl_errmsg, _("Couldn't open file %s"),  datfile);
+	    gretl_errmsg_sprintf(_("Couldn't open file %s"), datfile);
 	    return E_FOPEN;
 	} else {
 	    strcpy(datfile, tryfile);
@@ -1871,7 +1786,7 @@ int gretl_get_data (char *datfile, PATHS *ppaths,
 
     /* catch XML files that have strayed in here? */
     if (gdtsuff && gretl_is_xml_file(datfile)) {
-	return gretl_read_gdt(datfile, ppaths, pZ, pdinfo, OPT_NONE, prn);
+	return gretl_read_gdt(datfile, pZ, pdinfo, OPT_NONE, prn);
     }
 
     tmpdinfo = datainfo_new();
@@ -1973,10 +1888,6 @@ int gretl_get_data (char *datfile, PATHS *ppaths,
 
     err = merge_or_replace_data(pZ, pdinfo, &tmpZ, &tmpdinfo, opt, prn);
 
-    if (!err && newdata && ppaths != NULL && datfile != ppaths->datfile) {
-	strcpy(ppaths->datfile, datfile);
-    }
-
  bailout:
 
     if (err && tmpdinfo != NULL) {
@@ -2068,7 +1979,7 @@ static int extend_markers (DATAINFO *pdinfo, int old_n, int new_n)
 static void merge_error (char *msg, PRN *prn)
 {
     pputs(prn, msg);
-    strcpy(gretl_errmsg, msg);
+    gretl_errmsg_set(msg);
 }
 
 static int count_new_vars (const DATAINFO *pdinfo, const DATAINFO *addinfo,
@@ -2104,13 +2015,11 @@ static int compare_ranges (const DATAINFO *targ,
 			   const DATAINFO *src,
 			   int *offset)
 {
-    int ed0, sd1, ed1;
+    int ed0 = dateton(targ->endobs, targ);
+    int sd1 = merge_dateton(src->stobs, targ);
+    int ed1 = merge_dateton(src->endobs, targ);
     int addobs = -1;
 
-    ed0 = dateton(targ->endobs, targ);
-    sd1 = merge_dateton(src->stobs, targ);
-    ed1 = merge_dateton(src->endobs, targ);
-
 #if 0
     fprintf(stderr, "compare_ranges:\n"
 	    " targ->n = %d, src->n = %d\n"
@@ -2139,6 +2048,8 @@ static int compare_ranges (const DATAINFO *targ,
 	*offset = 0;
 	if (ed1 > ed0) {
 	    addobs = ed1 - ed0;
+	} else {
+	    addobs = 0;
 	}
     } else if (sd1 == ed0 + 1) {
 	/* case: new data start right after end of old */
@@ -2401,7 +2312,7 @@ static int merge_data (double ***pZ, DATAINFO *pdinfo,
 	    merge_error(_("Out of memory!\n"), prn);
 	} else {
 	    pdinfo->n = new_n;
-	    ntodate_full(pdinfo->endobs, new_n - 1, pdinfo);
+	    ntodate(pdinfo->endobs, new_n - 1, pdinfo);
 	    pdinfo->t2 = pdinfo->n - 1;
 	}
     }
@@ -2499,7 +2410,7 @@ static int check_marker (char *src, int i)
 	trstr = g_locale_to_utf8(src, -1, NULL, &bytes, NULL);
 
 	if (trstr == NULL) {
-	    sprintf(gretl_errmsg, "Invalid characters in marker, line %d", i);
+	    gretl_errmsg_sprintf("Invalid characters in marker, line %d", i);
 	    err = E_DATA;
 	} else {
 	    *src = '\0';
@@ -2545,13 +2456,14 @@ int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname)
     
     for (t=0; t<pdinfo->n && !err; t++) {
 	if (fgets(line, sizeof line, fp) == NULL) {
-	    sprintf(gretl_errmsg, "Expected %d markers; found %d\n", 
-		    pdinfo->n, t);
+	    gretl_errmsg_sprintf("Expected %d markers; found %d\n", 
+				 pdinfo->n, t);
 	    err = E_DATA;
 	} else if (sscanf(line, "%31[^\n\r]", marker) != 1) {
-	    sprintf(gretl_errmsg, "Couldn't read marker on line %d", t+1);
+	    gretl_errmsg_sprintf("Couldn't read marker on line %d", t+1);
 	    err = E_DATA;
 	} else {
+	    g_strstrip(marker);
 	    strncat(S[t], marker, OBSLEN - 1);
 	    err = check_marker(S[t], t+1);
 	}
@@ -2829,9 +2741,7 @@ import_octave (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	clear_datainfo(octinfo, CLEAR_FULL);
     }
 
-#ifdef ENABLE_NLS
     console_off();
-#endif
 
     return err;
 }
@@ -2861,9 +2771,7 @@ int import_other (const char *fname, int ftype,
 		     gretlopt, PRN *);
     int err = 0;
 
-#ifdef ENABLE_NLS
     check_for_console(prn);
-#endif
 
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
@@ -2885,6 +2793,8 @@ int import_other (const char *fname, int ftype,
 	importer = get_plugin_function("dta_get_data", &handle);
     } else if (ftype == GRETL_SAV) {
 	importer = get_plugin_function("sav_get_data", &handle);
+    } else if (ftype == GRETL_SAS) {
+	importer = get_plugin_function("xport_get_data", &handle);
     } else if (ftype == GRETL_JMULTI) {
 	importer = get_plugin_function("jmulti_get_data", &handle);
     } else {
@@ -2902,9 +2812,7 @@ int import_other (const char *fname, int ftype,
 
  bailout:
 
-#ifdef ENABLE_NLS
     console_off();
-#endif
 
     return err;
 }
@@ -2917,7 +2825,7 @@ int import_other (const char *fname, int ftype,
  * @sheetname: name of worksheet, or %NULL.
  * @pZ: pointer to data set.
  * @pdinfo: pointer to data information struct.
- * @opt: fir use with "append".
+ * @opt: for use with "append".
  * @prn: gretl printing struct.
  * 
  * Open a data file of a type that requires a special plugin.
@@ -2937,9 +2845,7 @@ int import_spreadsheet (const char *fname, int ftype,
 		     gretlopt, PRN *);
     int err = 0;
 
-#ifdef ENABLE_NLS
     check_for_console(prn);
-#endif
 
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
@@ -2971,9 +2877,7 @@ int import_spreadsheet (const char *fname, int ftype,
 
  bailout:
 
-#ifdef ENABLE_NLS
     console_off();
-#endif
 
     return err;
 }
@@ -3036,7 +2940,6 @@ int gretl_is_pkzip_file (const char *fname)
 /**
  * detect_filetype:
  * @fname: name of file to examine.
- * @ppaths: path information struct.
  * 
  * Attempt to determine the type of a file to be opened in gretl:
  * data file (of various formats), or command script.
@@ -3044,7 +2947,7 @@ int gretl_is_pkzip_file (const char *fname)
  * Returns: integer code indicating the type of file.
  */
 
-GretlFileType detect_filetype (char *fname, PATHS *ppaths)
+GretlFileType detect_filetype (char *fname)
 {
     int i, c, ftype = GRETL_NATIVE_DATA;
     char teststr[5];
@@ -3075,6 +2978,8 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths)
 	return GRETL_DTA;
     if (has_suffix(fname, ".sav"))
 	return GRETL_SAV;
+    if (has_suffix(fname, ".xpt"))
+	return GRETL_SAS;
     if (has_suffix(fname, ".bin"))
 	return GRETL_NATIVE_DB;
     if (has_suffix(fname, ".rat"))
@@ -3088,11 +2993,7 @@ GretlFileType detect_filetype (char *fname, PATHS *ppaths)
     if (has_suffix(fname, ".bn7"))
 	return GRETL_PCGIVE_DB;
 
-    if (ppaths == NULL) {
-	return GRETL_NATIVE_DATA; 
-    }
-
-    addpath(fname, ppaths, 0); 
+    addpath(fname, 0); 
 
     if (gretl_is_xml_file(fname)) {
 	return GRETL_XML_DATA;  
@@ -3151,21 +3052,21 @@ int check_atof (const char *numstr)
     if (*test == '\0' && errno != ERANGE) return 0;
 
     if (!strcmp(numstr, test)) {
-	sprintf(gretl_errmsg, M_("'%s' -- no numeric conversion performed!"), numstr);
+	gretl_errmsg_sprintf(M_("'%s' -- no numeric conversion performed!"), numstr);
 	return 1;
     }
 
     if (*test != '\0') {
 	if (isprint(*test)) {
-	    sprintf(gretl_errmsg, M_("Extraneous character '%c' in data"), *test);
+	    gretl_errmsg_sprintf(M_("Extraneous character '%c' in data"), *test);
 	} else {
-	    sprintf(gretl_errmsg, M_("Extraneous character (0x%x) in data"), *test);
+	    gretl_errmsg_sprintf(M_("Extraneous character (0x%x) in data"), *test);
 	}
 	return 1;
     }
 
     if (errno == ERANGE) {
-	sprintf(gretl_errmsg, M_("'%s' -- number out of range!"), numstr);
+	gretl_errmsg_sprintf(M_("'%s' -- number out of range!"), numstr);
     }
 
     return 1;
@@ -3194,21 +3095,21 @@ int check_atoi (const char *numstr)
     if (*test == '\0' && errno != ERANGE) return 0;
 
     if (!strcmp(numstr, test)) {
-	sprintf(gretl_errmsg, M_("'%s' -- no numeric conversion performed!"), numstr);
+	gretl_errmsg_sprintf(M_("'%s' -- no numeric conversion performed!"), numstr);
 	return 1;
     }
 
     if (*test != '\0') {
 	if (isprint(*test)) {
-	    sprintf(gretl_errmsg, M_("Extraneous character '%c' in data"), *test);
+	    gretl_errmsg_sprintf(M_("Extraneous character '%c' in data"), *test);
 	} else {
-	    sprintf(gretl_errmsg, M_("Extraneous character (0x%x) in data"), *test);
+	    gretl_errmsg_sprintf(M_("Extraneous character (0x%x) in data"), *test);
 	}
 	return 1;
     }
 
     if (errno == ERANGE || val <= INT_MIN || val >= INT_MAX) {
-	sprintf(gretl_errmsg, M_("'%s' -- number out of range!"), numstr);
+	gretl_errmsg_sprintf(M_("'%s' -- number out of range!"), numstr);
     }
 
     return 1;
@@ -3280,3 +3181,74 @@ void dataset_set_regular_markers (DATAINFO *pdinfo)
     pdinfo->markers = REGULAR_MARKERS;
 }
 
+struct filetype_info {
+    GretlFileType type;
+    const char *src;
+};
+
+/* On successful import of data from some "foreign" format,
+   add a note to the "descrip" member of the new dataset
+   saying where it came from and when.
+*/
+
+void dataset_add_import_info (DATAINFO *pdinfo, const char *fname,
+			      GretlFileType type)
+{
+    struct filetype_info ftypes[] = {
+	{ GRETL_CSV,      "CSV" },
+	{ GRETL_GNUMERIC, "Gnumeric" },
+	{ GRETL_XLS,      "Excel" },
+	{ GRETL_WF1,      "Eviews" },
+	{ GRETL_DTA,      "Stata" },
+	{ GRETL_SAV,      "SPSS" },
+	{ GRETL_SAS,      "SAS" },
+	{ GRETL_JMULTI,   "JMulTi" }
+    };
+    int i, nt = sizeof ftypes / sizeof ftypes[0];
+    const char *src = NULL;
+    gchar *note = NULL;
+    char tstr[48];
+
+    for (i=0; i<nt; i++) {
+	if (type == ftypes[i].type) {
+	    src = ftypes[i].src;
+	    break;
+	}
+    }
+
+    if (src == NULL) {
+	return;
+    }
+
+    print_time(tstr);
+
+    if (g_utf8_validate(fname, -1, NULL)) {
+	const char *p = strrchr(fname, SLASH);
+
+	if (p != NULL) {
+	    fname = p + 1;
+	}
+	note = g_strdup_printf("Data imported from %s file '%s', %s\n",
+			       src, fname, tstr);
+    } else {
+	note = g_strdup_printf("Data imported from %s, %s\n",
+			       src, tstr);
+    }
+
+    if (note != NULL) {
+	if (pdinfo->descrip == NULL) {
+	    pdinfo->descrip = gretl_strdup(note);
+	} else {
+	    int dlen = strlen(pdinfo->descrip);
+	    int nlen = strlen(note);
+	    char *tmp = realloc(pdinfo->descrip, dlen + nlen + 3);
+
+	    if (tmp != NULL) {
+		pdinfo->descrip = tmp;
+		strcat(pdinfo->descrip, "\n\n");
+		strncat(pdinfo->descrip, note, nlen);
+	    }
+	}
+	g_free(note);
+    }
+}
