@@ -34,6 +34,7 @@ typedef enum {
     GRETL_WF1,            /* Eviews workfile data */
     GRETL_DTA,            /* Stata .dta data */
     GRETL_SAV,            /* SPSS .sav data */
+    GRETL_SAS,            /* SAS xport data file */
     GRETL_SCRIPT,         /* file containing gretl commands */
     GRETL_SESSION,        /* zipped session file */
     GRETL_NATIVE_DB,      /* gretl database */
@@ -62,6 +63,7 @@ typedef enum {
 
 #define OTHER_IMPORT(f) (f == GRETL_DTA || \
                          f == GRETL_SAV || \
+			 f == GRETL_SAS || \
                          f == GRETL_JMULTI || \
                          f == GRETL_OCTAVE || \
 			 f == GRETL_WF1)
@@ -79,8 +81,6 @@ int dateton (const char *date, const DATAINFO *pdinfo);
 
 char *ntodate (char *datestr, int t, const DATAINFO *pdinfo);
 
-char *ntodate_full (char *datestr, int t, const DATAINFO *pdinfo);
-
 int get_subperiod (int t, const DATAINFO *pdinfo, int *err);
 
 int get_info (const char *hdrfile, PRN *prn);
@@ -91,9 +91,7 @@ double get_date_x (int pd, const char *obs);
 
 int write_data (const char *fname, int *list, 
 		const double **Z, const DATAINFO *pdinfo, 
-	        gretlopt opt, PATHS *ppaths);
-
-int data_report (const DATAINFO *pdinfo, PATHS *ppaths, PRN *prn);
+	        gretlopt opt, int progress);
 
 int is_gzipped (const char *fname);
 
@@ -105,8 +103,7 @@ int merge_or_replace_data (double ***pZ0, DATAINFO *pdinfo0,
 			   double ***pZ1, DATAINFO **ppdinfo1,
 			   gretlopt opt, PRN *prn);
 
-int gretl_get_data (char *datfile, PATHS *ppaths,
-		    double ***pZ, DATAINFO *pdinfo, 
+int gretl_get_data (char *datfile, double ***pZ, DATAINFO *pdinfo, 
 		    gretlopt opt, PRN *prn);
 
 int open_nulldata (double ***pZ, DATAINFO *pdinfo, 
@@ -127,7 +124,7 @@ int import_other (const char *fname, int ftype,
 
 int add_obs_markers_from_file (DATAINFO *pdinfo, const char *fname);
 
-GretlFileType detect_filetype (char *fname, PATHS *ppaths);
+GretlFileType detect_filetype (char *fname);
 
 gretlopt data_save_opt_from_suffix (const char *fname);
 
@@ -139,4 +136,7 @@ int check_atoi (const char *numstr);
 
 int transpose_data (double ***pZ, DATAINFO *pdinfo);
 
+void dataset_add_import_info (DATAINFO *pdinfo, const char *fname,
+			      GretlFileType type);
+
 #endif /* DATAIO_H */
