@@ -20,6 +20,7 @@
 #include "libgretl.h"
 #include "gretl_func.h"
 #include "gretl_scalar.h"
+#include "gretl_string_table.h"
 #include "libset.h"
 #include "dbread.h"
 
@@ -147,6 +148,11 @@ void clear_datainfo (DATAINFO *pdinfo, int code)
 	pdinfo->submask = NULL;
     }
 
+    if (pdinfo->restriction != NULL) {
+	free(pdinfo->restriction);
+	pdinfo->restriction = NULL;
+    }	
+
     if (pdinfo->paninfo != NULL) {
 	dataset_destroy_panel_info(pdinfo);
     }    
@@ -448,7 +454,7 @@ int dataset_finalize_panel_indices (DATAINFO *pdinfo)
     /* basic validity check */
     for (i=0; i<pdinfo->n; i++) {
 	if (pan->unit[i] < 0 || pan->period[i] < 0) {
-	    strcpy(gretl_errmsg, "Panel index information is corrupted");
+	    gretl_errmsg_set("Panel index information is corrupted");
 	    return E_DATA;
 	}
 	sprintf(test, "%d", pan->period[i] + 1);
@@ -618,6 +624,7 @@ DATAINFO *datainfo_new (void)
     dinfo->S = NULL;
     dinfo->descrip = NULL;
     dinfo->submask = NULL;
+    dinfo->restriction = NULL;
 
     dinfo->structure = CROSS_SECTION;
 
@@ -759,6 +766,7 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample)
     pdinfo->descrip = NULL;
     pdinfo->paninfo = NULL;
     pdinfo->submask = NULL;
+    pdinfo->restriction = NULL;
     
     return 0;
 }
@@ -1365,7 +1373,7 @@ int dataset_add_series_as (double *x, const char *newname,
     int v, t, err = 0;
 
     if (pdinfo->varinfo == NULL) {
-	strcpy(gretl_errmsg, _("Please open a data file first"));
+	gretl_errmsg_set(_("Please open a data file first"));
 	return 1;
     }
 
@@ -1502,7 +1510,7 @@ static int vars_renumbered (const int *list, DATAINFO *pdinfo,
 
 int overwrite_err (const char *name)
 {
-    sprintf(gretl_errmsg, "The variable %s is read-only", name);
+    gretl_errmsg_sprintf("The variable %s is read-only", name);
 
     return E_DATA;
 }
@@ -1552,7 +1560,7 @@ int dataset_rename_series (DATAINFO *pdinfo, int v, const char *name)
 
     if (strcmp(pdinfo->varname[v], name)) {
 	strcpy(pdinfo->varname[v], name);
-	dataset_changed = 1;
+	set_dataset_is_changed();
     }
 
     return 0;
@@ -1746,9 +1754,8 @@ int dataset_drop_listed_variables (int *list, double ***pZ,
     int lastvar[2];
     int err = 0;
 
-    if (pdinfo->n == 0) {
-	strcpy(gretl_errmsg, _("No dataset is in place"));
-	return E_DATA;
+    if (pdinfo->n == 0 || pdinfo->v == 0) {
+	return E_NODATA;
     }
 
     if (list == NULL) {
@@ -1868,6 +1875,93 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+/* intended for use with newly imported data: trash any 
+   series that contain nothing but NAs
+*/
+
+int maybe_prune_dataset (double ***pZ, DATAINFO **ppdinfo, void *p)
+{
+    DATAINFO *pdinfo = *ppdinfo;
+    int allmiss, prune = 0, err = 0;
+    int i, t;
+
+    for (i=1; i<pdinfo->v; i++) {
+	allmiss = 1;
+	for (t=0; t<pdinfo->n; t++) {
+	    if (!na((*pZ)[i][t])) {
+		allmiss = 0;
+		break;
+	    }
+	}
+	if (allmiss) {
+	    prune = 1;
+	    break;
+	}
+    }
+
+    if (prune) {
+	char *mask = calloc(pdinfo->v, 1);
+	double **newZ = NULL;
+	DATAINFO *newinfo = NULL;
+	int ndrop = 0;
+
+	if (mask == NULL) {
+	    return E_ALLOC;
+	}
+
+	for (i=1; i<pdinfo->v; i++) {
+	    allmiss = 1;
+	    for (t=0; t<pdinfo->n; t++) {
+		if (!na((*pZ)[i][t])) {
+		    allmiss = 0;
+		    break;
+		}
+	    }
+	    if (allmiss) {
+		mask[i] = 1;
+		ndrop++;
+	    }
+	}
+
+	newinfo = datainfo_new();
+	if (newinfo == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    newinfo->v = pdinfo->v - ndrop;
+	    newinfo->n = pdinfo->n;
+	    err = start_new_Z(&newZ, newinfo, 0);
+	}
+
+	if (!err) {
+	    gretl_string_table *st = (gretl_string_table *) p;
+	    size_t ssize = pdinfo->n * sizeof **newZ;
+	    int k = 1;
+
+	    for (i=1; i<pdinfo->v; i++) {
+		if (!mask[i]) {
+		    memcpy(newZ[k], (*pZ)[i], ssize);
+		    strcpy(newinfo->varname[k], pdinfo->varname[i]);
+		    strcpy(VARLABEL(newinfo, k), VARLABEL(pdinfo, i));
+		    if (st != NULL && k < i) {
+			gretl_string_table_reset_column_id(st, i, k);
+		    }
+		    k++;
+		}
+	    }
+
+	    destroy_dataset(*pZ, pdinfo);
+	    *pZ = newZ;
+	    *ppdinfo = newinfo;
+
+	    fprintf(stderr, "pruned dataset to %d variables\n", newinfo->v);
+	}
+
+	free(mask);
+    }
+
+    return err;
+}
+
 /* apparatus for sorting entire dataset */
 
 typedef struct spoint_t_ spoint_t;
@@ -1970,7 +2064,7 @@ static int dataset_sort (const char *s, double **Z, DATAINFO *pdinfo,
 
     if (dataset_is_time_series(pdinfo) ||
 	dataset_is_panel(pdinfo)) {
-	strcpy(gretl_errmsg, "You can only do this with undated data");
+	gretl_errmsg_set("You can only do this with undated data");
 	return E_DATA;
     }
 
@@ -2527,10 +2621,10 @@ void set_var_discrete (DATAINFO *pdinfo, int i, int s)
 
 	if (s && !(flags & VAR_DISCRETE)) {
 	    pdinfo->varinfo[i]->flags |= VAR_DISCRETE;
-	    dataset_changed = 1;
+	    set_dataset_is_changed();
 	} else if (!s && (flags & VAR_DISCRETE)) {
 	    pdinfo->varinfo[i]->flags &= ~VAR_DISCRETE;
-	    dataset_changed = 1;
+	    set_dataset_is_changed();
 	}
     }
 }
@@ -2595,7 +2689,7 @@ int var_set_description (DATAINFO *pdinfo, int i,
     if (strcmp(targ, s)) {
 	*targ = 0;
 	strncat(targ, s, MAXLABEL - 1);
-	dataset_changed = 1;
+	set_dataset_is_changed();
     }
 
     return 0;
@@ -2609,7 +2703,20 @@ int var_set_display_name (DATAINFO *pdinfo, int i,
     if (strcmp(targ, s)) {
 	*targ = 0;
 	strncat(targ, s, MAXDISP - 1);
-	dataset_changed = 1;
+	set_dataset_is_changed();
+    }
+
+    return 0;
+}
+
+int var_set_compact_method (DATAINFO *pdinfo, int i,
+			    int method) 
+{
+    int orig = COMPACT_METHOD(pdinfo, i);
+
+    if (method != orig) {
+	COMPACT_METHOD(pdinfo, i) = method;
+	set_dataset_is_changed();
     }
 
     return 0;
@@ -2720,7 +2827,7 @@ static int dataset_int_param (const char **ps, int op,
 
 	if (!ok) {
 	    *err = E_PDWRONG;
-	    strcpy(gretl_errmsg, "This conversion is not supported");
+	    gretl_errmsg_set("This conversion is not supported");
 	}
     } else if (op == DS_EXPAND) {
 	int ok = 0;
@@ -2733,7 +2840,7 @@ static int dataset_int_param (const char **ps, int op,
 
 	if (!ok) {
 	    *err = E_PDWRONG;
-	    strcpy(gretl_errmsg, "This conversion is not supported");
+	    gretl_errmsg_set("This conversion is not supported");
 	}
     }
 
@@ -2858,7 +2965,7 @@ int dataset_resample (int n, unsigned int seed,
     rinfo->t2 = n - 1;
     dataset_obs_info_default(rinfo);
 
-    rinfo->submask = RESAMPLED;
+    set_dataset_resampled(rinfo);
 
  bailout:
 
@@ -2893,7 +3000,7 @@ int modify_dataset (int op, const int *list, const char *s,
     }
 
     if (gretl_function_depth() > 0) {
-	strcpy(gretl_errmsg, _("The 'dataset' command is not available within functions"));
+	gretl_errmsg_set(_("The 'dataset' command is not available within functions"));
 	return 1;
     }
 
@@ -2913,7 +3020,7 @@ int modify_dataset (int op, const int *list, const char *s,
     }
 
     if (op != DS_RESTORE && complex_subsampled()) {
-	strcpy(gretl_errmsg, _("The data set is currently sub-sampled"));
+	gretl_errmsg_set(_("The data set is currently sub-sampled"));
 	return 1;
     }
 
