@@ -254,6 +254,16 @@ typedef enum {
  */
 #define unset_var_listarg(p, i) (p->varinfo[i]->flags &= ~VAR_LISTARG)
 
+/**
+ * series_set_flag:
+ * @p: pointer to data information struct.
+ * @i: index number of variable.
+ * @f: flag to set.
+ *
+ * Set the given flag on the given (series) variable.
+ */
+#define series_set_flag(p, i, f) (p->varinfo[i]->flags |= f)
+
 #define sample_size(p) ((p == NULL)? 0 : (p->t2 - p->t1 + 1))
 
 void free_Z (double **Z, DATAINFO *pdinfo);
@@ -340,6 +350,8 @@ int dataset_destroy_hidden_variables (double ***pZ, DATAINFO *pdinfo,
 
 int dataset_drop_last_variables (int delvars, double ***pZ, DATAINFO *pdinfo);
 
+int maybe_prune_dataset (double ***pZ, DATAINFO **ppdinfo, void *p);
+
 int dataset_stack_variables (const char *vname, const char *line,
 			     double ***pZ, DATAINFO *pdinfo, 
 			     PRN *prn);
@@ -364,6 +376,9 @@ int var_set_display_name (DATAINFO *pdinfo, int i,
 int var_set_description (DATAINFO *pdinfo, int i,
 			 const char *s); 
 
+int var_set_compact_method (DATAINFO *pdinfo, int i,
+			    int method);
+
 const char *var_get_graph_name (const DATAINFO *pdinfo, int i);
 
 unsigned int get_resampling_seed (void);
