@@ -30,7 +30,7 @@
 #include <unistd.h>
 #include <errno.h>
 
-#if WORDS_BIGENDIAN
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 # include <netinet/in.h>
 #endif
 
@@ -95,12 +95,7 @@ typedef struct {
 static char db_name[MAXLEN];
 static int db_type;
 
-#if WORDS_BIGENDIAN
-typedef struct {
-    long frac;
-    short exp;
-} netfloat;
-
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 float retrieve_float (netfloat nf)
 {
     short exp = ntohs(nf.exp);
@@ -186,11 +181,11 @@ int get_remote_db_data (const char *dbbase, SERIESINFO *sinfo,
     int v = sinfo->v;
     dbnumber x;
     size_t offset;
-#if WORDS_BIGENDIAN
+#if G_BYTE_ORDER == G_BIG_ENDIAN
     netfloat nf;
 #endif
 
-#if WORDS_BIGENDIAN
+#if G_BYTE_ORDER == G_BIG_ENDIAN
     err = retrieve_remote_db_data(dbbase, sinfo->varname, &getbuf,
 				  GRAB_NBO_DATA);
 #else
@@ -207,7 +202,7 @@ int get_remote_db_data (const char *dbbase, SERIESINFO *sinfo,
 
     offset = 0L;
     for (t=sinfo->t1; t<=t2; t++) {
-#if WORDS_BIGENDIAN
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 	/* go via network byte order */
 	memcpy(&(nf.frac), getbuf + offset, sizeof nf.frac);
 	offset += sizeof nf.frac;
@@ -247,7 +242,7 @@ int get_pcgive_db_data (const char *dbbase, SERIESINFO *sinfo,
 	    err = E_DATA;
 	    break;
 	}
-#if WORDS_BIGENDIAN
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 	reverse_double(x);
 #endif
 	if (x == -9999.99 || isnan(x)) {
@@ -349,7 +344,7 @@ open_native_db_files (const char *dname, FILE **f1, char *name1,
 	if (!err) {
 	    fidx = gretl_fopen(fname, "r");
 	    if (fidx == NULL) {
-		strcpy(gretl_errmsg, _("Couldn't open database index file"));
+		gretl_errmsg_set(_("Couldn't open database index file"));
 		err = E_FOPEN;
 	    } 
 	}
@@ -369,7 +364,7 @@ open_native_db_files (const char *dname, FILE **f1, char *name1,
 	if (!err) {
 	    fbin = gretl_fopen(fname, "rb");
 	    if (fbin == NULL) {
-		strcpy(gretl_errmsg, _("Couldn't open database binary file"));
+		gretl_errmsg_set(_("Couldn't open database binary file"));
 		err = E_FOPEN;
 	    } 
 	}
@@ -429,7 +424,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 	    get_native_series_comment(sinfo, s1);
 	    if (sscanf(s2, "%c %10s %*s %10s %*s %*s %d", 
 		       &pdc, stobs, endobs, &sinfo->nobs) != 4) {
-		strcpy(gretl_errmsg, _("Failed to parse series information"));
+		gretl_errmsg_set(_("Failed to parse series information"));
 		err = DB_PARSE_ERROR;
 	    } else {
 		get_native_series_pd(sinfo, pdc);
@@ -439,7 +434,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 	    }
 	} else {
 	    if (sscanf(s2, "%*c %*s %*s %*s %*s %*s %d", &n) != 1) {
-		strcpy(gretl_errmsg, _("Failed to parse series information"));
+		gretl_errmsg_set(_("Failed to parse series information"));
 		err = DB_PARSE_ERROR;
 	    } else {
 		offset += n * sizeof(dbnumber);
@@ -450,7 +445,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
     fclose(fp);
 
     if (!gotit) {
-	sprintf(gretl_errmsg, _("Series not found, '%s'"), series);
+	gretl_errmsg_sprintf(_("Series not found, '%s'"), series);
 	err = DB_NO_SUCH_SERIES;
     }
 
@@ -502,7 +497,7 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
 
 	if (sscanf(s2, "%c %10s %*s %10s %*s %*s %d", 
 		   &pdc, stobs, endobs, &sinfo->nobs) != 4) {
-	    strcpy(gretl_errmsg, _("Failed to parse series information"));
+	    gretl_errmsg_set(_("Failed to parse series information"));
 	    err = DB_PARSE_ERROR;
 	} else {
 	    get_native_series_pd(sinfo, pdc);
@@ -516,7 +511,7 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
     free(buf);
 
     if (!gotit) {
-	sprintf(gretl_errmsg, _("Series not found, '%s'"), series);
+	gretl_errmsg_sprintf(_("Series not found, '%s'"), series);
 	err = DB_NO_SUCH_SERIES;
     }
 
@@ -590,7 +585,7 @@ get_pcgive_series_info (const char *series, SERIESINFO *sinfo)
 
     fp = gretl_fopen(dbidx, "r");
     if (fp == NULL) {
-	strcpy(gretl_errmsg, _("Couldn't open database index file"));
+	gretl_errmsg_set(_("Couldn't open database index file"));
 	return E_FOPEN;
     }
 
@@ -635,7 +630,7 @@ get_pcgive_series_info (const char *series, SERIESINFO *sinfo)
     fclose(fp);
 
     if (!err && !gotit) {
-	sprintf(gretl_errmsg, _("Series not found, '%s'"), series);
+	gretl_errmsg_sprintf(_("Series not found, '%s'"), series);
 	err = DB_NO_SUCH_SERIES;
     }
 
@@ -675,19 +670,35 @@ static int get_endobs (char *datestr, int startyr, int startfrac,
 
 static int dinfo_sanity_check (const DATEINFO *dinfo)
 {
-    if (dinfo->info < 0 || dinfo->info > 365 ||
-	dinfo->year < 0 || dinfo->year > 3000 ||
-	dinfo->month < 0 || dinfo->month > 12 ||
-	dinfo->day < 0 || dinfo->day > 365) {
-	strcpy(gretl_errmsg, _("This is not a valid RATS 4.0 database"));
+    int err = 0;
+
+    if (dinfo->info < 0 || dinfo->info > 365) {
+	err = 1;
+    } else if (dinfo->day < 0 || dinfo->day > 365) {
+	err = 1;
+    } else if (dinfo->year < 0 || dinfo->year > 3000) {
+	err = 1;
+    } else if (dinfo->info == 52) {
+	/* note: "month" = week */
+	if (dinfo->month < 0 || dinfo->month > 52) {
+	    err = 1;
+	}
+    } else {
+	/* annual, quarterly, monthly */
+	if (dinfo->month < 0 || dinfo->month > 12) {
+	    err = 1;
+	}
+    }
+
+    if (err) {
+	gretl_errmsg_set(_("This is not a valid RATS 4.0 database"));
 	fprintf(stderr, "rats database: failed dinfo_sanity_check:\n"
 		" info=%ld, year=%d, month=%d, day=%d\n",
 		dinfo->info, (int) dinfo->year, (int) dinfo->month, 
 		(int) dinfo->day);
-	return 1;
     }
 
-    return 0;
+    return err;
 }
 
 static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
@@ -715,7 +726,7 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 	} else {
 	    startfrac = 4;
 	}
-    } else if (dinfo->info == 12) {
+    } else if (dinfo->info == 12 || dinfo->info == 52) {
 	sprintf(pdstr, ".%02d", dinfo->month);
 	startfrac = dinfo->month;
     } else if (dinfo->info == 1) {
@@ -724,8 +735,8 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 	fprintf(stderr, I_("frequency (%d) does not make seem to make sense"),
 		(int) dinfo->info);
 	fputc('\n', stderr);
-	sprintf(gretl_errmsg, ("frequency (%d) does not make seem to make sense"), 
-		(int) dinfo->info);
+	gretl_errmsg_sprintf(("frequency (%d) does not make seem to make sense"), 
+			     (int) dinfo->info);
 	err = 1;
     }   
 
@@ -767,11 +778,9 @@ static int in7_to_sinfo (const char *varname, const char *comment,
 	sprintf(sinfo->stobs, "%d", y0);
 	sprintf(sinfo->endobs, "%d", y1);
     } else {
-	fprintf(stderr, I_("frequency %d is not supported"),
-		pd);
+	fprintf(stderr, I_("frequency %d is not supported"), pd);
 	fputc('\n', stderr);
-	sprintf(gretl_errmsg, ("frequency %d is not supported"), 
-		pd);
+	gretl_errmsg_sprintf(_("frequency %d is not supported"), pd);
 	err = 1;
     }
 
@@ -1016,7 +1025,7 @@ static int count_in7_series (FILE *fp, int *err)
     while (fgets(line, sizeof line, fp)) {
 	if (i == 0 && strncmp(line, "pcgive 700", 10)) {
 	    *err = 1;
-	    strcpy(gretl_errmsg, "This is not a PcGive 700 data file");
+	    gretl_errmsg_set("This is not a PcGive 700 data file");
 	    return 0;
 	}
 	if (*line == '>') {
@@ -1055,7 +1064,7 @@ dbwrapper *read_pcgive_db (FILE *fp)
     ns = count_in7_series(fp, &err);
     if (ns == 0) {
 	if (!err) {
-	    strcpy(gretl_errmsg, _("No valid series found"));
+	    gretl_errmsg_set(_("No valid series found"));
 	}
 	return NULL;
     }
@@ -1067,7 +1076,7 @@ dbwrapper *read_pcgive_db (FILE *fp)
     /* allocate table for series rows */
     dw = dbwrapper_new(ns);
     if (dw == NULL) {
-	strcpy(gretl_errmsg, _("Out of memory!"));
+	gretl_errmsg_set(_("Out of memory!"));
 	return NULL;
     }
 
@@ -1111,7 +1120,7 @@ dbwrapper *read_rats_db (FILE *fp)
 
     /* basic check */
     if (forward <= 0) { 
-	strcpy(gretl_errmsg, _("This is not a valid RATS 4.0 database"));
+	gretl_errmsg_set(_("This is not a valid RATS 4.0 database"));
 	fprintf(stderr, "rats database: got forward = %ld\n", forward);
 	return NULL;
     }
@@ -1119,7 +1128,7 @@ dbwrapper *read_rats_db (FILE *fp)
     /* allocate table for series rows */
     dw = dbwrapper_new(0);
     if (dw == NULL) {
-	strcpy(gretl_errmsg, _("Out of memory!"));
+	gretl_errmsg_set(_("Out of memory!"));
 	return NULL;
     }
     
@@ -1134,7 +1143,7 @@ dbwrapper *read_rats_db (FILE *fp)
 	if (dw->nv > 0 && dw->nv % DB_INIT_ROWS == 0) {
 	    err = dbwrapper_expand(dw);
 	    if (err) {
-		strcpy(gretl_errmsg, _("Out of memory!"));
+		gretl_errmsg_set(_("Out of memory!"));
 	    }
 	}
 	if (!err) {
@@ -1192,7 +1201,7 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
 	fread(&rdata, sizeof rdata, 1, fp);
 	for (i=0; i<31 && t<T; i++) {
 	    x = rdata.data[i];
-#if WORDS_BIGENDIAN
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 	    reverse_double(x);
 #endif
 	    if (isnan(x)) {
@@ -1266,7 +1275,7 @@ static int get_rats_series_info (const char *series_name, SERIESINFO *sinfo)
 
     /* basic check */
     if (forward <= 0) {
-	strcpy(gretl_errmsg, _("This is not a valid RATS 4.0 database"));
+	gretl_errmsg_set(_("This is not a valid RATS 4.0 database"));
 	fprintf(stderr, "rats database: got forward = %ld\n", forward);
 	return DB_PARSE_ERROR;
     }
@@ -1457,8 +1466,7 @@ double *expand_db_series (const double *src, SERIESINFO *sinfo,
     return x;
 }
 
-int 
-set_db_name (const char *fname, int filetype, const PATHS *ppaths, PRN *prn)
+int set_db_name (const char *fname, int filetype, PRN *prn)
 {
     FILE *fp;
     int err = 0;
@@ -1487,14 +1495,18 @@ set_db_name (const char *fname, int filetype, const PATHS *ppaths, PRN *prn)
 
     if (fp == NULL) {
 	/* try looking a bit more */
-	if (filetype == GRETL_NATIVE_DB && 
-	    strstr(db_name, ppaths->binbase) == NULL) {
-	    build_path(db_name, ppaths->binbase, fname, NULL);
-	} else if (filetype == GRETL_RATS_DB && 
-		   strstr(db_name, ppaths->ratsbase) == NULL) {
-	    build_path(db_name, ppaths->ratsbase, fname, NULL);
+	const char *path = NULL;
+
+	if (filetype == GRETL_NATIVE_DB) {
+	    path = gretl_binbase();
+	} else if (filetype == GRETL_RATS_DB) {
+	    path = gretl_ratsbase();
+	}
+
+	if (path != NULL && strstr(db_name, path) == NULL) {
+	    build_path(db_name, path, fname, NULL);
+	    fp = gretl_fopen(db_name, "rb");
 	}
-	fp = gretl_fopen(db_name, "rb");
     }
 	    
     if (fp == NULL) {
@@ -1643,25 +1655,24 @@ int db_set_sample (const char *s, DATAINFO *pdinfo)
     int t1 = 0, t2 = 0;
 
     if (sscanf(s, "%10s %10s", start, stop) != 2) {
-	sprintf(gretl_errmsg, _("error reading smpl line"));
+	gretl_errmsg_set(_("error reading smpl line"));
 	return 1;
     }
 
     if (strcmp(start, ";")) {
 	t1 = dateton(start, pdinfo);
-	if (t1 < 0 || *gretl_errmsg != '\0') {
+	if (t1 < 0) {
 	    return 1;
 	}
     }
 
     t2 = dateton(stop, pdinfo);
-
-    if (*gretl_errmsg != '\0') {
+    if (t2 < 0) {
 	return 1;
     }
 
     if (t1 > t2) {
-	sprintf(gretl_errmsg, _("Invalid null sample"));
+	gretl_errmsg_set(_("Invalid null sample"));
 	return 1;
     }
 
@@ -1919,12 +1930,12 @@ static int odbc_get_series (char *line, double ***pZ, DATAINFO *pdinfo,
     int err = 0;
 
     if (gretl_odinfo.dsn == NULL) {
-	strcpy(gretl_errmsg, _("No database has been opened"));
+	gretl_errmsg_set(_("No database has been opened"));
 	return 1;
     } 
 
     if (pdinfo->n == 0) {
-	strcpy(gretl_errmsg, _("No series length has been defined"));
+	gretl_errmsg_set(_("No series length has been defined"));
 	return 1;
     }
 
@@ -2041,7 +2052,7 @@ int db_get_series (char *line, double ***pZ, DATAINFO *pdinfo,
 #endif
 
     if (*db_name == '\0') {
-	strcpy(gretl_errmsg, _("No database has been opened"));
+	gretl_errmsg_set(_("No database has been opened"));
 	return 1;
     }   
 
@@ -2086,7 +2097,7 @@ int db_get_series (char *line, double ***pZ, DATAINFO *pdinfo,
 	/* temporary dataset */
 	dbZ = new_dbZ(sinfo.nobs);
 	if (dbZ == NULL) {
-	    strcpy(gretl_errmsg, _("Out of memory!"));
+	    gretl_errmsg_set(_("Out of memory!"));
 	    return 1;
 	}
 
@@ -2123,7 +2134,7 @@ int db_get_series (char *line, double ***pZ, DATAINFO *pdinfo,
 	/* free up temp stuff */
 	free_dbZ(dbZ);
 
-	if (!err) {
+	if (!err && !(opt & OPT_Q)) {
 	    pprintf(prn, _("Series imported OK"));
 	    pputc(prn, '\n');
 	    if (newdata) {
@@ -2153,7 +2164,7 @@ static FILE *tempfile_open (char *fname, int *err)
 	fp = fdopen(fd, "w+");
 	if (fp == NULL) {
 	    *err = E_FOPEN;
-	    sprintf(gretl_errmsg, _("Couldn't open %s"), fname);
+	    gretl_errmsg_sprintf(_("Couldn't open %s"), fname);
 	    close(fd);
 	    gretl_remove(fname);
 	}
@@ -2190,10 +2201,10 @@ static int db_delete_series (char *line, const int *list,
 
     if (fname == NULL) {
 	if (*db_name == '\0') {
-	    strcpy(gretl_errmsg, _("No database has been opened"));
+	    gretl_errmsg_set(_("No database has been opened"));
 	    err = 1;
 	} else if (db_type != GRETL_NATIVE_DB) {
-	    strcpy(gretl_errmsg, "This only works for gretl databases");
+	    gretl_errmsg_set("This only works for gretl databases");
 	    err = 1;
 	} else {
 	    err = open_native_db_files(db_name, &fidx, src1, &fbin, src2);
@@ -2206,14 +2217,14 @@ static int db_delete_series (char *line, const int *list,
 	return err;
     }
 
-    strcpy(tmp1, gretl_dot_dir());
+    strcpy(tmp1, gretl_dotdir());
     strcat(tmp1, "tmpidx");
     f1 = tempfile_open(tmp1, &err);
     if (err) {
 	goto bailout;
     }
 
-    strcpy(tmp2, gretl_dot_dir());
+    strcpy(tmp2, gretl_dotdir());
     strcat(tmp2, "tmpbin");
     f2 = tempfile_open(tmp2, &err);
     if (err) {
@@ -2357,9 +2368,9 @@ int db_range_check (SERIESINFO *sinfo, DATAINFO *pdinfo)
     int err = 0;
 
     if (sd0 > sdn_orig || sdn < pdinfo->sd0) {
-	sprintf(gretl_errmsg, _("%s: observation range does not overlap\n"
-				"with the working data set"),
-		sinfo->varname);
+	gretl_errmsg_sprintf(_("%s: observation range does not overlap\n"
+			       "with the working data set"),
+			     sinfo->varname);
 	err = 1;
     }
 
@@ -2373,8 +2384,8 @@ int check_db_import (SERIESINFO *sinfo, DATAINFO *pdinfo)
     if (sinfo->pd < pdinfo->pd) {
 	if (sinfo->pd != 1 && sinfo->pd != 4 && 
 	    pdinfo->pd != 4 && pdinfo->pd != 12) {
-	    sprintf(gretl_errmsg, _("%s: can't handle conversion"),
-		    sinfo->varname);
+	    gretl_errmsg_sprintf(_("%s: can't handle conversion"),
+				 sinfo->varname);
 	    err = 1;
 	} 
     }
@@ -2386,7 +2397,6 @@ int check_db_import (SERIESINFO *sinfo, DATAINFO *pdinfo)
 #if DB_DEBUG
     if (err) {
 	fprintf(stderr, "check_db_import: err = %d\n", err);
-	fprintf(stderr, "%s\n", gretl_errmsg);
 	fprintf(stderr, "(pdinfo->n = %d)\n", pdinfo->n);
     }
 #endif
@@ -2441,11 +2451,11 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	pdinfo->v = 2;
 	dbv = 1;
 	if (start_new_Z(pZ, pdinfo, 0)) {
-	    strcpy(gretl_errmsg, _("Out of memory!"));
+	    gretl_errmsg_set(_("Out of memory!"));
 	    return 1;
 	}
     } else if (new && dataset_add_series(1, pZ, pdinfo)) {
-	strcpy(gretl_errmsg, _("Out of memory!"));
+	gretl_errmsg_set(_("Out of memory!"));
 	return 1;
     }
 
@@ -2459,7 +2469,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
     if (sinfo->pd > pdinfo->pd) {
 	if (pdinfo->pd != 1 && pdinfo->pd != 4 &&
 	    sinfo->pd != 12) {
-	    strcpy(gretl_errmsg, _("Sorry, can't handle this conversion yet!"));
+	    gretl_errmsg_set(_("Sorry, can't handle this conversion yet!"));
 	    if (new) {
 		dataset_drop_last_variables(1, pZ, pdinfo);
 	    }
@@ -2473,7 +2483,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	}
 	xvec = compact_db_series(dbZ[1], sinfo, pdinfo->pd, method);
 	if (xvec == NULL) {
-	    strcpy(gretl_errmsg, _("Out of memory!"));
+	    gretl_errmsg_set(_("Out of memory!"));
 	    if (new) {
 		dataset_drop_last_variables(1, pZ, pdinfo);
 	    }
@@ -2989,7 +2999,7 @@ weeks_to_months_exec (double **mZ, const double **Z, const DATAINFO *pdinfo,
 
     for (s=0; s<pdinfo->n; s++) {
 	/* loop across the weekly obs in this month */
-	ntodate_full(obsstr, s, pdinfo);
+	ntodate(obsstr, s, pdinfo);
 	sscanf(obsstr, "%d/%d/%d", &yr, &mon, &day);
 	if (monbak > 0 && mon != monbak) {
 	    /* new month: finalize the previous one */
@@ -3061,7 +3071,7 @@ weeks_to_months_check (const DATAINFO *pdinfo, int *startyr, int *endyr,
     int t, err = 0;
 
     for (t=0; t<pdinfo->n; t++) {
-	ntodate_full(obsstr, t, pdinfo);
+	ntodate(obsstr, t, pdinfo);
 	if (sscanf(obsstr, "%d/%d/%d", &yr, &mon, &day) != 3) {
 	    err = 1;
 	    break;
@@ -3173,7 +3183,7 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
     fprintf(stderr, "daily_dataset_to_weekly: repday = %d\n", repday);
 
     for (t=0; t<pdinfo->n; t++) {
-	ntodate_full(obs, t, pdinfo);
+	ntodate(obs, t, pdinfo);
 	wday = get_day_of_week(obs);
 	if (wday == repday) {
 	    ok = 0;
@@ -3212,7 +3222,7 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
 	int s = 0;
 
 	for (t=0; t<pdinfo->n; t++) {
-	    ntodate_full(obs, t, pdinfo);
+	    ntodate(obs, t, pdinfo);
 	    wday = get_day_of_week(obs);
 	    if (wday == repday) {
 		x[s++] = Z[i][t];
@@ -3239,7 +3249,7 @@ static int daily_dataset_to_weekly (double **Z, DATAINFO *pdinfo,
 	pdinfo->sd0 = get_date_x(pdinfo->pd, pdinfo->stobs);
 	pdinfo->t1 = 0;
 	pdinfo->t2 = pdinfo->n - 1;
-	ntodate_full(pdinfo->endobs, pdinfo->t2, pdinfo);
+	ntodate(pdinfo->endobs, pdinfo->t2, pdinfo);
 
 	dataset_destroy_obs_markers(pdinfo);
     }    
@@ -3356,7 +3366,7 @@ insert_missing_hidden_obs (double ***pZ, DATAINFO *pdinfo,
     if (!err) {
 	dataset_destroy_obs_markers(pdinfo);
 	pdinfo->t2 = pdinfo->n - 1;
-	ntodate_full(pdinfo->endobs, pdinfo->n - 1, pdinfo);
+	ntodate(pdinfo->endobs, pdinfo->n - 1, pdinfo);
     }
 
     return err;
@@ -3420,7 +3430,7 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 	*/
 	err = maybe_expand_daily_data(pZ, pdinfo);
 	if (err) {
-	    strcpy(gretl_errmsg, "Error expanding daily data with missing observations");
+	    gretl_errmsg_set("Error expanding daily data with missing observations");
 	    return err;
 	}
     }
@@ -3447,6 +3457,12 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 	} else {
 	    startmin = 1;
 	}
+    } else if (oldpd == 24 && newpd >= 5 && newpd <= 7) {
+	/* hourly to daily */
+	compfac = 24;
+	if (!get_obs_maj_min(pdinfo->stobs, &startmaj, &startmin)) {
+	    return 1;
+	}
     } else {
 	compfac = oldpd / newpd;
 	/* get starting obs major and minor components */
