@@ -77,6 +77,15 @@ struct ODBC_info_ {
     int ncols;
 };
 
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+typedef struct {
+    long frac;
+    short exp;
+} netfloat;
+
+float retrieve_float (netfloat nf);
+#endif
+
 int get_native_db_data (const char *dbbase, SERIESINFO *sinfo, 
 			double **Z);
 
@@ -101,8 +110,7 @@ double *compact_db_series (const double *src, SERIESINFO *sinfo,
 double *expand_db_series (const double *src, SERIESINFO *sinfo,
 			  int target_pd);
 
-int set_db_name (const char *fname, int filetype, const PATHS *ppaths, 
-		 PRN *prn);
+int set_db_name (const char *fname, int filetype, PRN *prn);
 
 const char *get_db_name (void);
 
