@@ -17,8 +17,6 @@
  * 
  */
 
-/* discrete.c for gretl: logit and probit models */
-
 #include "libgretl.h"
 #include "libset.h"
 #include "missing_private.h"
@@ -26,12 +24,1085 @@
 
 #include <errno.h>
 
-#define TINY 1.0e-13
+#define LPDEBUG 0
+
+#define CHOL_TINY 1.0e-13
+
+typedef struct op_container_ op_container;
+
+/* structure for handling ordered probit or logit */
+
+struct op_container_ {
+    int ci;           /* model command index (PROBIT or LOGIT) */
+    gretlopt opt;     /* option flags */
+    int *y;           /* dependent variable */
+    double **Z;       /* data */
+    int *list;        /* dependent var plus regular regressors */
+    int ymax;         /* max of (possibly normalized) y */
+    int t1;           /* beginning of sample */
+    int t2;           /* end of sample */
+    int nobs;         /* number of observations */
+    int nx;           /* number of explanatory variables */
+    int k;            /* total number of parameters */
+    double *theta;    /* real parameter estimates */
+    double *ndx;      /* index variable */
+    double *dP;       /* probabilities */
+    MODEL *pmod;      /* model struct, initially containing OLS */
+    double **G;       /* score matrix by observation */
+    double *g;        /* total score vector */
+};
+
+struct sorter {
+    double x;
+    int t;
+};
 
 static int neginv (const double *xpx, double *diag, int nv);
 static int cholesky_decomp (double *xpx, int nv);
 
-#define LPDEBUG 0
+static double lp_cdf (double x, int ci)
+{
+    switch (ci) {
+    case PROBIT:
+	return normal_cdf(x);
+    case LOGIT:
+	return 1.0 / (1.0 + exp(-x));
+    default:
+	return NADBL;
+    }
+}
+
+static double lp_pdf (double x, int ci)
+{
+    double tmp;
+
+    switch (ci) {
+    case PROBIT:
+	return normal_pdf(x);
+    case LOGIT:
+	tmp = 1.0 + exp(-x);
+	return (tmp - 1.0) / (tmp * tmp);
+    default:
+	return NADBL;
+    }
+}
+
+static void op_container_destroy (op_container *OC)
+{
+    free(OC->y);
+    free(OC->ndx);
+    free(OC->dP);
+    free(OC->list);
+    doubles_array_free(OC->G, OC->k);
+    free(OC->g);
+    free(OC->theta);
+
+    free(OC);
+}
+
+static op_container *op_container_new (int ci, int ndum,
+				       double **Z, MODEL *pmod,  
+				       gretlopt opt)
+{
+    op_container *OC;
+    int i, t, vy = pmod->list[1];
+    int nobs = pmod->nobs;
+
+    OC = malloc(sizeof *OC);
+    if (OC == NULL) {
+	return NULL;
+    }
+
+    OC->ci = ci;
+
+    OC->Z = Z;
+    OC->pmod = pmod;
+    OC->t1 = pmod->t1;
+    OC->t2 = pmod->t2;
+    OC->nobs = nobs;
+    OC->k = pmod->ncoeff;
+    OC->ymax = ndum;
+    OC->nx = OC->k - ndum;
+
+    OC->opt = opt;
+
+    OC->y = NULL;
+    OC->ndx = NULL;
+    OC->dP = NULL;
+    OC->list = NULL;
+    OC->G = NULL;
+    OC->g = NULL;
+
+    OC->y = malloc(nobs * sizeof *OC->y);
+    OC->ndx = malloc(nobs * sizeof *OC->ndx);
+    OC->dP = malloc(nobs * sizeof *OC->dP);
+
+    OC->list = gretl_list_new(1 + OC->nx);
+    OC->G = doubles_array_new(OC->k, nobs);
+    OC->g = malloc(OC->k * sizeof *OC->g);
+    OC->theta = malloc(OC->k * sizeof *OC->theta);
+
+    if (OC->y == NULL || OC->ndx == NULL || 
+	OC->dP == NULL || OC->list == NULL ||
+	OC->G == NULL || OC->g == NULL ||
+	OC->theta == NULL) {
+	op_container_destroy(OC);
+	return NULL;
+    }
+
+    i = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t])) {
+	    OC->y[i++] = (int) Z[vy][t];
+	}
+    }
+
+    OC->list[1] = vy;
+    for (i=0; i<OC->nx; i++) {
+	OC->list[i+2] = pmod->list[i+2];
+    }
+
+#if LPDEBUG
+    fprintf(stderr, "nobs = %d\n", OC->nobs);
+    fprintf(stderr, "t1-t2 = %d-%d\n", OC->t1, OC->t2);
+    fprintf(stderr, "k = %d\n", OC->k);
+    fprintf(stderr, "nx = %d\n", OC->nx);
+    fprintf(stderr, "Max(y) = M = %d\n", OC->ymax);
+    printlist(OC->list, "list, in op_container_new");
+#endif
+
+    return OC;
+}
+
+static int op_compute_score (op_container *OC, int yt, 
+			     double ystar0, double ystar1,
+			     double dP, int t, int s)
+{
+    double dm, mills0, mills1;
+    int M = OC->ymax;
+    int i, v;
+
+    if (ystar1 < 6.0 || OC->ci == LOGIT) {
+	mills0 = (yt == 0)? 0.0 : lp_pdf(ystar0, OC->ci) / dP;
+	mills1 = (yt == M)? 0.0 : lp_pdf(ystar1, OC->ci) / dP;
+    } else { 
+	/* L'Hopital-based approximation */
+	mills0 = (yt == 0)? 0.0 : -ystar0;
+	mills1 = (yt == M)? 0.0 : -ystar1;
+    }
+
+    dm = mills1 - mills0;
+
+    for (i=0; i<OC->nx; i++) {
+	v = OC->list[i+2];
+	OC->G[i][s] = -dm * OC->Z[v][t];
+	OC->g[i] += OC->G[i][s];
+    }
+
+    for (i=OC->nx; i<OC->k; i++) {
+	OC->G[i][s] = 0.0;
+	if (i == OC->nx + yt - 1) {
+	    OC->G[i][s] = -mills0;
+	    OC->g[i] += OC->G[i][s];
+	}
+	if (i == OC->nx + yt) {
+	    OC->G[i][s] = mills1;
+	    OC->g[i] += OC->G[i][s];
+	}
+    }
+
+    return 0;
+}
+
+#define dPMIN 1.0e-15
+
+static int op_compute_probs (const double *theta, op_container *OC)
+{
+    double m0, m1, ystar0 = 0.0, ystar1 = 0.0;
+    int M = OC->ymax;
+    int nx = OC->nx;
+    double P0, P1, h, adj, dP;
+    int i, t, s, yt;
+
+    /* initialize analytical score */
+    for (i=0; i<OC->k; i++) {
+	OC->g[i] = 0.0;
+    }
+
+    s = 0;
+
+    for (t=OC->pmod->t1; t<=OC->pmod->t2; t++) {
+	if (na(OC->pmod->uhat[t])) {
+#if LPDEBUG > 1
+	    fprintf(stderr, "obs %4d excluded\n", t);
+#endif
+	    continue;
+	}
+
+	yt = OC->y[s];
+
+	if (yt == 0) {
+	    m0 = theta[nx];
+	    ystar1 = OC->ndx[s] + m0;
+	} else {
+	    m0 = theta[nx + yt - 1];
+	    ystar0 = OC->ndx[s] + m0;
+	    if (yt < M) {
+		m1 = theta[nx + yt];
+		ystar1 = OC->ndx[s] + m1;
+	    }
+	} 
+
+#if LPDEBUG > 1
+	fprintf(stderr, "t:%4d/%d s=%d y=%d, ndx = %10.6f, ystar0 = %9.7f, ystar1 = %9.7f\n", 
+		t, OC->nobs, s, yt, OC->ndx[s], ystar0, ystar1);
+#endif
+
+	if (ystar0 < 6.0 || OC->ci == LOGIT) {
+	    P0 = (yt == 0)? 0.0 : lp_cdf(ystar0, OC->ci);
+	    P1 = (yt == M)? 1.0 : lp_cdf(ystar1, OC->ci);
+	    dP = P1 - P0;
+	} else { 
+	    /* Taylor-based 1st order approximation */
+	    h = ystar1 - ystar0;
+	    adj = lp_pdf(ystar1, OC->ci) + lp_pdf(ystar0, OC->ci);
+	    dP =  0.5 * h * adj;
+	}
+
+	if (dP > dPMIN) {
+	    OC->dP[s] = dP;
+	} else {
+#if LPDEBUG
+	    fprintf(stderr, "very small dP at obs %d; y=%d, ndx=%g, dP=%g\n", 
+ 		    t, yt, OC->ndx[s], dP);
+#endif
+	    return 1;
+	} 
+
+	op_compute_score(OC, yt, ystar0, ystar1, dP, t, s);
+
+	s++;
+    }
+
+    return 0;
+}
+
+/* Below: method for getting around the "non-increasing cut point"
+   issue in ordered models by construction: the 2nd and higher cut
+   points are represented to BFGS in the form of the log-difference
+   from the previous cut point.
+*/
+
+static void op_make_BFGS_theta (op_container *OC, double *theta)
+{
+    int i;
+
+    for (i=0; i<=OC->nx; i++) {
+	theta[i] = OC->theta[i];
+    }
+
+    for (i=OC->nx+1; i<OC->k; i++) {
+	/* convert cut point 2 and higher to log-difference form */
+	theta[i] = log(OC->theta[i] - OC->theta[i-1]);
+    }
+}
+
+/* Inverse operation for the transformation done by
+   op_make_BFGS_theta() */
+
+static void op_get_real_theta (op_container *OC, const double *theta)
+{
+    int i;
+
+    for (i=0; i<=OC->nx; i++) {
+	OC->theta[i] = theta[i];
+    }
+
+    for (i=OC->nx+1; i<OC->k; i++) {
+	/* retrieve cut point 2 and higher from log-difference form */
+	OC->theta[i] = exp(theta[i]) + OC->theta[i-1];
+    }
+}
+
+static double op_loglik (const double *theta, void *ptr)
+{
+    op_container *OC = (op_container *) ptr;
+    double x, ll = 0.0;
+    int i, s, t, v;
+    int err;
+
+    if (theta != OC->theta) {
+	op_get_real_theta(OC, theta);
+    }
+
+    s = 0;
+    for (t=OC->t1; t<=OC->t2; t++) {
+	if (na(OC->pmod->uhat[t])) {
+	    continue;
+	}
+	x = 0.0;
+	for (i=0; i<OC->nx; i++) {
+	    /* the independent variables */
+	    v = OC->list[i+2];
+	    x -= OC->theta[i] * OC->Z[v][t];
+	}
+	OC->ndx[s++] = x;
+#if LPDEBUG > 2
+	fprintf(stderr, "t = %d, s = %d, x = %g\n", t, s, x);
+#endif
+    }
+    
+    err = op_compute_probs(OC->theta, OC);
+
+    if (err) {
+	ll = NADBL;
+    } else {
+	s = 0;
+	for (t=OC->t1; t<=OC->t2; t++) {
+	    if (!na(OC->pmod->uhat[t])) {
+		ll += log(OC->dP[s++]);
+	    }
+	}
+    }
+
+#if LPDEBUG > 1
+    fprintf(stderr, "ll = %16.10f\n", ll);
+#endif
+
+    return ll;
+}
+
+static int op_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll, 
+		     void *ptr)
+{
+    op_container *OC = (op_container *) ptr;
+    int i, j;
+
+    for (i=0; i<npar; i++) {
+	s[i] = OC->g[i];
+    }
+
+    for (i=OC->nx; i<npar; i++) {
+	for (j=i+1; j<npar; j++) {
+	    /* add effects of changes in subsequent cut points */
+	    s[i] += OC->g[j];
+	}
+	if (i > OC->nx) {
+	    s[i] *= exp(theta[i]); /* apply jacobian */
+	}
+    }
+
+    return 1;
+}
+
+static int opg_from_ascore (op_container *OC, gretl_matrix *GG) 
+{
+    int s, t, i, j, k = OC->k;
+    double x, ll, *g0;
+
+    g0 = malloc(k * sizeof *g0);
+    if (g0 == NULL) {
+	return E_ALLOC;
+    }
+
+    ll = op_loglik(OC->theta, OC);
+
+    for (j=0; j<k; j++) {
+	g0[j] = OC->g[j];
+    }
+
+    for (i=0; i<k; i++) {
+	for (j=i; j<k; j++) {
+	    x = 0.0;
+	    s = 0;
+	    for (t=OC->t1; t<=OC->t2; t++) {
+		if (na(OC->pmod->uhat[t])) {
+		    continue;
+		}
+		x += OC->G[i][s] * OC->G[j][s];
+		s++;
+	    }
+	    gretl_matrix_set(GG, j, i, x);
+	    gretl_matrix_set(GG, i, j, x);
+	}
+    }
+
+    free(g0);
+
+    return 0;
+}
+
+static int ihess_from_ascore (op_container *OC, gretl_matrix *inH) 
+{
+    int i, j, err, k = OC->k;
+    double smal = 1.0e-07;  /* "small" is some sort of macro on win32 */
+    double smal2 = 2.0 * smal;
+    double ti, x, ll, *g0;
+
+    g0 = malloc(k * sizeof *g0);
+    if (g0 == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=0; i<k; i++) {
+	ti = OC->theta[i];
+	OC->theta[i] -= smal;
+	ll = op_loglik(OC->theta, OC);
+	for (j=0; j<k; j++) {
+	    g0[j] = OC->g[j];
+	}
+	OC->theta[i] += smal2;
+	ll = op_loglik(OC->theta, OC);
+	for (j=0; j<k; j++) {
+	    x = (OC->g[j] - g0[j]) / smal2;
+	    gretl_matrix_set(inH, i, j, -x);
+	}
+	/* restore original theta */
+	OC->theta[i] = ti;
+    }
+
+    gretl_matrix_xtr_symmetric(inH);
+
+    free(g0);
+
+#if LPDEBUG
+    gretl_matrix_print(inH, "inverse of Hessian");
+#endif
+
+    err = gretl_invert_symmetric_matrix(inH);
+    if (err) {
+	fprintf(stderr, "ihess_from_ascore: failed to invert numerical Hessian\n");
+    }
+
+    return err;
+}
+
+/* Get the predicted value of y; that is, the value for which the estimated 
+   probability is greatest.  This is also used in forecast.c */
+
+double ordered_model_prediction (const MODEL *pmod, double Xb)
+{
+    /* position of least cut point in coeff array */
+    int k = gretl_model_get_int(pmod, "nx");
+    int maxval = pmod->ncoeff - k;
+    double prob, pmax, cut;
+    double CDF, CDFbak;
+    int i, pred = 0;
+
+    cut = pmod->coeff[k];
+    pmax = CDFbak = lp_cdf(cut - Xb, pmod->ci);
+
+    for (i=1; i<maxval; i++) {
+	cut = pmod->coeff[++k];
+	CDF = lp_cdf(cut - Xb, pmod->ci);
+	prob = CDF - CDFbak;
+	if (prob > pmax) {
+	    pmax = prob;
+	    pred = i;
+	}
+	CDFbak = CDF;
+    }
+
+    prob = 1 - CDFbak;
+    if (prob > pmax) {
+	pred = maxval;
+    }
+
+    return (double) pred;
+}
+
+/* compute generalized residual for ordered models */
+
+static double op_gen_resid (op_container *OC, const double *theta, int t) 
+{
+    double ndxt, m0, m1, ystar0, f0, f1;
+    double ret, dP, ystar1 = 0.0;
+    int M = OC->ymax;
+    int nx = OC->nx;
+    int yt;
+
+    dP = OC->dP[t];
+    yt = OC->y[t];
+    ndxt = OC->ndx[t];
+
+    if (yt == 0) {
+	m0 = theta[nx];
+	ystar1 = ndxt + m0;
+    } else {
+	m0 = theta[nx + yt - 1];
+	ystar0 = ndxt + m0;
+	if (yt < M) {
+	    m1 = theta[nx + yt];
+	    ystar1 = ndxt + m1;
+	}
+    } 
+
+    if (ystar1 < 6.0 || OC->ci == LOGIT || 1) {
+	f0 = (yt == 0)? 0.0 : lp_pdf(ystar0, OC->ci) / dP;
+	f1 = (yt == M)? 0.0 : lp_pdf(ystar1, OC->ci) / dP;
+    } else { 
+	/* L'Hopital-based approximation */
+	f0 = (yt == 0)? 0.0 : -ystar0;
+	f1 = (yt == M)? 0.0 : -ystar1;
+    }
+
+    ret = (f0 - f1);
+
+    return ret;
+} 
+
+/* Initialize the cut-points by counting the occurrences of each value
+   of the (normalized) dependent variable, finding the sample
+   proportion (cumulating as we go), and taking the inverse of the
+   normal CDF.
+*/
+
+static void cut_points_init (op_container *OC, const MODEL *pmod, 
+			     const double **Z)
+{
+    const double *y = Z[pmod->list[1]];
+    double p = 0.0;
+    int i, j, t, nj;
+
+    for (i=OC->nx, j=0; i<OC->k; i++, j++) {
+	nj = 0;
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (!na(pmod->uhat[t]) && y[t] == j) {
+		nj++;
+	    }
+	}	
+	p += (double) nj / pmod->nobs;
+	OC->theta[i] = normal_cdf_inverse(p);
+    }
+}
+
+static void op_LR_test (MODEL *pmod, op_container *OC, 
+			const double **Z)
+{
+    int nx = OC->nx;
+    double L0;
+
+    OC->k -= OC->nx;
+    OC->nx = 0;
+
+    cut_points_init(OC, pmod, Z);
+
+    L0 = op_loglik(OC->theta, OC);
+
+    if (!na(L0) && L0 <= pmod->lnL) {
+	pmod->chisq = 2.0 * (pmod->lnL - L0);
+	gretl_model_set_int(pmod, "lr_df", nx);
+    }
+
+    /* restore original data on OC */
+    OC->nx = nx;
+    OC->k += nx;
+}
+
+static void fill_op_model (MODEL *pmod, const int *list,
+			   const double **Z, const DATAINFO *pdinfo, 
+			   op_container *OC, gretl_matrix *V, 
+			   int fnc, int grc)
+{
+    int npar = OC->k;
+    int nx = OC->nx;
+    int correct = 0;
+    double Xb;
+    int i, s, t, v;
+
+    pmod->ci = OC->ci;
+
+    gretl_model_set_int(pmod, "ordered", 1);
+    gretl_model_set_int(pmod, "nx", OC->nx);
+    gretl_model_set_int(pmod, "fncount", fnc);
+    gretl_model_set_int(pmod, "grcount", grc);
+
+    pmod->ncoeff = npar;
+
+    if (V != NULL) {
+	pmod->errcode = gretl_model_write_vcv(pmod, V);
+	if (pmod->errcode) {
+	    return;
+	}
+    }
+
+    for (i=0; i<npar; i++) {
+	pmod->coeff[i] = OC->theta[i];
+    }
+
+    if (OC->opt & OPT_R) {
+	gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
+	pmod->opt |= OPT_R;
+    }
+
+    s = 0;
+    for (t=OC->t1; t<=OC->t2; t++) {
+	if (na(OC->pmod->uhat[t])) {
+	    continue;
+	}
+
+	Xb = 0.0;
+	for (i=0; i<OC->nx; i++) {
+	    v = OC->list[i+2];
+	    Xb += OC->theta[i] * OC->Z[v][t];
+	}
+
+	/* yhat = X\hat{beta} */
+	pmod->yhat[t] = Xb;
+	if (ordered_model_prediction(pmod, Xb) == OC->y[s]) {
+	    correct++;
+	}
+
+	/* compute generalized residual */
+	pmod->uhat[t] = op_gen_resid(OC, OC->theta, s);
+	s++;
+    }
+
+    gretl_model_set_int(pmod, "correct", correct);
+
+    pmod->lnL = op_loglik(OC->theta, OC);
+    mle_criteria(pmod, 0);
+    pmod->rsq = pmod->adjrsq = NADBL;
+
+    gretl_model_allocate_params(pmod, npar);
+
+    if (pmod->errcode == 0) {
+	for (i=0; i<nx; i++) {
+	    v = OC->list[i+2];
+	    strcpy(pmod->params[i], pdinfo->varname[v]);
+	}
+	s = 1;
+	for (i=nx; i<npar; i++) {
+	    sprintf(pmod->params[i], "cut%d", s++);
+	}
+    }
+
+    /* trim the model list: remove references to the 'cut'
+       dummy variables */
+    for (i=pmod->list[0]; i>1; i--) {
+	if (!in_gretl_list(list, pmod->list[i])) {
+	    gretl_list_delete_at_pos(pmod->list, i);
+	}
+    }
+
+    if (nx > 0) {
+	op_LR_test(pmod, OC, Z);
+	gretl_model_set_coeff_separator(pmod, NULL, nx);
+    }
+}
+
+static gretl_matrix *oprobit_vcv (op_container *OC, int *err)
+{
+    gretl_matrix *V;
+    int k = OC->k;
+
+    V = gretl_matrix_alloc(k, k);
+    if (V == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    /* hessian from analytical score */
+    *err = ihess_from_ascore(OC, V);
+
+    if (!*err && (OC->opt & OPT_R)) {
+	gretl_matrix *GG = NULL;
+	gretl_matrix *Vr = NULL;
+
+	/* sandwich of hessian and OPG */
+
+	GG = gretl_matrix_alloc(k, k);
+	Vr = gretl_matrix_alloc(k, k);
+
+	if (GG == NULL || Vr == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    *err = opg_from_ascore(OC, GG);
+	    if (!*err) {
+#if LPDEBUG > 1
+		gretl_matrix_print(GG, "OPG matrix");
+#endif
+		gretl_matrix_qform(V, GRETL_MOD_NONE,
+				   GG, Vr, GRETL_MOD_NONE);
+		gretl_matrix_copy_values(V, Vr);
+	    }
+	}
+
+	gretl_matrix_free(GG);
+	gretl_matrix_free(Vr);
+    } 
+
+    if (*err) {
+	gretl_matrix_free(V);
+	V = NULL;
+    }
+
+#if LPDEBUG > 1
+    gretl_matrix_print(V, "Covariance matrix");
+#endif
+
+    return V;
+}
+
+/* Main ordered estimation function */
+
+static int do_ordered (int ci, int ndum, 
+		       double **Z, DATAINFO *pdinfo, 
+		       MODEL *pmod, const int *list, 
+		       gretlopt opt, PRN *prn)
+{
+    int maxit = 1000;
+    int fncount = 0;
+    int grcount = 0;
+    op_container *OC;
+    int i, npar;
+    gretl_matrix *V = NULL;
+    double *theta = NULL;
+    double toler;
+    int err;
+
+    OC = op_container_new(ci, ndum, Z, pmod, opt);
+    if (OC == NULL) {
+	return E_ALLOC;
+    }
+
+    npar = OC->k;
+
+    /* transformed theta to pass to BFGS */
+    theta = malloc(npar * sizeof *theta);
+    if (theta == NULL) {
+	op_container_destroy(OC);
+	return E_ALLOC;
+    }
+
+    /* initialize slopes */
+    for (i=0; i<OC->nx; i++) {
+	OC->theta[i] = 0.0001;
+    }
+
+    /* initialize cut points */
+    cut_points_init(OC, pmod, (const double **) Z);
+
+    /* transform theta to log-diff form */
+    op_make_BFGS_theta(OC, theta);
+
+#if LPDEBUG
+    for (i=0; i<npar; i++) {
+	fprintf(stderr, "theta[%d]: 'real' = %g, BFGS = %g\n", i, OC->theta[i], theta[i]);
+    }
+    fprintf(stderr, "\ninitial loglikelihood = %.12g\n", 
+	    op_loglik(theta, OC));
+#endif
+
+    BFGS_defaults(&maxit, &toler, PROBIT);
+
+    err = BFGS_max(theta, npar, maxit, toler, 
+		   &fncount, &grcount, op_loglik, C_LOGLIK,
+		   op_score, OC, (prn != NULL)? OPT_V : OPT_NONE,
+		   prn);
+
+    if (err) {
+	goto bailout;
+    } else {
+	fprintf(stderr, "Number of iterations = %d (%d)\n", fncount, grcount);
+    }
+
+    /* transform back to 'real' theta */
+    op_get_real_theta(OC, theta);
+
+    V = oprobit_vcv(OC, &err);
+
+    if (!err) {
+	fill_op_model(pmod, list, (const double **) Z, pdinfo, 
+		      OC, V, fncount, grcount);
+    }
+
+ bailout:
+
+    free(theta);
+    gretl_matrix_free(V);
+    op_container_destroy(OC);
+
+    return err;
+}
+
+/* We want to ensure that the values of the dependent variable
+   actually used in the analysis (after dropping any bad
+   observations) form a zero-based series of consecutive 
+   integers.
+*/
+
+static int maybe_fix_op_depvar (MODEL *pmod, double **Z,
+				     const DATAINFO *pdinfo,
+				     double **orig_y, int *ndum)
+{
+    struct sorter *s = NULL;
+    double *sy = NULL;
+    double nexty, bady;
+    int dv = pmod->list[1];
+    int i, t, n = 0;
+    int fixit = 0;
+    int err = 0;
+
+    /* make a copy of the observations on the dep. var. that
+       were actually used in the initial OLS, recording
+       the observation numbers, then sort
+    */
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t])) {
+	    n++;
+	}
+    }
+
+    s = malloc(n * sizeof *s);
+    if (s == NULL) {
+	return E_ALLOC;
+    }
+
+    i = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t])) {
+	    s[i].x = Z[dv][t];
+	    s[i].t = t;
+	    i++;
+	}
+    }
+
+    qsort(s, n, sizeof *s, gretl_compare_doubles);
+
+    /* normalize to minimum zero */
+    if (s[0].x != 0) {
+	double ymin = s[0].x;
+
+	for (i=0; i<n && s[i].x == ymin; i++) {
+	    s[i].x = 0.0;
+	}
+	fixit = 1;
+    }
+
+    /* ensure that the sorted values increase by steps of one */
+    for (i=1; i<n; i++) {
+	if (s[i].x != s[i-1].x) {
+	    nexty = s[i-1].x + 1;
+	    if (s[i].x != nexty) {
+		bady = s[i].x;
+		while (i < n && s[i].x == bady) {
+		    s[i++].x = nexty;
+		}
+		i--; /* compensate for outer i++ */
+		fixit = 1;
+	    } 
+	} 
+    }
+
+    /* the number of dummies actually used will equal the
+       max of normalized y
+    */
+    *ndum = (int) s[n-1].x;
+
+    if (fixit) {
+	/* the dependent var needs transforming */
+	sy = copyvec(Z[dv], pdinfo->n);
+	if (sy == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    for (i=0; i<n; i++) {
+		sy[s[i].t] = s[i].x;
+	    }
+	    /* back up the original dep. var and replace it for
+	       the duration of the ordered analysis */
+	    *orig_y = Z[dv];
+	    Z[dv] = sy;
+	}
+    }
+
+    if (fixit) {
+	fputs("oprobit: using normalized y\n", stderr);
+    } else {
+	fputs("oprobit: using original y\n", stderr);
+    }
+
+    free(s);
+
+    return err;
+}
+
+static void restore_depvar (double **Z, double *y, int v)
+{
+    free(Z[v]);
+    Z[v] = y;
+}
+
+static int *make_dummies_list (const int *list, 
+			       double ***pZ, DATAINFO *pdinfo,
+			       int *err)
+{
+    int *dumlist = gretl_list_new(1);
+
+    if (dumlist == NULL) {
+	*err = E_ALLOC;
+    } else {
+	dumlist[1] = list[1];
+
+	/* OPT_F -> drop first value */
+	*err = list_dumgenr(&dumlist, pZ, pdinfo, OPT_F);
+	if (*err) {
+	    free(dumlist);
+	    dumlist = NULL;
+	}
+    }
+
+    return dumlist;
+}
+
+/* make internal regression list for ordered model */
+
+static int *make_op_list (const int *list, double ***pZ,
+			  DATAINFO *pdinfo, int **pdumlist,
+			  int *err)
+{
+    int *dumlist;
+    int *biglist;
+    int i, k, nv;
+
+    dumlist = make_dummies_list(list, pZ, pdinfo, err);
+    if (dumlist == NULL) {
+	return NULL;
+    }
+
+    nv = list[0] + dumlist[0];
+
+    biglist = gretl_list_new(nv);
+    if (biglist == NULL) {
+	free(dumlist);
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    k = 1;
+    for (i=1; i<=list[0]; i++) {
+	biglist[k++] = list[i];
+    }
+    for (i=1; i<=dumlist[0]; i++) {
+	biglist[k++] = dumlist[i];
+    }
+
+    *pdumlist = dumlist;
+
+    return biglist;
+}
+
+static void list_purge_const (int *list)
+{
+    int i;
+
+    for (i=2; i<=list[0]; i++) {
+	if (list[i] == 0) {
+	    gretl_list_delete_at_pos(list, i);
+	    return;
+	}
+    }
+}
+
+static int check_for_missing_dummies (MODEL *pmod, int *dlist)
+{
+    int i, dv;
+
+    for (i=1; i<=dlist[0]; i++) {
+	dv = dlist[i];
+	if (!in_gretl_list(pmod->list, dv)) {
+	    fprintf(stderr, "check for missing dummies: var %d is gone!\n", dv);
+	    return E_SINGULAR;
+	}
+    }
+
+    return 0;
+}
+
+/* driver function for ordered logit/probit: note, prn is non-NULL
+   only if the verbose option has been selected
+*/
+
+static MODEL ordered_estimate (int *list, int ci, 
+			       double ***pZ, DATAINFO *pdinfo,
+			       gretlopt opt, PRN *prn) 
+{
+    MODEL model;
+    int orig_v = pdinfo->v;
+    double *orig_y = NULL;
+    int *biglist = NULL;
+    int *dumlist = NULL;
+    int ndum = 0;
+
+    gretl_model_init(&model);
+
+    /* remove the constant from the incoming list, if present */
+    list_purge_const(list);
+
+    /* construct augmented regression list, including dummies 
+       for the level of the dependent variable
+    */
+    biglist = make_op_list(list, pZ, pdinfo, &dumlist, &model.errcode);
+    if (model.errcode) {
+	return model;
+    }
+
+    /* run initial OLS, with dummies added */
+    model = lsq(biglist, pZ, pdinfo, OLS, OPT_A);
+    if (model.errcode) {
+	fprintf(stderr, "ordered_estimate: initial OLS failed\n");
+	free(dumlist);
+	free(biglist);
+	return model;
+    }
+
+#if LPDEBUG
+    pprintf(prn, "ordered_estimate: initial OLS\n");
+    printmodel(&model, pdinfo, OPT_S, prn);
+#endif
+
+    if (model.list[0] < biglist[0]) {
+	/* One or more regressors were dropped in OLS, most likely due
+	   to collinearity.  We can accept the dropping of
+	   user-selected regressors, but all the added dummies must be
+	   retained, so we'd better investigate.
+	*/
+	model.errcode = check_for_missing_dummies(&model, dumlist);
+    }
+	
+    if (!model.errcode) {
+	/* after accounting for any missing observations, check that
+	   the dependent variable is acceptable 
+	*/
+	model.errcode = maybe_fix_op_depvar(&model, *pZ, pdinfo,
+					    &orig_y, &ndum);
+    }
+
+    /* do the actual ordered probit analysis */
+    if (!model.errcode) {
+	clear_model_xpx(&model);
+	model.errcode = do_ordered(ci, ndum, *pZ, pdinfo, &model, list, 
+				   opt, prn);
+    }
+
+    free(dumlist);
+    free(biglist);
+
+    if (orig_y != NULL) {
+	/* if we messed with the dependent var, put the original back */
+	restore_depvar(*pZ, orig_y, list[1]);
+    }
+
+    if (pdinfo->v > orig_v) {
+	/* clean up any automatically-added dummies */
+	dataset_drop_last_variables(pdinfo->v - orig_v, pZ, pdinfo);
+    }
+
+    set_model_id(&model);
+
+    return model;
+}
 
 static double logit (double x)
 {
@@ -98,25 +1169,20 @@ static void Lr_chisq (MODEL *pmod, const double **Z)
     }
 }
 
-static double 
-logit_probit_llhood (const double *y, const MODEL *pmod, int opt)
+static double logit_probit_loglik (const double *y, const MODEL *pmod, 
+				   int ci)
 {
-    double q, lnL = 0.0;
+    double q, ll = 0.0;
     int t;
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	if (na(pmod->yhat[t])) {
-	    continue;
-	}
-	q = 2.0 * y[t] - 1.0;
-	if (opt == LOGIT) {
-	    lnL += log(logit(q * pmod->yhat[t]));
-	} else {
-	    lnL += log(normal_cdf(q * pmod->yhat[t]));
+	if (!na(pmod->yhat[t])) {
+	    q = 2.0 * y[t] - 1.0;
+	    ll += log(lp_cdf(q * pmod->yhat[t], ci));
 	}
     }
 
-    return lnL;
+    return ll;
 }
 
 static double *hess_wts (MODEL *pmod, const double **Z, int ci) 
@@ -151,7 +1217,7 @@ static double *hess_wts (MODEL *pmod, const double **Z, int ci)
     return w;
 }
 
-static double *hessian (MODEL *pmod, const double **Z, int ci) 
+static double *lp_hessian (MODEL *pmod, const double **Z, int ci) 
 {
     int i, j, li, lj, m, t;
     const int l0 = pmod->list[0];
@@ -282,32 +1348,6 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     return err;
 }
 
-static MODEL ordered_model (int *list, int ci, double ***pZ, 
-			    DATAINFO *pdinfo, gretlopt opt, PRN *prn)
-{
-    MODEL omod;
-    void *handle;
-    MODEL (* ordered_estimate) (int *, int, double ***, DATAINFO *, 
-				gretlopt, PRN *);
-
-    gretl_error_clear();
-
-    ordered_estimate = get_plugin_function("ordered_estimate", &handle);
-    if (ordered_estimate == NULL) {
-	gretl_model_init(&omod);
-	omod.errcode = E_FOPEN;
-	return omod;
-    }
-
-    omod = (*ordered_estimate) (list, ci, pZ, pdinfo, opt, prn);
-
-    close_plugin(handle);
-
-    set_model_id(&omod);
-
-    return omod;
-}
-
 /* calculate standard errors etc using the Hessian */
 
 static int logit_probit_vcv (MODEL *dmod, gretlopt opt, const double **Z)
@@ -323,9 +1363,9 @@ static int logit_probit_vcv (MODEL *dmod, gretlopt opt, const double **Z)
 	free(dmod->xpx);
     }
 
-    dmod->xpx = hessian(dmod, Z, dmod->ci);
+    dmod->xpx = lp_hessian(dmod, Z, dmod->ci);
     if (dmod->xpx == NULL) {
-	strcpy(gretl_errmsg, _("Failed to construct Hessian matrix"));
+	gretl_errmsg_set(_("Failed to construct Hessian matrix"));
 	return E_ALLOC;
     } 
 
@@ -464,7 +1504,7 @@ static int do_BRMR (const int *list, MODEL *dmod, int ci,
 	    break;
 	}
 
-	dmod->lnL = logit_probit_llhood(yvar, dmod, ci);
+	dmod->lnL = logit_probit_loglik(yvar, dmod, ci);
 
 	lldiff = fabs(dmod->lnL - llbak);
 	if (lldiff < tol) {
@@ -767,7 +1807,8 @@ static void binary_model_add_stats (MODEL *pmod, const double *y)
 	}
 
 	if (pmod->ci == LOGIT) {
-	    pmod->yhat[t] = exp(xb) / (1.0 + exp(xb)); 
+	    F = exp(xb) / (1.0 + exp(xb));
+	    pmod->yhat[t] = F; 
 	    pmod->uhat[t] = yt - pmod->yhat[t];
 	} else {
 	    f = normal_pdf(xb);
@@ -775,6 +1816,8 @@ static void binary_model_add_stats (MODEL *pmod, const double *y)
 	    pmod->yhat[t] = F; 
 	    pmod->uhat[t] = (yt != 0)? f/F : -f/(1-F);
 	}
+
+	pmod->llt[t] = (yt != 0)? log(F) : log(1-F);
     }
 
     pmod->ybar = xx / pmod->nobs;
@@ -865,7 +1908,7 @@ binary_logit_probit (const int *inlist, double ***pZ, DATAINFO *pdinfo,
 	dmod.coeff[i] = beta[i];
     }
 
-    dmod.lnL = logit_probit_llhood(Z[depvar], &dmod, ci);
+    dmod.lnL = logit_probit_loglik(Z[depvar], &dmod, ci);
     Lr_chisq(&dmod, Z);
     dmod.ci = ci;
 
@@ -1248,11 +2291,6 @@ static void mn_logit_yhat (MODEL *pmod, mnl_info *mnl,
     gretl_model_set_int(pmod, "correct", ncorrect);
 }
 
-struct sorter {
-    double x;
-    int t;
-};
-
 /* In case the dependent variable is not in canonical form for
    multinomial logit, construct a transformed version */
 
@@ -1491,10 +2529,7 @@ static void mnl_finish (mnl_info *mnl, MODEL *pmod,
 	pmod->dfn = df;
     }
 
-    if (pmod->xpx != NULL) {
-	free(pmod->xpx);
-	pmod->xpx = NULL;
-    }
+    clear_model_xpx(pmod);
 
     pmod->opt |= OPT_M;
 }
@@ -1596,8 +2631,8 @@ discrete_depvar_ok (double **Z, const DATAINFO *pdinfo, int v)
 {
     if (!var_is_discrete(pdinfo, v) && 
 	!gretl_is_oprobit_ok(pdinfo->t1, pdinfo->t2, Z[v])) {
-	sprintf(gretl_errmsg, _("The variable '%s' is not discrete"),
-		pdinfo->varname[v]);
+	gretl_errmsg_sprintf(_("The variable '%s' is not discrete"),
+			     pdinfo->varname[v]);
 	return 0;
     } 
 
@@ -1641,7 +2676,7 @@ MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo,
 	return binary_logit_probit(list, pZ, pdinfo, ci, opt, prn);
     } else if (discrete_depvar_ok(*pZ, pdinfo, yv)) {
 	vprn = (opt & OPT_V)? prn : NULL;
-	return ordered_model(list, ci, pZ, pdinfo, opt, vprn);
+	return ordered_estimate(list, ci, pZ, pdinfo, opt, vprn);
     } else {
 	MODEL dmod;
 
@@ -1731,7 +2766,7 @@ static int cholesky_decomp (double *xpx, int nv)
             k += nv-l;
         }
         test = xpx[kk] - d;
-        if (test / xpx[kk] < TINY) {
+        if (test / xpx[kk] < CHOL_TINY) {
 	    return 1;
 	}
         e = 1 / sqrt(test);
@@ -1765,8 +2800,8 @@ int logistic_ymax_lmax (const double *y, const DATAINFO *pdinfo,
 	    continue;
 	}
 	if (y[t] <= 0.0) {
-	    strcpy(gretl_errmsg, _("Illegal non-positive value of the "
-				   "dependent variable"));
+	    gretl_errmsg_set(_("Illegal non-positive value of the "
+			       "dependent variable"));
 	    return 1;
 	}
 	if (y[t] > *ymax) {
@@ -2046,6 +3081,7 @@ MODEL intreg (int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
     gretl_error_clear();
 
     interval_estimate = get_plugin_function("interval_estimate", &handle);
+
     if (interval_estimate == NULL) {
 	gretl_model_init(&intmod);
 	intmod.errcode = E_FOPEN;
