@@ -25,7 +25,7 @@
 #include "system.h"
 #include "libset.h"
 
-#define ARF_DEBUG 0
+#define AR_DEBUG 0
 
 #ifdef max
 # undef max
@@ -475,8 +475,6 @@ has_real_exog_regressors (MODEL *pmod, const int *dvlags,
     return ret;
 }
 
-#define AR_DEBUG 0
-
 /* Get a value for a lag of the dependent variable.  If method is
    dynamic we prefer lagged prediction to lagged actual.  If method is
    static, we never want the lagged prediction, only the actual.  If
@@ -537,7 +535,7 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 			const double **Z, const DATAINFO *pdinfo) 
 {
     gretl_matrix *V = NULL;
-    gretl_vector *Xs = NULL;
+    gretl_vector *Xt = NULL;
     gretl_vector *b = NULL;
     double s2 = pmod->sigma * pmod->sigma;
     double vyh, xval;
@@ -555,8 +553,8 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	goto bailout;
     }
 
-    Xs = gretl_vector_alloc(k);
-    if (Xs == NULL) {
+    Xt = gretl_vector_alloc(k);
+    if (Xt == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
@@ -569,7 +567,7 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	    missing = na(pmod->yhat[t]);
 	}   
 
-	/* populate Xs vector for observation */
+	/* populate Xt vector for observation */
 	for (i=0; i<k && !missing; i++) {
 	    vi = pmod->list[i + 2];
 	    xval = Z[vi][t];
@@ -577,20 +575,20 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 		fc->sderr[t] = fc->yhat[t] = NADBL;
 		missing = 1;
 	    } else {
-		gretl_vector_set(Xs, i, xval);
+		gretl_vector_set(Xt, i, xval);
 	    }
 	}
 
 	if (missing) {
 	    fc->sderr[t] = fc->yhat[t] = NADBL;
 	    continue;
-	}
+	} 
 
 	/* forecast value */
-	fc->yhat[t] = gretl_vector_dot_product(Xs, b, NULL);
+	fc->yhat[t] = gretl_vector_dot_product(Xt, b, NULL);
 
 	/* forecast variance */
-	vyh = gretl_scalar_qform(Xs, V, &err);
+	vyh = gretl_scalar_qform(Xt, V, &err);
 	if (na(vyh)) {
 	    err = 1;
 	} else {
@@ -607,7 +605,67 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
  bailout:
 
     gretl_matrix_free(V);
-    gretl_vector_free(Xs);
+    gretl_vector_free(Xt);
+    gretl_vector_free(b);
+
+    return err;
+}
+
+static int fixed_effects_fcast (Forecast *fc, MODEL *pmod, 
+				const double **Z, const DATAINFO *pdinfo) 
+{
+    gretl_vector *b = NULL;
+    double xval;
+    const double *ahat;
+    int k = pmod->ncoeff;
+    int i, vi, t;
+    int err = 0;
+
+    b = gretl_coeff_vector_from_model(pmod, NULL, &err);
+    if (err) {
+	return err;
+    }
+
+    ahat = gretl_model_get_data(pmod, "ahat");
+    if (ahat == NULL) {
+	fprintf(stderr, "fixed_effects_fcast: ahat is missing\n");
+	gretl_matrix_free(b);
+	return E_DATA;
+    }
+
+    for (t=fc->t1; t<=fc->t2 && !err; t++) {
+	int missing = 0;
+
+	/* skip if we can't compute forecast */
+	if (t >= pmod->t1 && t <= pmod->t2) {
+	    missing = na(pmod->yhat[t]);
+	} 
+
+	if (!missing) {
+	    /* per-unit intercept */
+	    if (na(ahat[t])) {
+		missing = 1;
+	    } else {
+		fc->yhat[t] = ahat[t];
+	    }
+	}
+
+	for (i=1; i<k && !missing; i++) {
+	    vi = pmod->list[i + 2];
+	    xval = Z[vi][t];
+	    if (na(xval)) {
+		missing = 1;
+	    } else {
+		fc->yhat[t] += b->val[i] * xval;
+	    }
+	}	
+
+	if (missing) {
+	    fc->sderr[t] = fc->yhat[t] = NADBL;
+	    continue;
+	} 
+    }
+
     gretl_vector_free(b);
 
     return err;
@@ -717,9 +775,9 @@ static int nls_fcast (Forecast *fc, const MODEL *pmod,
     return err;
 }
 
-#if ARF_DEBUG
+#if AR_DEBUG
 # include <stdarg.h>
-static void dprintf (const char *format, ...)
+static void my_dprintf (const char *format, ...)
 {
    va_list args;
 
@@ -729,7 +787,7 @@ static void dprintf (const char *format, ...)
 
    return;
 }
-# define DPRINTF(x) dprintf x
+# define DPRINTF(x) my_dprintf x
 #else 
 # define DPRINTF(x)
 #endif 
@@ -1066,6 +1124,8 @@ static double arma_variance (const double *phi, int p,
     static double sspsi;
     int i, j;
 
+    DPRINTF(("arma_variance: p=%d, q=%d, npsi=%d, l=%d\n", p, q, npsi, l));
+
     if (l == 1) {
 	sspsi = 0.0;
 	psi[0] = 1.0;
@@ -1079,20 +1139,92 @@ static double arma_variance (const double *phi, int p,
 		    psi[j] += theta[j];
 		}
 	    }
+	    DPRINTF(("psi[%d] = %g\n", j, psi[j]));
 	}
     } 
 
     sspsi += psi[l-1] * psi[l-1];
 
+    DPRINTF(("augmented 'sspsi' using psi(%d) = %g (sspsi = %g)\n", 
+	     l-1, psi[l-1], sspsi));
+
     return sspsi;
 }
 
+#define invalid_obs(x,t) ((t) < 0 || na(x[t]))
+
+static double arima_difference_obs (const double *x, int t, 
+				    int d, int D, int s)
+{
+    double dx = x[t];
+
+    if (na(dx)) {
+	return NADBL;
+    }
+
+    if (d > 0) {
+	if (invalid_obs(x, t-1)) {
+	    return NADBL;
+	}
+	dx -= x[t-1];
+    } 
+    if (d == 2) {
+	if (invalid_obs(x, t-2)) {
+	    return NADBL;
+	}
+	dx -= x[t-1] - x[t-2];
+    }
+    if (D > 0) {
+	if (invalid_obs(x, t-s)) {
+	    return NADBL;
+	}
+	dx -= x[t-s];
+	if (d > 0) {
+	    if (invalid_obs(x, t-s-1)) {
+		return NADBL;
+	    }
+	    dx += x[t-s-1];
+	}
+	if (d == 2) {
+	    if (invalid_obs(x, t-s-2)) {
+		return NADBL;
+	    }
+	    dx += x[t-s-1] - x[t-s-2];
+	}	    
+    } 
+    if (D == 2) {
+	if (invalid_obs(x, t-2*s)) {
+	    return NADBL;
+	}
+	dx -= x[t-s] - x[t-2*s];
+	if (d > 0) {
+	    if (invalid_obs(x, t-s-1) || invalid_obs(x, t-2*s-1)) {
+		return NADBL;
+	    }
+	    dx += x[t-s-1] - x[t-2*s-1];
+	}
+	if (d == 2) {
+	    if (invalid_obs(x, t-s-2) || invalid_obs(x, t-2*s-2)) {
+		return NADBL;
+	    }
+	    dx += x[t-s-1] - x[t-2*s-1];
+	    dx -= x[t-s-2] - x[t-2*s-2];
+	}
+    }
+
+    return dx;
+}
+
 /* When forecasting based on an armax model estimated using X12A,
    or via the Kalman filter, we need to form the series X\beta
    so that we can subtract X\beta_{t-i} from y_{t-i} in
    computing the AR portion of the forecast. "beta" below
    is the array of ARMAX coefficients, not including the
    constant.
+
+   If the model is ARIMAX and was NOT estimated with the
+   --y-diff-only option (OPT_Y) then the regressors X have
+   to be differenced in this context.
 */
 
 static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
@@ -1101,23 +1233,37 @@ static double *create_Xb_series (Forecast *fc, const MODEL *pmod,
 {
     double *Xb;
     double x;
-    int miss;
-    int i, j, t;
+    int miss, diff = 0;
+    int d, D, s = 0;
+    int i, j, t, vi;
 
     Xb = malloc((fc->t2 + 1) * sizeof *Xb);
     if (Xb == NULL) {
 	return NULL;
     }
 
+    d = gretl_model_get_int(pmod, "arima_d");
+    D = gretl_model_get_int(pmod, "arima_D");
+
+    if ((d > 0 || D > 0) && !(pmod->opt & OPT_Y)) {
+	s = gretl_model_get_int(pmod, "arma_pd");
+	diff = 1;
+    }
+
     for (t=0; t<=fc->t2; t++) {
 	Xb[t] = 0.0;
 	miss = 0;
 	j = 0;
 	for (i=1; i<=xlist[0] && !miss; i++) {
-	    if (xlist[i] == 0) {
+	    vi = xlist[i];
+	    if (vi == 0) {
 		Xb[t] += pmod->coeff[0];
 	    } else {
-		x = Z[xlist[i]][t];
+		if (diff) {
+		    x = arima_difference_obs(Z[vi], t, d, D, s);
+		} else {
+		    x = Z[vi][t];
+		}
 		if (na(x)) {
 		    Xb[t] = NADBL;
 		    miss = 1;
@@ -1161,7 +1307,6 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
     double *Xb = NULL;
     const double *beta;
     const double *y;
-
     double xval, yval, vl;
     double mu = NADBL;
     int xvars, yno;
@@ -1538,6 +1683,8 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
 	   sample */
 	pmax = max_ar_lag(fc, pmod, p);
 	npsi = fc->t2 - fc->t1 + 1;
+	/* npsi = fc->t2 - pmod->t2; */
+	DPRINTF(("pmax = %d, npsi = %d\n", pmax, npsi));
 	set_up_ar_fcast_variance(fc, pmod, pmax, npsi, &phi, &psi, &errphi);
     }
 
@@ -1623,6 +1770,8 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
 	    if (t > pmod->t2) {
 		vl = arma_variance(phi, pmax, NULL, 0, psi, npsi, t - pmod->t2); 
 		fc->sderr[t] = pmod->sigma * sqrt(vl);
+		DPRINTF(("sderr[%d] = %g * sqrt(%g) = %g\n", t, pmod->sigma, 
+			 vl, fc->sderr[t]));
 	    } else {
 		fc->sderr[t] = NADBL;
 	    }
@@ -1643,10 +1792,11 @@ static int ar_fcast (Forecast *fc, MODEL *pmod,
    type LOGISTIC, LOGIT, PROBIT, TOBIT and POISSON.
  */
 
-static double fcast_transform (double xb, int ci, int t, 
-			       const double *offset,
+static double fcast_transform (double xb, const MODEL *pmod,  
+			       int t, const double *offset,
 			       double lmax)
 {
+    int ci = pmod->ci;
     double yf = xb;
 
     if (ci == TOBIT) {
@@ -1654,9 +1804,17 @@ static double fcast_transform (double xb, int ci, int t,
 	    yf = 0.0;
 	}
     } else if (ci == LOGIT) {
-	yf = exp(xb) / (1.0 + exp(xb));
+	if (gretl_model_get_int(pmod, "ordered")) {
+	    yf = ordered_model_prediction(pmod, xb);
+	} else {
+	    yf = exp(xb) / (1.0 + exp(xb));
+	}
     } else if (ci == PROBIT) {
-	yf = normal_cdf(xb);
+	if (gretl_model_get_int(pmod, "ordered")) {
+	    yf = ordered_model_prediction(pmod, xb);
+	} else {	
+	    yf = normal_cdf(xb);
+	}
     } else if (ci == LOGISTIC) {
 	if (na(lmax)) {
 	    yf = 1.0 / (1.0 + exp(-xb));
@@ -1763,6 +1921,7 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod, int yno,
 {
     const double *offvar = NULL;
     double xval, lmax = NADBL;
+    int k = pmod->ncoeff;
     int i, vi, t;
 
     if (pmod->ci == POISSON) {
@@ -1774,13 +1933,22 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod, int yno,
 	}
     } else if (pmod->ci == LOGISTIC) {
 	lmax = gretl_model_get_double(pmod, "lmax");
+    } else if (pmod->ci == LOGIT || pmod->ci == PROBIT) {
+	if (gretl_model_get_int(pmod, "ordered")) {
+	    /* we need to know how many coeffs are not
+	       just estimated cut-points */
+	    k = gretl_model_get_int(pmod, "nx");
+	    if (k <= 0) {
+		return E_MISSDATA;
+	    }
+	}
     }
 
     for (t=fc->t1; t<=fc->t2; t++) {
 	double yht = 0.0;
 	int miss = 0;
 
-	for (i=0; i<pmod->ncoeff && !miss; i++) {
+	for (i=0; i<k && !miss; i++) {
 	    int lag;
 
 	    vi = pmod->list[i+2];
@@ -1800,7 +1968,7 @@ static int linear_fcast (Forecast *fc, const MODEL *pmod, int yno,
 	    fc->yhat[t] = NADBL;
 	} else if (FCAST_SPECIAL(pmod->ci)) {
 	    /* special handling for LOGIT and others */
-	    fc->yhat[t] = fcast_transform(yht, pmod->ci, t, offvar, lmax);
+	    fc->yhat[t] = fcast_transform(yht, pmod, t, offvar, lmax);
 	} else {
 	    fc->yhat[t] = yht;
 	}
@@ -2010,7 +2178,9 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
 	asy_errs = 1;
     }
 
-    if (!FCAST_SPECIAL(pmod->ci) && !integrate) {
+    if (pmod->ci == PANEL) {
+	; /* don't do the things below */
+    } else if (!FCAST_SPECIAL(pmod->ci) && !integrate) {
 	if (!AR_MODEL(pmod->ci) && fc.dvlags == NULL) {
 	    /* we'll do Davidson-MacKinnon error variance */
 	    DM_errs = 1;
@@ -2056,7 +2226,9 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
     }
 
     /* compute the actual forecast */
-    if (DM_errs) {
+    if (pmod->ci == PANEL && (pmod->opt & OPT_F)) {
+	err = fixed_effects_fcast(&fc, pmod, Z, pdinfo);
+    } else if (DM_errs) {
 	err = static_fcast_with_errs(&fc, pmod, Z, pdinfo);
     } else if (pmod->ci == NLS) {
 	err = nls_fcast(&fc, pmod, pZ, pdinfo);
@@ -2705,7 +2877,13 @@ static int model_do_forecast (const char *str, MODEL *pmod,
     }
 
     if (!err && !(opt & OPT_Q)) {
-	err = text_print_forecast(fr, pdinfo, opt, prn);
+	gretlopt printopt = opt;
+
+	if (opt & OPT_G) {
+	    /* do graph (from command line) */
+	    printopt |= OPT_P;
+	}
+	err = text_print_forecast(fr, pdinfo, printopt, prn);
     }
 
     if (!err && (opt & OPT_A)) {
@@ -2864,7 +3042,7 @@ static int system_do_forecast (const char *str, void *ptr, int type,
 
     if (!(opt & OPT_Q)) {
 	/* assemble and print per-equation forecasts */
-	gretlopt printopt = OPT_NONE;
+	gretlopt printopt = (opt & OPT_N)? OPT_N : OPT_NONE;
 	FITRESID *fr = NULL;
 	int i, asy;
 
@@ -2902,7 +3080,7 @@ static int system_do_forecast (const char *str, void *ptr, int type,
  * @str: command string, which may include a starting 
  * observation and ending observation, and/or the name of a 
  * variable for saving the forecast values.
- * @pZ: pointer to data matrix.
+ * @pZ: pointer to data array.
  * @pdinfo: pointer to data information struct.
  * @opt: if %OPT_D, force a dynamic forecast; if %OPT_S, force
  * a static forecast.  By default, the forecast is static within
@@ -2911,6 +3089,8 @@ static int system_do_forecast (const char *str, void *ptr, int type,
  * %OPT_R: do rolling/recursive forecast.
  * %OPT_Q: suppress printing of the forecast;
  * %OPT_P: ensure that the values are printed.
+ * %OPT_G: produce gnuplot plot.
+ * @prn: gretl printing struct.
  *
  * In the case of "simple" models with an autoregressive error term 
  * (%AR, %AR1) the predicted values incorporate the forecastable portion 
@@ -3361,3 +3541,27 @@ rolling_OLS_k_step_fcast (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	
     return fr;
 }
+
+void fcast_get_continuous_range (const FITRESID *fr, int *pt1, int *pt2)
+{
+    int t, t1 = fr->t1, t2 = fr->t2;
+
+    for (t=t1; t<=t2; t++) {
+	if (na(fr->actual[t]) || na(fr->fitted[t])) {
+	    t1++;
+	} else {
+	    break;
+	}
+    }
+
+    for (t=t2; t>=t1; t--) {
+	if (na(fr->actual[t]) || na(fr->fitted[t])) {
+	    t2--;
+	} else {
+	    break;
+	}
+    }
+
+    *pt1 = t1;
+    *pt2 = t2;
+}  
