@@ -23,6 +23,8 @@
 #include "libset.h"
 #include "monte_carlo.h"
 #include "gretl_string_table.h"
+#include "gretl_fft.h"
+#include "../../cephes/cephes.h"
 
 #include <errno.h>
 
@@ -373,48 +375,70 @@ int orthdev_series (const double *x, double *y, const DATAINFO *pdinfo)
  * @x: array of original data.
  * @y: array into which to write the result.
  * @d: fraction by which to difference.
+ * @diff: boolean variable 1 for fracdiff, 0 for fraclag
+ * @obs: used for autoreg calculation, -1 if whole series 
+ *	should be calculated otherwise just the observation for
+ *	obs is calculated 
  * @pdinfo: data set information.
  *
- * Calculates the fractionally differenced counterpart
- * to the input series @x.
+ * Calculates the fractionally differenced or lagged 
+ * counterpart to the input series @x. The fractional 
+ * difference operator is defined as (1-L)^d, while the
+ * fractional lag operator 1-(1-L)^d.
  *
  * Returns: 0 on success, non-zero error code on failure.
  */
 
 int fracdiff_series (const double *x, double *y, double d,
-		     const DATAINFO *pdinfo)
+		     int diff, int obs, const DATAINFO *pdinfo)
 {
-    int dd, t, T;
-    const double TOL = 1.0E-07;
+    int dd, t, tmiss, T;
+    const double TOL = 1.0E-12;
     int t1 = pdinfo->t1;
-    int t2 = pdinfo->t2;
-    double phi = -d;
-    int err;
+    int t2 = (obs >= 0)? obs : pdinfo->t2;
+    double phi = (diff)? -d : d;
 
 #if 0
     fprintf(stderr, "Doing fracdiff_series, with d = %g\n", d);
 #endif
 
-    err = array_adjust_t1t2(x, &t1, &t2);
-    if (err) {
-	return E_DATA;
+    tmiss = array_adjust_t1t2(x, &t1, &t2);
+
+    if (tmiss > 0 && tmiss < t2) {
+	t2 = tmiss;
     } 
-
-    T = t2 - t1 + 1;
-
-    for (t=0; t<pdinfo->n; t++) {
-	if (t >= t1 && t <= t2) {
-	    y[t] = x[t];
-	} else {
+ 
+    if (obs >= 0) {
+	/* doing a specific observation */
+	T = obs - t1 + 1;
+	for (t=0; t<pdinfo->n; t++) {
 	    y[t] = NADBL;
 	}
-    }   
-
-    for (dd=1; dd<=T && fabs(phi)>TOL; dd++) {
-	for (t=t1+dd; t<=t2; t++) {
-	    y[t] += phi * x[t - dd];
+	if (obs != t1) {
+	    y[obs] = (diff)? x[obs] : 0;
+	    for (dd=1; dd<T && fabs(phi)>TOL; dd++) {
+		y[obs] += phi * x[obs - dd];
+		phi *= (dd - d)/(dd + 1);
+	    }
+	} else if (diff) {
+	    y[obs] = x[obs];
+	}
+    } else {
+	/* doing the whole series */
+	T = t2 - t1 + 1;
+	for (t=0; t<=t2; t++) {
+	    if (t >= t1 && t <= t2) {
+		y[t] = (diff)? x[t] : 0;
+	    } else {
+		y[t] = NADBL;
+	    } 
+	}  
+	for (dd=1; dd<=T && fabs(phi)>TOL; dd++) {
+	    for (t=t1+dd; t<=t2; t++) {
+		y[t] += phi * x[t - dd];
+	    }
+	    phi *= (dd - d)/(dd + 1);
 	}
-	phi *= (dd - d)/(dd + 1);
     }
 
     return 0;
@@ -684,6 +708,62 @@ int filter_series (const double *x, double *y, const DATAINFO *pdinfo,
     return err;
 }
 
+int seasonally_adjust_series (const double *x, double *y, 
+			      DATAINFO *pdinfo, int tramo)
+{
+    void *handle;
+    int (*adjust_series) (const double *, double *, 
+			  const DATAINFO *, int);
+    int t1 = pdinfo->t1;
+    int t2 = pdinfo->t2;
+    int T, err = 0;
+
+    if (!quarterly_or_monthly(pdinfo)) {
+	gretl_errmsg_set(_("Input must be a monthly or quarterly time series"));	
+	return E_PDWRONG;
+    }
+
+    array_adjust_t1t2(x, &t1, &t2);
+    T = t2 - t1 + 1;
+
+    if (T < pdinfo->pd * 3) {
+	return E_TOOFEW;
+    } else if (tramo && T > 600) {
+	gretl_errmsg_set(_("TRAMO can't handle more than 600 observations.\n"
+			   "Please select a smaller sample."));
+	return E_EXTERNAL;
+    } else if (!tramo) {
+	int pdmax = get_x12a_maxpd();
+
+	if (T > 50 * pdmax) {
+	    gretl_errmsg_sprintf(_("X-12-ARIMA can't handle more than %d observations.\n"
+				   "Please select a smaller sample."), 50 * pdmax);
+	    return E_EXTERNAL;
+	}
+    }
+
+    adjust_series = get_plugin_function("adjust_series", &handle);
+    
+    if (adjust_series == NULL) {
+	err = E_FOPEN;
+    } else {
+	int save_t1 = pdinfo->t1;
+	int save_t2 = pdinfo->t2;
+
+	pdinfo->t1 = t1;
+	pdinfo->t2 = t2;
+
+	err = (*adjust_series) (x, y, pdinfo, tramo);
+
+	pdinfo->t1 = save_t1;
+	pdinfo->t2 = save_t2;
+
+	close_plugin(handle);
+    }
+
+    return err;
+}
+
 int panel_statistic (const double *x, double *y, const DATAINFO *pdinfo, 
 		     int k)
 {
@@ -870,17 +950,15 @@ static double hp_lambda (const DATAINFO *pdinfo)
 int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
 	       gretlopt opt)
 {
-    int i, t, T, t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int err = 0;
+    int t1 = pdinfo->t1, t2 = pdinfo->t2;
     double v00 = 1.0, v11 = 1.0, v01 = 0.0;
     double det, tmp0, tmp1;
+    double e0, e1, b00, b01, b11;
     double lambda;
-
     double **V = NULL;
     double m[2], tmp[2];
-
-    int tb;
-    double e0, e1, b00, b01, b11;
+    int i, t, T, tb;
+    int err = 0;
 
     for (t=t1; t<=t2; t++) {
 	hp[t] = NADBL;
@@ -888,7 +966,7 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
 
     err = array_adjust_t1t2(x, &t1, &t2);
     if (err) {
-	err = E_DATA;
+	err = E_MISSDATA;
 	goto bailout;
     }
 
@@ -1006,10 +1084,12 @@ int hp_filter (const double *x, double *hp, const DATAINFO *pdinfo,
 
  bailout:
 
-    for (i=0; i<4; i++) {
-	free(V[i]);
+    if (V != NULL) {
+	for (i=0; i<4; i++) {
+	    free(V[i]);
+	}
+	free(V);
     }
-    free(V);
 
     return err;
 }
@@ -1052,7 +1132,7 @@ int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
 #endif
 
     if (bkl >= bku) {
-	strcpy(gretl_errmsg, "Error in Baxter-King frequencies");
+	gretl_errmsg_set("Error in Baxter-King frequencies");
 	return 1;
     }
 
@@ -1062,7 +1142,7 @@ int bkbp_filter (const double *y, double *bk, const DATAINFO *pdinfo)
     } 
 
     if (2 * k >= t2 - t1 + 1) {
-	strcpy(gretl_errmsg, "Insufficient observations");
+	gretl_errmsg_set("Insufficient observations");
 	return E_DATA;
     }
 
@@ -1190,7 +1270,7 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
     double xx, dx;
 
     if (ndums == 1 || ndums > 99999) {
-	strcpy(gretl_errmsg, _("This command won't work with the current periodicity"));
+	gretl_errmsg_set(_("This command won't work with the current periodicity"));
 	return 0;
     }
 
@@ -1217,7 +1297,7 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
     }
 
     if (dataset_add_series(ndums, pZ, pdinfo)) {
-	strcpy(gretl_errmsg, _("Out of memory!"));
+	gretl_errmsg_set(_("Out of memory!"));
 	return 0;
     }
 
@@ -1391,8 +1471,8 @@ int gen_unit (double ***pZ, DATAINFO *pdinfo)
     int i, t;
 
     if (pdinfo->structure != STACKED_TIME_SERIES) {
-	strcpy(gretl_errmsg, "'genr unit' can be used only with "
-	       "panel data");
+	gretl_errmsg_set("'genr unit' can be used only with "
+			 "panel data");
 	return 1;
     }
 
@@ -1534,7 +1614,7 @@ int gen_wkday (double ***pZ, DATAINFO *pdinfo)
     strcpy(VARLABEL(pdinfo, i), _("day of week (1 = Monday)"));
     
     for (t=0; t<pdinfo->n; t++) {
-	ntodate_full(datestr, t, pdinfo);
+	ntodate(datestr, t, pdinfo);
 	(*pZ)[i][t] = get_day_of_week(datestr);
     }
 
@@ -1909,7 +1989,7 @@ int get_t_from_obs_string (const char *s, const double **Z,
     }
 
     if (t < 0) {
-	sprintf(gretl_errmsg, _("Observation number out of bounds"));
+	gretl_errmsg_set(_("Observation number out of bounds"));
     }
 
 #if OBS_DEBUG
@@ -2700,7 +2780,9 @@ gretl_matrix *multi_xcf (const void *px, int xtype,
 	T = Xmat->rows;
 	xvec = Xmat->val;
     } else {
+	/* VEC: note: px is of type *void */
 	xvec = px;
+	xvec += pdinfo->t1;
     }
 
     if (ytype == MAT) {
@@ -2762,3 +2844,366 @@ gretl_matrix *multi_xcf (const void *px, int xtype,
     return XCF;
 }
 
+static int theil_decomp (double *m, double MSE,
+			 const double *y, const double *f,
+			 int t1, int t2)
+{
+    double da, dp;
+    double Abar, Pbar;
+    double sa, sp, r;
+    int t, T = t2 - t1 + 1;
+    int err = 0;
+
+    if (MSE <= 0.0) {
+	m[0] = m[1] = m[2] = M_NA;
+	return E_DATA;
+    }
+
+    Abar = Pbar = 0.0;
+
+    for (t=t1; t<=t2; t++) {
+	Abar += y[t];
+	Pbar += f[t];
+    }
+
+    Abar /= T;
+    Pbar /= T;
+
+    sa = sp = r = 0.0;
+
+    for (t=t1; t<=t2; t++) {
+	da = y[t] - Abar;
+	sa += da * da;
+	dp = f[t] - Pbar;
+	sp += dp * dp;
+	r += da * dp;
+    }    
+
+    sa = sqrt(sa / T);
+    sp = sqrt(sp / T);
+    r /= T * sa * sp;
+
+    if (sa == 0.0 || sp == 0.0) {
+	err = E_DATA;
+	m[0] = m[1] = m[2] = M_NA;
+    } else {
+	m[0] = (Abar - Pbar) * (Abar - Pbar) / MSE; /* U^M */
+	m[1] = (sp - r * sa) * (sp - r * sa) / MSE; /* U^R */
+	m[2] = (1.0 - r * r) * sa * sa / MSE;       /* U^D */
+
+	if (m[2] > 0.99999999999999) {
+	    /* U^M and U^R are just machine noise? */
+	    m[2] = 1.0;
+	    m[0] = m[1] = 0.0;
+	} 
+    }
+
+    return err;
+}
+
+/* cf. http://www.economicsnetwork.ac.uk/showcase/cook_forecast
+   by Steven Cook of Swansea University <s.cook@Swansea.ac.uk>
+
+   OPT_D indicates that we should include the Theil decomposition.
+*/
+
+gretl_matrix *forecast_stats (const double *y, const double *f,
+			      int t1, int t2, gretlopt opt,
+			      int *err)
+{
+    gretl_matrix *m = NULL;
+    double ME, MSE, MAE, MPE, MAPE, U;
+    double x, u[2];
+    int t, T = t2 - t1 + 1;
+
+    for (t=t1; t<=t2; t++) {
+	if (na(y[t]) || na(f[t])) {
+	    *err = E_MISSDATA;
+	    return NULL;
+	}
+    }
+
+    ME = MSE = MAE = MPE = MAPE = U = 0.0;
+    u[0] = u[1] = 0.0;
+
+    for (t=t1; t<=t2; t++) {
+	x = y[t] - f[t];
+	ME += x;
+	MSE += x * x;
+	MAE += fabs(x);
+	if (y[t] == 0.0) {
+	    MPE = MAPE = U = M_NA;
+	} else {
+	    MPE += 100 * x / y[t];
+	    MAPE += 100 * fabs(x) / y[t];
+	    if (t < t2) {
+		x = (f[t+1] - y[t+1]) / y[t];
+		u[0] += x * x;
+		x = (y[t+1] - y[t]) / y[t];
+		u[1] += x * x;
+	    }
+	}
+    }
+
+    ME /= T;
+    MSE /= T;
+    MAE /= T;
+
+    if (!isnan(MPE)) {
+	MPE /= T;
+    }
+
+    if (!isnan(MAPE)) {
+	MAPE /= T;
+    }
+
+    if (!isnan(U) && u[1] > 0.0) {
+	U = sqrt(u[0] / T) / sqrt(u[1] / T);
+    }
+
+    if (opt & OPT_D) {
+	m = gretl_column_vector_alloc(9);
+    } else {
+	m = gretl_column_vector_alloc(6);
+    }
+    
+    if (m == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    gretl_vector_set(m, 0, ME);
+    gretl_vector_set(m, 1, MSE);
+    gretl_vector_set(m, 2, MAE);
+    gretl_vector_set(m, 3, MPE);
+    gretl_vector_set(m, 4, MAPE);
+    gretl_vector_set(m, 5, U);
+
+    if (opt & OPT_D) {
+	theil_decomp(m->val + 6, MSE, y, f, t1, t2);
+    } 
+
+    return m;
+}
+
+double gretl_round (double x)
+{
+    double fx = floor(x);
+
+    if (x < 0) {
+	return (x - fx <= 0.5)? fx : ceil(x);
+    } else {
+	return (x - fx < 0.5)? fx : ceil(x);
+    }
+}
+
+#define neg_x_real_v_err(t) (t == 'J' || t == 'I')
+
+/* evaluates bessel function for scalar nu and x */
+
+double gretl_bessel (char type, double v, double x, int *err)
+{
+    if (na(x) || na(v)) {
+	return NADBL;
+    }
+
+    if (x < 0) {
+	/* catch invalid cases for x < 0 */
+	if (type == 'K') {
+	    *err = E_INVARG;
+	    return NADBL;
+	} else if (v != floor(v) && (type == 'J' || type == 'I')) {
+	    *err = E_INVARG;
+	    return NADBL;
+	}
+    }
+
+    switch (type) {
+    case 'J':
+	return cephes_bessel_Jv(v, x);
+    case 'Y':
+	return cephes_bessel_Yv(v, x);
+    case 'I':
+	if (v == 0) {
+	    return cephes_bessel_I0(x);
+	} else if (v == 1) {
+	    return cephes_bessel_I1(x);
+	} else if (v > 0) {
+	    return cephes_bessel_Iv(v, x);
+	} else {
+	    /* cephes_bessel_Iv is not right for v < 0 */
+	    double b1 = netlib_bessel_K(-v, x, 1);
+	    double b2 = cephes_bessel_Iv(-v, x);
+
+	    return (2*b1*sin(-v*M_PI)) / M_PI + b2;
+	}
+	break;
+    case 'K':
+	/* bessel K is symmetric around v = 0 */
+	v = fabs(v);
+	if (v == 0) {
+	    return cephes_bessel_K0(x);
+	} else if (v == 1) {
+	    return cephes_bessel_K1(x);
+	} else if (v == floor(v) && v <= 30.0) {
+	    /* cephes doesn't do non-integer v, and also loses
+	       accuracy beyond |v| = 30
+	    */
+	    return cephes_bessel_Kn(v, x);
+	} else {
+	    /* accurate but expensive */
+	    return netlib_bessel_K(v, x, 1);
+	}
+	break;
+    default:
+	/* unknown function type */
+	return NADBL;
+    }
+}
+
+/* Net Present Value: note that the first value is taken as
+   occurring "now" and is not discounted */
+
+double gretl_npv (int t1, int t2, const double *x, double r, 
+		  int pd, int *err)
+{
+    double d, PV = 0.0;
+    int i, n = 0;
+
+    if (pd != 1 && pd != 4 && pd != 12) {
+	*err = E_PDWRONG;
+	return NADBL;
+    }
+
+    if (pd == 1) {
+	d = 1 + r;
+    } else if (r < -1.0) {
+	*err = E_NAN;
+	return 0.0 / 0.0;
+    } else {
+	d = pow(1 + r, 1.0 / pd);
+    }
+
+    for (i=t1; i<=t2; i++) {
+	if (!na(x[i])) {
+	    PV += x[i] / (pow(d, i-t1));
+	    n++;
+	}
+    }
+
+    if (n == 0) {
+	PV = NADBL;
+    }
+
+    return PV;
+}
+
+/* Internal Rate of Return */
+
+double gretl_irr (const double *x, int n, int pd, int *err)
+{
+    double PV, r, r1 = 0.02, r0 = -0.02;
+    int gotplus = 0, gotminus = 0;
+    int i, m = n;
+
+    for (i=0; i<n; i++) {
+	if (na(x[i])) {
+	    m--;
+	} else if (x[i] > 0) {
+	    gotplus = 1;
+	} else if (x[i] < 0) {
+	    gotminus = 1;
+	}
+    }
+
+    if (!gotplus && !gotminus) {
+	/* null payment stream */
+	return (m > 0)? 0 : NADBL;
+    }
+
+    if (gotplus && !gotminus) {
+	/* payments all positive */
+	return (x[0] > 0)? 0.0/0.0 : 1.0/0.0;
+    } else if (gotminus && !gotplus) {
+	/* payments all negative */
+	return (x[0] < 0)? 0.0/0.0 : -1.0/0.0;
+    }
+
+    /* find (r0, r1) bracket for solution, if possible */
+
+    while ((PV = gretl_npv(0, n-1, x, r0, pd, err)) < 0 && !*err) {
+	if (r0 < -DBL_MAX / 2.0) {
+	    return -1.0/0.0;
+	}
+	r1 = r0;
+	r0 *= 2.0;
+    }
+
+    while ((PV = gretl_npv(0, n-1, x, r1, pd, err)) > 0 && !*err) {
+	if (r1 > DBL_MAX / 2.0) {
+	    return 1.0/0.0;
+	}
+	r0 = r1;
+	r1 *= 2.0;
+    } 
+
+#if 0
+    fprintf(stderr, "initial bracket for r: %g to %g\n", r0, r1);
+#endif
+
+    r = r1;
+
+    /* now do binary search */
+
+    for (i=0; i<32 && !*err; i++) {
+	if (floateq(PV, 0.0)) {
+	    break;
+	}
+	if (PV < 0) {
+	    /* r is too high */
+	    if (r < r1) {
+		r1 = r;
+	    }
+	    r = (r + r0) / 2.0;
+	} else {
+	    /* r too low */
+	    if (r > r0) {
+		r0 = r;
+	    }
+	    r = (r + r1) / 2.0;
+	}
+	PV = gretl_npv(0, n-1, x, r, pd, err);
+#if 0
+	fprintf(stderr, "binary search: r = %.9g, PV = %g\n", r, PV);
+#endif
+    }
+
+    if (*err) {
+	r = NADBL;
+    }
+
+    return r;
+}
+
+double logistic_cdf (double x)
+{
+    double emx, ret;
+
+    errno = 0;
+
+    emx = exp(-x);
+
+    if (errno) {
+	errno = 0;
+	return NADBL;
+    }
+
+    ret = 1.0 / (1.0 + emx);
+
+    if (errno) {
+	ret = NADBL;
+	errno = 0;
+    }
+
+    return ret;
+}
