@@ -85,6 +85,8 @@ struct str_table dvars[] = {
     { R_PUNIT,     "$unit" },
     { R_WINDOWS,   "$windows" },
     { R_VERSION,   "$version" },
+    { R_ERRNO,     "$error" },
+    { R_SWITCH,    "$switch" },
     { 0,           NULL },
 };
 
@@ -106,6 +108,7 @@ struct str_table mvars[] = {
     { M_CHISQ,   "$chisq" },
     { M_UHAT,    "$uhat" },
     { M_YHAT,    "$yhat" },
+    { M_LLT,     "$llt" },
     { M_AHAT,    "$ahat" },
     { M_SAMPLE,  "$sample" },
     { M_H,       "$h" },
@@ -114,12 +117,14 @@ struct str_table mvars[] = {
     { M_VCV,     "$vcv" },
     { M_RHO,     "$rho" },
     { M_COMPAN,  "$compan" },
+    { M_XTXINV,  "$xtxinv" },
     { M_JALPHA,  "$jalpha" }, 
     { M_JBETA,   "$jbeta" },
     { M_JVBETA,  "$jvbeta" },
     { M_JS00,    "$s00" },
     { M_JS11,    "$s11" },
     { M_JS01,    "$s01" },
+    { M_EC,      "$ec" },
     { M_HAUSMAN, "$hausman" },
     { M_SARGAN,  "$sargan" },
     { M_SYSGAM,  "$sysGamma" },
@@ -142,6 +147,12 @@ struct str_table funcs[] = {
     { F_ASIN,     "asin" },
     { F_ACOS,     "acos" },
     { F_ATAN,     "atan" },
+    { F_SINH,     "sinh" },
+    { F_COSH,     "cosh" },
+    { F_TANH,     "tanh" },
+    { F_ASINH,    "asinh" },
+    { F_ACOSH,    "acosh" },
+    { F_ATANH,    "atanh" },
     { F_LOG,      "log" },
     { F_LOG,      "ln" },
     { F_LOG10,    "log10" },
@@ -208,10 +219,10 @@ struct str_table funcs[] = {
     { F_REPLACE,  "replace" },
     { F_MUNIF,    "muniform" },
     { F_MNORM,    "mnormal" },
-    { F_SUMR,     "sumr" },
     { F_SUMC,     "sumc" },
-    { F_MEANR,    "meanr" },
+    { F_SUMR,     "sumr" },
     { F_MEANC,    "meanc" },
+    { F_MEANR,    "meanr" },
     { F_SDC,      "sdc" },
     { F_MINC,     "minc" },
     { F_MAXC,     "maxc" },
@@ -282,8 +293,8 @@ struct str_table funcs[] = {
     { F_MPOLS,    "mpols" },
     { F_MREAD,    "mread" },
     { F_MWRITE,   "mwrite" },
-    { F_MRSEL,    "selifr" },
     { F_MCSEL,    "selifc" },
+    { F_MRSEL,    "selifr" },
     { F_POLROOTS, "polroots" },
     { F_DUMIFY,   "dummify" },
     { F_WMEAN,    "wmean" },
@@ -301,9 +312,9 @@ struct str_table funcs[] = {
     { F_READFILE, "readfile" },
     { F_BACKTICK, "grab" },
     { F_STRSTR,   "strstr" },
-    { F_STRCMP,   "strcmp" },
     { F_STRNCMP,  "strncmp" },
     { F_STRLEN,   "strlen" },
+    { F_SSCANF,   "sscanf" },
     { F_VARNAME,  "varname" },
     { F_VARNUM,   "varnum" },
     { F_TOLOWER,  "tolower" },
@@ -317,6 +328,16 @@ struct str_table funcs[] = {
     { F_XMAX,     "xmax" },
     { F_CORRGM,   "corrgm" },
     { F_MCOVG,    "mcovg" },
+    { F_FCSTATS,  "fcstats" },
+    { F_BESSEL,   "bessel" },
+    { F_FRACLAG,  "fraclag" },
+    { F_MREVERSE, "mreverse" },
+    { F_DESEAS,   "deseas" },
+    { F_PERGM,    "pergm" },
+    { F_IRR,      "irr" },
+    { F_NPV,      "npv" },
+    { F_LOGISTIC, "logistic" },
+    { F_WEEKDAY,  "weekday" },
     { 0,          NULL }
 };
 
@@ -488,6 +509,20 @@ const char *data_var_name (int i)
     return dvars[i].str;
 }
 
+const char *gretl_function_complete (const char *s)
+{
+    size_t n = strlen(s);
+    int i;
+
+    for (i=0; funcs[i].str != NULL; i++) {
+	if (!strncmp(s, funcs[i].str, n)) {
+	    return funcs[i].str;
+	}
+    }
+
+    return NULL;
+}
+
 /* end external stuff */
 
 static int dummy_lookup (const char *s)
@@ -593,9 +628,9 @@ static void undefined_symbol_error (const char *s, parser *p)
     parser_print_input(p);
 
     if (p->ch == '.') {
-	sprintf(gretl_errmsg, _("%s: no such object\n"), s);
+	gretl_errmsg_sprintf(_("%s: no such object\n"), s);
     } else {
-	sprintf(gretl_errmsg, _("The symbol '%s' is undefined\n"), s);
+	gretl_errmsg_sprintf(_("The symbol '%s' is undefined\n"), s);
     }
 
     p->err = E_UNKVAR;
@@ -606,9 +641,9 @@ static void function_noargs_error (const char *s, parser *p)
     parser_print_input(p);
 
     pprintf(p->prn, _("'%s': no argument was given\n"), s);
-    sprintf(gretl_errmsg, _("'%s': no argument was given\n"), s);
+    gretl_errmsg_sprintf(_("'%s': no argument was given\n"), s);
 
-    p->err = 1;
+    p->err = E_ARGS;
 }
 
 void context_error (int c, parser *p)
@@ -650,7 +685,7 @@ static char *get_quoted_string (parser *p)
     }
 
     if (!p->err) {
-	if (p->ch == '.' && parser_charpos(p, '$') == 0) {
+	if (p->ch == '.' && *p->point == '$') {
 	    /* maybe quoted name of saved object followed by 
 	       dollar variable? */
 	    p->sym = OVAR;
@@ -779,6 +814,22 @@ static void look_up_dollar_word (const char *s, parser *p)
     }
 }
 
+#ifdef USE_RLIB
+# include "gretl_foreign.h"
+
+static int maybe_get_R_function (const char *s)
+{
+    if (strlen(s) >= 3 && !strncmp(s, "R.", 2)) {
+	return get_R_function_by_name(s + 2);
+    } else {
+	return 0;
+    }
+}
+
+#else /* !USE_RLIB */
+# define maybe_get_R_function(s) (0)
+#endif
+
 static void look_up_word (const char *s, parser *p)
 {
     int fsym, err = 0;
@@ -822,11 +873,17 @@ static void look_up_word (const char *s, parser *p)
 		    p->sym = LIST;
 		    p->idstr = gretl_strdup(s);
 		} else if (!strcmp(s, "t")) {
-		    /* "t" as synonym for internal "obs" */
+		    /* if "t" has not been otherwise defined, treat it
+		       as a synonym for the internal variable "obs" 
+		    */
 		    p->sym = DVAR;
 		    p->idnum = R_INDEX;
+		} else if (maybe_get_R_function(s)) {
+		    /* note: all "native" types take precedence over this */
+		    p->sym = RFUN;
+		    p->idstr = gretl_strdup(s + 2);
 		} else {
-		    err = 1;
+		    err = E_UNKVAR;
 		}
 	    }
 	}
@@ -864,8 +921,9 @@ static void word_check_next_char (const char *s, parser *p)
 		   !func2_symb(p->sym) &&
 		   !func3_symb(p->sym) &&
 		   !funcn_symb(p->sym) && 
-		   p->sym != UFUN) {
-	    p->err = 1;
+		   p->sym != UFUN && 
+		   p->sym != RFUN) {
+	    p->err = E_PARSE;
 	} 
     } else if (p->ch == '[') {
 	if (p->sym == UMAT) {
@@ -878,9 +936,9 @@ static void word_check_next_char (const char *s, parser *p)
 	    /* observation from series */
 	    p->sym = OBS;
 	} else {
-	    p->err = 1;
+	    p->err = E_PARSE;
 	} 
-    } else if (p->ch == '.' && parser_charpos(p, '$') == 0) {
+    } else if (p->ch == '.' && *p->point == '$') {
 	if (p->sym == UOBJ) {
 	    /* name of saved object followed by dollar variable? */
 	    p->sym = OVAR;
@@ -889,13 +947,13 @@ static void word_check_next_char (const char *s, parser *p)
 	       dollar variable? */
 	    p->sym = OVAR;
 	} else {
-	    p->err = 1;
+	    p->err = E_PARSE;
 	}	    
     } else if (p->ch == '.' && isalpha(*p->point)) {
 	if (p->sym == LIST) {
 	    p->sym = LISTVAR;
 	} else {
-	    p->err = 1;
+	    p->err = E_PARSE;
 	}
     }
 
@@ -910,7 +968,7 @@ static int is_word_char (parser *p)
 	return 1;
     } else if (p->targ == LIST && p->ch == '*') {
 	return 1;
-    }
+    } 
 
     return 0;
 }
@@ -924,6 +982,14 @@ static void getword (parser *p)
     word[i++] = p->ch;
     parser_getc(p);
 
+#ifdef USE_RLIB
+    /* allow for R.foo function namespace */
+    if (*word == 'R' && p->ch == '.' && *p->point != '$') {
+	word[i++] = p->ch;
+	parser_getc(p);
+    }
+#endif
+
     while (p->ch != 0 && is_word_char(p) && i < 31) {
 	word[i++] = p->ch;
 	parser_getc(p);
@@ -977,6 +1043,18 @@ void set_matrix_slice_off (void)
     doing_matrix_slice = 0;
 }
 
+static int doing_lag_parse;
+
+void set_lag_parse_on (void)
+{
+    doing_lag_parse = 1;
+}
+
+void set_lag_parse_off (void)
+{
+    doing_lag_parse = 0;
+}
+
 static int colon_ok (char *s, int n)
 {
     int i;
@@ -1075,6 +1153,9 @@ static double getdbl (parser *p)
 	}
     } else {
 	d = dot_atof(xstr);
+#if LDEBUG
+	fprintf(stderr, "getdbl: dot_atof gave %g\n", d);
+#endif
     }
     
     return d;
@@ -1082,7 +1163,8 @@ static double getdbl (parser *p)
 
 #define word_start_special(c) (c == '$' || c == '@' || c == '_')
 
-#define lag_range_sym(p) (p->ch == 't' && *p->point == 'o' && \
+#define lag_range_sym(p) (doing_lag_parse && p->ch == 't' && \
+                          *p->point == 'o' && \
 			  *(p->point + 1) == ' ')
 
 void lex (parser *p)
@@ -1312,7 +1394,7 @@ void lex (parser *p)
 	    } else {
 		parser_print_input(p);
 		pprintf(p->prn, _("Invalid character '%c'\n"), p->ch);
-		p->err = 1;
+		p->err = E_PARSE;
 		return;
 	    }
 	} /* end ch switch */
@@ -1379,7 +1461,7 @@ const char *getsymb (int t, const parser *p)
 	    return dvarname(p->idnum);
 	} else if (t == MVAR) {
 	    return mvarname(p->idnum);
-	} else if (t == UFUN) {
+	} else if (t == UFUN || t == RFUN) {
 	    return p->idstr;
 	} else if (t == STR) {
 	    return p->idstr;
