@@ -251,9 +251,9 @@ int function_from_string (const char *s)
 
 int is_gretl_function_call (const char *s)
 {
-    char c, word[9];
+    char c, word[12];
 
-    sscanf(s, "%8[^ (]", word);
+    sscanf(s, "%10[^ (]", word);
     c = s[strlen(word)];
     if (c == ' ' || c == '(') {
 	return function_lookup(word);
@@ -262,15 +262,6 @@ int is_gretl_function_call (const char *s)
     }
 }
 
-/* "reserved word" data: 
-
-   smpl is special because it can legitimately be followed
-   by '+' or '-' and hence may be confused with the name
-   of a variable in parsing a command line: consider
-   "smpl +3 ;" (command) versus "smpl += 3" (revision of
-   value of variable).
-*/
-
 static const char *reswords[] = {
     /* constants */
     "const",
@@ -285,7 +276,12 @@ static const char *reswords[] = {
     "matrix",
     "string",
     "list",
-    "kalman"
+    "kalman",
+    "void",
+    /* debugging instructions, etc. */
+    "continue",
+    "next",
+    "to"
 };
 
 /**
@@ -404,19 +400,27 @@ int series_index (const DATAINFO *pdinfo, const char *varname)
 
     fd = gretl_function_depth();
 
-    for (i=1; i<pdinfo->v; i++) { 
-	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
-	    if (!strcmp(pdinfo->varname[i], s)) {
-		if (lists_protected() && var_is_listarg(pdinfo, i)) {
-		    /* variable is not visible by name in context */
-		    ;
-#if 0
-		    fprintf(stderr, "var %d (%s) is invisible\n", i, s);
-#endif
-		} else {
-		    ret = i;
-		    break;
-		}
+    if (fd == 0) {
+	/* not inside a user function: easy */
+	for (i=1; i<pdinfo->v; i++) { 
+	    if (strcmp(pdinfo->varname[i], s) == 0) {
+		ret = i;
+		break;
+	    }
+	}
+    } else {
+	/* The condition for recognizing a series by name, if we're
+	   inside a user function: it must exist at the current level
+	   of function execution, and its tenure at that level must
+	   not just be the result of its being a member of a list
+	   that was passed as an argument.
+	*/
+	for (i=1; i<pdinfo->v; i++) { 
+	    if (fd == STACK_LEVEL(pdinfo, i) &&
+		!var_is_listarg(pdinfo, i) && 
+		strcmp(pdinfo->varname[i], s) == 0) {
+		ret = i;
+		break;
 	    }
 	}
     }
@@ -891,8 +895,18 @@ parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 	flags |= P_PRIVATE;
     }
 
+    if (opt & OPT_U) {
+	flags |= P_UFUN;
+    }
+
     *err = realgen(s, p, pZ, pdinfo, NULL, flags);
 
+    if (*err) {
+	gen_cleanup(p);
+	free(p);
+	p = NULL;
+    }
+
 #if GDEBUG
     fprintf(stderr, "genr_compile: err = %d\n", *err);
 #endif
@@ -905,33 +919,23 @@ parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 int execute_genr (parser *p, double ***pZ, DATAINFO *pdinfo,
 		  gretlopt opt, PRN *prn)
 {
-    int flags = P_EXEC;
-
 #if GDEBUG
     fprintf(stderr, "\n*** execute_genr: p=%p, LHS='%s'\n", 
 	    (void *) p, p->lh.name);
 #endif
 
-    if (opt & OPT_L) {
-	flags |= P_LOOP;
-    } else if (opt & OPT_S) {
-	/* context is NLS/MLE or similar */
-	flags |= P_SLAVE;
-    }
+    realgen(NULL, p, pZ, pdinfo, prn, P_EXEC);
 
-    realgen(NULL, p, pZ, pdinfo, prn, flags);
-
-    if (flags & P_SLAVE) {
-	if (p->warn != 0 && p->err == 0) {
-	    /* warnings re. NAs become errors */
-	    p->err = p->warn;
-	}
-    }
-
-    if (p->err == 0) {
+    if (!p->err && !(p->flags & P_UFUN)) {
 	gen_save_or_print(p, prn);
     } 
 
+#if 0
+    if ((p->flags & P_SLAVE) && !p->err) {
+	p->err = p->warn;
+    }
+#endif
+
     gen_cleanup(p);
 
 #if GDEBUG
@@ -1014,6 +1018,11 @@ double genr_get_output_scalar (const parser *p)
     }
 }
 
+int genr_get_warning (const parser *p)
+{
+    return p->warn;
+}
+
 int genr_is_print (const parser *p)
 {
     return (p->flags & P_PRINT);
