@@ -265,7 +265,7 @@ static void expected_symbol_error (int c, parser *p)
 	pputs(p->prn, "(for logical OR, please use \"||\")\n");
     }
 
-    p->err = 1;
+    p->err = E_PARSE;
 }
 
 static void unmatched_symbol_error (int c, parser *p)
@@ -284,8 +284,8 @@ static void unmatched_symbol_error (int c, parser *p)
 static int unary_apost (parser *p) 
 {
     if (p->ch == '\'') {
-	/* peek at the next non-space character */
-	int c = parser_next_char(p);
+	/* peek ahead */
+	int c = parser_next_nonspace_char(p);
 
 	if (isalpha(c) || c == '$' || c == '(') {
 	    /* next symbol is variable or expression: the
@@ -454,6 +454,94 @@ static NODE *listvar_node (parser *p)
     return ret;    
 }
 
+/* here we grab all the arguments to sscanf() as one big string,
+   which will be parsed later */
+
+static NODE *get_sscanf_args (parser *p)
+{
+    const char *src = NULL;
+    int wrapped = 0;
+
+    while (p->ch == ' ') {
+	parser_getc(p);
+    }
+
+    if (p->ch == '"') {
+	wrapped = 1;
+	/* don't lose the leading quote */
+	src = p->point - 1;
+	parser_getc(p);
+    }
+
+    if (p->ch == ')') {
+	p->err = (wrapped)? E_PARSE : E_ARGS;
+	return NULL;
+    } else {
+	const char *s = p->point;
+	int i, paren = 1, quoted = wrapped;
+	int len, close = -1;
+
+	/* find length of string to closing paren */
+	i = 0;
+	while (*s) {
+	    if (!quoted && *s == ')') paren--;
+	    if (paren == 0) {
+		close = i;
+		break;
+	    }
+	    if (*s == '"') {
+		quoted = !quoted;
+	    } else if (!quoted && *s == '(') {
+		paren++;
+	    } 
+	    s++;
+	    i++;
+	}
+
+	if (close < 0) {
+	    unmatched_symbol_error('(', p);
+	    return NULL;
+	}
+
+	if (src == NULL) {
+	    src = p->point - 1;
+	    len = close + 1;
+	} else {
+	    len = close + 2;
+	}
+
+	p->idstr = gretl_strndup(src, len);
+
+	if (p->idstr == NULL) {
+	    p->err = E_ALLOC;
+	    return NULL;
+	}
+
+	tailstrip(p->idstr);
+	parser_advance(p, close);
+    }
+
+    parser_getc(p);
+    lex(p);
+
+#if SDEBUG
+    fprintf(stderr, "get_sscanf_args: '%s'\n", p->idstr);
+#endif
+
+    return newstr(p, STR);
+}
+
+static void unwrap_string_arg (parser *p)
+{
+    int n = strlen(p->idstr);
+
+    if (p->idstr[n-1] == '"') {
+	p->idstr[n-1] = '\0';
+    } else {
+	unmatched_symbol_error('"', p);
+    }
+}
+
 /* Grab a string argument.  Note: we have a mechanism in genlex.c for
    retrieving arguments that take the form of quoted string literals
    or names of string variables.  The special use of this function is
@@ -462,10 +550,15 @@ static NODE *listvar_node (parser *p)
    is a string.  This function assumes that the argument in question
    is the last (or only) argument to a function: we look for a closing
    parenthesis and flag an error if we don't find one.
+
+   Depending on the context we may or may not want to "consume" the
+   trailing right paren that follows the string of interest; that's
+   controlled by the second argument.
 */
 
-static NODE *get_string_arg (parser *p)
+static NODE *get_final_string_arg (parser *p, int eat_last)
 {
+    const char *src = NULL;
     int wrapped = 0;
 
     while (p->ch == ' ') {
@@ -478,7 +571,11 @@ static NODE *get_string_arg (parser *p)
     }
 
     if (p->ch == ')') {
-	/* allow empty arg string "()" */
+	if (wrapped) {
+	    p->err = E_PARSE;
+	    return NULL;
+	} 
+	/* handle empty arg */
 	p->idstr = gretl_strdup("");
     } else {
 	int i, paren = 1, quoted = wrapped;
@@ -497,7 +594,7 @@ static NODE *get_string_arg (parser *p)
 		quoted = !quoted;
 	    } else if (!quoted && *s == '(') {
 		paren++;
-	    }
+	    } 
 	    s++;
 	    i++;
 	}
@@ -509,7 +606,8 @@ static NODE *get_string_arg (parser *p)
 
 	for (i=0; i<=close; i++) {
 	    if (!started && !isspace(p->ch)) {
-		p->idstr = gretl_strndup(p->point - 1, close - i + 1);
+		src = p->point - 1;
+		p->idstr = gretl_strndup(src, close - i + 1);
 		started = 1;
 	    }
 	    parser_getc(p);
@@ -521,30 +619,117 @@ static NODE *get_string_arg (parser *p)
 	return NULL;
     }
 
-    parser_getc(p);
+    tailstrip(p->idstr);
+    if (eat_last) {
+	/* consume trailing right paren */
+	parser_getc(p);
+    }
     lex(p);
 
-    tailstrip(p->idstr);
-
     if (wrapped) {
-	int n = strlen(p->idstr);
-
-	if (p->idstr[n-1] == '"') {
-	    p->idstr[n-1] = '\0';
-	} else {
-	    unmatched_symbol_error('"', p);
-	}
+	unwrap_string_arg(p);
     }
 
 #if SDEBUG
-    fprintf(stderr, "get_string_arg: '%s'\n", p->idstr);
+    fprintf(stderr, "get_final_string_arg: '%s'\n", p->idstr);
 #endif
 
-    return newstr(p, STR);
+    return (p->err)? NULL : newstr(p, STR);
+}
+
+static NODE *get_middle_string_arg (parser *p)
+{
+    const char *src = NULL;
+    const char *s;
+    int gotparen = 0, close = 0;
+    int i, quoted, paren = 0;
+
+    while (p->ch == ' ') {
+	parser_getc(p);
+    }
+
+    if (p->ch == '"') {
+	/* special: arg is wrapped in quotes */
+	close = parser_charpos(p, '"');
+	if (close < 0) {
+	    unmatched_symbol_error('"', p);
+	    return NULL;
+	}
+	p->idstr = gretl_strndup(p->point, close);
+	close++;
+	goto post_process;
+    }
+
+    /* find length of string to bare comma (unquoted, not in parens) or
+       matched right paren */
+
+    quoted = 0;
+    s = p->point;
+    i = 0;
+
+    while (*s) {
+	if (*s == '"') {
+	    quoted = !quoted;
+	} 
+	if (!quoted) {
+	    if (*s == '(') {
+		gotparen++;
+		paren++;
+	    } else if (*s == ')') {
+		paren--;
+	    }
+	    if (paren == 0) {
+		if (gotparen) {
+		    /* include right paren */
+		    close = i + 1;
+		    break;
+		} else if (*s == ',') {
+		    /* leave comma */
+		    close = i;
+		    break;
+		} 
+	    } 
+	}
+	s++;
+	i++;
+    }
+
+    if (paren > 0) {
+	unmatched_symbol_error('(', p);
+    } else if (paren < 0) {
+	unmatched_symbol_error(')', p);
+    } else if (quoted) {
+	unmatched_symbol_error('"', p);
+    }
+
+    if (p->err) {
+	return NULL;
+    }
+
+    src = p->point - 1;
+    p->idstr = gretl_strndup(src, close + 1);
+
+ post_process:
+
+    if (p->idstr == NULL) {
+	p->err = E_ALLOC;
+	return NULL;
+    }  
+
+    tailstrip(p->idstr);
+    parser_advance(p, close);
+    lex(p);
+
+#if SDEBUG
+    fprintf(stderr, "get_middle_string_arg: '%s'\n", p->idstr);
+#endif
+
+    return (p->err)? NULL : newstr(p, STR);
 }
 
 enum {
-    RIGHT_STR = 1
+    RIGHT_STR = 1 << 0,
+    MID_STR   = 1 << 1
 };
 
 static void get_matrix_def (NODE *t, parser *p, int *sub)
@@ -680,14 +865,14 @@ static void get_slice_parts (NODE *t, parser *p)
 static void attach_child (NODE *parent, NODE *child, int k, int i,
 			  parser *p)
 {
-#if SDEBUG
-    fprintf(stderr, "attach_child: i=%d, type = %d\n", i, child->t);
-#endif
-
     if (p->err) {
 	return;
     }
 
+#if SDEBUG
+    fprintf(stderr, "attach_child: i=%d, type=%d\n", i, child->t);
+#endif
+
     if (k == 2) {
 	/* 2-place node */
 	if (i == 0) {
@@ -721,12 +906,15 @@ static void pad_parent (NODE *parent, int k, int i, parser *p)
     }
 }
 
+#define next_arg_is_string(i,k,o) ((i>0 && i<k-1 && (o & MID_STR)) || \
+                                   (i==k-1 && (o & RIGHT_STR)))
+
 /* Get up to k comma-separated arguments (possibly optional).
    However, if k < 0 this is a signal for get as many arguments as we
    can find (the number is unknown in advance).
 */
 
-static void get_args (NODE *t, parser *p, int k, int opt, int *next)
+static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
 {
     NODE *child;
     char cexp = 0;
@@ -745,27 +933,40 @@ static void get_args (NODE *t, parser *p, int k, int opt, int *next)
 
     while (((k > 0 && i < k) || p->ch) && !p->err) {
 	if (p->sym == G_RPR) {
+	    /* right paren: got to the end */
 	    break;
 	}
-	child = expr(p);
+
+	if (k > 0 && i == k) {
+	    gretl_errmsg_sprintf("%s: %s", getsymb(f, p), _("too many arguments"));
+	    p->err = E_ARGS;
+	    break;
+	}
+
+	/* get the next argument */
+	if (i > 0 && i < k - 1 && (opt & MID_STR)) {
+	    child = get_middle_string_arg(p);
+	} else if (i == k - 1 && (opt & RIGHT_STR)) {
+	    child = get_final_string_arg(p, 0);
+	} else {
+	    child = expr(p);
+	}
+
 	attach_child(t, child, k, i++, p);
-	if (p->err) {
-	    break;
-	}
-	if (p->sym == G_RPR) {
+
+	if (p->err || p->sym == G_RPR) {
 	    break;
 	} else if (p->sym == P_COM) {
 	    /* turn off flag for accepting string as first arg */
 	    p->flags &= ~P_GETSTR;
-	    /* and handle final string arg if relevant */
-	    if (i == k - 1 && opt == RIGHT_STR) {
-		child = get_string_arg(p);
-		attach_child(t, child, k, i++, p);
-	    } else {
+	    /* lex unless next arg needs special handling */
+	    if (!next_arg_is_string(i, k, opt)) {
 		lex(p);
 	    }
 	} else {
+	    /* arg-separating comma was expected */
 	    cexp = ',';
+	    break;
 	}
     }
 
@@ -776,7 +977,7 @@ static void get_args (NODE *t, parser *p, int k, int opt, int *next)
 	    if (i < k) {
 		pad_parent(t, k, i, p);
 	    } 
-	    /* handle trailing transpose */
+	    /* matrix functions: handle trailing transpose */
 	    if (p->ch == '\'') {
 		set_transpose(t);
 		parser_getc(p);
@@ -786,7 +987,11 @@ static void get_args (NODE *t, parser *p, int k, int opt, int *next)
 		*next = G_LBR;
 	    }
 	}
-    }	
+    }
+
+#if SDEBUG
+    fprintf(stderr, "get_args: returning with p->err=%d\n", p->err);
+#endif    
 }
 
 static void get_ovar_ref (NODE *t, parser *p)
@@ -833,10 +1038,14 @@ static NODE *powterm (parser *p)
 	    p->sym, p->ch? p->ch : '0', p->ch);
 #endif
 
-    if (sym == F_FDJAC || sym == F_BFGSMAX) {
-	opt = RIGHT_STR;
+    if (string_last_func(sym)) {
+	opt |= RIGHT_STR;
     } 
 
+    if (string_mid_func(sym)) {
+	opt |= MID_STR;
+    }    
+
     if (unary_op(sym)) {
 	if (p->ch == 0) {
 	    context_error(0, p);
@@ -851,22 +1060,25 @@ static NODE *powterm (parser *p)
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    lex(p);
-	    get_args(t, p, 2, opt, &next);
+	    get_args(t, p, sym, 2, opt, &next);
 	}
     } else if (func3_symb(sym)) {
 	t = newb3(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
-	    get_args(t, p, 3, opt, &next);
+	    if (char0_func(sym)) {
+		p->flags |= P_GETSTR;
+	    }
+	    get_args(t, p, sym, 3, opt, &next);
 	}
-    } else if (string0_func(sym)) {
+    } else if (char0_func(sym)) {
 	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
 		p->flags |= P_GETSTR;
-		get_args(t->v.b1.b, p, -1, opt, &next);
+		get_args(t->v.b1.b, p, sym, -1, opt, &next);
 	    }
 	}
     } else if (sym == F_URCPVAL) {
@@ -875,14 +1087,21 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
-		get_args(t->v.b1.b, p, -1, opt, &next);
+		get_args(t->v.b1.b, p, sym, -1, opt, &next);
 	    }
+	}
+    } else if (sym == F_SSCANF) {
+	/* string arg is handled specially */
+	t = newb1(sym, NULL);
+	if (t != NULL) {
+	    lex(p);
+	    t->v.b1.b = get_sscanf_args(p);
 	}	
     } else if (string_arg_func(sym)) {
 	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
-	    t->v.b1.b = get_string_arg(p);
+	    t->v.b1.b = get_final_string_arg(p, 1);
 	}	
     } else if (func1_symb(sym)) {
 	t = newb1(sym, NULL);
@@ -894,12 +1113,18 @@ static NODE *powterm (parser *p)
 	    }
 	}
     } else if (sym == LAG || sym == OBS) {
+	if (sym == LAG) {
+	    set_lag_parse_on();
+	}
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newref(p, USERIES); 
 	    lex(p);
 	    t->v.b2.r = base(p, t);
 	}
+	if (sym == LAG) {
+	    set_lag_parse_off();
+	}	
     } else if (sym == MSL || sym == DMSL) {
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
@@ -919,7 +1144,7 @@ static NODE *powterm (parser *p)
 	if (t != NULL) {
 	    t->v.b2.l = newref(p, MVAR);
 	    lex(p);
-	    t->v.b2.r = get_string_arg(p);
+	    t->v.b2.r = get_final_string_arg(p, 1);
 	}
     } else if (sym == OVAR) {
 	t = newb2(sym, NULL, NULL);
@@ -931,6 +1156,8 @@ static NODE *powterm (parser *p)
 	t = listvar_node(p);
 	if (t != NULL && (p->sym == G_LPR || p->sym == G_LBR)) {
 	    /* list.series node may be "inflected" as lag or obs */
+	    if (p->sym == G_LPR) 
+		fprintf(stderr, "*** listvar: setting LAG\n");
 	    p->sym = (p->sym == G_LPR)? LAG : OBS;
 	    t = newb2(p->sym, t, NULL);
 	    if (t != NULL) {
@@ -965,14 +1192,14 @@ static NODE *powterm (parser *p)
 		}		
 	    }
 	}
-    } else if (sym == UFUN) {
+    } else if (sym == UFUN || sym == RFUN) {
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newstr(p, STR);
 	    lex(p);
 	    t->v.b2.r = newbn(FARGS);
 	    if (t != NULL) {
-		get_args(t->v.b2.r, p, -1, opt, &next);
+		get_args(t->v.b2.r, p, sym, -1, opt, &next);
 	    }
 	}
     } else if (funcn_symb(sym)) {
@@ -981,7 +1208,7 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
-		get_args(t->v.b1.b, p, -1, opt, &next);
+		get_args(t->v.b1.b, p, sym, -1, opt, &next);
 	    }
 	}
     } else if (sym == STR || sym == VSTR) {
