@@ -37,22 +37,20 @@ typedef enum {
     GPT_GUI            = 1 << 7,  /* called from GUI context */
     GPT_FIT_OMIT       = 1 << 8,  /* User said don't draw fitted line on graph */
     GPT_DATA_STYLE     = 1 << 9,  /* data style is set by user */
-    GPT_FILE           = 1 << 10, /* send output to named file */
-    GPT_IDX            = 1 << 11, /* plot against time or obs index */
-    GPT_TS             = 1 << 12, /* doing time series plot */
-    GPT_Y2AXIS         = 1 << 13, /* plot has second y-axis */
-    GPT_AUTO_FIT       = 1 << 14, /* automatic (OLS) fitted line was added */
-    GPT_FIT_HIDDEN     = 1 << 15, /* autofit line calculated, but suppressed */
-    GPT_PNG_OUTPUT     = 1 << 16, /* output is to PNG file */
-    GPT_PRINT_MARKERS  = 1 << 17, /* print observation markers */
-    GPT_MARKERS_OK     = 1 << 18, /* OK to print observation markers */
-    GPT_LETTERBOX      = 1 << 19, /* special format for time series graphs */
-    GPT_PARAMETRIC     = 1 << 20, /* gnuplot should be in parametric mode */
-    GPT_XZEROAXIS      = 1 << 21, /* show x = 0 line */
-    GPT_YZEROAXIS      = 1 << 22, /* show y = 0 line */
-    GPT_FILL_SWITCH    = 1 << 23, /* switching from errorbars to fill */
-    GPT_ERR_SWITCH     = 1 << 24, /* switching from fill to errorbars */
-    GPT_MONO           = 1 << 25  /* monochrome output */
+    GPT_IDX            = 1 << 10, /* plot against time or obs index */
+    GPT_TS             = 1 << 11, /* doing time series plot */
+    GPT_Y2AXIS         = 1 << 12, /* plot has second y-axis */
+    GPT_AUTO_FIT       = 1 << 13, /* automatic (OLS) fitted line was added */
+    GPT_FIT_HIDDEN     = 1 << 14, /* autofit line calculated, but suppressed */
+    GPT_PNG_OUTPUT     = 1 << 15, /* output is to PNG file */
+    GPT_PRINT_MARKERS  = 1 << 16, /* print observation markers */
+    GPT_LETTERBOX      = 1 << 17, /* special format for time series graphs */
+    GPT_PARAMETRIC     = 1 << 18, /* gnuplot should be in parametric mode */
+    GPT_XZEROAXIS      = 1 << 19, /* show x = 0 line */
+    GPT_YZEROAXIS      = 1 << 20, /* show y = 0 line */
+    GPT_FILL_SWITCH    = 1 << 21, /* switching from errorbars to fill */
+    GPT_ERR_SWITCH     = 1 << 22, /* switching from fill to errorbars */
+    GPT_MONO           = 1 << 23  /* monochrome output */
 } GptFlags; 
 
 typedef struct gretlRGB_ gretlRGB;
@@ -95,6 +93,7 @@ typedef enum {
     PLOT_RQ_TAU,
     PLOT_BOXPLOTS,
     PLOT_CURVE,
+    PLOT_QQ,
     PLOT_TYPE_MAX
 } PlotType;
 
@@ -148,11 +147,13 @@ const char *gp_justification_string (int j);
 
 const char *gnuplot_label_front_string (void);
 
+int split_graph_fontspec (const char *s, char *name, int *psz);
+
 void gnuplot_missval_string (FILE *fp);
 
 int gnuplot_init (PlotType ptype, FILE **fpp);
 
-FILE *gnuplot_batch_init (int *err);
+FILE *gnuplot_batch_init (const char *optname, int *err);
 
 int specified_gp_output_format (void);
 
@@ -187,11 +188,17 @@ int plot_freq (FreqDist *freq, DistCode dist);
 int garch_resid_plot (const MODEL *pmod, const DATAINFO *pdinfo); 
 
 int rmplot (const int *list, const double **Z, DATAINFO *pdinfo, 
-	    PRN *prn);
+	    gretlopt opt, PRN *prn);
 
 int hurstplot (const int *list, const double **Z, DATAINFO *pdinfo, 
 	       PRN *prn);
 
+int qq_plot (const int *list, const double **Z, 
+	     const DATAINFO *pdinfo, gretlopt opt);
+
+int theil_forecast_plot (const int *plotlist, const double **Z, 
+			 const DATAINFO *pdinfo, gretlopt opt);
+
 int 
 gretl_panel_ts_plot (const int *list, const double **Z, DATAINFO *pdinfo,
 		     gretlopt opt);
@@ -251,6 +258,8 @@ int gnuplot_has_bbox (void);
 
 int gnuplot_has_utf8 (void);
 
+int gnuplot_uses_datafile_missing (void);
+
 void set_graph_palette (int i, gretlRGB color);
 
 void set_graph_palette_from_string (int i, const char *cstr);
@@ -269,5 +278,9 @@ int gnuplot_test_command (const char *cmd);
 
 void gnuplot_png_set_use_aa (int s);
 
+#ifndef WIN32
+int gnuplot_has_wxt (void);
+#endif
+
 #endif /* GRAPHING_H */
 
