@@ -23,12 +23,51 @@
 #include "gretl_bfgs.h"
 #include "libset.h"
 #include "matrix_extra.h"
+#include "usermat.h"
+#include "gretl_scalar.h"
 
 #include "gretl_f2c.h"
 #include "../../minpack/minpack.h"  
 
 #define BFGS_DEBUG 0
 
+#define BFGS_MAXITER_DEFAULT 600
+
+void BFGS_defaults (int *maxit, double *tol, int ci)
+{
+    *maxit = libset_get_int(BFGS_MAXITER);
+    *tol = libset_get_user_tolerance(BFGS_TOLER);
+
+    if (ci != MLE && ci != GMM && *maxit <= 0) {
+	*maxit = 1000;
+    }
+
+    if (ci == PROBIT || ci == INTREG || ci == ARMA) {
+	if (na(*tol)) {
+	    *tol = 1.0e-12;
+	}
+    } else if (ci == TOBIT) {
+	if (na(*tol)) {
+	    *tol = 1.0e-10; /* calibrated against Wm Greene */
+	}
+    } else if (ci == HECKIT) {
+	if (na(*tol)) {
+	    *tol = 1.0e-09;
+	}
+    } else if (ci == GARCH) {
+	if (na(*tol)) {
+	    *tol = 1.0e-13;
+	}
+    } else if (ci == MLE || ci == GMM) {
+	if (*maxit <= 0) {
+	    *maxit = BFGS_MAXITER_DEFAULT;
+	}
+	if (na(*tol)) {
+	    *tol = libset_get_double(BFGS_TOLER);
+	}
+    }
+}
+
 static void free_triangular_array (double **m, int n)
 {
     if (m != NULL) {
@@ -100,6 +139,8 @@ static void hess_b_adjust_ij (double *c, double *b, double *h, int n,
     }
 }
 
+#define RSTEPS 4
+
 /* The algorithm below implements the method of Richardson
    Extrapolation.  It is derived from code in the gnu R package
    "numDeriv" by Paul Gilbert, which was in turn derived from C code
@@ -167,12 +208,16 @@ double *numerical_hessian (const double *b, int n, BFGS_CRIT_FUNC func,
 	    hess_b_adjust_i(c, bcpy, h, n, i, 1);
 	    f1 = func(c, data);
 	    if (na(f1)) {
+		fprintf(stderr, "numerical_hessian: 1st derivative: "
+			"criterion = NA for theta[%d] = %g\n", i, c[i]);
 		*err = E_NAN;
 		goto bailout;
 	    }
 	    hess_b_adjust_i(c, bcpy, h, n, i, -1);
 	    f2 = func(c, data);
 	    if (na(f2)) {
+		fprintf(stderr, "numerical_hessian: 1st derivative: "
+			"criterion = NA for theta[%d] = %g\n", i, c[i]);
 		*err = E_NAN;
 		goto bailout;
 	    }
@@ -207,12 +252,16 @@ double *numerical_hessian (const double *b, int n, BFGS_CRIT_FUNC func,
 		    hess_b_adjust_ij(c, bcpy, h, n, i, j, 1);
 		    f1 = func(c, data);
 		    if (na(f1)) {
+			fprintf(stderr, "numerical_hessian: 2nd derivatives (%d,%d): "
+				"objective function gave NA\n", i, j);
 			*err = E_NAN;
 			goto bailout;
 		    }
 		    hess_b_adjust_ij(c, bcpy, h, n, i, j, -1);
 		    f2 = func(c, data);
 		    if (na(f2)) {
+			fprintf(stderr, "numerical_hessian: 2nd derivatives (%d,%d): "
+				"objective function gave NA\n", i, j);
 			*err = E_NAN;
 			goto bailout;
 		    }
@@ -255,7 +304,7 @@ double *numerical_hessian (const double *b, int n, BFGS_CRIT_FUNC func,
 
  bailout:
 
-    if (*err == E_NAN) {
+    if (*err != 0 && *err != E_ALLOC) {
 	gretl_errmsg_set(_("Failed to compute numerical Hessian"));
     }
 
@@ -272,7 +321,7 @@ int BFGS_numeric_gradient (double *b, double *g, int n,
 {
     double bi0, f1, f2;
     gretlopt opt = OPT_NONE;
-    int i;
+    int i, err = 0;
 
     if (opt == OPT_R) {
 	/* Richardson */
@@ -294,7 +343,8 @@ int BFGS_numeric_gradient (double *b, double *g, int n,
 		f2 = func(b, data);
 		if (na(f1) || na(f2)) {
 		    b[i] = bi0;
-		    return 1;
+		    err = 1;
+		    goto bailout;
 		}		    
 		df[k] = (f2 - f1) / (2.0 * h); 
 		h /= v;
@@ -321,7 +371,8 @@ int BFGS_numeric_gradient (double *b, double *g, int n,
 	    f2 = func(b, data);
 	    b[i] = bi0;
 	    if (na(f1) || na(f2)) {
-		return 1;
+		err = 1;
+		goto bailout;
 	    }
 	    g[i] = (f2 - f1) / (2.0 * h);
 #if BFGS_DEBUG > 1
@@ -331,7 +382,13 @@ int BFGS_numeric_gradient (double *b, double *g, int n,
 	}
     }
 
-    return 0;
+ bailout:
+
+#if BFGS_DEBUG
+    fprintf(stderr, "BFGS_numeric_gradient: returning %d\n", err);
+#endif
+
+    return err;
 }
 
 #define stepfrac	0.2
@@ -340,15 +397,6 @@ int BFGS_numeric_gradient (double *b, double *g, int n,
 
 #define coeff_unchanged(a,b) (reltest + a == reltest + b)
 
-static void reverse_gradient (double *g, int n)
-{
-    int i;
-
-    for (i=0; i<n; i++) {
-	g[i] = -g[i];
-    }
-}
-
 static int broken_gradient (double *g, int n)
 {
     int i;
@@ -375,9 +423,12 @@ static void BFGS_get_user_values (double *b, int n, int *maxit,
 {
     const gretl_matrix *uinit;
     int uilen, umaxit;
-    double utol, deftol;
+    double utol;
     int i;
 
+    /* we first check to see if we've been a usable initialization
+       for the parameter estimates */
+
     uinit = get_init_vals();
     uilen = gretl_vector_get_length(uinit);
 
@@ -404,29 +455,33 @@ static void BFGS_get_user_values (double *b, int n, int *maxit,
 	}
     }
 
+    /* then check for a setting of the maximum number
+       of iterations */
+
     umaxit = libset_get_int(BFGS_MAXITER);
-    if (umaxit > 0) {
+    if (umaxit >= 0) {
 	*maxit = umaxit;
     } else if (*maxit < 0) {
-	*maxit = 500;
+	*maxit = BFGS_MAXITER_DEFAULT;
     }
 
-    deftol = get_default_nls_toler();
-    if (*reltol == 0) {
-	/* unset: use default */
-	*reltol = deftol;
-    }
-    
-    utol = libset_get_double(BFGS_TOLER);
-    if (utol != deftol) {
-	/* user values has actually been set */
+    /* and then the convergence tolerance */
+
+    utol = libset_get_user_tolerance(BFGS_TOLER);
+    if (!na(utol)) {
+	/* the user has actually set a value */
 	*reltol = utol;
 	if (!(opt & OPT_Q)) {
 	    fprintf(stderr, "user-specified BFGS tolerance = %g\n", utol);
 	}
-    }	
+    } else if (*reltol == 0) {
+	/* use the generic BFGS default */
+	*reltol = libset_get_double(BFGS_TOLER);
+    }
 }
 
+#define bfgs_print_iter(v,s,i) (v && (s == 1 || i % s == 0))
+
 int BFGS_orig (double *b, int n, int maxit, double reltol,
 	       int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
 	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
@@ -436,9 +491,9 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
     double *wspace = NULL;
     double **H = NULL;
     double *g, *t, *X, *c;
-    int verbose = (opt & OPT_V);
+    int verbskip, verbose = (opt & OPT_V);
     int fcount, gcount, ndelta = 0;
-    double d, fmax, f, f0, sumgrad;
+    double fmax, f, f0, sumgrad, d = 0.0;
     int i, j, ilast, iter;
     double s, steplen = 0.0;
     double D1, D2;
@@ -467,18 +522,17 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 
     if (na(f)) {
 	gretl_errmsg_set(_("BFGS: initial value of objective function is not finite"));
-	err = E_DATA;
+	err = E_NAN;
 	goto bailout;
     }
 
 #if BFGS_DEBUG
-    fprintf(stderr, "BFGS: first evaluation of f = %g\n", f);
+    fprintf(stderr, "*** BFGS: first evaluation of f = %g\n", f);
 #endif
 
     f0 = fmax = f;
     iter = ilast = fcount = gcount = 1;
     gradfunc(b, g, n, cfunc, data);
-    reverse_gradient(g, n);
 
 #if BFGS_DEBUG
     fprintf(stderr, "initial gradient:\n");
@@ -487,11 +541,15 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
     }
 #endif
 
+    if (maxit == 0) {
+	goto skipcalc;
+    }
+
+    verbskip = libset_get_int("bfgs_verbskip");
+
     do {
-	if (verbose) {
-	    reverse_gradient(g, n);
+	if (bfgs_print_iter(verbose, verbskip, iter)) {
 	    print_iter_info(iter, f, crittype, n, b, g, steplen, prn);
-	    reverse_gradient(g, n);
 	}
 
 	if (ilast == gcount) {
@@ -514,16 +572,21 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 	for (i=0; i<n; i++) {
 	    s = 0.0;
 	    for (j=0; j<=i; j++) {
-		s -= H[i][j] * g[j];
+		s += H[i][j] * g[j];
 	    }
 	    for (j=i+1; j<n; j++) {
-		s -= H[j][i] * g[j];
+		s += H[j][i] * g[j];
 	    }
 	    t[i] = s;
 	    sumgrad += s * g[i];
 	}
 
-	if (sumgrad < 0.0) {	
+#if BFGS_DEBUG
+	fprintf(stderr, "\niter %d: sumgrad = %g\n", iter, sumgrad);
+#endif
+
+	if (sumgrad > 0.0) { 
+	    /* heading in the right direction */
 	    steplen = 1.0;
 	    crit_ok = 0;
 	    do {
@@ -539,9 +602,13 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 		}
 		if (ndelta > 0) {
 		    f = cfunc(b, data);
-		    d = sumgrad * steplen * acctol;
+		    d = -sumgrad * steplen * acctol;
 		    fcount++;
 		    crit_ok = !na(f) && (f >= fmax + d);
+#if BFGS_DEBUG
+		    fprintf(stderr, "crit_ok: f=%.10g, fmax=%.10g, d=%g, steplen=%g, ok=%d\n",
+			    f, fmax, d, steplen, crit_ok);
+#endif
 		    if (!crit_ok) {
 			/* calculated criterion no good: try smaller step */
 			steplen *= stepfrac;
@@ -565,17 +632,22 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 
 	    if (ndelta > 0) {
 		/* making progress */
+#if BFGS_DEBUG
+		fprintf(stderr, "making progress, ndelta = %d\n", ndelta);
+#endif
 		fmax = f;
 		gradfunc(b, g, n, cfunc, data);
-		reverse_gradient(g, n);
 		gcount++;
 		iter++;
 		D1 = 0.0;
 		for (i=0; i<n; i++) {
-		    t[i] = steplen * t[i];
-		    c[i] = g[i] - c[i];
+		    t[i] *= steplen;
+		    c[i] -= g[i];
 		    D1 += t[i] * c[i];
 		}
+#if BFGS_DEBUG
+		fprintf(stderr, "D1 = %g\n", D1);
+#endif
 		if (D1 > 0.0) {
 		    D2 = 0.0;
 		    for (i=0; i<n; i++) {
@@ -595,6 +667,9 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 			    H[i][j] += (D2 * t[i]*t[j] - X[i]*t[j] - t[i]*X[j]) / D1;
 			}
 		    }
+#if BFGS_DEBUG
+		    fprintf(stderr, "D2 = %g\n", D2);
+#endif
 		} else {
 		    /* D1 <= 0.0 */
 		    ilast = gcount;
@@ -635,8 +710,8 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
     } while (ndelta > 0 || ilast < gcount);
 
 #if BFGS_DEBUG
-    fprintf(stderr, "terminated: ndelta=%d, ilast=%d, gcount=%d\n",
-	    ndelta, ilast, gcount);
+    fprintf(stderr, "terminated: fmax=%g, ndelta=%d, ilast=%d, gcount=%d\n",
+	    fmax, ndelta, ilast, gcount);
 #endif
 
     if (iter >= maxit) {
@@ -644,15 +719,17 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
 	err = E_NOCONV;
     } else if (fmax < f0) {
 	/* FIXME this should never happen */
-	fprintf(stderr, "failed to match initial value of objective function, %g\n", f0);
+	fprintf(stderr, "failed to match initial value of objective function:\n"
+		" f0=%.15g, fmax=%.15g\n", f0, fmax);
 	err = E_NOCONV;
-    }
+    } 
+
+ skipcalc:
 
     *fncount = fcount;
     *grcount = gcount;
 
     if (verbose) {
-	reverse_gradient(g, n);
 	print_iter_info(-1, f, crittype, n, b, g, steplen, prn);
 	pputc(prn, '\n');
     }
@@ -669,6 +746,21 @@ int BFGS_orig (double *b, int n, int maxit, double reltol,
     return err;
 }
 
+/* Note: we need this because the original L-BFGS-B code is
+   set up as a minimizer.  We could get rid of it if anyone
+   has the strength to go into lbfgsb.c and make the
+   necessary adjustments.
+*/
+
+static void reverse_gradient (double *g, int n)
+{
+    int i;
+
+    for (i=0; i<n; i++) {
+	g[i] = -g[i];
+    }
+}
+
 int LBFGS_max (double *b, int n, int maxit, double reltol,
 	       int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
 	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
@@ -680,7 +772,6 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
     double *wa = NULL;
     int *nbd = NULL;
     int *iwa = NULL;
-
     int i, m, wadim;
     char task[60];
     char csave[60];
@@ -689,6 +780,7 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
     int isave[44];
     int lsave[4];
     int iter, ibak = 0;
+    int verbskip, verbose = (opt & OPT_V);
     int err = 0;
 
     *fncount = *grcount = 0;    
@@ -700,8 +792,10 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
       It is not altered by the routine.  Values of m < 3 are not
       recommended, and large values of m can result in excessive
       computing time. The range 3 <= m <= 20 is recommended.
+
+      Was initially set to 5 (then 10, then 8; and 8 is the default).
     */
-    m = 10; /* was initially set to 5 */
+    m = libset_get_int(LBFGS_MEM); 
 
     wadim = (2*m+4)*n + 12*m*m + 12*m;
 
@@ -718,6 +812,8 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 	goto bailout;
     }
 
+    verbskip = libset_get_int("bfgs_verbskip");
+
     if (gradfunc == NULL) {
 	gradfunc = BFGS_numeric_gradient;
     }
@@ -747,7 +843,7 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 	    /* Compute function value, f */
 	    f = cfunc(b, data);
 	    if (!na(f)) {
-		f = -f;
+		f = -f; /* maximize, don't minimize */
 	    } else if (*fncount == 0) {
 		fprintf(stderr, "initial value of f is not finite\n");
 		err = E_DATA;
@@ -773,7 +869,7 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 	    break;
 	}
 
-	if (opt & OPT_V) {
+	if (bfgs_print_iter(verbose, verbskip, iter)) {
 	    if (iter != ibak) {
 		double steplen = (iter == 1)? NADBL : dsave[13];
 
@@ -824,7 +920,8 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
  * gradient, or %NULL for default numerical calculation.
  * @data: pointer that will be passed as the last
  * parameter to the callback functions @cfunc and @gradfunc.
- * @opt: may contain %OPT_V for verbose operation.
+ * @opt: may contain %OPT_V for verbose operation, %OPT_L to
+ * force use of L-BFGS-B.
  * @prn: printing struct (or %NULL).  Only used if @opt
  * includes %OPT_V.
  *
@@ -844,7 +941,7 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 	      int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
 	      gretlopt opt, PRN *prn)
 {
-    if (libset_get_bool(USE_LBFGS)) {
+    if ((opt & OPT_L) || libset_get_bool(USE_LBFGS)) {
 	return LBFGS_max(b, n, maxit, reltol,
 			 fncount, grcount, cfunc, 
 			 crittype, gradfunc, data, 
@@ -857,44 +954,65 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
     }
 }
 
-/* user-level access to BFGS */
+/* user-level access to BFGS and fdjac */
 
 typedef struct umax_ umax;
 
 struct umax_ {
-    GretlType gentype;
-    gretl_matrix *b;
-    int ncoeff;
-    GENERATOR *g;
-    double x_out;
-    gretl_matrix *m_out;
-    double ***Z;
-    DATAINFO *dinfo;
-    PRN *prn;
+    GretlType gentype;    /* GRETL_TYPE_DOUBLE or GRETL_TYPE_MATRIX */
+    gretl_matrix *b;      /* parameter vector */
+    gretl_matrix *g;      /* gradient vector */
+    int ncoeff;           /* number of coefficients */
+    GENERATOR *gf;        /* for generating scalar or matrix result */
+    GENERATOR *gg;        /* for generating gradient */
+    double fx_out;        /* function double value */
+    gretl_matrix *fm_out; /* function matrix value */
+    char gmname[VNAMELEN]; /* name of user-defined gradient vector */
+    double ***Z;          /* pointer to data array */
+    DATAINFO *dinfo;      /* dataset info */
+    PRN *prn;             /* optional printing struct */
 };
 
-static void umax_init (umax *u, GretlType t)
+static umax *umax_new (GretlType t)
 {
-    u->gentype = t;
-    u->b = NULL;
-    u->ncoeff = 0;
-    u->g = NULL;
-    u->x_out = NADBL;
-    u->m_out = NULL;
-    u->Z = NULL;
-    u->dinfo = NULL;
-    u->prn = NULL;
+    umax *u = malloc(sizeof *u);
+
+    if (u != NULL) {
+	u->gentype = t;
+	u->b = NULL;
+	u->g = NULL;
+	u->ncoeff = 0;
+	u->gf = NULL;
+	u->gg = NULL;
+	u->fx_out = NADBL;
+	u->fm_out = NULL;
+	u->gmname[0] = '\0';
+	u->Z = NULL;
+	u->dinfo = NULL;
+	u->prn = NULL;
+    }
+
+    return u;
 }
 
-static void umax_clear (umax *u)
+static void umax_destroy (umax *u)
 {
     if (u->dinfo != NULL) {
-	/* drop any "$" variables created */
+	/* drop any private "$" series created */
 	dataset_drop_listed_variables(NULL, u->Z, u->dinfo, NULL, NULL);
     }
-    destroy_genr(u->g);
+
+    gretl_scalar_delete("$umax", NULL);
+    user_matrix_destroy_by_name("$umax", NULL);
+
+    destroy_genr(u->gf);
+    destroy_genr(u->gg);
+
+    free(u);
 }
 
+/* user-defined BFGS: get the criterion value */
+
 static double user_get_criterion (const double *b, void *p)
 {
     umax *u = (umax *) p;
@@ -905,38 +1023,105 @@ static double user_get_criterion (const double *b, void *p)
 	u->b->val[i] = b[i];
     }
 
-    err = execute_genr(u->g, u->Z, u->dinfo, OPT_S, u->prn); 
+    err = execute_genr(u->gf, u->Z, u->dinfo, OPT_S, u->prn); 
 
     if (err) {
 	return NADBL;
     }
 
-    t = genr_get_output_type(u->g);
+    t = genr_get_output_type(u->gf);
 
     if (t == GRETL_TYPE_DOUBLE) {
-	x = genr_get_output_scalar(u->g);
+	x = genr_get_output_scalar(u->gf);
     } else if (t == GRETL_TYPE_MATRIX) {
-	gretl_matrix *m = genr_get_output_matrix(u->g);
+	gretl_matrix *m = genr_get_output_matrix(u->gf);
 
-	if (m != NULL && m->rows == 1 && m->cols == 1) {
+	if (gretl_matrix_is_scalar(m)) {
 	    x = m->val[0];
+	} else {
+	    err = E_TYPES;
 	}
     } else {
 	err = E_TYPES;
     }
 
-    u->x_out = x;
+    u->fx_out = x;
     
     return x;
 }
 
+/* user-defined BFGS: get the gradient, if specified */
+
+static int user_get_gradient (double *b, double *g, int k,
+			      BFGS_CRIT_FUNC func, void *p)
+{
+    umax *u = (umax *) p;
+    gretl_matrix *ug;
+    int i, err;
+
+    for (i=0; i<k; i++) {
+	u->b->val[i] = b[i];
+    }
+
+    err = execute_genr(u->gg, u->Z, u->dinfo, OPT_S, u->prn); 
+
+    if (err) {
+	return err;
+    }
+
+    ug = get_matrix_by_name(u->gmname);
+
+    if (ug == NULL) {
+	err = E_UNKVAR;
+    } else if (gretl_vector_get_length(ug) != k) {
+	err = E_NONCONF;
+    } else {
+	for (i=0; i<k; i++) {
+	    g[i] = ug->val[i];
+	}
+    } 
+
+    return err;
+}
+
+/* parse the name of the user gradient matrix (vector) out of
+   the gradient function call, where it must be the first
+   argument, given in pointer form
+*/
+
+static int get_grad_vector_name (umax *u, const char *gradcall)
+{
+    const char *s = strchr(gradcall, '(');
+    int n, err = 0;
+
+    if (s == NULL) {
+	err = E_DATA;
+    } else {
+	s++;
+	s += strspn(s, " ");
+	if (*s != '&') {
+	    err = E_TYPES;
+	} else {
+	    s++;
+	    n = gretl_namechar_spn(s);
+	    if (n >= VNAMELEN) {
+		err = E_DATA;
+	    } else {
+		strncat(u->gmname, s, n);
+	    }
+	}
+    }
+
+    return err;
+}
+
 static int user_gen_setup (umax *u,
 			   const char *fncall,
+			   const char *gradcall,
 			   double ***pZ, 
 			   DATAINFO *pdinfo)
 {
     char formula[MAXLINE];
-    GENERATOR *g;
     int err = 0;
 
     if (u->gentype == GRETL_TYPE_MATRIX) {
@@ -945,47 +1130,65 @@ static int user_gen_setup (umax *u,
 	sprintf(formula, "$umax=%s", fncall);
     }
 
-    g = genr_compile(formula, pZ, pdinfo, OPT_P, &err);
+    u->gf = genr_compile(formula, pZ, pdinfo, OPT_P, &err);
 
     if (!err) {
 	/* see if the formula actually works */
-	err = execute_genr(g, pZ, pdinfo, OPT_S, u->prn);
+	err = execute_genr(u->gf, pZ, pdinfo, OPT_S, u->prn);
+    }
+
+    if (!err && gradcall != NULL) {
+	/* process gradient formula */
+	err = get_grad_vector_name(u, gradcall);
+	if (!err) {
+	    u->gg = genr_compile(gradcall, pZ, pdinfo, OPT_P | OPT_U, &err);
+	    if (!err) {
+		err = execute_genr(u->gg, pZ, pdinfo, OPT_S, u->prn);
+	    } 
+	}
     }
 
     if (!err) {
-	u->g = g;
 	u->Z = pZ;
 	u->dinfo = pdinfo;
-	u->m_out = genr_get_output_matrix(g);
+	u->fm_out = genr_get_output_matrix(u->gf);
     } else {
-	destroy_genr(g);
+	destroy_genr(u->gf);
+	destroy_genr(u->gg);
+	u->gf = u->gg = NULL;
     }
 
     return err;
 }
 
-double user_BFGS (gretl_matrix *b, const char *fncall,
+double user_BFGS (gretl_matrix *b, 
+		  const char *fncall,
+		  const char *gradcall, 
 		  double ***pZ, DATAINFO *pdinfo,
 		  PRN *prn, int *err)
 {
-    umax u;
+    umax *u;
     double ret = NADBL;
     gretlopt opt = OPT_NONE;
-    int maxit = 500;
+    int maxit = BFGS_MAXITER_DEFAULT;
     int fcount = 0, gcount = 0;
     double tol;
 
-    umax_init(&u, GRETL_TYPE_DOUBLE);
+    u = umax_new(GRETL_TYPE_DOUBLE);
+    if (u == NULL) {
+	*err = E_ALLOC;
+	return ret;
+    }
 
-    u.ncoeff = gretl_vector_get_length(b);
-    if (u.ncoeff == 0) {
+    u->ncoeff = gretl_vector_get_length(b);
+    if (u->ncoeff == 0) {
 	*err = E_DATA;
 	goto bailout;
     }
 
-    u.b = b;
+    u->b = b;
 
-    *err = user_gen_setup(&u, fncall, pZ, pdinfo);
+    *err = user_gen_setup(u, fncall, gradcall, pZ, pdinfo);
     if (*err) {
 	return NADBL;
     }
@@ -994,11 +1197,14 @@ double user_BFGS (gretl_matrix *b, const char *fncall,
 
     if (libset_get_bool(MAX_VERBOSE)) {
 	opt = OPT_V;
-	u.prn = prn;
+	u->prn = prn;
     }
 
-    *err = BFGS_max(b->val, u.ncoeff, maxit, tol, &fcount, &gcount,
-		    user_get_criterion, C_OTHER, NULL, &u, opt, prn);
+    *err = BFGS_max(b->val, u->ncoeff, 
+		    maxit, tol, &fcount, &gcount,
+		    user_get_criterion, C_OTHER, 
+		    (u->gg == NULL)? NULL : user_get_gradient, 
+		    u, opt, prn);
 
     if (fcount > 0) {
 	pprintf(prn, _("Function evaluations: %d\n"), fcount);
@@ -1006,16 +1212,18 @@ double user_BFGS (gretl_matrix *b, const char *fncall,
     }
 
     if (!*err) {
-	ret = u.x_out;
+	ret = u->fx_out;
     }
 
  bailout:
 
-    umax_clear(&u);
+    umax_destroy(u);
 
     return ret;
 }
 
+#define JAC_DEBUG 0
+
 static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
 			   integer *iflag, void *p)
 {
@@ -1027,7 +1235,11 @@ static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
 	u->b->val[i] = x[i];
     }
 
-    err = execute_genr(u->g, u->Z, u->dinfo, OPT_S, u->prn); 
+#if JAC_DEBUG
+    gretl_matrix_print(u->b, "user_calc_fvec: u->b");
+#endif
+
+    err = execute_genr(u->gf, u->Z, u->dinfo, OPT_S, u->prn); 
     if (err) {
 	fprintf(stderr, "execute_genr: err = %d\n", err); 
     }
@@ -1037,7 +1249,11 @@ static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
 	return 0;
     }
 
-    v = genr_get_output_matrix(u->g);
+    v = genr_get_output_matrix(u->gf);
+
+#if JAC_DEBUG
+    gretl_matrix_print(v, "matrix from u->f");
+#endif
 
     if (v == NULL || gretl_vector_get_length(v) != *m) {
 	fprintf(stderr, "user_calc_fvec: got bad matrix\n"); 
@@ -1074,7 +1290,7 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 		     double ***pZ, DATAINFO *pdinfo,
 		     int *err)
 {
-    umax u;
+    umax *u;
     gretl_matrix *J = NULL;
     integer m, n;
     integer iflag = 0;
@@ -1085,7 +1301,11 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 
     *err = 0;
 
-    umax_init(&u, GRETL_TYPE_MATRIX);
+    u = umax_new(GRETL_TYPE_MATRIX);
+    if (u == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
 
     n = gretl_vector_get_length(theta);
     if (n == 0) {
@@ -1095,22 +1315,22 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 	return NULL;
     }
 
-    u.b = theta;
-    u.ncoeff = n;
+    u->b = theta;
+    u->ncoeff = n;
 
-    *err = user_gen_setup(&u, fncall, pZ, pdinfo);
+    *err = user_gen_setup(u, fncall, NULL, pZ, pdinfo);
     if (*err) {
 	fprintf(stderr, "fdjac: error %d from user_gen_setup\n", *err);
 	goto bailout;
     }
 
-    if (u.m_out == NULL) {
-	fprintf(stderr, "fdjac: u.m_out is NULL\n");
-	*err = E_TYPES; /* FIXME */
+    if (u->fm_out == NULL) {
+	fprintf(stderr, "fdjac: u.fm_out is NULL\n");
+	*err = E_DATA; /* FIXME */
 	goto bailout;
     }
 
-    m = gretl_vector_get_length(u.m_out);
+    m = gretl_vector_get_length(u->fm_out);
     if (m == 0) {
 	*err = E_DATA;
 	goto bailout;
@@ -1122,11 +1342,11 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
     }
 
     for (i=0; i<m; i++) {
-	fvec[i] = u.m_out->val[i];
+	fvec[i] = u->fm_out->val[i];
     }
 
     fdjac2_(user_calc_fvec, &m, &n, theta->val, fvec, J->val, 
-	    &m, &iflag, &epsfcn, wa, &u);
+	    &m, &iflag, &epsfcn, wa, u);
 
  bailout:
 
@@ -1138,7 +1358,7 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 	J = NULL;
     }
 
-    umax_clear(&u);
+    umax_destroy(u);
 
     return J;
 }
