@@ -34,6 +34,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { SEMIC,    ";" },     
     { ADD,      "add" },
     { ADF,      "adf" }, 
+    { ANOVA,    "anova" }, 
     { APPEND,   "append" },
     { AR,       "ar" },  
     { AR1,      "ar1" },
@@ -75,6 +76,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { GMM,      "gmm" },
     { GNUPLOT,  "gnuplot" },  
     { GRAPH,    "graph" }, 
+    { GRAPHPG,  "graphpg" }, 
     { HAUSMAN,  "hausman" },
     { HECKIT,   "heckit" },
     { HELP,     "help" },    
@@ -121,6 +123,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { PVALUE,   "pvalue" }, 
     { QUANTREG, "quantreg" },
     { QLRTEST,  "qlrtest" }, 
+    { QQPLOT,   "qqplot" },
     { QUIT,     "quit" }, 
     { RENAME,   "rename" },
     { RESET,    "reset" },
@@ -154,6 +157,8 @@ static struct gretl_cmd gretl_cmds[] = {
     { WLS,      "wls" },
     { XCORRGM,  "xcorrgm" },
     { XTAB,     "xtab" },
+    { FUNDEBUG, "debug" },
+    { FUNCRET,  "return" },
     { NC,       NULL}
 }; 
 
@@ -233,6 +238,7 @@ const char *gretl_command_complete_next (const char *s,
 	    return gretl_cmds[i].cword;
 	}
     }
+
     return NULL;
 }  
 
