@@ -24,6 +24,7 @@ typedef enum {
     SEMIC = 0,    
     ADD,
     ADF,
+    ANOVA,
     APPEND,
     AR,  
     AR1,
@@ -65,6 +66,7 @@ typedef enum {
     GMM,
     GNUPLOT, 
     GRAPH,
+    GRAPHPG,
     HAUSMAN,
     HECKIT,
     HELP,    
@@ -111,6 +113,7 @@ typedef enum {
     PVALUE, 
     QUANTREG,
     QLRTEST,
+    QQPLOT,
     QUIT,
     RENAME,
     RESET,
@@ -144,6 +147,8 @@ typedef enum {
     WLS,
     XCORRGM,
     XTAB,
+    FUNDEBUG,
+    FUNCRET,
     NC
 } GretlCmdIndex;
 
