@@ -18,10 +18,15 @@
  */
 
 #include "libgretl.h"
+#include "gretl_func.h"
+#include "libset.h"
+#include "monte_carlo.h"
 
 #include <errno.h>
 
-char gretl_errmsg[ERRLEN];
+#define ERRLEN 2048
+
+static char gretl_errmsg[ERRLEN];
 
 static const char *gretl_error_messages[] = {
     NULL,
@@ -70,6 +75,8 @@ static const char *gretl_error_messages[] = {
     N_("Maximum length of command line (8192 bytes) exceeded"),  /* E_TOOLONG */
     N_("No dataset is in place"),                                /* E_NODATA */
     N_("Matrix is not positive definite"),                       /* E_NOTPD */
+    N_("Failed to calculate Jacobian"),                          /* E_JACOBIAN */
+    N_("Insufficient observations for this operation"),          /* E_TOOFEW */
     NULL,                                                        /* E_DB_DUP */
     NULL,                                                        /* E_OK */
     NULL                                                         /* E_MAX */
@@ -90,6 +97,7 @@ static const char *look_up_errmsg (int err)
 }
 
 static int error_printed;
+static int alarm_set;
 
 /**
  * errmsg_get_with_default: 
@@ -160,20 +168,27 @@ const char *gretl_errmsg_get (void)
 
 void gretl_errmsg_set (const char *str)
 {
+    if (alarm_set && *gretl_errmsg != '\0') {
+	/* leave the current error message in place */
+	return;
+    }
+
     if (*gretl_errmsg == '\0') {
 	strncat(gretl_errmsg, str, ERRLEN - 1);
-    } else {
+    } else if (strcmp(gretl_errmsg, str)) {
 	/* should we do the following? */
 	int n = strlen(gretl_errmsg);
 	int m = strlen(str);
 
-	if (n + m + 1 < ERRLEN) {
+	if (n + m + 2 < ERRLEN) {
 	    strcat(gretl_errmsg, "\n");
 	    strcat(gretl_errmsg, str);
 	}
     }
 }
 
+#if 0
+
 /**
  * gretl_errmsg_sprintf:
  * @fmt: format string.
@@ -185,16 +200,52 @@ void gretl_errmsg_set (const char *str)
 void gretl_errmsg_sprintf (const char *fmt, ...)
 {
     if (*gretl_errmsg == '\0' && fmt != NULL) {
-	va_list args;
+	va_list ap;
 
-	va_start(args, fmt);
-	vsnprintf(gretl_errmsg, ERRLEN, fmt, args);
-	va_end(args);
+	va_start(ap, fmt);
+	vsnprintf(gretl_errmsg, ERRLEN, fmt, ap);
+	va_end(ap);
+    }
+}
+
+#endif
+
+/**
+ * gretl_errmsg_sprintf:
+ * @fmt: format string.
+ *
+ * Append a formatted message to the current gretl
+ * error message.
+ */
+
+void gretl_errmsg_sprintf (const char *fmt, ...)
+{
+    if (*gretl_errmsg == '\0') {
+	va_list ap;
+
+	va_start(ap, fmt);
+	vsnprintf(gretl_errmsg, ERRLEN, fmt, ap);
+	va_end(ap);
+    } else {
+	int n = ERRLEN - strlen(gretl_errmsg) - 2;
+
+	if (n > 31) {
+	    char msg[ERRLEN];
+	    va_list ap;
+
+	    va_start(ap, fmt);
+	    vsnprintf(msg, n, fmt, ap);
+	    va_end(ap);
+
+	    strcat(gretl_errmsg, "\n");
+	    strcat(gretl_errmsg, msg);
+	}
     }
 }
 
 /**
  * gretl_errmsg_set_from_errno:
+ * @s: string to prepend to error message, or %NULL.
  *
  * If %gretl_errmsg is currently blank, copy the string 
  * returned by %strerror into the message space; or if the 
@@ -220,6 +271,8 @@ void gretl_errmsg_set_from_errno (const char *s)
     }
 }
 
+static int gretl_errno;
+
 /**
  * gretl_error_clear:
  *
@@ -228,11 +281,64 @@ void gretl_errmsg_set_from_errno (const char *s)
 
 void gretl_error_clear (void)
 {
-    *gretl_errmsg = '\0';
+    if (!alarm_set) {
+	*gretl_errmsg = '\0';
+    }
     error_printed = 0;
     errno = 0;
 }
 
+/**
+ * gretl_errmsg_is_set:
+ *
+ * Returns: 1 if the gretl error message is currently
+ * set (not blank), otherwise 0.
+ */
+
+int gretl_errmsg_is_set (void)
+{
+    return (*gretl_errmsg != '\0');
+}
+
+/* setting the "alarm" prevents gretl_errmsg from being
+   overwritten
+*/
+
+void set_gretl_alarm (int val)
+{
+    alarm_set = val;
+}
+
+void set_gretl_errno (int err)
+{
+    gretl_errno = err;
+}
+
+int get_gretl_errno (void)
+{
+    int err = gretl_errno;
+
+    gretl_errno = 0;
+    return err;
+}
+
+int check_gretl_errno (void)
+{
+    return gretl_errno;
+}
+
+int gretl_error_is_fatal (void)
+{
+    if (gretl_compiling_function() ||
+	gretl_compiling_loop()) {
+	return 1;
+    } else if (libset_get_bool(HALT_ON_ERR) == 0) {
+	return 0;
+    } else {
+	return gretl_in_batch_mode();
+    }
+}
+
 #if 0 /* not yet */
 
 static char *warnbuf;
@@ -253,5 +359,3 @@ const char *gretl_warning_get (void)
 }
 
 #endif
-
-
