@@ -20,8 +20,6 @@
 #ifndef GRETL_ERRORS_H
 #define GRETL_ERRORS_H
 
-extern char gretl_errmsg[ERRLEN];
-
 enum gretl_error_codes {
     E_DATA = 2,
     E_SINGULAR,     /* 3 */
@@ -67,9 +65,11 @@ enum gretl_error_codes {
     E_TOOLONG,     /* 43 */
     E_NODATA,      /* 44 */
     E_NOTPD,       /* 45 */
-    E_DB_DUP,      /* 46 : duplicate vars found when saving to database */
-    E_OK,          /* 47 : not really an error */
-    E_MAX          /* 48 */
+    E_JACOBIAN,    /* 46 */
+    E_TOOFEW,      /* 47 */
+    E_DB_DUP,      /* 48 : duplicate vars found when saving to database */
+    E_OK,          /* 49 : not really an error */
+    E_MAX          /* 50 */
 }; 
 
 void errmsg (int err, PRN *prn);
@@ -86,4 +86,16 @@ void gretl_errmsg_set_from_errno (const char *s);
 
 void gretl_error_clear (void);
 
+void set_gretl_alarm (int val);
+
+void set_gretl_errno (int err);
+
+int get_gretl_errno (void);
+
+int check_gretl_errno (void);
+
+int gretl_error_is_fatal (void);
+
+int gretl_errmsg_is_set (void);
+
 #endif /* GRETL_ERRORS_H */
