@@ -30,4 +30,28 @@ int write_gretl_R_files (const char *buf,
 
 void delete_gretl_R_files (void);
 
+int write_gretl_ox_file (const char *buf, gretlopt opt, const char **pfname);
+
+void delete_gretl_ox_file (void);
+
+#ifdef USE_RLIB
+
+int get_R_function_by_name (const char *name);
+
+int gretl_R_get_call (const char *name, int argc);
+
+int gretl_R_function_add_scalar (double x);
+
+int gretl_R_function_add_vector (const double *x, int t1, int t2);
+
+int gretl_R_function_add_matrix (const gretl_matrix *m);
+
+int gretl_R_function_exec (const char *name, int *rtype, void **ret);
+
+void gretl_R_reset_error (void);
+
+void gretl_R_cleanup (void);
+
+#endif
+
 #endif /* GRETL_FOREIGN_H */
