@@ -86,7 +86,7 @@ void gretl_pop_c_numeric_locale (void)
     return;
 }
 
-#endif /* ENABLE_NLS */
+#endif /* ENABLE_NLS or not */
 
 /**
  * doing_nls:
@@ -96,7 +96,6 @@ void gretl_pop_c_numeric_locale (void)
 
 int doing_nls (void)
 {
-#ifdef ENABLE_NLS
     static int called, nls;
 
     if (!called) {
@@ -105,15 +104,13 @@ int doing_nls (void)
 	       strcmp("annual", _("annual")));
 	called = 1;
     }
+
     return nls;
-#else
-    return 0;
-#endif
 }
 
 #ifdef ENABLE_NLS
+
 static int decpoint;
-#endif
 
 /**
  * reset_local_decpoint:
@@ -126,15 +123,12 @@ static int decpoint;
 
 int reset_local_decpoint (void)
 {
-#ifdef ENABLE_NLS
     struct lconv *lc;
 
     lc = localeconv();
     decpoint = *lc->decimal_point;
+    set_atof_point(decpoint);
     return decpoint;
-#else
-    return '.';
-#endif
 }
 
 /**
@@ -145,24 +139,32 @@ int reset_local_decpoint (void)
 
 int get_local_decpoint (void)
 {
-#ifdef ENABLE_NLS
     if (decpoint == 0) {
 	decpoint = reset_local_decpoint();
     }
     return decpoint;
-#else
-    return '.';
-#endif
 }
 
+#else
+
+int reset_local_decpoint (void)
+{
+    return '.';
+}
+
+int get_local_decpoint (void)
+{
+    return '.';
+}
+
+#endif
+
 /* fudges for strings that should not be in utf-8 under some
    conditions: under GTK translations always come out in utf-8 in the
    GUI, but when we're sending stuff to stderr we may have to put it
    into ISO-8859-N.
 */
 
-#ifdef ENABLE_NLS
-
 static int gretl_cset_maj;
 static int gretl_cset_min;
 # ifdef WIN32
@@ -216,14 +218,14 @@ void set_gretl_charset (const char *s)
 		gretl_cset_maj = gretl_cset_min = 0;
 	    }
 	} 
-# ifdef WIN32
+#ifdef WIN32
 	if (p == NULL) {
 	    sscanf(gretl_charset, "cp%d", &gretl_cpage);
 	}
-# endif
+#endif
     }
 
-# ifdef WIN32
+#ifdef WIN32
     fprintf(stderr, "codepage = %d\n", gretl_cpage);
     if (gretl_cpage != 1250) {
 	char *e = getenv("GRETL_CPAGE");
@@ -233,10 +235,10 @@ void set_gretl_charset (const char *s)
 	    fprintf(stderr, "revised codepage to 1250\n");
 	}
     }
-# endif
+#endif
 }
 
-# ifdef WIN32
+#ifdef WIN32
 
 static void set_cp_from_locale (const char *loc)
 {
@@ -248,7 +250,9 @@ static void set_cp_from_locale (const char *loc)
     }
 }
 
-# endif
+#endif
+
+#ifdef ENABLE_NLS
 
 static const char *get_gretl_charset (void)
 {
@@ -269,6 +273,8 @@ static const char *get_gretl_charset (void)
     return NULL;
 }
 
+#endif
+
 int iso_latin_version (void)
 {
     char *lang = NULL;
@@ -282,7 +288,7 @@ int iso_latin_version (void)
 	return gretl_cset_min;
     }
 
-# ifdef WIN32
+#ifdef WIN32
     if (gretl_cpage == 1252) {
 	return 1;
     } else if (gretl_cpage == 1250) {
@@ -292,7 +298,7 @@ int iso_latin_version (void)
     } else if (gretl_cpage == 1254) {
 	return 9;
     }
-# endif
+#endif
 
     /* Polish, Russian, Turkish: UTF-8 locale? */
     lang = getenv("LANG");
@@ -311,9 +317,9 @@ int iso_latin_version (void)
 
 int chinese_locale (void)
 {
-# ifdef WIN32
+#ifdef WIN32
     return (gretl_cpage == 950);
-# else
+#else
     char *lang = getenv("LANG");
 
     return (lang != NULL && !strncmp(lang, "zh", 2));
@@ -332,6 +338,8 @@ void unset_gui_native_printing (void)
     gui_native_printing = 0;
 }
 
+#ifdef ENABLE_NLS
+
 char *iso_gettext (const char *msgid)
 {
     static int iso_switch = -1;
@@ -386,7 +394,9 @@ char *iso_gettext (const char *msgid)
     return ret;
 } 
 
-# ifdef WIN32
+#endif /* ENABLE_NLS */
+
+#ifdef WIN32
 
 struct localeinfo {
     int id;
@@ -427,7 +437,7 @@ const char *locale_code_from_id (int langid)
     return NULL;
 }
 
-# endif
+#endif /* WIN32 */
 
 struct langinfo {
     int id;
@@ -496,7 +506,7 @@ const char *lang_code_from_id (int langid)
     return NULL;
 }
 
-# ifdef WIN32
+#ifdef WIN32
 
 static char *win32_set_numeric (const char *lang)
 {
@@ -516,19 +526,50 @@ static char *win32_set_numeric (const char *lang)
     return set;
 }
 
-# endif /* WIN32 */
+#else /* !WIN32 */
+
+# ifdef ENABLE_NLS
+
+static char *other_set_numeric (const char *lang)
+{
+    char *set = setlocale(LC_NUMERIC, lang);
+
+    if (set == NULL) {
+	char lfix[32];
+
+	sprintf(lfix, "%s.UTF-8", lang);
+	set = setlocale(LC_NUMERIC, lfix);
+    }
+
+    return set;
+}
+
+# endif /* ENABLE_NLS */
+
+#endif /* WIN32 or not */
 
 void set_lcnumeric (int langid, int lcnumeric)
 {
+#ifndef ENABLE_NLS
+    return;
+#else
     if (lcnumeric && langid != LANG_C) {
-	char *lang = getenv("LANG");
+	const char *lang;
 	char *set = NULL;
 
+	if (langid == LANG_AUTO) {
+	    /* respect the system LANG setting */
+	    lang = getenv("LANG");
+	} else {
+	    /* fake it from user preference */
+	    lang = lang_code_from_id(langid);
+	}
+
 	if (lang != NULL) {
 # ifdef WIN32
 	    set = win32_set_numeric(lang);
 # else
-	    set = setlocale(LC_NUMERIC, lang);
+	    set = other_set_numeric(lang);
 # endif
 	} 
 	if (set == NULL) {
@@ -536,30 +577,60 @@ void set_lcnumeric (int langid, int lcnumeric)
 	    putenv("LC_NUMERIC=");
 	}
     } else {
+	/* either lcnumeric is not chosen, or we're in LANG_C */
 	setlocale(LC_NUMERIC, "C");
 	putenv("LC_NUMERIC=C");
     }
 
     reset_local_decpoint();
+#endif
 }
 
-/* arg should be long English form of language, as displayed
-   in GUI, e.g. "German", "French" */
+#ifdef ENABLE_NLS
+
+static int set_locale_with_workaround (const char *lcode)
+{
+    char *test = setlocale(LC_ALL, lcode);
+
+# ifndef WIN32    
+    if (test == NULL) {
+	char lfix[32];
+
+	sprintf(lfix, "%s.UTF-8", lcode); /* Ubuntu, grr */
+	test = setlocale(LC_ALL, lfix);
+    }
+# endif
+
+    if (test != NULL) {
+	fprintf(stderr, "setlocale: '%s' -> '%s'\n", lcode, test);
+    }
+
+    return test == NULL;
+}
+
+# ifdef WIN32
+# define get_setlocale_string(i) (locale_code_from_id(i))
+# else
+# define get_setlocale_string(i) (lang_code_from_id(i))
+# endif
+
+#endif /* ENABLE_NLS */
+
+/* @langstr should be the English name of the selected language
+   as displayed in the GUI (e.g. "German", "French") 
+*/
 
 int test_locale (const char *langstr)
 {
+#ifndef ENABLE_NLS
+    return 1;
+#else
     const char *lcode;
-    int langid;
-    char *orig, *test;
-    char ocpy[64];
+    char *orig, ocpy[64];
+    int langid, err = 0;
 
     langid = lang_id_from_name(langstr);
-
-#ifdef WIN32
-    lcode = locale_code_from_id(langid);
-#else
-    lcode = lang_code_from_id(langid);
-#endif
+    lcode = get_setlocale_string(langid);
     orig = setlocale(LC_ALL, NULL);
 
     gretl_error_clear();
@@ -567,38 +638,45 @@ int test_locale (const char *langstr)
     *ocpy = '\0';
     strncat(ocpy, orig, 63);
 
-    if ((test = setlocale(LC_ALL, lcode)) != NULL) {
-        fprintf(stderr, "test_locale: '%s' -> '%s'\n", lcode, test);
-	setlocale(LC_ALL, ocpy); /* restore the original locale */
-	return 0;
-    } else {
+    err = set_locale_with_workaround(lcode);
+
+    if (err) {
 	gretl_errmsg_sprintf(_("%s: locale is not supported "
 			       "on this system"), lcode);
-	return 1;
-    }
+    } else {
+	setlocale(LC_ALL, ocpy); /* restore the original locale */
+    } 
+
+    return err;
+#endif
 }
 
-void force_language (int langid)
+int force_language (int langid)
 {
+#ifndef ENABLE_NLS
+    return 1;
+#else
     const char *lcode = NULL;
+    int err = 0;
 
     if (langid == LANG_C) {
 	putenv("LANGUAGE=english");
+	putenv("LANG=C");
 	setlocale(LC_ALL, "C");
     } else {
-# ifdef WIN32
-        lcode = locale_code_from_id(langid);
-# else
-	lcode = lang_code_from_id(langid);
-# endif
+	lcode = get_setlocale_string(langid);
 	if (lcode != NULL) {  
+# ifdef WIN32
 	    char *newloc = setlocale(LC_ALL, lcode);
 
             fprintf(stderr, "lcode='%s', newloc='%s'\n", lcode, newloc);
-# ifdef WIN32
 	    if (newloc != NULL) {
 		set_cp_from_locale(newloc);
+	    } else {
+		err = 1;
 	    }
+# else
+	    err = set_locale_with_workaround(lcode);
 # endif
 	}
     }
@@ -627,9 +705,10 @@ void force_language (int langid)
 	putenv(estr);
     }
 # endif
-}
 
-#endif  /* ENABLE_NLS */
+    return err;
+#endif /* ENABLE_NLS */
+}
 
 static void 
 iso_to_ascii_translate (char *targ, const char *src, int latin)
@@ -875,8 +954,6 @@ char *iso_to_ascii (char *s)
     return real_iso_to_ascii(s, 1);
 }
 
-#ifdef ENABLE_NLS
-
 char *sprint_l2_to_ascii (char *targ, const char *s, size_t len)
 {
     iso_to_ascii_translate(targ, s, 2);
@@ -1012,6 +1089,8 @@ int get_translated_width (const char *str)
 
 static int printing_to_console;
 
+#ifdef ENABLE_NLS
+
 char *maybe_iso_gettext (const char *msgid)
 {
    if (printing_to_console) {
@@ -1021,6 +1100,8 @@ char *maybe_iso_gettext (const char *msgid)
    }
 } 
 
+#endif
+
 void check_for_console (PRN *prn)
 {
     if (prn != NULL) {
@@ -1034,11 +1115,4 @@ void console_off (void)
     printing_to_console = 0;
 }
 
-#else
 
-int gretl_is_ascii (const char *buf)
-{
-    return 1;
-}
-
-#endif /* !ENABLE_NLS */
