@@ -17,6 +17,8 @@
  * 
  */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "gretl_list.h"
 #include "gretl_func.h"
@@ -24,6 +26,7 @@
 #include "gretl_xml.h"
 
 #include <errno.h>
+#include <glib.h>
 
 #define LDEBUG 0
 
@@ -911,6 +914,14 @@ int gretl_list_cmp (const int *list1, const int *list2)
 {
     int i;
 
+    if (list1 == NULL && list2 != NULL) {
+	return 1;
+    } else if (list1 != NULL && list2 == NULL) {
+	return 1;
+    } else if (list1 == NULL && list2 == NULL) {
+	return 0;
+    }
+
     for (i=0; i<=list1[0]; i++) {
 	if (list2[i] != list1[i]) {
 	    return 1;
@@ -1059,8 +1070,16 @@ int *gretl_list_from_string (const char *str, int *err)
     n = 0; /* value counter */
 
     while (*s && !*err) {
+	s += strspn(s, " ");
+	if (n > 0 && *s == ';') {
+	    /* list separator */
+	    n++;
+	    s++;
+	    continue;
+	}
 	r1 = strtol(s, &next, 10);
 	if (errno || next == s) {
+	    fprintf(stderr, "gretl_list_from_string: '%s'\n", s);
 	    *err = E_PARSE;
 	} else {
 	    s = next;
@@ -1103,6 +1122,12 @@ int *gretl_list_from_string (const char *str, int *err)
     n = 1; /* list position indicator */
 
     while (*s) {
+	s += strspn(s, " ");
+	if (*s == ';') {
+	    list[n++] = LISTSEP;
+	    s++;
+	    continue;
+	}
 	r1 = strtol(s, &s, 10);
 	if (*s == '-') {
 	    s++;
@@ -1591,14 +1616,15 @@ static int *real_gretl_list_omit (const int *orig, const int *omit,
 
     *err = 0;
 
-    /* check for spurious "omissions" */
+    /* check how many terms we're omitting */
     for (i=1; i<=nomit; i++) {
 	int pos = in_gretl_list(orig, omit[i]);
 
 	if (pos < minpos) {
 	    if (mode == LIST_OMIT_TIGHT) {
-		sprintf(gretl_errmsg, _("Variable %d was not in the original list"),
-			omit[i]);
+		/* "omission" is spurious */
+		gretl_errmsg_sprintf(_("Variable %d was not in the original list"),
+				     omit[i]);
 		*err = 1;
 		return NULL;
 	    } else {
@@ -1664,7 +1690,8 @@ static int *real_gretl_list_omit (const int *orig, const int *omit,
  * @err: pointer to receive error code.
  *
  * Creates a list containing the elements of @orig that are not
- * present in @omit.
+ * present in @omit.  It is an error if the @omit list contains
+ * members that are not present in @orig.
  *
  * Returns: new list on success, %NULL on error.
  */
@@ -1674,6 +1701,26 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
     return real_gretl_list_omit(orig, omit, minpos, LIST_OMIT_TIGHT, err);
 }
 
+/**
+ * gretl_list_drop:
+ * @orig: an array of integers, the first element of which holds
+ * a count of the number of elements following.
+ * @drop: list of variables to drop.
+ * @err: pointer to receive error code.
+ *
+ * Creates a list containing the elements of @orig that are not
+ * present in @drop.  Unlike gretl_list_omit(), processing always
+ * starts from position 1 in @orig, and it is not an error if
+ * some members of @drop are not present in @orig.  
+ *
+ * Returns: new list on success, %NULL on error.
+ */
+
+int *gretl_list_drop (const int *orig, const int *drop, int *err)
+{
+    return real_gretl_list_omit(orig, drop, 1, LIST_OMIT_SLOPPY, err);
+}
+
 /**
  * gretl_list_diff:
  * @targ: target list (must be pre-allocated).
@@ -1946,7 +1993,7 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	    continue;
 	}
 	if (list[j] >= pdinfo->v) {
-	    strcpy(gretl_errmsg, _(errmsg));
+	    gretl_errmsg_set(_(errmsg));
 	    return E_DATA;
 	}
 	label = VARLABEL(pdinfo, list[j]);
@@ -1956,7 +2003,7 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	    repl = 0;
 	    sscanf(label, "%*s %*s %*s %d", &repl);
 	    if (repl >= mc) {
-		strcpy(gretl_errmsg, _(errmsg));
+		gretl_errmsg_set(_(errmsg));
 		return E_DATA;
 	    }
 	}
@@ -1970,6 +2017,8 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
  * @list: an array of integer variable ID numbers, the first element
  * of which holds a count of the number of elements following.
  * @minpos: position in @list at which to start the search (>= 1).
+ * @Z: data array.
+ * @pdinfo: dataset information.
  *
  * Checks @list for the presence, in position @minpos or higher, of
  * a variable whose valid values in sample all equal 1.  This usually
@@ -2191,7 +2240,7 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
     int start = 2;
     int i, ret = -1;
 
-    if (ci == COINT) {
+    if (ci == COINT || ci == ANOVA) {
 	start = 1;
     } else if (ci == ARCH) {
 	start = 3;
@@ -2596,8 +2645,8 @@ int gretl_serialize_lists (const char *fname)
 /**
  * gretl_list_print:
  * @lname: name of list.
- * @pdinfo:
- * @prn:
+ * @pdinfo: dataset information.
+ * @prn: gretl printing struct.
  * 
  * Prints to @prn the given @list of variables, by name.
  */
@@ -2633,3 +2682,87 @@ void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
     }
 }
 
+/**
+ * varname_match_list:
+ * @pdinfo: pointer to dataset information.
+ * @pattern: pattern to be matched.
+ *
+ * Returns: a list of ID numbers of variables whose names
+ * match @pattern, or %NULL if there are no matches.
+ */
+
+int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
+{
+    GPatternSpec *pspec;
+    int *list = NULL;
+    int i, fd, n = 0;
+
+    if (pdinfo == NULL || pdinfo->v == 0) {
+	return NULL;
+    }
+
+    fd = gretl_function_depth();
+
+    pspec = g_pattern_spec_new(pattern);
+
+    for (i=1; i<pdinfo->v; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
+	    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+		n++;
+	    }
+	}
+    }
+
+    if (n > 0) {
+	list = gretl_list_new(n);
+	if (list != NULL) {
+	    int j = 1;
+
+	    for (i=1; i<pdinfo->v; i++) { 
+		if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
+		    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+			list[j++] = i;
+		    }
+		}
+	    }
+	}
+    }
+
+    g_pattern_spec_free(pspec);
+
+    return list;
+}
+
+/**
+ * varname_match_any:
+ * @pdinfo: pointer to dataset information.
+ * @pattern: pattern to be matched.
+ *
+ * Returns: 1 if at least one variable in the dataset has a
+ * name that matches @pattern, otherwise 0.
+ */
+
+int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
+{
+    GPatternSpec *pspec;
+    int i, fd, ret = 0;
+
+    fd = gretl_function_depth();
+
+    pspec = g_pattern_spec_new(pattern);
+
+    for (i=1; i<pdinfo->v; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
+	    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+		ret = 1;
+		break;
+	    }
+	}
+    }
+
+    g_pattern_spec_free(pspec);
+
+    return ret;
+}
+
+
