@@ -30,7 +30,9 @@ typedef enum {
     ARMA_VECH  = 1 << 4, /* using vech representation when computing
 			    variance matrix of state for Kalman filter
 			 */
-    ARMA_LS    = 1 << 5  /* using conditional ML, and O/NLS == CML */
+    ARMA_LS    = 1 << 5, /* using conditional ML, and O/NLS == CML */
+    ARMA_XDIFF = 1 << 6, /* ARIMA: exogenous regressors are differenced */
+    ARMA_LBFGS = 1 << 7  /* using L-BFGS-B with native exact ML */
 } ArmaFlags;
 
 typedef struct CoeffIntervals_ CoeffIntervals;
@@ -206,14 +208,8 @@ typedef enum {
     GRETL_TEST_MAX
 } ModelTestType;
 
-#ifndef GRETLCLI
-int attach_model_tests_from_xml (MODEL *pmod, xmlNodePtr node);
-#endif
-
 MODEL *gretl_model_new (void);
 
-void model_stats_init (MODEL *pmod);
-
 void gretl_model_init (MODEL *pmod);
 
 int gretl_model_allocate_storage (MODEL *pmod);
@@ -236,6 +232,8 @@ void clear_model (MODEL *pmod);
 
 void gretl_model_free (MODEL *pmod);
 
+void clear_model_xpx (MODEL *pmod);
+
 void gretl_model_free_on_exit (MODEL *pmod);
 
 void display_model_data_items (const MODEL *pmod);
@@ -441,12 +439,13 @@ gretl_model_get_data_element (MODEL *pmod, int idx, const char *s,
 int gretl_model_serialize (const MODEL *pmod, SavedObjectFlags flags,
 			   FILE *fp);
 
-#ifndef GRETLCLI
+#ifdef FULL_XML_HEADERS
+
+int attach_model_tests_from_xml (MODEL *pmod, xmlNodePtr node);
 
 MODEL *gretl_model_from_XML (xmlNodePtr node, xmlDocPtr doc, 
 			     const DATAINFO *pdinfo,
 			     int *err);
-
 #endif
 
 #endif /* GRETL_MODEL_H */
