@@ -30,6 +30,34 @@ typedef enum {
     USER_SEARCH
 } SearchLocation;
 
+typedef enum {
+    GRETL_HELPFILE,
+    GRETL_CMD_HELPFILE,
+    GRETL_CLI_HELPFILE
+} HelpPaths;
+
+typedef struct ConfigPaths_ ConfigPaths;
+
+/* these are all the gretl paths which are recorded in
+   the gretl config file or Windows registry entries 
+*/
+
+struct ConfigPaths_ {
+    char gretldir[MAXLEN];
+    char workdir[MAXLEN];
+    char ratsbase[MAXLEN];
+#ifndef WIN32
+    char gnuplot[MAXLEN];
+#endif
+    char x12a[MAXLEN];
+    char tramo[MAXLEN];
+    char rbinpath[MAXLEN];
+    char rlibpath[MAXLEN];
+    char oxlpath[MAXLEN];
+    char dbhost[32];
+    char pngfont[128];
+};
+
 void set_string_table_written (void);
 
 int gretl_string_table_written (void);
@@ -38,6 +66,8 @@ int gretl_path_prepend (char *file, const char *path);
 
 int gretl_normalize_path (char *path);
 
+int slash_terminate (char *path);
+
 void set_stdio_use_utf8 (void);
 
 int get_stdio_use_utf8 (void);
@@ -62,35 +92,41 @@ int gretl_chdir (const char *path);
 
 int gretl_deltree (const char *path);
 
+int gretl_setenv (const char *name, const char *value);
+
 int gretl_write_access (char *fname);
 
 int gretl_is_xml_file (const char *fname);
 
 int gretl_isdir (const char *path);
 
-char *addpath (char *fname, PATHS *ppaths, int script);
+char *addpath (char *fname, int script);
 
-int getopenfile (const char *line, char *fname, PATHS *ppaths,
-		 gretlopt opt);
+int getopenfile (const char *line, char *fname, gretlopt opt);
 
-int gretl_path_is_absolute (const char *fname);
+int fname_has_path (const char *fname);
 
-int has_system_prefix (const char *fname, const PATHS *ppaths,
-		       int locus);
+int has_system_prefix (const char *fname, int locus);
 
-void show_paths (const PATHS *ppaths);
+void show_paths (void);
 
-int gretl_set_paths (PATHS *ppaths, gretlopt opt);
+int gretl_set_paths (ConfigPaths *paths, gretlopt opt);
+
+int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt);
+
+const char *helpfile_path (int id);
+
+const char *gretl_home (void);
 
 const char *gretl_lib_path (void);
 
-const char *gretl_dot_dir (void);
+const char *gretl_dotdir (void);
 
-const char *gretl_work_dir (void);
+const char *gretl_workdir (void);
 
-char *gretl_default_workdir (PATHS *paths);
+char *gretl_default_workdir (void);
 
-int set_gretl_work_dir (const char *path, PATHS *ppaths);
+int set_gretl_work_dir (const char *path);
 
 const char *gretl_maybe_switch_dir (const char *fname);
 
@@ -102,15 +138,31 @@ const char *gretl_plotfile (void);
 
 char *set_gretl_plotfile (const char *fname);
 
+const char *gretl_binbase (void);
+
+const char *gretl_ratsbase (void);
+
 const char *gretl_tramo (void);
 
+const char *gretl_tramo_dir (void);
+
 const char *gretl_x12_arima (void);
 
 const char *gretl_x12_arima_dir (void);
 
+const char *gretl_rbin_path (void);
+
+const char *gretl_rlib_path (void);
+
 const char *gretl_png_font (void);
 
-void set_gretl_png_font (const char *s, PATHS *ppaths);
+const char *gretl_oxl_path (void);
+
+const char *gretl_current_dir (void);
+
+void gretl_set_current_dir (const char *s);
+
+void set_gretl_png_font (const char *s);
 
 #ifdef WIN32
 
@@ -118,7 +170,9 @@ int gretl_mkstemp (char *tmpl);
 
 #else
 
-int cli_read_rc (PATHS *paths);
+void get_gretl_rc_path (char *rcfile);
+
+int cli_read_rc (void);
 
 #endif
 
