@@ -87,7 +87,7 @@ static int prn_add_tempfile (PRN *prn)
 
     errno = 0;
 
-    sprintf(fname, "%sprntmp.XXXXXX", gretl_dot_dir());
+    sprintf(fname, "%sprntmp.XXXXXX", gretl_dotdir());
 
 #ifdef WIN32
     fd = gretl_mkstemp(fname);
@@ -340,7 +340,21 @@ PRN *gretl_print_new_with_stream (FILE *fp)
     } else {
 	return real_gretl_print_new(GRETL_PRINT_STREAM, NULL, NULL, fp, NULL);
     }
-}  
+} 
+ 
+/**
+ * gretl_print_detach_stream
+ * @prn: printing struct to operate on.
+ * 
+ * Sets the stream member of @prn to %NULL so that @prn can
+ * be destroyed without closing the associated stream.  May be
+ * used in conjunction with gretl_print_new_with_stream().
+ */
+
+void gretl_print_detach_stream (PRN *prn)
+{
+    prn->fp = NULL;
+}
 
 /**
  * gretl_print_rename_file:
@@ -397,10 +411,13 @@ int gretl_print_rename_file (PRN *prn, const char *oldpath,
  * gretl_print_reset_buffer:
  * @prn: printing struct to operate on.
  * 
- * Write a NUL byte to the start of the buffer associated
- * with @prn, if any.
+ * If @prn has an attached buffer, write a NUL byte to
+ * the start of the buffer and reset the count of bytes
+ * printed to zero.  The next call to @pprintf or
+ * similar will then overwite rather than cumulating
+ * the printed content.
  *
- * Returns: 0 on success, 1 on error.
+ * Returns: 0 on success, 1 if @prn has no buufer.
  */
 
 int gretl_print_reset_buffer (PRN *prn)
@@ -566,6 +583,7 @@ FILE *gretl_print_read_tempfile (PRN *prn)
 /**
  * gretl_print_stop_tempfile_read:
  * @prn: printing struct.
+ * @fp: auxiliary stream.
  * 
  * For @prn with tempfile attached, stops reading
  * of the tempfile and closes @fp (recording the
