@@ -17,6 +17,8 @@
  * 
  */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "gretl_func.h"
 #include "libset.h"
@@ -38,6 +40,8 @@ static gretl_scalar **scalars;
 static int n_scalars;
 static int scalar_imin;
 
+static void (*scalar_edit_callback)(const char *name, double val);
+
 #if SDEBUG
 
 static void debug_print_scalars (const char *s)
@@ -226,6 +230,7 @@ void gretl_scalar_set_value (const char *name, double val)
     gretl_scalar *s;
 
     s = get_scalar_pointer(name, gretl_function_depth());
+
     if (s != NULL) {
 	s->val = val;
     }
@@ -286,6 +291,10 @@ int gretl_scalar_add (const char *name, double val)
     debug_print_scalars("gretl_scalar_add");
 #endif
 
+    if (!err && level == 0 && scalar_edit_callback != NULL) {
+	scalar_edit_callback(name, val);
+    }
+
     return err;
 }
 
@@ -373,58 +382,23 @@ int gretl_scalar_delete (const char *name, PRN *prn)
     return err;
 }
 
-static void print_scalar (gretl_scalar *s, gretlopt opt, PRN *prn, int n)
+void set_scalar_edit_callback (void (*callback))
 {
-    if (n == 0) {
-	pputc(prn, '\n');
-    }
-
-    pprintf(prn, "%15s = ", s->name);
-
-    if (na(s->val)) {
-	pputs(prn, "NA");
-    } else {
-	if (s->val >= 0.0) {
-	    pputc(prn, ' ');
-	}
-	if (opt & OPT_L) {
-	    pprintf(prn, "%#.*E", libset_get_int(LONGDIGITS), s->val);
-	} else if (opt & OPT_T) {
-	    pprintf(prn, "%#.10E", s->val);
-	} else {
-	    pprintf(prn, "%#.6g", s->val);
-	}
-    }
-
-    pputc(prn, '\n');
+    scalar_edit_callback = callback; 
 }
 
-void print_scalar_by_name (const char *name, gretlopt opt, PRN *prn)
+void print_scalar_by_name (const char *name, PRN *prn)
 {
     gretl_scalar *s = get_scalar_pointer(name, gretl_function_depth());
 
     if (s != NULL) {
-	print_scalar(s, opt, prn, 0);
-    }
-}
-
-void print_all_scalars (gretlopt opt, PRN *prn)
-{
-    int n = 0;
-
-    if (n_scalars > 0) {
-	int i, level = gretl_function_depth();
-
-	for (i=0; i<n_scalars; i++) {
-	    if (scalars[i]->level == level) {
-		print_scalar(scalars[i], opt, prn, n++);
-	    }
+	pprintf(prn, "\n%15s = ", s->name);
+	if (na(s->val)) {
+	    pputs(prn, " NA\n");
+	} else {
+	    pprintf(prn, "% #.8g\n", s->val);
 	}
     }
-
-    if (n == 0) {
-	pputs(prn, "No scalars are defined\n");
-    }
 }
 
 /* "auxiliary scalars": this apparatus is used when we want to do
@@ -584,3 +558,43 @@ void write_scalars_to_file (FILE *fp)
 
     fputs("</gretl-scalars>\n", fp);
 }
+
+/**
+ * print_scalars:
+ * @prn: pointer to gretl printing struct.
+ *
+ * Prints names and values of any saved scalars.
+ */
+
+void print_scalars (PRN *prn)
+{
+    int level = gretl_function_depth();
+    int len, ns = 0, maxlen = 0;
+    int i;
+
+    for (i=0; i<n_scalars; i++) {
+	if (scalars[i]->level == level) {
+	    len = strlen(scalars[i]->name);
+	    if (len > maxlen) {
+		maxlen = len;
+	    }
+	    ns++;
+	}
+    }
+
+    if (ns == 0) {
+	pprintf(prn, "%s\n", _("none"));
+	return;
+    }
+
+    pputc(prn, '\n');
+
+    for (i=0; i<n_scalars; i++) {
+	if (scalars[i]->level == level) {
+	    pprintf(prn, " %*s = %.15g\n", maxlen, scalars[i]->name, 
+		    scalars[i]->val);
+	}
+    }
+
+    pputc(prn, '\n');
+}
