@@ -27,16 +27,16 @@
 typedef struct _col_table col_table;
 
 struct _col_table {
-    int idx;
-    int n_strs;
-    char **strs;
-    GHashTable *hash;
+    int idx;          /* column index (variable number) */
+    int n_strs;       /* number of strings in table */
+    char **strs;      /* saved strings */
+    GHashTable *hash; /* hash table for quick lookup */
 };
 
 struct _gretl_string_table {
-    int n_cols;
-    col_table **cols;
-    char *extra;
+    int n_cols;       /* number of columns (variables) included */
+    col_table **cols; /* column tables (see above) */
+    char *extra;      /* extra information */
 };
 
 static void clean_up_codevars (void);
@@ -55,6 +55,13 @@ static col_table *col_table_new (int colnum)
     return ct;
 }
 
+/**
+ * gretl_string_table_new:
+ * @err: location to receive error code.
+ *
+ * Returns: a pointer to a newly allocated string table.
+ */
+
 gretl_string_table *gretl_string_table_new (int *err)
 {
     gretl_string_table *st = malloc(sizeof *st);
@@ -70,6 +77,20 @@ gretl_string_table *gretl_string_table_new (int *err)
     return st;
 }
 
+/**
+ * gretl_string_table_new_from_cols_list:
+ * @list: list of 'columns' (variables) whose values are to be 
+ * given a string representation.  
+ * 
+ * These values in @list should correspond to the 0-based indices 
+ * of the variables in question within the dataset.  For example, 
+ * if strings are to be recorded for variables 2, 5 and 10 the
+ * @list argument would be {3, 2, 5, 10}.
+ *
+ * Returns: pointer to a newly allocated string table
+ * containing a column member for each ID in @list.
+ */
+
 gretl_string_table *string_table_new_from_cols_list (int *list)
 {
     gretl_string_table *st;
@@ -158,6 +179,26 @@ gretl_string_table_add_column (gretl_string_table *st, int colnum)
     return cols[n-1];
 }
 
+/**
+ * gretl_string_table_index:
+ * @st: a gretl string table.
+ * @s: the string to look up or add.
+ * @col: index of the column to be accessed or created.
+ * @addcol: non-zero to indicate that a column should be
+ * added if it's not already present.
+ * @prn: gretl printer (or %NULL).
+ *
+ * This function has two main uses: for lookup in the context of
+ * a completed string table, or for constructing such a
+ * table (with @addcol non-zero).  The returned index reflects 
+ * any additions to the table that may be required (if column
+ * @col does not already exist, or if string @s is not already
+ * stored for column @col).
+ *
+ * Returns: the 1-based index of @s within the column of
+ * @st that has index @col, if available, otherwise 0.
+ */
+
 int 
 gretl_string_table_index (gretl_string_table *st, const char *s, int col,
 			  int addcol, PRN *prn)
@@ -168,7 +209,7 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
     if (st == NULL) return idx;
 
     for (i=0; i<st->n_cols; i++) {
-	if ((st->cols[i])->idx == col) {
+	if (st->cols[i]->idx == col) {
 	    ct = st->cols[i];
 	    break;
 	}
@@ -193,6 +234,29 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
     return idx;
 }
 
+/* Used in the context of deletion of "empty" variables from
+   an imported dataset: the index of a given "column" in 
+   a string table is adjusted to match the new position of
+   the variable in question. 
+*/
+
+int gretl_string_table_reset_column_id (gretl_string_table *st, 
+					int oldid, int newid)
+{
+    if (st != NULL) {
+	int i;
+
+	for (i=0; i<st->n_cols; i++) {
+	    if (st->cols[i]->idx == oldid) {
+		st->cols[i]->idx = newid;
+		return 0;
+	    }
+	}
+    }
+
+    return E_DATA;
+}
+
 static void col_table_destroy (col_table *ct)
 {
     int i;
@@ -211,6 +275,13 @@ static void col_table_destroy (col_table *ct)
     free(ct);
 }
 
+/**
+ * gretl_string_table_destroy:
+ * @st: gretl string table.
+ *
+ * Frees all resources associated with @st.
+ */
+
 void gretl_string_table_destroy (gretl_string_table *st)
 {
     int i;
@@ -229,6 +300,18 @@ void gretl_string_table_destroy (gretl_string_table *st)
     free(st);
 }
 
+/**
+ * gretl_string_table_print:
+ * @st: gretl string table.
+ * @pdinfo: dataset information (for names of variables).
+ * @fname: name of file to which to print.
+ * @prn: gretl printer (or %NULL).
+ *
+ * Prints table @st to a file called @fname.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
 int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
 			      const char *fname, PRN *prn)
 {
@@ -244,7 +327,7 @@ int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
     }
 
     strcpy(stname, "string_table.txt");
-    gretl_path_prepend(stname, gretl_work_dir());
+    gretl_path_prepend(stname, gretl_workdir());
 
     fp = gretl_fopen(stname, "w");
     if (fp == NULL) {
@@ -290,9 +373,19 @@ int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
     return err;
 }
 
+/**
+ * gretl_string_table_add_extra:
+ * @st: gretl string table.
+ * @prn: gretl printer.
+ *
+ * Steals the printing buffer from @prn and adds it to @st.
+ * The buffer will be appended when @st is printed via
+ * gretl_string_table_print().
+ */
+
 void gretl_string_table_add_extra (gretl_string_table *st, PRN *prn)
 {
-    if (st != NULL) {
+    if (st != NULL && prn != NULL) {
 	st->extra = gretl_print_steal_buffer(prn);
     }
 }
@@ -320,7 +413,9 @@ static saved_string built_ins[] = {
     { "tramo",    0, NULL },
     { "tramodir", 0, NULL },
     { "seats",    0, NULL },
-    { "shelldir", 0, NULL }
+    { "shelldir", 0, NULL },
+    { "Rbin",     0, NULL },
+    { "Rlib",     0, NULL },
 };
 
 #ifdef WIN32
@@ -329,6 +424,15 @@ static saved_string dsep = { "dirsep", 0, "\\" };
 static saved_string dsep = { "dirsep", 0, "/" };
 #endif
 
+/**
+ * gretl_insert_builtin_string:
+ * @name: the name of the string to be added or replaced.
+ * @s: the value for this string variable.
+ *
+ * Inserts value @s for string @name in gretl's table
+ * of built-in string variables.
+ */
+
 void gretl_insert_builtin_string (const char *name, const char *s)
 {
     int n = sizeof built_ins / sizeof built_ins[0];
@@ -391,7 +495,13 @@ static saved_string *get_saved_string_by_name (const char *name,
     return NULL;
 }
 
-/* access a saved string by name */
+/**
+ * get_string_by_name:
+ * @name: the name of the string variable to access.
+ *
+ * Returns: the value of string variable @name, or %NULL
+ * if there is no such variable.
+ */
 
 char *get_string_by_name (const char *name)
 {
@@ -440,6 +550,17 @@ static saved_string *add_named_string (const char *name)
     return &S[n];
 }
 
+/**
+ * add_string_as:
+ * @s: string value to be added.
+ * @name: the name of the string variable to add.
+ *
+ * Adds @s to the saved array of string variables 
+ * under the name @name.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
 int add_string_as (const char *s, const char *name)
 {
     int n = n_saved_strings;
@@ -502,6 +623,17 @@ static int destroy_saved_string (saved_string *S)
     return err;
 }
 
+/**
+ * delete_saved_string:
+ * @name: the name of the string variable to delete.
+ * @prn: gretl printer (or %NULL).
+ *
+ * Deletes the string variable called @name.
+ *
+ * Returns: 0 on success, non-zero on failure (e.g.
+ * attempting to delete a built-in string variable).
+ */
+
 int delete_saved_string (const char *name, PRN *prn)
 {
     saved_string *S;
@@ -511,7 +643,7 @@ int delete_saved_string (const char *name, PRN *prn)
     if (S == NULL) {
 	err = E_UNKVAR;
     } else if (builtin) {
-	sprintf(gretl_errmsg, _("You cannot delete '%s'"), name);
+	gretl_errmsg_sprintf(_("You cannot delete '%s'"), name);
 	err = E_DATA;
     } else {
 	err = destroy_saved_string(S);
@@ -624,7 +756,7 @@ static int shell_grab (const char *arg, char **sout)
     }
 
     if (!libset_get_bool(SHELL_OK)) {
-	strcpy(gretl_errmsg, _("The shell command is not activated."));
+	gretl_errmsg_set(_("The shell command is not activated."));
 	return 1;
     }
 
@@ -694,18 +826,24 @@ static char *strim (char *s)
 
 char *retrieve_date_string (int t, const DATAINFO *pdinfo, int *err)
 {
-    char datestr[OBSLEN] = {0};
     char *ret = NULL;
 
-    if (t > 0 && t <= pdinfo->n) {
+    if (t <= 0 || t > pdinfo->n) {
+	*err = E_DATA;
+    } else if (pdinfo->S != NULL) {
+	ret = gretl_strdup(pdinfo->S[t-1]);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}	
+    } else {
+	char datestr[OBSLEN] = {0};
+
 	ntodate(datestr, t - 1, pdinfo);
 	ret = gretl_strdup(datestr);
 	if (ret == NULL) {
 	    *err = E_ALLOC;
 	}
-    } else {
-	*err = E_DATA;
-    }
+    } 
 
     return ret;
 }
@@ -724,7 +862,7 @@ char *retrieve_file_content (const char *fname, int *err)
 
 	*fullname = '\0';
 	strncat(fullname, fname, FILENAME_MAX - 1);
-	addpath(fullname, NULL, 0);
+	addpath(fullname, 0);
 
 	g_file_get_contents(fullname, &ret, &len, &gerr);
 
@@ -783,7 +921,7 @@ int save_named_string (const char *name, const char *s, PRN *prn)
 
     if (str != NULL && builtin) {
 	if (prn == NULL) {
-	    sprintf(gretl_errmsg, _("You cannot overwrite '%s'\n"), name);
+	    gretl_errmsg_sprintf(_("You cannot overwrite '%s'\n"), name);
 	} else {
 	    pprintf(prn, _("You cannot overwrite '%s'\n"), name);
 	}
@@ -888,8 +1026,8 @@ static char *maybe_get_subst (char *name, int *n, int quoted,
 
 static void too_long (void)
 {
-    sprintf(gretl_errmsg, _("Maximum length of command line "
-			    "(%d bytes) exceeded\n"), MAXLINE);
+    gretl_errmsg_sprintf(_("Maximum length of command line "
+			   "(%d bytes) exceeded\n"), MAXLINE);
 }
 
 #define var_context(s,i) (i > 8 && !strncmp(s - 9, "isstring(", 9))
@@ -907,14 +1045,8 @@ int substitute_named_strings (char *line, int *subst)
 	return 0;
     }
 
-    /* when using the genr apparatus for a string, let @foo be handled 
-       as a variable (FIXME?) */
-    if (!strncmp(line, "string ", 7)) {
-	return 0;
-    }
-
     if (!strncmp(line, "sscanf", 6)) {
-	/* when scanning, let @foo be handled as a variable */
+	/* when scanning, let @foo be handled as a variable (FIXME?) */
 	return 0;
     }    
 
