@@ -62,82 +62,6 @@ int read_reg_val (HKEY tree, const char *base,
 	err = 1;
     }
 
-    if (0) ;
-
-    RegCloseKey(regkey);
-
-    return err;
-}
-
-int read_reg_val_with_fallback (HKEY tree0, HKEY tree1, const char *base, 
-				char *keyname, char *keyval)
-{
-    int err;
-
-    err = read_reg_val(tree0, base, keyname, keyval);
-    if (err) {
-	err = read_reg_val(tree1, base, keyname, keyval);
-    }
-
-    return err;
-}
-
-static gchar *get_windows_string (char *s)
-{
-    gsize bytes;
-
-    return g_locale_from_utf8(s, -1, NULL, &bytes, NULL);
-}
-
-int write_reg_val (HKEY tree, const char *base, 
-		   const char *keyname, char *keyval,
-		   int ktype)
-{
-    gchar *tmp = NULL;
-    char regpath[64];
-    int err = 0;
-    HKEY regkey;
-
-    /* ensure that strings are in the locale encoding */
-    if (ktype == GRETL_TYPE_STRING && string_is_utf8(keyval)) {
-	tmp = get_windows_string(keyval);
-	if (tmp != NULL) {
-	    keyval = tmp;
-	}
-    }
-
-    sprintf(regpath, "Software\\%s", base);
-
-    if (RegCreateKeyEx(
-                       tree,
-                       regpath,
-                       0,
-                       NULL, 
-                       REG_OPTION_NON_VOLATILE,
-                       KEY_ALL_ACCESS,
-                       NULL,
-                       &regkey,
-                       NULL                         
-                       ) != ERROR_SUCCESS) {
-	fprintf(stderr, "RegCreateKeyEx: failed on '%s'\n", regpath);
-        return 1;
-    }
-
-    if (RegSetValueEx(
-                  regkey,
-                  keyname,
-                  0,
-                  REG_SZ,
-                  keyval,
-                  strlen(keyval) + 1) != ERROR_SUCCESS) {
-	fprintf(stderr, "RegSetValueEx: failed on '%s'\n", keyname);
-        err = 1;
-    }
-
-    if (tmp != NULL) {
-	g_free(tmp);
-    }
-                  
     RegCloseKey(regkey);
 
     return err;
@@ -165,109 +89,182 @@ DIR *win32_opendir (const char *dname)
     return opendir(tmp);
 }
 
-/* 
-   relevant extract from gretl.iss:
+static char netfile[FILENAME_MAX];
 
-   HKLM; "Software\gretl"; "gretldir";   "{app}"
-   HKLM; "Software\gretl"; "Rcommand";   "RGui.exe"
-   HKCU; "Software\gretl"; "binbase";    "{app}\db\"
-   HKCU; "Software\gretl"; "ratsbase";   "f:\"
-   HKCU; "Software\gretl"; "dbhost";     "ricardo.ecn.wfu.edu"
-   HKCU; "Software\gretl"; "dbproxy";    ""
-   HKCU; "Software\gretl"; "useproxy";   "false"
-   HKCU; "Software\gretl"; "updater";    "false"
-   HKCU; "Software\gretl"; "Fixed_font"; "Courier New 10"
-   HKCU; "Software\gretl"; "Png_font";   "verdana 8"
-   HKCU; "Software\gretl"; "Gp_colors";  ""
-*/
-
-void cli_read_registry (char *callname, PATHS *ppaths)
+const char *get_gretlnet_filename (void)
 {
-    char valstr[MAXLEN];
-    char dbproxy[21];
-    int use_proxy = 0;
-    char *tmp;
-    int drive = callname[0];
+    return (*netfile != '\0')? netfile : NULL;
+}
 
-    /* gretl installation directory */
-    ppaths->gretldir[0] = '\0';
-    read_reg_val(HKEY_LOCAL_MACHINE, "gretl", "gretldir", ppaths->gretldir);
-    if (ppaths->gretldir[0] == '\0') {
-	sprintf(ppaths->gretldir, "%c:\\userdata\\gretl\\", drive);
-    }
+int set_gretlnet_filename (const char *prog)
+{
+    char *p;
+    int i, n;
 
-    /* user's working directory */
-    ppaths->workdir[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "userdir", ppaths->workdir);
-    if (ppaths->workdir[0] == '\0') {
-	tmp = mydocs_path();
+    strcpy(netfile, prog);
+    n = strlen(netfile) - 1;
+    p = netfile;
 
-	if (tmp != NULL) {
-	    sprintf(ppaths->workdir, "%s\\gretl\\", tmp);
-	    free(tmp);
-	} else {
-	    sprintf(ppaths->workdir, "%c:\\userdata\\gretl\\user\\", drive);
+    for (i=n; i>0; i--) {
+	if (p[i] == '\\' || p[i] == '/') {
+	    strcpy(p + i,  "\\gretlnet.txt");
+	    break;
 	}
     }
 
-    /* "hidden" working dir: not user-configurable */
-    ppaths->dotdir[0] = '\0';
-    tmp = appdata_path();
-    if (tmp != NULL) {
-	sprintf(ppaths->dotdir, "%s\\gretl\\", tmp);
-	free(tmp);
-    } else {
-	sprintf(ppaths->dotdir, "%c:\\userdata\\gretl\\user\\", drive);
-    }    
+    return 0;
+}
 
-    /* base path for databases */
-    ppaths->binbase[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "binbase", ppaths->binbase);
-    if (ppaths->binbase[0] == '\0') {
-	sprintf(ppaths->binbase, "%c:\\userdata\\gretl\\db", drive);
+static FILE *cli_gretlnet_open (const char *prog)
+{
+    FILE *fp = NULL;
+
+    set_gretlnet_filename(prog);
+
+    if (*netfile != '\0') {
+	fp = gretl_fopen(netfile, "r");
     }
 
+    return fp;
+}
+
+static FILE *cli_rcfile_open (void)
+{
+    char *appdata = appdata_path();
+    FILE *fp = NULL;
+
+    if (appdata != NULL) {
+	char fname[FILENAME_MAX];
+
+	sprintf(fname, "%s\\gretl\\.gretl2rc", appdata);
+	free(appdata);
+	fp = fopen(fname, "r");
+    }
+
+    return fp;
+}
+
+int read_rc_string (FILE *fp, const char *key, char *value)
+{
+    int ret = 0;
+
+    if (fp != NULL) {
+	char line[MAXLEN];
+	char keystr[32];
+	int n;
+
+	rewind(fp);
+
+	sprintf(keystr, "%s = ", key);
+	n = strlen(keystr);
+
+	while (fgets(line, sizeof line, fp) && !ret) {
+	    chopstr(line);
+	    if (!strncmp(line, keystr, n)) {
+		strcpy(value, line + n);
+		ret = 1;
+	    }
+	}
+    }
+
+    return ret;
+}
+
+static int cli_read_gretl_var (char *key, char *val,
+			       FILE **fp, HKEY tree)
+{
+    int done = 0;
+
+    *val = '\0';
+
+    if (fp[0] != NULL) {
+	done = read_rc_string(fp[0], key, val);
+    }
+
+    if (!done && fp[1] != NULL) {
+	done = read_rc_string(fp[1], key, val);
+    }
+
+    if (!done) {
+	read_reg_val(tree, "gretl", key, val);
+    }
+
+    return *val != '\0';
+}
+
+void cli_read_registry (char *callname)
+{
+    ConfigPaths cpaths = {0};
+    char valstr[MAXLEN];
+    char dbproxy[21];
+    int done, use_proxy = 0;
+    FILE *fp[2];
+
+    fp[0] = cli_gretlnet_open(callname);
+    fp[1] = cli_rcfile_open();
+
+    /* gretl installation directory */
+    cli_read_gretl_var("gretldir", cpaths.gretldir, fp, HKEY_LOCAL_MACHINE);
+
+    /* user's working directory */
+    cli_read_gretl_var("userdir", cpaths.workdir, fp, HKEY_CURRENT_USER);
+
     /* base path for RATS databases */
-    ppaths->ratsbase[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "ratsbase", ppaths->ratsbase);
+    cli_read_gretl_var("ratsbase", cpaths.ratsbase, fp, HKEY_CURRENT_USER);
 
     /* path to X-12-ARIMA */
-    ppaths->x12a[0] = '\0';
-    read_reg_val_with_fallback(HKEY_LOCAL_MACHINE, HKEY_CLASSES_ROOT,
-			       "x12arima", "x12a", ppaths->x12a);
-    if (ppaths->x12a[0] == '\0') {
-	sprintf(ppaths->x12a, "%c:\\userdata\\x12arima\\x12a.exe", drive);
+    done = read_rc_string(fp[0], "x12a", cpaths.x12a);
+    if (!done) {
+	read_reg_val(HKEY_LOCAL_MACHINE, "x12arima", "x12a", cpaths.x12a);
     }
 
+    /* path to tramo */
+    done = read_rc_string(fp[0], "tramo", cpaths.tramo);
+    if (!done) {
+	read_reg_val(HKEY_LOCAL_MACHINE, "tramo", "tramo", cpaths.tramo);
+    }
+
+    /* path to R binary (non-interactive use) */
+    read_rc_string(fp[0], "Rbin", cpaths.rbinpath);
+
+    /* path to R shared library */
+    read_rc_string(fp[0], "Rlib", cpaths.rlibpath);
+
+#ifdef USE_OX
+    /* path to oxl */
+    read_rc_string(fp[0], "ox", cpaths.oxlpath);
+#endif
+
     /* remote database host */
-    ppaths->dbhost[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "dbhost", ppaths->dbhost);
-    if (ppaths->dbhost[0] == '\0') {
-	strcpy(ppaths->dbhost, "ricardo.ecn.wfu.edu");
-    }
+    cli_read_gretl_var("dbhost", cpaths.dbhost, fp, HKEY_CURRENT_USER);
 
     /* www proxy for reading remote databases */
-    dbproxy[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "dbproxy", dbproxy);
+    cli_read_gretl_var("dbproxy", dbproxy, fp, HKEY_CURRENT_USER);
 
     /* should a proxy be used? */
-    valstr[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "use_proxy", valstr);
+    cli_read_gretl_var("useproxy", valstr, fp, HKEY_CURRENT_USER);
     if (!strcmp(valstr, "true") || !strcmp(valstr, "1")) {
 	use_proxy = 1;
     } 
 
     /* do we allow the shell command within gretl? */
-    valstr[0] = '\0';
-    read_reg_val(HKEY_CURRENT_USER, "gretl", "shellok", valstr);
+    cli_read_gretl_var("shellok", valstr, fp, HKEY_CURRENT_USER);
     if (!strcmp(valstr, "true") || !strcmp(valstr, "1")) {
 	libset_set_bool(SHELL_OK, 1);
     } else {
 	libset_set_bool(SHELL_OK, 0);
     }
 
-    gretl_set_paths(ppaths, OPT_NONE);
-    gretl_www_init(ppaths->dbhost, dbproxy, use_proxy);
+    gretl_set_paths(&cpaths, OPT_NONE);
+    gretl_www_init(cpaths.dbhost, dbproxy, use_proxy);
+
+    if (fp[0] != NULL) {
+	fclose(fp[0]);
+    }
+
+    if (fp[1] != NULL) {
+	fclose(fp[1]);
+    }
 }
 
 void win_show_last_error (void)
@@ -275,8 +272,7 @@ void win_show_last_error (void)
     DWORD dw = GetLastError();
     LPVOID buf;
 
-    FormatMessage( 
-		  FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
 		  FORMAT_MESSAGE_FROM_SYSTEM | 
 		  FORMAT_MESSAGE_IGNORE_INSERTS,
 		  NULL,
@@ -284,8 +280,7 @@ void win_show_last_error (void)
 		  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
 		  (LPTSTR) &buf,
 		  0,
-		  NULL 
-		  );
+		  NULL); 
 
     MessageBox(NULL, (LPCTSTR) buf, "Error", MB_OK | MB_ICONERROR);
     LocalFree(buf);
@@ -296,8 +291,7 @@ void win_copy_last_error (void)
     DWORD dw = GetLastError();
     LPVOID buf;
 
-    FormatMessage( 
-		  FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
 		  FORMAT_MESSAGE_FROM_SYSTEM | 
 		  FORMAT_MESSAGE_IGNORE_INSERTS,
 		  NULL,
@@ -305,8 +299,7 @@ void win_copy_last_error (void)
 		  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
 		  (LPTSTR) &buf,
 		  0,
-		  NULL 
-		  );
+		  NULL);
 
     gretl_errmsg_set((const char *) buf);
     LocalFree(buf);
@@ -318,8 +311,7 @@ int winfork (char *cmdline, const char *dir,
     STARTUPINFO si;
     PROCESS_INFORMATION pi; 
     DWORD exitcode;
-    int child;
-    int err = 0;
+    int ok, err = 0;
 
     ZeroMemory(&si, sizeof si);
     ZeroMemory(&pi, sizeof pi);  
@@ -329,21 +321,21 @@ int winfork (char *cmdline, const char *dir,
     si.wShowWindow = wshow;
 
     /* zero return means failure */
-    child = CreateProcess(NULL, cmdline, 
-			  NULL, NULL, FALSE,
-			  flags,
-			  NULL, dir,
-			  &si, &pi);
+    ok = CreateProcess(NULL, cmdline, 
+		       NULL, NULL, FALSE,
+		       flags,
+		       NULL, dir,
+		       &si, &pi);
 
-    if (!child) {
+    if (!ok) {
 	win_copy_last_error();
 	err = 1;
     } else {
 	WaitForSingleObject(pi.hProcess, INFINITE); 
 	if (GetExitCodeProcess(pi.hProcess, &exitcode)) {
 	    if (exitcode != 0) {
-		sprintf(gretl_errmsg, "%s: exit code %d\n",
-			cmdline, exitcode);
+		gretl_errmsg_sprintf("%s: exit code %d\n", cmdline, 
+				     exitcode);
 		err = 1;
 	    }
 	} else {
@@ -409,6 +401,11 @@ char *mydocs_path (void)
     return win_special_path(CSIDL_PERSONAL);
 }
 
+char *program_files_path (void)
+{
+    return win_special_path(CSIDL_PROGRAM_FILES);
+}
+
 static char *compose_command_line (const char *arg)
 {
     CHAR cmddir[MAX_PATH];
@@ -447,11 +444,12 @@ static int read_from_pipe (HANDLE hwrite, HANDLE hread,
     if (!ok) {
 	fputs("Closing handle failed\n", stderr); 
     } else {
-	/* read output from the child process */
+	/* read output from the child process: note that the buffer
+	   must be NUL-terminated for use with pputs() */
 	while (1) { 
 	    memset(buf, '\0', BUFSIZE);
-	    if (!ReadFile(hread, buf, BUFSIZE, &dwread, NULL) || 
-		dwread == 0) {
+	    ok = ReadFile(hread, buf, BUFSIZE-1, &dwread, NULL);
+	    if (!ok || dwread == 0) {
 		break;
 	    }
 	    pputs(prn, buf);
@@ -466,15 +464,25 @@ static int read_from_pipe (HANDLE hwrite, HANDLE hread,
     return ok;
 } 
 
+enum {
+    SHELL_RUN,
+    PROG_RUN
+};
+
 static int 
-run_child_with_pipe (const char *arg, HANDLE hwrite, HANDLE hread) 
+run_child_with_pipe (const char *arg, HANDLE hwrite, HANDLE hread,
+		     int flag) 
 { 
     PROCESS_INFORMATION pinfo; 
     STARTUPINFO sinfo;
     char *cmdline = NULL;
     int ok;
-    
-    cmdline = compose_command_line(arg);
+
+    if (flag == SHELL_RUN) {
+	cmdline = compose_command_line(arg);
+    } else {
+	cmdline = g_strdup(arg);
+    }
  
     ZeroMemory(&pinfo, sizeof pinfo);
     ZeroMemory(&sinfo, sizeof sinfo);
@@ -488,11 +496,11 @@ run_child_with_pipe (const char *arg, HANDLE hwrite, HANDLE hread)
  
     ok = CreateProcess(NULL, 
 		       cmdline,
-		       NULL,          // process security attributes 
-		       NULL,          // primary thread security attributes 
-		       TRUE,          // handles are inherited 
+		       NULL,          /* process security attributes */
+		       NULL,          /* primary thread security attributes */
+		       TRUE,          /* handles are inherited */
 		       CREATE_NO_WINDOW,
-		       NULL,          // use parent's environment 
+		       NULL,          /* use parent's environment */
 		       get_shelldir(),          
 		       &sinfo,
 		       &pinfo);
@@ -509,7 +517,8 @@ run_child_with_pipe (const char *arg, HANDLE hwrite, HANDLE hread)
     return ok;
 }
 
-static int run_cmd_with_pipes (const char *arg, char **sout, PRN *prn) 
+static int run_cmd_with_pipes (const char *arg, char **sout, PRN *prn,
+			       int flag) 
 { 
     HANDLE hread, hwrite;
     SECURITY_ATTRIBUTES sa; 
@@ -529,7 +538,7 @@ static int run_cmd_with_pipes (const char *arg, char **sout, PRN *prn)
 	/* ensure that the read handle to the child process's pipe for 
 	   STDOUT is not inherited */
 	SetHandleInformation(hread, HANDLE_FLAG_INHERIT, 0);
-	ok = run_child_with_pipe(arg, hwrite, hread);
+	ok = run_child_with_pipe(arg, hwrite, hread, flag);
 	if (ok) {
 	    /* read from child's output pipe */
 	    read_from_pipe(hwrite, hread, sout, prn); 
@@ -575,9 +584,14 @@ static int run_cmd_wait (const char *cmd, PRN *prn)
     return err;
 }
 
+int gretl_win32_grab_output (const char *cmdline, char **sout)
+{
+    return run_cmd_with_pipes(cmdline, sout, NULL, PROG_RUN);
+}
+
 int gretl_shell_grab (const char *arg, char **sout)
 {
-    return run_cmd_with_pipes(arg, sout, NULL);
+    return run_cmd_with_pipes(arg, sout, NULL, SHELL_RUN);
 }
 
 int gretl_shell (const char *arg, PRN *prn)
@@ -591,7 +605,7 @@ int gretl_shell (const char *arg, PRN *prn)
     }
 
     if (!libset_get_bool(SHELL_OK)) {
-	strcpy(gretl_errmsg, _("The shell command is not activated."));
+	gretl_errmsg_set(_("The shell command is not activated."));
 	return 1;
     }
 
@@ -610,7 +624,7 @@ int gretl_shell (const char *arg, PRN *prn)
 	    err = 1;
 	}
     } else if (getenv("GRETL_SHELL_NEW")) {
-	err = run_cmd_with_pipes(arg, NULL, prn);
+	err = run_cmd_with_pipes(arg, NULL, prn, SHELL_RUN);
     } else {
 	err = run_cmd_wait(arg, prn);
     } 
@@ -669,11 +683,14 @@ int win32_write_access (char *path)
     if (!err) {
 	/* get the access mask for this trustee */
 	ret = GetEffectiveRightsFromAcl(dacl, &t, &amask);
-	err = (ret != ERROR_SUCCESS);
-    }
-
-    if (!err && (amask & STANDARD_RIGHTS_WRITE)) {
-	ok = 1;
+        if (ret != ERROR_SUCCESS) {
+            fprintf(stderr, "GetEffectiveRights...: ret=%d\n", ret);   
+            if (ret != RPC_S_SERVER_UNAVAILABLE && ret != ERROR_NO_SUCH_DOMAIN) {
+                err = 1;
+            }
+        } else if (amask & STANDARD_RIGHTS_WRITE) {
+	    ok = 1;
+	}
     }
 
     if (dacl != NULL) {
@@ -746,24 +763,125 @@ char *slash_convert (char *str, int which)
     return str;
 }
 
-char *R_path_from_registry (void)
+int R_path_from_registry (char *s, int which)
 {
-    char tmp[MAX_PATH] = {0}; 
-    char *ret = NULL;
     int err;
 
-    err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", tmp);
+    *s = '\0';
+
+    err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", s);
 
     if (err) {
-	err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", tmp);
+	err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", s);
     }
 
     if (!err) {
-	strcat(tmp, "\\bin\\");
-	strcat(tmp, "Rterm.exe");
-	ret = gretl_strdup(tmp);
+	if (which != RBASE) {
+	    strcat(s, "\\bin\\");
+	    if (which == RGUI) {
+		strcat(s, "Rgui.exe");
+	    } else if (which == RTERM) {
+		strcat(s, "Rterm.exe");
+	    } else if (which == RLIB) {
+		strcat(s, "R.dll");
+	    } 
+	}
     }
 
-    return ret;
+    return err;
 }
 
+/* for use in R, we need to form a version of the PATH with all
+   backslashes doubled 
+*/
+
+static char *get_fixed_R_path (const char *path, const char *rpath)
+{
+    char *fixpath;
+    int plen = (path != NULL)? strlen(path) : 0;
+    int rlen = strlen(rpath);
+    int i, ns = 0;
+
+    for (i=0; i<plen; i++) {
+	if (path[i] == '\\') ns++;
+    }
+
+    for (i=0; i<rlen; i++) {
+	if (rpath[i] == '\\') ns++;
+    }
+ 
+    fixpath = malloc(plen + rlen + ns + 1);
+
+    if (fixpath != NULL) {
+	int j = 0;
+
+	for (i=0; i<plen; i++) {
+	    if (path[i] == '\\') {
+		fixpath[j++] = '\\';
+		fixpath[j++] = '\\';
+	    } else {
+		fixpath[j++] = path[i];
+	    }
+	}
+
+	if (plen > 0) {
+	    fixpath[j++] = ';';
+	}
+
+	for (i=0; i<rlen; i++) {
+	    if (rpath[i] == '\\') {
+		fixpath[j++] = '\\';
+		fixpath[j++] = '\\';
+	    } else {
+		fixpath[j++] = rpath[i];
+	    }
+	}
+
+	fixpath[j] = '\0';
+    }
+
+    return fixpath;
+}
+
+int maybe_print_R_path_addition (FILE *fp)
+{
+    static char *fixpath;
+    static int ok;
+    int err = 0;
+
+    if (ok) {
+	; /* no need to amend the path */
+    } else if (fixpath != NULL) {
+	/* revised path already built */
+	fprintf(fp, "Sys.setenv(PATH=\"%s\")\n", fixpath);
+    } else {
+	char rpath[MAXLEN];
+
+	strcpy(rpath, gretl_rlib_path());
+
+	if (*rpath == '\0') {
+	    err = 1;
+	} else {
+	    char *p = strrchr(rpath, '\\');
+	    char *path = getenv("PATH");
+
+	    if (p != NULL) {
+		/* chop off "\R.dll" */
+		*p = '\0';
+	    }
+
+	    if (path != NULL && strstr(path, rpath) != NULL) {
+		ok = 1; /* nothing to be done */
+	    } else {
+		fixpath = get_fixed_R_path(path, rpath);
+		if (fixpath == NULL) {
+		    err = E_ALLOC;
+		} else {
+		    fprintf(fp, "Sys.setenv(PATH=\"%s\")\n", fixpath);
+		}
+	    }
+	}
+    }
+
+    return err;
+}
