@@ -62,14 +62,14 @@ extern int errno;
 extern int h_errno;
 #endif
 
-#define GRETLHOST  "ricardo.ecn.wfu.edu"
-#define DATACGI    "/gretl/cgi-bin/gretldata.cgi"
-#define UPDATECGI  "/gretl/cgi-bin/gretl_update.cgi"
-#define MPATH      "/pub/gretl/manual/PDF/"
 #define DBHLEN     64
 
 static int wproxy;
-static char dbhost[DBHLEN] = "ricardo.ecn.wfu.edu";
+static char dbhost[DBHLEN]    = "ricardo.ecn.wfu.edu";
+static char gretlhost[DBHLEN] = "ricardo.ecn.wfu.edu";
+static char datacgi[DBHLEN]   = "/gretl/cgi-bin/gretldata.cgi";
+static char updatecgi[DBHLEN] = "/gretl/cgi-bin/gretl_update.cgi";
+static char mpath[DBHLEN]     = "/pub/gretl/manual/PDF/";
 
 typedef enum {
     SAVE_NONE,
@@ -344,7 +344,7 @@ static int iwrite (int fd, const char *buf, int len);
 static char *print_option (int opt);
 
 #ifdef STANDALONE
-# define errmsg_set(s) fprintf(stderr, s)
+# define errmsg_set(s) fprintf(stderr, "%s\n", s)
 #else
 # define errmsg_set(s) gretl_errmsg_set(s)
 #endif
@@ -1515,7 +1515,7 @@ urlinfo_set_path (urlinfo *u, const char *cgi, const char *mname)
     if (cgi != NULL && *cgi != '\0') {
 	len += strlen(cgi);
     } else if (mname != NULL && *mname != '\0') {
-	len += strlen(MPATH) + strlen(mname);
+	len += strlen(mpath) + strlen(mname);
     }
 
     u->path = malloc(len);
@@ -1532,7 +1532,7 @@ urlinfo_set_path (urlinfo *u, const char *cgi, const char *mname)
     if (cgi != NULL && *cgi != '\0') {
 	strcat(u->path, cgi);
     } else if (mname != NULL && *mname != '\0') {
-	strcat(u->path, MPATH);
+	strcat(u->path, mpath);
 	strcat(u->path, mname);
     }
 	
@@ -1606,6 +1606,16 @@ static int maybe_register_type_error (urlinfo *u, CGIOpt opt)
     return err;
 }
 
+static void maybe_revise_www_paths (void)
+{
+    if (!strcmp(dbhost, "www.wfu.edu")) {
+	strcpy(gretlhost, "www.wfu.edu");
+	strcpy(datacgi, "/~cottrell/gretl/gretldata.cgi");
+	strcpy(updatecgi, "/~cottrell/gretl/gretl_update.cgi");
+	strcpy(mpath, "/~cottrell/gretl/manual/");
+    }
+}
+
 /* grab data from URL.  If saveopt = SAVE_TO_FILE then data is stored
    to a local file whose name is given by "savefile".  If saveopt =
    SAVE_TO_BUFFER then "getbuf" is presumed to point to a char buffer
@@ -1621,6 +1631,8 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
     uerr_t result;
     int err = 0;
 
+    maybe_revise_www_paths();
+
     if (getbuf != NULL) {
 	*getbuf = NULL;
     }
@@ -1631,11 +1643,11 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
     }
     
     if (opt == GRAB_FILE) {
-	urlinfo_set_path(u, UPDATECGI, NULL);
+	urlinfo_set_path(u, updatecgi, NULL);
     } else if (opt == GRAB_PDF) {
 	urlinfo_set_path(u, NULL, fname);
     } else {
-	urlinfo_set_path(u, DATACGI, NULL);
+	urlinfo_set_path(u, datacgi, NULL);
     }
 
     if (u->path == NULL) {
@@ -1679,7 +1691,12 @@ retrieve_url (const char *host, CGIOpt opt, const char *fname,
 #endif
 
     if (result != RETROK) {
-	strcpy(gretl_errmsg, u->errbuf);
+	if (u->errbuf != NULL && *u->errbuf != '\0') {
+	    fprintf(stderr, "u->errbuf = '%s'\n", u->errbuf);
+	} else {
+	    fprintf(stderr, "u->path = '%s'\n", u->path);
+	}
+	errmsg_set(u->errbuf);
 	err = 1;
     }
 
@@ -1722,8 +1739,8 @@ int gretl_www_init (const char *host, const char *proxy, int use_proxy)
 
     p = strrchr(proxy, ':');
     if (p == NULL) {
-	strcpy(gretl_errmsg, _("Failed to parse HTTP proxy:\n"
-	       "format must be ipnumber:port"));
+	errmsg_set(_("Failed to parse HTTP proxy:\n"
+		     "format must be ipnumber:port"));
 	return E_DATA;
     }
 
@@ -1731,7 +1748,7 @@ int gretl_www_init (const char *host, const char *proxy, int use_proxy)
 
     iplen = p - proxy;
     if (iplen > 15) {
-	strcpy(gretl_errmsg, _("HTTP proxy: first field must be an IP number"));
+	errmsg_set(_("HTTP proxy: first field must be an IP number"));
 	return E_DATA;	
     }
 
@@ -1783,7 +1800,7 @@ int get_update_info (char **saver, time_t filedate, int queryopt)
 	return E_ALLOC;
     }
 
-    urlinfo_set_path(u, UPDATECGI, NULL);
+    urlinfo_set_path(u, updatecgi, NULL);
     urlinfo_set_update_params(u, filedate, queryopt);
     getbuf_init(u);
 
@@ -1793,7 +1810,7 @@ int get_update_info (char **saver, time_t filedate, int queryopt)
 	return E_ALLOC;
     }
 
-    err = get_host_ip(u, GRETLHOST);
+    err = get_host_ip(u, gretlhost);
     if (err) {
 	free(u->getbuf);
 	urlinfo_destroy(u, 0);
@@ -1810,7 +1827,7 @@ int get_update_info (char **saver, time_t filedate, int queryopt)
 #endif
 
     if (result != RETROK) {
-	strcpy(gretl_errmsg, u->errbuf);
+	errmsg_set(u->errbuf);
 	err = 1;
     }
 
@@ -1859,7 +1876,7 @@ int upload_function_package (const char *login, const char *pass,
 	return E_ALLOC;
     }
 
-    urlinfo_set_path(u, DATACGI, NULL);
+    urlinfo_set_path(u, datacgi, NULL);
     urlinfo_set_upload_params(u, login, pass, fname);
     getbuf_init(u);
 
@@ -1871,7 +1888,7 @@ int upload_function_package (const char *login, const char *pass,
     u->upload = buf;
     u->upsize = strlen(buf) + 1;
 
-    err = get_host_ip(u, GRETLHOST);
+    err = get_host_ip(u, gretlhost);
     if (err) {
 	urlinfo_destroy(u, 0);
 	return E_ALLOC;
@@ -1882,7 +1899,7 @@ int upload_function_package (const char *login, const char *pass,
     result = try_http(u);
 
     if (result != RETROK) {
-	strcpy(gretl_errmsg, u->getbuf);
+	gretl_errmsg_set(u->getbuf);
 	err = 1;
     } else if (retbuf != NULL) {
 	*retbuf = u->getbuf;
@@ -1902,13 +1919,13 @@ int list_remote_dbs (char **getbuf)
 
 int list_remote_function_packages (char **getbuf)
 {
-    return retrieve_url (GRETLHOST, LIST_FUNCS, NULL, NULL, SAVE_TO_BUFFER, 
+    return retrieve_url (gretlhost, LIST_FUNCS, NULL, NULL, SAVE_TO_BUFFER, 
 			 NULL, getbuf);
 }
 
 int list_remote_data_packages (char **getbuf)
 {
-    return retrieve_url (GRETLHOST, LIST_PKGS, NULL, NULL, SAVE_TO_BUFFER, 
+    return retrieve_url (gretlhost, LIST_PKGS, NULL, NULL, SAVE_TO_BUFFER, 
 			 NULL, getbuf);
 }
 
@@ -1950,14 +1967,14 @@ int check_remote_db (const char *dbname)
 int retrieve_remote_function_package (const char *pkgname, 
 				      const char *localname)
 {
-    return retrieve_url (GRETLHOST, GRAB_FUNC, pkgname, NULL, SAVE_TO_FILE, 
+    return retrieve_url (gretlhost, GRAB_FUNC, pkgname, NULL, SAVE_TO_FILE, 
 			 localname, NULL);
 }
 
 int retrieve_remote_datafiles_package (const char *pkgname, 
 				       const char *localname)
 {
-    return retrieve_url (GRETLHOST, GRAB_PKG, pkgname, NULL, SAVE_TO_FILE, 
+    return retrieve_url (gretlhost, GRAB_PKG, pkgname, NULL, SAVE_TO_FILE, 
 			 localname, NULL);
 }
 
@@ -1972,7 +1989,7 @@ int retrieve_remote_db_data (const char *dbname,
 
 int retrieve_manfile (const char *fname, const char *localname)
 {
-    return retrieve_url (GRETLHOST, GRAB_PDF, fname, NULL, SAVE_TO_FILE,
+    return retrieve_url (gretlhost, GRAB_PDF, fname, NULL, SAVE_TO_FILE,
 			 localname, NULL);
 }
 
