@@ -17,6 +17,8 @@
  * 
  */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "gretl_xml.h"
 #include "gretl_panel.h"
@@ -77,14 +79,14 @@ int gretl_xml_open_doc_root (const char *fname,
 
     doc = gretl_xmlParseFile(fname);
     if (doc == NULL) {
-	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
+	gretl_errmsg_sprintf(_("xmlParseFile failed on %s"), fname);
 	err = 1;
     }
 
     if (!err) {
 	node = xmlDocGetRootElement(doc);
 	if (node == NULL) {
-	    sprintf(gretl_errmsg, _("%s: empty document"), fname);
+	    gretl_errmsg_sprintf(_("%s: empty document"), fname);
 	    xmlFreeDoc(doc);
 	    err = 1;
 	}
@@ -92,8 +94,8 @@ int gretl_xml_open_doc_root (const char *fname,
 
     if (!err) {
 	if (xmlStrcmp(node->name, (XUC) rootname)) {
-	    sprintf(gretl_errmsg, _("File of the wrong type, root node not %s"),
-		    rootname);
+	    gretl_errmsg_sprintf(_("File of the wrong type, root node not %s"),
+				 rootname);
 	    fprintf(stderr, "Unexpected root node '%s'\n", (char *) node->name);
 	    xmlFreeDoc(doc);
 	    err = 1;
@@ -820,72 +822,127 @@ int gretl_xml_child_get_string (xmlNodePtr node, xmlDocPtr doc,
     return ret;
 }
 
-static const char *skip_to_next (const char *s)
-{
-    s += strspn(s, " \r\n");
-    s += strcspn(s, " \r\n");
-    s += strspn(s, " \r\n");
-
-    return s;
-}
-
 static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
 				  GretlType type,
 				  int *nelem, int *err)
 {
     xmlChar *tmp = xmlGetProp(node, (XUC) "count");
-    const char *p;
     int *ivals = NULL;
     double *xvals = NULL;
     cmplx *cvals = NULL;
     void *ptr = NULL;
+    int nread = 0;
     int i, n = 0;
 
-    if (tmp != NULL) {
-	n = atoi((const char *) tmp);
-	free(tmp);
-	if (n > 0) {
-	    if (type == GRETL_TYPE_INT_ARRAY) {
-		ivals = malloc(n * sizeof *ivals);
-		ptr = ivals;
-	    } else if (type == GRETL_TYPE_DOUBLE_ARRAY) {
-		xvals = malloc(n * sizeof *xvals);
-		ptr = xvals;
-	    } else if (type == GRETL_TYPE_CMPLX_ARRAY) {
-		cvals = malloc(n * sizeof *cvals);
-		ptr = cvals;
-	    }
-	    if (ptr == NULL) {
-		*err = E_ALLOC;
-	    } else {
-		tmp = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
-		if (tmp == NULL) {
+    *nelem = 0;
+
+    if (tmp == NULL) {
+	fprintf(stderr, "gretl_xml_get_array: failed\n");
+	*err = E_DATA;
+	return NULL;
+    }
+
+    n = atoi((const char *) tmp);
+    free(tmp);
+
+    if (n <= 0) {
+	return NULL;
+    }    
+
+    if (type == GRETL_TYPE_INT_ARRAY) {
+	ivals = malloc(n * sizeof *ivals);
+	ptr = ivals;
+    } else if (type == GRETL_TYPE_DOUBLE_ARRAY) {
+	xvals = malloc(n * sizeof *xvals);
+	ptr = xvals;
+    } else if (type == GRETL_TYPE_CMPLX_ARRAY) {
+	cvals = malloc(n * sizeof *cvals);
+	ptr = cvals;
+    }
+
+    if (ptr == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    tmp = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+
+    if (tmp == NULL) {
+	*err = E_DATA;
+    } else {
+	const char *s = (const char *) tmp;
+	char *test;
+
+	errno = 0;
+	
+	if (type == GRETL_TYPE_DOUBLE_ARRAY) {
+	    double x;
+
+	    for (i=0; i<n && !*err && *s; i++) {
+		while (isspace(*s)) s++;
+		x = strtod(s, &test);
+		if (errno) {
+		    fprintf(stderr, "strtod failed on '%s'\n", s);
+		    perror(NULL);
+		    *err = E_DATA;
+		} else if (!strncmp(test, "NA", 2)) {
+		    x = NADBL;
+		    s = test + 2;
+		} else if (*test != '\0' && !isspace(*test)) {
 		    *err = E_DATA;
 		} else {
-		    p = (const char *) tmp;
-		    p += strspn(p, " \r\n");
-		    for (i=0; i<n && !*err; i++) {
-			if (type == GRETL_TYPE_INT_ARRAY) {
-			    if (sscanf(p, "%d", &ivals[i]) != 1) {
-				*err = E_DATA;
-			    }
-			} else if (type == GRETL_TYPE_DOUBLE_ARRAY) {
-			    if (!strncmp(p, "NA", 2)) {
-				xvals[i] = NADBL;
-			    } else if (sscanf(p, "%lf", &xvals[i]) != 1) {
-				*err = E_DATA;
-			    }
-			} else if (type == GRETL_TYPE_CMPLX_ARRAY) {
-			    if (sscanf(p, "%lf %lf", &cvals[i].r, &cvals[i].i) != 2) {
-				*err = E_DATA;
-			    } 
-			    p = skip_to_next(p);
-			}
-			p = skip_to_next(p);
-		    }
-		    free(tmp);
+		    s = test;
+		}
+		xvals[i] = x;
+		nread++;
+	    }
+	} if (type == GRETL_TYPE_INT_ARRAY) {
+	    long kl;
+
+	    for (i=0; i<n && !*err && *s; i++) {
+		while (isspace(*s)) s++;
+		kl = strtol(s, &test, 10);
+		if (errno) {
+		    *err = E_DATA;
+		} else if (*test != '\0' && !isspace(*test)) {
+		    *err = E_DATA;
+		} else {
+		    s = test;
+		    ivals[i] = kl;
+		    nread++;
 		}
 	    }
+	} else if (type == GRETL_TYPE_CMPLX_ARRAY) { 
+	    double x;
+	    int n2 = n * 2;
+	    int rval = 1;
+
+	    for (i=0; i<n2 && !*err && *s; i++) {
+		while (isspace(*s)) s++;
+		x = strtod(s, &test);
+		if (errno) {
+		    *err = E_DATA;
+		} else if (*test != '\0' && !isspace(*test)) {
+		    *err = E_DATA;
+		} else {
+		    s = test;
+		    if (rval) {
+			cvals[nread].r = x;
+			rval = 0;
+		    } else {
+			cvals[nread].i = x;
+			rval = 1;
+			nread++;
+		    }
+		}
+	    }
+	}   
+
+	free(tmp);
+
+	if (nread < n) {
+	    fprintf(stderr, "expected %d items in array, but got %d\n", n, nread);
+	    *err = E_DATA;
 	}
     }
 
@@ -933,6 +990,12 @@ int *gretl_xml_get_int_array (xmlNodePtr node, xmlDocPtr doc,
 double *gretl_xml_get_double_array (xmlNodePtr node, xmlDocPtr doc,
 				    int *nelem, int *err)
 {
+    int myerr = 0;
+
+    if (err == NULL) {
+	err = &myerr;
+    }
+	
     return gretl_xml_get_array(node, doc, GRETL_TYPE_DOUBLE_ARRAY,
 			       nelem, err);
 }
@@ -955,19 +1018,61 @@ cmplx *gretl_xml_get_cmplx_array (xmlNodePtr node, xmlDocPtr doc,
 			       nelem, err);
 }
 
+static char *chunk_strdup (const char *src, const char **ptr, int *err)
+{
+    char *targ = NULL;
+
+    if (*src == '\0') {
+	*ptr = src;
+    } else {
+	const char *p;
+	int len = 0;
+
+	while (*src && isspace(*src)) {
+	    src++;
+	}
+
+	p = src;
+
+	while (*src && !isspace(*src)) {
+	    len++;
+	    src++;
+	}
+
+	if (ptr != NULL) {
+	    *ptr = src;
+	}
+
+	if (len > 0) {
+	    targ = gretl_strndup(p, len);
+	    if (targ == NULL) {
+		*err = E_ALLOC;
+	    }
+	}
+    }
+
+    if (targ == NULL && !*err) {
+	*err = E_DATA;
+    }
+
+    return targ;
+}
+
 /**
  * gretl_xml_get_strings_array:
  * @node: XML node pointer.
  * @doc: XML document pointer.
  * @nelem: location to receive number of elements in array.
+ * @slop: if non-zero, allow the number of strings to fall
+ * short of the recorded string count by one.
  * @err: location to receive error code.
  * 
- * Returns: allocated array of cmplx (complex numbers) read from 
- * @node, or %NULL on failure.
+ * Returns: allocated array of strings read from @node, or 
+ * %NULL on failure.
  */
 
 char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
-				    int *nelem, int *err)
+				    int *nelem, int slop, int *err)
 {
     xmlChar *tmp = xmlGetProp(node, (XUC) "count");
     char **S = NULL;
@@ -992,10 +1097,11 @@ char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
 		*err = E_DATA;
 	    } else {
 		p = (const char *) tmp;
-		for (i=0; i<n; i++) {
-		    S[i] = gretl_word_strdup(p, &p);
-		    if (S[i] == NULL) {
-			*err = E_ALLOC;
+		for (i=0; i<n && !*err; i++) {
+		    S[i] = chunk_strdup(p, &p, err);
+		    if (*err == E_DATA && i == n - 1 && slop) {
+			*err = 0;
+			n--;
 		    }
 		}
 		free(tmp);
@@ -1102,6 +1208,8 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
     p = (const char *) tmp;
     p += strspn(p, " \r\n");
 
+    gretl_push_c_numeric_locale();
+
     for (i=0; i<rows && !*err; i++) {
 	for (j=0; j<cols && !*err; j++) {
 	    if (sscanf(p, "%lf", &x) != 1) {
@@ -1115,6 +1223,8 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
 	}
     }
 
+    gretl_pop_c_numeric_locale();
+
     free(tmp);
 
     if (*err) {
@@ -1295,7 +1405,7 @@ int gretl_write_matrix_as_gdt (const char *fname,
     fz = gretl_gzopen(fname, "wb");
 
     if (fz == Z_NULL) {
-	sprintf(gretl_errmsg, _("Couldn't open %s for writing"), fname);
+	gretl_errmsg_sprintf(_("Couldn't open %s for writing"), fname);
 	return 1;
     }
 
@@ -1380,7 +1490,8 @@ int gretl_write_matrix_as_gdt (const char *fname,
  * @Z: data matrix.
  * @pdinfo: data information struct.
  * @opt: if %OPT_Z write gzipped data, else uncompressed.
- * @ppaths: pointer to paths information (or NULL).
+ * @progress: may be 1 when called from gui to display progress
+ * bar in case of a large data write; generally should be 0.
  * 
  * Write out in xml a data file containing the values of the given set
  * of variables.
@@ -1390,7 +1501,7 @@ int gretl_write_matrix_as_gdt (const char *fname,
 
 int gretl_write_gdt (const char *fname, const int *list, 
 		     const double **Z, const DATAINFO *pdinfo, 
-		     gretlopt opt, PATHS *ppaths)
+		     gretlopt opt, int progress)
 {
     FILE *fp = NULL;
     gzFile *fz = Z_NULL;
@@ -1418,7 +1529,7 @@ int gretl_write_gdt (const char *fname, const int *list,
     }
 
     if (err) {
-	sprintf(gretl_errmsg, _("Couldn't open %s for writing"), fname);
+	gretl_errmsg_sprintf(_("Couldn't open %s for writing"), fname);
 	return 1;
     }
 
@@ -1437,7 +1548,7 @@ int gretl_write_gdt (const char *fname, const int *list,
     sz = (tsamp * nvars * sizeof(double));
     if (sz > 100000) {
 	fprintf(stderr, I_("Writing %ld Kbytes of data\n"), sz / 1024);
-	if (ppaths == NULL) {
+	if (!progress) {
 	    sz = 0L;
 	}
     } else {
@@ -1458,8 +1569,8 @@ int gretl_write_gdt (const char *fname, const int *list,
 	pmax[i-1] = get_precision(&Z[v][pdinfo->t1], tsamp, 10);
     }
 
-    ntodate_full(startdate, pdinfo->t1, pdinfo);
-    ntodate_full(enddate, pdinfo->t2, pdinfo);
+    ntodate(startdate, pdinfo->t1, pdinfo);
+    ntodate(enddate, pdinfo->t2, pdinfo);
 
     simple_fname(datname, fname);
     uerr = gretl_xml_encode_to_buf(xmlbuf, datname, sizeof xmlbuf);
@@ -1697,7 +1808,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	if (sscanf((char *) tmp, "%d", &v) == 1) {
 	    pdinfo->v = v + 1;
 	} else {
-	    sprintf(gretl_errmsg, _("Failed to parse count of variables"));
+	    gretl_errmsg_set(_("Failed to parse count of variables"));
 	    err = 1;
 	}
 	if (!err && dataset_allocate_varnames(pdinfo)) {
@@ -1711,7 +1822,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	}		
 	free(tmp);
     } else {
-	sprintf(gretl_errmsg, _("Got no variables"));
+	gretl_errmsg_set(_("Got no variables"));
 	err = 1;
     }
 
@@ -1724,7 +1835,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
     }
 
     if (cur == 0) {
-	sprintf(gretl_errmsg, _("Got no variables"));
+	gretl_errmsg_set(_("Got no variables"));
 	return 1;
     }
 
@@ -1736,7 +1847,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 		transcribe_string(pdinfo->varname[i], (char *) tmp, VNAMELEN);
 		free(tmp);
 	    } else {
-		sprintf(gretl_errmsg, _("Variable %d has no name"), i);
+		gretl_errmsg_sprintf(_("Variable %d has no name"), i);
 		return 1;
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "label");
@@ -1776,7 +1887,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
 	    tmp = xmlGetProp(cur, (XUC) "discrete");
 	    if (tmp != NULL) {
 		if (!strcmp((char *) tmp, "true")) {
-		    set_var_discrete(pdinfo, i, 1);
+		    series_set_flag(pdinfo, i, VAR_DISCRETE);
 		}
 		free(tmp);
 	    }
@@ -1804,7 +1915,7 @@ static int process_varlist (xmlNodePtr node, DATAINFO *pdinfo, double ***pZ)
     }
    
     if (i != pdinfo->v) {
-	sprintf(gretl_errmsg, _("Number of variables does not match declaration"));
+	gretl_errmsg_set(_("Number of variables does not match declaration"));
 	err = 1;
     } 
 
@@ -1837,13 +1948,15 @@ static int process_values (double **Z, DATAINFO *pdinfo, int t, char *s)
 	}
     }	
 
-    if (err && *gretl_errmsg == '\0') {
-	sprintf(gretl_errmsg, _("Failed to parse data values at obs %d"), t+1);
+    if (err && !gretl_errmsg_is_set()) {
+	gretl_errmsg_sprintf(_("Failed to parse data values at obs %d"), t+1);
     }
 
     return err;
 }
 
+#define GDT_DEBUG 0
+
 static int process_observations (xmlDocPtr doc, xmlNodePtr node, 
 				 double ***pZ, DATAINFO *pdinfo,
 				 long progress)
@@ -1865,7 +1978,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 	pdinfo->n = n;
 	free(tmp);
     } else {
-	sprintf(gretl_errmsg, _("Failed to parse number of observations"));
+	gretl_errmsg_set(_("Failed to parse number of observations"));
 	free(tmp);
 	return E_DATA;
     }
@@ -1884,8 +1997,8 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		return E_ALLOC;
 	    }
 	} else if (strcmp((char *) tmp, "false")) {
-	    sprintf(gretl_errmsg, _("labels attribute for observations must be "
-		    "'true' or 'false'"));
+	    gretl_errmsg_set(_("labels attribute for observations must be "
+			       "'true' or 'false'"));
 	    return E_DATA;
 	}
 	free(tmp);
@@ -1929,12 +2042,16 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
     }
 
     if (cur == NULL) {
-	sprintf(gretl_errmsg, _("Got no observations\n"));
+	gretl_errmsg_set(_("Got no observations\n"));
 	return E_DATA;
     }
 
     if (progress) {
 	(*show_progress)(0L, progress, SP_LOAD_INIT);
+#if GDT_DEBUG
+	fprintf(stderr, "process_observations: inited progess bar (n=%d)\n",
+		pdinfo->n);
+#endif
     }
 
     t = 0;
@@ -1947,7 +2064,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		    transcribe_string(pdinfo->S[t], (char *) tmp, OBSLEN);
 		    free(tmp);
 		} else {
-		    sprintf(gretl_errmsg, _("Case marker missing at obs %d"), t+1);
+		    gretl_errmsg_sprintf(_("Case marker missing at obs %d"), t+1);
 		    return E_DATA;
 		}
 	    }
@@ -1966,7 +2083,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		    free(tmp);
 		} 
 		if (ok < 2) {
-		    sprintf(gretl_errmsg, "Panel index missing at obs %d", t+1);
+		    gretl_errmsg_sprintf("Panel index missing at obs %d", t+1);
 		    return E_DATA;
 		}
 		pdinfo->paninfo->unit[t] = j;
@@ -1982,7 +2099,7 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
 		free(tmp);
 		t++;
 	    } else {
-		sprintf(gretl_errmsg, _("Values missing at observation %d"), t+1);
+		gretl_errmsg_sprintf(_("Values missing at observation %d"), t+1);
 		err = E_DATA;
 		goto bailout;
 	    }
@@ -2004,12 +2121,15 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
  bailout:
 
     if (progress) {
+#if GDT_DEBUG
+	fprintf(stderr, "finalizing progress bar (n = %d)\n", pdinfo->n);
+#endif
 	(*show_progress)(0L, (long) pdinfo->n, SP_FINISH);
 	close_plugin(handle);
     }
 
     if (!err && t != pdinfo->n) {
-	sprintf(gretl_errmsg, _("Number of observations does not match declaration"));
+	gretl_errmsg_set(_("Number of observations does not match declaration"));
 	err = E_DATA;
     }
 
@@ -2035,8 +2155,7 @@ static int xml_get_data_structure (xmlNodePtr node, int *dtype)
     int err = 0;
 
     if (tmp == NULL) {
-	sprintf(gretl_errmsg, 
-		_("Required attribute 'type' is missing from data file"));
+	gretl_errmsg_set(_("Required attribute 'type' is missing from data file"));
 	err = 1;
     } else {
 	if (!strcmp((char *) tmp, "cross-section")) {
@@ -2048,7 +2167,7 @@ static int xml_get_data_structure (xmlNodePtr node, int *dtype)
 	} else if (!strcmp((char *) tmp, "stacked-cross-section")) {
 	    *dtype = STACKED_CROSS_SECTION;
 	} else {
-	    sprintf(gretl_errmsg, _("Unrecognized type attribute for data file"));
+	    gretl_errmsg_set(_("Unrecognized type attribute for data file"));
 	    err = 1;
 	}
 	free(tmp);
@@ -2073,7 +2192,7 @@ static int xml_get_data_frequency (xmlNodePtr node, int *pd, int *dtype)
 		fprintf(stderr, "custom time series, using frequency 1\n");
 	    }
 	} else if (sscanf((char *) tmp, "%d", pd) != 1) {
-	    strcpy(gretl_errmsg, _("Failed to parse data frequency"));
+	    gretl_errmsg_set(_("Failed to parse data frequency"));
 	    err = 1;
 	}
 	free(tmp);
@@ -2114,7 +2233,7 @@ static int xml_get_startobs (xmlNodePtr node, double *sd0, char *stobs,
 	}
 
 	if (err) {
-	    strcpy(gretl_errmsg, _("Failed to parse startobs"));
+	    gretl_errmsg_set(_("Failed to parse startobs"));
 	} else {
 	    stobs[0] = '\0';
 	    strncat(stobs, (char *) tmp, OBSLEN - 1);
@@ -2146,7 +2265,7 @@ static int xml_get_endobs (xmlNodePtr node, char *endobs, int caldata)
 	} 
 
 	if (err) {
-	    strcpy(gretl_errmsg, _("Failed to parse endobs"));
+	    gretl_errmsg_set(_("Failed to parse endobs"));
 	} else {
 	    endobs[0] = '\0';
 	    strncat(endobs, (char *) tmp, OBSLEN - 1);
@@ -2238,10 +2357,9 @@ static long get_filesize (const char *fname)
 /**
  * gretl_read_gdt:
  * @fname: name of file to try.
- * @ppaths: path information struct.
  * @pZ: pointer to data set.
  * @pdinfo: pointer to data information struct.
- * @opt: use %OPT_P to display gui progress bar.
+ * @opt: use %OPT_B to display gui progress bar.
  * @prn: where messages should be written.
  * 
  * Read data from file into gretl's work space, allocating space as
@@ -2251,15 +2369,13 @@ static long get_filesize (const char *fname)
  * Returns: 0 on successful completion, non-zero otherwise.
  */
 
-int gretl_read_gdt (char *fname, PATHS *ppaths,
-		    double ***pZ, DATAINFO *pdinfo, 
+int gretl_read_gdt (char *fname, double ***pZ, DATAINFO *pdinfo, 
 		    gretlopt opt, PRN *prn) 
 {
     DATAINFO *tmpdinfo;
     double **tmpZ = NULL;
     xmlDocPtr doc = NULL;
     xmlNodePtr cur;
-    int newdata = (*pZ == NULL);
     int gotvars = 0, gotobs = 0, err = 0;
     int caldata = 0;
     double gdtversion = 1.0;
@@ -2279,12 +2395,12 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 	fprintf(stderr, "%s %ld bytes %s...\n", 
 		(is_gzipped(fname))? I_("Uncompressing") : I_("Reading"),
 		fsz, I_("of data"));
-	if (opt & OPT_P) progress = fsz;
+	if (opt & OPT_B) {
+	    progress = fsz;
+	}
     }
 
-#ifdef ENABLE_NLS
     check_for_console(prn);
-#endif
 
     tmpdinfo = datainfo_new();
     if (tmpdinfo == NULL) {
@@ -2294,20 +2410,20 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 
     doc = gretl_xmlParseFile(fname);
     if (doc == NULL) {
-	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
+	gretl_errmsg_sprintf(_("xmlParseFile failed on %s"), fname);
 	err = 1;
 	goto bailout;
     }
 
     cur = xmlDocGetRootElement(doc);
     if (cur == NULL) {
-        sprintf(gretl_errmsg, _("%s: empty document"), fname);
+        gretl_errmsg_sprintf(_("%s: empty document"), fname);
 	err = 1;
 	goto bailout;
     }
 
     if (xmlStrcmp(cur->name, (XUC) "gretldata")) {
-        sprintf(gretl_errmsg, _("File of the wrong type, root node not gretldata"));
+        gretl_errmsg_set(_("File of the wrong type, root node not gretldata"));
 	err = 1;
 	goto bailout;
     }
@@ -2346,6 +2462,10 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 	goto bailout;
     }     
 
+#if GDT_DEBUG
+    fprintf(stderr, "starting to walk XML tree...\n");
+#endif
+
     /* Now walk the tree */
     cur = cur->xmlChildrenNode;
     while (cur != NULL && !err) {
@@ -2360,7 +2480,7 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 	    }
 	} else if (!xmlStrcmp(cur->name, (XUC) "observations")) {
 	    if (!gotvars) {
-		sprintf(gretl_errmsg, _("Variables information is missing"));
+		gretl_errmsg_set(_("Variables information is missing"));
 		err = 1;
 	    }
 	    if (process_observations(doc, cur, &tmpZ, tmpdinfo, progress)) {
@@ -2369,9 +2489,15 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 		gotobs = 1;
 	    }
 	}
-	if (!err) cur = cur->next;
+	if (!err) {
+	    cur = cur->next;
+	}
     }
 
+#if GDT_DEBUG
+    fprintf(stderr, "done walking XML tree...\n");
+#endif
+
     gretl_pop_c_numeric_locale();
 
     if (err) {
@@ -2379,13 +2505,13 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
     }
 
     if (!gotvars) {
-	sprintf(gretl_errmsg, _("Variables information is missing"));
+	gretl_errmsg_set(_("Variables information is missing"));
 	err = 1;
 	goto bailout;
     }
 
     if (!gotobs) {
-	sprintf(gretl_errmsg, _("No observations were found"));
+	gretl_errmsg_set(_("No observations were found"));
 	err = 1;
 	goto bailout;
     }
@@ -2394,10 +2520,6 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 
     err = merge_or_replace_data(pZ, pdinfo, &tmpZ, &tmpdinfo, opt, prn);
 
-    if (!err && newdata && ppaths != NULL && fname != ppaths->datfile) {
-	strcpy(ppaths->datfile, fname);
-    }
-
  bailout:
 
     if (doc != NULL) {
@@ -2428,8 +2550,10 @@ int gretl_read_gdt (char *fname, PATHS *ppaths,
 	destroy_dataset(tmpZ, tmpdinfo);
     }
 
-#ifdef ENABLE_NLS
     console_off();
+
+#if GDT_DEBUG
+    fprintf(stderr, "gretl_read_gdt: returning %d\n", err);
 #endif
 
     return err;
@@ -2457,19 +2581,19 @@ char *gretl_get_gdt_description (const char *fname)
 
     doc = gretl_xmlParseFile(fname);
     if (doc == NULL) {
-	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
+	gretl_errmsg_sprintf(_("xmlParseFile failed on %s"), fname);
 	return NULL;
     }
 
     cur = xmlDocGetRootElement(doc);
     if (cur == NULL) {
-        sprintf(gretl_errmsg, _("%s: empty document"), fname);
+        gretl_errmsg_sprintf(_("%s: empty document"), fname);
 	xmlFreeDoc(doc);
 	return NULL;
     }
 
     if (xmlStrcmp(cur->name, (XUC) "gretldata")) {
-        sprintf(gretl_errmsg, _("File of the wrong type, root node not gretldata"));
+        gretl_errmsg_set(_("File of the wrong type, root node not gretldata"));
 	xmlFreeDoc(doc);
 	return NULL;
     }
@@ -2498,12 +2622,12 @@ static char *gretl_xml_get_doc_type (const char *fname, int *err)
     doc = gretl_xmlParseFile(fname);
 
     if (doc == NULL) {
-	sprintf(gretl_errmsg, _("xmlParseFile failed on %s"), fname);
+	gretl_errmsg_sprintf(_("xmlParseFile failed on %s"), fname);
 	*err = 1;
     } else {
 	node = xmlDocGetRootElement(doc);
 	if (node == NULL) {
-	    sprintf(gretl_errmsg, _("%s: empty document"), fname);
+	    gretl_errmsg_sprintf(_("%s: empty document"), fname);
 	    *err = 1;
 	} else {
 	    ret = gretl_strdup((char *) node->name);
@@ -2605,6 +2729,11 @@ int load_user_scalars_file (const char *fname)
     return err;
 }
 
+/* This is called in response to the "include" command in
+   the CLI program, the GUI program, and in interact.c,
+   if we detect that the named file is XML.
+*/
+
 int load_user_XML_file (const char *fname)
 {
     char *rootname = NULL;
@@ -2613,9 +2742,11 @@ int load_user_XML_file (const char *fname)
     rootname = gretl_xml_get_doc_type(fname, &err);
 
     if (!strcmp(rootname, "gretl-functions")) {
-	err = load_user_function_file(fname);
+	err = load_function_package_from_file(fname);
     } else if (!strcmp(rootname, "gretl-matrices")) {
 	err = load_user_matrix_file(fname);
+    } else if (!strcmp(rootname, "gretl-scalars")) {
+	err = load_user_scalars_file(fname);
     }
 
     free(rootname);
