@@ -1890,7 +1890,7 @@ static gretl_matrix *k_matrix_from_dataset (char *s,
 
     if (m != NULL) {
 	strcpy(s, GENERIC_MATNAME);
-    }	
+    }
 
     return m;
 }
@@ -1901,25 +1901,26 @@ static gretl_matrix *k_matrix_from_dataset (char *s,
    record its name so we're able to update its value later if need be.
 */
 
-static gretl_matrix *k_matrix_from_scalar (char *s, int *err)
+static gretl_matrix *k_matrix_from_scalar (char *s, int *errp)
 {
     gretl_matrix *m = NULL;
     double x;
+    int err = 0;
 
-    x = gretl_double_from_string(s, err);
+    x = gretl_double_from_string(s, &err);
 
-    if (!*err && na(x)) {
-	*err = E_MISSDATA;
+    if (!err && na(x)) {
+	*errp = err = E_MISSDATA;
     }
 
-    if (!*err) {
+    if (!err) {
 	m = gretl_matrix_from_scalar(x);
 	if (m == NULL) {
-	    *err = E_ALLOC;
+	    *errp = err = E_ALLOC;
 	}
     }
 
-    if (!*err) {
+    if (!err) {
 	if (gretl_is_scalar(s)) {
 	    /* got a scalar variable: record its name */
 	    char tmp[VNAMELEN];
@@ -2306,6 +2307,8 @@ static int get_kalman_matrix_id (const char **ps)
     }
 
     /* failed */
+    gretl_errmsg_sprintf("kalman: invalid input '%s'", *ps);
+
     return -1;
 }
 
@@ -2493,93 +2496,103 @@ static void transcribe_and_free (double *x, gretl_matrix *m,
     gretl_matrix_free(m);
 }
 
-/* called by the kfilter() function: run a user-defined Kalman
-   filter in forecasting mode */
+/* Called on behalf of the kfilter() function: run a user-defined Kalman
+   filter in forecasting mode. The doc for kfilter says that it return
+   0 on successful completion or 1 if numerical problems are encountered.
+*/
 
 int user_kalman_run (const char *E, const char *V, 
 		     const char *S, const char *P,
 		     const char *G, double **Z,
 		     const DATAINFO *pdinfo,
-		     PRN *prn, int *err)
+		     PRN *prn, int *errp)
 {
     user_kalman *u = get_user_kalman(-1);
     int smat[K_K+1] = {0};
     kalman *K;
-    int ret = 0;
+    int err = 0;
 
     if (u == NULL) {
-	*err = missing_kalman_error();
+	err = missing_kalman_error();
 	return 1;
     }
 
     K = u->K;
 
-    if (!*err) {
+    if (!err) {
 	/* forecast errors */
-	*err = attach_export_matrix(K, E, pdinfo, smat, K_E);
+	err = attach_export_matrix(K, E, pdinfo, smat, K_E);
     }
 
-    if (!*err) {
+    if (!err) {
 	/* MSE for observables */
-	*err = attach_export_matrix(K, V, pdinfo, smat, K_V);
+	err = attach_export_matrix(K, V, pdinfo, smat, K_V);
     } 
 
-    if (!*err) {
+    if (!err) {
 	/* estimate of state */
-	*err = attach_export_matrix(K, S, pdinfo, smat, K_BIG_S);
+	err = attach_export_matrix(K, S, pdinfo, smat, K_BIG_S);
     }
 
-    if (!*err) {
+    if (!err) {
 	/* MSE of estimate of state */
-	*err = attach_export_matrix(K, P, pdinfo, smat, K_BIG_P);
+	err = attach_export_matrix(K, P, pdinfo, smat, K_BIG_P);
     } 
 
-    if (!*err) {
+    if (!err) {
 	/* Kalman gain */
-	*err = attach_export_matrix(K, G, pdinfo, smat, K_K);
+	err = attach_export_matrix(K, G, pdinfo, smat, K_K);
     } 
 
-    if (!*err && K->LL == NULL) {
+    if (!err && K->LL == NULL) {
 	/* log-likelihood: available via accessor */
 	K->LL = gretl_matrix_alloc(K->T, 1);
 	if (K->LL == NULL) {
-	    *err = E_ALLOC;
+	    err = E_ALLOC;
 	}
     }
 
-    if (!*err) {
-	*err = user_kalman_recheck_matrices(u);
+    if (!err) {
+	err = user_kalman_recheck_matrices(u);
     }
 
-    if (!*err) {
-	*err = kalman_forecast(K, prn);
+    if (!err) {
+	err = kalman_forecast(K, prn);
     }
 
     if (pdinfo != NULL) {
 	int t1 = pdinfo->t1;
 
 	if (smat[K_E] > 0) 
-	    transcribe_and_free(Z[smat[K_E]] + t1, K->E, *err);
+	    transcribe_and_free(Z[smat[K_E]] + t1, K->E, err);
 	if (smat[K_V > 0]) 
-	    transcribe_and_free(Z[smat[K_V]] + t1, K->V, *err);
+	    transcribe_and_free(Z[smat[K_V]] + t1, K->V, err);
 	if (smat[K_BIG_S] > 0) 
-	    transcribe_and_free(Z[smat[K_BIG_S]] + t1, K->S, *err);
+	    transcribe_and_free(Z[smat[K_BIG_S]] + t1, K->S, err);
 	if (smat[K_BIG_P] > 0) 
-	    transcribe_and_free(Z[smat[K_BIG_P]] + t1, K->P, *err);
+	    transcribe_and_free(Z[smat[K_BIG_P]] + t1, K->P, err);
 	if (smat[K_K] > 0) 
-	    transcribe_and_free(Z[smat[K_K]] + t1, K->K, *err);
+	    transcribe_and_free(Z[smat[K_K]] + t1, K->K, err);
     }
 
-    /* reverted (??) */
-    if (*err != 0 && *err != E_NAN) {
-	ret = 1;
+    if (err != E_NAN) {
+	*errp = err;
+    } else {
+	/* we'll flag E_NAN with a return value of 1 but 
+	   won't count it as a 'true' error */
+	*errp = 0;
     }
 
+#if KDEBUG
+    fprintf(stderr, "user_kalman_run: returning %d, *errp = %d\n",
+	    err, *errp);
+#endif
+
     /* detach matrices */
     K->E = K->V = K->S = NULL;
     K->P = K->K = NULL;
 
-    return ret;    
+    return err;    
 }
 
 /* Copy row @t from @src into @targ; or add row @t of @src to
@@ -3391,7 +3404,6 @@ gretl_matrix *user_kalman_simulate (const gretl_matrix *V,
 
 /**
  * user_kalman_get_loglik:
- * @K: pointer to Kalman struct.
  * 
  * Retrieves the log-likelhood calculated via the last run of 
  * a kalman forecast, if applicable.
@@ -3412,7 +3424,6 @@ double user_kalman_get_loglik (void)
 
 /**
  * user_kalman_get_matrix:
- * @K: pointer to Kalman struct.
  * @idx: identifier for matrix.
  * @err: location to receive error code.
  * 
@@ -3453,7 +3464,6 @@ gretl_matrix *user_kalman_get_matrix (int idx, int *err)
 
 /**
  * user_kalman_get_s2:
- * @K: pointer to Kalman struct.
  * 
  * Retrieves the scale factor, \hat{\sigma}^2, calculated 
  * via the last run of a kalman forecast, if applicable.
@@ -3474,7 +3484,6 @@ double user_kalman_get_s2 (void)
 
 /**
  * user_kalman_get_time_step:
- * @K: pointer to Kalman struct.
  * 
  * Retrieves the time step, t, from the current run of a
  * kalman forecast, if applicable.
