@@ -71,7 +71,7 @@ int kalman_parse_line (const char *line, const double **Z,
 
 int user_kalman_run (const char *E, const char *V, const char *S,
 		     const char *P, const char *G, double **Z,
-		     const DATAINFO *pdinfo, PRN *prn, int *err);
+		     const DATAINFO *pdinfo, PRN *prn, int *errp);
 
 gretl_matrix *user_kalman_smooth (const char *Pname, const char *Uname,
 				  int *err);
