@@ -28,7 +28,7 @@
 
 #include <zlib.h>
 
-#ifndef GRETLCLI
+#ifdef FULL_XML_HEADERS
 # include <libxml/xmlmemory.h>
 # include <libxml/parser.h>
 #endif
@@ -65,7 +65,6 @@
 #define MAXLINE 8192  /* maximum length of command line */
 #define MAXLABEL 128  /* maximum length of descriptive labels for variables */
 #define MAXLEN   512  /* max length of regular "long" strings */
-#define ERRLEN   256  /* max length of libgretl error messages */
 #define MAXDISP   32  /* max length of "display names" for variables */
 #define VNAMELEN  16  /* space allocated for var names (including termination) */
 #define OBSLEN    16  /* space allocated for obs strings (including termination) */
@@ -78,8 +77,9 @@
 # define M_2PI 6.2831853071795864769252864
 #endif
 
-#define LN_2_PI       1.837877066409345
-#define LN_SQRT_2_PI  0.9189385332056725
+#define SQRT_2_PI     2.506628274631000502415765284811
+#define LN_2_PI       1.837877066409345483560659472811
+#define LN_SQRT_2_PI  0.918938533204672741780329736406
 
 #define LISTSEP            (-100)
 #define PMAX_NOT_AVAILABLE 666
@@ -106,7 +106,8 @@ typedef enum {
     GRETL_TYPE_MATRIX_REF,
     GRETL_TYPE_USCALAR,
     GRETL_TYPE_USERIES,
-    GRETL_TYPE_DATE
+    GRETL_TYPE_DATE,
+    GRETL_TYPE_VOID
 } GretlType;
 
 #define gretl_scalar_type(t) (t == GRETL_TYPE_BOOL || \
@@ -223,7 +224,6 @@ typedef unsigned int gretlopt;
 typedef struct VARINFO_ VARINFO;
 typedef struct PANINFO_ PANINFO;
 typedef struct DATAINFO_ DATAINFO;
-typedef struct PATHS_ PATHS;
 typedef struct VMatrix_ VMatrix;
 typedef struct SAMPLE_ SAMPLE;
 typedef struct ARINFO_ ARINFO;
@@ -281,6 +281,7 @@ struct DATAINFO_ {
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
     char *submask;      /* subsampling mask */
+    char *restriction;  /* record of sub-sampling restriction */
 };
 
 /* wrapper for the two main elements of a gretl data set */
@@ -289,26 +290,6 @@ struct DATASET_ {
     double **Z;
 };
 
-struct PATHS_ {
-    char currdir[MAXLEN];
-    char workdir[MAXLEN];
-    char dotdir[MAXLEN];
-    char gretldir[MAXLEN];
-    char helpfile[MAXLEN];
-    char cmd_helpfile[MAXLEN];
-    char cli_helpfile[MAXLEN];
-    char datfile[MAXLEN];
-    char binbase[MAXLEN];
-    char ratsbase[MAXLEN];
-    char gnuplot[MAXLEN];
-    char x12a[MAXLEN];
-    char x12adir[MAXLEN];
-    char tramo[MAXLEN];
-    char tramodir[MAXLEN];
-    char dbhost[32];
-    char pngfont[128];
-};
-
 struct VMatrix_ {
     int ci;
     int dim;
@@ -356,6 +337,7 @@ struct MODEL_ {
     double *sderr;               /* array of estimated std. errors */
     double *uhat;                /* regression residuals */
     double *yhat;                /* fitted values from regression */
+    double *llt;                 /* log-likelihood by observation */
     double *xpx;                 /* X'X matrix, in packed form */
     double *vcv;                 /* VCV matrix for coefficient estimates */
     double ess, tss;             /* Error and Total Sums of Squares */
