@@ -82,12 +82,13 @@ struct set_vars_ {
     double nls_toler;           /* NLS convergence criterion */
     int loop_maxiter;           /* max no. of iterations in non-for loops */
     char delim;                 /* delimiter for CSV data export */
-    int longdigits;             /* digits for printing data in long form */
     int vecm_norm;              /* VECM beta normalization */
     int bfgs_maxiter;           /* max iterations, BFGS */         
     double bfgs_toler;          /* convergence tolerance, BFGS */
+    int bfgs_verbskip;          /* BFGS: show one in n iterations  */
     int bhhh_maxiter;           /* max iterations, BHHH */          
     double bhhh_toler;          /* convergence tolerance, BHHH */
+    int lbfgs_mem;              /* memory span for L-BFGS-B */
     int garch_vcv;              /* GARCH vcv variant */
     int garch_robust_vcv;       /* GARCH vcv variant, robust estimation */
     int arma_vcv;               /* ARMA vcv variant */
@@ -98,6 +99,7 @@ struct set_vars_ {
     gretl_matrix *initvals;     /* parameter initializer */
     struct robust_opts ropts;   /* robust standard error options */
     char shelldir[MAXLEN];      /* working dir for shell commands */
+    char csv_na[8];             /* representation of NA in CSV output */
 };
 
 #define ECHO "echo"
@@ -105,7 +107,6 @@ struct set_vars_ {
 #define WARNINGS "warnings"
 #define GRETL_DEBUG "debug"
 #define BLAS_NMK_MIN "blas_nmk_min"
-#define PROTECT_LISTS "protect_lists"
 
 #define libset_boolvar(s) (!strcmp(s, ECHO) || \
                            !strcmp(s, MESSAGES) || \
@@ -121,9 +122,10 @@ struct set_vars_ {
 			   !strcmp(s, SHELL_OK) || \
 			   !strcmp(s, USE_CWD) || \
 			   !strcmp(s, USE_FCP) || \
-                           !strcmp(s, PROTECT_LISTS) || \
                            !strcmp(s, VERBOSE_INCLUDE) || \
-                           !strcmp(s, SKIP_MISSING))
+                           !strcmp(s, SKIP_MISSING) || \
+			   !strcmp(s, R_FUNCTIONS) || \
+			   !strcmp(s, R_LIB))
 
 #define libset_double(s) (!strcmp(s, BFGS_TOLER) || \
 			  !strcmp(s, BHHH_TOLER) || \
@@ -132,13 +134,14 @@ struct set_vars_ {
 			  !strcmp(s, QS_BANDWIDTH))
 
 #define libset_int(s) (!strcmp(s, BFGS_MAXITER) || \
+		       !strcmp(s, BFGS_VERBSKIP) || \
 		       !strcmp(s, BHHH_MAXITER) || \
+		       !strcmp(s, LBFGS_MEM) || \
                        !strcmp(s, BKBP_K) || \
 		       !strcmp(s, BOOTREP) || \
 		       !strcmp(s, HAC_KERNEL) || \
                        !strcmp(s, HC_VERSION) || \
 		       !strcmp(s, HORIZON) || \
-		       !strcmp(s, LONGDIGITS) || \
 		       !strcmp(s, LOOP_MAXITER) || \
                        !strcmp(s, RQ_MAXITER) || \
 		       !strcmp(s, VECM_NORM) || \
@@ -148,11 +151,15 @@ struct set_vars_ {
 /* global state */
 set_vars *state;
 static int gretl_debug;
-static int protect_lists = 1;
 static int user_mp_bits;
+static int R_functions;
+static int R_lib = 1;
+static char include_path[MAXLEN];
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
+static int real_libset_read_script (const char *fname,
+				    PRN *prn);
 
 static void robust_opts_init (struct robust_opts *r)
 {
@@ -278,13 +285,16 @@ static const char *libset_option_string (const char *s)
     }
 }
 
-static void print_initvals (const gretl_matrix *ivals, PRN *prn)
+static void print_initvals (const gretl_matrix *ivals, PRN *prn,
+			    gretlopt opt)
 {
-    if (ivals == NULL) {
-	pputs(prn, " initvals = auto\n");
-    } else {
-	gretl_matrix_print_to_prn(ivals, " initvals =", prn);
-    }
+    if (opt & OPT_D) {
+	if (ivals == NULL) {
+	    pputs(prn, " initvals = auto\n");
+	} else {
+	    gretl_matrix_print_to_prn(ivals, " initvals =", prn);
+	}
+    } 
 }
 
 /* check_for_state() returns non-zero if the program options
@@ -330,12 +340,13 @@ static void state_vars_copy (set_vars *sv)
     sv->rq_maxiter = state->rq_maxiter;
     sv->nls_toler = state->nls_toler;
     sv->delim = state->delim; 
-    sv->longdigits = state->longdigits; 
     sv->vecm_norm = state->vecm_norm;
     sv->bfgs_maxiter = state->bfgs_maxiter;
     sv->bfgs_toler = state->bfgs_toler;
+    sv->bfgs_verbskip = state->bfgs_verbskip;
     sv->bhhh_maxiter = state->bhhh_maxiter;
     sv->bhhh_toler = state->bhhh_toler;
+    sv->lbfgs_mem = state->lbfgs_mem;
     sv->garch_vcv = state->garch_vcv;
     sv->garch_robust_vcv = state->garch_robust_vcv;
     sv->bkbp_k = state->bkbp_k;
@@ -344,6 +355,7 @@ static void state_vars_copy (set_vars *sv)
 
     sv->initvals = gretl_matrix_copy(state->initvals);
     strcpy(sv->shelldir, state->shelldir);
+    strcpy(sv->csv_na, state->csv_na);
 
     robust_opts_copy(&sv->ropts);
 }
@@ -363,14 +375,15 @@ static void state_vars_init (set_vars *sv)
     sv->loop_maxiter = 250;
     sv->rq_maxiter = 1000;
     sv->delim = UNSET_INT;
-    sv->longdigits = 10;
     sv->vecm_norm = NORM_PHILLIPS;
     sv->initvals = NULL;
 
     sv->bfgs_maxiter = -1;
     sv->bfgs_toler = NADBL;
+    sv->bfgs_verbskip = 1;
     sv->bhhh_maxiter = 500;
     sv->bhhh_toler = NADBL;
+    sv->lbfgs_mem = 8;
     sv->garch_vcv = VCV_UNSET;
     sv->arma_vcv = VCV_HESSIAN;
     sv->garch_robust_vcv = VCV_UNSET;
@@ -380,6 +393,7 @@ static void state_vars_init (set_vars *sv)
     sv->bkbp_p1 = UNSET_INT;
 
     *sv->shelldir = '\0';
+    *sv->csv_na = '\0';
 
     robust_opts_init(&sv->ropts);
 }
@@ -519,11 +533,6 @@ int gretl_debugging_on (void)
     return gretl_debug;
 }
 
-int lists_protected (void)
-{
-    return protect_lists;
-}
-
 #define DEFAULT_MP_BITS 256
 #define mp_bits_ok(b) (b >= 256 && b <= 8192)
 
@@ -536,7 +545,7 @@ void set_mp_bits (int b)
 
 int get_mp_bits (void)
 {
-    if (user_mp_bits >= 256) {
+    if (user_mp_bits >= DEFAULT_MP_BITS) {
 	return user_mp_bits;
     } else {
 	char *s = getenv("GRETL_MP_BITS");
@@ -652,7 +661,7 @@ static const char *hac_lag_string (void)
 
 static int parse_hac_lag_variant (const char *s)
 {
-    int err = 1;
+    int err = E_DATA;
 
     if (!strcmp(s, "nw1")) {
 	state->ropts.auto_lag = AUTO_LAG_STOCK_WATSON;
@@ -741,7 +750,7 @@ static int libset_get_scalar (const char *var, const char *arg,
     if (gretl_is_scalar(arg)) {
 	x = gretl_scalar_get_value(arg);
     } else {
-	sprintf(gretl_errmsg, "'%s': not a scalar", arg);
+	gretl_errmsg_sprintf("'%s': not a scalar", arg);
 	return E_UNKVAR;
     }
 
@@ -772,7 +781,7 @@ static int parse_hc_variant (const char *s)
 	s += 2;
     }
 
-    for (i=0; i<4; i++) {
+    for (i=0; hc_version_strs[i] != NULL; i++) {
 	if (!strcmp(s, hc_version_strs[i])) {
 	    state->ropts.hc_version = i;
 	    return 0;
@@ -787,47 +796,54 @@ static int parse_hc_variant (const char *s)
     return 1;
 }
 
-static int parse_libset_int_code (const char *s, 
-				  const char *code)
+static int parse_libset_int_code (const char *key, 
+				  const char *val)
 {
-    int i;
+    int i, err = E_DATA;
 
-    if (!strcmp(s, HC_VERSION)) {
-	return parse_hc_variant(code);
-    } else if (!strcmp(s, HAC_LAG)) {
-	return parse_hac_lag_variant(code);
-    } else if (!strcmp(s, GARCH_VCV)) {
+    if (!strcmp(key, HC_VERSION)) {
+	err = parse_hc_variant(val);
+    } else if (!strcmp(key, HAC_LAG)) {
+	err = parse_hac_lag_variant(val);
+    } else if (!strcmp(key, GARCH_VCV)) {
 	for (i=0; i<VCV_MAX; i++) {
-	    if (!strcmp(code, garch_vcv_strs[i])) {
+	    if (!strcmp(val, garch_vcv_strs[i])) {
 		state->garch_vcv = i;
-		return 0;
+		err = 0;
+		break;
 	    }
 	}
-    } else if (!strcmp(s, ARMA_VCV)) {
-	if (!strcmp(code, "op")) {
+    } else if (!strcmp(key, ARMA_VCV)) {
+	if (!strcmp(val, "op")) {
 	    state->arma_vcv = VCV_OP;
-	    return 0;
-	} else if (!strcmp(code, "hessian")) {
+	    err = 0;
+	} else if (!strcmp(val, "hessian")) {
 	    state->arma_vcv = VCV_HESSIAN;
-	    return 0;
+	    err = 0;
 	}
-    } else if (!strcmp(s, HAC_KERNEL)) {
+    } else if (!strcmp(key, HAC_KERNEL)) {
 	for (i=0; i<KERNEL_MAX; i++) {
-	    if (!strcmp(code, hac_kernel_strs[i])) {
+	    if (!strcmp(val, hac_kernel_strs[i])) {
 		state->ropts.hkern = i;
-		return 0;
+		err = 0;
+		break;
 	    }
 	}
-    } else if (!strcmp(s, VECM_NORM)) {
+    } else if (!strcmp(key, VECM_NORM)) {
 	for (i=0; i<NORM_MAX; i++) {
-	    if (!strcmp(code, vecm_norm_strs[i])) {
+	    if (!strcmp(val, vecm_norm_strs[i])) {
 		state->vecm_norm = i;
-		return 0;
+		err = 0;
+		break;
 	    }
 	}
     }
 
-    return 1;
+    if (err) {
+	gretl_errmsg_sprintf("%s: invalid value '%s'\n", key, val);
+    }
+
+    return err;
 }	
 
 void set_xsect_hccme (const char *s)
@@ -899,6 +915,11 @@ static int set_line_width (const char *s0, const char *s1,
 {
     int v, w, err = 0;
 
+    if (pdinfo == NULL) {
+	/* treat as no-op */
+	return 0;
+    }
+
     if (!isdigit((unsigned char) *s1)) {
 	return 1;
     }
@@ -1079,20 +1100,20 @@ void set_workdir_callback (int (*callback)())
 
 static int set_workdir (const char *s)
 {
+    int err = 0;
+
     if (gretl_function_depth() > 0) {
 	gretl_errmsg_set("set workdir: cannot be done inside a function");
 	return 1;
     }
 
-    if (workdir_callback == NULL) {
-	return E_DATA;
-    }
-
     /* skip past "set workdir" and space */
     s += 11;
     s += strspn(s, " ");
 
-    if (*s != '\0') {
+    if (*s == '\0') {
+	err = E_DATA;
+    } else {
 	char workdir[MAXLEN];
 	int n = 0;
 
@@ -1101,35 +1122,12 @@ static int set_workdir (const char *s)
 	} else {
 	    n = sscanf(s, "%511s", workdir);
 	}
-	return (*workdir_callback)(workdir);
-    } else {
-	return E_DATA;
-    }
-}
-
-static int parse_set_plotfile (const char *s)
-{
-    char *fname;
-    int err = 0;
-
-    while (isspace((unsigned char) *s)) {
-	s++;
-    }
-
-    /* now skip two words, "set" and "plotfile" */
-    s += strcspn(s, " ");
-    s += strspn(s, " ");
-    s += strcspn(s, " ");
-    s += strspn(s, " ");
-
-    fname = gretl_strdup(s);
-    if (fname != NULL) {
-	tailstrip(fname);
-	set_gretl_plotfile(fname);
-	free(fname);
-    } else {
-	err = E_ALLOC;
-    }
+	if (workdir_callback != NULL) {
+	    err = (*workdir_callback)(workdir);
+	} else {
+	    err = set_gretl_work_dir(workdir);
+	}
+    } 
 
     return err;
 }
@@ -1162,9 +1160,16 @@ static const char *arg_from_delim (char c)
     return "unset";
 }
 
-static void libset_print_bool (const char *s, PRN *prn)
+static void libset_print_bool (const char *s, PRN *prn,
+			       gretlopt opt)
 {
-    pprintf(prn, " %s = %d\n", s, libset_get_bool(s));
+    int v = libset_get_bool(s);
+
+    if (opt & OPT_D) {
+	pprintf(prn, " %s = %d\n", s, v);
+    } else {
+	pprintf(prn, "set %s %s\n", s, v? "on" : "off");
+    }
 }
 
 #define coded_intvar(s) (!strcmp(s, GARCH_VCV) || \
@@ -1183,121 +1188,205 @@ const char *intvar_code_string (const char *s)
     }
 }
 
-static void libset_print_int (const char *s, PRN *prn)
+static void libset_print_int (const char *s, PRN *prn,
+			      gretlopt opt)
 {
     if (coded_intvar(s)) {
-	pprintf(prn, " %s = %s\n", s, intvar_code_string(s));
+	if (opt & OPT_D) {
+	    pprintf(prn, " %s = %s\n", s, intvar_code_string(s));
+	} else {
+	    pprintf(prn, "set %s %s\n", s, intvar_code_string(s));
+	}
     } else {
 	int k = libset_get_int(s);
 
-	if (is_unset(k)) {
-	    pprintf(prn, " %s = auto\n", s);
-	} else {
-	    pprintf(prn, " %s = %d\n", s, k);
+	if (opt & OPT_D) {
+	    if (is_unset(k)) {
+		pprintf(prn, " %s = auto\n", s);
+	    } else {
+		pprintf(prn, " %s = %d\n", s, k);
+	    }
+	} else if (!is_unset(k)) {
+	    pprintf(prn, "set %s %d\n", s, k);
 	}
     }
 }
 
-static void libset_print_double (const char *s, PRN *prn)
+static void libset_print_double (const char *s, PRN *prn,
+				 gretlopt opt)
 {
     double x = libset_get_double(s);
 
-    if (na(x)) {
-	pprintf(prn, " %s = auto\n", s);
-    } else {
-	pprintf(prn, " %s = %g\n", s, x);
+    if (opt & OPT_D) {
+	if (na(x)) {
+	    pprintf(prn, " %s = auto\n", s);
+	} else {
+	    pprintf(prn, " %s = %.15g\n", s, x);
+	}
+    } else if (!na(x)) {
+	pprintf(prn, "set %s %.15g\n", s, x);
     }
 }
 
-static void libset_header (char *s, PRN *prn) 
+static const char *normal_rand_string (void)
 {
-    pputs(prn, "\n --- ");
-    pputs(prn, s);
-    pputs(prn, " ---\n");
+    if (gretl_rand_get_box_muller()) {
+	return "box-muller";
+    } else {
+	return "ziggurat";
+    }
 }
 
-static int display_settings (PRN *prn)
+static int set_normal_rand (const char *s)
 {
-    pputs(prn, _("Variables that can be set using \"set\""));
-    pputs(prn, " (");
-    pputs(prn, _("\"help set\" for details"));
-    pputs(prn, "):\n");
+    int err = E_PARSE;
 
-    libset_header(_("Program interaction and behavior"), prn);
+    if (!strcmp(s, "box-muller")) {
+	gretl_rand_set_box_muller(1);
+	err = 0;
+    } else if (!strcmp(s, "ziggurat")) {
+	gretl_rand_set_box_muller(0);
+	err = 0;
+    }    
 
-    pprintf(prn, " csv_delim = %s\n", arg_from_delim(state->delim));
+    return err;
+}
 
-    libset_print_bool(ECHO, prn);
-    libset_print_bool(FORCE_DECP, prn);
-    libset_print_bool(HALT_ON_ERR, prn);
-    libset_print_int(LONGDIGITS, prn);
-    libset_print_int(LOOP_MAXITER, prn);
-    libset_print_bool(MAX_VERBOSE, prn);
-    libset_print_bool(MESSAGES, prn);
-    libset_print_bool(WARNINGS, prn);
-    libset_print_int(GRETL_DEBUG, prn);
-    libset_print_int(BLAS_NMK_MIN, prn);
-    libset_print_bool(SHELL_OK, prn);
-
-    if (*state->shelldir) {
-	pprintf(prn, " shelldir = '%s'\n", state->shelldir);
+static void libset_header (char *s, PRN *prn, gretlopt opt) 
+{
+    if (opt & OPT_D) {
+	pputs(prn, "\n --- ");
+	pputs(prn, _(s));
+	pputs(prn, " ---\n");
     } else {
-	pputs(prn, " shelldir = unset\n");
+	pprintf(prn, "# %s\n", s);
+    }
+}
+
+/* print_settings: use OPT_D for "display", otherwise
+   this gives script-type output */
+
+static int print_settings (PRN *prn, gretlopt opt)
+{
+    if (opt & OPT_D) {
+	pputs(prn, _("Variables that can be set using \"set\""));
+	pputs(prn, " (");
+	pputs(prn, _("\"help set\" for details"));
+	pputs(prn, "):\n");
     }
 
-    libset_print_bool(USE_CWD, prn);
-    libset_print_bool(PROTECT_LISTS, prn);
-    libset_print_bool(VERBOSE_INCLUDE, prn);
-    libset_print_bool(SKIP_MISSING, prn);
+    libset_header(N_("Program interaction and behavior"), prn, opt);
 
-    libset_header(_("Numerical methods"), prn);
-
-    libset_print_int(BFGS_MAXITER, prn);
-    libset_print_double(BFGS_TOLER, prn);
-    libset_print_int(BHHH_MAXITER, prn);
-    libset_print_double(BHHH_TOLER, prn);
-    libset_print_int(RQ_MAXITER, prn);
-    print_initvals(state->initvals, prn);
-    libset_print_bool(USE_LBFGS, prn);
-    libset_print_double(NLS_TOLER, prn);
-    libset_print_bool(USE_SVD, prn);
-    libset_print_bool(USE_FCP, prn);
-
-    libset_header(_("Random number generation"), prn);
-
-    pprintf(prn, " seed = %u\n", gretl_rand_get_seed());
-
-    libset_header(_("Robust estimation"), prn);
-
-    libset_print_int(BOOTREP, prn);
-    libset_print_int(GARCH_VCV, prn);
-    libset_print_int(ARMA_VCV, prn);
-    libset_print_bool(FORCE_HC, prn);
-    libset_print_int(HAC_LAG, prn);
-    libset_print_int(HAC_KERNEL, prn);
-    libset_print_bool(PREWHITEN, prn);
-    libset_print_int(HC_VERSION, prn);
-    libset_print_bool(PCSE, prn);
-    libset_print_double(QS_BANDWIDTH, prn);
-
-    libset_header(_("Filtering"), prn);
-
-    if (is_unset(state->bkbp_p0) ||
-	is_unset(state->bkbp_p1)) {
-	pputs(prn, " bkbp_limits = auto\n");
+    if (opt & OPT_D) {
+	pprintf(prn, " csv_delim = %s\n", arg_from_delim(state->delim));
+	pprintf(prn, " csv_na = %s\n", get_csv_na_string());
     } else {
-	pprintf(prn, " bkbp_limits = (%d, %d)\n", 
-		state->bkbp_p0, 
-		state->bkbp_p1);
+	const char *dl = arg_from_delim(state->delim);
+
+	if (strcmp(dl, "unset")) {
+	    pprintf(prn, "set csv_delim %s\n", arg_from_delim(state->delim));
+	}
+	pprintf(prn, "set csv_na %s\n", get_csv_na_string());
+    }	
+
+    libset_print_bool(ECHO, prn, opt);
+    libset_print_bool(FORCE_DECP, prn, opt);
+    libset_print_bool(HALT_ON_ERR, prn, opt);
+    libset_print_int(LOOP_MAXITER, prn, opt);
+    libset_print_bool(MAX_VERBOSE, prn, opt);
+    libset_print_int(BFGS_VERBSKIP, prn, opt);
+    libset_print_bool(MESSAGES, prn, opt);
+    libset_print_bool(WARNINGS, prn, opt);
+    libset_print_int(GRETL_DEBUG, prn, opt);
+    libset_print_int(BLAS_NMK_MIN, prn, opt);
+
+    if (opt & OPT_D) {
+	libset_print_bool(SHELL_OK, prn, opt);
+	if (*state->shelldir) {
+	    pprintf(prn, " shelldir = '%s'\n", state->shelldir);
+	} else {
+	    pputs(prn, " shelldir = unset\n");
+	}
+    } 
+
+    libset_print_bool(USE_CWD, prn, opt);
+    libset_print_bool(VERBOSE_INCLUDE, prn, opt);
+    libset_print_bool(SKIP_MISSING, prn, opt);
+
+    libset_print_bool(R_LIB, prn, opt);
+    libset_print_bool(R_FUNCTIONS, prn, opt);
+
+    if (opt & OPT_D) {
+	if (*include_path) {
+	    pprintf(prn, " include_path = '%s'\n", include_path);
+	} else {
+	    pputs(prn, " include_path = unset\n");
+	}
+    } else if (*include_path) {
+	pprintf(prn, "set include_path \"%s\"\n", include_path);
     }
 
-    libset_print_int(BKBP_K, prn);
-    libset_print_double(HP_LAMBDA, prn);
+    libset_header(N_("Numerical methods"), prn, opt);
 
-    libset_header(_("Time series"), prn);
+    libset_print_int(BFGS_MAXITER, prn, opt);
+    libset_print_double(BFGS_TOLER, prn, opt);
+    libset_print_int(BHHH_MAXITER, prn, opt);
+    libset_print_double(BHHH_TOLER, prn, opt);
+    libset_print_int(RQ_MAXITER, prn, opt);
+    print_initvals(state->initvals, prn, opt);
+    libset_print_bool(USE_LBFGS, prn, opt);
+    libset_print_int(LBFGS_MEM, prn, opt);
+    libset_print_double(NLS_TOLER, prn, opt);
+    libset_print_bool(USE_SVD, prn, opt);
+    libset_print_bool(USE_FCP, prn, opt);
 
-    libset_print_int(HORIZON, prn);
-    libset_print_int(VECM_NORM, prn);
+    libset_header(N_("Random number generation"), prn, opt);
+
+    if (opt & OPT_D) {
+	pprintf(prn, " seed = %u\n", gretl_rand_get_seed());
+	pprintf(prn, " normal_rand = %s\n", normal_rand_string());
+    } else {
+	pprintf(prn, "set seed %u\n", gretl_rand_get_seed());
+	pprintf(prn, "set normal_rand %s\n", normal_rand_string());
+    }
+
+    libset_header(N_("Robust estimation"), prn, opt);
+
+    libset_print_int(BOOTREP, prn, opt);
+    libset_print_int(GARCH_VCV, prn, opt);
+    libset_print_int(ARMA_VCV, prn, opt);
+    libset_print_bool(FORCE_HC, prn, opt);
+    libset_print_int(HAC_LAG, prn, opt);
+    libset_print_int(HAC_KERNEL, prn, opt);
+    libset_print_bool(PREWHITEN, prn, opt);
+    libset_print_int(HC_VERSION, prn, opt);
+    libset_print_bool(PCSE, prn, opt);
+    libset_print_double(QS_BANDWIDTH, prn, opt);
+
+    libset_header(N_("Filtering"), prn, opt);
+
+    if (opt & OPT_D) {
+	if (is_unset(state->bkbp_p0) ||
+	    is_unset(state->bkbp_p1)) {
+	    pputs(prn, " bkbp_limits = auto\n");
+	} else {
+	    pprintf(prn, " bkbp_limits = (%d, %d)\n", 
+		    state->bkbp_p0, 
+		    state->bkbp_p1);
+	}
+    } else if (!is_unset(state->bkbp_p0) &&
+	       !is_unset(state->bkbp_p1)) {
+	pprintf(prn, "set bkbp_limits %d %d\n", 
+		state->bkbp_p0, state->bkbp_p1);
+    }	
+
+    libset_print_int(BKBP_K, prn, opt);
+    libset_print_double(HP_LAMBDA, prn, opt);
+
+    libset_header(N_("Time series"), prn, opt);
+
+    libset_print_int(HORIZON, prn, opt);
+    libset_print_int(VECM_NORM, prn, opt);
 
     pputc(prn, '\n');
     
@@ -1342,7 +1431,7 @@ libset_query_settings (const char *s, PRN *prn)
 	}
     } else if (!strcmp(s, "seed")) {
 	pprintf(prn, "%s: unsigned int, currently %u\n",
-		s, state->seed);
+		s, state->seed ? state->seed : gretl_rand_get_seed());
     } else if (!strcmp(s, "csv_delim")) {
 	pprintf(prn, "%s: named character, currently \"%s\"\n", s,
 		arg_from_delim(state->delim));
@@ -1369,7 +1458,27 @@ libset_query_settings (const char *s, PRN *prn)
 #define boolean_off(s) (!strcmp(s, "off") || !strcmp(s, "0") || \
                         !strcmp(s, "false"))
 
-int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
+static int write_or_read_settings (gretlopt opt, PRN *prn)
+{
+    int err = incompatible_options(opt, (OPT_T | OPT_F));
+
+    if (!err) {
+	const char *fname = get_optval_string(SET, opt);
+
+	if (fname == NULL) {
+	    err = E_DATA;
+	} else if (opt == OPT_T) {
+	    err = libset_write_script(fname);
+	} else {
+	    err = real_libset_read_script(fname, prn);
+	}
+    }
+
+    return err;
+}
+
+int execute_set_line (const char *line, DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn)
 {
     char setobj[32], setarg[32], setarg2[32];
     int k, nw, err = E_PARSE;
@@ -1377,19 +1486,21 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 
     check_for_state();
 
+    if (opt != OPT_NONE) {
+	return write_or_read_settings(opt, prn);
+    }
+
     *setobj = *setarg = *setarg2 = '\0';
 
     nw = sscanf(line, "%*s %31s %31s %31s", setobj, setarg, setarg2);
 
     if (nw <= 0) {
-	return display_settings(prn);
+	return print_settings(prn, OPT_D);
     }
 
     /* specials which need the whole line */
     if (nw > 1) {
-	if (!strcmp(setobj, "plotfile")) {
-	    return parse_set_plotfile(line);
-	} else if (!strcmp(setobj, "initvals")) {
+	if (!strcmp(setobj, "initvals")) {
 	    return set_initvals(line, prn);
 	} else if (!strcmp(setobj, "shelldir")) {
 	    return set_shelldir(line);
@@ -1397,6 +1508,9 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 	    return set_codevars(line);
 	} else if (!strcmp(setobj, "workdir")) {
 	    return set_workdir(line);
+	} else if (!strcmp(setobj, "include_path")) {
+	    set_include_path(line);
+	    return 0;
 	}
     }
 
@@ -1411,6 +1525,11 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 	    return libset_query_settings(setobj, prn);
 	}
     } else if (nw == 2) {
+	if (!strcmp(setobj, "csv_na")) {
+	    set_csv_na_string(setarg);
+	    return 0;
+	} 	    
+
 	lower(setarg);
 
 	if (libset_boolvar(setobj)) {
@@ -1418,11 +1537,9 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 		pprintf(prn, "You can only set this variable "
 			"via the gretl GUI\n");
 	    } else if (boolean_on(setarg)) {
-		libset_set_bool(setobj, 1);
-		err = 0;
+		err = libset_set_bool(setobj, 1);
 	    } else if (boolean_off(setarg)) {
-		libset_set_bool(setobj, 0);
-		err = 0;
+		err = libset_set_bool(setobj, 0);
 	    }
 	} else if (libset_double(setobj)) {
 	    if (default_ok(setobj) && default_str(setarg)) {
@@ -1451,6 +1568,8 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
 		}
 		state->seed = k;
 	    }
+	} else if (!strcmp(setobj, "normal_rand")) {
+	    err = set_normal_rand(setarg);
 	} else if (!strcmp(setobj, HORIZON)) {
 	    /* horizon for VAR impulse responses */
 	    if (!strcmp(setarg, "auto")) {
@@ -1487,41 +1606,60 @@ int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn)
     return err;
 }
 
-double libset_get_double (const char *s)
+double libset_get_double (const char *key)
 {
     if (check_for_state()) {
-	return 1;
+	return NADBL;
     }
 
-    if (!strcmp(s, QS_BANDWIDTH)) {
+    if (!strcmp(key, QS_BANDWIDTH)) {
 	if (!na(state->ropts.qsband) && state->ropts.qsband > 0) {
 	    return state->ropts.qsband;
 	} else {
 	    /* what's a sensible default here? */
 	    return 2.0;
 	}
-    } else if (!strcmp(s, NLS_TOLER)) {
+    } else if (!strcmp(key, NLS_TOLER)) {
 	if (na(state->nls_toler)) {
-	    state->nls_toler = get_default_nls_toler();
+	    return get_default_nls_toler();
+	} else {
+	    return state->nls_toler;
 	}
-	return state->nls_toler;
-    } else if (!strcmp(s, BHHH_TOLER)) {
-	return state->bhhh_toler;
-    } else if (!strcmp(s, BFGS_TOLER)) {
+    } else if (!strcmp(key, BHHH_TOLER)) {
+	if (na(state->bhhh_toler)) {
+	    return 1.0e-6;
+	} else {
+	    return state->bhhh_toler;
+	}
+    } else if (!strcmp(key, BFGS_TOLER)) {
 	if (na(state->bfgs_toler)) {
-	    state->bfgs_toler = get_default_nls_toler();
+	    return get_default_nls_toler();
+	} else {
+	    return state->bfgs_toler;
 	}
-	return state->bfgs_toler;
-    } else if (!strcmp(s, HP_LAMBDA)) {
+    } else if (!strcmp(key, HP_LAMBDA)) {
 	return state->hp_lambda;
     } else {
 	fprintf(stderr, "libset_get_double: unrecognized "
-		"variable '%s'\n", s);	
-	return 1;
+		"variable '%s'\n", key);	
+	return 0;
     }
 }
 
-int libset_set_double (const char *s, double x)
+double libset_get_user_tolerance (const char *key)
+{
+    if (!strcmp(key, NLS_TOLER)) {
+	return state->nls_toler;
+    } else if (!strcmp(key, BHHH_TOLER)) {
+	return state->bhhh_toler;
+    } else if (!strcmp(key, BFGS_TOLER)) {
+	return state->bfgs_toler;
+    } else {
+	return NADBL;
+    }
+}
+
+int libset_set_double (const char *key, double val)
 {
     int err = 0;
 
@@ -1530,70 +1668,72 @@ int libset_set_double (const char *s, double x)
     }
 
     /* all the libset double vals must be positive */
-    if (x <= 0.0) {
+    if (val <= 0.0) {
 	return E_DATA;
     }
 
-    if (!strcmp(s, QS_BANDWIDTH)) {
-	state->ropts.qsband = x;
-    } else if (!strcmp(s, NLS_TOLER)) {
-	state->nls_toler = x;
-    } else if (!strcmp(s, BHHH_TOLER)) {
-	state->bhhh_toler = x;
-    } else if (!strcmp(s, BFGS_TOLER)) {
-	state->bfgs_toler = x;
-    } else if (!strcmp(s, HP_LAMBDA)) {
-	state->hp_lambda = x;
+    if (!strcmp(key, QS_BANDWIDTH)) {
+	state->ropts.qsband = val;
+    } else if (!strcmp(key, NLS_TOLER)) {
+	state->nls_toler = val;
+    } else if (!strcmp(key, BHHH_TOLER)) {
+	state->bhhh_toler = val;
+    } else if (!strcmp(key, BFGS_TOLER)) {
+	state->bfgs_toler = val;
+    } else if (!strcmp(key, HP_LAMBDA)) {
+	state->hp_lambda = val;
     } else {
 	fprintf(stderr, "libset_set_double: unrecognized "
-		"variable '%s'\n", s);	
+		"variable '%s'\n", key);	
 	err = E_UNKVAR;
     }
 
     return err;
 }
 
-int libset_get_int (const char *s)
+int libset_get_int (const char *key)
 {
     if (check_for_state()) {
 	return 0;
     }
 
-    if (!strcmp(s, BFGS_MAXITER)) {
+    if (!strcmp(key, BFGS_MAXITER)) {
 	return state->bfgs_maxiter;
-    } else if (!strcmp(s, BHHH_MAXITER)) {
+    } else if (!strcmp(key, BHHH_MAXITER)) {
 	return state->bhhh_maxiter;
-    } else if (!strcmp(s, RQ_MAXITER)) {
+    } else if (!strcmp(key, RQ_MAXITER)) {
 	return state->rq_maxiter;
-    } else if (!strcmp(s, BKBP_K)) {
+    } else if (!strcmp(key, LBFGS_MEM)) {
+	return state->lbfgs_mem;
+    } else if (!strcmp(key, BKBP_K)) {
 	return state->bkbp_k;
-    } else if (!strcmp(s, BOOTREP)) {
+    } else if (!strcmp(key, BOOTREP)) {
 	return state->bootrep;
-    } else if (!strcmp(s, GARCH_VCV)) {
+    } else if (!strcmp(key, GARCH_VCV)) {
 	return state->garch_vcv;
-    } else if (!strcmp(s, GARCH_ROBUST_VCV)) {
+    } else if (!strcmp(key, GARCH_ROBUST_VCV)) {
 	return state->garch_robust_vcv;
-    } else if (!strcmp(s, ARMA_VCV)) {
+    } else if (!strcmp(key, ARMA_VCV)) {
 	return state->arma_vcv;
-    } else if (!strcmp(s, HAC_KERNEL)) {
+    } else if (!strcmp(key, HAC_KERNEL)) {
 	return state->ropts.hkern;
-    } else if (!strcmp(s, HC_VERSION)) {
+    } else if (!strcmp(key, HC_VERSION)) {
 	return state->ropts.hc_version;
-    } else if (!strcmp(s, HORIZON)) {
+    } else if (!strcmp(key, HORIZON)) {
 	return state->horizon;
-    } else if (!strcmp(s, LONGDIGITS)) {
-	return state->longdigits;
-    } else if (!strcmp(s, LOOP_MAXITER)) {
+    } else if (!strcmp(key, LOOP_MAXITER)) {
 	return state->loop_maxiter;
-    } else if (!strcmp(s, VECM_NORM)) {
+    } else if (!strcmp(key, VECM_NORM)) {
 	return state->vecm_norm;
-    } else if (!strcmp(s, GRETL_DEBUG)) {
+    } else if (!strcmp(key, GRETL_DEBUG)) {
 	return gretl_debug;
-    } else if (!strcmp(s, BLAS_NMK_MIN)) {
+    } else if (!strcmp(key, BLAS_NMK_MIN)) {
 	return get_blas_nmk_min();
+    } else if (!strcmp(key, BFGS_VERBSKIP)) {
+	return state->bfgs_verbskip;
     } else {
 	fprintf(stderr, "libset_get_int: unrecognized "
-		"variable '%s'\n", s);	
+		"variable '%s'\n", key);	
 	return 0;
     }
 }
@@ -1604,14 +1744,21 @@ static int intvar_min_max (const char *s, int *min, int *max,
     *max = 100000;
 
     if (!strcmp(s, BFGS_MAXITER)) {
-	*min = 1;
+	*min = 0;
 	*var = &state->bfgs_maxiter;
+    } else if (!strcmp(s, BFGS_VERBSKIP)) {
+	*min = 1;
+	*var = &state->bfgs_verbskip;
     } else if (!strcmp(s, BHHH_MAXITER)) {
 	*min = 1;
 	*var = &state->bhhh_maxiter;
     } else if (!strcmp(s, RQ_MAXITER)) {
 	*min = 1;
 	*var = &state->rq_maxiter;
+    } else if (!strcmp(s, LBFGS_MEM)) {
+	*min = 3;
+	*max = 20;
+	*var = &state->lbfgs_mem;
     } else if (!strcmp(s, BKBP_K)) {
 	*min = 1;
 	*var = &state->bkbp_k;
@@ -1628,10 +1775,6 @@ static int intvar_min_max (const char *s, int *min, int *max,
     } else if (!strcmp(s, HORIZON)) {
 	*min = 1;
 	*var = &state->horizon;
-    } else if (!strcmp(s, LONGDIGITS)) {
-	*min = 1;
-	*max = 21;
-	*var = &state->longdigits;
     } else if (!strcmp(s, LOOP_MAXITER)) {
 	*min = 1;
 	*var = &state->loop_maxiter;
@@ -1646,13 +1789,13 @@ static int intvar_min_max (const char *s, int *min, int *max,
     } else {
 	fprintf(stderr, "libset_set_int: unrecognized "
 		"variable '%s'\n", s);	
-	return E_DATA;
+	return E_UNKVAR;
     }
 
     return 0;
 }
 
-int libset_set_int (const char *s, int k)
+int libset_set_int (const char *key, int val)
 {
     int min = 0, max = 0;
     int *ivar = NULL;
@@ -1662,42 +1805,24 @@ int libset_set_int (const char *s, int k)
 	return 1;
     }
 
-    if (!strcmp(s, BLAS_NMK_MIN)) {
-	set_blas_nmk_min(k);
+    if (!strcmp(key, BLAS_NMK_MIN)) {
+	set_blas_nmk_min(val);
 	return 0;
     }
 
-    err = intvar_min_max(s, &min, &max, &ivar);
+    err = intvar_min_max(key, &min, &max, &ivar);
 
     if (!err) {
-	if (k < min || k >= max || ivar == NULL) {
+	if (val < min || val >= max || ivar == NULL) {
 	    err = E_DATA;
 	} else {
-	    *ivar = k;
+	    *ivar = val;
 	}
     }
 
     return err;
 }
 
-#ifndef WIN32
-static int read_cli_shell_status (void)
-{
-    char shellstamp[FILENAME_MAX];
-    FILE *fp;
-    int ok = 0;
-
-    sprintf(shellstamp, "%s.gretl_shell_stamp", gretl_dot_dir());
-    fp = fopen(shellstamp, "r");
-    if (fp != NULL) {
-	ok = 1;
-	fclose(fp);
-    }
-
-    return ok;
-}
-#endif
-
 static int boolvar_get_flag (const char *s)
 {
     if (!strcmp(s, ECHO)) {
@@ -1768,28 +1893,21 @@ static void maybe_check_env (const char *s)
     } else if (!strcmp(s, HALT_ON_ERR)) {
 	set_flag_from_env(STATE_HALT_ON_ERR, "GRETL_KEEP_GOING", 1);
     }
-
-#ifndef WIN32
-    if (!strcmp(s, SHELL_OK) && !gretl_in_gui_mode()) {
-	if (read_cli_shell_status()) {
-	    state->flags |= STATE_SHELL_OK;
-	} else {
-	    state->flags &= ~STATE_SHELL_OK;
-	}
-    }
-#endif
 }
 
-int libset_get_bool (const char *s)
+int libset_get_bool (const char *key)
 {
     int flag, ret = 0;
 
-    if (!strcmp(s, PROTECT_LISTS)) {
-	/* global special */
-	return protect_lists;
+    /* global specials */
+
+    if (!strcmp(key, R_FUNCTIONS)) {
+	return R_functions;
+    } else if (!strcmp(key, R_LIB)) {
+	return R_lib;
     }
 
-    if (!strcmp(s, MAX_VERBOSE) && gretl_debug > 1) {
+    if (!strcmp(key, MAX_VERBOSE) && gretl_debug > 1) {
 	/* strong debugging turns on max_verbose */
 	return 1;
     }
@@ -1798,12 +1916,13 @@ int libset_get_bool (const char *s)
 	return 0;
     }
 
-    maybe_check_env(s);
+    maybe_check_env(key);
 
-    flag = boolvar_get_flag(s);
+    flag = boolvar_get_flag(key);
     if (flag == 0) {
 	fprintf(stderr, "libset_get_bool: unrecognized "
-		"variable '%s'\n", s);
+		"variable '%s'\n", key);
+	ret = 0;
     } else {
 	ret = flag_to_bool(state, flag);
     }
@@ -1814,13 +1933,13 @@ int libset_get_bool (const char *s)
 static void libset_set_decpoint (int on)
 {
 #ifdef ENABLE_NLS
-    static char num_locale[16];
+    static char num_locale[32];
 
     if (on) {
 	char *orig = setlocale(LC_NUMERIC, "");
 
 	*num_locale = '\0';
-	strncat(num_locale, orig, 15);
+	strncat(num_locale, orig, 31);
 	setlocale(LC_NUMERIC, "C");
     } else {
 	setlocale(LC_NUMERIC, num_locale);
@@ -1830,34 +1949,55 @@ static void libset_set_decpoint (int on)
 #endif
 }
 
-void libset_set_bool (const char *s, int set)
+static int check_R_setting (int *var, int val, const char *key)
 {
-    int flag;
+    int err = 0;
+
+#ifdef USE_RLIB
+    *var = val;
+#else
+    if (val) {
+	gretl_errmsg_sprintf("%s: not supported.", key);
+	err = E_EXTERNAL;
+    }
+#endif
+
+    return err;
+}
+
+int libset_set_bool (const char *key, int val)
+{
+    int flag, err = 0;
 
     if (check_for_state()) {
-	return;
+	return E_ALLOC;
     }
 
-    if (!strcmp(s, PROTECT_LISTS)) {
-	/* global special */
-	protect_lists = set;
-	return;
+    /* global specials */
+
+    if (!strcmp(key, R_FUNCTIONS)) {
+	return check_R_setting(&R_functions, val, key);
+    } else if (!strcmp(key, R_LIB)) {
+	return check_R_setting(&R_lib, val, key);
     }
 
-    flag = boolvar_get_flag(s);
+    flag = boolvar_get_flag(key);
 
     if (flag == 0) {
 	fprintf(stderr, "libset_set_bool: unrecognized "
-		"variable '%s'\n", s);
-    } else if (set) {
+		"variable '%s'\n", key);
+	err = E_UNKVAR;
+    } else if (val) {
 	state->flags |= flag;
     } else {
 	state->flags &= ~flag;
     }
 
     if (flag == STATE_FORCE_DECPOINT) {
-	libset_set_decpoint(set);
+	libset_set_decpoint(val);
     }
+
+    return err;
 }
 
 /* Mechanism for pushing and popping program state for user-defined
@@ -1884,12 +2024,12 @@ int push_program_state (void)
     newstate = malloc(sizeof *newstate);
 
     if (newstate == NULL) {
-	err = 1;
+	err = E_ALLOC;
     } else {
 	sstack = realloc(state_stack, (ns + 1) * sizeof *sstack);
 	if (sstack == NULL) {
 	    free(newstate);
-	    err = 1;
+	    err = E_ALLOC;
 	}
     }
 
@@ -1999,7 +2139,7 @@ void libset_cleanup (void)
 }
 
 /* switches for looping and batch mode: output: these depend on the
-   state of the program calling libgretl, but are not user-settable
+   state of the program calling libgretl, they are not user-settable
 */
 
 static int batch_mode;
@@ -2102,3 +2242,177 @@ int iter_print_callback (int i, PRN *prn)
 
     return ret;
 }
+
+/* mechanism for interactive debugging */
+
+static DEBUG_READLINE dbg_readline;
+
+void set_debug_read_func (DEBUG_READLINE dfunc) 
+{
+
+    if (check_for_state()) {
+	return;
+    }
+
+    dbg_readline = dfunc;
+}
+
+DEBUG_READLINE get_debug_read_func (void) 
+{
+    return dbg_readline;
+}
+
+static DEBUG_OUTPUT dbg_output;
+
+void set_debug_output_func (DEBUG_OUTPUT dout) 
+{
+
+    if (check_for_state()) {
+	return;
+    }
+
+    dbg_output = dout;
+}
+
+DEBUG_OUTPUT get_debug_output_func (void) 
+{
+    return dbg_output;
+}
+
+/* mechanism for passing an integer parameter to
+   a gretl script, accessible via $switch
+*/
+
+static int script_switch;
+
+void set_script_switch (int s)
+{
+    script_switch = s;
+}
+
+int get_script_switch (void)
+{
+    return script_switch;
+}
+
+/* for setting what we print for NAs on CSV output */
+
+const char *get_csv_na_string (void)
+{
+    if (check_for_state() || *state->csv_na == '\0') {
+	return "NA";
+    } else {
+	return state->csv_na;
+    }
+}
+
+void set_csv_na_string (const char *s)
+{
+    if (check_for_state()) {
+	return;
+    }
+
+    *state->csv_na = '\0';
+    strncat(state->csv_na, s, 7);
+}
+
+void set_include_path (const char *s)
+{
+    if (gretl_function_depth() > 0) {
+	gretl_errmsg_set("set include_path: cannot be done inside a function");
+	return;
+    }
+
+    /* skip past "include_path" and space */
+    s += 16;
+    s += strspn(s, " ");
+
+    if (*s != '\0') {
+	char tmp[MAXLEN];
+	int n = 0;
+
+	if (*s == '"') {
+	    n = sscanf(s+1, "%511[^\"]", tmp);
+	} else {
+	    n = sscanf(s, "%511s", tmp);
+	}
+
+	if (n > 0) {
+	    *include_path = '\0';
+	    strncat(include_path, tmp, MAXLEN - 2);
+	    slash_terminate(include_path);
+	}
+    }
+}
+
+const char *get_include_path (void)
+{
+    return (*include_path == '\0')? NULL : include_path;
+}
+
+int libset_write_script (const char *fname)
+{
+    PRN *prn;
+    int err = 0;
+
+    /* FIXME maybe adjust path for fname? */
+
+    prn = gretl_print_new_with_filename(fname, &err);
+
+    if (!err) {
+	print_settings(prn, OPT_NONE);
+	gretl_print_destroy(prn);
+    }
+
+    return err;
+}
+
+/* If @prn is non-NULL we're called via the "set" command.
+   Otherwise we're called by the gretl session apparatus.
+*/
+
+static int real_libset_read_script (const char *fname,
+				    PRN *prn)
+{
+    FILE *fp;
+    int err = 0;
+
+    fp = gretl_fopen(fname, "r");
+
+    if (fp == NULL) {
+	char fullname[FILENAME_MAX];
+
+	strcpy(fullname, fname);
+	addpath(fullname, 0);
+	fp = gretl_fopen(fname, "r");
+	if (fp == NULL) {
+	    err = E_FOPEN;
+	}
+    }
+
+    if (!err) {
+	char line[1024];
+
+	while (fgets(line, sizeof line, fp)) {
+	    if (*line == '#' || string_is_blank(line)) {
+		continue;
+	    }
+	    tailstrip(line);
+	    err = execute_set_line(line, NULL, OPT_NONE, prn);
+	    if (err && prn != NULL) {
+		break;
+	    }
+	}
+
+	fclose(fp);
+    }
+
+    return err;
+}
+
+int libset_read_script (const char *fname)
+{
+    return real_libset_read_script(fname, NULL);
+}
+
+
