@@ -32,8 +32,10 @@ typedef enum {
 
 #define BFGS_MAXITER     "bfgs_maxiter"
 #define BFGS_TOLER       "bfgs_toler"
+#define BFGS_VERBSKIP    "bfgs_verbskip"
 #define BHHH_MAXITER     "bhhh_maxiter"
 #define BHHH_TOLER       "bhhh_toler"
+#define LBFGS_MEM        "lbfgs_mem"
 #define BKBP_K           "bkbp_k"
 #define BOOTREP          "bootrep"
 #define FORCE_DECP       "force_decpoint"
@@ -47,7 +49,6 @@ typedef enum {
 #define HORIZON          "horizon"
 #define HP_LAMBDA        "hp_lambda"
 #define USE_LBFGS        "lbfgs"
-#define LONGDIGITS       "longdigits"
 #define LOOP_MAXITER     "loop_maxiter"
 #define RQ_MAXITER       "rq_maxiter"
 #define MAX_VERBOSE      "max_verbose"
@@ -63,8 +64,12 @@ typedef enum {
 #define ARMA_VCV         "arma_vcv"
 #define VERBOSE_INCLUDE  "verbose_include"
 #define SKIP_MISSING     "skip_missing"
+#define R_FUNCTIONS      "R_functions"
+#define R_LIB            "R_lib"
 
 typedef int (*ITER_PRINT_FUNC) (int, PRN *);
+typedef int (*DEBUG_READLINE) (void *);
+typedef int (*DEBUG_OUTPUT) (void *);
 
 #define set_nls_toler(x) (libset_set_double(NLS_TOLER, x))
 
@@ -75,14 +80,16 @@ int libset_restore_state_zero (DATAINFO *pdinfo);
 int push_program_state (void);
 int pop_program_state (void);
 
-int libset_get_bool (const char *s);
-void libset_set_bool (const char *s, int set);
+int libset_get_bool (const char *key);
+int libset_set_bool (const char *key, int val);
 
-double libset_get_double (const char *s);
-int libset_set_double (const char *s, double x);
+double libset_get_double (const char *key);
+int libset_set_double (const char *key, double val);
 
-int libset_get_int (const char *s);
-int libset_set_int (const char *s, int k);
+double libset_get_user_tolerance (const char *key);
+
+int libset_get_int (const char *key);
+int libset_set_int (const char *key, int val);
 
 /* GUI setter functions */
 void set_xsect_hccme (const char *s);
@@ -127,20 +134,38 @@ int gretl_messages_on (void);
 
 int gretl_warnings_on (void);
 int gretl_debugging_on (void);
-int lists_protected (void);
 
 void shelldir_init (const char *s);
 char *get_shelldir (void);
 
 char get_csv_delim (const DATAINFO *pdinfo);
 
-int execute_set_line (const char *line, DATAINFO *pdinfo, PRN *prn);
+const char *get_csv_na_string (void);
+void set_csv_na_string (const char *s);
+
+const char *get_include_path (void);
+void set_include_path (const char *s);
+
+int execute_set_line (const char *line, DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn);
 
 void set_iter_print_func (ITER_PRINT_FUNC func);
 int iter_print_callback (int i, PRN *prn);
 int iter_print_func_installed (void);
 
+void set_debug_read_func (DEBUG_READLINE dfunc);
+DEBUG_READLINE get_debug_read_func (void);
+
+void set_debug_output_func (DEBUG_OUTPUT dout);
+DEBUG_OUTPUT get_debug_output_func (void);
+
 void set_workdir_callback (int (*callback)());
 
+void set_script_switch (int s);
+int get_script_switch (void);
+
+int libset_write_script (const char *fname);
+int libset_read_script (const char *fname);
+
 #endif /* LIBSET_H */
 
