@@ -562,9 +562,15 @@ int list_adjust_t1t2 (const int *list, const double **Z, DATAINFO *pdinfo)
     return err;
 }
 
-/* drop first/last observations from sample if missing obs 
-   encountered -- also check for missing vals within the
-   remaining sample */
+/* 
+   Drop leading and trailing observations from the sample defined by
+   the incoming content of @t1 and @t2 if missing observations are
+   encountered in the series @x.  Also check for missing values within
+   the remaining sample, defined by the content of the adjusted @t1
+   and @t2. If such a missing value is found, this function returns
+   its observation number (that is, for the first such value, starting
+   at @t1), otherwise it returns 0.
+*/
 
 int array_adjust_t1t2 (const double *x, int *t1, int *t2)
 {
@@ -586,7 +592,8 @@ int array_adjust_t1t2 (const double *x, int *t1, int *t2)
 	}
     }
 
-    *t1 = t1min; *t2 = t2max;
+    *t1 = t1min; 
+    *t2 = t2max;
 
     return 0;
 }
@@ -634,39 +641,25 @@ int varlist_adjust_sample (const int *list, int *t1, int *t2,
  * @t2: on entry, initial end of sample range; on exit, end
  *      of sample range adjusted for missing values.
  * @Z: data array.
- * @misst: return location for index of the first missing observation
- *         inside the (possibly reduced) sample range, or %NULL.
  *
  * Drops leading or trailing observations from the sample range
  * initially given by the values in @t1 and @t2, if missing values are 
  * found among the variables given in @list.  Then checks for any
- * missing values within the adjusted range.  If such are found,
- * the return will be non-zero (see below).  In addition, if
- * @misst is non-%NULL it will receive the index number of the
- * observation where the first such missing value was found.
+ * missing values within the adjusted range.
  * 
  * If you don't care about missing values inside the sample range,
- * use the simpler varlist_adjust_sample().
+ * use varlist_adjust_sample().
  *
- * Returns: the (non-zero) ID number of the first variable for 
- * which a missing value is first found inside the adjusted sample 
- * range or 0 if there is no such variable.
+ * Returns: %E_MISSDATA if missing values are found inside the
+ * (possible adjusted) sample range, else 0.
  */
 
 int check_for_missing_obs (const int *list, int *t1, int *t2,
-			   const double **Z, int *misst)
+			   const double **Z)
 {
-    int missv = 0;
+    int missv = adjust_t1t2(NULL, list, t1, t2, 0, Z, NULL);
 
-    if (misst != NULL) {
-	missv = adjust_t1t2(NULL, list, t1, t2, 0, Z, misst);
-    } else {
-	int tmp;
-
-	missv = adjust_t1t2(NULL, list, t1, t2, 0, Z, &tmp);
-    }
-
-    return missv;
+    return (missv > 0)? E_MISSDATA : 0;
 }
 
 /* For handling the "omit" command, applied to a model that has
@@ -866,3 +859,33 @@ double missing_obs_fraction (const double **Z, const DATAINFO *pdinfo)
 
     return (double) totmiss / pdinfo->n;
 }
+
+/**
+ * any_missing_user_values:
+ * @Z: data array.
+ * @pdinfo: pointer to data information struct.
+ * 
+ * Returns: 1 if there are missing values for any non-hidden
+ * variables within the current sample range, otherwise 0.
+ */
+
+int any_missing_user_values (const double **Z, const DATAINFO *pdinfo)
+{
+    int i, t;
+
+    if (pdinfo->n == 0) {
+	return 0;
+    }
+
+    for (i=1; i<pdinfo->v; i++) {
+	if (!var_is_hidden(pdinfo, i)) {
+	    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+		if (na(Z[i][t])) {
+		    return 1;
+		}
+	    }
+	}
+    }
+
+    return 0;
+}
