@@ -17,7 +17,7 @@
  * 
  */
 
-/*  monte_carlo.c - loop simulation procedures */
+/*  monte_carlo.c - loop procedures */
 
 #include "libgretl.h" 
 #include "monte_carlo.h"
@@ -28,6 +28,7 @@
 #include "objstack.h"
 #include "gretl_func.h"
 #include "gretl_scalar.h"
+#include "flow_control.h"
 
 #include <time.h>
 #include <unistd.h>
@@ -73,7 +74,7 @@ typedef struct {
     int *diff;     /* indicator for difference */
 } LOOP_PRINT;  
 
-/* below: used for special "progressive" loop */ 
+/* below: used only in "progressive" loops */ 
 
 typedef struct {
     int linenum;            /* location: line number in loop */
@@ -336,11 +337,11 @@ int ok_in_loop (int c)
 	c == DATA ||
 	c == DELEET ||
 	c == EQNPRINT ||
+	c == FOREIGN ||
 	c == FUNC ||
 	c == HURST ||
 	c == INCLUDE ||
 	c == LEVERAGE ||
-	c == MODELTAB ||
 	c == NULLDATA ||
 	c == OPEN ||
 	c == RMPLOT ||
@@ -364,7 +365,7 @@ static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
 
     children = realloc(loop->children, (nc + 1) * sizeof *children);
     if (children == NULL) {
-	return 1;
+	return E_ALLOC;
     } 
 
     loop->children = children;
@@ -638,10 +639,12 @@ static int parse_as_indexed_loop (LOOPSET *loop,
 	if (loop->init.val < 0) {
 	    err = E_DATA;
 	} else {
+	    loop->init.val += 1;
 	    loop->final.val = dateton(end, pdinfo);
 	    if (loop->final.val < 0) {
 		err = E_DATA;
 	    } else {
+		loop->final.val += 1;
 		loop->type = DATED_LOOP;
 	    }
 	}
@@ -1081,7 +1084,7 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
     }
 
     /* syntactic slop: accept "for i=lo..hi" -> "i=lo..hi" */
-    if (!strncmp(s, "for ", 4) && !strstr(s, ";")) {
+    if (!strncmp(s, "for ", 4) && !strchr(s, ';')) {
 	s += 4;
     }
 
@@ -1102,7 +1105,7 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
 	err = parse_as_count_loop(loop, (const double **) *pZ, pdinfo, lvar);
     } else {
 	printf("parse_first_loopline: failed on '%s'\n", s);
-	strcpy(gretl_errmsg, _("No valid loop condition was given."));
+	gretl_errmsg_set(_("No valid loop condition was given."));
 	err = 1;
     }
 
@@ -1187,8 +1190,8 @@ static int loop_count_too_high (LOOPSET *loop)
 
     if (loop->type == FOR_LOOP) {
 	if (nt >= MAX_FOR_TIMES) {
-	    sprintf(gretl_errmsg, _("Reached maximum iterations, %d"),
-		    MAX_FOR_TIMES);
+	    gretl_errmsg_sprintf(_("Reached maximum iterations, %d"),
+				 MAX_FOR_TIMES);
 	    loop->err = 1;
 	}
     } else {
@@ -1196,8 +1199,8 @@ static int loop_count_too_high (LOOPSET *loop)
 	    max_iters = libset_get_int(LOOP_MAXITER);
 	}
 	if (nt >= max_iters) {
-	    sprintf(gretl_errmsg, _("Warning: no convergence after %d iterations"),
-		    max_iters);
+	    gretl_errmsg_sprintf(_("Warning: no convergence after %d iterations"),
+				 max_iters);
 	    loop->err = 1;
 	}
     }
@@ -1548,7 +1551,7 @@ static int loop_store_start (LOOPSET *loop, const int *list,
     int i, n, err = 0;
 
     if (list == NULL || list[0] == 0) {
-	strcpy(gretl_errmsg, "'store' list is empty");
+	gretl_errmsg_set("'store' list is empty");
 	return E_DATA;
     }
 
@@ -1590,8 +1593,8 @@ static int loop_store_update (LOOPSET *loop, int lno,
     int i, t, err = 0;
 
     if (loop->store.linenum >= 0 && loop->store.linenum != lno) {
-	strcpy(gretl_errmsg, "Only one 'store' command is allowed in a "
-	       "progressive loop");
+	gretl_errmsg_set("Only one 'store' command is allowed in a "
+			 "progressive loop");
 	return E_DATA;
     }
 
@@ -1873,8 +1876,9 @@ static int add_more_loop_lines (LOOPSET *loop)
 
 static int real_append_line (ExecState *s, LOOPSET *loop)
 {
+    const char *flagstr = NULL;
     int nc = loop->n_cmds;
-    int err = 0;
+    int len, err = 0;
 
 #if LOOP_DEBUG
     fprintf(stderr, "real_append_line: s->line = '%s'\n", s->line);
@@ -1882,39 +1886,42 @@ static int real_append_line (ExecState *s, LOOPSET *loop)
 
     if ((nc + 1) % LOOP_BLOCK == 0) {
 	if (add_more_loop_lines(loop)) {
-	    gretl_errmsg_set(_("Out of memory!"));
 	    return E_ALLOC;
 	}
     }
 
-    loop->lines[nc] = malloc(MAXLEN);
-    if (loop->lines[nc] == NULL) {
-	gretl_errmsg_set(_("Out of memory!"));
-	return E_ALLOC;
-    }
-
-    if (s->cmd->ci == PRINT && (!loop_is_progressive(loop) || strchr(s->line, '"'))) {
-	loop->ci[nc] = 0;
-    } else {
-	loop->ci[nc] = s->cmd->ci;
-    }
-
-    loop->lines[nc][0] = '\0';
+    len = strlen(s->line);
 
     if (s->cmd->opt) {
-	const char *flagstr = print_flags(s->cmd->opt, s->cmd->ci);
-
-	if (strlen(s->line) + strlen(flagstr) >= MAXLEN) {
-	    strcpy(gretl_errmsg, "loop: line is too long");
+	flagstr = print_flags(s->cmd->opt, s->cmd->ci);
+	len += strlen(flagstr);
+	if (len >= MAXLINE) {
+	    gretl_errmsg_set("loop: line is too long");
 	    err = 1;
-	} else {
-	    sprintf(loop->lines[nc], "%s%s", s->line, flagstr);
 	}
-    } else {
-	strcpy(loop->lines[nc], s->line);
     }
 
-    loop->n_cmds += 1;
+    if (!err) {
+	if (flagstr != NULL) {
+	    loop->lines[nc] = malloc(len + 1);
+	} else {
+	    loop->lines[nc] = gretl_strdup(s->line);
+	}
+	if (loop->lines[nc] == NULL) {
+	    err = E_ALLOC;
+	} else if (flagstr != NULL) {
+	    sprintf(loop->lines[nc], "%s%s", s->line, flagstr);
+	}
+    }
+
+    if (!err) {
+	if (s->cmd->ci == PRINT && (!loop_is_progressive(loop) || strchr(s->line, '"'))) {
+	    loop->ci[nc] = 0;
+	} else {
+	    loop->ci[nc] = s->cmd->ci;
+	}
+	loop->n_cmds += 1;
+    }
 
 #if LOOP_DEBUG
     fprintf(stderr, "loop %p: n_cmds=%d, line[%d]='%s', ci=%d\n",
@@ -2201,7 +2208,7 @@ static int loop_store_save (LOOP_STORE *lstore, PRN *prn)
     pprintf(prn, _("store: using filename %s\n"), lstore->fname);
 
     err = write_data(lstore->fname, list, (const double **) lstore->Z, 
-		     lstore->dinfo, lstore->opt, NULL);
+		     lstore->dinfo, lstore->opt, 0);
 
     if (!err) {
 	pprintf(prn, _("Data written OK.\n"));
@@ -2414,8 +2421,7 @@ static void progressive_loop_zero (LOOPSET *loop)
     loop_store_free(&loop->store);
 }
 
-static int 
-top_of_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
+static int top_of_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
 {
     int err = 0;
 
@@ -2432,7 +2438,9 @@ top_of_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo)
     if (!err && (loop->type == COUNT_LOOP || indexed_loop(loop))) {
 	loop->final.val = controller_get_val(&loop->final);
 	if (na(loop->init.val) || na(loop->final.val)) {
-	    err = 1;
+	    gretl_errmsg_set(_("error evaluating loop condition"));
+	    fprintf(stderr, "loop: got NA for init and/or final value\n");
+	    err = E_DATA;
 	} else {
 	    loop->itermax = loop->final.val - loop->init.val + 1;
 #if LOOP_DEBUG
@@ -2474,7 +2482,7 @@ print_loop_progress (const LOOPSET *loop, const DATAINFO *pdinfo,
     } else if (loop->type == DATED_LOOP) {
 	char obs[OBSLEN];
 
-	ntodate(obs, i, pdinfo);
+	ntodate(obs, i - 1, pdinfo);
 	pprintf(prn, "loop: %s = %s\n\n", loop->idxname, obs);
     }
 }
@@ -2522,7 +2530,7 @@ static int modifies_loop_list (const LOOPSET *loop, const char *s)
 	s += 4;
 	s += strspn(s, " ");
 	*lname = '\0';
-	sscanf(s, "%15[^ +-=]", lname);
+	sscanf(s, "%15[^ +=-]", lname);
 	ret = !strcmp(lname, loop->listname);
     }
 
@@ -2583,12 +2591,19 @@ static GENERATOR *get_loop_genr_by_line (LOOPSET *loop, int lno,
     for (i=0; i<loop->n_genrs; i++) {
 	ll = genr_get_loopline(loop->genrs[i]);
 	if (ll == lno) {
+#if LOOP_DEBUG > 1
+	    fprintf(stderr, "retrieving loop genr %d\n", i);
+#endif
 	    return loop->genrs[i];
 	}
     }
 
     genr = genr_compile(line, pZ, pdinfo, OPT_NONE, err);
 
+#if LOOP_DEBUG > 1
+    fprintf(stderr, "compiled loop genr, err = %d\n", *err);
+#endif
+
     if (!*err) {
 	*err = add_loop_genr(loop, genr, lno);
     }
@@ -2638,8 +2653,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     LOOP_MODEL *lmod;
     LOOP_PRINT *lprn;
     char errline[MAXLINE];
-    int indent0, mod_id = 0;
-    int subst, lrefresh;
+    int indent0, subst, lrefresh;
     int j, err = 0;
 
     /* for the benefit of the caller: register the fact that execution
@@ -2661,6 +2675,9 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 #endif
 
     err = top_of_loop(loop, pZ, pdinfo);
+    if (err) {
+	*errline = '\0';
+    }
     
     while (!err && loop_condition(loop, pZ, pdinfo, &err)) {
 #if LOOP_DEBUG
@@ -2681,19 +2698,25 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    fprintf(stderr, " j=%d, line='%s'\n", j, line);
 #endif
 	    strcpy(errline, line);
-	    err = make_dollar_substitutions(line, loop, 
-					    (const double **) *pZ,
-					    pdinfo, &subst);
-	    if (err) {
-		break;
+
+	    if (strchr(line, '$')) {
+		err = make_dollar_substitutions(line, loop, 
+						(const double **) *pZ,
+						pdinfo, &subst);
+		if (err) {
+		    break;
+		}
+	    }
+
+	    if (loop_is_progressive(loop)) {
+		cmd->flags |= CMD_PROG;
+	    } else {
+		cmd->flags &= ~CMD_PROG;
 	    }
 
 	    /* We already have the "ci" index recorded, but here
 	       we do some further parsing. 
 	    */
-	    if (loop_is_progressive(loop)) {
-		cmd->flags |= CMD_PROG;
-	    }
 	    err = parse_command_line(line, cmd, pZ, pdinfo);
 
 	    if (cmd->ci < 0) {
@@ -2717,9 +2740,11 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		lrefresh = 1;
 	    }
 
-	    if (gretl_echo_on() && indexed_loop(loop)) {
+	    if (gretl_echo_on()) {
 		if (s->cmd->ci == ENDLOOP) {
-		    pputc(prn, '\n');
+		    if (indexed_loop(loop)) {
+			pputc(prn, '\n');
+		    }
 		} else if (!loop_is_quiet(loop)) {
 		    echo_cmd(cmd, pdinfo, line, CMD_BATCH_MODE, prn);
 		}
@@ -2757,8 +2782,15 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		    err = gretl_cmd_exec(s, pZ, pdinfo);
 		}
 		if (!err) {
-		    /* record the results */
-		    if (loop_is_progressive(loop) && !(cmd->opt & OPT_Q)) {
+		    int moderr = check_gretl_errno();
+
+		    if (moderr) {
+			if (loop_is_progressive(loop) || model_print_deferred(cmd->opt)) {
+			    err = moderr;
+			} else {
+			    errmsg(moderr, prn);
+			}
+		    } else if (loop_is_progressive(loop) && !(cmd->opt & OPT_Q)) {
 			err = loop_model_update(lmod, s->models[0]);
 			set_as_last_model(s->models[0], GRETL_OBJ_EQN);
 		    } else if (model_print_deferred(cmd->opt)) {
@@ -2767,10 +2799,10 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 			set_as_last_model(pmod, GRETL_OBJ_EQN);
 			model_count_minus();
 		    } else {
-			s->models[0]->ID = ++mod_id; /* ?? */
 			if (!(cmd->opt & OPT_Q)) {
 			    printmodel(s->models[0], pdinfo, cmd->opt, prn);
 			}
+			attach_subsample_to_model(s->models[0], pdinfo);
 			set_as_last_model(s->models[0], GRETL_OBJ_EQN);
 		    }
 		}
@@ -2786,8 +2818,8 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 					(const double **) *pZ, 
 					pdinfo, cmd->opt);
 	    } else if (loop_is_progressive(loop) && not_ok_in_progloop(cmd->ci)) {
-		sprintf(gretl_errmsg, _("%s: not implemented in 'progressive' loops"),
-			cmd->word);
+		gretl_errmsg_sprintf(_("%s: not implemented in 'progressive' loops"),
+				     cmd->word);
 		err = 1;
 	    } else if (cmd->ci == GENR) {
 		if (!loop_is_verbose(loop)) {
@@ -2808,7 +2840,6 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		err = gretl_cmd_exec(s, pZ, pdinfo);
 		if (!err && block_model(cmd)) {
 		    /* NLS, etc. */
-		    s->models[0]->ID = ++mod_id;
 		    printmodel(s->models[0], pdinfo, cmd->opt, prn);
 		    set_as_last_model(s->models[0], GRETL_OBJ_EQN);
 		}
@@ -2846,7 +2877,9 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     if (err) {
 	if (!s->funcerr) {
 	    errmsg(err, prn);
-	    pprintf(prn, ">> %s\n", errline);
+	    if (*errline != '\0') {
+		pprintf(prn, ">> %s\n", errline);
+	    }
 	}
     } else if (loop->err) {
 	errmsg(loop->err, prn);
@@ -2876,6 +2909,8 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	*line = '\0';
     } 
 
+    cmd->flags &= ~CMD_PROG;
+
     if (loop->parent == NULL) {
 	/* reached top of stack: clean up */
 	gretl_loop_destroy(loop);
@@ -2883,11 +2918,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	set_loop_off();
     }
 
-    if (libset_get_bool(HALT_ON_ERR)) {
-	return err;
-    } else {
-	return 0;
-    }
+    return (libset_get_bool(HALT_ON_ERR))? err : 0;
 }
 
 
