@@ -263,17 +263,23 @@ static int nls_genr_setup (nlspec *s)
 	    if (genr_is_print(genrs[i])) {
 		continue;
 	    } else if (i >= s->naux && (dname == NULL || !gretl_is_scalar(dname))) {
-		fprintf(stderr, "nls_genr_setup: bad type: %s\n", formula);
+		gretl_errmsg_sprintf(_("The formula '%s'\n produced a scalar result"), 
+				     formula);
 		err = E_TYPES;
 		break;
 	    }
 	}
 
 	if (i == s->naux) {
-	    s->lhv = v;
-	    s->lvec = m;
-	    if (s->lvec != NULL) {
+	    if (m != NULL) {
+		s->lvec = m;
+		s->lhtype = GRETL_TYPE_MATRIX;
 		err = check_lhs_vec(s);
+	    } else if (v > 0) {
+		s->lhv = v;
+		s->lhtype = GRETL_TYPE_SERIES;
+	    } else {
+		err = E_TYPES;
 	    }
 	} else if (j > 0) {
 	    int k = j - 1;
@@ -336,7 +342,7 @@ static int nls_auto_genr (nlspec *s, int i)
 	s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, OPT_S, s->prn);
 	if (s->generr) {
 	    return s->generr;
-	}
+	} 
     }
 
     if (i == 0 && s->nlfunc == NULL) {
@@ -580,6 +586,8 @@ nlspec_add_params_from_line (nlspec *s, const char *str,
  * @np: number of parameters.
  * @vals: array of initial parameter values.
  * @names: array of parameter names.
+ * @pZ: pointer to data array.
+ * @pdinfo: dataset information.
  *
  * Adds to @spec a list of (scalar) parameters to be estimated.
  * For an example of use see arma.c in the gretl plugin directory.
@@ -725,7 +733,7 @@ static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
    nls residual variable.  
 */
 
-static int nl_missval_check (nlspec *s)
+static int nl_missval_check (nlspec *s, const DATAINFO *pdinfo)
 {
     int t1 = s->t1, t2 = s->t2;
     int t, v;
@@ -780,7 +788,8 @@ static int nl_missval_check (nlspec *s)
     for (t=t1; t<=t2; t++) {
 	if (na((*s->Z)[v][t])) {
 	    fprintf(stderr, "  after setting t1=%d, t2=%d, "
-		    "got NA for var %d at obs %d\n", t1, t2, v, t);
+		    "got NA for var %d (%s) at obs %d\n", t1, t2, v, 
+		    pdinfo->varname[v], t);
 #if NLS_SKIP_MISSING
 	    return nls_make_trimmed_dataset(s, t1, t2);
 #else
@@ -803,28 +812,41 @@ static int nl_missval_check (nlspec *s)
     return err;
 }
 
-/* the next two functions are used in the context of BFGS */
+/* get_mle_ll: callback used by BFGS.  Note that this should return
+   NADBL in case of numerical problems.  This will signal to BFGS to
+   try a smaller step length.
+*/
 
 static double get_mle_ll (const double *b, void *p)
 {
     nlspec *s = (nlspec *) p;
     double x;
-    int t, k;
+    int t, k, err;
 
     update_coeff_values(b, s);
 
-    if (nl_calculate_fvec(s)) {
+    err = nl_calculate_fvec(s);
+    if (err) {
 	return NADBL;
     }
 
     s->crit = 0.0;
 
+    if (s->lhtype == GRETL_TYPE_MATRIX) {
+	s->lvec = get_matrix_by_name(s->lhname);
+	if (s->lvec == NULL) {
+	    fprintf(stderr, "get_mle_ll: s->lvec is gone!\n");
+	    return NADBL;
+	}
+    }
+
     if (s->lvec != NULL) {
 	k = gretl_vector_get_length(s->lvec);
 	for (t=0; t<k; t++) {
 	    x = s->lvec->val[t];
-	    if (na(x)) {
-		return NADBL;
+	    if (xna(x)) {
+		s->crit = NADBL;
+		break;
 	    }
 	    s->crit += x;
 	}
@@ -832,8 +854,9 @@ static double get_mle_ll (const double *b, void *p)
 	k = s->lhv;
 	for (t=s->t1; t<=s->t2; t++) {
 	    x = (*s->Z)[k][t];
-	    if (na(x)) {
-		return NADBL;
+	    if (xna(x)) {
+		s->crit = NADBL;
+		break;
 	    }
 	    s->crit += x;
 	}
@@ -857,7 +880,8 @@ static int nl_function_calc (double *f, void *p)
 #endif
 
     /* calculate residual given current parameter estimates */
-    if ((err = nl_calculate_fvec(s))) {
+    err = nl_calculate_fvec(s);
+    if (err) {
 	return err;
     }
 
@@ -1172,17 +1196,28 @@ static int QML_vcv (nlspec *spec, gretl_matrix *V)
 static double *mle_score_callback (const double *b, int i, void *p)
 {
     nlspec *s = (nlspec *) p;
+    int err;
 
     update_coeff_values(b, s);
-    nl_calculate_fvec(s);
+    err = nl_calculate_fvec(s);
 
-    if (s->lvec != NULL) {
-	return s->lvec->val;
+    if (err) {
+	return NULL;
+    } else if (s->lhtype == GRETL_TYPE_MATRIX) {
+	s->lvec = get_matrix_by_name(s->lhname);
+	if (s->lvec == NULL) {
+	    fprintf(stderr, "mle_score_callback: s->lvec is gone!\n");
+	    return NULL;
+	} else {
+	    return s->lvec->val;
+	}
     } else {
 	return (*s->Z)[s->lhv] + s->t1;
     }
 }
 
+#define ALT_OPG 0
+
 /* build the G matrix, given a final set of coefficient
    estimates, b, and a function for calculating the score
    vector, scorefun
@@ -1328,6 +1363,9 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
     } else {
 	/* plain OPG */
 	err = gretl_invert_symmetric_matrix(V);
+	if (err) {
+	    gretl_errmsg_set("failed to invert OPG matrix GG'");
+	}
 	*vcvopt = VCV_OP;
     }
 
@@ -1751,8 +1789,7 @@ static MODEL GNR (nlspec *spec, const double **Z, DATAINFO *pdinfo,
     } 
 
     if (gnr.list[0] != glist[0]) {
-	strcpy(gretl_errmsg, _("Failed to calculate Jacobian"));
-	gnr.errcode = E_DATA;
+	gnr.errcode = E_JACOBIAN;
     }
 
     if (gnr.errcode == 0) {
@@ -1949,6 +1986,7 @@ static void clear_nlspec (nlspec *spec)
     spec->opt = OPT_NONE;
 
     spec->dv = 0;
+    spec->lhtype = GRETL_TYPE_NONE;
     spec->lhv = 0;
     spec->lvec = NULL;
 
@@ -2116,8 +2154,7 @@ static int check_derivatives (integer m, integer n, double *x,
     }
 
     if (zerocount > 0) {
-	strcpy(gretl_errmsg, 
-	       _("NLS: The supplied derivatives seem to be incorrect"));
+	gretl_errmsg_set(_("NLS: The supplied derivatives seem to be incorrect"));
 	fprintf(stderr, _("%d out of %d tests gave zero\n"), zerocount, (int) m);
     } else if (badcount > 0) {
 	pputs(prn, _("Warning: The supplied derivatives may be incorrect, or perhaps\n"
@@ -2145,8 +2182,10 @@ static int mle_calculate (nlspec *s, PRN *prn)
 	integer n = s->ncoeff;
 	integer ldjac = m; 
 
-	err = check_derivatives(m, n, s->coeff, s->fvec, s->jac, ldjac, 
-				prn, s);
+	if (!(s->opt & OPT_G)) {
+	    err = check_derivatives(m, n, s->coeff, s->fvec, 
+				    s->jac, ldjac, prn, s);
+	}
     }
 
     if (!err) {
@@ -2209,7 +2248,7 @@ static int lm_calculate (nlspec *spec, PRN *prn)
 	err = 1;
 	break;
     case 0:
-	strcpy(gretl_errmsg, _("Invalid NLS specification"));
+	gretl_errmsg_set(_("Invalid NLS specification"));
 	err = 1;
 	break;
     case 1:
@@ -2220,9 +2259,8 @@ static int lm_calculate (nlspec *spec, PRN *prn)
     case 5:
     case 6:
     case 7:
-	sprintf(gretl_errmsg, 
-		_("NLS: failed to converge after %d iterations"),
-		spec->iters);
+	gretl_errmsg_sprintf(_("NLS: failed to converge after %d iterations"),
+			     spec->iters);
 	err = 1;
 	break;
     default:
@@ -2313,7 +2351,7 @@ static int lm_approximate (nlspec *spec, PRN *prn)
 	err = 1;
 	break;
     case 0:
-	strcpy(gretl_errmsg, _("Invalid NLS specification"));
+	gretl_errmsg_set(_("Invalid NLS specification"));
 	err = 1;
 	break;
     case 1:
@@ -2325,9 +2363,8 @@ static int lm_approximate (nlspec *spec, PRN *prn)
     case 6:
     case 7:
     case 8:
-	sprintf(gretl_errmsg, 
-		_("NLS: failed to converge after %d iterations"),
-		spec->iters);
+	gretl_errmsg_sprintf(_("NLS: failed to converge after %d iterations"),
+			     spec->iters);
 	err = 1;
 	break;
     default:
@@ -2390,7 +2427,7 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
     int err = 0;
 
     if (spec->ci == GMM) {
-	strcpy(gretl_errmsg, _("Analytical derivatives cannot be used with GMM"));
+	gretl_errmsg_set(_("Analytical derivatives cannot be used with GMM"));
 	return E_DATA;
     }
 
@@ -2448,8 +2485,8 @@ static int screen_bad_aux (const char *line, const DATAINFO *pdinfo)
     } else if (get_user_function_by_name(word)) {
 	err = 0;
     } else {
-	sprintf(gretl_errmsg, _("command '%s' not valid in this context"), 
-		word);
+	gretl_errmsg_sprintf(_("command '%s' not valid in this context"), 
+			     word);
     }
 
     return err;
@@ -2533,12 +2570,12 @@ nlspec_set_regression_function (nlspec *spec, const char *fnstr,
     }
 
     if (equation_get_lhs_and_rhs(p, &vname, &rhs)) { 
-	sprintf(gretl_errmsg, _("parse error in '%s'\n"), fnstr);
+	gretl_errmsg_sprintf(_("parse error in '%s'\n"), fnstr);
 	err =  E_PARSE;
     } else if (spec->ci == NLS) {
 	spec->dv = series_index(pdinfo, vname);
 	if (spec->dv == pdinfo->v) {
-	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), vname);
+	    gretl_errmsg_sprintf(_("Unknown variable '%s'"), vname);
 	    err = E_UNKVAR;
 	}
     } else {
@@ -2638,13 +2675,13 @@ int nl_parse_line (int ci, const char *line, const double **Z,
 	}	
     } else if (param_line(line)) {
 	if (s->nlfunc == NULL && s->ci != GMM) {
-	    strcpy(gretl_errmsg, _("No regression function has been specified"));
+	    gretl_errmsg_set(_("No regression function has been specified"));
 	    err = E_PARSE;
 	} else {
 	    if (!strncmp(line, "deriv", 5)) {
 		if (numeric_mode(s) && s->params != NULL) {
-		    strcpy(gretl_errmsg, _("You cannot supply both a \"params\" "
-					   "line and analytical derivatives"));
+		    gretl_errmsg_set(_("You cannot supply both a \"params\" "
+				       "line and analytical derivatives"));
 		    err = E_PARSE;
 		} else {
 		    err = nlspec_add_param_with_deriv(s, line, Z, pdinfo);
@@ -2696,7 +2733,7 @@ double get_default_nls_toler (void)
 static int check_spec_requirements (nlspec *spec)
 {
     if (spec->nparam < 1) {
-	strcpy(gretl_errmsg, _("No parameters have been specified"));
+	gretl_errmsg_set(_("No parameters have been specified"));
 	return 1;
     }
 
@@ -2792,7 +2829,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (spec->nlfunc == NULL && spec->ci != GMM) {
-	strcpy(gretl_errmsg, _("No function has been specified"));
+	gretl_errmsg_set(_("No function has been specified"));
 	nlmod.errcode = E_PARSE;
 	goto bailout;
     } 
@@ -2824,7 +2861,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
     if (spec->ci == GMM) {
 	err = gmm_missval_check_etc(spec);
     } else {
-	err = nl_missval_check(spec);
+	err = nl_missval_check(spec, pdinfo);
     }
 
     if (err) {
@@ -2859,9 +2896,11 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	spec->tol = libset_get_double(NLS_TOLER);
     }
 
-    pputs(prn, (numeric_mode(spec))?
-	  _("Using numerical derivatives\n") :
-	  _("Using analytical derivatives\n"));
+    if (!(opt & OPT_Q)) {
+	pputs(prn, (numeric_mode(spec))?
+	      _("Using numerical derivatives\n") :
+	      _("Using analytical derivatives\n"));
+    }
 
     /* now start the actual calculations */
 
@@ -2881,7 +2920,9 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    pprintf(prn, _("Tolerance = %g\n"), spec->tol);
+    if (!(opt & OPT_Q)) {
+	pprintf(prn, _("Tolerance = %g\n"), spec->tol);
+    }
 
     if (!err) {
 	if (spec->ci == NLS) {
@@ -2919,7 +2960,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 
     destroy_private_scalars();
 
-    if (spec->Z != pZ) {
+    if (spec->Z != pZ && spec->Z != NULL) {
 	if (!nlmod.errcode) {
 	    nls_model_fix_sample(&nlmod, spec, pdinfo);
 	}
@@ -2936,8 +2977,14 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 
     dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
 
-    if (nlmod.errcode == 0 && !(opt & OPT_A)) {
-	set_model_id(&nlmod);
+    if (!(opt & OPT_A)) {
+	if (nlmod.errcode) {
+	    if (opt & OPT_U) {
+		nlmod.opt |= OPT_U; /* continue on error */
+	    }
+	} else {
+	    set_model_id(&nlmod);
+	}
     }
 
     return nlmod;
@@ -3037,6 +3084,7 @@ nlspec *nlspec_new (int ci, const DATAINFO *pdinfo)
     spec->opt = OPT_NONE;
 
     spec->dv = 0;
+    spec->lhtype = GRETL_TYPE_NONE;
     *spec->lhname = '\0';
     spec->lhv = 0;
     spec->lvec = NULL;
@@ -3160,7 +3208,7 @@ static int ivreg_weights_setup (nlspec *spec, const int *ilist,
 	if (V == NULL) {
 	    return E_ALLOC;
 	}
-	err = user_matrix_add(V, IVREG_WEIGHTNAME);
+	err = private_matrix_add(V, IVREG_WEIGHTNAME);
 	if (err) {
 	    gretl_matrix_free(V);
 	} else {
@@ -3218,10 +3266,19 @@ static int finalize_ivreg_model (MODEL *pmod, MODEL *ols,
     pmod->ybar = ols->ybar;
     pmod->sdy = ols->sdy;
 
-    /* and steal uhat, yhat */
+    /* steal uhat */
+    if (pmod->uhat != NULL) {
+	free(pmod->uhat);
+    }
     pmod->uhat = ols->uhat;
+    ols->uhat = NULL;
+
+    /* and yhat */
+    if (pmod->yhat != NULL) {
+	free(pmod->yhat);
+    }    
     pmod->yhat = ols->yhat;
-    ols->uhat = ols->yhat = NULL;
+    ols->yhat = NULL;
 
     /* insert residuals and fitted */
     for (t=pmod->t1; t<=pmod->t2; t++) {
