@@ -20,19 +20,14 @@
 #ifndef GRETL_NLS_H
 #define GRETL_NLS_H
 
-typedef struct _nlspec nlspec;
-
-typedef double (*BFGS_CRIT_FUNC) (const double *, void *);
-typedef int (*BFGS_GRAD_FUNC) (double *, double *, int, 
-			       BFGS_CRIT_FUNC, void *);
-typedef double *(*BFGS_SCORE_FUNC) (const double *, int, void *);
-
 enum {
     C_LOGLIK,
     C_GMM,
     C_OTHER
 };
 
+typedef struct _nlspec nlspec;
+
 nlspec *nlspec_new (int ci, const DATAINFO *pdinfo);
 
 void nlspec_destroy (nlspec *spec);
@@ -43,8 +38,9 @@ nlspec_add_param_with_deriv (nlspec *spec,
 			     const double **Z, 
 			     const DATAINFO *pdinfo);
 
-int nlspec_add_param_list (nlspec *spec, const int *list,
-			   const double **Z, const DATAINFO *pdinfo);
+int nlspec_add_param_list (nlspec *spec, int np, double *vals,
+			   char **names, double ***pZ,
+			   DATAINFO *pdinfo);
 
 int 
 nlspec_set_regression_function (nlspec *spec, 
@@ -53,43 +49,21 @@ nlspec_set_regression_function (nlspec *spec,
 
 void nlspec_set_t1_t2 (nlspec *spec, int t1, int t2);
 
-int nls_parse_line (int ci, const char *line, const double **Z,
-		    const DATAINFO *pdinfo, PRN *prn);
+int nl_parse_line (int ci, const char *line, const double **Z,
+		   const DATAINFO *pdinfo, PRN *prn);
 
-MODEL nls (double ***pZ, DATAINFO *pdinfo, gretlopt opt, PRN *prn);
+MODEL nl_model (double ***pZ, DATAINFO *pdinfo, gretlopt opt, PRN *prn);
 
 MODEL model_from_nlspec (nlspec *spec, double ***pZ, 
 			 DATAINFO *pdinfo, gretlopt opt, 
 			 PRN *prn);
 
+MODEL ivreg_via_gmm (const int *list, double ***pZ,
+		     DATAINFO *pdinfo, gretlopt opt);
+
+int nls_boot_calc (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+		   int ft1, int ft2, double *fcerr); 
+
 double get_default_nls_toler (void);
 
-int BFGS_max (double *b, int n, int maxit, double reltol,
-	      int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
-	      int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
-	      gretlopt opt, PRN *prn);
-
-int LBFGS_max (double *b, int n, int maxit, double reltol,
-	       int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
-	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
-	       gretlopt opt, PRN *prn);
-
-int BFGS_numeric_gradient (double *b, double *g, int n,
-			   BFGS_CRIT_FUNC func, void *data);
-
-gretl_matrix *build_OPG_matrix (double *b, int k, int T,
-				BFGS_SCORE_FUNC scorefun,
-				void *data, int *err);
-
-double *numerical_hessian (double *b, int n, BFGS_CRIT_FUNC func, 
-			   void *data, int *err);
-
-double user_BFGS (gretl_matrix *b, const char *fncall,
-		  double ***pZ, DATAINFO *pdinfo,
-		  PRN *prn, int *err);
-
-gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
-		     double ***pZ, DATAINFO *pdinfo,
-		     int *err);
-
 #endif /* GRETL_NLS_H */
