@@ -31,10 +31,12 @@ struct _nlspec {
     gretlopt opt;       /* can include OPT_V for verbose output; if ci = MLE
 			   can also include OPT_H (Hessian) or OPT_R (QML)
 			   to control the estimator of the variance matrix;
-			   new: can also include OPT_N to force use of 
-			   numerical derivatives.
+			   can also include OPT_N to force use of 
+			   numerical derivatives, OPT_U to specify
+			   continuation on error.
 			*/
     int dv;             /* ID number of dependent variable (NLS) */
+    int lhtype;         /* type of the LHS variable (series or matrix) */
     char lhname[VNAMELEN]; /* name of LHS var in criterion function */
     int lhv;            /* ID number of LHS variable in function being
 			   minimized or maximized... */
