@@ -97,19 +97,29 @@ enum {
     UNSTACK_DESTROY
 };
 
+static int unstack_replace;
+
 static void gretl_object_unstack (void *ptr, int action)
 {
     int i, pos = -1;
 
+#if ODEBUG
+    fprintf(stderr, "gretl_object_unstack: ptr=%p, action=%d\n", ptr, action);
+#endif
+
     if (action == UNSTACK_DESTROY && ptr == last_model.ptr) {
 #if ODEBUG
-	fprintf(stderr, "gretl_object_unstack: %p is 'last_model'\n", ptr);
+	fprintf(stderr, " %p is 'last_model'\n", ptr);
 #endif
 	/* avoid double-freeing */
 	last_model.ptr = NULL;
 	last_model.type = GRETL_OBJ_NULL;
     }
 
+    if (unstack_replace) {
+	return;
+    }
+
     for (i=0; i<n_obj; i++) {
 	if (ptr == ostack[i].ptr) {
 	    pos = i;
@@ -118,8 +128,8 @@ static void gretl_object_unstack (void *ptr, int action)
     }
 
 #if ODEBUG
-    fprintf(stderr, "gretl_object_unstack: stack pos for %p = %d\n",
-	    ptr, pos);
+    fprintf(stderr, " stack pos for %p = %d, n_obj = %d\n",
+	    ptr, pos, n_obj);
 #endif
     
     if (pos >= 0) {
@@ -353,7 +363,7 @@ void set_as_last_model (void *ptr, GretlObjType type)
 
     if (last_model.ptr != ptr && last_model.ptr != NULL) {
 #if ODEBUG
-	fprintf(stderr, " kicking old object at %p (type %d) off the stack\n",
+	fprintf(stderr, " unrefing old object at %p (type %d)\n",
 		last_model.ptr, last_model.type);
 #endif
 	gretl_object_unref(last_model.ptr, last_model.type);
@@ -612,6 +622,53 @@ int gretl_object_compose_name (void *p, GretlObjType type)
     return err;
 }
 
+int gretl_object_compose_unique_name (void *p, GretlObjType type)
+{
+    char name[32];
+    int id, err = 0;
+
+    if (type == GRETL_OBJ_EQN) {
+	MODEL *pmod = (MODEL *) p;
+	
+	id = pmod->ID;
+	sprintf(name, "%s %d", _("Model"), id);
+	while (get_model_by_name(name) != NULL) {
+	    sprintf(name, "%s %d", _("Model"), ++id);
+	}
+	gretl_model_set_name(pmod, name);
+    } else if (type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = (GRETL_VAR *) p;
+
+	if (var->ci == VAR) {
+	    id = ++n_vars;
+	    sprintf(name, "%s %d", _("VAR"), id);
+	    while (get_VAR_by_name(name) != NULL) {
+		sprintf(name, "%s %d", _("VAR"), ++id);
+	    }
+	} else {
+	    id = gretl_VECM_id(var);
+	    sprintf(name, "%s %d", _("VECM"), id);
+	    while (get_VECM_by_name(name) != NULL) {
+		sprintf(name, "%s %d", _("VECM"), ++id);
+	    }
+	}
+	gretl_VAR_set_name(var, name);
+    } else if (type == GRETL_OBJ_SYS) {
+	equation_system *sys = (equation_system *) p;
+	
+	id = ++n_sys;
+	sprintf(name, "%s %d", _("System"), id);
+	while (get_equation_system_by_name(name) != NULL) {
+	    sprintf(name, "%s %d", _("System"), ++id);
+	}
+	equation_system_set_name(sys, name);
+    } else {
+	err = 1;
+    }
+
+    return err;
+}
+
 int gretl_object_rename (void *p, GretlObjType type, const char *oname)
 {
     int err = 0;
@@ -653,6 +710,9 @@ void remove_model_from_stack_on_exit (MODEL *pmod)
 
 void gretl_object_remove_from_stack (void *ptr, GretlObjType type)
 {
+#if ODEBUG
+    fprintf(stderr, "gretl_object_remove_from_stack\n");
+#endif
     gretl_object_unstack(ptr, UNSTACK_REMOVE);
     gretl_object_unref(ptr, type);
 }
@@ -690,7 +750,7 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
     int onum, err = 0;
 
 #if ODEBUG
-    fprintf(stderr, "real_stack_object: on entry, p=%p, type=%d, name='%s'\n", 
+    fprintf(stderr, " real_stack_object: on entry, p=%p, type=%d, name='%s'\n", 
 	    (void *) p, type, name);
 #endif
 
@@ -699,6 +759,9 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
     }
 
     if (object_stack_index(p) >= 0) {
+#if ODEBUG
+	fprintf(stderr, " real_stack_object: done, no-op\n");
+#endif
 	return 0;
     }
 
@@ -717,9 +780,12 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
     if (orig != NULL) {
 	/* replace existing object of same name */
 #if ODEBUG
-	fprintf(stderr, "stack_object: replacing at %p\n", orig->ptr);
+	fprintf(stderr, "  replacing at %p (onum = %d, ptr = %p)\n", 
+		orig, onum, orig->ptr);
 #endif
+	unstack_replace = 1;
 	gretl_object_unref(orig->ptr, orig->type);
+	unstack_replace = 0;
 	ostack[onum].ptr = p;
 	ostack[onum].type = type;
 	pprintf(prn, "Replaced object '%s'\n", name);
@@ -737,11 +803,10 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
 	gretl_object_ref(p, type);
 	n_obj++;
 	pprintf(prn, "Added object '%s'\n", name);
-
     }
 
 #if ODEBUG
-    fprintf(stderr, "stack_object: name='%s', type=%d, ptr=%p (n_obj=%d)\n",
+    fprintf(stderr, " real_stack_object, on exit: '%s', type=%d, ptr=%p (n_obj=%d)\n",
 	    name, type, (void *) p, n_obj);
 #endif  
 
@@ -868,12 +933,18 @@ static double real_get_obj_scalar (void *p, GretlObjType type,
 	    x = sys->ll;
 	} else if (idx == M_ESS) {
 	    x = sys->ess;
+	} else if (idx == M_DF) {
+	    x = sys->df;
 	}
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) p;
 
 	if (idx == M_T) {
 	    x = var->T;
+	} else if (idx == M_DF) {
+	    x = var->df;
+	} else if (idx == M_NCOEFF) {
+	    x = var->ncoeff;
 	} else if (idx == M_LNL) {
 	    x = var->ll;
 	} else if (idx == M_AIC) {
@@ -1009,8 +1080,10 @@ static stacker *find_smatch (const char *oname)
     return smatch;
 }
 
-/* retrieve the type and command index of a named model, or of 
-   the last model if @name is NULL */
+/* Retrieve the type of a named model, or of the last model if @name
+   is NULL.  If it's a single-equation model and @ci is non-NULL,
+   also retrieve the model's command index in @ci.
+*/
 
 GretlObjType gretl_model_get_type_and_ci (const char *name,
 					  int *ci)
@@ -1018,11 +1091,13 @@ GretlObjType gretl_model_get_type_and_ci (const char *name,
     stacker *smatch = find_smatch(name);
     GretlObjType ret = GRETL_OBJ_NULL;
 
-    *ci = 0;
+    if (ci != NULL) {
+	*ci = 0;
+    }
 
     if (smatch != NULL) {
 	ret = smatch->type;
-	if (ret == GRETL_OBJ_EQN) {
+	if (ret == GRETL_OBJ_EQN && ci != NULL) {
 	    MODEL *pmod = smatch->ptr;
 
 	    *ci = pmod->ci;
