@@ -60,6 +60,7 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_F, "difference", 0 },
     { ADF,      OPT_E, "test-down", 0 },
     { ADF,      OPT_G, "gls", 0 },
+    { ANOVA,    OPT_Q, "quiet", 0 },
     { AR1,      OPT_B, "no-corc", 0 },
     { AR1,      OPT_H, "hilu", 0 },
     { AR1,      OPT_P, "pwe", 0 },
@@ -73,10 +74,13 @@ struct gretl_option gretl_opts[] = {
     { ARMA,     OPT_C, "conditional", 0 },
     { ARMA,     OPT_G, "opg", 0 },
     { ARMA,     OPT_H, "hessian", 0 },
+    { ARMA,     OPT_L, "lbfgs", 0 },
     { ARMA,     OPT_N, "nc", 0 },    
     { ARMA,     OPT_Q, "quiet", 0 },
+    { ARMA,     OPT_U, "continue", 0 },
     { ARMA,     OPT_V, "verbose", 0 },
     { ARMA,     OPT_X, "x-12-arima", 0 },
+    { ARMA,     OPT_Y, "y-diff-only", 0 },
     { BXPLOT,   OPT_O, "notches", 0 },
     { BXPLOT,   OPT_U, "output", 2 },
     { CHOW,     OPT_D, "dummy", 0 },
@@ -101,6 +105,7 @@ struct gretl_option gretl_opts[] = {
     { CUSUM,    OPT_Q, "quiet", 0 },
     { CUSUM,    OPT_R, "squares", 0 },
     { DATA,     OPT_O, "odbc", 0 },
+    { DATA,     OPT_Q, "quiet", 0 },
     { DATAMOD,  OPT_P, "preserve", 0 },
     { DELEET,   OPT_D, "db", 0 },
     { DIFFTEST, OPT_G, "sign", 0 },
@@ -120,6 +125,8 @@ struct gretl_option gretl_opts[] = {
     { ESTIMATE, OPT_Q, "quiet", 0 },
     { ESTIMATE, OPT_V, "verbose", 0 },
     { FCAST,    OPT_D, "dynamic", 0 },
+    { FCAST,    OPT_G, "plot", 1 },
+    { FCAST,    OPT_N, "no-stats", 0 },
     { FCAST,    OPT_S, "static", 0 },
     { FCAST,    OPT_Q, "quiet", 0 },
     { FCAST,    OPT_R, "rolling", 0 },
@@ -127,19 +134,26 @@ struct gretl_option gretl_opts[] = {
     { FCAST,    OPT_I, "integrate", 0 },
     { FOREIGN,  OPT_D, "send-data", 0 },
     { FOREIGN,  OPT_Q, "quiet", 0 },
+    { FOREIGN,  OPT_V, "verbose", 0 },
     { FREQ,     OPT_O, "gamma", 0 },
     { FREQ,     OPT_Q, "quiet", 0 },
     { FREQ,     OPT_S, "silent", 0 },
     { FREQ,     OPT_Z, "normal", 0 },
+    { FUNDEBUG, OPT_C, "continue", 0 },
+    { FUNDEBUG, OPT_N, "next", 0 },
+    { FUNDEBUG, OPT_Q, "quit", 0 },
     { GARCH,    OPT_A, "arma-init", 0 },
+    { GARCH,    OPT_E, "stdresid", 0 },
     { GARCH,    OPT_F, "fcp", 0 }, 
     { GARCH,    OPT_N, "nc", 0 }, 
     { GARCH,    OPT_R, "robust", 0 },
-    { GARCH,    OPT_U, "stdresid", 0 },
+    { GARCH,    OPT_U, "continue", 0 },
     { GARCH,    OPT_V, "verbose", 0 },
     { GMM,      OPT_I, "iterate", 0 },
+    { GMM,      OPT_L, "lbfgs", 0 },
     { GMM,      OPT_Q, "quiet", 0 },
     { GMM,      OPT_T, "two-step", 0 },
+    { GMM,      OPT_U, "continue", 0 },
     { GMM,      OPT_V, "verbose", 0 },
     { GNUPLOT,  OPT_O, "with-lines", 0 },
     { GNUPLOT,  OPT_I, "inverse-fit", 0 },
@@ -202,9 +216,12 @@ struct gretl_option gretl_opts[] = {
     { MAHAL,    OPT_V, "vcv", 0 },
     { MEANTEST, OPT_O, "unequal-vars", 0 },
     { MLE,      OPT_H, "hessian", 0 },
+    { MLE,      OPT_G, "no-gradient-check", 0 },
+    { MLE,      OPT_L, "lbfgs", 0 },
     { MLE,      OPT_N, "numerical", 0 },
     { MLE,      OPT_Q, "quiet", 0 },
     { MLE,      OPT_R, "robust", 0 },
+    { MLE,      OPT_U, "continue", 0 },
     { MLE,      OPT_V, "verbose", 0 },
     { MODPRINT, OPT_C, "csv", 0 },
     { MODPRINT, OPT_O, "complete", 0 },
@@ -217,6 +234,7 @@ struct gretl_option gretl_opts[] = {
     { NLS,      OPT_O, "vcv", 0 },
     { NLS,      OPT_Q, "quiet", 0 },
     { NLS,      OPT_R, "robust", 0 },
+    { NLS,      OPT_U, "continue", 0 },
     { NLS,      OPT_V, "verbose", 0 },
     { NORMTEST, OPT_A, "all", 0 },
     { NORMTEST, OPT_D, "dhansen", 0 },
@@ -236,10 +254,10 @@ struct gretl_option gretl_opts[] = {
     { OMIT,     OPT_A, "auto", 1 },
     { OMIT,     OPT_B, "both", 0 },
     { OMIT,     OPT_I, "silent", 0 },
-    { OMIT,     OPT_P, "bootstrap", 0 },
     { OMIT,     OPT_Q, "quiet", 0 },
     { OMIT,     OPT_T, "inst", 0 },
     { OMIT,     OPT_W, "wald", 0 },
+    { OPEN,     OPT_B, "progress-bar", 0 },
     { OPEN,     OPT_C, "coded", 0 },
     { OPEN,     OPT_D, "drop-empty", 0 },
     { OPEN,     OPT_F, "cols", 2 },    
@@ -253,7 +271,7 @@ struct gretl_option gretl_opts[] = {
     { PANEL,    OPT_B, "between", 0 },
     { PANEL,    OPT_D, "time-dummies", 0 },
     { PANEL,    OPT_F, "fixed-effects", 0 },
-    { PANEL,    OPT_H, "hausman-reg", 0 },
+    { PANEL,    OPT_H, "hausman-reg", 0 }, /* backward compatibility */
     { PANEL,    OPT_I, "iterate", 0 },
     { PANEL,    OPT_M, "matrix-diff", 0 },
     { PANEL,    OPT_O, "vcv", 0 },
@@ -272,12 +290,13 @@ struct gretl_option gretl_opts[] = {
     { PERGM,    OPT_L, "log", 0 },
     { PLOT,     OPT_O, "one-scale", 0 },
     { PRINT,    OPT_O, "byobs", 0 },
-    { PRINT,    OPT_L, "long", 0 },
     { PRINT,    OPT_N, "no-dates", 0 },
     { PROBIT,   OPT_P, "p-values", 0 },
     { PROBIT,   OPT_Q, "quiet", 0 },
     { PROBIT,   OPT_R, "robust", 0 },
     { PROBIT,   OPT_V, "verbose", 0 },
+    { QQPLOT,   OPT_R, "raw", 0 },
+    { QQPLOT,   OPT_Z, "z-scores", 0 },      
     { QUANTREG, OPT_I, "intervals", 1 },
     { QUANTREG, OPT_N, "no-df-corr", 0 },
     { QUANTREG, OPT_Q, "quiet", 0 },
@@ -293,9 +312,14 @@ struct gretl_option gretl_opts[] = {
     { RESTRICT, OPT_V, "verbose", 0 },
     { RESTRICT, OPT_L, "lbfgs", 0 },
     { RESTRICT, OPT_N, "no-scaling", 0 },
+    { RESTRICT, OPT_S, "silent", 0 },
+    { RMPLOT,   OPT_Q, "quiet", 0 },
     { RUNS,     OPT_D, "difference", 0 },
     { RUNS,     OPT_E, "equal", 0 },
     { SCATTERS, OPT_L, "with-lines", 0 },
+    { SCATTERS, OPT_U, "output", 2 },
+    { SET,      OPT_F, "from-file", 2 },
+    { SET,      OPT_T, "to-file", 2 },
     { SETINFO,  OPT_C, "continuous", 0 },
     { SETINFO,  OPT_D, "discrete", 0 },
     { SETOBS,   OPT_C, "stacked-cross-section", 0 },
@@ -305,6 +329,7 @@ struct gretl_option gretl_opts[] = {
     { SETOBS,   OPT_T, "time-series", 0 },
     { SETOBS,   OPT_X, "cross-section", 0 },
     { SETOBS,   OPT_N, "special-time-series", 0 },
+    { SMPL,     OPT_B, "balanced", 0 },
     { SMPL,     OPT_F, "full", 0 },
     { SMPL,     OPT_O, "dummy", 0 },
     { SMPL,     OPT_M, "no-missing", 0 },
@@ -319,10 +344,13 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_G, "dat", 0 },
     { STORE,    OPT_J, "jmulti", 0 },
     { STORE,    OPT_M, "gnu-octave", 0 },
+    { STORE,    OPT_N, "no-header", 0 },
     { STORE,    OPT_R, "gnu-R", 0 },
     { STORE,    OPT_T, "traditional", 0 },
     { STORE,    OPT_Z, "gzipped", 0 },
     { STORE,    OPT_X, "omit-obs", 0 },
+    { SUMMARY,  OPT_B, "by", 2 },
+    { SUMMARY,  OPT_S, "simple", 0 },
     { SYSTEM,   OPT_I, "iterate", 0 },
     { SYSTEM,   OPT_V, "verbose", 0 },
     { TOBIT,    OPT_V, "verbose", 0 },
@@ -335,6 +363,7 @@ struct gretl_option gretl_opts[] = {
     { VAR,      OPT_R, "robust", 0 }, 
     { VAR,      OPT_T, "trend", 0 }, 
     { VAR,      OPT_S, "lags", 2 },
+    { VARLIST,  OPT_S, "scalars", 0 },
     { VECM,     OPT_A, "crt", 0 },
     { VECM,     OPT_D, "seasonals", 0 },
     { VECM,     OPT_F, "variance-decomp", 0 },
@@ -348,6 +377,7 @@ struct gretl_option gretl_opts[] = {
     { WLS,      OPT_Q, "quiet", 0 },
     { XCORRGM,  OPT_Q, "quiet", 0 },
     { XTAB,     OPT_C, "column", 0 },
+    { XTAB,     OPT_M, "matrix", 2 },
     { XTAB,     OPT_R, "row", 0 },
     { XTAB,     OPT_Z, "zeros", 0 },
     { 0,        0L,    NULL, 0 }
@@ -516,7 +546,7 @@ static int opt_is_valid (gretlopt opt, int ci, char c)
     }
 
     if (c != 0) {
-	sprintf(gretl_errmsg, "Invalid option '-%c'", c);
+	gretl_errmsg_sprintf("Invalid option '-%c'", c);
     } 
 
     return 0;
@@ -735,9 +765,9 @@ static int option_parm_status (int ci, gretlopt opt)
 }
 
 /* We got an "=val" parameter value in connection with a given option:
-   check this for validity.  Note that (at present), for all options
-   that accept but do not mandate a parameter value, the parameter
-   must be a numerical value.  
+   check this for validity.  Note that (at present), for almost all
+   options that accept but do not mandate a parameter value, the
+   parameter must be a numerical value.
 */
 
 static int check_optval (int ci, gretlopt opt, int status, char *val)
@@ -746,8 +776,15 @@ static int check_optval (int ci, gretlopt opt, int status, char *val)
 
     if (status == NEEDS_PARM) {
 	err = push_optparm(ci, opt, val);
-    } else if (status == ACCEPTS_PARM && numeric_string(val)) {
-	err = push_optparm(ci, opt, val);
+    } else if (status == ACCEPTS_PARM) {
+	if (ci == FCAST && opt == OPT_G) {
+	    /* --plot=filename */
+	    err = push_optparm(ci, opt, val);
+	} else if (numeric_string(val)) {
+	    err = push_optparm(ci, opt, val);
+	} else {
+	    err = 1;
+	}
     } else {
 	err = 1;
     }
@@ -886,11 +923,12 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 	    if (match > 0) {
 		/* recognized an acceptable option flag */
 		ret |= match;
-	    } else if (ci == OPEN && data_open_special(longopt)) {
+	    } else if ((ci == OPEN || ci == APPEND) && 
+		       data_open_special(longopt)) {
 		; /* no-op here: handled elsewhere (FIXME) */
 	    } else {
 		/* not a valid flag, or not applicable in context */
-		sprintf(gretl_errmsg, _("Invalid option '--%s'"), longopt);
+		gretl_errmsg_sprintf(_("Invalid option '--%s'"), longopt);
 		fprintf(stderr, " line='%s', ci = %d\n", line, ci);
 		*err = 1;
 		return 0L;
@@ -911,8 +949,8 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 		}
 	    } else if (status == NEEDS_PARM) {
 		/* we need a param value but there's none */
-		sprintf(gretl_errmsg, _("The option '--%s' requires a parameter"), 
-			longopt);
+		gretl_errmsg_sprintf(_("The option '--%s' requires a parameter"), 
+				     longopt);
 		*err = E_PARSE;
 	    }
 	    if (*err) {
@@ -926,9 +964,25 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
     return ret;
 }
 
+/* Try to get the command word, skipping a possible assignment to a
+   named object as in "<objectname> <- command", and bearing in mind
+   that <objectname> may have embedded spaces (in which case it will
+   be wrapped in quotes).
+*/
+
 static void get_cmdword (const char *line, char *word)
 {
-    if (!sscanf(line, "%*s <- %8s", word)) {
+    if (*line == '"') {
+	/* skip to closing quote */
+	line++;
+	line += strcspn(line, "\"");
+	if (*line) line++;
+	line += strspn(line, " ");
+	if (!strncmp(line, "<-", 2)) {
+	    line += 2;
+	}
+	sscanf(line, "%8s", word);
+    } else if (!sscanf(line, "%*s <- %8s", word)) {
 	sscanf(line, "%8s", word);
     }
 }
@@ -961,6 +1015,8 @@ static int end_block_ci (const char *s)
 	    return RESTRICT;
 	} else if (!strcmp(word, "kalman")) {
 	    return KALMAN;
+	} else if (!strcmp(word, "foreign")) {
+	    return FOREIGN;
 	}
     }
 
@@ -1034,7 +1090,7 @@ gretlopt get_gretl_options (char *line, int *err)
 	return oflags;
     }
 
-    if (ci != SETINFO && ci != TABPRINT && ci != EQNPRINT) {
+    if (ci != SET && ci != SETINFO && ci != TABPRINT && ci != EQNPRINT) {
 	/* try for short-form options (e.g. "-o") */
 	opt = get_short_opts(line, ci, (ci == SMPL)? NULL : &myerr);
 	if (!myerr && opt) {
@@ -1078,6 +1134,11 @@ const char *print_flags (gretlopt oflags, int ci)
     gretlopt opt;
     int i;
 
+    if (ci == FUNDEBUG) {
+	/* options are for internal use only */
+	return "";
+    }
+
     if (flagprn == NULL) {
 	int err = 0;
 
