@@ -301,7 +301,7 @@ int plotspec_add_line (GPT_SPEC *spec)
     lines[n].formula[0] = '\0';
     lines[n].rgb[0] = '\0';
     lines[n].yaxis = 1;
-    lines[n].type = LT_NONE;
+    lines[n].type = LT_AUTO;
     lines[n].ptype = 0;
     lines[n].width = 1;
     lines[n].ncols = 0;
@@ -375,6 +375,19 @@ GPT_LINE *plotspec_clone_lines (GPT_SPEC *spec, int *err)
     return lines;
 }
 
+int plotspec_max_line_width (GPT_SPEC *spec)
+{
+    int i, lw = 0;
+
+    for (i=0; i<spec->n_lines; i++) {
+	if (spec->lines[i].width > lw) {
+	    lw = spec->lines[i].width;
+	}
+    } 
+
+    return lw;
+}
+
 int plotspec_add_label (GPT_SPEC *spec)
 {
     GPT_LABEL *labels;
@@ -543,8 +556,8 @@ static int print_data_labels (const GPT_SPEC *spec, FILE *fp)
     x = spec->data;
     y = x + spec->nobs;
 
-    xrange = spec->range[0][1] - spec->range[0][0];
-    yrange = spec->range[1][1] - spec->range[1][0];
+    xrange = spec->range[GP_X_RANGE][1] - spec->range[GP_X_RANGE][0];
+    yrange = spec->range[GP_Y_RANGE][1] - spec->range[GP_Y_RANGE][0];
 
     if (xrange == 0.0 || yrange == 0.0) {
 	double ymin = 1.0e+16, ymax = -1.0e+16;
@@ -636,8 +649,8 @@ void print_plot_ranges_etc (const GPT_SPEC *spec, FILE *fp)
 	    continue;
 	}
 
-	if ((i == 2 && !(spec->flags & GPT_Y2AXIS)) ||
-	    (i == 3 && !(spec->flags & GPT_PARAMETRIC))) {
+	if ((i == GP_Y2_RANGE && !(spec->flags & GPT_Y2AXIS)) ||
+	    (i == GP_T_RANGE && !(spec->flags & GPT_PARAMETRIC))) {
 	    continue;
 	}
 
@@ -713,7 +726,7 @@ static void print_linestyle (const GPT_SPEC *spec, int i, FILE *fp)
 
     if (i < spec->n_lines) {
 	line = &spec->lines[i];
-	if (*line->rgb != '\0' && line->type != LT_NONE) {
+	if (*line->rgb != '\0' && line->type != LT_AUTO) {
 	    fprintf(fp, "lc rgb \"%s\" lt %d\n", line->rgb, line->type);
 	    done = 1;
 	} else if (*line->rgb != '\0') {
@@ -995,7 +1008,7 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 
 	fprintf(fp, "w %s", gp_line_style_string(line->style));
 
-	if (line->type != LT_NONE) {
+	if (line->type != LT_AUTO) {
 	    fprintf(fp, " lt %d", line->type);
 	}
 
@@ -1030,6 +1043,10 @@ int plotspec_print (const GPT_SPEC *spec, FILE *fp)
 	    continue;
 	}
 
+	if (i == skipline) {
+	    continue;
+	}
+
 	if (!started_data_lines) {
 	    x[0] = spec->data;
 	    /* see below for subsequent adjustment of x[1] */
@@ -1132,41 +1149,87 @@ static int set_loess_fit (GPT_SPEC *spec, int d, double q, gretl_matrix *x,
     return 0;
 }
 
-static void set_fitted_line (GPT_SPEC *spec, FitType f)
+static void set_plotfit_formula (char *formula, FitType f, const double *b,
+				 double x0, double pd)
+{
+    gretl_push_c_numeric_locale();
+
+    if (f == PLOT_FIT_OLS) {
+	if (!na(x0)) {
+	    double c = b[1] * pd;
+
+	    sprintf(formula, "%.10g + %.10g*x", b[0] - c*x0, c);
+	} else {
+	    sprintf(formula, "%.10g + %.10g*x", b[0], b[1]);
+	}
+    } else if (f == PLOT_FIT_QUADRATIC) {
+	if (!na(x0)) {
+	    double c = b[1] * pd;
+	    double g = b[2] * pd * pd;
+
+	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2", 
+		    b[0] - c*x0 + g*x0*x0, c - 2*g*x0, g);
+	} else {
+	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2", b[0], b[1], b[2]);
+	}	
+    } else if (f == PLOT_FIT_INVERSE) {
+	if (!na(x0)) {
+	    double c = x0 * pd;
+
+	    sprintf(formula, "%.10g + %.10g/(%g*x - %.10g)", b[0], b[1], 
+		    pd, c);
+	} else {
+	    sprintf(formula, "%.10g + %.10g/x", b[0], b[1]);
+	}
+    }
+
+    gretl_pop_c_numeric_locale();
+}
+
+void set_plotfit_line (char *title, char *formula,
+		       FitType f, const double *b, 
+		       double x0, double pd)
+{
+    char xc = (na(x0)) ? 'X' : 't';
+
+    if (f == PLOT_FIT_OLS) {
+	sprintf(title, "Y = %#.3g %c %#.3g%c", b[0],
+		(b[1] > 0)? '+' : '-', fabs(b[1]), xc);
+    } else if (f == PLOT_FIT_QUADRATIC) {
+	sprintf(title, "Y = %#.3g %c %#.3g%c %c %#.3g%c^2", b[0],
+		(b[1] > 0)? '+' : '-', fabs(b[1]), xc,
+		(b[2] > 0)? '+' : '-', fabs(b[2]), xc);
+    } else if (f == PLOT_FIT_INVERSE) {
+	sprintf(title, "Y = %#.3g %c %#.3g(1/%c)", b[0],
+		(b[1] > 0)? '+' : '-', fabs(b[1]), xc);
+    }
+
+    set_plotfit_formula(formula, f, b, x0, pd);
+}
+
+static void plotspec_set_fitted_line (GPT_SPEC *spec, FitType f, double x0)
 {
     char *formula = spec->lines[1].formula;
     char *title = spec->lines[1].title;
+    double pd = spec->pd;
     const double *b;
 
     if (f == PLOT_FIT_OLS) {
 	b = spec->b_ols->val;
-	sprintf(title, "Y = %#.3g %c %#.3gX", b[0],
-		(b[1] > 0)? '+' : '-', fabs(b[1]));
-	gretl_push_c_numeric_locale();
-	sprintf(formula, "%g + %g*x", b[0], b[1]);
-	gretl_pop_c_numeric_locale();
     } else if (f == PLOT_FIT_QUADRATIC) {
 	b = spec->b_quad->val;
-	sprintf(title, "Y = %#.3g %c %#.3gX %c %#.3gX^2", b[0],
-		(b[1] > 0)? '+' : '-', fabs(b[1]),
-		(b[2] > 0)? '+' : '-', fabs(b[2]));
-	gretl_push_c_numeric_locale();
-	sprintf(formula, "%g + %g*x + %g*x**2", b[0], b[1], b[2]);
-	gretl_pop_c_numeric_locale();
     } else if (f == PLOT_FIT_INVERSE) {
 	b = spec->b_inv->val;
-	sprintf(title, "Y = %#.3g %c %#.3g(1/X)", b[0],
-		(b[1] > 0)? '+' : '-', fabs(b[1]));
-	gretl_push_c_numeric_locale();
-	sprintf(formula, "%g + %g/x", b[0], b[1]);
-	gretl_pop_c_numeric_locale();
+    } else {
+	return;
     }
 
+    set_plotfit_line(title, formula, f, b, x0, pd);
+
+    spec->fit = f;
     spec->lines[1].scale = NADBL;
     spec->lines[1].style = GP_STYLE_LINES;
     spec->lines[1].ncols = 0;
-
-    spec->fit = f;
 }
 
 int plotspec_add_fit (GPT_SPEC *spec, FitType f)
@@ -1175,19 +1238,26 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
     gretl_matrix *X = NULL;
     gretl_matrix *b = NULL;
     gretl_matrix *yh = NULL;
+    const double *px = spec->data;
     const double *py;
-    const double *px;
     int T = spec->okobs;
-    double q = 0.5;
+    double x0 = NADBL;
+    double xt, q = 0.5;
     int d = 1;
     int i, t, k;
     int err = 0;
 
+    if ((spec->flags & GPT_TS) && (f == PLOT_FIT_OLS || f == PLOT_FIT_QUADRATIC)) {
+	if (spec->pd == 1 || spec->pd == 4 || spec->pd == 12) {
+	    x0 = px[0];
+	}
+    } 
+
     if ((f == PLOT_FIT_OLS && spec->b_ols != NULL) ||
 	(f == PLOT_FIT_QUADRATIC && spec->b_quad != NULL) ||
 	(f == PLOT_FIT_INVERSE && spec->b_inv != NULL)) {
 	/* just activate existing setup */
-	set_fitted_line(spec, f);
+	plotspec_set_fitted_line(spec, f, x0);
 	return 0;
     }
 	
@@ -1221,19 +1291,24 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 
     i = 0;
     for (t=0; t<spec->nobs; t++) {
-	if (!na(py[t]) && !na(px[t])) {
+	if (!na(x0)) {
+	    xt = t;
+	} else {
+	    xt = px[t];
+	}
+	if (!na(py[t]) && !na(xt)) {
 	    y->val[i] = py[t];
 	    if (f == PLOT_FIT_LOESS) {
-		gretl_matrix_set(X, i, 0, px[t]);
+		gretl_matrix_set(X, i, 0, xt);
 	    } else {
 		gretl_matrix_set(X, i, 0, 1.0);
 		if (f == PLOT_FIT_INVERSE) {
-		    gretl_matrix_set(X, i, 1, 1.0 / px[t]);
+		    gretl_matrix_set(X, i, 1, 1.0 / xt);
 		} else {
-		    gretl_matrix_set(X, i, 1, px[t]);
+		    gretl_matrix_set(X, i, 1, xt);
 		}
 		if (f == PLOT_FIT_QUADRATIC) {
-		    gretl_matrix_set(X, i, 2, px[t] * px[t]);
+		    gretl_matrix_set(X, i, 2, xt * xt);
 		}
 	    }
 	    i++;
@@ -1270,7 +1345,7 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 	if (f == PLOT_FIT_LOESS) {
 	    set_loess_fit(spec, d, q, X, y, yh);
 	} else {
-	    set_fitted_line(spec, f);
+	    plotspec_set_fitted_line(spec, f, x0);
 	}
     }
 
