@@ -23,7 +23,7 @@
 #define GP_MAXFORMULA 128
 
 #define GP_BORDER_DEFAULT (-1)
-#define LT_NONE (-2)
+#define LT_AUTO (-2)
 
 typedef enum {
     GP_STYLE_NONE,
@@ -48,6 +48,13 @@ typedef enum {
     GP_KEY_NONE
 } GpKeyPos;
 
+typedef enum {
+    GP_X_RANGE,
+    GP_Y_RANGE,
+    GP_Y2_RANGE,
+    GP_T_RANGE
+} GpRange;
+
 typedef struct gp_style_spec_ gp_style_spec;
 
 struct gp_style_spec_ {
@@ -147,6 +154,8 @@ int plotspec_delete_line (GPT_SPEC *spec, int i);
 
 GPT_LINE *plotspec_clone_lines (GPT_SPEC *spec, int *err);
 
+int plotspec_max_line_width (GPT_SPEC *spec);
+
 int plotspec_add_label (GPT_SPEC *spec);
 
 int plotspec_delete_label (GPT_SPEC *spec, int i);
@@ -175,4 +184,8 @@ gp_style_spec *get_keypos_spec (int t);
 
 void print_keypos_string (int t, FILE *fp);
 
+void set_plotfit_line (char *title, char *formula,
+		       FitType f, const double *b, 
+		       double x0, double pd);
+
 #endif /* PLOTSPEC_H */
