@@ -56,14 +56,15 @@ enum {
     P_EVIEWS_IMPORT,
     P_STATA_IMPORT,
     P_SPSS_IMPORT,
+    P_SAS_IMPORT,
     P_JMULTI_IMPORT,
     P_ZIPFILE,
-    P_OPROBIT,
     P_ARBOND,
     P_HECKIT,
     P_ODBC,
     P_QUANTREG,
-    P_INTREG
+    P_INTREG,
+    P_ANOVA
 } plugin_codes;
 
 struct plugin_info {
@@ -106,14 +107,15 @@ struct plugin_info plugins[] = {
     { P_EVIEWS_IMPORT,   "eviews_import" },
     { P_STATA_IMPORT,    "stata_import" },
     { P_SPSS_IMPORT,     "spss_import" },
+    { P_SAS_IMPORT,      "sas_import" },
     { P_JMULTI_IMPORT,   "jmulti_import" },
     { P_ZIPFILE,         "gretlzip" },
-    { P_OPROBIT,         "oprobit" },
     { P_ARBOND,          "arbond" },
     { P_HECKIT,          "heckit" },
     { P_ODBC,            "odbc_import" },
     { P_QUANTREG,        "quantreg" },
-    { P_INTREG,          "interval" }
+    { P_INTREG,          "interval" },
+    { P_ANOVA,           "anova" }
 };  
 
 struct plugin_function plugin_functions[] = { 
@@ -124,6 +126,7 @@ struct plugin_function plugin_functions[] = {
     { "wf1_get_data",      P_EVIEWS_IMPORT },
     { "dta_get_data",      P_STATA_IMPORT },
     { "sav_get_data",      P_SPSS_IMPORT },
+    { "xport_get_data",    P_SAS_IMPORT },
     { "jmulti_get_data",   P_JMULTI_IMPORT },
 
     /* Johansen cointegration test and VECM */
@@ -169,6 +172,7 @@ struct plugin_function plugin_functions[] = {
 
     /* TRAMO/SEATS and X12A */
     { "write_tx_data", P_TRAMO_X12A },
+    { "adjust_series", P_TRAMO_X12A },
 
     /* NIST test suite */
     { "run_nist_tests", P_NISTCHECK },    
@@ -179,7 +183,6 @@ struct plugin_function plugin_functions[] = {
     { "tobit_estimate",    P_TOBIT },
     { "garch_model",       P_GARCH },
     { "poisson_estimate",  P_POISSON },
-    { "ordered_estimate",  P_OPROBIT },
     { "heckit_estimate",   P_HECKIT },
     { "interval_estimate", P_INTREG },
 
@@ -203,7 +206,6 @@ struct plugin_function plugin_functions[] = {
     /* zip and unzip */
     { "gretl_make_zipfile",       P_ZIPFILE},
     { "gretl_unzip_file",         P_ZIPFILE},
-    { "gretl_is_zipfile",         P_ZIPFILE},
     { "gretl_zipfile_get_topdir", P_ZIPFILE},
 
     /* Arellano-Bond estimation */
@@ -216,6 +218,9 @@ struct plugin_function plugin_functions[] = {
     /* quantreg */
     { "rq_driver", P_QUANTREG},
 
+    /* analysis of variance */
+    { "gretl_anova", P_ANOVA},
+
     /* sentinel */
     { NULL, 0 }
 };
@@ -234,40 +239,73 @@ static const char *get_plugin_name_for_function (const char *func)
     return plugins[idx].pname;
 }
 
-static void *get_plugin_handle (const char *plugin)
+void *gretl_dlopen (const char *path, int now)
 {
 #ifdef OSX_NATIVE
     NSObjectFileImage file;
     NSObjectFileImageReturnCode rc;
 #endif
+    void *handle = NULL;
+
+#if defined(WIN32)
+    if (strstr(path, "R.dll")) {
+	handle = LoadLibraryEx(path, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
+    } else {
+	handle = LoadLibrary(path);
+    }
+#elif defined(OSX_NATIVE)
+    rc = NSCreateObjectFileImageFromFile(path, &file);
+    if (rc == NSObjectFileImageSuccess) {
+	Rhandle = NSLinkModule(file, path,
+			       NSLINKMODULE_OPTION_BINDNOW |
+			       NSLINKMODULE_OPTION_PRIVATE |
+			       NSLINKMODULE_OPTION_RETURN_ON_ERROR);
+    }
+#else
+    handle = dlopen(path, (now)? RTLD_NOW : RTLD_LAZY);
+#endif
+
+    if (handle == NULL) {
+        gretl_errmsg_sprintf(_("Failed to load plugin: %s"), path);
+#if !defined(WIN32) && !defined(OSX_NATIVE)
+	fprintf(stderr, "%s\n", dlerror());
+#endif
+    }   
+
+    return handle;
+}
+
+void *gretl_dlsym (void *handle, const char *name)
+{
+#if defined(WIN32)
+    return GetProcAddress(handle, name);
+#elif defined(OSX_NATIVE)
+    return NSLookupSymbolInModule(handle, name);
+#else
+    return dlsym(handle, name);
+#endif
+}
+
+static void *get_plugin_handle (const char *plugin)
+{
     char pluginpath[MAXLEN];
     void *handle = NULL;
 
     strcpy(pluginpath, gretl_lib_path());
 
-#if defined(WIN32)
+#ifdef WIN32
     append_dir(pluginpath, "plugins");
     strcat(pluginpath, plugin);
     strcat(pluginpath, ".dll");
-    handle = LoadLibrary(pluginpath);
-#elif defined(OSX_NATIVE)
-    strcat(pluginpath, plugin);
-    strcat(pluginpath, ".so");
-    rc = NSCreateObjectFileImageFromFile(pluginpath, &file);
-    if (rc == NSObjectFileImageSuccess) {
-	handle = NSLinkModule(file, pluginpath,
-			      NSLINKMODULE_OPTION_BINDNOW |
-			      NSLINKMODULE_OPTION_PRIVATE |
-			      NSLINKMODULE_OPTION_RETURN_ON_ERROR);
-    }
 #else
     strcat(pluginpath, plugin);
     strcat(pluginpath, ".so");
-    handle = dlopen(pluginpath, RTLD_LAZY);
 #endif 
 
+    handle = gretl_dlopen(pluginpath, 0);
+
     if (handle == NULL) {
-        sprintf(gretl_errmsg, _("Failed to load plugin: %s"), pluginpath);
+        gretl_errmsg_sprintf(_("Failed to load plugin: %s"), pluginpath);
 #if !defined(WIN32) && !defined(OSX_NATIVE)
 	fprintf(stderr, "%s\n", dlerror());
 #endif
@@ -283,7 +321,8 @@ void *get_plugin_function (const char *funcname, void **handle)
 
     plugname = get_plugin_name_for_function(funcname);
     if (plugname == NULL) {
-	strcpy(gretl_errmsg, _("Couldn't load plugin function"));
+	gretl_errmsg_set(_("Couldn't load plugin function"));
+	fprintf(stderr, "plugname == NULL for '%s'\n", funcname);
 	*handle = NULL;
 	return NULL;
     }
@@ -311,7 +350,7 @@ void *get_plugin_function (const char *funcname, void **handle)
 #endif   
 
     if (funp == NULL) {
-	strcpy(gretl_errmsg, _("Couldn't load plugin function"));
+	gretl_errmsg_set(_("Couldn't load plugin function"));
 	close_plugin(*handle);
 	*handle = NULL;
     }
