@@ -28,6 +28,7 @@
 #include "gretl_scalar.h"
 
 #include <time.h>
+#include <glib.h>
 
 #define PDEBUG 0
 
@@ -71,7 +72,7 @@ static void covhdr (PRN *prn)
 
 void session_time (PRN *prn)
 {
-    time_t runtime = time(NULL);
+    char timestr[48];
     PRN *myprn = NULL;
 
     if (prn == NULL) {
@@ -79,7 +80,8 @@ void session_time (PRN *prn)
 	prn = myprn;
     }
 
-    pprintf(prn, "%s: %s\n", _("Current session"), print_time(&runtime));
+    print_time(timestr);
+    pprintf(prn, "%s: %s\n", _("Current session"), timestr);
     
     if (myprn != NULL) {
 	gretl_print_destroy(myprn);
@@ -88,15 +90,20 @@ void session_time (PRN *prn)
 
 /**
  * logo:
+ * @quiet: if non-zero, just print version ID, else print
+ * copyright info also.
  *
  * Print to stdout gretl version information.
  */
 
-void logo (void)
+void logo (int quiet)
 {
     printf(_("gretl version %s\n"), GRETL_VERSION);
-    puts(_("Copyright Ramu Ramanathan, Allin Cottrell and Riccardo \"Jack\" Lucchetti"));
-    puts(_("This is free software with ABSOLUTELY NO WARRANTY"));
+
+    if (!quiet) {
+	puts(_("Copyright Ramu Ramanathan, Allin Cottrell and Riccardo \"Jack\" Lucchetti"));
+	puts(_("This is free software with ABSOLUTELY NO WARRANTY"));
+    }
 }
 
 /**
@@ -147,10 +154,11 @@ void lib_logo (void)
 
 void gui_script_logo (PRN *prn)
 {
-    time_t runtime = time(NULL);
+    char timestr[48];
 
     pprintf(prn, _("gretl version %s\n"), GRETL_VERSION);
-    pprintf(prn, "%s: %s\n", _("Current session"), print_time(&runtime));
+    print_time(timestr);
+    pprintf(prn, "%s: %s\n", _("Current session"), timestr);
 }
 
 /* ----------------------------------------------------- */
@@ -221,29 +229,45 @@ int max_obs_label_length (const DATAINFO *pdinfo)
     char s[OBSLEN];
     int t, n, nmax = 0;
 
-    if (pdinfo->S == NULL) {
-	if (dataset_is_time_series(pdinfo)) {
-	    switch (pdinfo->pd) {
-	    case 1:
-	    case 10:
-		nmax = 4; break;
-	    case 4:
-		nmax = 6; break;
-	    case 12:
-		nmax = 7; break;
-	    default:
+    if (pdinfo->S != NULL) {
+	/* we have specific observation strings */
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	    get_obs_string(s, t, pdinfo);
+	    n = strlen(s);
+	    if (n > nmax) {
+		nmax = n;
+	    }
+	    if (nmax == OBSLEN - 1) {
 		break;
 	    }
-	} 
-	if (nmax == 0) {
-	    get_obs_string(s, pdinfo->t2, pdinfo);
-	    nmax = strlen(s);
 	}
     } else if (dated_daily_data(pdinfo)) {
 	get_obs_string(s, pdinfo->t2, pdinfo);
 	nmax = strlen(s);
+    } else if (dataset_is_time_series(pdinfo)) {
+	switch (pdinfo->pd) {
+	case 1:   /* annual: YYYY */
+	case 10:  /* decennial: YYYY */
+	    nmax = 4; 
+	    break;
+	case 4:   /* quarterly: YYYY:Q */
+	    nmax = 6; 
+	    break;
+	case 12:  /* monthly: YYYY:MM */
+	    nmax = 7; 
+	    break;
+	default:
+	    break;
+	}
+	if (nmax == 0) {
+	    get_obs_string(s, pdinfo->t2, pdinfo);
+	    nmax = strlen(s);
+	}
     } else {
-	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	int T = pdinfo->t2 - pdinfo->t1 + 1;
+	int incr = (T < 120)? 1 : (T / 100.0);
+
+	for (t=pdinfo->t1; t<=pdinfo->t2; t+=incr) {
 	    get_obs_string(s, t, pdinfo);
 	    n = strlen(s);
 	    if (n > nmax) {
@@ -491,11 +515,15 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
     double pearson = 0.0;
     int i, j;
 
-    pputc(prn, '\n');
-    pprintf(prn, _("Cross-tabulation of %s (rows) against %s (columns)"),
-	    tab->rvarname, tab->cvarname);
+    if (*tab->rvarname != '\0' && *tab->cvarname != '\0') {
+	pputc(prn, '\n');
+	pprintf(prn, _("Cross-tabulation of %s (rows) against %s (columns)"),
+		tab->rvarname, tab->cvarname);
+	pputs(prn, "\n\n       ");
+    } else {
+	pputs(prn, "\n       ");
+    }
 
-    pputs(prn, "\n\n       ");
     for (j=0; j<c; j++) {
 	pprintf(prn, "[%4g]", tab->cval[j]);
     } 
@@ -573,15 +601,18 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
     } else {
 	double n5p = (double) n5 / (r * c);
 	int df = (r - 1) * (c - 1);
+	double pval = chisq_cdf_comp(df, pearson);
 
-	pputc(prn, '\n');
-	pprintf(prn, _("Pearson chi-square test = %g (%d df, p-value = %g)"), 
-		pearson, df, chisq_cdf_comp(df, pearson));
-	pputc(prn, '\n');
-	if (n5p < 0.80) {
-	    /* xgettext:no-c-format */
-	    pputs(prn, _("Warning: Less than of 80% of cells had expected "
-			 "values of 5 or greater.\n"));
+	if (!na(pval)) {
+	    pputc(prn, '\n');
+	    pprintf(prn, _("Pearson chi-square test = %g (%d df, p-value = %g)"), 
+		    pearson, df, pval);
+	    pputc(prn, '\n');
+	    if (n5p < 0.80) {
+		/* xgettext:no-c-format */
+		pputs(prn, _("Warning: Less than of 80% of cells had expected "
+			     "values of 5 or greater.\n"));
+	    }
 	}
     }
 
@@ -606,10 +637,13 @@ void print_smpl (const DATAINFO *pdinfo, int fulln, PRN *prn)
     }
 
     if (fulln && !dataset_is_panel(pdinfo)) {
-	pprintf(prn, _("Full data set: %d observations\n"),
-		fulln);
-	pprintf(prn, _("Current sample: %d observations\n"),
-		pdinfo->n);
+	pprintf(prn, _("Full data set: %d observations\n"), fulln);
+	if (sample_size(pdinfo) < pdinfo->n) {
+	    print_sample_obs(pdinfo, prn);
+	} else {
+	    pprintf(prn, _("Current sample: %d observations\n"),
+		    pdinfo->n);
+	}
 	return;
     }
 
@@ -622,12 +656,7 @@ void print_smpl (const DATAINFO *pdinfo, int fulln, PRN *prn)
 
     if (pdinfo->t1 > 0 || pdinfo->t2 < pdinfo->n - 1 ||
 	(fulln && dataset_is_panel(pdinfo))) {
-	char d1[OBSLEN], d2[OBSLEN];
-	ntodate_full(d1, pdinfo->t1, pdinfo);
-	ntodate_full(d2, pdinfo->t2, pdinfo);
-
-	pprintf(prn, "%s:  %s - %s", _("Current sample"), d1, d2);
-	pprintf(prn, " (n = %d)\n", pdinfo->t2 - pdinfo->t1 + 1);
+	print_sample_obs(pdinfo, prn);
     }
 
     pputc(prn, '\n');
@@ -641,8 +670,8 @@ static void print_var_smpl (int v, const double **Z,
 
     if (pdinfo->t1 > 0 || pdinfo->t2 < pdinfo->n - 1) {
 	char d1[OBSLEN], d2[OBSLEN];
-	ntodate_full(d1, pdinfo->t1, pdinfo);
-	ntodate_full(d2, pdinfo->t2, pdinfo);
+	ntodate(d1, pdinfo->t1, pdinfo);
+	ntodate(d2, pdinfo->t2, pdinfo);
 
 	pprintf(prn, "%s:  %s - %s", _("Current sample"), d1, d2);
     } else {
@@ -792,7 +821,7 @@ static char *cut_trailing_point (char *s)
 void gretl_sprint_fullwidth_double (double x, int digits, char *targ,
 				    PRN *prn)
 {
-    char decpoint = '.';
+    char decpoint;
     int n;
 
     if (na(x)) {
@@ -800,9 +829,7 @@ void gretl_sprint_fullwidth_double (double x, int digits, char *targ,
 	return;
     }
 
-#ifdef ENABLE_NLS
     decpoint = get_local_decpoint();
-#endif
 
     if (digits == -4) {
 	if (x < .0001 && x > 0.0) {
@@ -850,12 +877,10 @@ void gretl_print_fullwidth_double (double x, int digits, PRN *prn)
     char numstr[36], final[36];
     int totlen = 2 * digits + 5; /* try changing this? */
     int i, tmp, forept = 0;
-    char decpoint = '.';
+    char decpoint;
     char *p;
 
-#ifdef ENABLE_NLS
     decpoint = get_local_decpoint();
-#endif
 
     /* let's not print non-zero values for numbers smaller than
        machine zero */
@@ -1247,73 +1272,49 @@ static void fcast_print_x (double x, int n, int pmax, PRN *prn)
     }
 }
 
-static void printstr_long (PRN *prn, double xx, int d, int *ls)
-{
-    char str[64];
-    int n;
-
-    if (na(xx)) {
-	strcpy(str, "NA  ");
-	n = 4;
-    } else {
-	sprintf(str, "% .*E  ", d, xx);
-	n = strlen(str);
-    }
-    if (*ls + n > 78) {
-	pputc(prn, '\n');
-	*ls = 0;
-    }
-    pputs(prn, str);
-    *ls += n;
-}
-
 /* prints series z from current sample t1 to t2 */
 
 static void print_series_by_var (const double *z, const DATAINFO *pdinfo, 
-				 gretlopt opt, PRN *prn)
+				 PRN *prn)
 {
     char format[12];
-    int t, dig = 11, ls = 0;
+    int t, ls = 0;
     int anyneg = 0;
     double xx;
 
-    if (opt & OPT_L) {
-	dig = libset_get_int(LONGDIGITS);
-    } else {
-	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    if (z[t] < 0) {
-		anyneg = 1;
-		break;
-	    }
-	}
-	if (anyneg) {
-	    sprintf(format, "%% #.%dg  ", GRETL_DIGITS);
-	} else {
-	    sprintf(format, "%%#.%dg  ", GRETL_DIGITS);
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (z[t] < 0) {
+	    anyneg = 1;
+	    break;
 	}
     }
 
-    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xx = z[t];
-	if (opt & OPT_L) {
-	    printstr_long(prn, xx, dig, &ls);
-	} else {
-	    char str[32];
-	    int n;
+    if (anyneg) {
+	sprintf(format, "%% #.%dg  ", GRETL_DIGITS);
+    } else {
+	sprintf(format, "%%#.%dg  ", GRETL_DIGITS);
+    }
 
-	    if (na(xx)) {
-		sprintf(str, "%*s  ", GRETL_DIGITS + 1 + anyneg, "NA");
-	    } else {
-		sprintf(str, format, xx);
-	    }
-	    n = strlen(str);
-	    if (ls + n > 78) {
-		pputc(prn, '\n');
-		ls = 0;
-	    }
-	    pputs(prn, str);
-	    ls += n;
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	char str[32];
+	int n;
+
+	xx = z[t];
+
+	if (na(xx)) {
+	    sprintf(str, "%*s  ", GRETL_DIGITS + 1 + anyneg, "NA");
+	} else {
+	    sprintf(str, format, xx);
 	}
+
+	n = strlen(str);
+	if (ls + n > 78) {
+	    pputc(prn, '\n');
+	    ls = 0;
+	}
+
+	pputs(prn, str);
+	ls += n;
     }
 
     pputc(prn, '\n');
@@ -1347,11 +1348,9 @@ static int get_signif (const double *x, int n)
     int gotdec, trail, trailmax = 0;
     double xx;
     int allfrac = 1;
-    char decpoint = '.';
+    char decpoint;
 
-#ifdef ENABLE_NLS
     decpoint = get_local_decpoint();
-#endif
 
     for (i=0; i<n; i++) {
 
@@ -1730,9 +1729,9 @@ static void print_varlist (const char *name, const int *list,
     }
 }
 
-static void 
-print_listed_objects (const char *s, const DATAINFO *pdinfo, 
-		      gretlopt opt, PRN *prn)
+static void print_listed_objects (const char *s, 
+				  const DATAINFO *pdinfo, 
+				  PRN *prn)
 {
     const gretl_matrix *m;
     const int *list;
@@ -1740,14 +1739,12 @@ print_listed_objects (const char *s, const DATAINFO *pdinfo,
 
     while ((name = gretl_word_strdup(s, &s)) != NULL) {
 	if (gretl_is_scalar(name)) {
-	    print_scalar_by_name(name, opt, prn);
+	    print_scalar_by_name(name, prn);
 	} else if ((m = get_matrix_by_name(name)) != NULL) {
 	    gretl_matrix_print_to_prn(m, name, prn);
 	} else if ((list = get_list_by_name(name)) != NULL) {
 	    print_varlist(name, list, pdinfo, prn);
-	} else if (!strcmp(name, "scalars")) {
-	    print_all_scalars(opt, prn);
-	}
+	} 
 	    
 	free(name);
     }
@@ -1928,8 +1925,7 @@ static int print_by_obs (int *list, const double **Z,
 }
 
 static int print_by_var (const int *list, const double **Z,
-			 const DATAINFO *pdinfo, gretlopt opt,
-			 PRN *prn)
+			 const DATAINFO *pdinfo, PRN *prn)
 {
     int i, vi;
 
@@ -1945,7 +1941,7 @@ static int print_by_var (const int *list, const double **Z,
 	}
 	print_var_smpl(vi, Z, pdinfo, prn);
 	pputc(prn, '\n');
-	print_series_by_var(Z[vi], pdinfo, opt, prn);
+	print_series_by_var(Z[vi], pdinfo, prn);
 	pputc(prn, '\n');
     }
 
@@ -1959,9 +1955,7 @@ static int print_by_var (const int *list, const double **Z,
  * @Z: data matrix.
  * @pdinfo: data information struct.
  * @opt: if %OPT_O, print the data by observation (series in columns);
- * if %OPT_N, use simple obs numbers, not dates; if %OPT_L, print the 
- * data to a number of digits set by "set longdigits" (default 10).
- * Note that %OPT_L nullifies %OPT_O.
+ * if %OPT_N, use simple obs numbers, not dates.
  * @prn: gretl printing struct.
  *
  * Print the data for the variables in @list over the currently
@@ -1978,11 +1972,6 @@ int printdata (const int *list, const char *mstr,
     int *plist = NULL;
     int err = 0;
 
-    if (opt & OPT_L) {
-	/* can't do both --long and --byobs */
-	opt &= ~OPT_O;
-    }
-
     if (list != NULL && list[0] == 0) {
 	/* explicitly empty list given */
 	if (mstr == NULL) {
@@ -2036,13 +2025,13 @@ int printdata (const int *list, const char *mstr,
     if (opt & OPT_O) {
 	err = print_by_obs(plist, Z, pdinfo, opt, screenvar, prn);
     } else {
-	err = print_by_var(plist, Z, pdinfo, opt, prn);
+	err = print_by_var(plist, Z, pdinfo, prn);
     }
 
  endprint:
 
     if (!err && mstr != NULL) {
-	print_listed_objects(mstr, pdinfo, opt, prn);
+	print_listed_objects(mstr, pdinfo, prn);
     }
 
     free(plist);
@@ -2344,18 +2333,87 @@ int print_data_sorted (const int *list, const int *obsvec,
     return 0;
 }
 
+static int print_fcast_stats (const FITRESID *fr, gretlopt opt,
+			      PRN *prn)
+{
+    const char *strs[] = {
+	N_("Mean Error"),
+	N_("Mean Squared Error"),
+	N_("Root Mean Squared Error"),
+	N_("Mean Absolute Error"),
+	N_("Mean Percentage Error"),
+	N_("Mean Absolute Percentage Error"),
+	N_("Theil's U"),
+	N_("Bias proportion, UM"),
+	N_("Regression proportion, UR"),
+	N_("Disturbance proportion, UD")
+    };
+    gretl_matrix *m;
+    double x;
+    int i, j, t1, t2;
+    int n, nmax = 0;
+    int len, err = 0;
+
+    fcast_get_continuous_range(fr, &t1, &t2);
+
+    if (t2 - t1 + 1 <= 0) {
+	return E_MISSDATA;
+    }
+
+    m = forecast_stats(fr->actual, fr->fitted, t1, t2, opt, &err);
+    if (err) {
+	return err;
+    }
+
+    len = gretl_vector_get_length(m);
+
+    j = 0;
+    for (i=0; i<len; i++) {
+	x = gretl_vector_get(m, i);
+	if (!isnan(x)) {
+	    n = g_utf8_strlen(_(strs[j]), -1);	    
+	    if (n > nmax) nmax = n;
+	    if (i == 2) {
+		n = g_utf8_strlen(_(strs[j+1]), -1);
+		if (n > nmax) nmax = n;
+	    }
+	}
+	j += (i == 2)? 2 : 1;
+    }
+
+    nmax += 2;
+
+    pputs(prn, "  ");
+    pputs(prn, _("Forecast evaluation statistics"));
+    pputs(prn, "\n\n");
+
+    j = 0;
+    for (i=0; i<len; i++) {
+	x = gretl_vector_get(m, i);
+	if (!isnan(x)) {
+	    pprintf(prn, "  %-*s % .5g\n", UTF_WIDTH(_(strs[j]), nmax), _(strs[j]), x);
+	    if (i == 1) {
+		pprintf(prn, "  %-*s % .5g\n", UTF_WIDTH(_(strs[j+1]), nmax), 
+			_(strs[j+1]), sqrt(x));	
+	    }
+	}
+	j += (i == 1)? 2 : 1;
+    }
+    pputc(prn, '\n');
+    
+    gretl_matrix_free(m);
+
+    return err;
+}
+
 #define SIGMA_MIN 1.0e-18
 
-int
-text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, 
-		      PRN *prn)
+int text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, 
+			  PRN *prn)
 {
     int kstep = fr->method == FC_KSTEP;
     int t, anyast = 0;
     double yt, yf, et;
-    double MSE = 0.0;
-    double AE = 0.0;
-    int effn = 0;
     int err = 0;
 
     fit_resid_head(fr, pdinfo, prn); 
@@ -2386,11 +2444,7 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo,
 	} else {
 	    int ast = 0;
 
-	    if (kstep) {
-		MSE += et * et;
-		AE += fabs(et);
-		effn++;
-	    } else if (fr->sigma > SIGMA_MIN) {
+	    if (!kstep && fr->sigma > SIGMA_MIN) {
 		ast = (fabs(et) > 2.5 * fr->sigma);
 		if (ast) {
 		    anyast = 1;
@@ -2416,12 +2470,7 @@ text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo,
 		     "2.5 standard errors\n"));
     }
 
-    if (effn > 0) {
-	MSE /= effn;
-	pprintf(prn, "%s = %g\n", _("Mean Squared Error"), MSE);
-	pprintf(prn, "%s = %g\n", _("Root Mean Squared Error"), sqrt(MSE));
-	pprintf(prn, "%s = %g\n", _("Mean Absolute Error"), AE / effn);
-    }
+    print_fcast_stats(fr, OPT_NONE, prn);
 
     if (kstep && fr->nobs > 0 && gretl_in_gui_mode()) {
 	err = plot_fcast_errs(fr, NULL, pdinfo, OPT_NONE);
@@ -2483,7 +2532,11 @@ int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo,
 	}
     }
 
-    pprintf(prn, "\n     %s ", _("Obs"));
+    if (!(opt & OPT_Q)) {
+	pputc(prn, '\n');
+    }
+
+    pprintf(prn, "     %s ", _("Obs"));
     pprintf(prn, "%12s", fr->depvar);
     pprintf(prn, "%*s", UTF_WIDTH(_("prediction"), 14), _("prediction"));
 
@@ -2534,10 +2587,17 @@ int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo,
 
     pputc(prn, '\n');
 
+    if (!(opt & OPT_N)) {
+	print_fcast_stats(fr, OPT_D, prn);
+    }
+
     /* do we really want a plot for non-time series? */
 
     if ((opt & OPT_P) && fr->nobs > 0) {
 	err = plot_fcast_errs(fr, maxerr, pdinfo, opt);
+	if (!err && (opt & OPT_G)) {
+	    pprintf(prn, _("wrote %s\n"), gretl_plotfile());
+	}
     }
 
     if (maxerr != NULL) {
