@@ -100,53 +100,49 @@ static char *printf_get_string (char *s, double ***pZ,
 				DATAINFO *pdinfo, int t, 
 				int *err)
 {
-    const char *p = NULL;
-    const char *q = NULL;
     char *ret = NULL;
-    int len = 0;
-
-    /* special, not yet in genr */
 
     if (!strncmp(s, "marker", 6) && pdinfo != NULL && pdinfo->S != NULL) {
-	/* observation label */
+	/* special, not in regular genr */
+	const char *p = pdinfo->S[t];
+	const char *q = s + 6;
+	int len = strlen(p);
 	int offset = 0;
 
-	p = pdinfo->S[t];
-	len = strlen(p);
-	q = s + 6;
 	while (isspace(*q)) q++;
+
 	if (*q == '+') {
-	    q++;
-	    offset = atoi(q);
+	    offset = atoi(++q);
 	    len -= offset;
 	}
 	if (len >= 0) {
 	    ret = gretl_strndup(p + offset, len);
+	    if (ret == NULL) {
+		*err = E_ALLOC;
+	    }
 	}
     } else {
 	ret = generate_string(s, pZ, pdinfo, err);
     }
 
-    if (ret == NULL) {
-	ret = gretl_strdup("");
-    }
-
-    if (ret == NULL) {
-	*err = E_ALLOC;
+    if (ret == NULL && !*err) {
+	*err = E_DATA;
     }
 
     return ret;
 }
 
-static double printf_get_scalar (char *s, double ***pZ,
-				 DATAINFO *pdinfo, int t, 
-				 int *err)
+/* here we're trying to get a field width or precision
+   specifier */
+
+static int printf_get_int (const char *s, double ***pZ,
+			   DATAINFO *pdinfo, int *err)
 {
-    double x = NADBL;
-    int v;
+    double x;
+    int ret = 0;
 
 #if PSDEBUG
-    fprintf(stderr, "printf_get_scalar: looking at '%s'\n", s);
+    fprintf(stderr, "printf_get_int: looking at '%s'\n", s);
 #endif
 
     if (numeric_string(s)) {
@@ -154,23 +150,71 @@ static double printf_get_scalar (char *s, double ***pZ,
     } else if (gretl_is_scalar(s)) {
 	x = gretl_scalar_get_value(s);
     } else {
-	v = current_series_index(pdinfo, s);
+	x = generate_scalar(s, pZ, pdinfo, err);
+    }
 
-	if (v >= 0) {
-	    char genstr[32];
+    if (!*err && (xna(x) || fabs(x) > 255)) {
+	*err = E_DATA;
+    } 
 
-	    sprintf(genstr, "%s[%d]", s, t + 1);
-	    x = generate_scalar(genstr, pZ, pdinfo, err);
-	} else {
-	    x = generate_scalar(s, pZ, pdinfo, err);
-	}
+    if (!*err) {
+	ret = (int) x;
     }
 
 #if PSDEBUG
-    fprintf(stderr, "printf_get_scalar: returning %g\n", x);
+    fprintf(stderr, "printf_get_int: returning %g\n", x);
 #endif
 
-    return x;
+    return ret;
+}
+
+/* gen_arg_val: when this function is called we have already
+   determined that the argument string @s is not associated with a
+   string format specifier, is not a literal numerical value, and is
+   not the name of an existing scalar, matrix or series.  So now we
+   attempt to generate a value (of type unknown).
+*/
+
+static int gen_arg_val (const char *s, double ***pZ, DATAINFO *pdinfo, 
+			double *px, gretl_matrix **pm, int *pv,
+			int *scalar)
+{
+    const char *name = "$ptmp";
+    char *genstr;
+    int err = 0;
+
+    genstr = gretl_strdup_printf("%s=%s", name, s); 
+    if (genstr == NULL) {
+	return E_ALLOC;
+    }
+
+    err = generate(genstr, pZ, pdinfo, OPT_P, NULL);
+
+    if (!err) {
+	int type = genr_get_last_output_type();
+
+	if (type == GRETL_TYPE_DOUBLE) {
+	    *scalar = 1;
+	    *px = gretl_scalar_get_value(name);
+	    gretl_scalar_delete(name, NULL);
+	} else if (type == GRETL_TYPE_MATRIX) {
+	    gretl_matrix *m = get_matrix_by_name(name);
+
+	    *pm = gretl_matrix_copy(m);
+	    if (*pm == NULL) {
+		err = E_ALLOC;
+	    }
+	    user_matrix_destroy_by_name(name, NULL);
+	} else if (type == GRETL_TYPE_SERIES) {
+	    *pv = current_series_index(pdinfo, name);
+	} else {
+	    err = E_TYPES; /* FIXME cleanup? */
+	}
+    }
+
+    free(genstr);
+
+    return err;
 }
 
 /* dup argv (s) up to the next free comma */
@@ -324,7 +368,8 @@ get_printf_format_chunk (const char *s, int *fc,
 }
 
 /* extract the next conversion spec from *pfmt, find and evaluate the
-   corresponding elements in *pargs, and print the result */
+   corresponding elements in *pargs, and print the result 
+*/
 
 static int print_arg (char **pfmt, char **pargs, 
 		      double ***pZ, DATAINFO *pdinfo,
@@ -335,11 +380,14 @@ static int print_arg (char **pfmt, char **pargs,
     char *arg = NULL;
     char *str = NULL;
     gretl_matrix *m = NULL;
+    int free_m = 0;
     int series_v = -1;
+    int free_v = 0;
     double x = NADBL;
     int flen = 0, alen = 0;
     int wstar = 0, pstar = 0;
     int wid = 0, prec = 0;
+    int got_scalar = 0;
     int fc = 0;
     int err = 0;
 
@@ -360,10 +408,7 @@ static int print_arg (char **pfmt, char **pargs,
 	/* evaluate field width specifier */
 	arg = get_next_arg(*pargs, &alen, &err);
 	if (!err) {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && (na(x) || fabs(x) > 255)) {
-		err = E_DATA;
-	    }
+	    x = printf_get_int(arg, pZ, pdinfo, &err);
 	}
 	free(arg);
 	if (err) {
@@ -377,10 +422,7 @@ static int print_arg (char **pfmt, char **pargs,
 	/* evaluate precision specifier */
 	arg = get_next_arg(*pargs, &alen, &err);
 	if (!err) {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && (na(x) || fabs(x) > 255)) {
-		err = E_DATA;
-	    }
+	    x = printf_get_int(arg, pZ, pdinfo, &err);
 	}
 	free(arg);
 	if (err) {
@@ -392,30 +434,50 @@ static int print_arg (char **pfmt, char **pargs,
 
     /* get next substantive arg */
     arg = get_next_arg(*pargs, &alen, &err);
+
     if (!err) {
-	int v;
+	int v = -1;
 
 	if (fc == 's') {
+	    /* must be printing a string */
 	    str = printf_get_string(arg, pZ, pdinfo, t, &err);
+	} else if (numeric_string(arg)) {
+	    /* printing a scalar value */
+	    x = dot_atof(arg);
+	    got_scalar = 1;
+	} else if (gretl_is_scalar(arg)) {
+	    /* printing a named scalar */
+	    x = gretl_scalar_get_value(arg);
+	    got_scalar = 1;
 	} else if ((m = get_matrix_by_name(arg)) != NULL) {
-	    ; /* OK, we'll print the matrix */
+	    ; /* printing a named matrix */
 	} else if ((v = current_series_index(pdinfo, arg)) >= 0) {
-	    series_v = v; /* OK, we'll print the series */
+	    /* printing a named series */
+	    series_v = v;
 	} else {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && na(x)) {
-		fc = fmt[flen - 1] = 's';
-		str = gretl_strdup("NA");
+	    /* try treating as generated value */
+	    err = gen_arg_val(arg, pZ, pdinfo, &x, &m, &v, &got_scalar);
+	    if (m != NULL) {
+		free_m = 1;
+	    } else if (v > 0) {
+		series_v = v;
+		free_v = 1;
 	    }
-	}
+	} 
 	*pargs += alen;
     }
+
     free(arg);
 
     if (err) {
 	goto bailout;
     } 
 
+    if (got_scalar && na(x)) {
+	fc = fmt[flen - 1] = 's';
+	str = gretl_strdup("NA");
+    }	
+
     /* do the actual printing */
 
     if (m != NULL) {
@@ -423,7 +485,7 @@ static int print_arg (char **pfmt, char **pargs,
 	if (!wstar) wid = -1;
 	if (!pstar) prec = -1;
 	gretl_matrix_print_with_format(m, fmt, wid, prec, prn);
-    } else if (series_v > 0) {
+    } else if (series_v >= 0) {
 	/* printing a series */
 	printf_series(series_v, (const double **) *pZ,
 		      pdinfo, fmt, wid, prec, 
@@ -448,9 +510,11 @@ static int print_arg (char **pfmt, char **pargs,
 	} else {
 	    pprintf(prn, fmt, (int) x);
 	}
-    } else {
-	/* printing a scalar value */
-	x *= (1 + 0x1p-52);
+    } else if (got_scalar) {
+	/* printing a (non-missing) scalar value */
+	if (!isnan(x) && !isinf(x)) {
+	    x *= (1 + 0x1p-52);
+	}
 	if (wstar && pstar) {
 	    pprintf(prn, fmt, wid, prec, x);
 	} else if (wstar || pstar) {
@@ -470,6 +534,12 @@ static int print_arg (char **pfmt, char **pargs,
     free(fmt);
     free(str);
     
+    if (free_m) {
+	gretl_matrix_free(m);
+    } else if (free_v) {
+	dataset_drop_last_variables(1, pZ, pdinfo);
+    }
+    
     return err;
 }
 
@@ -664,7 +734,7 @@ sscanf_target_var (const char *vname, DATAINFO *pdinfo, int *err)
     if (gretl_is_scalar(vname)) {
 	return 0;
     } else {
-	strcpy(gretl_errmsg, _("sscanf: numerical target must be scalar"));
+	gretl_errmsg_set(_("sscanf: numerical target must be scalar"));
 	*err = E_DATA;
 	return 0;
     }
@@ -814,7 +884,7 @@ scan_string (const char *targ, char **psrc, int width,
 
     if (targ != NULL) {
 	if (!is_user_string(targ)) {
-	    sprintf(gretl_errmsg, _("%s: not a string variable"), targ);
+	    gretl_errmsg_sprintf(_("%s: not a string variable"), targ);
 	    err = E_UNKVAR;
 	} else if (n > 0) {
 	    char *conv = gretl_strndup(*psrc, n);
@@ -1049,28 +1119,16 @@ static int get_literal_length (const char *s)
     return n;
 }
 
-/* split line into source string, format and arguments: 
-   acceptable sources are string literals or string variables
-*/
-
-static int 
-split_scanf_line (const char *s, char **src, char **format, 
-		  char **args, int *literal)
+static char *get_literal_or_stringvar (const char **src, int *err)
 {
-    const char *p;
-    char *srcname;
-    int gotcom = 0;
+    const char *s = *src;
+    char *ret = NULL;
+    char *tmp = NULL;
+    int literal = 0;
     int n = 0;
 
-    if (!strncmp(s, "sscanf ", 7)) {
-	s += 7;
-    } 
-
-    s += strspn(s, " ");
-
-    /* string literal or string variable as source */
     if (*s == '"') {
-	*literal = 1;
+	literal = 1;
 	s++;
 	n = get_literal_length(s);
     } else {
@@ -1079,77 +1137,104 @@ split_scanf_line (const char *s, char **src, char **format,
     } 
 
     if (n == 0) {
-	return E_PARSE;
+	/* nothing there */
+	*err = E_PARSE;
+	return NULL;
     }
 
-    srcname = gretl_strndup(s, n);
-    if (srcname == NULL) {
-	return E_ALLOC;
+    tmp = gretl_strndup(s, n);
+    if (tmp == NULL) {
+	*err = E_ALLOC;
+	return NULL;
     }
 
-    if (*literal) {
-	*src = srcname;
+    if (literal) {
+	ret = tmp;
 	s++;
     } else {
-	*src = get_string_by_name(srcname);
-	free(srcname);
-	if (*src == NULL) {
-	    return E_UNKVAR;
+	char *svar = get_string_by_name(tmp);
+
+	if (svar == NULL) {
+	    *err = E_UNKVAR;
+	} else {
+	    ret = gretl_strdup(svar);
+	    if (ret == NULL) {
+		*err = E_ALLOC;
+	    }
 	}
+	free(tmp);
     }
 
-    s += n;
-    if (*s == ',') {
-	gotcom = 1;
-	s++;
-    }
+    /* advance the caller's pointer */
+    *src = s + n;
+
+    return ret;
+}
+
+/* Eat white space; detect comma and eat if found; eat more white
+   space.  Return 1 if comma was found else 0.
+*/
+
+int eat_comma (const char **src) 
+{
+    const char *s = *src;
+    int ret = 0;
 
-    /* skip to next field */
     s += strspn(s, " ");
-    if (!gotcom && *s == ',') {
+
+    if (*s == ',') {
+	ret = 1;
 	s++;
 	s += strspn(s, " ");
     }
 
-    /* get format string */
-    if (*s != '"' || *(s+1) == '\0') {
+    *src = s;
+
+    return ret;
+}
+
+/* split line into source string, format and arguments: 
+   acceptable sources are string literals or string variables
+*/
+
+static int 
+split_scanf_line (const char *s, char **psrc, char **pfmt, 
+		  char **args)
+{
+    int gotcom, err = 0;
+
+    if (!strncmp(s, "sscanf ", 7)) {
+	s += 7;
+    } 
+
+    s += strspn(s, " ");
+
+    /* get the source string */
+    *psrc = get_literal_or_stringvar(&s, &err);
+    if (err) {
+	return err;
+    }
+    
+    /* skip to format string */
+    gotcom = eat_comma(&s);
+    if (!gotcom) {
+	/* empty format */
 	return E_PARSE;
     }
 
-    s++;
-    p = s;
-
-    n = 0;
-    while (*s) {
-	if (*s == '"' && *(s-1) != '\\') {
-	    break;
-	}
-	n++;
-	s++;
+    /* get format string */
+    *pfmt = get_literal_or_stringvar(&s, &err);
+    if (err) {
+	return err;
     }
 
-    if (n == 0) {
-	/* empty format string */
-	return 0; 
-    }
-
-    *format = gretl_strndup(p, n);
-    if (*format == NULL) {
-	return E_ALLOC;
-    }
-
-    s++;
-    s += strspn(s, " ");
-
-    if (*s != ',') {
+    /* skip to first arg */
+    gotcom = eat_comma(&s);
+    if (!gotcom) {
 	/* empty args */
-	*args = NULL;
-	return 0;
+	return E_PARSE;
     }
 
-    s++;
-    s += strspn(s, " ");
-
     *args = gretl_strdup(s);
     if (*args == NULL) {
 	return E_ALLOC;
@@ -1185,19 +1270,21 @@ int do_sscanf (const char *line, double ***pZ,
     char *src = NULL;
     char *format = NULL;
     char *args = NULL;
-    int literal = 0;
     int err = 0;
 
     nscan = 0;
     gretl_error_clear();
 
-    err = split_scanf_line(line, &src, &format, &args, &literal);
+#if PSDEBUG
+    fprintf(stderr, "do_sscanf: line = '%s'\n", line);
+#endif
+
+    err = split_scanf_line(line, &src, &format, &args);
     if (err) {
 	return err;
     }
 
 #if PSDEBUG
-    fprintf(stderr, "do_sscanf: line = '%s'\n", line);
     fprintf(stderr, " src = '%s'\n", src);
     fprintf(stderr, " format = '%s'\n", format);
     fprintf(stderr, " args = '%s'\n", args);
@@ -1228,14 +1315,11 @@ int do_sscanf (const char *line, double ***pZ,
 	}
     }
 
-    if (literal) {
-	free(src);
-    }
-
+    free(src);
     free(format);
     free(args);
 
-    if (!err && gretl_messages_on()) {
+    if (!err && gretl_messages_on() && !gretl_looping_quietly()) {
 	pprintf(prn, "Number of items successfully scanned = %d\n", 
 		nscan);
     }
