@@ -155,6 +155,27 @@ static double binomial_critval (double p, int n, double a)
     return (double) (k + 1);
 }
 
+/**
+ * binomial_pmf:
+ * @p: success probability.
+ * @n: number of trials.
+ * @k: number of successes.
+ * 
+ * Returns: the probability mass for @k successes in @n
+ * binomial trials with success probability @p.
+ */
+
+double binomial_pmf (double p, int n, int k)
+{
+    double pm = binomial_cdf(p, n, k);
+
+    if (k > 0 && !na(pm)) {
+	pm -= binomial_cdf(p, n, k - 1);
+    }
+
+    return pm;
+}
+
 /**
  * x_factorial:
  * @x: input value.
@@ -972,7 +993,7 @@ double normal_critval (double a)
 
 static int normal_pdf_array (double *x, int n)
 {
-    double s = 1.0 / sqrt(M_2PI);
+    double s = 1.0 / SQRT_2_PI;
     int i;
 
     for (i=0; i<n; i++) {
@@ -1000,29 +1021,7 @@ static int normal_pdf_array (double *x, int n)
 
 double normal_pdf (double x)
 {
-    return (1 / sqrt(M_2PI)) * exp(-0.5 * x * x);
-}
-
-/**
- * general_normal_pdf:
- * @mu: mean.
- * @sigma: standard deviation.
- * @x: abscissa value.
- * 
- * Returns: the value of the normal PDF with mean @mu
- * and standard deviation @sigma evaluated at @x.
- */
-
-double general_normal_pdf (double mu, double sigma, double x)
-{
-    if (na(x) || na(mu) || na(sigma) || sigma <= 0.0) {
-	return NADBL;
-    } else {
-	double d = x - mu;
-
-	return (1 / (sigma * sqrt(M_2PI))) * 
-	    exp(-0.5 * d * d / (sigma * sigma));
-    }
+    return exp(-0.5 * x * x) / SQRT_2_PI;
 }
 
 /**
@@ -1035,7 +1034,7 @@ double general_normal_pdf (double mu, double sigma, double x)
 
 double log_normal_pdf (double x)
 {
-    return (x * x) / 2 - 0.91893853320467274178;
+    return (x * x) / 2 - LN_SQRT_2_PI;
 }
 
 /**
@@ -1291,7 +1290,16 @@ static double poisson_cdf_comp (double lambda, int k)
     return x;
 }
 
-static double poisson_pmf (double lambda, int k)
+/**
+ * poisson_pmf:
+ * @lambda: mean (also variance).
+ * @k: test value.
+ *
+ * Returns: the probability mass at @k, for an r.v. that follows
+ * the Poisson distribution with parameter @lambda.
+ */
+
+double poisson_pmf (double lambda, int k)
 {
     double den, l0, p;
 
@@ -1719,7 +1727,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'n':
     case 'N':
     case '1':
-	pprintf(prn, "\n%s: ", _("Standard normal"));
+	pprintf(prn, "%s: ", _("Standard normal"));
 	err = print_pv_string(p[0], pv, prn);
 	if (err) return;
 	if (pv < 0.5) {
@@ -1735,7 +1743,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
 
     case 't':
     case '2':
-	pprintf(prn, "\nt(%d): ", (int) p[0]);
+	pprintf(prn, "t(%d): ", (int) p[0]);
 	err = print_pv_string(p[1], pv, prn);
 	if (err) return;
 	if (pv < 0.5) {
@@ -1753,7 +1761,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'x':
     case 'c':
     case '3':
-	pprintf(prn, "\n%s(%d): ", _("Chi-square"), (int) p[0]);
+	pprintf(prn, "%s(%d): ", _("Chi-square"), (int) p[0]);
 	err = print_pv_string(p[1], pv, prn);
 	if (err) return;
 	pc = chisq_cdf(p[0], p[1]);
@@ -1763,7 +1771,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'F':
     case 'f':
     case '4':
-	pprintf(prn, "\nF(%d, %d): ", (int) p[0], (int) p[1]);
+	pprintf(prn, "F(%d, %d): ", (int) p[0], (int) p[1]);
 	err = print_pv_string(p[2], pv, prn);
 	if (err) return;
 	pc = snedecor_cdf((int) p[0], (int) p[1], p[2]);
@@ -1773,7 +1781,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'G':
     case 'g':
     case '5':
-	pprintf(prn, _("\nGamma (shape %g, scale %g, mean %g, variance %g):"
+	pprintf(prn, _("Gamma (shape %g, scale %g, mean %g, variance %g):"
 		       "\n area to the right of %g = %g\n"), 
 		p[0], p[1], p[0] * p[1], p[0] * p[1] * p[1],
 		p[2], pv);
@@ -1782,7 +1790,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'B':
     case 'b':
     case '6':
-	pprintf(prn, _("\nBinomial (p = %g, n = %d):"
+	pprintf(prn, _("Binomial (p = %g, n = %d):"
 		       "\n Prob(x > %d) = %g\n"), 
 		p[0], (int) p[1], (int) p[2], pv);
 	pc = binomial_cdf(p[0], p[1], p[2]);
@@ -1798,7 +1806,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'p':
     case 'P':
     case '8':
-	pprintf(prn, _("\nPoisson (mean = %g): "), p[0]);
+	pprintf(prn, _("Poisson (mean = %g): "), p[0]);
 	err = print_pv_string(p[1], pv, prn);
 	if (err) return;
 	pc = poisson_cdf(p[0], (int) p[1]);
@@ -1814,7 +1822,7 @@ void print_pvalue (char st, double *p, double pv, PRN *prn)
     case 'w':
     case 'W':
     case '9':
-	pprintf(prn, _("\nWeibull (shape = %g, scale = %g): "), 
+	pprintf(prn, _("Weibull (shape = %g, scale = %g): "), 
 		p[0], p[1]);
 	err = print_pv_string(p[2], pv, prn);
 	if (err) return;
