@@ -50,6 +50,13 @@ int string_is_blank (const char *s)
     return ret;
 }
 
+static int atof_point;
+
+void set_atof_point (char c)
+{
+    atof_point = c;
+}
+
 /**
  * dot_atof:
  * @s: the string to convert.
@@ -61,29 +68,26 @@ int string_is_blank (const char *s)
 
 double dot_atof (const char *s)
 {
+#ifndef ENABLE_NLS
+    return atof(s);
+#else
     double x;
 
-#ifdef ENABLE_NLS
-    static int decpoint = 0;
+    if (atof_point == 0) {
+	struct lconv *lc = localeconv();
 
-    if (decpoint == 0) {
-	struct lconv *lc;
-
-	lc = localeconv();
-	decpoint = *lc->decimal_point;
+	atof_point = *lc->decimal_point;
     }
-    if (decpoint == '.') {
+    if (atof_point == '.') {
 	x = atof(s);
     } else {
 	gretl_push_c_numeric_locale();
 	x = atof(s);
 	gretl_pop_c_numeric_locale();
     }
-#else
-    x = atof(s);
-#endif
 
     return x;
+#endif
 }
 
 /**
@@ -293,10 +297,11 @@ char *comma_separate_numbers (char *s)
 /**
  * has_suffix:
  * @str: the string to check.
- * @sfx: the suffix to check for.
+ * @sfx: the suffix to check for, including a leading '.'
+ * if applicable.
  *
  * Returns: 1 if @str ends with @sfx (on a case-insensitive
- * comparison), 0 otherwise.
+ * comparison), 0 otherwise.  
  */
 
 int has_suffix (const char *str, const char *sfx)
@@ -863,7 +868,7 @@ char *chopstr (char *str)
 
     n = strlen(str);
 
-    for (i = n - 1; i >= 0; i--) {
+    for (i=n-1; i>=0; i--) {
 	if (isspace(str[i]) || str[i] == '\r') {
 	    str[i] = '\0';
 	} else {
@@ -1417,6 +1422,33 @@ char **strings_array_dup (char **strs, int n)
     return S;
 }
 
+/**
+ * strings_array_cmp:
+ * @strs1: first array of strings.
+ * @strs2: second array of strings.
+ * @n: number of strings to examine.
+ *
+ * Compares for equality two arrays of strings, each of
+ * which must contain at least @n elements.  Equality
+ * of the arrays means that %strcmp returns 0 for
+ * each pair of strings @strs1[i], @strs2[i], for i
+ * equals 0 to @n - 1.
+ *
+ * Returns: 0 if the arrays compare equal, non-zero
+ * otherwise.
+ */
+
+int strings_array_cmp (char **strs1, char **strs2, int n)
+{
+    int i, ret = 0;
+
+    for (i=0; i<n && !ret; i++) {
+	ret = strcmp(strs1[i], strs2[i]);
+    }
+
+    return ret;
+}
+
 /**
  * free_strings_array:
  * @strs: array of allocated strings.
@@ -1438,45 +1470,6 @@ void free_strings_array (char **strs, int nstrs)
     }
 }
 
-static int maybe_year_plus (const char *s)
-{
-    int n = strlen(s);
-
-    if (n > 8 && isdigit(s[0]) && isdigit(s[1]) &&
-	isdigit(s[2]) && isdigit(s[3]) && !isdigit(s[4])) {
-	return 1;
-    }
-
-    return 0;
-}
-
-static char *
-real_get_obs_string (char *s, int t, const DATAINFO *pdinfo, int full)
-{
-    if (pdinfo->markers && pdinfo->S != NULL) { 
-	/* data marker strings present */
-	if (full) {
-	    strcpy(s, pdinfo->S[t]);
-	} else {
-	    *s = '\0';
-	    if (maybe_year_plus(pdinfo->S[t])) {
-		/* daily date, year first? */
-		strncat(s, pdinfo->S[t] + 2, 8);
-	    } else {
-		strncat(s, pdinfo->S[t], 8);
-	    }
-	}
-    } else {
-	if (full) {
-	    ntodate_full(s, t, pdinfo);
-	} else {
-	    ntodate(s, t, pdinfo);
-	}
-    }
-
-    return s;
-}
-
 /**
  * get_obs_string:
  * @obs: char array big enough to hold the observation (#OBSLEN).
@@ -1488,22 +1481,14 @@ real_get_obs_string (char *s, int t, const DATAINFO *pdinfo, int full)
 
 char *get_obs_string (char *obs, int t, const DATAINFO *pdinfo)
 {
-    return real_get_obs_string(obs, t, pdinfo, 0);
-}
+    if (pdinfo->markers && pdinfo->S != NULL) { 
+	/* data marker strings present */
+	strcpy(obs, pdinfo->S[t]);
+    } else {
+	ntodate(obs, t, pdinfo);
+    }
 
-/**
- * get_full_obs_string:
- * @obs: char array big enough to hold the observation (#OBSLEN).
- * @t: zero-based observation number.
- * @pdinfo: pointer to dataset information.
- *
- * Returns: the observation string corresponding to @t, using
- * a four-digit year in case of dated daily data.
- */
-
-char *get_full_obs_string (char *obs, int t, const DATAINFO *pdinfo)
-{
-    return real_get_obs_string(obs, t, pdinfo, 1);
+    return obs;
 }
 
 /**
@@ -1591,7 +1576,7 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
     } else if (pdinfo->structure != CROSS_SECTION) {
 	char tmp[OBSLEN];
 
-	ntodate_full(tmp, t, pdinfo);
+	ntodate(tmp, t, pdinfo);
 	if (quarterly_or_monthly(pdinfo)) {
 	    modify_date_for_csv(tmp, pdinfo->pd);
 	}
@@ -1601,23 +1586,21 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
 	
 /**
  * print_time:
- * @timep: time to print.
+ * @s: string into which to print: must be at least 48 bytes.
  *
- * Returns: pointer to a static string containing a locale-dependent
- * representation of @timep.  In English, this will be in the format
- * Y/m/d H:M.
+ * Returns: @s, which will contain a locale-dependent representation 
+ * of the current time.  In English, this will be in the format Y/m/d H:M.
  */
 
-const char *print_time (const time_t *timep)
+char *print_time (char *s)
 {
-    static char timestr[48];
+    time_t now = time(NULL);
     struct tm *local;
 
-    local = localtime(timep);
+    local = localtime(&now);
+    strftime(s, 47, "%Y-%m-%d %H:%M", local);
 
-    strftime(timestr, 47, "%Y/%m/%d %H:%M", local);
-
-    return timestr;
+    return s;
 }
 
 /**
@@ -1666,7 +1649,7 @@ char *gretl_xml_encode (const char *str)
 
     targ = malloc(len);
     if (targ == NULL) {
-	sprintf(gretl_errmsg, _("out of memory in XML encoding"));
+	gretl_errmsg_set(_("out of memory in XML encoding"));
 	return NULL;
     }
 
@@ -1967,26 +1950,6 @@ const char *path_last_element (const char *path)
     return p;
 }
 
-/**
- * ensure_slash:
- * @s: string to work on.
- *
- * If @s does not ends with #SLASH, append this character.
- *
- * Returns: the (possibly) modified string.
- */
-
-char *ensure_slash (char *s)
-{
-    int n = strlen(s);
-
-    if (n > 0 && s[n-1] != SLASH) {
-	strcat(s, SLASHSTR);
-    }
-
-    return s;
-}
-
 /**
  * trim_slash:
  * @s: string to work on.
@@ -2030,58 +1993,24 @@ int gretl_string_ends_with (const char *s, const char *test)
     return ret;
 }
 
-int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
+/**
+ * get_column_widths:
+ * @strs: array of @n strings.
+ * @widths: array of @n default column widths.
+ * @n: number of columns.
+ *
+ * If need be, increases the column widths in @widths to
+ * accomodate the current translations of @strs.
+ */
+
+void get_column_widths (const char **strs, int *widths, int n)
 {
-    GPatternSpec *pspec;
-    int *list = NULL;
-    int i, n = 0;
+    int i, len;
 
-    if (pdinfo == NULL) {
-	return NULL;
-    }
-
-    pspec = g_pattern_spec_new(pattern);
-
-    for (i=1; i<pdinfo->v; i++) { 
-	if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
-	    n++;
+    for (i=0; i<n; i++) {
+	len = g_utf8_strlen(_(strs[i]), -1);
+	if (len > widths[i]) {
+	    widths[i] = len;
 	}
     }
-
-    if (n > 0) {
-	list = malloc((n + 1) * sizeof *list);
-	if (list != NULL) {
-	    int j = 1;
-
-	    list[0] = n;
-	    for (i=1; i<pdinfo->v; i++) { 
-		if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
-		    list[j++] = i;
-		}
-	    }
-	}
-    }
-
-    g_pattern_spec_free(pspec);
-
-    return list;
-}
-
-int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
-{
-    GPatternSpec *pspec;
-    int i, ret = 0;
-
-    pspec = g_pattern_spec_new(pattern);
-
-    for (i=1; i<pdinfo->v; i++) { 
-	if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
-	    ret = 1;
-	    break;
-	}
-    }
-
-    g_pattern_spec_free(pspec);
-
-    return ret;
 }
