@@ -48,6 +48,8 @@ int integer_string (const char *str);
 int count_fields (const char *s);
 
 double dot_atof (const char *s);
+
+void set_atof_point (char c);
  
 int dotpos (const char *str);
 
@@ -122,12 +124,12 @@ char **strings_array_new_with_length (int nstrs, int len);
 
 char **strings_array_dup (char **strs, int n);
 
+int strings_array_cmp (char **strs1, char **strs2, int n);
+
 void free_strings_array (char **strs, int nstrs);
 
 char *get_obs_string (char *obs, int t, const DATAINFO *pdinfo);
 
-char *get_full_obs_string (char *obs, int t, const DATAINFO *pdinfo);
-
 double obs_str_to_double (const char *obs);
 
 char *colonize_obs (char *obs);
@@ -136,7 +138,7 @@ void modify_date_for_csv (char *s, int pd);
 
 void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn);
 
-const char *print_time (const time_t *timep);
+char *print_time (char *s);
 
 int gretl_xml_validate (const char *s);
 
@@ -157,14 +159,10 @@ char *build_path (char *targ, const char *dirname, const char *fname,
 
 const char *path_last_element (const char *path);
 
-char *ensure_slash (char *s);
-
 char *trim_slash (char *s);
 
 int gretl_string_ends_with (const char *s, const char *test);
 
-int *varname_match_list (const DATAINFO *pdinfo, const char *pattern);
-
-int varname_match_any (const DATAINFO *pdinfo, const char *pattern);
+void get_column_widths (const char **strs, int *widths, int n);
 
 #endif /* STRUTILS_H */
