@@ -19,6 +19,8 @@
 
 /* subsample.c for gretl */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "libset.h"
 #include "gretl_func.h"
@@ -26,6 +28,7 @@
 #include "cmd_private.h"
 #include "dbread.h"
 #include "gretl_scalar.h"
+#include "gretl_xml.h"
 
 #define SUBDEBUG 0
 #define FULLDEBUG 0
@@ -110,7 +113,7 @@ void free_subsample_mask (char *s)
     }
 }
 
-char *copy_subsample_mask (const char *src)
+char *copy_subsample_mask (const char *src, int *err)
 {
     char *ret = NULL;
 
@@ -122,18 +125,20 @@ char *copy_subsample_mask (const char *src)
 	ret = malloc(n * sizeof *ret);
 	if (ret != NULL) {
 	    memcpy(ret, src, n);
+	} else {
+	    *err = E_ALLOC;
 	}
     }
 
     return ret;
 }
 
-char *copy_datainfo_submask (const DATAINFO *pdinfo)
+char *copy_datainfo_submask (const DATAINFO *pdinfo, int *err)
 {
     char *mask = NULL;
 
     if (complex_subsampled()) {
-	mask = copy_subsample_mask(pdinfo->submask);
+	mask = copy_subsample_mask(pdinfo->submask, err);
     }
 
     return mask;
@@ -177,6 +182,11 @@ int write_datainfo_submask (const DATAINFO *pdinfo, FILE *fp)
 	    fprintf(fp, "%d ", (int) pdinfo->submask[i]);
 	}
 	fputs("</submask>\n", fp);
+
+	if (pdinfo->restriction != NULL) {
+	    gretl_xml_put_tagged_string("restriction", pdinfo->restriction, fp);
+	}
+
 	ret = 1;
     }
 
@@ -216,6 +226,16 @@ static char *make_submask (int n)
     return mask;
 }
 
+void set_dataset_resampled (DATAINFO *pdinfo)
+{
+    pdinfo->submask = RESAMPLED;
+}
+
+int dataset_is_resampled (const DATAINFO *pdinfo)
+{
+    return (pdinfo != NULL && pdinfo->submask == RESAMPLED);
+}
+
 void maybe_free_full_dataset (const DATAINFO *pdinfo)
 {
     if (pdinfo == peerinfo) {
@@ -256,13 +276,13 @@ relink_to_full_dataset (double ***pZ, DATAINFO *pdinfo)
    have been moved via realloc
 */
 
-static void sync_dataset_elements (const DATAINFO *pdinfo)
+static void sync_datainfo_members (const DATAINFO *pdinfo)
 {
     if (fullinfo->v > pdinfo->v) {
 	int i;
 
 #if FULLDEBUG
-	fprintf(stderr, "*** sync_dataset_elements: fullinfo->v = %d but pdinfo->v = %d\n",
+	fprintf(stderr, "*** sync_datainfo_members: fullinfo->v = %d but pdinfo->v = %d\n",
 		fullinfo->v, pdinfo->v);
 	fprintf(stderr, " deleting the last %d element(s) of fullZ\n", fullinfo->v - pdinfo->v);
 #endif
@@ -299,16 +319,13 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 
     if (fullZ != NULL) {
 	/* sync, in case anything has moved */
-	sync_dataset_elements(pdinfo);
+	sync_datainfo_members(pdinfo);
 
 	if (pmod->submask != NULL) {
 	    free_subsample_mask(pmod->submask);
 	}
 
-	pmod->submask = copy_subsample_mask(pdinfo->submask);
-	if (pmod->submask == NULL) {
-	    err = E_ALLOC;
-	}
+	pmod->submask = copy_subsample_mask(pdinfo->submask, &err);
     }
 
     return err;
@@ -442,11 +459,34 @@ static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
     return 0;
 }
 
+static int sync_data_to_full (double ***pZ, DATAINFO *pdinfo)
+{
+    int err;
+
+    /* update values for pre-existing series, which may have been
+       modified via genr etc */
+    update_full_data_values((const double **) *pZ, pdinfo);
+
+    /* if case markers were added when subsampled, carry them back */
+    update_case_markers(pdinfo);
+
+    /* delete any newly added hidden vars */
+    err = dataset_destroy_hidden_variables(pZ, pdinfo, fullinfo->v);
+
+    /* in case any new vars were added when subsampled, try to merge
+       them into the full dataset */
+    if (!err) {
+	err = add_new_vars_to_full((const double **) *pZ, pdinfo);
+    }
+
+    return err;
+}
+
 static char *make_current_sample_mask (DATAINFO *pdinfo)
 {
     int n = full_data_length(pdinfo);
     char *currmask = NULL;
-    int s, t;
+    int s, t, err = 0;
 
     if (pdinfo->submask == NULL) {
 	/* no pre-existing mask: not sub-sampled */
@@ -458,7 +498,7 @@ static char *make_current_sample_mask (DATAINFO *pdinfo)
 	}
     } else {
 	/* there's a pre-existing mask */
-	currmask = copy_subsample_mask(pdinfo->submask);
+	currmask = copy_subsample_mask(pdinfo->submask, &err);
 	if (currmask != NULL) {
 	    s = -1;
 	    for (t=0; t<n; t++) {
@@ -530,32 +570,17 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
 
     /* reattach malloc'd elements of subsampled dataset,
        which may have moved */
-    sync_dataset_elements(pdinfo);
+    sync_datainfo_members(pdinfo);
 
     if (pdinfo->submask == RESAMPLED) {
 	/* not regular subsample but resampled dataset */
 	resample_trim_dataset(pZ, pdinfo);
     } else {
-	/* update values for pre-existing series, which may have been
-	   modified via genr etc */
-	update_full_data_values((const double **) *pZ, pdinfo);
-
-	/* if case markers were added when subsampled, carry them back */
-	update_case_markers(pdinfo);
-
-	/* delete any newly added hidden vars */
-	err = dataset_destroy_hidden_variables(pZ, pdinfo, fullinfo->v);
-
-	/* in case any new vars were added when subsampled, try to merge
-	   them into the full dataset */
-	if (!err) {
-	    err = add_new_vars_to_full((const double **) *pZ, pdinfo);
-	}
+	err = sync_data_to_full(pZ, pdinfo);
     }
 
     if (err == E_NOMERGE) {
-        sprintf(gretl_errmsg, 
-		_("Missing sub-sample information; can't merge data\n"));
+        gretl_errmsg_set(_("Missing sub-sample information; can't merge data\n"));
     }
 
     /* destroy sub-sampled data array */
@@ -575,7 +600,8 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
     if (!err && state != NULL) {
 	/* "full" really means, relative to the original state */
 	if (state->submask != NULL) {
-	    err = restrict_sample_from_mask(state->submask, pZ, pdinfo);
+	    err = restrict_sample_from_mask(state->submask, pZ, pdinfo,
+					    OPT_NONE);
 	} else {
 	    int t1min, t2max;
 
@@ -672,7 +698,7 @@ static int mask_from_temp_dummy (const char *s, double ***pZ, DATAINFO *pdinfo,
     if (!err) {
 	err = copy_dummy_to_mask(mask, x, pdinfo->n);
 	if (err) {
-	    sprintf(gretl_errmsg, _("'%s' is not a dummy variable"), "mask");
+	    gretl_errmsg_sprintf(_("'%s' is not a dummy variable"), "mask");
 	}
     }
 
@@ -691,12 +717,12 @@ static int mask_from_dummy (const char *s, const double **Z, const DATAINFO *pdi
     dnum = series_index(pdinfo, dname);
 
     if (dnum == pdinfo->v) {
-	sprintf(gretl_errmsg, _("Variable '%s' not defined"), dname);
+	gretl_errmsg_sprintf(_("Variable '%s' not defined"), dname);
 	err = 1;
     } else {
 	err = copy_dummy_to_mask(mask, Z[dnum], pdinfo->n);
 	if (err) {
-	    sprintf(gretl_errmsg, _("'%s' is not a dummy variable"), dname);
+	    gretl_errmsg_sprintf(_("'%s' is not a dummy variable"), dname);
 	}
     }
 
@@ -780,7 +806,7 @@ static int make_random_mask (const char *s, const char *oldmask,
     }	
 
     if (err) {
-	sprintf(gretl_errmsg, _("Invalid number of cases %d"), subn);
+	gretl_errmsg_sprintf(_("Invalid number of cases %d"), subn);
 	return err;
     }	
 
@@ -931,9 +957,9 @@ static int mask_contiguous (const char *mask,
 static void 
 copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
 			const double **Z, const DATAINFO *pdinfo,
-			const char *mask)
+			int maxv, const char *mask)
 {
-    int i, t, st;
+    int i, t, s;
 
 #if SUBDEBUG
     fprintf(stderr, "copy_data_to_subsample: subinfo = %p, pdinfo = %p\n",
@@ -941,26 +967,26 @@ copy_data_to_subsample (double **subZ, DATAINFO *subinfo,
 #endif
 
     /* copy data values */
-    for (i=1; i<pdinfo->v; i++) {
-	st = 0;
+    for (i=1; i<maxv; i++) {
+	s = 0;
 	for (t=0; t<pdinfo->n; t++) {
 	    if (mask == NULL) {
-		subZ[i][st++] = Z[i][t];
+		subZ[i][s++] = Z[i][t];
 	    } else if (mask[t] == 1) {
-		subZ[i][st++] = Z[i][t];
+		subZ[i][s++] = Z[i][t];
 	    } else if (mask[t] == 'p') {
 		/* panel padding */
-		subZ[i][st++] = NADBL;
+		subZ[i][s++] = NADBL;
 	    }
 	}
     }
 
     /* copy observation markers, if any */
     if (pdinfo->markers && subinfo->markers) {
-	st = 0;
+	s = 0;
 	for (t=0; t<pdinfo->n; t++) {
 	    if (mask == NULL || mask[t] == 1 || mask[t] == 'p') {
-		strcpy(subinfo->S[st++], pdinfo->S[t]);
+		strcpy(subinfo->S[s++], pdinfo->S[t]);
 	    }
 	}
     }
@@ -986,6 +1012,78 @@ int get_restriction_mode (gretlopt opt)
     return mode;
 }
 
+/* Copy list of panel periods from Ti to T0; return 1 if members of
+   the list Ti are consecutive, 0 if they are not. 
+*/
+
+static int copy_periods_list (int *T0, const int *Ti)
+{
+    int j, ret = 1;
+
+    for (j=0; j<=Ti[0] && ret; j++) {
+	if (j > 1 && Ti[j] != Ti[j-1] + 1) {
+	    ret = 0;
+	} else {
+	    T0[j] = Ti[j];
+	}
+    }
+
+    return ret;
+}
+
+/* When sub-sampling panel data with the "--no-missing" option: see if
+   the exclusion of missing values leaves a balanced panel.  Note that
+   the requirement is not simply that each unit has the same number of
+   temporal observations -- they must have the _same_ observations, 
+   and in addition the observations must be temporally contiguous.
+*/
+
+static int mask_leaves_balanced_panel (char *mask, const DATAINFO *pdinfo)
+{
+    int *T0, *Ti;
+    int i, u, ubak = -1;
+    int ret = 1;
+
+    T0 = gretl_list_new(pdinfo->paninfo->Tmax);
+    Ti = gretl_list_new(pdinfo->paninfo->Tmax);
+
+    if (T0 == NULL || Ti == NULL) {
+	free(T0);
+	free(Ti);
+	return 0;
+    }
+
+    T0[0] = Ti[0] = 0;
+
+    for (i=0; i<pdinfo->n && ret; i++) {
+	if (mask[i]) {
+	    u = pdinfo->paninfo->unit[i];
+	    if (u != ubak) {
+		if (Ti[0] > 0) {
+		    if (T0[0] == 0) {
+			/* we haven't made the reference list, T0, yet */
+			ret = copy_periods_list(T0, Ti);
+		    } else if (gretl_list_cmp(Ti, T0)) {
+			/* the current list differs from the reference one */
+			ret = 0;
+		    }
+		}
+		Ti[0] = 1;
+		Ti[1] = pdinfo->paninfo->period[i];
+	    } else {
+		Ti[0] += 1;
+		Ti[Ti[0]] = pdinfo->paninfo->period[i];
+	    }
+	    ubak = u;
+	}
+    }
+
+    free(T0);
+    free(Ti);
+
+    return ret;
+}
+
 static int 
 make_panel_submask (char *mask, const DATAINFO *pdinfo, int *err)
 {
@@ -1066,7 +1164,7 @@ make_restriction_mask (int mode, const char *s, const int *list,
     } else if (mode == SUBSAMPLE_BOOLEAN) {
 	err = mask_from_temp_dummy(s, pZ, pdinfo, mask, prn);
     } else {
-	strcpy(gretl_errmsg, _("Sub-sample command failed mysteriously"));
+	gretl_errmsg_set(_("Sub-sample command failed mysteriously"));
 	err = 1;
     }
 
@@ -1087,7 +1185,7 @@ make_restriction_mask (int mode, const char *s, const int *list,
        sample, perchance? */
 
     if (sn == 0) {
-	strcpy(gretl_errmsg, _("No observations would be left!"));
+	gretl_errmsg_set(_("No observations would be left!"));
 	err = 1;
     } else if (sn == pdinfo->n) {
 	/* not really an error, just a no-op */
@@ -1108,14 +1206,15 @@ make_restriction_mask (int mode, const char *s, const int *list,
     return err;
 }
 
-/* This is also used in session.c, when re-establishing a previously
-   sub-sampled data state on reopening a saved session, and also on
-   exit from a user function when the dataset was sub-sampled on
+/* This is also used elsewhere: in session.c, when re-establishing a
+   previously sub-sampled data state on reopening a saved session, and
+   on exit from a user function when the dataset was sub-sampled on
    entry to the function.
 */
 
 int 
-restrict_sample_from_mask (char *mask, double ***pZ, DATAINFO *pdinfo)
+restrict_sample_from_mask (char *mask, double ***pZ, DATAINFO *pdinfo,
+			   gretlopt opt)
 {
     double **subZ = NULL;
     DATAINFO *subinfo;
@@ -1141,19 +1240,30 @@ restrict_sample_from_mask (char *mask, double ***pZ, DATAINFO *pdinfo)
 
     if (dataset_is_panel(pdinfo)) {
 	/* are we able to reconstitute a panel? */
-	int npad, nunits = count_panel_units(mask, pdinfo);
+	int nunits = count_panel_units(mask, pdinfo);
+	int ok = 0, npad = 0;
+
+	fprintf(stderr, "count_panel_units: got %d (subinfo->n = %d)\n", nunits, subinfo->n);
 
 	if (nunits > 1 && subinfo->n > nunits) {
-	    /* add padding rows if need be */
-	    npad = make_panel_submask(mask, pdinfo, &err);
-	    if (err) {
-		free(subinfo);
-		return err;
+	    if (opt & OPT_B) {
+		/* only add padding rows if this was requested */
+		npad = make_panel_submask(mask, pdinfo, &err);
+		fprintf(stderr, "npad = %d\n", npad);
+		if (err) {
+		    free(subinfo);
+		    return err;
+		}
+		ok = 1;
+	    } else {		
+		ok = mask_leaves_balanced_panel(mask, pdinfo);
+	    } 
+	    if (ok) {
+		subinfo->structure = STACKED_TIME_SERIES;
+		subinfo->n += npad;
+		subinfo->pd = subinfo->n / nunits;
+		/* note: revised panel indices are added below */
 	    }
-	    subinfo->structure = STACKED_TIME_SERIES;
-	    subinfo->n += npad;
-	    subinfo->pd = subinfo->n / nunits;
-	    /* note: panel indices are added below */
 	} else if (nunits == 1 && subinfo->n == pdinfo->pd) {
 	    /* time series for single panel unit */
 	    subinfo->structure = SPECIAL_TIME_SERIES;
@@ -1202,11 +1312,11 @@ restrict_sample_from_mask (char *mask, double ***pZ, DATAINFO *pdinfo)
 
     /* copy across data (and case markers, if any) */
     copy_data_to_subsample(subZ, subinfo, (const double **) *pZ, 
-			   pdinfo, mask);
+			   pdinfo, pdinfo->v, mask);
 
     err = backup_full_dataset(*pZ, pdinfo);
 
-    subinfo->submask = copy_subsample_mask(mask);
+    subinfo->submask = copy_subsample_mask(mask, &err);
 
     /* switch pointers */
     *pZ = subZ;
@@ -1282,6 +1392,42 @@ static int restriction_uses_obs (const char *s)
 
 #endif
 
+static int make_restriction_string (DATAINFO *pdinfo, char *old, 
+				    const char *restr, int mode)
+{
+    int err = 0;
+
+    if (pdinfo->restriction != NULL) {
+	free(pdinfo->restriction);
+	pdinfo->restriction = NULL;
+    }
+
+    if (mode == SUBSAMPLE_RANDOM) {
+	pdinfo->restriction = gretl_strdup("random");
+    } else {
+	if (old != NULL) {
+	    char *s = malloc(strlen(old) + strlen(restr) + 5);
+
+	    if (s != NULL) {
+		sprintf(s, "%s && %s", old, restr);
+		pdinfo->restriction = s;
+	    }
+	} else {	    
+	    pdinfo->restriction = gretl_strdup(restr);
+	} 
+    }
+
+    if (old != NULL) {
+	free(old);
+    }
+
+    if (pdinfo->restriction == NULL) {
+	err = E_ALLOC;
+    }
+
+    return err;
+}
+
 /* restrict_sample: 
  * @line: command line (or %NULL).  
  * @pZ: pointer to original data array.  
@@ -1304,6 +1450,10 @@ static int restriction_uses_obs (const char *s)
  * will be the logical product of the new restriction and any
  * existing restriction.
  *
+ * In case the original dataset was a panel and OPT_B was given,
+ * we'll pad with missing values if necessary, to try to reconstitute 
+ * a balanced panel.
+ *
  * Returns: 0 on success, non-zero error code on failure.
  */
 
@@ -1312,6 +1462,7 @@ int restrict_sample (const char *line, const int *list,
 		     ExecState *state, gretlopt opt, 
 		     PRN *prn)
 {
+    char *oldrestr = NULL;
     char *oldmask = NULL;
     char *mask = NULL;
     int free_oldmask = 0;
@@ -1334,7 +1485,7 @@ int restrict_sample (const char *line, const int *list,
 
     mode = get_restriction_mode(opt);
     if (mode == SUBSAMPLE_UNKNOWN) {
-	strcpy(gretl_errmsg, "Unrecognized sample command");
+	gretl_errmsg_set("Unrecognized sample command");
 	return 1;
     }
 
@@ -1345,6 +1496,9 @@ int restrict_sample (const char *line, const int *list,
 	    return E_ALLOC;
 	}
 	free_oldmask = 1;
+	if (pdinfo->restriction != NULL) {
+	    oldrestr = gretl_strdup(pdinfo->restriction);
+	}
     } else if (state != NULL && state->submask != NULL) {
 	/* subsampling within a function, with incoming
 	   restriction recorded in state
@@ -1397,7 +1551,7 @@ int restrict_sample (const char *line, const int *list,
 #if SUBDEBUG
 	    fprintf(stderr, "restrict sample: using mask\n");
 #endif
-	    err = restrict_sample_from_mask(mask, pZ, pdinfo);
+	    err = restrict_sample_from_mask(mask, pZ, pdinfo, opt);
 	}
     }
 
@@ -1407,6 +1561,8 @@ int restrict_sample (const char *line, const int *list,
 	free(oldmask);
     }
 
+    make_restriction_string(pdinfo, oldrestr, line, mode);
+
     return err;
 }
 
@@ -1526,12 +1682,12 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
 	
     if (nf == 1) {
 	if (sscanf(line, "%16s", newstart) != 1) {
-	    strcpy(gretl_errmsg, _("error reading smpl line"));
+	    gretl_errmsg_set(_("error reading smpl line"));
 	    return 1;
 	} else {
 	    new_t1 = get_sample_limit(newstart, pZ, pdinfo, SMPL_T1);
 	    if (new_t1 < tmin || new_t1 > tmax) {
-		strcpy(gretl_errmsg, _("error in new starting obs"));
+		gretl_errmsg_set(_("error in new starting obs"));
 		return 1;
 	    }
 	    pdinfo->t1 = new_t1;
@@ -1542,14 +1698,14 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
     /* now we're looking at nf = 2 (2 fields) case */
 
     if (sscanf(line, "%16s %16s", newstart, newstop) != 2) {
-	strcpy(gretl_errmsg, _("error reading smpl line"));
+	gretl_errmsg_set(_("error reading smpl line"));
 	return 1;
     }
 
     if (strcmp(newstart, ";")) {
 	new_t1 = get_sample_limit(newstart, pZ, pdinfo, SMPL_T1);
 	if (new_t1 < tmin || new_t1 > tmax) {
-	    strcpy(gretl_errmsg, _("error in new starting obs"));
+	    gretl_errmsg_set(_("error in new starting obs"));
 	    return 1;
 	}	
     }
@@ -1557,13 +1713,13 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
     if (strcmp(newstop, ";")) {
 	new_t2 = get_sample_limit(newstop, pZ, pdinfo, SMPL_T2);
 	if (new_t2 < tmin || new_t2 > tmax) {
-	    strcpy(gretl_errmsg, _("error in new ending obs"));
+	    gretl_errmsg_set(_("error in new ending obs"));
 	    return 1;
 	}
     }
 
     if (new_t1 < tmin || new_t1 > new_t2) {
-	strcpy(gretl_errmsg, _("Invalid null sample"));
+	gretl_errmsg_set(_("Invalid null sample"));
 	return 1;
     }
 
@@ -1577,15 +1733,16 @@ int set_sample (const char *line, double ***pZ, DATAINFO *pdinfo)
  * count_missing_values:
  * @Z: data array.
  * @pdinfo: dataset information.
- * @opt: use %OPT_V for verbose operation.
+ * @opt: use %OPT_V for verbose operation, %OPT_A to 
+ * examine all data.
  * @prn: printing struct.
  * @err: location to receive error code.
  *
  * Prints a count of missing values (if any) in the current
- * dataset over the currently defined sample range. If
- * %OPT_V is given this includes a count of missing values
- * at each observation; otherwise it just includes global
- * and per-variable counts.
+ * dataset over the currently defined sample range (or the 
+ * entire data range if %OPT_A is given). If %OPT_V is given 
+ * this includes a count of missing values at each observation; 
+ * otherwise it just includes global and per-variable counts.
  *
  * Returns: 0 if no missing values are found (or on error),
  * otherwise the total number of missing values.
@@ -1595,11 +1752,21 @@ int count_missing_values (const double **Z, const DATAINFO *pdinfo,
 			  gretlopt opt, PRN *prn, int *err)
 {
     int missval = 0, missobs = 0, totvals = 0, oldmiss = 0;
-    int T = pdinfo->t2 - pdinfo->t1 + 1;
+    int T, t1, t2;
     int *missvec;
     double missfrac;
     int i, t, tmiss;
 
+    if (opt & OPT_A) {
+	t1 = 0;
+	t2 = pdinfo->n - 1;
+    } else {
+	t1 = pdinfo->t1;
+	t2 = pdinfo->t2;
+    }
+
+    T = t2 - t1 + 1;
+
     missvec = malloc(pdinfo->v * sizeof missvec);
 
     if (missvec == NULL) {
@@ -1611,7 +1778,7 @@ int count_missing_values (const double **Z, const DATAINFO *pdinfo,
 	missvec[i] = 0;
     }
 
-    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+    for (t=t1; t<=t2; t++) {
 	tmiss = 0;
 	for (i=1; i<pdinfo->v; i++) {
 	    if (var_is_hidden(pdinfo, i)) {
@@ -1676,11 +1843,12 @@ int count_missing_values (const double **Z, const DATAINFO *pdinfo,
     return missval;
 }
 
-static void copy_series_info (DATAINFO *dest, const DATAINFO *src)
+static void copy_series_info (DATAINFO *dest, const DATAINFO *src,
+			      int maxv)
 {
     int i;
 
-    for (i=1; i<src->v; i++) {
+    for (i=1; i<maxv; i++) {
 	strcpy(dest->varname[i], src->varname[i]);
 	if (src->varinfo != NULL) {
 	    copy_varinfo(dest->varinfo[i], src->varinfo[i]);
@@ -1688,6 +1856,18 @@ static void copy_series_info (DATAINFO *dest, const DATAINFO *src)
     }
 }
 
+void free_model_dataset (MODEL *pmod)
+{
+    if (pmod->dataset != NULL) {
+#if SUBDEBUG
+	fprintf(stderr, "freeing pmod->dataset\n");
+#endif
+	destroy_dataset(pmod->dataset->Z, pmod->dataset->dinfo);
+	free(pmod->dataset);
+	pmod->dataset = NULL;
+    }
+}
+
 /* Below: For a model that was estimated using a data sample which
    differs from the current sample, reconstitute the dataset
    appropriate to the model.  If the model used a subsample, we use
@@ -1698,54 +1878,54 @@ static void copy_series_info (DATAINFO *dest, const DATAINFO *src)
    We attach this dataset to the model, so that it can be used for,
    e.g., residual or fitted versus actual plots.  The attached data
    set will be destroyed when the model itself is destroyed.
+
+   By default we create a dataset containing series up to the
+   highest numbered series associated with the model (regressand
+   and regressors).  If OPT_F is given we include all series; if
+   OPT_G is given we include only the constant.
 */
 
-int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
+int add_dataset_to_model (MODEL *pmod, const double **Z, 
+			  const DATAINFO *pdinfo,
+			  gretlopt opt)
 {
-    int sn = 0;
+    const double **srcZ;
+    const DATAINFO *srcinfo;
     double **modZ = NULL;
     DATAINFO *modinfo = NULL;
     char *mask = NULL;
-
-    if (fullZ == NULL || fullinfo == NULL) {
-#if SUBDEBUG
-	fputs("add_subsampled_dataset_to_model: got NULL full dataset\n",
-	      stderr);
-#endif
-	return 1;
-    }
+    int maxv, sn = 0;
 
     if (pmod->dataset != NULL) {
-#if SUBDEBUG
-	fputs("add_subsampled_dataset_to_model: job already done\n",
-	      stderr);
-#endif	
-	return 0;
+	free_model_dataset(pmod);
+    }    
+
+    if (fullZ != NULL) {
+	sync_datainfo_members(pdinfo);
+	srcZ = (const double **) fullZ;
+	srcinfo = fullinfo;
+    } else {
+	srcZ = Z;
+	srcinfo = pdinfo;
     }
 
-    /* sync malloced elements that may have moved */
-    sync_dataset_elements(pdinfo);
-
     if (pmod->submask == NULL) {
-	/* no subsample info: model was estimated on full dataset,
-	   so we reconstruct the full dataset */
-	sn = fullinfo->n;
+	/* no subsample info: pmod was estimated on the full dataset,
+	   so we'll reconstruct the full dataset */
+	sn = srcinfo->n;
 #if SUBDEBUG
 	fprintf(stderr, "pmod->submask = NULL, set sn = %d\n", sn);
 #endif
     } else {
-	/* model estimated on subsample, which has to be reconstructed */
+	/* pmod was estimated on a subsample, which has to 
+	   be reconstructed */
 	int t;
 
-	mask = calloc(fullinfo->n, 1);
+	mask = calloc(srcinfo->n, 1);
 	if (mask == NULL) {
 	    return E_ALLOC;
 	}
-#if SUBDEBUG
-	fprintf(stderr, "setting mask from pmod->submask: fullinfo->n = %d\n", 
-		fullinfo->n);
-#endif
-	for (t=0; t<fullinfo->n; t++) {
+	for (t=0; t<srcinfo->n; t++) {
 	    if (pmod->submask[t] > 0) {
 		mask[t] = 1;
 		sn++;
@@ -1768,9 +1948,16 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	    (void *) pmod->dataset);
 #endif
 
-    /* initial allocation of sub-sampled dataset */
-    modinfo = create_new_dataset(&modZ, fullinfo->v, sn,
-				 fullinfo->markers);
+    if (opt & OPT_F) {
+	maxv = srcinfo->v;
+    } else if (opt & OPT_G) {
+	maxv = 1;
+    } else {
+	maxv = highest_numbered_var_in_model(pmod, pdinfo) + 1;
+    }
+
+    /* allocate auxiliary dataset */
+    modinfo = create_auxiliary_dataset(&modZ, maxv, sn);
     if (modinfo == NULL) {
 	free(mask);
 	free(pmod->dataset);
@@ -1778,33 +1965,23 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	return E_ALLOC;
     }
 
-#if SUBDEBUG
-    fprintf(stderr, "dataset created, copying series info\n");
-#endif
-
     /* copy across info on series */
-    copy_series_info(modinfo, fullinfo);
+    copy_series_info(modinfo, srcinfo, maxv);
 
-    /* copy across data (and case markers, if any) */
-    copy_data_to_subsample(modZ, modinfo,
-			   (const double **) fullZ, fullinfo,
-			   mask);
-
-#if SUBDEBUG
-    fputs("data copied to subsampled dataset\n", stderr);
-#endif
+    /* copy across data */
+    copy_data_to_subsample(modZ, modinfo, srcZ, srcinfo, maxv, mask);
 
     /* dataset characteristics such as pd: if we're rebuilding the
-       _full_ dataset copy these across; but if we're reconstructing a
+       full dataset copy these across; but if we're reconstructing a
        subsampled dataset these features from the full dataset will be
        wrong, and we stay with the simple defaults
     */
     if (pmod->submask == NULL) {
-	modinfo->pd = fullinfo->pd;
-	modinfo->sd0 = fullinfo->sd0;
-	strcpy(modinfo->stobs, fullinfo->stobs);
-	strcpy(modinfo->endobs, fullinfo->endobs);
-	modinfo->structure = fullinfo->structure;
+	modinfo->pd = srcinfo->pd;
+	modinfo->sd0 = srcinfo->sd0;
+	strcpy(modinfo->stobs, srcinfo->stobs);
+	strcpy(modinfo->endobs, srcinfo->endobs);
+	modinfo->structure = srcinfo->structure;
     }
 
     pmod->dataset->Z = modZ;
@@ -1819,18 +1996,6 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     return 0;
 }
 
-void free_model_dataset (MODEL *pmod)
-{
-    if (pmod->dataset != NULL) {
-#if SUBDEBUG
-	fprintf(stderr, "Deep freeing model->dataset\n");
-#endif
-	destroy_dataset(pmod->dataset->Z, pmod->dataset->dinfo);
-	free(pmod->dataset);
-	pmod->dataset = NULL;
-    }
-}
-
 static int submask_match (const char *s1, const char *s2, int n)
 {
     int t;
@@ -1863,21 +2028,21 @@ int model_sample_problem (MODEL *pmod, const DATAINFO *pdinfo)
 	    ret = 0;
 	} else {
 	    fputs(I_("dataset is subsampled, model is not\n"), stderr);
-	    strcpy(gretl_errmsg, _("dataset is subsampled, model is not\n"));
+	    gretl_errmsg_set(_("dataset is subsampled, model is not\n"));
 	    ret = 1;
 	}
     } else {
 	/* the model does have sub-sampling info recorded */
 	if (pdinfo->submask == NULL) {
 	    fputs(I_("model is subsampled, dataset is not\n"), stderr);
-	    strcpy(gretl_errmsg, _("model is subsampled, dataset is not\n"));
+	    gretl_errmsg_set(_("model is subsampled, dataset is not\n"));
 	    ret = 1;
 	} else if (submask_match(pdinfo->submask, pmod->submask, n)) {
 	    /* the subsamples (model and current data set) agree, OK */
 	    ret = 0;
 	} else {
 	    /* the subsamples differ */
-	    strcpy(gretl_errmsg, _("model and dataset subsamples not the same\n"));
+	    gretl_errmsg_set(_("model and dataset subsamples not the same\n"));
 	    ret = 1;
 	}
     }
@@ -1889,3 +2054,153 @@ int model_sample_problem (MODEL *pmod, const DATAINFO *pdinfo)
 
     return ret;
 }
+
+static void dataset_type_string (char *str, const DATAINFO *pdinfo)
+{
+    if (dataset_is_time_series(pdinfo)) {
+	strcpy(str, _("time series"));
+    } else if (dataset_is_panel(pdinfo)) {
+        strcpy(str, _("panel"));
+    } else {
+        strcpy(str, _("undated"));
+    }
+}
+
+static void pd_string (char *str, const DATAINFO *pdinfo)
+{
+    if (custom_time_series(pdinfo)) {
+	strcpy(str, _("special"));
+    } else {
+	switch (pdinfo->pd) {
+	case 1:
+	    strcpy(str, _("annual")); break;
+	case 4:
+	    strcpy(str, _("quarterly")); break;
+	case 12:
+	    strcpy(str, _("monthly")); break;
+	case 24:
+	    strcpy(str, _("hourly")); break;
+	case 52:
+	    strcpy(str, _("weekly")); break;
+	case 5:
+	case 6:
+	case 7:
+	    strcpy(str, _("daily")); break;
+	case 10:
+	    strcpy(str, _("decennial")); break;
+	default:
+	    strcpy(str, _("unknown")); break;
+	}
+    }
+}
+
+void print_sample_obs (const DATAINFO *pdinfo, PRN *prn)
+{
+    char d1[OBSLEN], d2[OBSLEN];
+
+    ntodate(d1, pdinfo->t1, pdinfo);
+    ntodate(d2, pdinfo->t2, pdinfo);
+
+    pprintf(prn, "%s: %s - %s", _("Current sample"), d1, d2);
+    pprintf(prn, " (n = %d)\n", pdinfo->t2 - pdinfo->t1 + 1);
+}
+
+void print_sample_status (const DATAINFO *pdinfo, PRN *prn)
+{
+    char tmp[128];
+
+    if (complex_subsampled()) {
+	pprintf(prn, "%s\n\n", _("Full dataset"));
+	dataset_type_string(tmp, fullinfo);
+	pprintf(prn, "%s: %s\n", _("Type"), tmp);
+	if (dataset_is_time_series(fullinfo)) {
+	    pd_string(tmp, fullinfo);
+	    pprintf(prn, "%s: %s\n", _("Frequency"), tmp);
+	} else if (dataset_is_panel(fullinfo)) {
+	    int nu = fullinfo->n / fullinfo->pd;
+
+	    pprintf(prn, "%s: %d\n", _("Number of cross-sectional units"), nu);
+	    pprintf(prn, "%s: %d\n", _("Number of time periods"), fullinfo->pd);
+	}
+	pprintf(prn, "%s: %s - %s (n = %d)\n", _("Range"), 
+		fullinfo->stobs, fullinfo->endobs, fullinfo->n);
+
+	pprintf(prn, "\n%s\n", _("Subsampled data"));
+	if (pdinfo->restriction != NULL) {
+	    pprintf(prn, "(%s: %s)\n\n", _("restriction"), pdinfo->restriction);
+	} else {
+	    pputc(prn, '\n');
+	}
+    }	
+
+    dataset_type_string(tmp, pdinfo);
+    pprintf(prn, "%s: %s\n", _("Type"), tmp);
+    if (dataset_is_time_series(pdinfo)) {
+	pd_string(tmp, pdinfo);
+	pprintf(prn, "%s: %s\n", _("Frequency"), tmp);
+    } else if (dataset_is_panel(pdinfo)) {
+	int nu = pdinfo->n / pdinfo->pd;
+
+	pprintf(prn, "%s: %d\n", _("Number of cross-sectional units"), nu);
+	pprintf(prn, "%s: %d\n", _("Number of time periods"), pdinfo->pd);
+    }	
+    pprintf(prn, "%s: %s - %s (n = %d)\n", _("Full range"), 
+	    pdinfo->stobs, pdinfo->endobs, pdinfo->n);
+    print_sample_obs(pdinfo, prn); 
+}
+
+/**
+ * data_report:
+ * @pdinfo: data information struct.
+ * @fname: filename for current datafile.
+ * @prn: gretl printing struct.
+ * 
+ * Write out a summary of the content of the current data set.
+ * 
+ * Returns: 0 on successful completion, non-zero on error.
+ * 
+ */
+
+int data_report (const DATAINFO *pdinfo, const char *fname, PRN *prn)
+{
+    char startdate[OBSLEN], enddate[OBSLEN], tmp[MAXLEN];
+    char tstr[48];
+    int i;
+
+    ntodate(startdate, 0, pdinfo);
+    ntodate(enddate, pdinfo->n - 1, pdinfo);
+
+    sprintf(tmp, _("Data file %s\nas of"), 
+	    (*fname != '\0')? fname : _("(unsaved)"));
+
+    print_time(tstr);
+    pprintf(prn, "%s %s\n\n", tmp, tstr);
+
+    if (pdinfo->descrip != NULL && *pdinfo->descrip != '\0') {
+	pprintf(prn, "%s:\n\n", _("Description"));
+	pputs(prn, pdinfo->descrip);
+	pputs(prn, "\n\n");
+    }
+
+    dataset_type_string(tmp, pdinfo);
+    pprintf(prn, "%s: %s\n", _("Type of data"), tmp);
+    
+    if (dataset_is_time_series(pdinfo)) {
+	pd_string(tmp, pdinfo);
+	pprintf(prn, "%s: %s\n", _("Frequency"), tmp);
+    }	
+
+    pprintf(prn, "%s: %s - %s (n = %d)\n\n", _("Range"),
+	    startdate, enddate, pdinfo->n);
+
+    pprintf(prn, "%s:\n\n", _("Listing of variables"));
+
+    for (i=1; i<pdinfo->v; i++) {
+	pprintf(prn, "%*s  %s\n", VNAMELEN, pdinfo->varname[i], 
+		VARLABEL(pdinfo, i));
+    }
+
+    return 0;
+}
+
+
