@@ -19,6 +19,8 @@
 
 /* mechanisms for defining and handling systems of equations */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "system.h"
 #include "objstack.h"
@@ -534,7 +536,7 @@ int equation_system_append (equation_system *sys,
     int n;
 
     if (sys == NULL) {
-	strcpy(gretl_errmsg, _(nosystem));
+	gretl_errmsg_set(_(nosystem));
 	return E_DATA;
     }
 
@@ -685,7 +687,7 @@ equation_system *equation_system_start (const char *line,
 
     if (method == SYS_METHOD_MAX) {
 	/* invalid method was given */
-	strcpy(gretl_errmsg, _(badsystem));
+	gretl_errmsg_set(_(badsystem));
 	*err = E_DATA;
 	return NULL;
     }
@@ -694,7 +696,7 @@ equation_system *equation_system_start (const char *line,
 
     if (method < 0 && sysname == NULL) {
 	/* neither a method nor a name was specified */
-	strcpy(gretl_errmsg, _(badsystem));
+	gretl_errmsg_set(_(badsystem));
 	*err = E_DATA;
     }
 
@@ -1103,7 +1105,7 @@ equation_system_estimate (equation_system *sys,
 int system_adjust_t1t2 (equation_system *sys, const double **Z, 
 			const DATAINFO *pdinfo)
 {
-    int err = 0;
+    int err;
 
     if (sys->biglist == NULL) {
 	fprintf(stderr, "system_adjust_t1t2: no 'biglist' present!\n");
@@ -1113,7 +1115,7 @@ int system_adjust_t1t2 (equation_system *sys, const double **Z,
     sys->t1 = pdinfo->t1;
     sys->t2 = pdinfo->t2;
 
-    err = check_for_missing_obs(sys->biglist, &sys->t1, &sys->t2, Z, NULL);
+    err = check_for_missing_obs(sys->biglist, &sys->t1, &sys->t2, Z);
 
     if (!err) {
 	sys->T = sys->t2 - sys->t1 + 1;
@@ -1218,7 +1220,7 @@ static int sys_check_lists (equation_system *sys, const double **Z,
     int user_ylist = (sys->ylist != NULL);
     int *ylist = NULL;
     int *xplist = NULL;
-    int src, lag, nxp;
+    int src, lag, nlhs, nxp;
     int i, j, k, vj;
     int err = 0;
 
@@ -1234,7 +1236,8 @@ static int sys_check_lists (equation_system *sys, const double **Z,
        of structural equations or identities.
     */
 
-    ylist = gretl_list_new(sys->neqns + sys->nidents);
+    nlhs = sys->neqns + sys->nidents;
+    ylist = gretl_list_new(nlhs);
     if (ylist == NULL) {
 	return E_ALLOC;
     }
@@ -1272,9 +1275,9 @@ static int sys_check_lists (equation_system *sys, const double **Z,
 	for (j=1; j<=ylist[0] && !err; j++) {
 	    vj = ylist[j];
 	    if (!in_gretl_list(sys->ylist, vj)) {
-		sprintf(gretl_errmsg, "%s appears on the left-hand side "
-			"of an equation but is not marked as endogenous", 
-			pdinfo->varname[vj]);
+		gretl_errmsg_sprintf("%s appears on the left-hand side "
+				     "of an equation but is not marked as endogenous", 
+				     pdinfo->varname[vj]);
 		err = E_DATA;
 	    }
 	}
@@ -1309,8 +1312,9 @@ static int sys_check_lists (equation_system *sys, const double **Z,
 	for (j=1; j<=sys->ylist[0]; j++) {
 	    vj = sys->ylist[j];
 	    if (!in_gretl_list(sys->biglist, vj)) {
-		sprintf(gretl_errmsg, "%s is marked as endogenous but is "
-			"not present in the system", pdinfo->varname[vj]);
+		gretl_errmsg_sprintf("%s is marked as endogenous but is "
+				     "not present in the system", 
+				     pdinfo->varname[vj]);
 		err = E_DATA;
 		goto bailout;
 	    }
@@ -1348,8 +1352,9 @@ static int sys_check_lists (equation_system *sys, const double **Z,
 	for (j=1; j<=sys->ilist[0]; j++) {
 	    vj = sys->ilist[j];
 	    if (in_gretl_list(sys->ylist, vj)) {
-		sprintf(gretl_errmsg, "%s is marked as an instrument "
-			"but is endogenous", pdinfo->varname[vj]);
+		gretl_errmsg_sprintf("%s is marked as an instrument "
+				     "but is endogenous", 
+				     pdinfo->varname[vj]);
 		err = E_DATA;
 	    }
 	}
@@ -1386,6 +1391,13 @@ static int sys_check_lists (equation_system *sys, const double **Z,
     printlist(xplist, "final system exog list");
 #endif
 
+    if (!err && sys->ylist[0] != nlhs) {
+	/* Note: added 2009-08-10 */
+	gretl_errmsg_sprintf("Found %d endogenous variables but %d equations",
+			     sys->ylist[0], nlhs);
+	err = E_DATA;
+    }
+
     if (!err) {
 	free(sys->xlist);
 	sys->xlist = xplist;
@@ -1428,18 +1440,18 @@ int equation_system_finalize (equation_system *sys,
     gretl_error_clear();
 
     if (sys == NULL) {
-	strcpy(gretl_errmsg, _(nosystem));
+	gretl_errmsg_set(_(nosystem));
 	return 1;
     }
 
     if (sys->neqns < mineq) {
-	strcpy(gretl_errmsg, _(toofew));
+	gretl_errmsg_set(_(toofew));
 	equation_system_destroy(sys);
 	return 1;
     }
 
     if (sys->method >= SYS_METHOD_MAX) {
-	strcpy(gretl_errmsg, _(badsystem));
+	gretl_errmsg_set(_(badsystem));
 	equation_system_destroy(sys);
 	return 1;
     } 
@@ -1472,21 +1484,21 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
     int method;
 
     if (strlen(line) < 12) {
-	strcpy(gretl_errmsg, "estimate: no system name was provided");
-	return 1;
+	gretl_errmsg_set("estimate: no system name was provided");
+	return E_DATA;
     }
 
     sysname = get_system_name_from_line(line, SYSNAME_EST);
     if (sysname == NULL) {
-	strcpy(gretl_errmsg, "estimate: no system name was provided");
-	return 1;
+	gretl_errmsg_set("estimate: no system name was provided");
+	return E_DATA;
     }
 
     sys = get_equation_system_by_name(sysname);
     if (sys == NULL) {
-	sprintf(gretl_errmsg, _("'%s': unrecognized name"), sysname);
+	gretl_errmsg_sprintf(_("'%s': unrecognized name"), sysname);
 	free(sysname);
-	return 1;
+	return E_UNKVAR;
     }
 
     free(sysname);
@@ -1497,8 +1509,8 @@ int estimate_named_system (const char *line, double ***pZ, DATAINFO *pdinfo,
     }
 
     if (method < 0 || method >= SYS_METHOD_MAX) {
-	strcpy(gretl_errmsg, "estimate: no valid method was specified");
-	return 1;
+	gretl_errmsg_set("estimate: no valid method was specified");
+	return E_DATA;
     }
 
     sys->method = method;
@@ -2127,12 +2139,12 @@ add_aux_list_to_sys (equation_system *sys, const char *line,
 
     if (which == ENDOG_LIST) {
 	if (sys->ylist != NULL) {
-	    strcpy(gretl_errmsg, "Only one list of endogenous variables may be given");
+	    gretl_errmsg_set("Only one list of endogenous variables may be given");
 	    return 1;
 	} 
     } else if (which == INSTR_LIST) {
 	if (sys->ilist != NULL) {
-	    strcpy(gretl_errmsg, "Only one list of instruments may be given");
+	    gretl_errmsg_set("Only one list of instruments may be given");
 	    return 1;
 	} 
     } else {
@@ -2283,12 +2295,32 @@ static int system_add_yhat_matrix (equation_system *sys)
 	nc += sys->models[i]->ncoeff;
     }
 
-    avc = ceil((double) nc / sys->neqns);
-    sys->df = sys->T - avc;
+    avc = nc / (double) sys->neqns;
+    sys->df = sys->T - floor(avc);
 
     return 0;
 }
 
+static gretl_matrix *get_stderr_vec (const gretl_matrix *V)
+{
+    gretl_matrix *se;
+    int k = V->rows;
+
+    se = gretl_column_vector_alloc(k);
+    
+    if (se != NULL) {
+	double x;
+	int i;
+
+	for (i=0; i<k; i++) {
+	    x = gretl_matrix_get(V, i, i);
+	    se->val[i] = sqrt(x);
+	}
+    }
+
+    return se;
+}
+
 /* retrieve a copy of a specified matrix from an equation
    system */
 
@@ -2318,8 +2350,11 @@ equation_system_get_matrix (const equation_system *sys, int idx,
 	M = gretl_matrix_copy(sys->yhat);
 	break;
     case M_VCV:
+    case M_SE:
 	if (sys->vcv == NULL) {
 	    *err = E_BADSTAT;
+	} else if (idx == M_SE) {
+	    M = get_stderr_vec(sys->vcv);
 	} else {
 	    M = gretl_matrix_copy(sys->vcv);
 	}
@@ -2775,10 +2810,10 @@ static int get_col_and_lag (int vnum, const equation_system *sys,
 
 /* reduced-form error covariance matrix */
 
-static int sys_add_RF_covariance_matrix (equation_system *sys)
+static int sys_add_RF_covariance_matrix (equation_system *sys,
+					 int n)
 {
     gretl_matrix *G = NULL, *S = NULL;
-    int n = sys->neqns + sys->nidents;
     int err = 0;
 
     if (!gretl_is_identity_matrix(sys->Gamma)) {
@@ -2805,10 +2840,12 @@ static int sys_add_RF_covariance_matrix (equation_system *sys)
     }
 
     if (G != NULL) {
-	gretl_SVD_invert_matrix(G);
-	gretl_matrix_qform(G, GRETL_MOD_NONE,
-			   (S != NULL)? S : sys->S,
-			   sys->Sr, GRETL_MOD_NONE);
+	err = gretl_SVD_invert_matrix(G);
+	if (!err) {
+	    err = gretl_matrix_qform(G, GRETL_MOD_NONE,
+				     (S != NULL)? S : sys->S,
+				     sys->Sr, GRETL_MOD_NONE);
+	}
 	gretl_matrix_free(G);
     } else {
 	gretl_matrix_copy_values(sys->Sr, (S != NULL)? S : sys->S);
@@ -2843,6 +2880,10 @@ static int sys_add_structural_form (equation_system *sys,
     printlist(xlist, "exogenous vars");
 #endif
 
+    if (n != ylist[0]) {
+	return E_DATA;
+    }
+
     sys->order = sys_max_predet_lag(sys);
 
     /* allocate coefficient matrices */
@@ -2931,7 +2972,7 @@ static int sys_add_structural_form (equation_system *sys,
     }
 
     if (!err) {
-	err = sys_add_RF_covariance_matrix(sys);
+	err = sys_add_RF_covariance_matrix(sys, n);
 	if (err) {
 	    fprintf(stderr, "error %d in sys_add_RF_covariance_matrix\n", err);
 	}
