@@ -24,6 +24,8 @@
 #include "johansen.h"
 #include "texprint.h"
 
+#include <glib.h>
+
 static char colspec[4][8];
 static int use_custom;
 static int use_pdf;
@@ -1065,7 +1067,7 @@ static PRN *make_tex_prn (int ID, char *fname,
     PRN *prn;
 
     if (*fname == '\0') {
-	sprintf(texfile, "%s%s_%d.tex", gretl_work_dir(),
+	sprintf(texfile, "%s%s_%d.tex", gretl_workdir(),
 		(eqn)? "equation" : "model", ID);
 	strcpy(fname, texfile);
     } else {
@@ -1094,7 +1096,7 @@ static PRN *make_rtf_prn (int ID, char *fname, int *err)
     PRN *prn;
 
     if (*fname == '\0') {
-	sprintf(rtffile, "%smodel_%d.rtf", gretl_work_dir(), ID);
+	sprintf(rtffile, "%smodel_%d.rtf", gretl_workdir(), ID);
 	strcpy(fname, rtffile);
     } else {
 	gretl_maybe_switch_dir(fname);
@@ -1114,7 +1116,6 @@ static PRN *make_rtf_prn (int ID, char *fname, int *err)
 
 static char tex_preamble_file[MAXLEN];
 
-#ifdef ENABLE_NLS
 static const char *get_gretltex_local (void)
 {
     static char localtex[32] = {0};
@@ -1129,32 +1130,57 @@ static const char *get_gretltex_local (void)
 
     return localtex;
 }
-#endif
 
 void set_gretl_tex_preamble (void)
 {
+    const char *wdir = gretl_workdir();
     char test[MAXLEN];
     FILE *fp;
+    int gotit = 0;
 
-#ifdef ENABLE_NLS
-    /* first choice: localized preamble file */
-    sprintf(test, "%s%s", gretl_work_dir(), get_gretltex_local());
-    fp = gretl_fopen(test, "r");
-    if (fp != NULL) {
-	strcpy(tex_preamble_file, test);
-	fclose(fp);
-	return;
-    }    
-#endif
-
-    /* preamble file on disk */
-    sprintf(test, "%sgretlpre.tex", gretl_work_dir());
+    /* localized preamble file in working dir */
+    sprintf(test, "%s%s", wdir, get_gretltex_local());
     fp = gretl_fopen(test, "r");
     if (fp != NULL) {
 	strcpy(tex_preamble_file, test);
 	fclose(fp);
+	gotit = 1;
+    } else {
+	/* regular preamble file in working dir */
+	sprintf(test, "%sgretlpre.tex", wdir);
+	fp = gretl_fopen(test, "r");
+	if (fp != NULL) {
+	    strcpy(tex_preamble_file, test);
+	    fclose(fp);
+	    gotit = 1;
+	}
     }
 
+    if (!gotit) {
+	gchar *ddir = gretl_default_workdir();
+
+	if (ddir != NULL) {
+	    /* localized preamble file in standard working dir */
+	    sprintf(test, "%s%s", ddir, get_gretltex_local());
+	    fp = gretl_fopen(test, "r");
+	    if (fp != NULL) {
+		strcpy(tex_preamble_file, test);
+		fclose(fp);
+		gotit = 1;
+	    } else {
+		/* regular preamble file in standard working dir */
+		sprintf(test, "%sgretlpre.tex", ddir);
+		fp = gretl_fopen(test, "r");
+		if (fp != NULL) {
+		    strcpy(tex_preamble_file, test);
+		    fclose(fp);
+		    gotit = 1;
+		}
+	    }
+	    g_free(ddir);
+	}
+    }    
+
     gretl_error_clear();
 }
 
@@ -1198,9 +1224,7 @@ void set_tex_use_utf (int s)
 
 void gretl_tex_preamble (PRN *prn, int fmt)
 {
-#ifdef ENABLE_NLS
     char* lang = getenv("LANG");
-#endif
     FILE *fp = NULL;
     int userfile = 0;
 
@@ -1238,7 +1262,6 @@ void gretl_tex_preamble (PRN *prn, int fmt)
 
 	pputs(prn, "{article}\n");
 
-#ifdef ENABLE_NLS
 	if (tex_use_utf) {
 	    pputs(prn, "\\usepackage{ucs}\n");
 	    pputs(prn, "\\usepackage[utf8x]{inputenc}\n");
@@ -1248,7 +1271,7 @@ void gretl_tex_preamble (PRN *prn, int fmt)
 	if (lang != NULL && !strncmp(lang, "ru", 2)) {
 	    pputs(prn, "\\usepackage[russian]{babel}\n");
 	}
-#endif
+
 	if (fmt & GRETL_FORMAT_EQN) {
 	    pputs(prn, "\\usepackage{amsmath}\n\n");
 	} else if (fmt & GRETL_FORMAT_MODELTAB) {
@@ -1471,10 +1494,9 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 
     /* additional info (R^2 etc) */
     if (pmod->ci == LAD) { 
-	double SAR = gretl_model_get_double(pmod, "ladsum");
-
-	if (!na(SAR)) {
-	    tex_sprint_math_double_digits(SAR, tmp, 6);
+	x = gretl_model_get_double(pmod, "ladsum");
+	if (!na(x)) {
+	    tex_sprint_math_double_digits(x, tmp, 6);
 	    pprintf(prn, "\\quad \\sum |\\hat{u}_t| = %s ", tmp);
 	}
     } else {
@@ -1496,9 +1518,8 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	}
 
 	if (!na(gretl_model_get_double(pmod, "rho_in"))) {
-	    double r = gretl_model_get_double(pmod, "rho_in");
-
-	    tex_sprint_math_double_digits(r, tmp, 5);
+	    x = gretl_model_get_double(pmod, "rho_in");
+	    tex_sprint_math_double_digits(x, tmp, 5);
 	    pprintf(prn, " \\quad \\rho = %s", tmp);
 	}
     }
