@@ -630,8 +630,8 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
    of this matrix; return Q */
 
 static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
-			     const double **Z, int t1, int t2, char *mask,
-			     int *err)
+			     const double **Z, const DATAINFO *pdinfo, 
+			     char *mask, int *err)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
@@ -640,7 +640,8 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     double test;
     int i, j, k;
 
-    Q = gretl_matrix_data_subset_masked(instlist, Z, t1, t2, mask, err);
+    Q = gretl_matrix_data_subset_masked(instlist, Z, pdinfo->t1, pdinfo->t2, 
+					mask, err);
     if (*err) {
 	return NULL;
     }
@@ -688,8 +689,8 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 		    droplist[0] += 1;
 		    droplist[droplist[0]] = instlist[j];
 		}	    
-		fprintf(stderr, "tsls_Q: dropping redundant instrument %d\n", 
-			instlist[j]);
+		fprintf(stderr, "tsls_Q: dropping redundant instrument %d (%s)\n", 
+			instlist[j], pdinfo->varname[instlist[j]]);
 		gretl_list_delete_at_pos(instlist, j--);
 	    }
 	    j++;
@@ -697,7 +698,8 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 
 	k = instlist[0];
 	gretl_matrix_free(Q);
-	Q = gretl_matrix_data_subset_masked(instlist, Z, t1, t2, mask, err);
+	Q = gretl_matrix_data_subset_masked(instlist, Z, pdinfo->t1, pdinfo->t2, 
+					    mask, err);
 	if (!*err) {
 	    R = gretl_matrix_reuse(R, k, k);
 	    *err = gretl_matrix_QR_decomp(Q, R);
@@ -806,15 +808,22 @@ static void tsls_residuals (MODEL *pmod, const int *reglist,
     }
 }
 
+#define TSLS_CORR_RSQ 1
+
 static void tsls_extra_stats (MODEL *pmod, int overid, const double **Z,
 			      const DATAINFO *pdinfo)
 {
-    int yno = pmod->list[1];
     double r;
 
-    pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, Z[yno], pmod->yhat);
-    r = 1.0 - pmod->rsq;
+#if TSLS_CORR_RSQ
+    int yno = pmod->list[1];
 
+    pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, Z[yno], pmod->yhat);
+#else
+    pmod->rsq = 1.0 - pmod->ess / pmod->tss;
+#endif
+    
+    r = 1.0 - pmod->rsq;
     pmod->adjrsq = 1.0 - (r * (pmod->nobs - 1.0) / pmod->dfd);
 
     pmod->fstt = pmod->chisq = NADBL;
@@ -834,7 +843,7 @@ static void tsls_extra_stats (MODEL *pmod, int overid, const double **Z,
     pmod->criterion[C_BIC] = NADBL;
     pmod->criterion[C_HQC] = NADBL;
     
-     if (dataset_is_time_series(pdinfo) && pmod->missmask == NULL) {
+    if (dataset_is_time_series(pdinfo) && pmod->missmask == NULL) {
 	/* time series, no missing obs within sample range */
 	pmod->rho = rhohat(1, pmod->t1, pmod->t2, pmod->uhat);
 	pmod->dw = dwstat(1, pmod, Z);
@@ -1306,8 +1315,7 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
     } else {
 	for (i=1; i<=zlist[0]; i++) {
 	    if (zlist[i] == list[1]) {
-		strcpy(gretl_errmsg, 
-		       "You can't use the dependent variable as an instrument");
+		gretl_errmsg_set("You can't use the dependent variable as an instrument");
 		err = E_DATA;
 		break;
 	    }
@@ -1317,9 +1325,9 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
     if (!err) {
 	oid = zlist[0] - rlist[0] + 1;
 	if (oid < 0) {
-	    sprintf(gretl_errmsg, 
-		    _("The order condition for identification is not satisfied.\n"
-		      "At least %d more instruments are needed."), -oid);
+	    gretl_errmsg_sprintf(_("The order condition for identification is not satisfied.\n"
+				   "At least %d more instruments are needed."), -oid);
+	    fprintf(stderr, "zlist[0] = %d, rlist[0] = %d\n", zlist[0], rlist[0]);
 	    err = E_DATA; 
 	}
     }
@@ -1438,7 +1446,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     if (!err) {
 	Q = tsls_Q(instlist, reglist, &droplist,
-		   (const double **) *pZ, pdinfo->t1, pdinfo->t2,
+		   (const double **) *pZ, pdinfo,
 		   missmask, &err);
     }
 
@@ -1451,9 +1459,8 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     */
     OverIdRank = instlist[0] - reglist[0] + 1;
     if (OverIdRank < 0) {
-        sprintf(gretl_errmsg, 
-		_("The order condition for identification is not satisfied.\n"
-		  "At least %d more instruments are needed."), -OverIdRank);
+        gretl_errmsg_sprintf(_("The order condition for identification is not satisfied.\n"
+			       "At least %d more instruments are needed."), -OverIdRank);
 	err = E_DATA; 
 	goto bailout;
     }
@@ -1511,7 +1518,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	if (nendo == 1) {
 	    /* handles robust estimation, for single endogenous regressor */
 	    compute_first_stage_F(&tsls, ev, reglist, instlist, pZ, pdinfo, opt);
-	} else if (!(opt & OPT_R)) {
+	} else if (!(opt & OPT_R) && nendo > 0) {
 	    /* at present, only handles case of i.i.d. errors */
 	    compute_stock_yogo(&tsls, endolist, instlist, hatlist,
 			       (const double **) *pZ);
@@ -1524,9 +1531,11 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	reglist_remove_redundant_vars(&tsls, s2list, reglist);
     }
 
-    /* special: we need to use the original RHS vars to compute
-       residuals and associated statistics */
-    tsls_residuals(&tsls, reglist, (const double **) *pZ, opt);
+    if (nendo > 0) {
+	/* special: we need to use the original RHS vars to compute
+	   residuals and associated statistics */
+	tsls_residuals(&tsls, reglist, (const double **) *pZ, opt);
+    }
 
     if (opt & OPT_R) {
 	/* robust standard errors called for */
@@ -1536,10 +1545,12 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}	
     } 
 
-    /* compute additional statistics (R^2, F, etc.) */
-    tsls_extra_stats(&tsls, OverIdRank, (const double **) *pZ, pdinfo);
+    if (nendo > 0) {
+	/* compute additional statistics (R^2, F, etc.) */
+	tsls_extra_stats(&tsls, OverIdRank, (const double **) *pZ, pdinfo);
+    }
 
-    if (!sysest) {
+    if (!sysest && nendo > 0) {
 	if (hatlist != NULL) {
 	    tsls_hausman_test(&tsls, reglist, hatlist, pZ, pdinfo);
 	}
