@@ -17,6 +17,8 @@
  * 
  */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "gretl_matrix.h"
 #include "matrix_extra.h"
@@ -29,9 +31,15 @@
 #define MDEBUG 0
 #define LEVEL_AUTO -1
 
+typedef enum {
+    UM_PRIVATE = 1 << 0,
+    UM_SHELL   = 1 << 1
+} UMFlags;
+
 struct user_matrix_ {
     gretl_matrix *M;
     int level;
+    UMFlags flags;
     char name[VNAMELEN];
     char **colnames;
 };
@@ -39,6 +47,9 @@ struct user_matrix_ {
 static user_matrix **matrices;
 static int n_matrices;
 
+#define matrix_is_private(u) ((u->flags & UM_PRIVATE) || *u->name == '$')
+#define matrix_is_shell(u) (u->flags & UM_SHELL)
+
 #if MDEBUG > 1
 static void print_matrix_stack (const char *msg)
 {
@@ -74,15 +85,15 @@ const char *get_matrix_name_by_index (int idx)
 
 static user_matrix *user_matrix_new (gretl_matrix *M, const char *name)
 {
-    user_matrix *u;
+    user_matrix *u = malloc(sizeof *u);
 
-    u = malloc(sizeof *u);
     if (u == NULL) {
 	return NULL;
     }
 
     u->M = M;
     u->level = gretl_function_depth();
+    u->flags = 0;
     *u->name = '\0';
     strncat(u->name, name, VNAMELEN - 1);
     u->colnames = NULL;
@@ -123,18 +134,20 @@ void set_matrix_add_callback (void (*callback))
     matrix_add_callback = callback; 
 }
 
-static int real_user_matrix_add (gretl_matrix *M, const char *name,
-				 int callback_ok)
+static user_matrix *real_user_matrix_add (gretl_matrix *M, const char *name,
+					  int callback_ok)
 {
     user_matrix **tmp;
+    user_matrix *u;
+    int n = n_matrices;
 
     if (M == NULL) {
 	return 0;
     }
 
-    tmp = realloc(matrices, (n_matrices + 1) * sizeof *tmp);
+    tmp = realloc(matrices, (n + 1) * sizeof *tmp);
     if (tmp == NULL) {
-	return E_ALLOC;
+	return NULL;
     } else {
 	matrices = tmp;
     }
@@ -144,42 +157,50 @@ static int real_user_matrix_add (gretl_matrix *M, const char *name,
 	gretl_matrix *Mcpy = gretl_matrix_copy(M);
 
 	if (Mcpy == NULL) {
-	    return E_ALLOC;
+	    return NULL;
 	}
 	M = Mcpy;
     }
 
-    matrices[n_matrices] = user_matrix_new(M, name);
+    matrices[n] = u = user_matrix_new(M, name);
 
-    if (matrices[n_matrices] == NULL) {
-	return E_ALLOC;
+    if (matrices[n] == NULL) {
+	return NULL;
     }
 
     n_matrices++;
 
 #if MDEBUG
     fprintf(stderr, "add_user_matrix: allocated '%s' at %p (M at %p, %dx%d)\n",
-	    name, (void *) matrices[n_matrices-1], (void *) M,
+	    name, (void *) matrices[n], (void *) M,
 	    gretl_matrix_rows(M), gretl_matrix_cols(M));
 #endif
 
-    if (callback_ok && 
-	matrix_add_callback != NULL && 
+    if (callback_ok && matrix_add_callback != NULL && 
 	gretl_function_depth() == 0) {
 	(*matrix_add_callback)();
     }
 
-    return 0;
+    return u;
 }
 
 int user_matrix_add (gretl_matrix *M, const char *name)
 {
-    return real_user_matrix_add(M, name, 1);
+    user_matrix *u = real_user_matrix_add(M, name, 1);
+
+    return (u == NULL)? E_ALLOC : 0;
 }
 
-int matrix_copy_add (gretl_matrix *M, const char *name)
+int private_matrix_add (gretl_matrix *M, const char *name)
 {
-    return real_user_matrix_add(M, name, 0);
+    user_matrix *u = real_user_matrix_add(M, name, 0);
+
+    if (u == NULL) {
+	return E_ALLOC;
+    } else {
+	u->flags |= UM_PRIVATE;
+	return 0;
+    }
 }
 
 static int 
@@ -315,7 +336,7 @@ int user_matrix_replace_matrix_by_name (const char *name,
     return user_matrix_replace_matrix(u, M);
 }
 
-static user_matrix *get_user_matrix_by_data (const gretl_matrix *M)
+user_matrix *get_user_matrix_by_data (const gretl_matrix *M)
 {
     int level = gretl_function_depth();
     int i;
@@ -484,7 +505,7 @@ int copy_named_matrix_as (const char *orig, const char *newname)
  * @newname: the name to be given to the copy.
  *
  * A copy of matrix @m is added to the stack of saved matrices
- * under the name @newname.  This is intended for use when a matrix is given
+ * under the name @newname.  This is intended for use when a matrix
  * is given as the argument to a user-defined function: it is copied
  * under the name assigned by the function's parameter list.
  *
@@ -494,18 +515,48 @@ int copy_named_matrix_as (const char *orig, const char *newname)
 int copy_matrix_as (const gretl_matrix *m, const char *newname)
 {
     gretl_matrix *m2 = gretl_matrix_copy(m);
-    int err;
+    user_matrix *u;
+    int err = 0;
 
     if (m2 == NULL) {
 	err = E_ALLOC;
     } else {
-	err = matrix_copy_add(m2, newname);
+	u = real_user_matrix_add(m2, newname, 0);
+	if (u == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    u->level += 1;
+	}
     }
 
-    if (!err) {
-	/* increment level of last-added matrix */
-	user_matrix *u = matrices[n_matrices - 1];
+    return err;
+}
 
+/**
+ * matrix_add_as_shell:
+ * @M: the matrix to add.
+ * @name: the name to be given to the "shell".
+ *
+ * Matrix @M is added to the stack of saved matrices under
+ * the name @name with the shell flag set.  This is used
+ * when an anonymous matrix is given as a %const argument to a 
+ * user-defined function: it is temporarily given user_matrix, 
+ * status, so that it is accessible by name within the function,
+ * but the content @M is protected from destruction on exit 
+ * from the function.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int matrix_add_as_shell (gretl_matrix *M, const char *name)
+{
+    user_matrix *u = real_user_matrix_add(M, name, 0);
+    int err = 0;
+
+    if (u == NULL) {
+	err = E_ALLOC;
+    } else {
+	u->flags |= UM_SHELL;
 	u->level += 1;
     }
 
@@ -523,8 +574,8 @@ static int msel_out_of_bounds (int *range, int n)
     }
 
     if (bad != NULL) {
-	sprintf(gretl_errmsg, _("Index value %d is out of bounds"), 
-		*bad);
+	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), 
+			     *bad);
 	return 1;
     } else {
 	return 0;
@@ -565,15 +616,15 @@ static int *mspec_to_list (int type, union msel *sel, int n,
 	} else {
 	    ns = sel->range[1] - sel->range[0] + 1;
 	    if (ns <= 0) {
-		sprintf(gretl_errmsg, _("Range %d to %d is non-positive!"),
-			sel->range[0], sel->range[1]); 
+		gretl_errmsg_sprintf(_("Range %d to %d is non-positive!"),
+				     sel->range[0], sel->range[1]); 
 		*err = E_DATA;
 	    }
 	}
     } else {
 	/* SEL_MATRIX */
 	if (sel->m == NULL) {
-	    strcpy(gretl_errmsg, _("Range is non-positive!"));
+	    gretl_errmsg_set(_("Range is non-positive!"));
 	    *err = E_DATA;
 	} else {
 	    ns = gretl_vector_get_length(sel->m);
@@ -607,8 +658,8 @@ static int *mspec_to_list (int type, union msel *sel, int n,
 
     for (i=1; i<=slice[0] && !*err; i++) {
 	if (slice[i] < 1 || slice[i] > n) {
-	    sprintf(gretl_errmsg, _("Index value %d is out of bounds"), 
-		    slice[i]);
+	    gretl_errmsg_sprintf(_("Index value %d is out of bounds"), 
+				 slice[i]);
 	    *err = 1;
 	}
     }
@@ -873,6 +924,12 @@ static void destroy_user_matrix (user_matrix *u)
 	return;
     }
 
+    if (matrix_is_shell(u)) {
+	/* don't free content */
+	free(u);
+	return;
+    }
+
 #if MDEBUG
     fprintf(stderr, "destroy_user_matrix: freeing matrix at %p...", 
 	    (void *) u->M);
@@ -891,15 +948,17 @@ static void destroy_user_matrix (user_matrix *u)
 
 #define LEV_PRIVATE -1
 
-static int levels_match (user_matrix *u, int lev)
+static int matrix_levels_match (user_matrix *u, int lev)
 {
+    int ret = 0;
+
     if (u->level == lev) {
-	return 1;
-    } else if (lev == LEV_PRIVATE && *u->name == '$') {
-	return 1;
+	ret = 1;
+    } else if (lev == LEV_PRIVATE && matrix_is_private(u)) {
+	ret = 1;
     }
 
-    return 0;
+    return ret;
 }
 
 /**
@@ -932,7 +991,7 @@ int destroy_user_matrices_at_level (int level)
 	if (matrices[i] == NULL) {
 	    break;
 	}
-	if (levels_match(matrices[i], level)) {
+	if (matrix_levels_match(matrices[i], level)) {
 #if MDEBUG
 	    fprintf(stderr, "destroying matrix[%d] ('%s' at %p)\n",
 		    i, matrices[i]->name, (void *) matrices[i]->M);
@@ -1402,7 +1461,7 @@ real_user_matrix_QR_decomp (const gretl_matrix *m, gretl_matrix **Q,
     }
 
     if (err) {
-	strcpy(gretl_errmsg, _("Matrix decomposition failed"));
+	gretl_errmsg_set(_("Matrix decomposition failed"));
 	gretl_matrix_free(*Q);
 	*Q = NULL;
 	if (R != NULL) {
@@ -1535,18 +1594,45 @@ gretl_matrix *user_matrix_SVD (const gretl_matrix *m,
     return S;
 }
 
+static gretl_matrix *get_ols_matrix (const char *mname, 
+				     int r, int c,
+				     int *newmat, int *err)
+{
+    gretl_matrix *m = get_matrix_by_name(mname);
+
+    if (m == NULL) {
+	gretl_errmsg_sprintf(_("'%s': no such matrix"), mname);
+	*err = E_UNKVAR;
+	return NULL;
+    } 
+
+    if (newmat != NULL && (m->rows != r || m->cols != c)) {
+	m = gretl_matrix_alloc(r, c);
+	if (m == NULL) {
+	    *err = E_ALLOC;
+	    return NULL;
+	}
+	*newmat = 1;
+    }
+
+    return m;
+}
+
 gretl_matrix *user_matrix_ols (const gretl_matrix *Y, 
 			       const gretl_matrix *X, 
 			       const char *Uname, 
+			       const char *Vname, 
 			       gretlopt opt,
 			       int *err)
 {
     gretl_matrix *B = NULL;
     gretl_matrix *U = NULL;
+    gretl_matrix *V = NULL;
+    int newU = 0, newV = 0;
+    double s2, *ps2 = NULL;
     int g, k, T;
-    int newU = 0;
 
-    if (gretl_is_null_matrix(X) || gretl_is_null_matrix(X)) {
+    if (gretl_is_null_matrix(Y) || gretl_is_null_matrix(X)) {
 	*err = E_DATA;
 	return NULL;
     }
@@ -1567,50 +1653,67 @@ gretl_matrix *user_matrix_ols (const gretl_matrix *Y,
     }
 
     if (!nullarg(Uname)) {
-	U = get_matrix_by_name(Uname);
-	if (U == NULL) {
-	    gretl_errmsg_sprintf(_("'%s': no such matrix"), Uname);
-	    *err = E_UNKVAR;
+	U = get_ols_matrix(Uname, T, g, &newU, err);
+	if (*err) {
 	    return NULL;
-	} 
-	if (U->rows != T || U->cols != g) {
-	    newU = 1;
+	}
+    } 
+
+    if (!nullarg(Vname)) {
+	if (g > 1) {
+	    /* multiple dependent variables */
+	    get_ols_matrix(Vname, 0, 0, NULL, err);
+	    if (!*err) {
+		newV = 1;
+	    }
+	} else {
+	    /* single dependent variable */
+	    int nv = g * k;
+
+	    V = get_ols_matrix(Vname, nv, nv, &newV, err);
+	    if (!*err) {
+		ps2 = &s2;
+	    }
 	}
     }
 
-    if (newU) {
-	U = gretl_matrix_alloc(T, g);
-	if (U == NULL) {
+    if (!*err) {
+	B = gretl_matrix_alloc(k, g);
+	if (B == NULL) {
 	    *err = E_ALLOC;
-	    return NULL;
 	}
     }
 
-    B = gretl_matrix_alloc(k, g);
-    if (B == NULL) {
-	*err = E_ALLOC;
-    }
-
     if (!*err) {
 	if (g == 1) {
 	    if (opt & OPT_M) {
 		/* use multiple precision */
-		*err = gretl_matrix_mp_ols(Y, X, B, NULL, U, NULL);
+		*err = gretl_matrix_mp_ols(Y, X, B, V, U, ps2);
 	    } else {
-		*err = gretl_matrix_ols(Y, X, B, NULL, U, NULL);
+		*err = gretl_matrix_ols(Y, X, B, V, U, ps2);
 	    }
 	} else {
-	    *err = gretl_matrix_multi_ols(Y, X, B, U, NULL);
+	    if (newV) {
+		/* note: "V" will actually be (X'X)^{-1} */
+		*err = gretl_matrix_multi_ols(Y, X, B, U, &V);
+	    } else {
+		*err = gretl_matrix_multi_ols(Y, X, B, U, NULL);
+	    }
 	}
     }
 
     if (*err) {
 	gretl_matrix_free(B);
+	B = NULL;
+	if (newU) gretl_matrix_free(U);
+	if (newV) gretl_matrix_free(V);
+    } else {
 	if (newU) {
-	    gretl_matrix_free(U);
+	    user_matrix_replace_matrix_by_name(Uname, U);
 	}
-    } else if (newU) {
-	user_matrix_replace_matrix_by_name(Uname, U);
+	if (newV) {
+	    user_matrix_replace_matrix_by_name(Vname, V);
+	}	
     }
 
     return B;
