@@ -17,7 +17,7 @@
  * 
  */
 
-/* var.c - vector autoregressions */  
+#define FULL_XML_HEADERS
 
 #include "libgretl.h" 
 #include "var.h"  
@@ -30,8 +30,11 @@
 
 #define VDEBUG 0
 
+#define VAR_SE_DFCORR 1
+#define VAR_S_DFCORR 0
+
 enum {
-    VAR_ESTIMATE,
+    VAR_ESTIMATE = 1,
     VAR_LAGSEL,
     VECM_ESTIMATE,
     VECM_CTEST
@@ -70,19 +73,20 @@ static int VAR_add_models (GRETL_VAR *var, const DATAINFO *pdinfo)
 
 static int VAR_add_companion_matrix (GRETL_VAR *var)
 {
-    int g = var->neqns;
-    int n = g * effective_order(var);
-    int i, j, err = 0;
+    int n, i, j, err = 0;
 
     if (var->A != NULL) {
 	return 0;
     }
 
+    n = var->neqns * effective_order(var);
+
     var->A = gretl_matrix_alloc(n, n);
 
     if (var->A == NULL) {
 	err = E_ALLOC;
     } else {
+	int g = var->neqns;
 	double x;
 
 	for (i=g; i<n; i++) {
@@ -96,26 +100,25 @@ static int VAR_add_companion_matrix (GRETL_VAR *var)
     return err;
 }
 
-static int VAR_add_cholesky_matrix (GRETL_VAR *var)
+static int VAR_allocate_cholesky_matrix (GRETL_VAR *var)
 {
-    int n = var->neqns * effective_order(var);
-    int err = 0;
+    int n, err = 0;
 
     if (var->C != NULL) {
 	return 0;
-    }    
+    } 
 
-    var->C = gretl_matrix_alloc(n, var->neqns);
+    n = var->neqns * effective_order(var);
+
+    var->C = gretl_zero_matrix_new(n, var->neqns);
     if (var->C == NULL) {
 	err = E_ALLOC;
-    } else {
-	gretl_matrix_zero(var->C);
-    }
+    } 
 
     return err;
 }
 
-static int VAR_add_residuals_matrix (GRETL_VAR *var)
+static int VAR_allocate_residuals_matrix (GRETL_VAR *var)
 {
     int err = 0;
 
@@ -123,7 +126,7 @@ static int VAR_add_residuals_matrix (GRETL_VAR *var)
 	return 0;
     }      
 
-    var->E = gretl_matrix_alloc(var->T, var->neqns);
+    var->E = gretl_zero_matrix_new(var->T, var->neqns);
     if (var->E == NULL) {
 	err = E_ALLOC;
     } 
@@ -131,20 +134,21 @@ static int VAR_add_residuals_matrix (GRETL_VAR *var)
     return err;
 }
 
-static int VAR_add_vcv_matrix (GRETL_VAR *var)
+static int VAR_add_XTX_matrix (GRETL_VAR *var)
 {
-    int k, err = 0;
+    int k = var->X->cols;
+    int err = 0;
 
-    if (var->vcv != NULL) {
-	return 0;
-    }      
+    var->XTX = gretl_zero_matrix_new(k, k);
 
-    k = var->ncoeff * var->neqns;
-
-    var->vcv = gretl_matrix_alloc(k, k);
-    if (var->vcv == NULL) {
+    if (var->XTX == NULL) {
 	err = E_ALLOC;
-    } 
+    } else {
+	gretl_matrix_multiply_mod(var->X, GRETL_MOD_TRANSPOSE,
+				  var->X, GRETL_MOD_NONE,
+				  var->XTX, GRETL_MOD_NONE);
+	err = gretl_invert_matrix(var->XTX);
+    }
 
     return err;
 }
@@ -192,7 +196,6 @@ void gretl_VAR_clear (GRETL_VAR *var)
     var->E = NULL;
     var->C = NULL;
     var->S = NULL;
-    var->vcv = NULL;
     var->F = NULL;
 
     var->models = NULL;
@@ -290,6 +293,11 @@ void VAR_fill_X (GRETL_VAR *v, int p, const double **Z,
 	k++;
     }
 
+    if (v->X != NULL) {
+	gretl_matrix_set_t1(v->X, v->t1);
+	gretl_matrix_set_t2(v->X, v->t2);
+    }
+
 #if VDEBUG
     gretl_matrix_print(v->X, "X");
 #endif
@@ -336,7 +344,12 @@ static void VAR_fill_Y (GRETL_VAR *v, int mod, const double **Z)
 	    }
 	    k++;
 	}
-    }    
+    }  
+
+    if (v->Y != NULL) {
+	gretl_matrix_set_t1(v->Y, v->t1);
+	gretl_matrix_set_t2(v->Y, v->t2);
+    }
 
 #if VDEBUG
     gretl_matrix_print(v->Y, "Y");
@@ -619,7 +632,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
     ci = (code >= VECM_ESTIMATE)? VECM : VAR;
 
     if ((ci == VAR && order < 1) || (ci == VECM && order < 0)) {
-	sprintf(gretl_errmsg, _("Invalid lag order %d"), order);
+	gretl_errmsg_sprintf(_("Invalid lag order %d"), order);
 	*errp = E_DATA;
 	return NULL;
     }
@@ -643,7 +656,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
     err = VAR_make_lists(var, list, Z, pdinfo);
 
     if (!err && rank > var->ylist[0]) {
-	sprintf(gretl_errmsg, _("vecm: rank %d is out of bounds"), rank);
+	gretl_errmsg_sprintf(_("vecm: rank %d is out of bounds"), rank);
 	err = E_DATA;
     }
 
@@ -673,11 +686,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
     }
 
     if (!err && var->ci == VAR) {
-	err = VAR_add_cholesky_matrix(var);
-    }
-
-    if (!err && var->ci == VAR) {
-	err = VAR_add_vcv_matrix(var);
+	err = VAR_allocate_cholesky_matrix(var);
     }
 
     if (!err && code != VAR_LAGSEL) {
@@ -772,7 +781,6 @@ void gretl_VAR_free (GRETL_VAR *var)
     gretl_matrix_free(var->X);
     gretl_matrix_free(var->B);
     gretl_matrix_free(var->XTX);
-    gretl_matrix_free(var->vcv);
 
     gretl_matrix_free(var->A);
     gretl_matrix_free(var->L);
@@ -1001,8 +1009,10 @@ VAR_add_forecast (GRETL_VAR *var, int t1, int t2,
 
     VAR_add_fcast_variance(var, var->F, tdyn - t1);
 
-    gretl_matrix_set_t1(var->F, t1);
-    gretl_matrix_set_t2(var->F, t2);
+    if (var->F != NULL) {
+	gretl_matrix_set_t1(var->F, t1);
+	gretl_matrix_set_t2(var->F, t2);
+    }
 
 #if VDEBUG
     gretl_matrix_print(var->F, "var->F");
@@ -1247,7 +1257,11 @@ int set_VAR_model_stats (GRETL_VAR *var, int i)
     }
 
     pmod->ess = SSR;
+#if VAR_SE_DFCORR
     pmod->sigma = sqrt(SSR / pmod->dfd);
+#else
+    pmod->sigma = sqrt(SSR / var->T);
+#endif
     pmod->tss = TSS;
     pmod->rsq = 1.0 - SSR / TSS;
     pmod->adjrsq = 1.0 - (SSR / (pmod->dfd)) / (TSS / (pmod->nobs - 1));
@@ -1316,6 +1330,25 @@ gretl_VAR_get_residual_matrix (const GRETL_VAR *var)
     return var->E;
 }
 
+#define samesize(p,q) (p->rows == q->rows && p->cols == q->cols)
+
+gretl_matrix *
+gretl_VAR_get_fitted_matrix (const GRETL_VAR *var)
+{
+    gretl_matrix *Yh = NULL;
+
+    if (var->Y != NULL && var->E != NULL && samesize(var->Y, var->E)) {
+	Yh = gretl_matrix_copy(var->Y);
+	if (Yh != NULL) {
+	    gretl_matrix_subtract_from(Yh, var->E);
+	    Yh->t1 = var->t1;
+	    Yh->t2 = var->t2;
+	}
+    }
+
+    return Yh;
+}
+
 int gretl_VAR_get_variable_number (const GRETL_VAR *var, int k)
 {
     if (var->models != NULL && k >= 0 && k < var->neqns) {
@@ -1450,6 +1483,7 @@ gretl_VAR_get_point_responses (GRETL_VAR *var, int targ, int shock,
  * @targ: index of the target or response variable.
  * @shock: index of the source or shock variable.
  * @periods: number of periods over which to compute the response.
+ * @alpha: determines confidence level for bootstrap interval.
  * @Z: data array (or %NULL).
  * @pdinfo: dataset information.
  *
@@ -1463,10 +1497,10 @@ gretl_VAR_get_point_responses (GRETL_VAR *var, int targ, int shock,
  * If @Z is %NULL, the response matrix returned is a column vector 
  * of length @periods, giving the point estimate of the response 
  * function.  If @Z is not %NULL, the response matrix returned
- * has three columns, containing the point estimate, the 0.025
- * and the 0.975 quantile, where the quantiles are based on 999
- * bootstrap replications, with resampling of the original
- * residuals with replacement.
+ * has three columns, containing the point estimate, the @alpha / 2
+ * quantile and the 1 - @alpha / 2 quantile, where the quantiles
+ * are based on 999 bootstrap replications, with resampling of the 
+ * original residuals with replacement.
  *
  * Returns: matrix containing the estimated impulse responses,
  * with or without a confidence interval depending on whether
@@ -1753,7 +1787,10 @@ const gretl_matrix *gretl_VAR_get_roots (GRETL_VAR *var, int *err)
     return var->L;
 }
 
-double gretl_VAR_ldet (GRETL_VAR *var, int *err)
+/* used in context of LR tests: see vartest.c */
+
+double gretl_VAR_ldet (GRETL_VAR *var, const gretl_matrix *E,
+		       int *err)
 {
     gretl_matrix *S = NULL;
     double ldet = NADBL;
@@ -1763,10 +1800,10 @@ double gretl_VAR_ldet (GRETL_VAR *var, int *err)
     if (S == NULL) {
 	*err = E_ALLOC;
     } else {
-	gretl_matrix_multiply_mod(var->F, GRETL_MOD_TRANSPOSE,
-				  var->F, GRETL_MOD_NONE,
+	gretl_matrix_multiply_mod(E, GRETL_MOD_TRANSPOSE,
+				  E, GRETL_MOD_NONE,
 				  S, GRETL_MOD_NONE);
-	gretl_matrix_divide_by_scalar(S, var->T); /* or var->df? */
+	gretl_matrix_divide_by_scalar(S, var->T);
 	ldet = gretl_vcv_log_determinant(S);
 	if (na(ldet)) {
 	    *err = 1;
@@ -1777,10 +1814,79 @@ double gretl_VAR_ldet (GRETL_VAR *var, int *err)
     return ldet;
 }
 
-static int VAR_add_stats (GRETL_VAR *var)
+/* identify columns of the VAR X matrix that contain the final
+   lag of an endogenous variable */
+
+static int omit_column (GRETL_VAR *var, int nl, int j)
 {
+    if (var->ifc) {
+	return j % nl == 0 && j < 1 + var->neqns * nl;
+    } else {
+	return (j+1) % nl == 0 && j < var->neqns * nl;
+    }
+}
+
+/* make and record residuals for LR test on last lag */
+
+static gretl_matrix *VAR_short_residuals (GRETL_VAR *var, int *err)
+{
+    gretl_matrix *X = NULL;
+    gretl_matrix *B = NULL;
+    gretl_matrix *E = NULL;
+    double x;
+    /* note: removing one lag from each equation */
+    int g = var->ncoeff - var->neqns;
+    int j, t, k, nl;
+
+    E = gretl_matrix_alloc(var->T, var->neqns);
+    X = gretl_matrix_alloc(var->T, g);
+    B = gretl_matrix_alloc(g, var->neqns);
+
+    if (E == NULL || X == NULL || B == NULL) {
+	*err = E_ALLOC;
+	goto bailout;
+    }
+
+    nl = var_n_lags(var);
+
+    k = 0;
+    for (j=0; j<var->ncoeff; j++) {
+	/* loop across the cols of var->X */
+	if (j > 0 && omit_column(var, nl, j)) {
+	    continue;
+	}
+	for (t=0; t<var->T; t++) {
+	    x = gretl_matrix_get(var->X, t, j);
+	    gretl_matrix_set(X, t, k, x);
+	}
+	k++;
+    }
+
+    /* put residuals from "short" estimation into E */
+    *err = gretl_matrix_multi_ols(var->Y, X, B, E, NULL);
+
+ bailout:
+
+    gretl_matrix_free(X);
+    gretl_matrix_free(B);
+    
+    if (*err) {
+	gretl_matrix_free(E);
+	E = NULL;
+    }
+
+    return E;
+}
+
+static int VAR_add_stats (GRETL_VAR *var, int code)
+{
+    gretl_matrix *E1 = NULL;
     int err = 0;
 
+    if (var->order > 1 && code == VAR_ESTIMATE) {
+	E1 = VAR_short_residuals(var, &err);
+    }
+
     var->S = gretl_matrix_alloc(var->neqns, var->neqns);
     if (var->S == NULL) {
 	err = E_ALLOC;
@@ -1790,7 +1896,9 @@ static int VAR_add_stats (GRETL_VAR *var)
 	gretl_matrix_multiply_mod(var->E, GRETL_MOD_TRANSPOSE,
 				  var->E, GRETL_MOD_NONE,
 				  var->S, GRETL_MOD_NONE);
-	gretl_matrix_divide_by_scalar(var->S, var->T); /* OK, for ldet? */
+	/* for computing log-determinant, don't apply df
+	   correction (?) */
+	gretl_matrix_divide_by_scalar(var->S, var->T);
     }
 
     if (!err) {
@@ -1798,7 +1906,16 @@ static int VAR_add_stats (GRETL_VAR *var)
 	if (na(var->ldet)) {
 	    err = 1;
 	}
-	gretl_matrix_multiply_by_scalar(var->S, var->T / var->df);
+
+#if VAR_S_DFCORR
+	/* Hmm, should we df-adjust var->S here?  Note that this
+	   will affect the impulse response output */
+	if (1) {
+	    double cfac = var->T / (double) var->df;
+	
+	    gretl_matrix_multiply_by_scalar(var->S, cfac);
+	}
+#endif
     }    
 
     if (!err) {
@@ -1813,8 +1930,11 @@ static int VAR_add_stats (GRETL_VAR *var)
 	var->HQC = (-2.0 * var->ll + 2.0 * k * log(log(T))) / T;
     }
 
-    if (!err && var->F != NULL) {
-	VAR_LR_lag_test(var);
+    if (E1 != NULL) {
+	if (!err) {
+	    VAR_LR_lag_test(var, E1);
+	}
+	gretl_matrix_free(E1);
     }
 
     if (!err) {
@@ -1907,37 +2027,6 @@ void VAR_write_A_matrix (GRETL_VAR *v)
 #endif
 }
 
-static void VAR_write_vcv_matrix (GRETL_VAR *v)
-{
-    if (v->S != NULL && v->XTX != NULL && v->vcv != NULL) {
-	gretl_matrix_kronecker_product(v->S, v->XTX, v->vcv);
-    }
-}
-
-static int VAR_finalize (GRETL_VAR *var)
-{
-    int err = 0;
-
-    VAR_write_A_matrix(var);
-
-    err = VAR_wald_omit_tests(var, var->ifc);
-
-    if (!err && var->order > 1) {
-	err = last_lag_LR_prep(var, var->ifc);
-    }
-
-    if (!err) {
-	err = VAR_add_stats(var);
-	VAR_write_vcv_matrix(var);
-    }
-
-    if (!err) {
-	err = gretl_VAR_do_error_decomp(var->S, var->C);
-    }
-
-    return err;
-}
-
 static int VAR_depvar_name (GRETL_VAR *var, int i, const char *yname)
 {
     MODEL *pmod = var->models[i];
@@ -1972,17 +2061,15 @@ int transcribe_VAR_models (GRETL_VAR *var,
     int i, j, jmax;
     int err = 0;
 
-    jmax = (var->B != NULL)? var->B->rows : 0;
-
     params = strings_array_new_with_length(var->ncoeff, VNAMELEN);
     if (params == NULL) {
 	return E_ALLOC;
     }
 
-    jmax = (var->B != NULL)? var->B->rows : 0;
-
     gretl_VAR_param_names(var, params, pdinfo);
 
+    jmax = (var->B != NULL)? var->B->rows : 0;
+
     for (i=0; i<var->neqns && !err; i++) {
 	yno = var->ylist[i+1];
 	y = Z[yno];
@@ -2106,6 +2193,7 @@ GRETL_VAR *gretl_VAR (int order, int *list,
 	}
     }
 
+    /* allocation and initial set-up */
     var = gretl_VAR_new(code, order, 0, lags, list, Z, pdinfo, 
 			opt, err);
     if (var == NULL) {
@@ -2119,15 +2207,29 @@ GRETL_VAR *gretl_VAR (int order, int *list,
 
     if (!*err) {
 	if (code == VAR_LAGSEL) {
-	    *err = VAR_add_stats(var);
+	    /* doing lag-length selection */
+	    *err = VAR_add_stats(var, code);
 	    if (!*err) {
 		*err = VAR_do_lagsel(var, Z, pdinfo, prn);
 	    }
 	} else {
+	    /* regular VAR estimation */
 	    *err = transcribe_VAR_models(var, Z, pdinfo, NULL);
+
 	    if (!*err) {
-		*err = VAR_finalize(var);
+		VAR_write_A_matrix(var);
+		/* note: the following also sets std. errors */
+		*err = VAR_wald_omit_tests(var);
 	    }
+
+	    if (!*err) {
+		*err = VAR_add_stats(var, code);
+	    }
+
+	    if (!*err) {
+		*err = gretl_VAR_do_error_decomp(var->S, var->C);
+	    }
+
 	    if (!*err) {
 		gretl_VAR_print(var, pdinfo, opt, prn);
 	    }
@@ -2174,6 +2276,9 @@ print_johansen_sigmas (const JohansenInfo *jv, PRN *prn)
     }     
 }
 
+/* called from gretl_restriction_finalize() if the target
+   is a VECM */
+
 int gretl_VECM_test (GRETL_VAR *vecm, 
 		     gretl_restriction *rset,
 		     const DATAINFO *pdinfo, 
@@ -2507,10 +2612,10 @@ static int jvar_check_allocation (GRETL_VAR *v, const DATAINFO *pdinfo)
 	err = VAR_add_companion_matrix(v);
     }
     if (!err) {
-	err = VAR_add_cholesky_matrix(v);
+	err = VAR_allocate_cholesky_matrix(v);
     }
     if (!err) {
-	err = VAR_add_residuals_matrix(v);
+	err = VAR_allocate_residuals_matrix(v);
     }
 
     return err;
@@ -2672,7 +2777,7 @@ GRETL_VAR *gretl_VECM (int order, int rank, int *list,
     int *lags = NULL;
 
     if (rank <= 0) {
-	sprintf(gretl_errmsg, _("vecm: rank %d is out of bounds"), rank);
+	gretl_errmsg_sprintf(_("vecm: rank %d is out of bounds"), rank);
 	*err = E_DATA;
 	return NULL;
     }
@@ -3141,9 +3246,46 @@ VAR_matrix_from_models (const GRETL_VAR *var, int idx, int *err)
     return m;
 }
 
+static gretl_matrix *VECM_get_EC_matrix (const GRETL_VAR *v, int *err)
+{
+    gretl_matrix *EC = NULL;
+    double x;
+    int rank, k, k0;
+    int j, t, T;
+
+    if (v->X == NULL) {
+	fprintf(stderr, "VECM_get_EC_matrix: v->X is NULL\n");
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    rank = jrank(v);
+    T = v->X->rows;
+
+    EC = gretl_matrix_alloc(T, rank);
+    if (EC == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    k0 = v->ncoeff - rank;
+
+    for (j=0, k=k0; j<rank; j++, k++) {
+	for (t=0; t<T; t++) {
+	    x = gretl_matrix_get(v->X, t, k);
+	    gretl_matrix_set(EC, t, j, x);
+	}
+    }
+
+    gretl_matrix_set_t1(EC, v->t1);
+    gretl_matrix_set_t2(EC, v->t2);
+	
+    return EC;
+}
+
 #define vecm_matrix(i) (i == M_JALPHA || i == M_JBETA || \
                         i == M_JVBETA || i == M_JS00 || \
-                        i == M_JS11 || i == M_JS01)
+                        i == M_JS11 || i == M_JS01 || i == M_EC)
 
 gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx, 
 				    int *err)
@@ -3159,13 +3301,16 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 
     if (idx == M_UHAT) {
 	src = gretl_VAR_get_residual_matrix(var);
+    } else if (idx == M_YHAT) {
+	M = gretl_VAR_get_fitted_matrix(var);
+	copy = 0;
     } else if (idx == M_COMPAN) {
 	src = var->A;
     } else if (idx == M_COEFF || idx == M_SE) {
 	M = VAR_matrix_from_models(var, idx, err);
 	copy = 0;
-    } else if (idx == M_VCV) {
-	src = var->vcv;
+    } else if (idx == M_XTXINV) {
+	src = var->XTX;
     } else if (idx == M_SIGMA) {
 	src = var->S;
     } else if (vecm_matrix(idx)) {
@@ -3189,8 +3334,12 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 	    case M_JS01:
 		src = var->jinfo->S01;
 		break;
+	    case M_EC:
+		M = VECM_get_EC_matrix(var, err);
+		copy = 0;
+		break;
 	    }
-	}
+	} 
     }
 
     if (copy) {
@@ -3466,24 +3615,23 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
     double x;
     int gotA = (var->A != NULL);
     int gotX = (var->X != NULL);
-    int gotV = (var->vcv != NULL);
     int j, i;
     int err = 0;
 
     if (var->E == NULL) {
-	err = VAR_add_residuals_matrix(var);
+	err = VAR_allocate_residuals_matrix(var);
     }
 
     if (!err && var->A == NULL) {
 	err = VAR_add_companion_matrix(var);
     }
 
-    if (!err && var->vcv == NULL) {
-	err = VAR_add_vcv_matrix(var);
-    } 
+    if (!err && gotX && var->XTX == NULL) {
+	err = VAR_add_XTX_matrix(var);
+    }     
 
     if (!err && var->C == NULL) {
-	err = VAR_add_cholesky_matrix(var);
+	err = VAR_allocate_cholesky_matrix(var);
     } 
 
     if (!err && var->B == NULL) {
@@ -3517,10 +3665,6 @@ static int rebuild_VAR_matrices (GRETL_VAR *var)
 	fprintf(stderr, "Can't we rebuild VAR->X somehow?\n");
     }    
 
-    if (!err && !gotV) {
-	VAR_write_vcv_matrix(var);
-    }
-
     return err;
 }
 
@@ -3558,6 +3702,7 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc,
     }
 
     /* these are not show-stoppers */
+    gretl_xml_get_prop_as_int(node, "robust", &var->robust);
     gretl_xml_get_prop_as_int(node, "detflags", &var->detflags);
     gretl_xml_get_prop_as_int(node, "LBs", &var->LBs);
     gretl_xml_get_prop_as_double(node, "LB", &var->LB);
@@ -3638,8 +3783,7 @@ GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc,
     }
 
     if (!*err) {
-	/* covariance matrix and related things */
-	*err = VAR_add_stats(var);
+	*err = VAR_add_stats(var, 0);
     }
 
     if (!*err) {
@@ -3706,6 +3850,10 @@ int gretl_VAR_serialize (const GRETL_VAR *var, SavedObjectFlags flags,
     fprintf(fp, "ecm=\"%d\" neqns=\"%d\" order=\"%d\" detflags=\"%d\" ",
 	    (var->ci == VECM), var->neqns, var->order, var->detflags);
 
+    if (var->robust) {
+	gretl_xml_put_int("robust", 1, fp);
+    }
+
     if (var->LBs > 0 && !na(var->LB)) {
 	/* Portmanteau test */
 	gretl_xml_put_double("LB", var->LB, fp);
