@@ -48,21 +48,6 @@ int gretl_VAR_print_sigma (const GRETL_VAR *var, PRN *prn)
     return err;
 }
 
-static void tex_print_double (double x, PRN *prn)
-{
-    char number[16];
-
-    x = screen_zero(x);
-
-    sprintf(number, "%#.*g", GRETL_DIGITS, x);
-
-    if (x < 0.) {
-	pprintf(prn, "$-$%s", number + 1);
-    } else {
-	pputs(prn, number);
-    }
-}
-
 /* printing of impulse responses and variance decompositions */
 
 #define IRF_ROW_MAX 4
@@ -243,7 +228,6 @@ static int varprint_namelen (const GRETL_VAR *var, const DATAINFO *pdinfo,
  * @shock: index number of the "shock" variable.
  * @periods: number of periods over which to print response.
  * @pdinfo: dataset information.
- * @pause: if non-zero, pause between sections of output.
  * @prn: gretl printing object.
  *
  * Prints to @prn the estimated responses of the endogenous
@@ -259,7 +243,7 @@ static int varprint_namelen (const GRETL_VAR *var, const DATAINFO *pdinfo,
 int 
 gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 				  int periods, const DATAINFO *pdinfo, 
-				  int pause, PRN *prn)
+				  PRN *prn)
 {
     gretl_matrix *rtmp, *ctmp;
     int rows = var->neqns * effective_order(var);
@@ -360,10 +344,6 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 	}
 
 	VAR_info_end_table(prn);
-
-	if (pause && block < blockmax - 1) {
-	    scroll_pause();
-	}
     }
 
     if (rtmp != NULL) gretl_matrix_free(rtmp);
@@ -375,21 +355,19 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 int gretl_VAR_print_all_impulse_responses (GRETL_VAR *var, const DATAINFO *pdinfo, 
 					   int horizon, PRN *prn)
 {
-    int i, pause = 0, err = 0;
+    int i, err = 0;
 
     if (horizon <= 0) {
 	horizon = default_VAR_horizon(pdinfo);
     }
 
-    if (plain_format(prn)) {
-	pause = gretl_get_text_pause();
-    } else if (rtf_format(prn)) {
+    if (rtf_format(prn)) {
 	pputs(prn, "{\\rtf1\\par\n\\qc ");
     }
 
     for (i=0; i<var->neqns && !err; i++) {
 	err = gretl_VAR_print_impulse_response(var, i, horizon, pdinfo, 
-					       pause, prn);
+					       prn);
     }
 
     if (rtf_format(prn)) {
@@ -405,7 +383,6 @@ int gretl_VAR_print_all_impulse_responses (GRETL_VAR *var, const DATAINFO *pdinf
  * @targ:
  * @periods: number of periods over which to print decomposition.
  * @pdinfo: dataset information.
- * @pause: if non-zero, pause between sections of output.
  * @prn: gretl printing struct.
  *
  *
@@ -415,7 +392,7 @@ int gretl_VAR_print_all_impulse_responses (GRETL_VAR *var, const DATAINFO *pdinf
 int 
 gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 			      int periods, const DATAINFO *pdinfo, 
-			      int pause, PRN *prn)
+			      PRN *prn)
 {
     int i, t;
     int vtarg;
@@ -519,10 +496,6 @@ gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 	}
 
 	VAR_info_end_table(prn);
-
-	if (pause && block < blockmax - 1) {
-	    scroll_pause();
-	}
     }
 
     if (vd != NULL) {
@@ -535,21 +508,19 @@ gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 int gretl_VAR_print_all_fcast_decomps (GRETL_VAR *var, const DATAINFO *pdinfo, 
 				       int horizon, PRN *prn)
 {
-    int i, pause = 0, err = 0;
+    int i, err = 0;
 
     if (horizon <= 0) {
 	horizon = default_VAR_horizon(pdinfo);
     }
 
-    if (plain_format(prn)) {
-	pause = gretl_get_text_pause();
-    } else if (rtf_format(prn)) {
+    if (rtf_format(prn)) {
 	pputs(prn, "{\\rtf1\\par\n\\qc ");
     }
 
     for (i=0; i<var->neqns && !err; i++) {
 	err = gretl_VAR_print_fcast_decomp(var, i, horizon, pdinfo, 
-					   pause, prn);
+					   prn);
     }
 
     if (rtf_format(prn)) {
@@ -620,6 +591,7 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
 		       PRN *prn)
 {
     JohansenInfo *jv = jvar->jinfo;
+    int utf = plain_format(prn);
     int rtf = rtf_format(prn);
     char namefmt[8];
     char s[16], vname[32];
@@ -628,9 +600,11 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
     int i, j;
     double x;
 
-    pprintf(prn, "beta (%s", _("cointegrating vectors"));
+    pprintf(prn, "beta (%s", (utf)? _("cointegrating vectors") :
+	    I_("cointegrating vectors"));
     if (jv->Bse != NULL) {
-	pprintf(prn, ", %s)", _("standard errors in parentheses"));
+	pprintf(prn, ", %s)", (utf)? _("standard errors in parentheses") :
+		I_("standard errors in parentheses"));
     } else {
 	pputc(prn, ')');
     }
@@ -684,9 +658,11 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
 
     rows = gretl_matrix_rows(jv->Alpha);
 
-    pprintf(prn, "alpha (%s", _("adjustment vectors"));
+    pprintf(prn, "alpha (%s", (utf)? _("adjustment vectors") :
+	    I_("adjustment vectors"));
     if (jv->Ase != NULL) {
-	pprintf(prn, ", %s)", _("standard errors in parentheses"));
+	pprintf(prn, ", %s)", (utf)? _("standard errors in parentheses") :
+		I_("standard errors in parentheses"));
     } else {
 	pputc(prn, ')');
     }
@@ -750,13 +726,15 @@ static int max_vlen (const int *list, const DATAINFO *pdinfo)
 
 static void print_VECM_omega (GRETL_VAR *jvar, const DATAINFO *pdinfo, PRN *prn)
 {
+    int utf = plain_format(prn);
     int rtf = rtf_format(prn);
     int *list = jvar->ylist;
     char s[32];
     int w0, wi = 12;
     int i, j;
 
-    pprintf(prn, "%s\n", _("Cross-equation covariance matrix"));
+    pprintf(prn, "%s:\n", (utf)? _("Cross-equation covariance matrix") :
+	    I_("Cross-equation covariance matrix"));
     gretl_prn_newline(prn);
 
     w0 = max_vlen(list, pdinfo) + 1;
@@ -812,7 +790,9 @@ static void print_VECM_omega (GRETL_VAR *jvar, const DATAINFO *pdinfo, PRN *prn)
 
     gretl_prn_newline(prn);
 
-    pprintf(prn, "%s = %g", _("determinant"), exp(jvar->ldet));
+    pprintf(prn, "%s = %g", (utf)? _("determinant") : I_("determinant"),
+	    exp(jvar->ldet));
+
     gretl_prn_newline(prn);
 }
 
@@ -847,6 +827,10 @@ static void print_LR_stat (double x, int df, PRN *prn)
 {
     double pv = chisq_cdf_comp(df, x);
 
+    if (na(pv)) {
+	return;
+    }
+
     if (tex_format(prn)) {
 	pprintf(prn, "$2 (l_u - l_r) = %g$", x);
 	gretl_prn_newline(prn);
@@ -951,17 +935,17 @@ static void VAR_print_LB_stat (const GRETL_VAR *var, PRN *prn)
     double pv = chisq_cdf_comp(df, var->LB);
 
     if (tex_format(prn)) {
-	pprintf(prn, "\\noindent\n%s: LB(%d) = %g (%s = %d, %s %f)\\par\n",
+	pprintf(prn, "\\noindent\n%s: LB(%d) = %g, %s = %d [%.4f]\\par\n",
 		I_("Portmanteau test"), var->LBs, var->LB,
-		I_("df"), df, I_("p-value"), pv);	    
+		I_("df"), df, pv);	    
     } else if (rtf_format(prn)) {
-	pprintf(prn, "%s: LB(%d) = %g (%s = %d, %s %f)\\par\n",
+	pprintf(prn, "%s: LB(%d) = %g, %s = %d [%.4f]\\par\n",
 		I_("Portmanteau test"), var->LBs, var->LB,
-		I_("df"), df, I_("p-value"), pv);
+		I_("df"), df, pv);
     } else {
-	pprintf(prn, "%s: LB(%d) = %g (%s = %d, %s %f)\n",
+	pprintf(prn, "%s: LB(%d) = %g, %s = %d [%.4f]\n",
 		_("Portmanteau test"), var->LBs, var->LB,
-		_("df"), df, _("p-value"), pv);
+		_("df"), df, pv);
     }
 }
 
@@ -990,14 +974,24 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
     int quiet = (opt & OPT_Q);
+    int nlags, maxlag, nextlag;
     int lldone = 0;
-    int pause = 0;
+    double pv;
     int i, j, k, v;
 
     if (prn == NULL) {
 	return 0;
     }
 
+    nlags = var_n_lags(var);
+    maxlag = var_max_lag(var);
+    
+    if (var->lags != NULL) {
+	nextlag = var->lags[nlags - 1];
+    } else {
+	nextlag = maxlag - 1;
+    }
+
     ntodate(startdate, var->t1, pdinfo);
     ntodate(enddate, var->t2, pdinfo);
 
@@ -1038,7 +1032,6 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	}	
 	pputs(prn, "\\par\n\n");
     } else {
-	pause = gretl_get_text_pause();
 	pprintf(prn, "\n%s\n", Vstr);
 	pprintf(prn, _("%s estimates, observations %s-%s (T = %d)"),
 		(vecm)? _("Maximum likelihood") : _("OLS"), startdate, enddate, var->T);
@@ -1110,10 +1103,6 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	    }
 	}
 
-	if (pause) {
-	    scroll_pause();
-	}
-
 	if (vecm) {
 	    continue;
 	}
@@ -1125,52 +1114,44 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	} else if (rtf) {
 	    pprintf(prn, "%s:\\par\n\n", I_("F-tests of zero restrictions"));
 	} else {
-	    pprintf(prn, "  %s:\n\n", _("F-tests of zero restrictions"));
+	    pprintf(prn, "%s:\n\n", _("F-tests of zero restrictions"));
 	}
 
 	for (j=0; j<var->neqns; j++) {
+	    pv = snedecor_cdf_comp(nlags, dfd, var->Fvals[k]);
 	    v = (var->models[j])->list[1];
 	    if (tex) {
 		pprintf(prn, I_("All lags of %-15s "), pdinfo->varname[v]);
 		pputs(prn, "& ");
-		pprintf(prn, "$F(%d, %d) = %g$ & ", var->order, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			snedecor_cdf_comp(var->order, dfd, var->Fvals[k]));
+		pprintf(prn, "$F(%d, %d) = %g$ & ", nlags, dfd, var->Fvals[k]);
+		pprintf(prn, "[%.4f]\\\\\n", pv);
 	    } else if (rtf) {
 		pprintf(prn, I_("All lags of %-15s "), pdinfo->varname[v]);
-		pprintf(prn, "F(%d, %d) = %8.5g, ", var->order, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			snedecor_cdf_comp(var->order, dfd, var->Fvals[k]));
+		pprintf(prn, "F(%d, %d) = %8.5g ", nlags, dfd, var->Fvals[k]);
+		pprintf(prn, "[%.4f]\\par\n", pv);
 	    } else {
-		pputs(prn, "  ");
 		pprintf(prn, _("All lags of %-15s "), pdinfo->varname[v]);
-		sprintf(Fstr, "F(%d, %d)", var->order, dfd);
-		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			snedecor_cdf_comp(var->order, dfd, var->Fvals[k]));
+		sprintf(Fstr, "F(%d, %d)", nlags, dfd);
+		pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, var->Fvals[k], pv);
 	    }
 	    k++;
 	}
 
 	if (var->order > 1) {
+	    pv = snedecor_cdf_comp(var->neqns, dfd, var->Fvals[k]);
 	    if (tex) {
-		pprintf(prn, I_("All vars, lag %-13d "), var->order);
+		pprintf(prn, I_("All vars, lag %-13d "), maxlag);
 		pputs(prn, "& ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->neqns, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			snedecor_cdf_comp(var->neqns, dfd, var->Fvals[k]));
+		pprintf(prn, "[%.4f]\\\\\n", pv);
 	    } else if (rtf) {
-		pprintf(prn, I_("All vars, lag %-13d "), var->order);
-		pprintf(prn, "F(%d, %d) = %8.5g, ", var->neqns, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			snedecor_cdf_comp(var->neqns, dfd, var->Fvals[k]));
+		pprintf(prn, I_("All vars, lag %-13d "), maxlag);
+		pprintf(prn, "F(%d, %d) = %8.5g ", var->neqns, dfd, var->Fvals[k]);
+		pprintf(prn, "[%.4f]\\par\n", pv);
 	    } else {
-		pputs(prn, "  ");
-		pprintf(prn, _("All vars, lag %-13d "), var->order);
+		pprintf(prn, _("All vars, lag %-13d "), maxlag);
 		sprintf(Fstr, "F(%d, %d)", var->neqns, dfd);
-		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			snedecor_cdf_comp(var->neqns, dfd, var->Fvals[k]));
+		pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, var->Fvals[k], pv);
 	    } 
 	    k++;
 	}
@@ -1181,50 +1162,47 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 		  "\\clearpage\n\n");
 	} else if (rtf) {
 	    pputs(prn, "\\par\\n\n");
-	} else if (pause) {
-	    scroll_pause();
-	}
+	} 
     }
 
-    pputc(prn, '\n');
-
     /* global LR test on max lag */
     if (!na(var->LR)) {
 	char h0str[64];
 	char h1str[64];
 	int df = var->neqns * var->neqns;
 
-	if (tex || rtf) {
-	    sprintf(h0str, I_("the longest lag is %d"), var->order - 1);
-	    sprintf(h1str, I_("the longest lag is %d"), var->order);
+	pputc(prn, '\n');
+
+	if (rtf || tex) {
+	    sprintf(h0str, I_("the longest lag is %d"), nextlag);
+	    sprintf(h1str, I_("the longest lag is %d"), maxlag);
 	} else {
-	    sprintf(h0str, _("the longest lag is %d"), var->order - 1);
-	    sprintf(h1str, _("the longest lag is %d"), var->order);
-	}	    
+	    sprintf(h0str, _("the longest lag is %d"), nextlag);
+	    sprintf(h1str, _("the longest lag is %d"), maxlag);
+	}
+
+	pv = chisq_cdf_comp(df, var->LR);
 
 	if (tex) {
 	    pprintf(prn, "\\noindent %s ---\\par\n", I_("For the system as a whole"));
 	    pprintf(prn, "%s: %s\\par\n", I_("Null hypothesis"), h0str);
 	    pprintf(prn, "%s: %s\\par\n", I_("Alternative hypothesis"), h1str);
-	    pprintf(prn, "%s: $\\chi^2_{%d}$ = %.3f (%s %f)\\par\n",
-		    I_("Likelihood ratio test"), 
-		    df, var->LR, I_("p-value"), chisq_cdf_comp(df, var->LR));
+	    pprintf(prn, "%s: $\\chi^2_{%d}$ = %.3f [%.4f]\\par\n",
+		    I_("Likelihood ratio test"), df, var->LR, pv);
 	} else if (rtf) {
 	    pprintf(prn, "\\par %s\n", I_("For the system as a whole"));
 	    pprintf(prn, "\\par %s: %s\n", I_("Null hypothesis"), h0str);
 	    pprintf(prn, "\\par %s: %s\n", I_("Alternative hypothesis"), h1str);
-	    pprintf(prn, "\\par %s: %s(%d) = %g (%s %f)\n",
-		    I_("Likelihood ratio test"), I_("Chi-square"), 
-		    df, var->LR, I_("p-value"), chisq_cdf_comp(df, var->LR));
+	    pprintf(prn, "\\par %s: %s(%d) = %g [%.4f]\n", I_("Likelihood ratio test"), 
+		    I_("Chi-square"), df, var->LR, pv);
 	} else {
 	    int ordlen = (var->order > 10)? 2 : 1;
 
 	    pprintf(prn, "%s:\n\n", _("For the system as a whole"));
 	    pprintf(prn, "  %s: %s\n", _("Null hypothesis"), h0str);
 	    pprintf(prn, "  %s: %s\n", _("Alternative hypothesis"), h1str);
-	    pprintf(prn, "  %s: %s(%d) = %g (%s %f)\n",
-		    _("Likelihood ratio test"), _("Chi-square"), 
-		    df, var->LR, _("p-value"), chisq_cdf_comp(df, var->LR));
+	    pprintf(prn, "  %s: %s(%d) = %g [%.4f]\n", _("Likelihood ratio test"), 
+		    _("Chi-square"), df, var->LR, pv);
 	    /* Info criteria comparison */
 	    pprintf(prn, "\n  %s:\n", _("Comparison of information criteria"));
 	    pputs(prn, "  ");
