@@ -32,18 +32,17 @@ void VAR_write_A_matrix (GRETL_VAR *v);
 int johansen_stage_1 (GRETL_VAR *jvar, const double **Z, 
 		      const DATAINFO *pdinfo);
 
-double gretl_VAR_ldet (GRETL_VAR *var, int *err);
+double gretl_VAR_ldet (GRETL_VAR *var, const gretl_matrix *E,
+		       int *err);
 
-int VAR_LR_lag_test (GRETL_VAR *var);
-
-int last_lag_LR_prep (GRETL_VAR *var, int ifc);
+int VAR_LR_lag_test (GRETL_VAR *var, const gretl_matrix *E);
 
 int VAR_portmanteau_test (GRETL_VAR *var);
 
 int VAR_do_lagsel (GRETL_VAR *var, const double **Z, 
 		   const DATAINFO *pdinfo, PRN *prn);
 
-int VAR_wald_omit_tests (GRETL_VAR *var, int ifc);
+int VAR_wald_omit_tests (GRETL_VAR *var);
 
 gretl_matrix *VAR_coeff_matrix_from_VECM (const GRETL_VAR *var);
 
