@@ -21,13 +21,17 @@ export "GRETL_HOME=$TOP/share/gretl/"
 export "GTKSOURCEVIEW_LANGUAGE_DIR=$TOP/share/gretl/gtksourceview"
 
 # location of gnuplot help file
-export "GNUHELP=$TOP/share/gnuplot/4.3/gnuplot.gih"
+export "GNUHELP=$TOP/share/gnuplot/4.5/gnuplot.gih"
 # location of gnuplot X11 driver
-export "GNUPLOT_DRIVER_DIR=$TOP/libexec/gnuplot/4.3"
+export "GNUPLOT_DRIVER_DIR=$TOP/libexec/gnuplot/4.5"
 # we do not support the 'aqua' terminal type
 if [ "$GNUTERM" = "aqua" ] ; then
    export GNUTERM=x11
 fi
+# R shared library: may require platform-specific symlink
+if [ "x$R_HOME" = "x" ] ; then
+   export R_HOME=/Library/Frameworks/R.framework/Resources
+fi
 
 export "PATH=$CWD:$PATH"
 
