@@ -61,7 +61,7 @@ static void bchecker_free (struct bchecker *b)
     }
 }
 
-static struct bchecker *bchecker_allocate (struct arma_info *ainfo)
+static struct bchecker *bchecker_allocate (arma_info *ainfo)
 {
     struct bchecker *b;
 
@@ -92,7 +92,7 @@ static struct bchecker *bchecker_allocate (struct arma_info *ainfo)
    course of iteration */
 
 static int 
-ma_out_of_bounds (struct arma_info *ainfo, const double *theta,
+ma_out_of_bounds (arma_info *ainfo, const double *theta,
 		  const double *Theta)
 {
     static struct bchecker *b = NULL;
@@ -202,7 +202,7 @@ static void bounds_checker_cleanup (void)
 }
 
 static void do_MA_partials (double *drv,
-			    struct arma_info *ainfo,
+			    arma_info *ainfo,
 			    const double *theta,
 			    const double *Theta,
 			    int t)
@@ -261,7 +261,7 @@ static int arma_ll (double *coeff,
     const double *theta, *Theta;
     const double *beta;
 
-    struct arma_info *ainfo;
+    arma_info *ainfo;
 
     double ll, s2 = 0.0;
     int err = 0;
@@ -505,19 +505,18 @@ static int arma_ll (double *coeff,
   returns: zero on success, non-zero on failure
 */
 
-static int arma_model_add_roots (MODEL *pmod, struct arma_info *ainfo,
+static int arma_model_add_roots (MODEL *pmod, arma_info *ainfo,
 				 const double *coeff)
 {
     const double *phi = coeff + ainfo->ifc;
     const double *Phi = phi + ainfo->np;
     const double *theta = Phi + ainfo->P;
     const double *Theta = theta + ainfo->nq;
-
     int nr = ainfo->p + ainfo->P + ainfo->q + ainfo->Q;
     int pmax, qmax, lmax;
     double *temp = NULL, *tmp2 = NULL;
     cmplx *rptr, *roots = NULL;
-    int i, k;
+    int i, k, cerr = 0;
 
     pmax = (ainfo->p > ainfo->P)? ainfo->p : ainfo->P;
     qmax = (ainfo->q > ainfo->Q)? ainfo->q : ainfo->Q;
@@ -551,20 +550,20 @@ static int arma_model_add_roots (MODEL *pmod, struct arma_info *ainfo,
 		temp[i+1] = 0;
 	    }
 	}
-	polrt(temp, tmp2, ainfo->p, rptr);
+	cerr = polrt(temp, tmp2, ainfo->p, rptr);
 	rptr += ainfo->p;
     }
 
-    if (ainfo->P > 0) {
+    if (!cerr && ainfo->P > 0) {
 	/* B(L), seasonal */
 	for (i=0; i<ainfo->P; i++) {
 	    temp[i+1] = -Phi[i];
 	}    
-	polrt(temp, tmp2, ainfo->P, rptr);
+	cerr = polrt(temp, tmp2, ainfo->P, rptr);
 	rptr += ainfo->P;
     }
 
-    if (ainfo->q > 0) {
+    if (!cerr && ainfo->q > 0) {
 	/* C(L), non-seasonal */
 	k = 0;
 	for (i=0; i<ainfo->q; i++) {
@@ -574,23 +573,27 @@ static int arma_model_add_roots (MODEL *pmod, struct arma_info *ainfo,
 		temp[i+1] = 0;
 	    }
 	}  
-	polrt(temp, tmp2, ainfo->q, rptr);
+	cerr = polrt(temp, tmp2, ainfo->q, rptr);
 	rptr += ainfo->q;
     }
 
-    if (ainfo->Q > 0) {
+    if (!cerr && ainfo->Q > 0) {
 	/* D(L), seasonal */
 	for (i=0; i<ainfo->Q; i++) {
 	    temp[i+1] = Theta[i];
 	}  
-	polrt(temp, tmp2, ainfo->Q, rptr);
+	cerr = polrt(temp, tmp2, ainfo->Q, rptr);
     }
     
     free(temp);
     free(tmp2);
 
-    gretl_model_set_data(pmod, "roots", roots, GRETL_TYPE_CMPLX_ARRAY,
-			 nr * sizeof *roots);
+    if (cerr) {
+	free(roots);
+    } else {
+	gretl_model_set_data(pmod, "roots", roots, GRETL_TYPE_CMPLX_ARRAY,
+			     nr * sizeof *roots);
+    }
 
     return 0;
 }
@@ -612,7 +615,7 @@ static gretl_matrix *vQ;
 static double *ac;
 static double *mc;
 
-static struct arma_info *kainfo;   
+static arma_info *kainfo;   
 
 static void allocate_kalman_matrices (int r, int r2,
 				      int k, int T)
@@ -639,7 +642,7 @@ static void free_kalman_matrices (void)
     gretl_matrix_free(Svar);
 }
 
-static int ainfo_get_r (struct arma_info *ainfo)
+static int ainfo_get_r (arma_info *ainfo)
 {
     int pmax = ainfo->p + ainfo->pd * ainfo->P;
     int qmax = ainfo->q + ainfo->pd * ainfo->Q;
@@ -647,7 +650,7 @@ static int ainfo_get_r (struct arma_info *ainfo)
     return (pmax > qmax + 1)? pmax : qmax + 1;
 }
 
-static int allocate_ac_mc (struct arma_info *ainfo)
+static int allocate_ac_mc (arma_info *ainfo)
 {
     if (ainfo->P > 0) {
 	int pmax = ainfo->p + ainfo->pd * ainfo->P;
@@ -678,7 +681,7 @@ static void free_ac_mc (void)
 
 static void write_big_phi (const double *phi, 
 			   const double *Phi,
-			   struct arma_info *ainfo,
+			   arma_info *ainfo,
 			   gretl_matrix *F)
 {
     int pmax = ainfo->p + ainfo->pd * ainfo->P;
@@ -712,7 +715,7 @@ static void write_big_phi (const double *phi,
 
 static void write_big_theta (const double *theta, 
 			     const double *Theta,
-			     struct arma_info *ainfo,
+			     arma_info *ainfo,
 			     gretl_matrix *H)
 {
     int qmax = ainfo->q + ainfo->pd * ainfo->Q;
@@ -783,7 +786,7 @@ static void condense_state_vcv (gretl_matrix *targ,
     }
 }
 
-static void kalman_matrices_init (struct arma_info *ainfo)
+static void kalman_matrices_init (arma_info *ainfo)
 {
     int r = F->rows;
 
@@ -1093,6 +1096,16 @@ static void maybe_rescale_hessian (int kopt, double *vcv,
     }
 }
 
+static int arima_ydiff_only (arma_info *ainfo)
+{
+    if ((ainfo->d > 0 || ainfo->D > 0) &&
+	ainfo->nexo > 0 && !(ainfo->flags & ARMA_XDIFF)) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
 static int arma_use_hessian (gretlopt opt)
 {
     int ret = 1;
@@ -1107,21 +1120,21 @@ static int arma_use_hessian (gretlopt opt)
 }
 
 static int kalman_arma_finish (MODEL *pmod, const int *alist,
-			       struct arma_info *ainfo,
+			       arma_info *ainfo,
 			       const double **Z, const DATAINFO *pdinfo, 
-			       kalman *K, double *b, int k, int T,
+			       kalman *K, double *b, 
 			       gretlopt opt, PRN *prn)
 {
-    int do_hess = arma_use_hessian(opt);
     double s2;
-    int kopt, i;
+    int do_hess = arma_use_hessian(opt);
+    int kopt, i, k = ainfo->nc;
     int err;
 
     pmod->t1 = ainfo->t1;
     pmod->t2 = ainfo->t2;
-    pmod->nobs = T;
-    pmod->ncoeff = k;
-    pmod->full_n = ainfo->T;
+    pmod->nobs = ainfo->T;
+    pmod->ncoeff = ainfo->nc;
+    pmod->full_n = pdinfo->n;
     
     /* in the Kalman case the basic model struct is empty, so we 
        have to allocate for coefficients, residuals and so on
@@ -1144,7 +1157,7 @@ static int kalman_arma_finish (MODEL *pmod, const int *alist,
 
     /* rescale if we're using average loglikelihood */
     if (kopt & KALMAN_AVG_LL) {
-	pmod->lnL *= T;
+	pmod->lnL *= ainfo->T;
     }
 
 #if ARMA_DEBUG
@@ -1154,7 +1167,7 @@ static int kalman_arma_finish (MODEL *pmod, const int *alist,
 
     if (!do_hess) {
 	/* try OPG first, hessian as fallback */
-	err = arma_OPG_vcv(pmod, K, b, s2, k, T, prn);
+	err = arma_OPG_vcv(pmod, K, b, s2, k, ainfo->T, prn);
 	if (err) {
 	    err = 0;
 	    do_hess = 1;
@@ -1171,7 +1184,7 @@ static int kalman_arma_finish (MODEL *pmod, const int *alist,
 	vcv = numerical_hessian(b, ainfo->nc, kalman_arma_ll, K, &err);
 	kalman_do_ma_check = 1;
 	if (!err) {
-	    maybe_rescale_hessian(kopt, vcv, k, T);
+	    maybe_rescale_hessian(kopt, vcv, k, ainfo->T);
 	    arma_hessian_vcv(pmod, vcv, k);
 	    gretl_model_set_vcv_info(pmod, VCV_ML, VCV_HESSIAN);
 	}
@@ -1181,6 +1194,12 @@ static int kalman_arma_finish (MODEL *pmod, const int *alist,
 	write_arma_model_stats(pmod, alist, ainfo, Z, pdinfo);
 	arma_model_add_roots(pmod, ainfo, b);
 	gretl_model_set_int(pmod, "arma_flags", ARMA_EXACT);
+	if (ainfo->flags & ARMA_LBFGS) {
+	    pmod->opt |= OPT_L;
+	}
+	if (arima_ydiff_only(ainfo)) {
+	    pmod->opt |= OPT_Y;
+	}
     }
 
     return err;
@@ -1188,39 +1207,25 @@ static int kalman_arma_finish (MODEL *pmod, const int *alist,
 
 static gretl_matrix *form_arma_y_vector (const int *alist, 
 					 const double **Z,
-					 struct arma_info *ainfo,
+					 arma_info *ainfo,
 					 int *err)
 {
     gretl_matrix *yvec;
     const double *y;
-    int s, t, T;
+    int s, t;
 
 #if ARMA_DEBUG
     fprintf(stderr, "ainfo->t1 = %d, ainfo->t2 = %d\n",
 	    ainfo->t1, ainfo->t2);
 #endif
 
-    if (ainfo->dy != NULL) {
-	y = ainfo->dy;
-	/* should be handled earlier? */
-	for (t=ainfo->t1; t<=ainfo->t2; t++) {
-	    if (na(y[t])) {
-		ainfo->t1 += 1;
-	    } else {
-		break;
-	    }
-	}
+    if (ainfo->y != NULL) {
+	y = ainfo->y;
     } else {
 	y = Z[ainfo->yno];
     }
 
-    T = ainfo->t2 - ainfo->t1 + 1;
-    if (T == 0) {
-	*err = E_DATA;
-	return NULL;
-    }
-
-    yvec = gretl_column_vector_alloc(T);
+    yvec = gretl_column_vector_alloc(ainfo->T);
     if (yvec == NULL) {
 	*err = E_ALLOC;
 	return NULL;
@@ -1228,9 +1233,6 @@ static gretl_matrix *form_arma_y_vector (const int *alist,
 
     s = 0;
     for (t=ainfo->t1; t<=ainfo->t2; t++) {
-	if (na(y[t])) {
-	    *err = E_DATA;
-	}
 	gretl_vector_set(yvec, s++, y[t]);
     }
 
@@ -1247,45 +1249,27 @@ static gretl_matrix *form_arma_y_vector (const int *alist,
     return yvec;
 }
 
-static gretl_matrix *form_arma_x_matrix (const int *alist, 
+static gretl_matrix *form_arma_X_matrix (const int *alist, 
 					 const double **Z,
-					 struct arma_info *ainfo)
+					 arma_info *ainfo,
+					 int *err)
 {
-    gretl_matrix *x;
-    int i, xstart;
-    int *xlist;
-    int err = 0;
-
-    xlist = gretl_list_new(ainfo->nexo);
-    if (xlist == NULL) {
-	return NULL;
-    }
-
-    xstart = arma_list_y_position(ainfo) + 1;
-    for (i=xstart; i<=alist[0]; i++) {
-	xlist[i - xstart + 1] = alist[i];
-    }
+    gretl_matrix *X;
 
 #if ARMA_DEBUG
     printlist(alist, "alist (arma list)");
-    printlist(xlist, "xlist (exog vars)");
+    printlist(ainfo->xlist, "xlist (exog vars)");
 #endif
 
-    x = gretl_matrix_data_subset(xlist, Z, ainfo->t1, ainfo->t2, 
-				 M_MISSING_ERROR, &err);
-    if (err) {
-	free(xlist);
-	return NULL;
-    }
+    X = gretl_matrix_data_subset(ainfo->xlist, Z, ainfo->t1, ainfo->t2, 
+				 M_MISSING_ERROR, err);
 
 #if ARMA_DEBUG
-    gretl_matrix_print(x, "x");
-    fprintf(stderr, "x has %d rows\n", gretl_matrix_rows(x));
+    gretl_matrix_print(X, "X");
+    fprintf(stderr, "X has %d rows\n", gretl_matrix_rows(X));
 #endif
 
-    free(xlist);
-    
-    return x;
+    return X;
 }
 
 /* Given an estimate of the ARMA constant via OLS, convert to the form
@@ -1293,7 +1277,7 @@ static gretl_matrix *form_arma_x_matrix (const int *alist,
    goes: const, phi, Phi, theta, Theta, beta.
 */
 
-static void transform_arma_const (double *b, struct arma_info *ainfo)
+static void transform_arma_const (double *b, arma_info *ainfo)
 {
     const double *phi = b + 1;
     const double *Phi = phi + ainfo->np;
@@ -1318,7 +1302,7 @@ static void transform_arma_const (double *b, struct arma_info *ainfo)
     b[0] /= (narfac * sarfac);
 }
 
-static int kalman_undo_y_scaling (struct arma_info *ainfo,
+static int kalman_undo_y_scaling (arma_info *ainfo,
 				  gretl_matrix *y, double *b, 
 				  kalman *K)
 {
@@ -1327,16 +1311,16 @@ static int kalman_undo_y_scaling (struct arma_info *ainfo,
     int i, t, T = ainfo->t2 - ainfo->t1 + 1;
     int err = 0;
 
-    b[0] *= ainfo->dyscale;
+    b[0] *= ainfo->yscale;
 
     for (i=0; i<ainfo->nexo; i++) {
-	beta[i] *= ainfo->dyscale;
+	beta[i] *= ainfo->yscale;
     }
 
     i = ainfo->t1;
     for (t=0; t<T; t++) {
-	y->val[t] *= ainfo->dyscale;
-	ainfo->dy[i++] *= ainfo->dyscale;
+	y->val[t] *= ainfo->yscale;
+	ainfo->y[i++] *= ainfo->yscale;
     }
 
     if (na(kalman_arma_ll(b, K))) {
@@ -1346,26 +1330,29 @@ static int kalman_undo_y_scaling (struct arma_info *ainfo,
     return err;
 }
 
+static void free_arma_X_matrix (arma_info *ainfo, gretl_matrix *X)
+{
+    if (X == ainfo->dX) {
+	gretl_matrix_free(ainfo->dX);
+	ainfo->dX = NULL;
+    } else {
+	gretl_matrix_free(X);
+    }
+}
+
 static int kalman_arma (const int *alist, double *coeff, 
 			const double **Z, const DATAINFO *pdinfo,
-			struct arma_info *ainfo, MODEL *pmod,
+			arma_info *ainfo, MODEL *pmod,
 			gretlopt opt, PRN *prn)
 {
     kalman *K = NULL;
     gretl_matrix *y = NULL;
-    gretl_matrix *x = NULL;
-
+    gretl_matrix *X = NULL;
     int k = 1 + ainfo->nexo; /* number of exog vars plus space for const */
     int r, r2, m;
-
-    /* BFGS apparatus */
-    int maxit = 1000;
-    double reltol = 1.0e-12;
-    int fncount = 0;
-    int grcount = 0;
-
+    int fncount = 0, grcount = 0;
     double *b;
-    int i, T, err = 0;
+    int i, err = 0;
 
     b = malloc(ainfo->nc * sizeof *b);
     if (b == NULL) {
@@ -1384,31 +1371,35 @@ static int kalman_arma (const int *alist, double *coeff,
 #endif
 
     y = form_arma_y_vector(alist, Z, ainfo, &err);
-    if (y == NULL) {
+    if (err) {
 	free(b);
 	return err;
     }
 
     if (ainfo->nexo > 0) {
-	x = form_arma_x_matrix(alist, Z, ainfo);
-	if (x == NULL) {
-	    free(b);
-	    gretl_matrix_free(y);
-	    return E_ALLOC;
+	if (ainfo->dX != NULL) {
+	    X = ainfo->dX;
+	} else {
+	    X = form_arma_X_matrix(alist, Z, ainfo, &err);
+	    if (err) {
+		free(b);
+		gretl_matrix_free(y);
+		return err;
+	    }
 	}
     }
 
-    if (allocate_ac_mc(ainfo)) {
+    err = allocate_ac_mc(ainfo);
+    if (err) {
 	free(b);
 	gretl_matrix_free(y);
-	gretl_matrix_free(x);
-	return E_ALLOC;
+	free_arma_X_matrix(ainfo, X);
+	return err;
     }
 
     r = ainfo_get_r(ainfo);
     r2 = r * r;
     m = r * (r + 1) / 2;
-    T = gretl_matrix_rows(y);
 
     /* when should we use vech apparatus? */
     if (r > 4) {
@@ -1417,7 +1408,7 @@ static int kalman_arma (const int *alist, double *coeff,
 
     clear_gretl_matrix_err();
 
-    allocate_kalman_matrices(r, r2, k, T);
+    allocate_kalman_matrices(r, r2, k, ainfo->T);
 
     if (arma_using_vech(ainfo)) {
 	vQ = gretl_column_vector_alloc(m);
@@ -1430,7 +1421,7 @@ static int kalman_arma (const int *alist, double *coeff,
     if (err) {
 	free(b);
 	gretl_matrix_free(y);
-	gretl_matrix_free(x);
+	free_arma_X_matrix(ainfo, X);
 	return E_ALLOC;
     }
 
@@ -1442,38 +1433,56 @@ static int kalman_arma (const int *alist, double *coeff,
 	    ainfo->p, ainfo->P, ainfo->q, ainfo->Q, ainfo->ifc, 
 	    ainfo->nexo, ainfo->t1, ainfo->t2);
     fprintf(stderr, "Kalman dims: r = %d, k = %d, T = %d, ncoeff=%d\n", 
-	    r, k, T, ainfo->nc);
+	    r, k, ainfo->T, ainfo->nc);
 #endif
 
     /* publish ainfo */
     kainfo = ainfo;
 
-    K = kalman_new(S, P, F, A, H, Q, NULL, y, x, E, &err);
+    K = kalman_new(S, P, F, A, H, Q, NULL, y, X, E, &err);
 
     if (err) {
 	fprintf(stderr, "kalman_new(): err = %d\n", err);
     } else {
+	int maxit, save_lbfgs = 0;
+	double toler;
+
 	if (r > 3) {
 	    kalman_set_nonshift(K, 1);
 	} else {
 	    kalman_set_nonshift(K, r);
 	}
 
-	if (T > 3072 || getenv("KALMAN_AVG_LL") != NULL) {
+	if (ainfo->T > 3072 || getenv("KALMAN_AVG_LL") != NULL) {
 	    kalman_set_options(K, KALMAN_ARMA_LL | KALMAN_AVG_LL);
 	} else {
 	    kalman_set_options(K, KALMAN_ARMA_LL);
 	}
 
-	err = BFGS_max(b, ainfo->nc, maxit, reltol, 
+	save_lbfgs = libset_get_bool(USE_LBFGS);
+
+	if (save_lbfgs) {
+	    ainfo->flags |= ARMA_LBFGS;
+	} else if (opt & OPT_L) {
+	    libset_set_bool(USE_LBFGS, 1);
+	    ainfo->flags |= ARMA_LBFGS;
+	}
+
+	BFGS_defaults(&maxit, &toler, ARMA);
+
+	err = BFGS_max(b, ainfo->nc, maxit, toler, 
 		       &fncount, &grcount, kalman_arma_ll, C_LOGLIK,
 		       NULL, K, opt, prn);
 	if (err) {
 	    fprintf(stderr, "BFGS_max returned %d\n", err);
 	} 
+
+	if (!save_lbfgs && (opt & OPT_L)) {
+	    libset_set_bool(USE_LBFGS, 0);
+	}
     }
 
-    if (!err && ainfo->dyscale != 1.0) {
+    if (!err && ainfo->yscale != 1.0) {
 	kalman_undo_y_scaling(ainfo, y, b, K);
     }
 
@@ -1482,8 +1491,7 @@ static int kalman_arma (const int *alist, double *coeff,
 	gretl_model_set_int(pmod, "grcount", grcount);
 	err = kalman_arma_finish(pmod, alist, ainfo, 
 				 Z, pdinfo, K, b, 
-				 ainfo->nc, T, opt,
-				 prn);
+				 opt, prn);
     } 
 
     if (err) {
@@ -1495,7 +1503,7 @@ static int kalman_arma (const int *alist, double *coeff,
     free_kalman_matrices();
 
     gretl_matrix_free(y);
-    gretl_matrix_free(x);
+    free_arma_X_matrix(ainfo, X);
 
     gretl_matrix_free(Svar2);
     gretl_matrix_free(vQ);
@@ -1518,7 +1526,7 @@ static int kalman_arma (const int *alist, double *coeff,
 */
 
 static const double **
-make_armax_X (const int *list, struct arma_info *ainfo, const double **Z)
+make_armax_X (const int *list, arma_info *ainfo, const double **Z)
 {
     const double **X;
     int ypos, nx;
@@ -1538,8 +1546,8 @@ make_armax_X (const int *list, struct arma_info *ainfo, const double **Z)
     }
 
     /* the dependent variable */
-    if (ainfo->dy != NULL) {
-	X[0] = ainfo->dy;
+    if (ainfo->y != NULL) {
+	X[0] = ainfo->y;
     } else {
 	X[0] = Z[list[ypos]];
     }
@@ -1567,7 +1575,7 @@ static int add_to_spec (char *targ, const char *src)
    that takes the form (y_{t-i} - X_{t-i} \beta)
 */
 
-static int y_Xb_at_lag (char *spec, struct arma_info *ainfo, 
+static int y_Xb_at_lag (char *spec, arma_info *ainfo, 
 			int narmax, int lag)
 {
     char chunk[32];
@@ -1641,7 +1649,7 @@ static void nls_kickstart (MODEL *pmod, double ***pZ,
     clear_model(pmod);
 }
 
-static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
+static int arma_get_nls_model (MODEL *amod, arma_info *ainfo,
 			       int narmax, const double *coeff,
 			       double ***pZ, DATAINFO *pdinfo,
 			       PRN *prn) 
@@ -1834,7 +1842,7 @@ static int arma_get_nls_model (MODEL *amod, struct arma_info *ainfo,
    ARMA via OLS (not NLS)
 */
 
-static int *make_ar_ols_list (struct arma_info *ainfo, int av)
+static int *make_ar_ols_list (arma_info *ainfo, int av)
 {
     int * alist = gretl_list_new(av);
     int i, k, vi;
@@ -1875,7 +1883,7 @@ static int *make_ar_ols_list (struct arma_info *ainfo, int av)
 
 /* compose variable names for temporary dataset */
 
-static void arma_init_add_varnames (struct arma_info *ainfo, 
+static void arma_init_add_varnames (arma_info *ainfo, 
 				    int ptotal, int narmax, 
 				    DATAINFO *adinfo)
 {
@@ -1924,13 +1932,24 @@ static void arma_init_add_varnames (struct arma_info *ainfo,
     }
 }
 
+static int use_preprocessed_y (arma_info *ainfo)
+{
+    if (ainfo->flags & ARMA_XDIFF) {
+	/* for initialization, use the level of y */
+	return 0;
+    } else {
+	/* use preprocessed y if available */
+	return (ainfo->y != NULL);
+    }
+}
+
 /* Build temporary dataset including lagged vars: if we're doing exact
    ML on an ARMAX model we need lags of the exogenous variables as
    well as lags of y_t.  Note that the auxiliary dataset has "t = 0"
    at an offset of ainfo->t1 into the "real", external dataset.
 */
 
-static void arma_init_build_dataset (struct arma_info *ainfo, 
+static void arma_init_build_dataset (arma_info *ainfo, 
 				     int ptotal, int narmax, 
 				     const int *list,
 				     const double **Z,
@@ -1943,8 +1962,8 @@ static void arma_init_build_dataset (struct arma_info *ainfo,
     int lag, xstart;
 
     /* dependent variable */
-    if (ainfo->dy != NULL) {
-	y = ainfo->dy;
+    if (use_preprocessed_y(ainfo)) {
+	y = ainfo->y;
     } else {
 	y = Z[ainfo->yno];
     }
@@ -2052,7 +2071,7 @@ static void arma_init_build_dataset (struct arma_info *ainfo,
    into the array that will be passed to the maximizer.
 */
 
-static void arma_init_transcribe_coeffs (struct arma_info *ainfo,
+static void arma_init_transcribe_coeffs (arma_info *ainfo,
 					 MODEL *pmod, double *b)
 {
     int q0 = ainfo->ifc + ainfo->np + ainfo->P;
@@ -2071,6 +2090,11 @@ static void arma_init_transcribe_coeffs (struct arma_info *ainfo,
 	b[j++] = pmod->coeff[i];
     }
 
+    if ((ainfo->flags & ARMA_XDIFF) && ainfo->ifc) {
+	/* is this a good idea? */
+	b[0] /= ainfo->T;
+    }
+
     /* insert near-zeros for nonseasonal MA */
     for (i=0; i<ainfo->nq; i++) {
 	b[q0 + i] = 0.0001;
@@ -2082,19 +2106,42 @@ static void arma_init_transcribe_coeffs (struct arma_info *ainfo,
     }	
 }
 
-static void maybe_rescale_dy (struct arma_info *ainfo)
+/* try to avoid numerical problems when doing exact ML: 
+   scale the dependent variable if it's "too big" 
+*/
+
+static void maybe_rescale_y (arma_info *ainfo, const double **Z,
+			     const DATAINFO *pdinfo)
 {
-    double ybar = gretl_mean(0, ainfo->T - 1, ainfo->dy);
+    double ybar;
+    int t, doit = 0;
 
-    if (fabs(ybar) > 250) {
-	int t;
+    if (ainfo->y != NULL) {
+	ybar = gretl_mean(ainfo->t1, ainfo->t2, ainfo->y);
+	doit = (fabs(ybar) > 250);
+    } else {
+	const double *y = Z[ainfo->yno];
 
-	for (t=0; t<ainfo->T; t++) {
-	    if (!na(ainfo->dy[t])) {
-		ainfo->dy[t] /= ybar;
+	ybar = gretl_mean(ainfo->t1, ainfo->t2, y);
+	if (fabs(ybar) > 250) {
+	    ainfo->y = malloc(pdinfo->n * sizeof *ainfo->y);
+	    if (ainfo->y != NULL) {
+		for (t=0; t<pdinfo->n; t++) {
+		    ainfo->y[t] = y[t];
+		}
+		doit = 1;
 	    }
 	}
-	ainfo->dyscale = ybar;
+    }
+
+    if (doit) {
+	fprintf(stderr, "arma: ybar = %g, rescaling y\n", ybar);
+	for (t=0; t<=ainfo->t2; t++) {
+	    if (!na(ainfo->y[t])) {
+		ainfo->y[t] /= ybar;
+	    }
+	}
+	ainfo->yscale = ybar;
     }
 }
 
@@ -2111,10 +2158,9 @@ static void maybe_rescale_dy (struct arma_info *ainfo)
 
 static int ar_arma_init (const int *list, double *coeff, 
 			 const double **Z, const DATAINFO *pdinfo,
-			 struct arma_info *ainfo, 
+			 arma_info *ainfo, 
 			 MODEL *pmod, PRN *prn)
 {
-    int an = pdinfo->t2 - ainfo->t1 + 1;
     int nmixed = ainfo->np * ainfo->P;
     int ptotal = ainfo->np + ainfo->P + nmixed;
     int av = ptotal + ainfo->nexo + 2;
@@ -2148,11 +2194,11 @@ static int ar_arma_init (const int *list, double *coeff,
 	av += ainfo->nexo * ptotal;
     } 
 
-    if (arma_exact_ml(ainfo) && ainfo->ifc && ainfo->dy != NULL) {
-	maybe_rescale_dy(ainfo);
+    if (arma_exact_ml(ainfo) && ainfo->ifc) {
+	maybe_rescale_y(ainfo, Z, pdinfo);
     }
 
-    adinfo = create_auxiliary_dataset(&aZ, av, an);
+    adinfo = create_auxiliary_dataset(&aZ, av, ainfo->T);
     if (adinfo == NULL) {
 	return E_ALLOC;
     }
@@ -2236,10 +2282,9 @@ static int ar_arma_init (const int *list, double *coeff,
 
 static int arma_by_ls (const int *list, double *coeff, 
 		       const double **Z, const DATAINFO *pdinfo,
-		       struct arma_info *ainfo, 
+		       arma_info *ainfo, 
 		       MODEL *pmod, PRN *prn)
 {
-    int an = pdinfo->t2 - ainfo->t1 + 1;
     int nmixed = ainfo->np * ainfo->P;
     int ptotal = ainfo->np + ainfo->P + nmixed;
     int av = ptotal + ainfo->nexo + 2;
@@ -2248,7 +2293,7 @@ static int arma_by_ls (const int *list, double *coeff,
     int *alist = NULL;
     int nonlin = 0;
 
-    adinfo = create_auxiliary_dataset(&aZ, av, an);
+    adinfo = create_auxiliary_dataset(&aZ, av, ainfo->T);
     if (adinfo == NULL) {
 	return E_ALLOC;
     }
@@ -2321,9 +2366,9 @@ static int arma_by_ls (const int *list, double *coeff,
 
 #define MINLAGS 16
 
-static int hr_init_check (const DATAINFO *pdinfo, struct arma_info *ainfo)
+static int hr_init_check (const DATAINFO *pdinfo, arma_info *ainfo)
 {
-    int nobs = pdinfo->t2 - ainfo->t1 + 1; /* ?? */
+    int nobs = ainfo->T;
     int nlags = (ainfo->P + ainfo->Q) * pdinfo->pd;
     int ncoeff, df;
     int err = 0;
@@ -2348,7 +2393,7 @@ static int hr_init_check (const DATAINFO *pdinfo, struct arma_info *ainfo)
     return err;
 }
 
-static int hr_transcribe_coeffs (struct arma_info *ainfo,
+static int hr_transcribe_coeffs (arma_info *ainfo,
 				 MODEL *pmod, double *b)
 {
     const double *theta = NULL;
@@ -2359,6 +2404,9 @@ static int hr_transcribe_coeffs (struct arma_info *ainfo,
 
     if (ainfo->ifc) {
 	b[0] = pmod->coeff[0];
+	if (ainfo->flags & ARMA_XDIFF) {
+	    b[0] /= ainfo->T;
+	}
 	k = 1;
     } 
 
@@ -2412,16 +2460,14 @@ static int hr_transcribe_coeffs (struct arma_info *ainfo,
 
 static int hr_arma_init (const int *list, double *coeff, 
 			 const double **Z, const DATAINFO *pdinfo,
-			 struct arma_info *ainfo, PRN *prn)
+			 arma_info *ainfo, PRN *prn)
 {
-    int an = pdinfo->t2 - ainfo->t1 + 1;
     int np = ainfo->p, nq = ainfo->q;
     int nP = ainfo->P, nQ = ainfo->Q;
     int ptotal = np + nP + np * nP;
     int qtotal = nq + nQ + nq * nQ;
     int nexo = ainfo->nexo;
     int pass1lags, pass1v;
-
     const double *y;
     double **aZ = NULL;
     DATAINFO *adinfo = NULL;
@@ -2442,13 +2488,13 @@ static int hr_arma_init (const int *list, double *coeff,
     pass1v = pass1lags + nexo + 2;
 
     /* dependent variable */
-    if (ainfo->dy != NULL) {
-	y = ainfo->dy;
+    if (use_preprocessed_y(ainfo)) {
+	y = ainfo->y;
     } else {
 	y = Z[ainfo->yno];
     }
 
-    adinfo = create_auxiliary_dataset(&aZ, pass1v + qtotal, an);
+    adinfo = create_auxiliary_dataset(&aZ, pass1v + qtotal, ainfo->T);
     if (adinfo == NULL) {
 	return E_ALLOC;
     }
@@ -2496,7 +2542,7 @@ static int hr_arma_init (const int *list, double *coeff,
 	xstart = (arma_has_seasonal(ainfo))? 8 : 5;
     }
 
-    for (t=0; t<an; t++) {
+    for (t=0; t<ainfo->T; t++) {
 	s = t + ainfo->t1;
 	aZ[1][t] = y[s];
 	for (i=0, pos=2; i<nexo; i++) {
@@ -2572,7 +2618,7 @@ static int hr_arma_init (const int *list, double *coeff,
     pos = pass1v;
     for (i=0; i<qtotal; i++) {
 	sprintf(adinfo->varname[pos], "e_%d", malags[i]);
-	for (t=0; t<an; t++) {
+	for (t=0; t<ainfo->T; t++) {
 	    s = t - malags[i];
 	    aZ[pos][t] = (s >= 0)? armod.uhat[s] : NADBL;
 	}
@@ -2642,8 +2688,8 @@ static int hr_arma_init (const int *list, double *coeff,
     return err;
 }
 
-static int user_arma_init (double *coeff, struct arma_info *ainfo, 
-			   char flags, int *init_done, PRN *prn)
+static int user_arma_init (double *coeff, arma_info *ainfo, 
+			   int *init_done, PRN *prn)
 {
     int i, nc = n_init_vals();
 
@@ -2655,7 +2701,7 @@ static int user_arma_init (double *coeff, struct arma_info *ainfo,
 	return E_DATA;
     }
 
-    if (flags & ARMA_EXACT) {
+    if (ainfo->flags & ARMA_EXACT) {
 	/* initialization is handled within BFGS */
 	for (i=0; i<ainfo->nc; i++) {
 	    coeff[i] = 0.0;
@@ -2677,17 +2723,13 @@ static int user_arma_init (double *coeff, struct arma_info *ainfo,
 
 /* set up a model_info struct for passing to bhhh_max */
 
-static model_info *
-set_up_arma_model_info (struct arma_info *ainfo)
+static model_info *set_up_arma_model_info (arma_info *ainfo, 
+					   const DATAINFO *pdinfo)
 {
     double tol = libset_get_double(BHHH_TOLER);
     model_info *arma;
 
-    if (na(tol)) {
-	tol = 1.0e-6;
-    }
-
-    arma = model_info_new(ainfo->nc, ainfo->t1, ainfo->t2, ainfo->T, tol);
+    arma = model_info_new(ainfo->nc, ainfo->t1, ainfo->t2, pdinfo->n, tol);
 
     if (arma == NULL) return NULL;
 
@@ -2725,7 +2767,7 @@ conditional_arma_model_prep (MODEL *pmod, model_info *minfo,
 
 static int bhhh_arma (const int *alist, double *coeff, 
 		      const double **Z, const DATAINFO *pdinfo,
-		      struct arma_info *ainfo, MODEL *pmod,
+		      arma_info *ainfo, MODEL *pmod,
 		      gretlopt opt, PRN *prn)
 {
     model_info *minfo = NULL;
@@ -2740,7 +2782,7 @@ static int bhhh_arma (const int *alist, double *coeff,
     }
 
     /* create model_info struct to feed to bhhh_max() */
-    minfo = set_up_arma_model_info(ainfo);
+    minfo = set_up_arma_model_info(ainfo, pdinfo);
     if (minfo == NULL) {
 	pmod->errcode = E_ALLOC;
 	free(X);
@@ -2773,37 +2815,31 @@ static int bhhh_arma (const int *alist, double *coeff,
 /* Should we try Hannan-Rissanen initialization of ARMA
    coefficients? */
 
-static int prefer_hr_init (struct arma_info *ainfo)
+static int prefer_hr_init (arma_info *ainfo)
 {
     int ret = 0;
 
     if (ainfo->q > 1 || ainfo->Q > 0) {
 	ret = 1;
-
-	/* don't use for gappy arma (yet?) */
 	if (ainfo->pqspec != NULL && *ainfo->pqspec != '\0') {
+	    /* don't use for gappy arma (yet?) */
 	    ret = 0;
-	}
-
-	/* unlikely to work well with small sample */
-	if (ainfo->t2 - ainfo->t1 < 100) {
+	} else if (ainfo->flags & ARMA_XDIFF) {
+	    /* don't use for ARIMAX (yet?) */
 	    ret = 0;
-	}
-
-	/* not sure about this: HR catches the MA terms, but NLS
-	   handles better the AR interactions
-	*/
-	if (ainfo->p > 0 && ainfo->P > 0) {
+	} else if (ainfo->t2 - ainfo->t1 < 100) {
+	    /* unlikely to work well with small sample */
 	    ret = 0;
-	}
-
-	/* overlapping orders screw things up */
-	if ((ainfo->P > 0 && ainfo->p >= ainfo->pd) ||
-	    (ainfo->Q > 0 && ainfo->q >= ainfo->pd)) {
+	} else if (ainfo->p > 0 && ainfo->P > 0) {
+	    /* not sure about this: HR catches the MA terms, but NLS
+	       handles the AR interactions better
+	    */
 	    ret = 0;
-	}
-
-	if (ret && arma_exact_ml(ainfo)) {
+	} else if ((ainfo->P > 0 && ainfo->p >= ainfo->pd) ||
+		   (ainfo->Q > 0 && ainfo->q >= ainfo->pd)) {
+	    /* overlapping orders screw things up */
+	    ret = 0;
+	} else if (ret && arma_exact_ml(ainfo)) {
 	    /* screen for cases where we'll use NLS */
 	    if (ainfo->P > 0) {
 		ret = 0;
@@ -2820,6 +2856,20 @@ static int prefer_hr_init (struct arma_info *ainfo)
     return ret;
 }
 
+/* Set flag to indicate differencing of exogenous regressors, in the
+   case of an ARIMAX model using native exact ML -- unless this is
+   forbidden by OPT_Y.
+*/
+
+static void maybe_set_xdiff_flag (arma_info *ainfo, gretlopt opt)
+{
+    if ((ainfo->flags & ARMA_EXACT) &&
+	(ainfo->d > 0 || ainfo->D > 0) &&
+	ainfo->nexo > 0 && !(opt & OPT_Y)) {
+	ainfo->flags |= ARMA_XDIFF;
+    }
+}
+
 MODEL arma_model (const int *list, const char *pqspec,
 		  const double **Z, const DATAINFO *pdinfo, 
 		  gretlopt opt, PRN *prn)
@@ -2827,110 +2877,116 @@ MODEL arma_model (const int *list, const char *pqspec,
     double *coeff = NULL;
     int *alist = NULL;
     MODEL armod;
-    struct arma_info ainfo;
+    arma_info ainfo;
     int init_done = 0;
-    char flags = 0;
     int err = 0;
 
-    if (!(opt & OPT_C)) {
-	flags = ARMA_EXACT;
-    }
-
     vprn = set_up_verbose_printer(opt, prn);
 
-    arma_info_init(&ainfo, flags, pqspec, pdinfo);
+    arma_info_init(&ainfo, opt, pqspec, pdinfo);
     gretl_model_init(&armod); 
 
-    alist = gretl_list_copy(list);
-    if (alist == NULL) {
-	err = E_ALLOC;
+    err = incompatible_options(opt, OPT_C | OPT_L);
+
+    if (!err) {
+	alist = gretl_list_copy(list);
+	if (alist == NULL) {
+	    err = E_ALLOC;
+	}
     } 
 
     if (!err) {
 	err = arma_check_list(alist, opt, Z, pdinfo, &ainfo);
     }
 
+    if (!err) {
+	/* calculate maximum lag */
+	maybe_set_xdiff_flag(&ainfo, opt);
+	calc_max_lag(&ainfo);
+    }
+
+    if (!err) {
+	/* adjust sample range if need be */
+	err = arma_adjust_sample(pdinfo, Z, alist, &ainfo);
+    }
+
+    if (!err) {
+	/* allocate initial coefficient vector */
+	coeff = malloc(ainfo.nc * sizeof *coeff);
+	if (coeff == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (!err && (ainfo.d > 0 || ainfo.D > 0)) {
+	err = arima_difference(&ainfo, Z, pdinfo);
+    }
+
     if (err) {
-	armod.errcode = err;
 	goto bailout;
-    } 
-
-    /* calculate maximum lag */
-    calc_max_lag(&ainfo);
-
-    /* adjust sample range if need be */
-    if (arma_adjust_sample(pdinfo, Z, alist, &ainfo)) {
-        armod.errcode = E_DATA;
-	goto bailout;
-    }
-
-    /* allocate initial coefficient vector */
-    coeff = malloc(ainfo.nc * sizeof *coeff);
-    if (coeff == NULL) {
-	armod.errcode = E_ALLOC;
-	goto bailout;
-    }
-
-    /* create differenced series if needed */
-    if (ainfo.d > 0 || ainfo.D > 0) {
-	err = arima_difference(Z[ainfo.yno], &ainfo);
     }
 
     /* initialize the coefficients: there are 3 possible methods */
 
     /* first pass: see if the user specified some values */
-    err = user_arma_init(coeff, &ainfo, flags, &init_done, vprn);
+    err = user_arma_init(coeff, &ainfo, &init_done, vprn);
     if (err) {
-	armod.errcode = err;
 	goto bailout;
     }
 
-    if (!(flags & ARMA_EXACT) && ainfo.q == 0 && ainfo.Q == 0) {
+    if (!(ainfo.flags & ARMA_EXACT) && ainfo.q == 0 && ainfo.Q == 0) {
 	/* pure AR model can be estimated via least squares */
 	ainfo.flags |= ARMA_LS;
-	arma_by_ls(alist, (init_done)? coeff : NULL, Z, pdinfo, 
-		   &ainfo, &armod, vprn);
+	err = arma_by_ls(alist, (init_done)? coeff : NULL, Z, pdinfo, 
+			 &ainfo, &armod, vprn);
 	goto bailout;
     }
 
     /* second pass: try Hannan-Rissanen, if suitable */
     if (!init_done && prefer_hr_init(&ainfo)) {
-	err = hr_init_check(pdinfo, &ainfo);
-	if (!err) {
-	    err = hr_arma_init(alist, coeff, Z, pdinfo, &ainfo, vprn);
+	int h_err = hr_init_check(pdinfo, &ainfo);
+
+	if (!h_err) {
+	    h_err = hr_arma_init(alist, coeff, Z, pdinfo, &ainfo, vprn);
 #if AINIT_DEBUG
-	    if (err) {
+	    if (h_err) {
 		fputs("*** hr_arma_init failed, will try ar_arma_init\n", stderr);
 	    } else {
 		fputs("*** hr_arma_init OK\n", stderr);
 	    }
 #endif
 	}
-	if (!err) {
+	if (!h_err) {
 	    init_done = 1;
 	}
     }
 
     /* third pass: estimate pure AR model by OLS or NLS */
-    if (!init_done) {
+    if (!err && !init_done) {
 	err = ar_arma_init(alist, coeff, Z, pdinfo, &ainfo, 
 			   &armod, vprn);
     }
 
-    if (err) {
-	armod.errcode = err;
-	goto bailout;
-    }
-
-    if (flags & ARMA_EXACT) {
-	kalman_arma(alist, coeff, Z, pdinfo, &ainfo, &armod, opt, vprn);
-    } else {
-	bhhh_arma(alist, coeff, Z, pdinfo, &ainfo, &armod, opt, vprn);
+    if (!err) {
+	clear_model_xpx(&armod);
+	if (ainfo.flags & ARMA_EXACT) {
+	    kalman_arma(alist, coeff, Z, pdinfo, &ainfo, &armod, opt, vprn);
+	} else {
+	    bhhh_arma(alist, coeff, Z, pdinfo, &ainfo, &armod, opt, vprn);
+	}
     }
 
  bailout:
 
-    if (!armod.errcode) {
+    if (err && !armod.errcode) {
+	armod.errcode = err;
+    }
+
+    if (armod.errcode) {
+	if (opt & OPT_U) {
+	    armod.opt |= OPT_U; /* continue on error */
+	}
+    } else {
 	gretl_model_smpl_init(&armod, pdinfo);
     }
 
