@@ -3,7 +3,9 @@
 #define MAX_ARMA_ORDER 128
 #define MAX_ARIMA_DIFF 2
 
-struct arma_info {
+typedef struct arma_info_ arma_info;
+
+struct arma_info_ {
     int yno;         /* ID of dependent variable */
     ArmaFlags flags; /* specification flags */
     int ifc;         /* specification includes a constant? */
@@ -23,9 +25,11 @@ struct arma_info {
     int t1;          /* starting observation */
     int t2;          /* ending observation */
     int pd;          /* periodicity of data */
-    int T;           /* full length of data series */
-    double *dy;      /* differenced dependent variable */
-    double dyscale;  /* scale factor for dy */
+    int T;           /* sample size for estimation */
+    double *y;       /* dependent variable (possibly differenced) */
+    double yscale;   /* scale factor for y */
+    int *xlist;      /* list of regressors (armax) */
+    gretl_matrix *dX;   /* differenced regressors (ARIMAX) */
     const char *pqspec; /* input string with specific AR, MA lags */
 };
 
@@ -46,11 +50,19 @@ struct arma_info {
 #define MA_included(a,i) (a->qmask == NULL || a->qmask[i] == '1')
 
 static void 
-arma_info_init (struct arma_info *ainfo, char flags, 
+arma_info_init (arma_info *ainfo, gretlopt opt, 
 		const char *pqspec, const DATAINFO *pdinfo)
 {
     ainfo->yno = 0;
-    ainfo->flags = flags;
+    ainfo->flags = 0;
+
+    if (opt & OPT_X) {
+	ainfo->flags |= ARMA_X12A;
+    }    
+
+    if (!(opt & OPT_C)) {
+	ainfo->flags |= ARMA_EXACT;
+    }
 
     ainfo->p = 0;
     ainfo->d = 0;
@@ -73,18 +85,24 @@ arma_info_init (struct arma_info *ainfo, char flags,
     ainfo->t1 = pdinfo->t1;
     ainfo->t2 = pdinfo->t2;
     ainfo->pd = pdinfo->pd;
-    ainfo->T = pdinfo->n;
+    ainfo->T = 0;
+
+    ainfo->y = NULL;
+    ainfo->yscale = 1.0;
+
+    ainfo->xlist = NULL;
+    ainfo->dX = NULL;
 
-    ainfo->dy = NULL;
-    ainfo->dyscale = 1.0;
     ainfo->pqspec = pqspec;
 }
 
-static void arma_info_cleanup (struct arma_info *ainfo)
+static void arma_info_cleanup (arma_info *ainfo)
 {
     free(ainfo->pmask);
     free(ainfo->qmask);
-    free(ainfo->dy);
+    free(ainfo->y);
+    free(ainfo->xlist);
+    gretl_matrix_free(ainfo->dX);
 }
 
 enum {
@@ -99,7 +117,7 @@ enum {
 */
 
 static char *mask_from_vec (const gretl_vector *v, 
-			    struct arma_info *ainfo,
+			    arma_info *ainfo,
 			    int m, int *err)
 {
     int vlen = gretl_vector_get_length(v);
@@ -181,7 +199,7 @@ matrix_from_spec (const char *s, int *tmp, int *err)
     return m;
 }
 
-static gretl_matrix *get_arma_pq_vec (struct arma_info *ainfo,
+static gretl_matrix *get_arma_pq_vec (arma_info *ainfo,
 				      int t, int *tmp, int *err)
 {
     gretl_matrix *m = NULL;
@@ -214,7 +232,7 @@ static gretl_matrix *get_arma_pq_vec (struct arma_info *ainfo,
 }
 
 static int 
-arma_make_masks (struct arma_info *ainfo, int *list)
+arma_make_masks (arma_info *ainfo, int *list)
 {
     gretl_matrix *m;
     int tmp, err = 0;
@@ -248,14 +266,14 @@ arma_make_masks (struct arma_info *ainfo, int *list)
     return err;
 }
 
-static int arma_list_y_position (struct arma_info *ainfo)
+static int arma_list_y_position (arma_info *ainfo)
 {
     int ypos;
 
     if (arma_is_arima(ainfo)) {
-	ypos = (arma_has_seasonal(ainfo))? 9 : 5;
+	ypos = arma_has_seasonal(ainfo) ? 9 : 5;
     } else {
-	ypos = (arma_has_seasonal(ainfo))? 7 : 4;
+	ypos = arma_has_seasonal(ainfo) ? 7 : 4;
     }
 
     return ypos;
@@ -333,7 +351,7 @@ static int arima_integrate (double *dx, const double *x,
     return 0;
 }
 
-static void ainfo_data_to_model (struct arma_info *ainfo, MODEL *pmod)
+static void ainfo_data_to_model (arma_info *ainfo, MODEL *pmod)
 {
     pmod->ifc = ainfo->ifc;
     pmod->dfn = ainfo->nc - pmod->ifc;
@@ -370,7 +388,7 @@ static void ainfo_data_to_model (struct arma_info *ainfo, MODEL *pmod)
    a gretl MODEL struct */
 
 static void write_arma_model_stats (MODEL *pmod, const int *list, 
-				    struct arma_info *ainfo,
+				    arma_info *ainfo,
 				    const double **Z, 
 				    const DATAINFO *pdinfo)
 {
@@ -387,7 +405,7 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
     pmod->list = gretl_list_copy(list);
 
     if (arma_is_arima(ainfo)) {
-	y = ainfo->dy;
+	y = ainfo->y;
     } else {
 	y = Z[ainfo->yno];
     }
@@ -442,18 +460,18 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
 				  ainfo->nexo);
 }
 
-static void calc_max_lag (struct arma_info *ainfo)
+static void calc_max_lag (arma_info *ainfo)
 {
-    int pmax = ainfo->p;
-    int dmax = ainfo->d;
+    if (ainfo->flags & ARMA_EXACT) {
+	ainfo->maxlag = ainfo->d + ainfo->D * ainfo->pd;
+    } else {
+	/* conditional ML */
+	int pmax = ainfo->p + ainfo->P * ainfo->pd;
+	int dmax = ainfo->d + ainfo->D * ainfo->pd;
 
-    if (arma_has_seasonal(ainfo)) {
-	pmax += ainfo->P * ainfo->pd;
-	dmax += ainfo->D * ainfo->pd;
+	ainfo->maxlag = pmax + dmax;
     }
 
-    ainfo->maxlag = pmax + dmax;
-
 #if ARMA_DEBUG
     fprintf(stderr, "calc_max_lag: ainfo->maxlag = %d\n", ainfo->maxlag);
 #endif
@@ -463,11 +481,11 @@ static void calc_max_lag (struct arma_info *ainfo)
 
 static int 
 arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
-		    struct arma_info *ainfo)
+		    arma_info *ainfo)
 {
-    int vstart = arma_list_y_position(ainfo);
-    int t1 = pdinfo->t1, t2 = pdinfo->t2;
-    int an, i, v, t, t1min;
+    int ypos = arma_list_y_position(ainfo);
+    int t0, t1 = pdinfo->t1, t2 = pdinfo->t2;
+    int i, vi, vlmax, k, t;
     int anymiss;
 
 #if SAMPLE_DEBUG
@@ -475,81 +493,75 @@ arma_adjust_sample (const DATAINFO *pdinfo, const double **Z, const int *list,
 	    t1, t2, ainfo->maxlag);
 #endif
 
-    /* determine the starting point for valid data, t1min */
-    t1min = 0;
-    for (t=0; t<=pdinfo->t2; t++) {
+    t0 = t1 - ainfo->maxlag;
+    if (t0 < 0) {
+	t1 -= t0;
+    }
+
+    /* list position of last var to check for lags */
+    if (ainfo->flags & ARMA_XDIFF) {
+	vlmax = list[0];
+    } else {
+	vlmax = ypos;
+    }
+
+    /* advance the starting point if need be */
+
+    for (t=t1; t<=t2; t++) {
 	anymiss = 0;
-	for (i=vstart; i<=list[0]; i++) {
-	    if (na(Z[list[i]][t])) {
+	for (i=ypos; i<=list[0] && !anymiss; i++) {
+	    vi = list[i];
+	    if (na(Z[vi][t])) {
+		/* current value missing */
 		anymiss = 1;
-		t1min++;
-		break;
+	    }
+	    if (i <= vlmax) {
+		for (k=1; k<=ainfo->maxlag && !anymiss; k++) {
+		    if (na(Z[vi][t-k])) {
+			/* lagged value missing */
+			anymiss = 1;
+		    }
+		}
 	    }
 	}
-	if (!anymiss) {
+	if (anymiss) {
+	    t1++;
+	} else {
 	    break;
 	}
     }
 
-#if SAMPLE_DEBUG
-    fprintf(stderr, " phase 1: t1min = %d\n", t1min);
-#endif
+    /* retard the ending point if need be */
 
-    /* if the notional starting point, t1, is before the start of
-       valid data, t1min, advance t1 */
-    if (t1 < t1min) {
-	t1 = t1min;
-    }
-    
-    if (!arma_exact_ml(ainfo)) {
-	/* conditional ML: ensure that the sample start allows for
-	   the required lags of y */
-	int t0;
-
-	if (t1 < ainfo->maxlag) {
-	    t1 = ainfo->maxlag;
-	}
-	t0 = t1 - ainfo->maxlag;
-	t1min = t1;
-	v = list[vstart];
-	for (t=t0; t<t1min; t++) {
-	    if (na(Z[v][t])) {
-		t1 = t + ainfo->maxlag + 1;
-	    }
-	}
-    }
-
-    /* trim any missing obs from the end of the specified sample
-       range 
-    */
-    for (t=pdinfo->t2; t>=t1; t--) {
+    for (t=t2; t>=t1; t--) {
 	anymiss = 0;
-	for (i=vstart; i<=list[0]; i++) {
-	    if (na(Z[list[i]][t])) {
+	for (i=ypos; i<=list[0] && !anymiss; i++) {
+	    vi = list[i];
+	    if (na(Z[vi][t])) {
 		anymiss = 1;
-		t2--;
-		break;
 	    }
 	}
-	if (!anymiss) {
+	if (anymiss) {
+	    t2--;
+	} else {
 	    break;
 	}
     }
-
+	
     /* check for missing obs within the sample range */
     for (t=t1; t<t2; t++) {
-	for (i=vstart; i<=list[0]; i++) {
-	    v = list[i];
-	    if (na(Z[v][t])) {
+	for (i=ypos; i<=list[0]; i++) {
+	    vi = list[i];
+	    if (na(Z[vi][t])) {
 		gretl_errmsg_sprintf(_("Missing value encountered for "
-				       "variable %d, obs %d"), v, t + 1);
-		return 1;
+				       "variable %d, obs %d"), vi, t + 1);
+		return E_MISSDATA;
 	    }
 	}
     }
 
-    an = t2 - t1 + 1;
-    if (an <= ainfo->nc) {
+    ainfo->T = t2 - t1 + 1;
+    if (ainfo->T <= ainfo->nc) {
 	/* insufficient observations */
 	return E_DF; 
     }
@@ -595,7 +607,7 @@ static int arma_remove_const (int *list, int seasonal, int diffs,
     return ret;
 }
 
-static int check_arma_sep (int *list, int sep1, struct arma_info *ainfo)
+static int check_arma_sep (int *list, int sep1, arma_info *ainfo)
 {
     int sep2 = (sep1 == 3)? 6 : 8;
     int i, err = 0;
@@ -635,22 +647,35 @@ static int check_arma_sep (int *list, int sep1, struct arma_info *ainfo)
     return err;
 }
 
+static int arma_add_xlist (arma_info *ainfo, const int *list,
+			   int ypos)
+{
+    ainfo->xlist = gretl_list_new(ainfo->nexo);
+
+    if (ainfo->xlist == NULL) {
+	return E_ALLOC;
+    } else {
+	int i;
+
+	for (i=1; i<=ainfo->nexo; i++) {
+	    ainfo->xlist[i] = list[ypos + i];
+	}
+
+	return 0;
+    }
+}
+
 #define count_arma_coeffs(a) (a->ifc + a->np + a->nq + a->P + a->Q + a->nexo)
 
 static int check_arma_list (int *list, gretlopt opt, 
 			    const double **Z, const DATAINFO *pdinfo,
-			    struct arma_info *ainfo)
+			    arma_info *ainfo)
 {
-    int armax = 0;
+    int ypos = arma_has_seasonal(ainfo) ? 7 : 4;
+    int armax = (list[0] > ypos);
     int hadconst = 0;
     int err = 0;
 
-    if (arma_has_seasonal(ainfo)) {
-	armax = (list[0] > 7);
-    } else {
-	armax = (list[0] > 4);
-    }
-
     if (list[1] < 0 || list[1] > MAX_ARMA_ORDER) {
 	err = 1;
     } else if (list[2] < 0 || list[2] > MAX_ARMA_ORDER) {
@@ -706,9 +731,12 @@ static int check_arma_list (int *list, gretlopt opt,
     if (err) {
 	gretl_errmsg_set(_("Error in arma command"));
     } else {
-	ainfo->nexo = list[0] - ((arma_has_seasonal(ainfo))? 7 : 4);
+	ainfo->nexo = list[0] - ypos;
 	ainfo->nc = count_arma_coeffs(ainfo);
-	ainfo->yno = (arma_has_seasonal(ainfo))? list[7] : list[4];
+	ainfo->yno = list[ypos];
+	if (ainfo->nexo > 0) {
+	    err = arma_add_xlist(ainfo, list, ypos);
+	}
     }
 
     return err;
@@ -716,9 +744,10 @@ static int check_arma_list (int *list, gretlopt opt,
 
 static int check_arima_list (int *list, gretlopt opt, 
 			     const double **Z, const DATAINFO *pdinfo,
-			     struct arma_info *ainfo)
+			     arma_info *ainfo)
 {
-    int armax = 0;
+    int ypos = arma_has_seasonal(ainfo) ? 9 : 5;
+    int armax = (list[0] > ypos);
     int hadconst = 0;
     int err = 0;
 
@@ -726,12 +755,6 @@ static int check_arima_list (int *list, gretlopt opt,
     printlist(list, "check_arima_list");
 #endif
 
-    if (arma_has_seasonal(ainfo)) {
-	armax = (list[0] > 9);
-    } else {
-	armax = (list[0] > 5);
-    }
-
     if (list[1] < 0 || list[1] > MAX_ARMA_ORDER) {
 	err = 1;
     } else if (list[2] < 0 || list[2] > MAX_ARIMA_DIFF) {
@@ -793,9 +816,12 @@ static int check_arima_list (int *list, gretlopt opt,
     if (err) {
 	gretl_errmsg_set(_("Error in arma command"));
     } else {
-	ainfo->nexo = list[0] - ((arma_has_seasonal(ainfo))? 9 : 5);
+	ainfo->nexo = list[0] - ypos;
 	ainfo->nc = count_arma_coeffs(ainfo);
-	ainfo->yno = (arma_has_seasonal(ainfo))? list[9] : list[5];
+	ainfo->yno = list[ypos];
+	if (ainfo->nexo > 0) {
+	    err = arma_add_xlist(ainfo, list, ypos);
+	}
     }
 
     return err;
@@ -803,7 +829,7 @@ static int check_arima_list (int *list, gretlopt opt,
 
 static int arma_check_list (int *list, gretlopt opt,
 			    const double **Z, const DATAINFO *pdinfo,
-			    struct arma_info *ainfo)
+			    arma_info *ainfo)
 {
     int sep1 = gretl_list_separator_position(list);
     int err = 0;
@@ -854,24 +880,68 @@ static int arma_check_list (int *list, gretlopt opt,
     return err;
 }
 
-static int
-arima_difference (const double *y, struct arma_info *ainfo)
+static void 
+real_arima_difference_series (double *dx, const double *x,
+			      int t1, arma_info *ainfo)
 {
-    double *dy;
+    int t, i, s = ainfo->pd;
+    
+    for (i=0, t=t1; t<=ainfo->t2; i++, t++) {
+	dx[i] = x[t];
+	if (ainfo->d > 0) {
+	    dx[i] -= x[t-1];
+	} 
+	if (ainfo->d == 2) {
+	    dx[i] -= x[t-1] - x[t-2];
+	}
+	if (ainfo->D > 0) {
+	    dx[i] -= x[t-s];
+	    if (ainfo->d > 0) {
+		dx[i] += x[t-s-1];
+	    }
+	    if (ainfo->d == 2) {
+		dx[i] += x[t-s-1] - x[t-s-2];
+	    }	    
+	} 
+	if (ainfo->D == 2) {
+	    dx[i] -= x[t-s] - x[t-2*s];
+	    if (ainfo->d > 0) {
+		dx[i] += x[t-s-1] - x[t-2*s-1];
+	    }
+	    if (ainfo->d == 2) {
+		dx[i] += x[t-s-1] - x[t-2*s-1];
+		dx[i] -= x[t-s-2] - x[t-2*s-2];
+	    }
+	}
+    }
+}
+
+static int arima_difference (arma_info *ainfo, const double **Z,
+			     const DATAINFO *pdinfo)
+{
+    const double *y = Z[ainfo->yno];
+    double *dy = NULL;
     int s = ainfo->pd;
     int t, t1 = 0;
+    int err = 0;
 
 #if ARMA_DEBUG
     fprintf(stderr, "doing arima_difference: d = %d, D = %d\n",
 	    ainfo->d, ainfo->D);
+    fprintf(stderr, "ainfo->t1 = %d, ainfo->t2 = %d\n", ainfo->t1,
+	    ainfo->t2);
 #endif
 
-    dy = malloc(ainfo->T * sizeof *dy);
+    dy = malloc(pdinfo->n * sizeof *dy);
     if (dy == NULL) {
 	return E_ALLOC;
     }
 
-    for (t=0; t<ainfo->T; t++) {
+    for (t=0; t<pdinfo->n; t++) {
+	dy[t] = NADBL;
+    }
+
+    for (t=0; t<pdinfo->n; t++) {
 	if (na(y[t])) {
 	    t1++;
 	} else {
@@ -881,47 +951,33 @@ arima_difference (const double *y, struct arma_info *ainfo)
 
     t1 += ainfo->d + ainfo->D * s;
 
-    for (t=0; t<t1; t++) {
-	dy[t] = NADBL;
-    }
-
-    for (t=t1; t<ainfo->T; t++) {
-	dy[t] = y[t];
-	if (ainfo->d > 0) {
-	    dy[t] -= y[t-1];
-	} 
-	if (ainfo->d == 2) {
-	    dy[t] -= y[t-1] - y[t-2];
-	}
-	if (ainfo->D > 0) {
-	    dy[t] -= y[t-s];
-	    if (ainfo->d > 0) {
-		dy[t] += y[t-s-1];
-	    }
-	    if (ainfo->d == 2) {
-		dy[t] += y[t-s-1] - y[t-s-2];
-	    }	    
-	} 
-	if (ainfo->D == 2) {
-	    dy[t] -= y[t-s] - y[t-2*s];
-	    if (ainfo->d > 0) {
-		dy[t] += y[t-s-1] - y[t-2*s-1];
-	    }
-	    if (ainfo->d == 2) {
-		dy[t] += y[t-s-1] - y[t-2*s-1];
-		dy[t] -= y[t-s-2] - y[t-2*s-2];
-	    }
-	}
-    }
+    real_arima_difference_series(dy + t1, y, t1, ainfo);
 
 #if ARMA_DEBUG > 1
-    for (t=0; t<ainfo->T; t++) {
-	fprintf(stderr, "dy[%d] = % 12.5f\n", t, dy[t]);
+    for (t=0; t<pdinfo->n; t++) {
+	fprintf(stderr, "dy[%d] = % 12.7g\n", t, dy[t]);
     }
 #endif    
 
-    ainfo->dy = dy;
+    ainfo->y = dy;
 
-    return 0;
+    if (ainfo->flags & ARMA_XDIFF) {
+	/* also difference the ARIMAX regressors */
+	ainfo->dX = gretl_matrix_alloc(ainfo->T, ainfo->nexo);
+	if (ainfo->dX == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    double *val = ainfo->dX->val;
+	    int i, vi;
+
+	    for (i=0; i<ainfo->nexo; i++) {
+		vi = ainfo->xlist[i+1];
+		real_arima_difference_series(val, Z[vi], ainfo->t1, ainfo);
+		val += ainfo->T;
+	    }
+	}
+    }
+
+    return err;
 }
 
