@@ -155,17 +155,23 @@ static int print_iterations (const char *path, PRN *prn)
     return err;
 }
 
-#define daily_frequency(f) (f == 5 || f == 6 || f == 7)
+#define non_yearly_frequency(f) (f != 1 && f != 4 && f != 12)
 
 static int x12_date_to_n (const char *s, const DATAINFO *pdinfo)
 {
     char date[12] = {0};
 
-    if (daily_frequency(pdinfo->pd)) {
-	int t, maj, min;
+    if (non_yearly_frequency(pdinfo->pd)) {
+	int t, maj = 0, min = 0, n = strlen(s);
+	char fmt[16];
+
+	sprintf(fmt, "%%%dd%%2d", n - 2);
+	if (sscanf(s, fmt, &maj, &min) == 2) {
+	    t = (maj - 1) * pdinfo->pd + min - 1;
+	} else {
+	    t = -1;
+	}
 
-	sscanf(s, "%1d%2d", &maj, &min);
-	t = (maj - 1) * pdinfo->pd + min - 1;
 	return t;
     }
 
@@ -234,7 +240,7 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 /* Parse the roots information from the X12ARIMA output file foo.rts */
 
 static int get_roots (const char *fname, MODEL *pmod, 
-		      struct arma_info *ainfo)
+		      arma_info *ainfo)
 {
     FILE *fp;
     char line[132];
@@ -352,7 +358,7 @@ static int get_x12a_vcv (const char *fname, MODEL *pmod, int nc)
 */
 
 static int 
-get_estimates (const char *fname, MODEL *pmod, struct arma_info *ainfo)
+get_estimates (const char *fname, MODEL *pmod, arma_info *ainfo)
 {
     double *ar_coeff = pmod->coeff + ainfo->ifc;
     double *ma_coeff = ar_coeff + ainfo->np + ainfo->P;
@@ -475,7 +481,7 @@ get_uhat (const char *fname, MODEL *pmod, const DATAINFO *pdinfo)
 static void 
 populate_arma_model (MODEL *pmod, const int *list, const char *path, 
 		     const double **Z, const DATAINFO *pdinfo, 
-		     struct arma_info *ainfo)
+		     arma_info *ainfo)
 {
     char fname[MAXLEN];
     int t, err = 0;
@@ -544,7 +550,7 @@ output_series_to_spc (const int *list, const double **Z,
 	    if (na(Z[list[i]][t])) {
 		fputs("-9999.0 ", fp);
 	    } else {
-		fprintf(fp, "%g ", Z[list[i]][t]);
+		fprintf(fp, "%.15g ", Z[list[i]][t]);
 	    }
 	}
 	fputc('\n', fp);
@@ -554,7 +560,7 @@ output_series_to_spc (const int *list, const double **Z,
 }
 
 static int *
-arma_info_get_x_list (struct arma_info *ainfo, const int *alist)
+arma_info_get_x_list (arma_info *ainfo, const int *alist)
 {
     int *xlist = NULL;
     int start = arma_list_y_position(ainfo);
@@ -578,10 +584,9 @@ make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
     int yr, subper = 0;
     char *s;
 
-    /* daily data: FIXME when dated? */
-    if (daily_frequency(pdinfo->pd)) {
+    if (non_yearly_frequency(pdinfo->pd)) {
 	int maj = t / pdinfo->pd + 1;
-	int min = (t + 1) % pdinfo->pd;
+	int min = t % pdinfo->pd + 1;
 
 	sprintf(str, "%d.%d", maj, min);
 	return;
@@ -606,7 +611,7 @@ make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
     }    
 }
 
-static void x12_pdq_string (struct arma_info *ainfo, FILE *fp)
+static void x12_pdq_string (arma_info *ainfo, FILE *fp)
 {
     fputc('(', fp);
     
@@ -649,14 +654,13 @@ static void x12_pdq_string (struct arma_info *ainfo, FILE *fp)
     fputc(')', fp);
 }
 
-#define MAXOBS 720
-#define MAXFCAST 60
-
 static int write_spc_file (const char *fname, 
 			   const double **Z, const DATAINFO *pdinfo,
-			   struct arma_info *ainfo, const int *alist,
-			   gretlopt opt)
+			   arma_info *ainfo, const int *alist,
+			   int pdmax, gretlopt opt)
 {
+    int maxobs = pdmax * 50;
+    int maxfcast = pdmax * 5;
     int ylist[2];
     int *xlist = NULL;
     FILE *fp;
@@ -681,12 +685,10 @@ static int write_spc_file (const char *fname,
 
     gretl_push_c_numeric_locale();
 
-    fprintf(fp, "series {\n period = %d\n title = \"%s\"\n", pdinfo->pd, 
-	    pdinfo->varname[ainfo->yno]);
+    fprintf(fp, "series {\n title = \"%s\"\n period = %d\n", 
+	    pdinfo->varname[ainfo->yno], pdinfo->pd);
 
-    if (!arma_exact_ml(ainfo)) {
-	t1 -= ainfo->maxlag;
-    }
+    t1 -= ainfo->maxlag;
     
     make_x12a_date_string(t1, pdinfo, datestr);
     fprintf(fp, " start = %s\n", datestr);
@@ -703,13 +705,13 @@ static int write_spc_file (const char *fname,
 
 	tmax = pdinfo->n - 1;
 	nobs = tmax - ainfo->t1 + 1; /* t1? */
-	if (nobs > MAXOBS) {
-	    tmax -= nobs - MAXOBS;
+	if (nobs > maxobs) {
+	    tmax -= nobs - maxobs;
 	}
 	nfcast = tmax - ainfo->t2;
-	if (nfcast > MAXFCAST) {
-	    tmax -= nfcast - MAXFCAST;
-	    nfcast -= nfcast - MAXFCAST;
+	if (nfcast > maxfcast) {
+	    tmax -= nfcast - maxfcast;
+	    nfcast -= nfcast - maxfcast;
 	}
 #if 0
 	fprintf(stderr, "x12a: doing forecast: nfcast = %d\n", nfcast);
@@ -782,6 +784,20 @@ static int write_spc_file (const char *fname,
     return 0;
 }
 
+static void print_x12a_error_file (const char *fname, PRN *prn)
+{
+    FILE *fp = gretl_fopen(fname, "r");
+
+    if (fp != NULL) {
+	char line[512];
+
+	while (fgets(line, sizeof line, fp)) {
+	    pputs(prn, line);
+	}
+	fclose(fp);
+    }
+}
+
 static void delete_old_files (const char *path)
 {
     const char *exts[] = {
@@ -801,7 +817,7 @@ static void delete_old_files (const char *path)
 
 MODEL arma_x12_model (const int *list, const char *pqspec,
 		      const double **Z, const DATAINFO *pdinfo,
-		      gretlopt opt, PRN *prn)
+		      int pdmax, gretlopt opt, PRN *prn)
 {
     int verbose = (opt & OPT_V);
     const char *prog = gretl_x12_arima();
@@ -810,8 +826,7 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
     int *alist = NULL;
     PRN *vprn = NULL;
     MODEL armod;
-    struct arma_info ainfo;
-    char flags = ARMA_X12A;
+    arma_info ainfo;
 #ifdef WIN32
     char *cmd;
 #endif
@@ -824,11 +839,7 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 	opt |= OPT_F;
     }
 
-    if (!(opt & OPT_C)) {
-	flags |= ARMA_EXACT;
-    }
-
-    arma_info_init(&ainfo, flags, pqspec, pdinfo);
+    arma_info_init(&ainfo, opt | OPT_X, pqspec, pdinfo);
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
@@ -857,14 +868,14 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 
     /* create differenced series if needed */
     if (ainfo.d > 0 || ainfo.D > 0) {
-	err = arima_difference(Z[ainfo.yno], &ainfo);
+	err = arima_difference(&ainfo, Z, pdinfo);
     }  
 
-    sprintf(yname, pdinfo->varname[ainfo.yno]);
+    strcpy(yname, pdinfo->varname[ainfo.yno]);
 
     /* write out an .spc file */
     sprintf(path, "%s%c%s.spc", workdir, SLASH, yname);
-    write_spc_file(path, Z, pdinfo, &ainfo, alist, opt);
+    write_spc_file(path, Z, pdinfo, &ainfo, alist, pdmax, opt);
 
     /* remove any files from on old run, in case of error */
     delete_old_files(path);
@@ -891,13 +902,18 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 	    if (gretl_in_gui_mode()) {
 		add_unique_output_file(&armod, path);
 	    }
-	    gretl_model_set_int(&armod, "arma_flags", (int) flags);
+	    gretl_model_set_int(&armod, "arma_flags", (int) ainfo.flags);
 	}	
     } else {
 	armod.errcode = E_UNSPEC;
 	gretl_errmsg_set(_("Failed to execute x12arima"));
     }
 
+    if (armod.errcode && vprn != NULL) {
+	sprintf(path, "%s%c%s.err", workdir, SLASH, yname);
+	print_x12a_error_file(path, vprn);
+    }
+
     if (vprn != NULL && vprn != prn) {
 	iter_print_callback(0, vprn);
 	close_down_verbose_printer(vprn);
@@ -908,6 +924,11 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
     free(alist);
     arma_info_cleanup(&ainfo);
 
+    if (armod.errcode && (opt & OPT_U)) {
+	/* continue on error */
+	armod.opt |= OPT_U;
+    }
+
     return armod;
 }
 
