@@ -1042,6 +1042,7 @@ garch_hessian (fcpinfo *f, gretl_matrix *V, double toler,
 	       int *count)
 {
     static double ll1 = 0.0, fs = 0.0;
+    int i, sign_done = 0;
     int err;
 
     vcv_setup(f, V, VCV_HESSIAN);
@@ -1050,7 +1051,22 @@ garch_hessian (fcpinfo *f, gretl_matrix *V, double toler,
 	*count += 1;
     }
 
-    err = gretl_invert_symmetric_indef_matrix(V);
+    if (toler == 0.0) {
+	for (i=0; i<V->rows; i++) {
+	    if (gretl_matrix_get(V, i, i) < 0.0) {
+		gretl_matrix_switch_sign(V);
+		sign_done = 1;
+		break;
+	    }
+	}
+    }
+
+    if (toler == 0.0) {
+	err = gretl_invert_symmetric_matrix(V);
+    } else {
+	err = gretl_invert_symmetric_indef_matrix(V);
+    }
+
     if (err) {
 	fprintf(stderr, "garch_hessian: matrix inversion failed\n");
 	return err;
@@ -1060,7 +1076,9 @@ garch_hessian (fcpinfo *f, gretl_matrix *V, double toler,
 	fcp_iterate(f, V, &ll1, &fs, toler, *count);
     }
 
-    gretl_matrix_switch_sign(V);
+    if (!sign_done) {
+	gretl_matrix_switch_sign(V);
+    }
 
     return err;
 } 
