@@ -35,10 +35,11 @@
 static void add_garch_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 				const int *list)
 {
-    int p = list[1];
-    int q = list[2];
-    int r = list[0] - 4;
-    int i, j, np = 2 + p + q + r;
+    int p = list[1];        /* GARCH beta terms */
+    int q = list[2];        /* ARCH alpha terms > 0 */
+    int r = list[0] - 4;    /* regressors */
+    int np = 1 + p + q + r; /* the "1" is for alpha(0) */
+    int i, j;
 
     free(pmod->list);
     pmod->list = gretl_list_copy(list);
@@ -49,6 +50,7 @@ static void add_garch_varnames (MODEL *pmod, const DATAINFO *pdinfo,
     }
 
     j = 0;
+
     for (i=0; i<r; i++) {
 	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[5+i]]);
     }
@@ -96,6 +98,7 @@ write_garch_stats (MODEL *pmod, const int *list,
 {
     double *garch_h;
     double den;
+    const double *vcoef;
     int ynum = list[4];
     int nvp = list[1] + list[2];
     int xvars = list[0] - 4;
@@ -131,12 +134,14 @@ write_garch_stats (MODEL *pmod, const int *list,
 	pmod->yhat[i] = Z[ynum][i] * scale - pmod->uhat[i];
     }
 
+    vcoef = pmod->coeff + xvars;
+
     /* set sigma to its unconditional or steady-state value */
     den = 1.0;
-    for (i=0; i<nvp; i++) {
-	den -= pmod->coeff[i+xvars+1];
+    for (i=1; i<=nvp; i++) {
+	den -= vcoef[i];
     }
-    pmod->sigma = sqrt(pmod->coeff[xvars] / den);
+    pmod->sigma = sqrt(vcoef[0] / den);
 
     pmod->adjrsq = NADBL; 
     pmod->fstt = NADBL;
@@ -835,9 +840,24 @@ static MODEL garch_run_ols (const int *list, double ***pZ,
 
     free(ols_list);
 
+    if (!model.errcode) {
+	clear_model_xpx(&model);
+    }
+
     return model;
 }
 
+static void clean_dropped_vars (MODEL mod, int *list)
+{
+    if (list[0] - mod.list[0] > 3) {
+	int i;
+        list[0] = mod.list[0] + 3;
+	for (i=4; i<=list[0]; i++) {
+	    list[i] = mod.list[i-3];
+	}
+    }
+}
+
 static void garch_add_lr_test (MODEL *pmod, double llr,
 			       const int *list)
 {
@@ -860,7 +880,7 @@ static void garch_standardize_residuals (MODEL *pmod)
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    pmod->uhat[t] /= sqrt(h[t]);
 	}
-	pmod->opt |= OPT_U;
+	pmod->opt |= OPT_E;
     }
 }
 
@@ -894,6 +914,7 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 	return model;
     }
 
+    clean_dropped_vars(model, list);
     llr = model.lnL;
     ols_T = model.nobs;
 
@@ -935,12 +956,15 @@ MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 #endif
 
     if (model.errcode == 0) {
-	if (opt & OPT_U) {
+	if (opt & OPT_E) {
 	    garch_standardize_residuals(&model);
 	}
 	if (!na(llr) && ols_T == model.nobs) {
 	    garch_add_lr_test(&model, llr, cmdlist);
 	}
+    } else if (opt & OPT_U) {
+	/* continue on error */
+	model.opt |= OPT_U;
     }
 
     free(list);
