@@ -24,19 +24,22 @@
 #include "gretl_bfgs.h"
 
 #define HDEBUG 0
-#define HTOL 1.0e-09
 
 typedef struct h_container_ h_container;
 
 struct h_container_ {
     const int *list;         /* incoming model specification */
+    int t1, t2;              /* start and end of sample range */
     int kmain;		     /* no. of params in the main eq. */
     int ksel;		     /* no. of params in the selection eq. */
     double ll;		     /* log-likelihood */
+    gretl_matrix *score;     /* score matrix */
+    gretl_matrix *sscore;    /* score vector (sum) */
 
-    int ntot, nunc;	     /* total & uncensored obs */
+    int ntot, nunc;	     /* total and uncensored obs */
     int depvar;		     /* location of y in array Z */
     int selvar;		     /* location of selection variable in array Z */
+    int millsvar;            /* location of Mills ratios in array Z */
     int *Xlist;		     /* regressor list for the main eq. */
     int *Zlist;		     /* regressor list for the selection eq. */
 
@@ -84,6 +87,8 @@ static void h_container_destroy (h_container *HC)
     gretl_vector_free(HC->fitted);
     gretl_vector_free(HC->u);
     gretl_vector_free(HC->ndx);
+    gretl_matrix_free(HC->score);
+    gretl_vector_free(HC->sscore);
 
     gretl_vector_free(HC->beta);
     gretl_vector_free(HC->gama);
@@ -107,6 +112,7 @@ static h_container *h_container_new (const int *list)
     }
 
     HC->list = list;
+    HC->t1 = HC->t2 = 0;
     HC->ll = NADBL;
 
     HC->Xlist = NULL;
@@ -122,6 +128,8 @@ static h_container *h_container_new (const int *list)
     HC->fitted = NULL;
     HC->u = NULL;
     HC->ndx = NULL;
+    HC->score = NULL;
+    HC->sscore = NULL;
 
     HC->beta = NULL;
     HC->gama = NULL;
@@ -148,43 +156,42 @@ static h_container *h_container_new (const int *list)
 */
 
 static int 
-make_heckit_NA_mask (h_container *HC, const int *Xlist, const int *Zlist,
+make_heckit_NA_mask (h_container *HC, const int *reglist, const int *sellist,
 		     const double **Z, const DATAINFO *pdinfo)
 {
-    int sel = Zlist[1];
     int T = sample_size(pdinfo);
-    int i, t, s = 0, m = 0;
-    int err = 0;
+    int totmiss = 0;
+    int i, t, err = 0;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	int miss = 0;
 
-	if (Z[sel][t] == 1.0) {
-	    /* screen the X variables */
-	    for (i=1; i<=Xlist[0] && !miss; i++) {
-		if (na(Z[Xlist[i]][t])) {
+	/* screen the primary equation variables */
+	if (Z[HC->selvar][t] == 1.0) {
+	    for (i=1; i<=reglist[0] && !miss; i++) {
+		if (na(Z[reglist[i]][t])) {
 		    miss = 1;
-		    m++;
-		    s++;
+		    totmiss++;
 		}
 	    }
 	}
-	if (!miss) {
-	    /* screen the Z variables */
-	    for (i=1; i<=Zlist[0]; i++) {
-		if (na(Z[Zlist[i]][t])) {
-		    m++;
-		    break;
-		}
+
+	/* screen the selection equation variables */
+	for (i=1; i<=sellist[0] && !miss; i++) {
+	    if (na(Z[sellist[i]][t])) {
+		miss = 1;
+		totmiss++;
 	    }
 	}
     }
 
-    if (m == T || s == T) {
+    if (totmiss == T) {
 	return E_MISSDATA;
     }
 
-    if (m > 0) {
+    if (totmiss > 0) {
+	int s;
+
 	HC->probmask = malloc(pdinfo->n + 1);
 	HC->fullmask = calloc(T, 1);
 
@@ -199,9 +206,9 @@ make_heckit_NA_mask (h_container *HC, const int *Xlist, const int *Zlist,
 	for (s=0, t=pdinfo->t1; t<=pdinfo->t2; t++, s++) {
 	    int miss = 0;
 
-	    if (Z[sel][t] == 1.0) {
-		for (i=1; i<=Xlist[0] && !miss; i++) {
-		    if (na(Z[Xlist[i]][t])) {
+	    if (Z[HC->selvar][t] == 1.0) {
+		for (i=1; i<=reglist[0] && !miss; i++) {
+		    if (na(Z[reglist[i]][t])) {
 			HC->fullmask[s] = 1;
 			HC->probmask[t] = '1';
 			miss = 1;
@@ -209,8 +216,8 @@ make_heckit_NA_mask (h_container *HC, const int *Xlist, const int *Zlist,
 		}
 	    }
 
-	    for (i=1; i<=Zlist[0] && !miss; i++) {
-		if (na(Z[Zlist[i]][t])) {
+	    for (i=1; i<=sellist[0] && !miss; i++) {
+		if (na(Z[sellist[i]][t])) {
 		    HC->fullmask[s] = 1;
 		    HC->probmask[t] = '1';
 		    miss = 1;
@@ -229,13 +236,11 @@ make_heckit_NA_mask (h_container *HC, const int *Xlist, const int *Zlist,
     return err;
 }
 
-static int make_uncens_mask (h_container *HC, const int *Xl, 
-			     const int *Zl, const double **Z, 
+static int make_uncens_mask (h_container *HC, const double **Z, 
 			     DATAINFO *pdinfo)
 {
     int T = sample_size(pdinfo);
     int t, s = 0;
-    int selvar = Zl[1];
 
     HC->uncmask = calloc(T, 1);
     if (HC->uncmask == NULL) {
@@ -244,14 +249,14 @@ static int make_uncens_mask (h_container *HC, const int *Xl,
 
     if (HC->fullmask == NULL) {
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    if (Z[selvar][t] == 0.0) {
+	    if (Z[HC->selvar][t] == 0.0) {
 		HC->uncmask[s] = 1;
 	    }
 	    s++;
 	}
     } else {
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    if (Z[selvar][t] == 0.0 || HC->fullmask[s]) {
+	    if (Z[HC->selvar][t] == 0.0 || HC->fullmask[s]) {
 		HC->uncmask[s] = 1;
 	    }
 	    s++;
@@ -261,6 +266,72 @@ static int make_uncens_mask (h_container *HC, const int *Xl,
     return 0;
 }
 
+#if HDEBUG > 1
+
+static void debug_print_masks (h_container *HC,
+			       const int *Xl,
+			       const double **Z)
+{
+    double x;
+    int t, i;
+
+    for (t=HC->t1; t<=HC->t2; t++) {
+	if (HC->fullmask[t] == 1) {
+	    fputc('F', stderr);
+	}
+	if (HC->uncmask[t] == 1) {
+	    fputc('U', stderr);
+	}
+	fputc('\t', stderr);
+	x = Z[HC->selvar][t];
+	if (na(x)) {
+	    fprintf(stderr, "          NA");
+	} else {
+	    fprintf(stderr, "%12.4f", x);
+	}
+	for (i=1; i<=Xl[0]; i++) {
+	    x = Z[Xl[i]][t];
+	    if (na(x)) {
+		fprintf(stderr, "          NA");
+	    } else {
+		fprintf(stderr, "%12.4f", x);
+	    }
+	}
+	fputc('\n', stderr);
+    }
+}
+
+#endif
+
+/* make copies of the primary and selection lists with the first
+   terms (i.e. the dependent variable and the selection variable,
+   respectively) removed.
+*/
+
+static int h_make_data_lists (h_container *HC,
+			      const int *reglist,
+			      const int *sellist)
+{
+    int i;
+
+    HC->Xlist = gretl_list_new(HC->kmain);
+    HC->Zlist = gretl_list_new(HC->ksel);
+
+    if (HC->Xlist == NULL || HC->Zlist == NULL) {
+	return E_ALLOC;
+    }
+
+    for (i=1; i<=HC->Xlist[0]; i++) {
+	HC->Xlist[i] = reglist[i+1];
+    }
+
+    for (i=1; i<=HC->Zlist[0]; i++) {
+	HC->Zlist[i] = sellist[i+1];
+    }
+
+    return 0;
+}
+
 static int h_container_fill (h_container *HC, const int *Xl, 
 			     const int *Zl, const double **Z, 
 			     DATAINFO *pdinfo, MODEL *probmod, 
@@ -269,21 +340,13 @@ static int h_container_fill (h_container *HC, const int *Xl,
     gretl_vector *tmp = NULL;
     double bmills, s2, mdelta;
     int tmplist[2];
-    int t1 = pdinfo->t1;
-    int t2 = pdinfo->t2;
-    int depvar, selvar;
-    int v, i, err = 0;
+    int t1, t2, npar;
+    int i, err = 0;
 
-    /* it is assumed that the Mills ratios have just been added to the dataset;
-       hence they're the last variable
-    */
-    v = pdinfo->v - 1;
-
-    /* X does NOT include the Mills ratios: hence the "-2" */
-    HC->depvar = depvar = Xl[1];
-    HC->selvar = selvar = Zl[1];
+    /* X does NOT include the Mills ratios: hence the "-1" */
     HC->kmain = olsmod->ncoeff - 1;
     HC->ksel = probmod->ncoeff;
+    npar = HC->kmain + HC->ksel + 2;
 
     if (HC->kmain < 1) {
 	/* FIXME? */
@@ -310,98 +373,67 @@ static int h_container_fill (h_container *HC, const int *Xl,
     HC->lambda = olsmod->coeff[HC->kmain];
 
     /*
-      we'll be working with 2 masks: fullmask just masks unusable
+      we'll be working with two masks: fullmask just masks unusable
       observations, and may be NULL; uncmasks skips censored
       observations too.
     */
-    err = make_uncens_mask(HC, Xl, Zl, Z, pdinfo);
+    err = make_uncens_mask(HC, Z, pdinfo);
 
-#if HDEBUG
-    if (!err) {
-	double x;
-	int t;
-
-	if (HC->fullmask != NULL) {
-	    for (t=t1; t<=t2; t++) {
-		if (HC->fullmask[t] == 1) {
-		    fputc('F', stderr);
-		}
-		if (HC->uncmask[t] == 1) {
-		    fputc('U', stderr);
-		}
-		fputc('\t', stderr);
-		x = Z[selvar][t];
-		if (na(x)) {
-		    fprintf(stderr, "          NA");
-		} else {
-		    fprintf(stderr, "%12.4f", x);
-		}
-		for (i=1; i<=Xl[0]; i++) {
-		    x = Z[Xl[i]][t];
-		    if (na(x)) {
-			fprintf(stderr, "          NA");
-		    } else {
-			fprintf(stderr, "%12.4f", x);
-		    }
-		}
-		fputc('\n', stderr);
-	    }
-	}
+#if HDEBUG > 1
+    if (!err && HC->fullmask != NULL) {
+	debug_print_masks(HC, Xl, Z);
     }
 #endif
 
+    t1 = pdinfo->t1;
+    t2 = pdinfo->t2;
+
     tmplist[0] = 1;
 
     if (!err) {
-	tmplist[1] = depvar;
+	/* dependent variable vector */
+	tmplist[1] = HC->depvar;
 	HC->y = gretl_matrix_data_subset_masked(tmplist, Z, t1, t2, 
 						HC->uncmask, &err);
     }
 
     if (!err) {
 	HC->nunc = gretl_matrix_rows(HC->y);
-	tmplist[1] = selvar;
+	/* selection variable vector */
+	tmplist[1] = HC->selvar;
 	HC->d = gretl_matrix_data_subset_masked(tmplist, Z, t1, t2, 
 						HC->fullmask, &err);
     }
 
     if (!err) {
 	HC->ntot = gretl_matrix_rows(HC->d);
-	tmplist[1] = v;
+	/* Mills ratio vector */
+	tmplist[1] = HC->millsvar;
 	HC->mills = gretl_matrix_data_subset_masked(tmplist, Z, t1, t2, 
 						    HC->uncmask, &err);
     }
 
     if (!err) {
-	HC->Xlist = gretl_list_new(HC->kmain);
-	HC->Zlist = gretl_list_new(HC->ksel);
-	if (HC->Xlist == NULL || HC->Zlist == NULL) {
-	    err = E_ALLOC;
-	}
+	err = h_make_data_lists(HC, Xl, Zl);
     }
 
     if (err) {
 	return err;
     }
 
-    for (i=1; i<=HC->Xlist[0]; i++) {
-	HC->Xlist[i] = Xl[i+1];
-    }
-
-    for (i=1; i<=HC->Zlist[0]; i++) {
-	HC->Zlist[i] = Zl[i+1];
-    }
-
+    /* regressors in primary equation */
     HC->reg = gretl_matrix_data_subset_masked(HC->Xlist, Z, t1, t2, 
 					      HC->uncmask, &err);
 
     if (!err) {
+	/* regressors in selection equation */
 	HC->selreg = 
 	    gretl_matrix_data_subset_masked(HC->Zlist, Z, t1, t2, 
 					    HC->fullmask, &err);
     }
 
     if (!err) {
+	/* regressors in selection equation, uncensored sample */
 	HC->selreg_u = 
 	    gretl_matrix_data_subset_masked(HC->Zlist, Z, t1, t2, 
 					    HC->uncmask, &err);
@@ -421,7 +453,7 @@ static int h_container_fill (h_container *HC, const int *Xl,
 
     if (!err) {
 	bmills = olsmod->coeff[olsmod->ncoeff-1];
-	mdelta  = gretl_vector_mean(HC->delta);
+	mdelta = gretl_vector_mean(HC->delta);
 	s2 = olsmod->ess / olsmod->nobs + mdelta * bmills * bmills;
 	HC->sigma = sqrt(s2);
 	HC->rho = bmills / HC->sigma;
@@ -441,8 +473,11 @@ static int h_container_fill (h_container *HC, const int *Xl,
 	HC->fitted = gretl_matrix_alloc(HC->nunc, 1);
 	HC->u      = gretl_matrix_alloc(HC->nunc, 1);
 	HC->ndx    = gretl_matrix_alloc(HC->ntot, 1);
+	HC->score  = gretl_matrix_alloc(HC->ntot, npar); 
+	HC->sscore = gretl_vector_alloc(npar); 
 
-	if (HC->fitted == NULL || HC->u == NULL || HC->ndx == NULL) {
+	if (HC->fitted == NULL || HC->u == NULL || HC->ndx == NULL
+	    || HC->score == NULL || HC->sscore == NULL) {
 	    err = E_ALLOC;
 	}
     }
@@ -453,40 +488,46 @@ static int h_container_fill (h_container *HC, const int *Xl,
 static double h_loglik (const double *param, void *ptr)
 {
     h_container *HC = (h_container *) ptr;
-    double lnsig, isqrtrhoc, x, ll = NADBL;
-    int k1 = HC->kmain;
-    int k2 = HC->ksel;
+    double lnsig, x, ll = NADBL;
+    double ca, sa, arho;
     int i, j, err = 0;
+    int kmain, ksel, kmax, npar;
+
+    kmain = HC->kmain;
+    ksel  = HC->ksel;
+    kmax  = kmain + ksel;
+    npar  = kmax + 2;
     
-    for (i=0; i<k1; i++) {
+    for (i=0; i<kmain; i++) {
 	gretl_vector_set(HC->beta, i, param[i]);
     }
 
     j = 0;
-    for (i=k1; i<k1+k2; i++) {
+    for (i=kmain; i<kmax; i++) {
 	gretl_vector_set(HC->gama, j++, param[i]);
     }
 
-    HC->sigma = param[k1+k2];
+    HC->sigma = param[npar-2];
     lnsig = log(HC->sigma);
+    
+    arho = param[npar-1];
+    HC->rho = tanh(arho);
+    ca = cosh(arho);
+    sa = sinh(arho);
 
-    HC->rho = param[k1+k2+1];
-
-#if HDEBUG
+#if HDEBUG > 1
     gretl_matrix_print(HC->beta, "beta");
     gretl_matrix_print(HC->gama, "gama");
     fprintf(stderr, "sigma = %12.6f, rho = %12.6f", HC->sigma, HC->rho);
     fputc('\n', stderr);
 #endif
 
-    if (HC->sigma <= 0 || fabs(HC->rho) >= 1) {
+    if (HC->sigma <= 0) {
 	return NADBL;
-    } else {
-	isqrtrhoc = 1 / sqrt(1 - HC->rho * HC->rho);
-    }
+    } 
 
     err = gretl_matrix_multiply(HC->reg, HC->beta, HC->fitted);
-    
+
     if (!err) {
 	gretl_matrix_copy_values(HC->u, HC->y);
 	err = gretl_matrix_subtract_from(HC->u, HC->fitted);
@@ -501,36 +542,167 @@ static double h_loglik (const double *param, void *ptr)
     }
 
     if (!err) {
-	double ut, ndxt;
-	double ll0 = 0;
-	double ll1 = 0;
-	double ll2 = 0;
-	int sel;
-	/* 
-	   i goes through all obs, while j keeps track of the uncensored
-	   ones
-	*/
+	double ll0 = 0, ll1 = 0, ll2 = 0;
+	double ndxt, ut = 0;
+	double P, f, mills, tmp, psum;
+	int sel, k;
+
+	gretl_matrix_zero(HC->score);
+	gretl_matrix_zero(HC->sscore);
+
+	/* i goes through all obs, while j keeps track of the uncensored
+	   ones */
 	j = 0;
 	for (i=0; i<HC->ntot; i++) {
-	    sel = (1.0 == gretl_vector_get(HC->d, i));
-	    ndxt = gretl_vector_get(HC->ndx,i);
+	    sel = (gretl_vector_get(HC->d, i) == 1.0);
+	    ndxt = gretl_vector_get(HC->ndx, i);
 	    if (sel) {
-		ut = gretl_vector_get(HC->u, j++);
-		x = (ndxt + HC->rho*ut) * isqrtrhoc;
-		ll1 += log(normal_pdf(ut)) - lnsig;
-		ll2 += log(normal_cdf(x));
+		ut = gretl_vector_get(HC->u, j);
+		x = ca * (ndxt + HC->rho*ut);
+
+		ll1 -= LN_SQRT_2_PI + 0.5*ut*ut + lnsig;
+		P = normal_cdf(x);
+		f = normal_pdf(x);
+		mills = f/P;
+		ll2 += log(P);
 	    } else {
-		ll0 += log(normal_cdf(-ndxt));
+		P = normal_cdf(-ndxt);
+		f = normal_pdf(ndxt);
+		mills = -f/P;
+		ll0 += log(P);
+	    }
+
+	    /* score for beta */
+	    if (sel) {
+		tmp = (ut - sa*mills)/HC->sigma;
+		for (k=0; k<kmain; k++) {
+		    x = tmp * gretl_matrix_get(HC->reg, j, k);
+		    gretl_matrix_set(HC->score, i, k, x); 
+		    psum = x + gretl_vector_get(HC->sscore, k);
+		    gretl_vector_set(HC->sscore, k, psum);
+		}
+	    }
+
+	    /* score for gamma */
+	    tmp = sel ? ca*mills : mills;
+	    for (k=0; k<ksel; k++) {
+		x = tmp * gretl_matrix_get(HC->selreg, i, k);
+		gretl_matrix_set(HC->score, i, kmain+k, x); 
+		psum = x + gretl_vector_get(HC->sscore, kmain+k);
+		gretl_vector_set(HC->sscore, kmain+k, psum);
+	    }
+
+	    /* score for sigma and arho */
+	    if (sel) {
+		x = (ut * (ut - sa*mills) - 1) / HC->sigma;
+		gretl_matrix_set(HC->score, i, npar-2, x); 
+		psum = x + gretl_vector_get(HC->sscore, npar-2);
+		gretl_vector_set(HC->sscore, npar-2, psum);
+
+		x = mills * ca * (ut + HC->rho*ndxt);
+		gretl_matrix_set(HC->score, i, npar-1, x); 
+		psum = x + gretl_vector_get(HC->sscore, npar-1);
+		gretl_vector_set(HC->sscore, npar-1, psum);
+	    }
+
+	    if (sel) {
+		j++;
 	    }
 	}
-	
-#if HDEBUG
-	fprintf(stderr, "ll0 = %g, ll1 = %g, ll2 = %g\n", ll0, ll1, ll2);
-#endif
 	ll = ll0 + ll1 + ll2;
+#if HDEBUG
+	fprintf(stderr, "ll0 = %g, ll1 = %g, ll2 = %g: ll = %g (ntot = %d)\n", 
+		ll0, ll1, ll2, ll, HC->ntot);
+#endif
+    }
+    
+    return ll;
+}
+
+static int heckit_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll, 
+			 void *ptr)
+{
+    h_container *HC = (h_container *) ptr;
+    int i;
+
+    for (i=0; i<npar; i++) {
+	s[i] = gretl_vector_get(HC->sscore,i);
     }
 
-    return ll;
+    return 1;
+}
+
+double *heckit_hessian (const double *b, int n, BFGS_CRIT_FUNC func, 
+			h_container *HC, int *err)
+{
+    double x, eps = 1.0e-05;
+    gretl_matrix *H = NULL;
+    gretl_matrix *splus = NULL;
+    gretl_matrix *sminus = NULL;
+    double *hess;
+    double *theta;
+    int m = n*(n+1)/2;
+    int i, j, k;
+
+    hess   = malloc(m * sizeof *hess);
+    theta  = malloc(n * sizeof *theta);
+    H      = gretl_matrix_alloc(n, n);
+    splus  = gretl_matrix_alloc(1, n);
+    sminus = gretl_matrix_alloc(1, n);
+    
+    if (hess == NULL || theta == NULL || H == NULL ||
+	splus == NULL || sminus == NULL) {
+	*err = E_ALLOC;
+	free(hess);
+	hess = NULL;
+	goto bailout;
+    }
+
+    for (i=0; i<n; i++) {
+	theta[i] = b[i];
+    }
+
+    for (i=0; i<n; i++) {
+	theta[i] += eps;
+	h_loglik(theta, HC);
+	for (j=0; j<n; j++) {
+	    x = gretl_vector_get(HC->sscore, j);
+	    gretl_vector_set(splus, j, x);
+	}
+
+	theta[i] -= 2*eps;
+	h_loglik(theta, HC);
+	for (j=0; j<n; j++) {
+	    x = gretl_vector_get(HC->sscore, j);
+	    gretl_vector_set(sminus, j, x);
+	}
+
+	theta[i] += eps;
+	for (j=0; j<n; j++) {
+	    x = gretl_vector_get(splus, j);
+	    x -= gretl_vector_get(sminus, j);
+	    gretl_matrix_set(H, i, j, -x/(2*eps));
+	}
+    }
+
+    gretl_matrix_xtr_symmetric(H);
+    gretl_invert_symmetric_matrix(H);
+
+    k = 0;
+    for (i=0; i<n; i++) {
+	for (j=i; j<n; j++) {
+	    hess[k++] = gretl_matrix_get(H, i, j);
+	}
+    }
+
+ bailout:
+
+    gretl_matrix_free(splus);
+    gretl_matrix_free(sminus);
+    gretl_matrix_free(H);
+    free(theta);
+
+    return hess;
 }
 
 /*
@@ -546,19 +718,17 @@ static double h_loglik (const double *param, void *ptr)
 static void heckit_yhat_uhat (MODEL *hm, h_container *HC, 
 			      const double **Z, DATAINFO *pdinfo)
 {
-
-    int i, t, v, kb, kg;
-    int depvar, selvar;
-    double x, xb, zg;
+    double x, xb, zg, u;
     double f, F, lam = HC->lambda;
-
-    depvar = HC->depvar;
-    selvar = HC->selvar;
-    kb = HC->kmain;
-    kg = HC->ksel;
+    double rho = HC->rho;
+    double rhofunc = 1 / sqrt(1 - rho * rho);
+    double lnsig = log(HC->sigma);
+    int kb = HC->kmain;
+    int kg = HC->ksel;
+    int i, t, v;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xb = 0;
+	xb = 0.0;
 	for (i=0; i<kb; i++) {
 	    v = HC->Xlist[i+1];
 	    x = Z[v][t];
@@ -574,34 +744,35 @@ static void heckit_yhat_uhat (MODEL *hm, h_container *HC,
 	    hm->yhat[t] = xb;
 	}
 
-	if (Z[selvar][t] == 0) {
-	    /* censored */
-	    zg = 0;
-	    for (i=0; i<kg; i++) {
-		v = HC->Zlist[i+1];
-		x = Z[v][t];
-		if (na(x)) {
-		    zg = NADBL;
-		    break;
-		} else {
-		    zg += x * gretl_vector_get(HC->gama, i);
-		}
-	    }
-
-	    if (!na(zg)) {
-		f = normal_pdf(zg);
-		F = normal_cdf(-zg);
-		hm->uhat[t] = -lam * f / F;
+	zg = 0.0;
+	for (i=0; i<kg; i++) {
+	    v = HC->Zlist[i+1];
+	    x = Z[v][t];
+	    if (na(x)) {
+		zg = NADBL;
+		break;
 	    } else {
-		hm->uhat[t] = NADBL;
+		zg += x * gretl_vector_get(HC->gama, i);
 	    }
-	} else {
+	}
+
+	if (na(zg)) {
+	    hm->uhat[t] = NADBL;
+	    continue;
+	}
+
+	if (Z[HC->selvar][t] == 0) {
+	    /* censored */
+	    f = normal_pdf(zg);
+	    F = normal_cdf(-zg);
+	    hm->uhat[t] = -lam * f / F;
+	    hm->llt[t] = log(F);
+	} else if (!na(xb) && Z[HC->selvar][t] == 1) {
 	    /* uncensored */
-	    if (!na(xb)) {
-		if (Z[selvar][t] == 1) {
-		    hm->uhat[t] = Z[depvar][t] - xb;
-		}
-	    }
+	    hm->uhat[t] = u = Z[HC->depvar][t] - xb;
+	    u /= HC->sigma;
+	    x = (zg + rho*u) * rhofunc;
+	    hm->llt[t] = -LN_SQRT_2_PI - 0.5*u*u - lnsig + log(normal_cdf(x));
 	}
     }
 
@@ -623,32 +794,31 @@ static void heckit_yhat_uhat (MODEL *hm, h_container *HC,
 static int transcribe_heckit_params (MODEL *hm, h_container *HC, DATAINFO *pdinfo)
 {
     double *fullcoeff;
-    int ko = HC->kmain;
-    int kp = HC->ksel;
-    int k = ko + kp + 1;
+    int kb = HC->kmain;
+    int npar = kb + HC->ksel + 1;
     int i, err = 0;
 
-    fullcoeff = malloc(k * sizeof *fullcoeff);
+    fullcoeff = malloc(npar * sizeof *fullcoeff);
     if (fullcoeff == NULL) {
 	err = E_ALLOC;
     }
 
     if (!err) {
-	err = gretl_model_allocate_params(hm, k);
+	err = gretl_model_allocate_params(hm, npar);
     }
 
     if (!err) {
-	for (i=0; i<ko; i++) {
+	for (i=0; i<kb; i++) {
 	    strcpy(hm->params[i], pdinfo->varname[hm->list[i+2]]);
 	    fullcoeff[i] = gretl_vector_get(HC->beta, i);
 	}
 	
-	strcpy(hm->params[ko], "lambda");
-	fullcoeff[ko] = HC->lambda;
+	strcpy(hm->params[kb], "lambda");
+	fullcoeff[kb] = HC->lambda;
 
-	for (i=0; i<kp; i++) {
-	    strcpy(hm->params[i+ko+1], pdinfo->varname[HC->Zlist[i+1]]);
-	    fullcoeff[i+ko+1] = gretl_vector_get(HC->gama, i);
+	for (i=0; i<HC->ksel; i++) {
+	    strcpy(hm->params[i+kb+1], pdinfo->varname[HC->Zlist[i+1]]);
+	    fullcoeff[i+kb+1] = gretl_vector_get(HC->gama, i);
 	}
     }
 
@@ -662,9 +832,11 @@ static int transcribe_heckit_params (MODEL *hm, h_container *HC, DATAINFO *pdinf
 	hm->list = gretl_list_copy(HC->list);
 	free(hm->coeff);
 	hm->coeff = fullcoeff;
-	hm->ncoeff = k;
-	gretl_model_set_coeff_separator(hm, N_("Selection equation"), ko + 1);
-	gretl_model_set_int(hm, "base-coeffs", ko);
+	hm->ncoeff = npar;
+	hm->t1 = HC->t1;
+	hm->t2 = HC->t2;
+	gretl_model_set_coeff_separator(hm, N_("Selection equation"), kb + 1);
+	gretl_model_set_int(hm, "base-coeffs", kb);
     }
     
     return err;
@@ -727,49 +899,36 @@ static int transcribe_2step_vcv (MODEL *pmod, const gretl_matrix *S,
 
 static int heckit_2step_vcv (h_container *HC, MODEL *olsmod)
 {
-    gretl_matrix *X = NULL;
-    gretl_matrix *Xw = NULL;
     gretl_matrix *W = HC->selreg_u;
     gretl_matrix *w = HC->delta;
-
-    gretl_matrix *XX = NULL;
-    gretl_matrix *XXi = NULL;
-    gretl_matrix *XXw = NULL;
-    gretl_matrix *XwZ = NULL;
-    gretl_matrix *S = NULL;
-
     gretl_matrix *Vp = HC->VProbit;
-
+    gretl_matrix_block *B = NULL;
+    gretl_matrix *X, *XX, *XXi;
+    gretl_matrix *XXw, *XwZ, *S;
+    gretl_matrix *Xw = NULL;
     int nX = HC->kmain + 1;
     int nZ = HC->ksel;
     int err = 0;
 
-    double s2, sigma, rho;
-
-    sigma = HC->sigma;
-    rho = HC->rho;
-    s2 = sigma * sigma;
-
     X = gretl_matrix_col_concat(HC->reg, HC->mills, &err);
-    if (err) {
-	goto bailout;
+
+    if (!err) {
+	Xw = gretl_matrix_dot_op(w, X, '*', &err);
     }
 
-    Xw = gretl_matrix_dot_op(w, X, '*', &err);
-    if (err) {
-	goto bailout;
+    if (!err) {
+	B = gretl_matrix_block_new(&XX, nX, nX,
+				   &XXi, nX, nX,
+				   &XXw, nX, nX,
+				   &XwZ, nX, nZ,
+				   &S, nX, nX,
+				   NULL);
+	if (B == NULL) {
+	    err = E_ALLOC;
+	}
     }
 
-    XX = gretl_matrix_alloc(nX, nX);
-    XXi = gretl_matrix_alloc(nX, nX);
-    XXw = gretl_matrix_alloc(nX, nX);
-    XwZ = gretl_matrix_alloc(nX, nZ);
-    S = gretl_matrix_alloc(nX, nX);
-
-    if (X == NULL || w == NULL || W == NULL || Xw == NULL ||
-	XX == NULL || XXi == NULL || XXw == NULL ||
-	XwZ == NULL || S == NULL) {
-	err = E_ALLOC;
+    if (err) {
 	goto bailout;
     }
 
@@ -793,11 +952,11 @@ static int heckit_2step_vcv (h_container *HC, MODEL *olsmod)
     gretl_matrix_qform(XwZ, GRETL_MOD_NONE,
 		       Vp, S, GRETL_MOD_NONE);
     gretl_matrix_subtract_from(XXw, S);
-    gretl_matrix_multiply_by_scalar(XXw, rho * rho);
+    gretl_matrix_multiply_by_scalar(XXw, HC->rho * HC->rho);
     gretl_matrix_subtract_from(XX, XXw);
     gretl_matrix_qform(XXi, GRETL_MOD_NONE,
 		       XX, S, GRETL_MOD_NONE);
-    gretl_matrix_multiply_by_scalar(S, s2);
+    gretl_matrix_multiply_by_scalar(S, HC->sigma * HC->sigma);
 
     err = transcribe_2step_vcv(olsmod, S, Vp);
 
@@ -805,11 +964,7 @@ static int heckit_2step_vcv (h_container *HC, MODEL *olsmod)
 
     gretl_matrix_free(X);
     gretl_matrix_free(Xw);
-    gretl_matrix_free(XX);
-    gretl_matrix_free(XXi);
-    gretl_matrix_free(XXw);
-    gretl_matrix_free(XwZ);
-    gretl_matrix_free(S);
+    gretl_matrix_block_destroy(B);
 
     return err;
 }
@@ -821,10 +976,11 @@ static int heckit_2step_vcv (h_container *HC, MODEL *olsmod)
 
 int add_lambda_to_ml_vcv (h_container *HC)
 {
-    int i, k, npar = HC->vcv->rows;
     gretl_matrix *J = NULL;
     gretl_matrix *tmp = NULL;
-    int err = 0;
+    int npar = HC->vcv->rows;
+    int kb = HC->kmain;
+    int i, err = 0;
 
     tmp = gretl_matrix_alloc(npar+1, npar+1);
     J = gretl_zero_matrix_new(npar+1, npar);
@@ -835,16 +991,14 @@ int add_lambda_to_ml_vcv (h_container *HC)
 	return E_ALLOC;
     }
 
-    k = HC->kmain;
-
-    for (i=0; i<k; i++) {
+    for (i=0; i<kb; i++) {
 	gretl_matrix_set(J, i, i, 1);
     }
 
-    gretl_matrix_set(J, k, npar-2, HC->rho);
-    gretl_matrix_set(J, k, npar-1, HC->sigma);
+    gretl_matrix_set(J, kb, npar-2, HC->rho);
+    gretl_matrix_set(J, kb, npar-1, HC->sigma);
 
-    for (i=k+1; i<=npar; i++) {
+    for (i=kb+1; i<=npar; i++) {
 	gretl_matrix_set(J, i, i-1, 1);
     }
 
@@ -857,10 +1011,38 @@ int add_lambda_to_ml_vcv (h_container *HC)
     return err;
 }
 
+static int adjust_ml_vcv_hyperbolic (h_container *HC)
+{
+    /*
+      Here we modify $vcv as if rho was used in BFGS instead 
+      of atanh(rho); we use the fact that 
+      \frac{\partial tanh(a)}{\partial a} = (cosh(a))^{-2}
+      which translates into
+      \frac{\partial \rho}{\partial a} = 1 - \rho^2
+    */
+
+    double x, jac = 1 - (HC->rho * HC->rho);
+    int npar = HC->vcv->rows;
+    int irho = npar - 1;
+    int i, err = 0;
+
+    for (i=0; i<npar; i++) {
+	x = gretl_matrix_get(HC->vcv, i, irho);
+	if (i == irho) {
+	    gretl_matrix_set(HC->vcv, irho, irho, x*jac*jac);
+	} else {
+	    gretl_matrix_set(HC->vcv, irho, i, x*jac);
+	    gretl_matrix_set(HC->vcv, i, irho, x*jac);
+	}
+    }
+
+    return err;
+}
+
 int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 {
-    int fncount, grcount;
-    double hij, rho;
+    int maxit, fncount, grcount;
+    double hij, rho, toler;
     double *hess = NULL;
     double *theta = NULL;
     int i, j, k, np = HC->kmain + HC->ksel + 2;
@@ -887,22 +1069,33 @@ int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 	rho = (rho > 0)? 0.99 : -0.99;
     }
 
-    theta[np-1] = rho;
+    theta[np-1] = atanh(rho);
+
+    BFGS_defaults(&maxit, &toler, HECKIT);
+
+    err = BFGS_max(theta, np, maxit, toler, &fncount, 
+		   &grcount, h_loglik, C_LOGLIK,
+		   heckit_score, HC, 
+		   (prn != NULL)? OPT_V : OPT_NONE, prn);
+
 
-    err = BFGS_max(theta, np, 1000, HTOL, 
-		   &fncount, &grcount, h_loglik, C_LOGLIK,
-		   NULL, HC, (prn != NULL)? OPT_V : OPT_NONE, prn);
 
     if (!err) {
 	HC->ll = hm->lnL = h_loglik(theta, HC);
 	gretl_model_set_int(hm, "fncount", fncount);	
 	gretl_model_set_int(hm, "grcount", grcount);	
 	HC->lambda = HC->sigma * HC->rho;
-	hess = numerical_hessian(theta, np, h_loglik, HC, &err);
+	hess = heckit_hessian(theta, np, h_loglik, HC, &err);
+    }
+
+    if (!err) {
+	HC->vcv = gretl_matrix_alloc(np, np);
+	if (HC->vcv == NULL) {
+	    err = E_ALLOC;
+	}
     }
 
     if (!err) {
-        HC->vcv = gretl_matrix_alloc(np,np);
 	k = 0;
 	for (i=0; i<np; i++) {
 	    for (j=i; j<np; j++) {
@@ -913,6 +1106,8 @@ int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 		}
 	    }
 	}
+
+	adjust_ml_vcv_hyperbolic(HC);
 	add_lambda_to_ml_vcv(HC);
 
 #if HDEBUG
@@ -931,17 +1126,13 @@ int heckit_ml (MODEL *hm, h_container *HC, PRN *prn)
 
 /*
   This function just copies the VCV matrix for the ML estimator into
-  the model struct
+  the model struct.  Note that we don't transcribe the variances for
+  sigma and rho into the model.
 */
 
 static int transcribe_ml_vcv (MODEL *pmod, h_container *HC)
 {
-    int nvc, npar;
-
-    /* We don't transcribe the variances for sigma and rho into the
-       model */
-    npar = HC->vcv->rows - 2;
-
+    int nvc, npar = HC->vcv->rows - 2;
     int i, j, k = 0;
     double vij;
 
@@ -982,38 +1173,39 @@ static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
 #endif
     MODEL hm;
     MODEL probmod;
-    int *Xlist = NULL;
-    int *Zlist = NULL;
-    int v = pdinfo->v;
-    int t, sel;
-    int err = 0;
+    int *reglist = NULL;
+    int *sellist = NULL;
+    int t, err = 0;
 
     gretl_model_init(&hm);
     gretl_model_init(&probmod);
 
-    err = gretl_list_split_on_separator(HC->list, &Xlist, &Zlist);
-    if (err) {
-	goto bailout;
-    } 
-
-    if (Xlist == NULL || Zlist == NULL) {
+    err = gretl_list_split_on_separator(HC->list, &reglist, &sellist);
+    if (!err && (reglist == NULL || sellist == NULL)) {
 	err = E_ARGS;
-	goto bailout;
     }
 
-    err = make_heckit_NA_mask(HC, Xlist, Zlist, (const double **) *pZ, 
-			      pdinfo);
+    if (!err) {
+	HC->depvar = reglist[1];
+	HC->selvar = sellist[1];
+	err = make_heckit_NA_mask(HC, reglist, sellist, (const double **) *pZ, 
+				  pdinfo);
+    }
+
     if (err) {
 	goto bailout;
     }
 
     /* run initial auxiliary probit */
-    probmod = logit_probit(Zlist, pZ, pdinfo, PROBIT, OPT_A, NULL);
+    probmod = logit_probit(sellist, pZ, pdinfo, PROBIT, OPT_A, NULL);
     if (probmod.errcode) {
 	hm.errcode = probmod.errcode;
 	goto bailout;
     }
 
+    HC->t1 = probmod.t1;
+    HC->t2 = probmod.t2;
+
 #if HDEBUG
     printmodel(&probmod, pdinfo, OPT_NONE, prn);
 #endif
@@ -1025,23 +1217,24 @@ static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
 	goto bailout;
     } 
 
+    HC->millsvar = pdinfo->v - 1;
+
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	sel = ((*pZ)[Zlist[1]][t] == 1.0);
-	if (sel) {
-	    (*pZ)[v][t] = probmod.uhat[t];
+	if ((*pZ)[HC->selvar][t] == 1.0) {
+	    (*pZ)[HC->millsvar][t] = probmod.uhat[t];
 	} else {
-	    (*pZ)[v][t] = NADBL;
+	    (*pZ)[HC->millsvar][t] = NADBL;
 	}
     }
 
-    strcpy(pdinfo->varname[v], "lambda");
-    gretl_list_append_term(&Xlist, v);
+    strcpy(pdinfo->varname[HC->millsvar], "lambda");
+    gretl_list_append_term(&reglist, HC->millsvar);
 
-    /* run OLS */
-    hm = lsq(Xlist, pZ, pdinfo, OLS, OPT_A);
+    /* run OLS including Mills variable */
+    hm = lsq(reglist, pZ, pdinfo, OLS, OPT_A);
     if (!hm.errcode) {
 	hm.ci = HECKIT;
-	gretl_model_set_int(&hm, "totobs", probmod.nobs);
+	gretl_model_set_int(&hm, "totobs", probmod.nobs); 
     }
 
 #if HDEBUG
@@ -1051,15 +1244,15 @@ static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
 
     if (!hm.errcode) {
 	/* fill the container with all the relevant data */
-	err = h_container_fill(HC, Xlist, Zlist, (const double **) *pZ, 
+	err = h_container_fill(HC, reglist, sellist, (const double **) *pZ, 
 			       pdinfo, &probmod, &hm);
     }
 
  bailout:
 
     clear_model(&probmod);
-    free(Xlist);
-    free(Zlist);
+    free(reglist);
+    free(sellist);
 
     if (err && !hm.errcode) {
 	hm.errcode = err;
@@ -1116,6 +1309,7 @@ MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
     } else {
 	err = transcribe_heckit_params(&hm, HC, pdinfo);
 	heckit_yhat_uhat(&hm, HC, (const double **) *pZ, pdinfo);
+	clear_model_xpx(&hm);
     }
 
     if (err && hm.errcode == 0) {
@@ -1132,5 +1326,14 @@ MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     h_container_destroy(HC);
 
+#if 0
+    int t;
+
+    for (t=hm.t1; t<=hm.t2; t++) {
+	fprintf(stderr, "%4d: u = %12.6f, llt = %12.6f, missing = %d\n", 
+		t, hm.uhat[t], hm.llt[t], model_missing(&hm, t));
+    }
+#endif
+
     return hm;
 }
