@@ -17,6 +17,10 @@
  * 
  */
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)
+# include "gtk_compat.h"
+#endif
+
 static void invalid_varname (PRN *prn)
 {
     pputs(prn, gretl_errmsg_get());
@@ -228,6 +232,7 @@ void infobox (const char *template, ...)
 				     GTK_DIALOG_DESTROY_WITH_PARENT,
 				     GTK_MESSAGE_INFO,
 				     GTK_BUTTONS_CLOSE,
+				     "%s",
 				     msg);
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -396,6 +401,11 @@ void add_sheets_list (GtkWidget *vbox, wbook *book)
     gtk_container_add (GTK_CONTAINER(sw), view); 
 }
 
+static void make_wmenu_modal (GtkWidget *w, gpointer p)
+{
+    gtk_window_set_modal(GTK_WINDOW(w), TRUE);
+}
+
 static void wsheet_menu (wbook *book, int multisheet)
 {
     GtkWidget *w, *tmp, *label;
@@ -410,8 +420,10 @@ static void wsheet_menu (wbook *book, int multisheet)
 			   G_CALLBACK(wsheet_menu_cancel), book);
     g_signal_connect(G_OBJECT(w), "destroy",  
 		     G_CALLBACK(gtk_main_quit), NULL);
+    g_signal_connect(G_OBJECT(w), "realize",
+		     G_CALLBACK(make_wmenu_modal), NULL);
 
-    vbox = GTK_DIALOG(w)->vbox;
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(w));
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
     /* selection of starting column and row */
@@ -466,7 +478,7 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 5);
 #endif
 
-    hbox = GTK_DIALOG(w)->action_area;
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(w));
     gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
     gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 10);
 
@@ -492,8 +504,6 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_entry_set_activates_default(GTK_ENTRY(book->rowspin), TRUE);
 
     gtk_widget_show_all(w);
-
-    gtk_window_set_modal(GTK_WINDOW(w), TRUE);
-
     gtk_main();
 }
+
