@@ -23,8 +23,6 @@
 
 #define INTDEBUG 0
 
-#define INTERVAL_TOL 1.0e-12
-
 enum {
     INT_LOW,   /* no lower bound */
     INT_MID,   /* both bounds */
@@ -674,7 +672,7 @@ static int intreg_normtest (int_container *IC, double *teststat)
 	gretl_vector_set(g, noc-1, kurt);
 	*teststat = gretl_scalar_qform(g, GG, &err);
     } else {
-	gretl_matrix_print(GG, "GG (inverse)");
+	gretl_matrix_print(condmom, "conditional moment matrix");
 	*teststat = NADBL;
     }
 
@@ -777,7 +775,7 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
 	pmod->opt |= OPT_R;
     }
 
-    if (IC->pmod->ifc) {
+    if (IC->pmod->ifc && IC->nx > 1) {
 	pmod->chisq = chisq_overall_test(IC);
     } else {
 	pmod->chisq = NADBL;
@@ -792,18 +790,20 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
 static int do_interval (int *list, double **Z, DATAINFO *pdinfo, 
 			MODEL *mod, gretlopt opt, PRN *prn) 
 {
-    int err;
     int_container *IC;
     gretl_matrix *V = NULL;
-    int fncount, grcount;
-    double normtest = NADBL;
+    int maxit, fncount, grcount;
+    double toler, normtest = NADBL;
+    int err = 0;
 
     IC = int_container_new(list, Z, pdinfo, mod);
     if (IC == NULL) {
 	return E_ALLOC;
     }
 
-    err = BFGS_max(IC->theta, IC->k, 1000, INTERVAL_TOL, 
+    BFGS_defaults(&maxit, &toler, INTREG);
+
+    err = BFGS_max(IC->theta, IC->k, maxit, toler, 
 		   &fncount, &grcount, int_loglik, C_LOGLIK,
 		   int_score, IC, opt & OPT_V, prn);
 
@@ -821,10 +821,11 @@ static int do_interval (int *list, double **Z, DATAINFO *pdinfo,
 
     if (!err) {
 	err = intreg_normtest(IC, &normtest);
-    }
-
-    if (!err) {
-	err = add_norm_test_to_model(IC->pmod, normtest);
+	if (!err) {
+	    err = add_norm_test_to_model(IC->pmod, normtest);
+	}
+	/* don't let this be a show-stopper */
+	err = 0;
     }
 
     gretl_matrix_free(V);
@@ -890,5 +891,7 @@ MODEL interval_estimate (int *list, double ***pZ, DATAINFO *pdinfo,
     /* do the actual analysis */
     model.errcode = do_interval(list, *pZ, pdinfo, &model, opt, prn);
 
+    clear_model_xpx(&model);
+
     return model;
 }
