@@ -63,22 +63,22 @@ const double s_mTrace_m_time[5][7] = {
 };
 
 const double s_mTrace_v_time[5][7] = {
-  {-0.204,   0.980,    3.11,   -2.14,  0.0499, -0.0103,-0.00902}, 
-  { 0.224,   0.863,    3.38,  -0.807,       0,       0, -0.0091}, 
-  { 0.422,   0.734,    3.76,    4.32,-0.00606,       0,-0.00718}, 
-  {     0,   0.836,    3.99,   -1.33,-0.00298,-0.00139,-0.00268}, 
-  { -1.29,    1.01,    3.92,    4.67, 0.00484,-0.00127, -0.0199}
+  {-0.204,   0.980,    3.11,   -2.14,    0.0499, -0.01030,  -0.00902}, 
+  { 0.224,   0.863,    3.38,   -0.807,        0,        0,  -0.0091}, 
+  { 0.422,   0.734,    3.76,    4.320, -0.00606,        0,  -0.00718}, 
+  { 0.000,   0.836,    3.99,   -1.330, -0.00298, -0.00139,  -0.00268}, 
+  { -1.29,   1.010,    3.92,    4.670,  0.00484, -0.00127,  -0.0199}
 };
 
 /* Matrices for the lambdamax test */
 
 const double s_mMaxev_m_coef[5][5] = {
-/*   n            1        n==1         n==2        n^1/2 */
-  {6.0019,     -2.7558,     0.67185,     0.11490,     -2.7764},  
-  {5.9498,     0.43402,    0.048360,    0.018198,     -2.3669},  
-  {5.8271,     -1.6487,     -1.6118,    -0.25949,     -1.5666},  
-  {5.8658,      2.5595,    -0.34443,   -0.077991,     -1.7552},  
-  {5.6364,    -0.90531,     -3.5166,    -0.47966,    -0.21447}
+/*   n            1         n==1       n==2        n^1/2 */
+  {6.0019,  -2.7558,    0.67185,    0.11490,    -2.7764},  
+  {5.9498,   0.43402,   0.048360,   0.018198,   -2.3669},  
+  {5.8271,  -1.6487,   -1.6118,    -0.25949,    -1.5666},  
+  {5.8658,   2.5595,   -0.34443,   -0.077991,   -1.7552},  
+  {5.6364,  -0.90531,  -3.5166,    -0.47966,    -0.21447}
 }; 
 
 const double s_mMaxev_v_coef[5][5] = {
@@ -105,7 +105,7 @@ gamma_par_asymp (double tracetest, double lmaxtest, JohansenCode det,
         J_REST_TREND   = restricted trend
         J_UNREST_TREND = unrestricted trend
       N: cointegration rank under H0;
-      pval: on output, array of pvalues, for the two tests;
+      pval: on output, array of pvalues for the two tests
     */
     
     double mt, vt, ml, vl;
@@ -232,9 +232,10 @@ static void print_beta_or_alpha (const GRETL_VAR *jvar, int k,
 }
 
 /* Calculate \alpha (adjustments) matrix as per Johansen, 1991, eqn
-   2.8, p. 1554.  Required for the cointegration test, but not
-   needed when doing a VECM (in which case we get \alpha via
-   VECM_estimate_full() below).
+   2.8, p. 1554.  Not needed when doing a VECM (without restrictions
+   on alpha), in which case we get \alpha via VECM_estimate_full()
+   below.  We use this to support verbose output when testing a
+   restriction on \beta.
 */
 
 static int compute_alpha (JohansenInfo *jv)
@@ -243,11 +244,13 @@ static int compute_alpha (JohansenInfo *jv)
     gretl_matrix *alpha = NULL;
     gretl_matrix *BSB = NULL;
     gretl_matrix *Tmp = NULL;
+    int p = jv->S01->rows;
+    int r = B->cols;
     int err = 0;
 
-    BSB = gretl_matrix_alloc(B->cols, B->cols);
-    Tmp = gretl_matrix_alloc(B->rows, B->cols);
-    alpha = gretl_matrix_alloc(jv->S01->rows, B->cols);
+    BSB = gretl_matrix_alloc(r, r);
+    Tmp = gretl_matrix_alloc(B->rows, r);
+    alpha = gretl_matrix_alloc(p, r);
 
     if (BSB == NULL || Tmp == NULL || alpha == NULL) {
 	err = E_ALLOC;
@@ -451,11 +454,14 @@ static void add_Ai_to_VAR_A (gretl_matrix *Ai, GRETL_VAR *vecm, int k)
 /* flags for controlling "full" estimation of VECM */
 
 enum {
-    NET_OUT_ALPHA  = 0,
-    ESTIMATE_ALPHA = 1 << 0,
+    NET_OUT_ALPHA  = 1 << 0,
     BOOTSTRAPPING  = 1 << 1
 };
 
+#define bootstrap(f)      (f & BOOTSTRAPPING)
+#define net_out_alpha(f)  (f & NET_OUT_ALPHA)
+#define estimate_alpha(f) (!(f & NET_OUT_ALPHA))
+
 /* write pre-computed ML alpha into model structs */
 
 static void transcribe_alpha (GRETL_VAR *v)
@@ -487,7 +493,7 @@ static int vecm_check_size (GRETL_VAR *v, int flags)
     int xc = (v->X != NULL)? v->X->cols : 0;
     int err = 0;
 
-    if (flags & BOOTSTRAPPING) {
+    if (bootstrap(flags)) {
 	/* in this case the matrices will already
 	   be fully allocated */
 	v->X->cols = v->ncoeff;
@@ -502,7 +508,7 @@ static int vecm_check_size (GRETL_VAR *v, int flags)
 
     v->ncoeff += jrank(v);
 
-    if (flags & ESTIMATE_ALPHA) {
+    if (estimate_alpha(flags)) {
 	xc += jrank(v);
     } else if (xc == 0) {
 	return 0;
@@ -608,7 +614,7 @@ static int make_vecm_Y (GRETL_VAR *v, const double **Z,
     double pij, yti, xti;
     int err = 0;
 
-    if (flags & ESTIMATE_ALPHA) {
+    if (estimate_alpha(flags)) {
 	/* "Y" is composed of plain DYt */
 	for (i=0; i<v->neqns; i++) {
 	    vi = v->ylist[i+1];
@@ -620,13 +626,12 @@ static int make_vecm_Y (GRETL_VAR *v, const double **Z,
 	}
     } else {
 	/* netting out \alpha: "Y" = DY_t - \Pi Y*_t */
-	int j, p1 = v->jinfo->Beta->rows;
+	int j, k, wexo, p1 = v->jinfo->Beta->rows;
 
 	form_Pi(v, Pi);
 
 	for (i=0; i<v->neqns; i++) {
-	    int wexo = 1;
-
+	    wexo = 1;
 	    vi = v->ylist[i+1];
 	    s = 0;
 	    for (t=v->t1; t<=v->t2; t++) {
@@ -643,8 +648,7 @@ static int make_vecm_Y (GRETL_VAR *v, const double **Z,
 			} else if (j == v->neqns && auto_restr(v)) {
 			    xti = (jcode(v) == J_REST_TREND)? t : 1;
 			} else {
-			    int k = j - v->ylist[0] - auto_restr(v) + 1;
-
+			    k = j - v->ylist[0] - auto_restr(v) + 1;
 			    vj = v->rlist[k];
 			    xti = Z[vj][t-1];
 			} 
@@ -663,111 +667,11 @@ static int make_vecm_Y (GRETL_VAR *v, const double **Z,
     return err;
 }
 
-#if 1
-
-static int eqn_is_unrestricted (const gretl_matrix *R, int j0, int r)
-{
-    int i, j;
-
-    for (j=j0; j<R->cols; j+=r) {
-	for (i=0; i<R->rows; i++) {
-	    if (gretl_matrix_get(R, i, j) != 0.0) {
-		return 0;
-	    }
-	}
-    }
-
-    return 1;
-}
-
-/* (Experimental) apparatus for getting correct (?) standard errors
-   for Gamma etc., in case we've estimated the models netting out
-   \Pi*X^*_{t-1}.  At present we attempt a correction only for
-   equations where the \alpha estimates are unrestricted; we recompute
-   the std errors using the full X'X matrix, including the
-   \beta'X^*_{t-1} column(s).  Is this right?
-   
-   Note: if the dependent variable in a given equation is weakly
-   exogenous, then netting out \Pi*X^*_{t-1} should be OK, with regard
-   to the standard errors for Gamma etc., shouldn't it?  Because in
-   that case the "missing" \beta'X^*_{t-1} column(s) are all zeros.
-*/
-
-static int 
-correct_variance (GRETL_VAR *v, const gretl_restriction *rset,
-		  const double **Z)
-{
-    const gretl_matrix *R;
-    gretl_matrix *X = NULL;
-    gretl_matrix *XTX = NULL;
-    double x;
-    int xc1, nse;
-    int r = jrank(v);
-    int i, j, err = 0;
-
-    R = rset_get_Ra_matrix(rset);
-    if (R == NULL) {
-	/* shouldn't be here */
-	return 0;
-    }
-
-#if JDEBUG
-    gretl_matrix_print(R, "Ra, in correct_variance()");
-#endif
-
-    nse = (v->X != NULL)? v->X->cols : 0;
-    xc1 = nse + jrank(v);
-
-    X = gretl_zero_matrix_new(v->T, xc1);
-    XTX = gretl_matrix_alloc(xc1, xc1);
-
-    if (X == NULL || XTX == NULL) {
-	err = E_ALLOC;
-    }
-
-    if (!err) {
-	/* copy original X cols and append ECs */
-	if (v->X != NULL) {
-	    gretl_matrix_inscribe_matrix(X, v->X, 0, 0, GRETL_MOD_NONE);
-	}
-	err = add_EC_terms_to_X(v, X, Z);
-    }
-
-    if (!err) {
-	/* form X'X and invert */
-	gretl_matrix_multiply_mod(X, GRETL_MOD_TRANSPOSE,
-				  X, GRETL_MOD_NONE,
-				  XTX, GRETL_MOD_NONE);
-	gretl_invert_symmetric_matrix(XTX);
-    }	
-
-    for (i=0; i<v->neqns && !err; i++) {
-	MODEL *pmod = v->models[i];
-	
-	if (eqn_is_unrestricted(R, i, r)) {
-	    for (j=0; j<nse; j++) {
-		x = gretl_matrix_get(XTX, j, j);
-		x = sqrt(x);
-		x *= pmod->sigma;
-#if JDEBUG
-		fprintf(stderr, "original se[%d,%d] = %g\n", i, j, pmod->sderr[j]);
-		fprintf(stderr, " revised se[%d,%d] = %g\n", i, j, x);
-#endif
-		pmod->sderr[j] = x;
-	    }
-	}
-    }
-
-    gretl_matrix_free(X);
-    gretl_matrix_free(XTX);
-
-    return err;
-}
-
-#endif
-
 /* As in PcGive: df = T - c, where c is the "average number of
    estimated parameters per equation, rounded towards zero".
+   Note that this function is _not_invoked in the case of
+   "general" restriction on alpha/beta; in that case the df
+   calculation is handled specially.
 */
 
 static void vecm_set_df (GRETL_VAR *v, const gretl_matrix *H,
@@ -776,7 +680,7 @@ static void vecm_set_df (GRETL_VAR *v, const gretl_matrix *H,
     int p = v->neqns;
     int r = v->jinfo->rank;
     int p1 = v->jinfo->Beta->rows;
-    int K = 0;
+    int Kpi, K = 0;
     double c;
 
     /* lagged differences */
@@ -795,25 +699,19 @@ static void vecm_set_df (GRETL_VAR *v, const gretl_matrix *H,
 
     K *= p;
 
-    /* free beta terms */
-    if (H == NULL) {
-	K += r * (p1 - r);
-    } else {
-	K += H->cols * r;
+    /* df loss in \Pi = \alpha \beta': see Johansen (1995) chapter 7 */
+    Kpi = (p + p1 - r) * r;
+    if (H != NULL || R != NULL) {
+	Kpi -= v->jinfo->lrdf;
     }
+    K += Kpi;
 
-    /* free alpha terms */
-    if (R == NULL) {
-	K += r * p; 
-    } else {
-	K += r * (p - R->rows);
-    }
-
-    c = floor(K / p);
-    v->df = v->T - c;
+    c = K / (double) p;
+    v->df = v->T - floor(c);
 
 #if JDEBUG
-    fprintf(stderr, "vecm_set_df: global K = %d, c = %g\n", K, c);
+    fprintf(stderr, "vecm_set_df: T = %d, global K = %d, c = %g, df = %d\n", 
+	    v->T, K, c, v->df);
 #endif
 }
 
@@ -832,7 +730,8 @@ static void fill_residuals_matrix (GRETL_VAR *v)
 }
 
 /* The following is designed to accommodate the case where alpha is
-   restricted, so we can't just run OLS conditional on beta.
+   restricted, in which case we can't just run OLS conditional on
+   beta.
 
    DY_t = \Pi Y*_{t-1} + \sum_{i=1}^{k-1}\Gamma_i DY_{t-1} + ...
 
@@ -849,25 +748,22 @@ VECM_estimate_full (GRETL_VAR *v, const gretl_restriction *rset,
 		    const double **Z, const DATAINFO *pdinfo,
 		    int flags)
 {
+    gretl_matrix *beta = v->jinfo->Beta;
     gretl_matrix *Pi = NULL;
-    gretl_matrix *XTX = NULL;
     gretl_matrix *Ai = NULL;
     gretl_matrix **G = NULL;
-
-    gretl_matrix *beta = v->jinfo->Beta;
-
     int order = v->order;
     int xc, n = v->neqns;
     int i, err;
 
 #if JDEBUG
     fprintf(stderr, "VECM_estimate_full: %s\n", 
-	    (flags & ESTIMATE_ALPHA)? "including alpha in estimation" :
+	    (estimate_alpha(flags))? "including alpha in estimation" :
 	    "netting out the EC terms");
 #endif
 
-    if (!(flags & ESTIMATE_ALPHA) && v->jinfo->Alpha == NULL) {
-	/* alpha must be pre-computed */
+    if (net_out_alpha(flags) && v->jinfo->Alpha == NULL) {
+	/* error: alpha must be pre-computed */
 	return E_DATA;
     }
 
@@ -895,20 +791,24 @@ VECM_estimate_full (GRETL_VAR *v, const gretl_restriction *rset,
 	err = make_vecm_Y(v, Z, Pi, flags);
     }
 
-    if (!err && (flags & ESTIMATE_ALPHA)) {
+    if (!err && estimate_alpha(flags)) {
 	err = add_EC_terms_to_X(v, v->X, Z);
     }
 
     if (!err) {
 	if (xc > 0) {
 	    /* run the regressions */
-	    if (flags & BOOTSTRAPPING) {
+	    if (bootstrap(flags)) {
 		err = gretl_matrix_multi_ols(v->Y, v->X, v->B, v->E, NULL);
 	    } else {
-		err = gretl_matrix_multi_SVD_ols(v->Y, v->X, v->B, v->E, &XTX);
+		if (v->XTX != NULL) {
+		    gretl_matrix_free(v->XTX);
+		    v->XTX = NULL;
+		}
+		err = gretl_matrix_multi_SVD_ols(v->Y, v->X, v->B, v->E, &v->XTX);
 	    }
 	} else {
-	    /* nothing to estimate, with alpha in hand */
+	    /* nothing to estimate, with alpha already in hand */
 	    fill_residuals_matrix(v);
 	}
     }
@@ -917,7 +817,7 @@ VECM_estimate_full (GRETL_VAR *v, const gretl_restriction *rset,
 	copy_coeffs_to_Gamma(v, G);
     }
 
-    if (!err && (flags & ESTIMATE_ALPHA)) {
+    if (!err && estimate_alpha(flags)) {
 	err = copy_to_alpha(v);
 	if (!err) {
 	    form_Pi(v, Pi);
@@ -961,24 +861,21 @@ VECM_estimate_full (GRETL_VAR *v, const gretl_restriction *rset,
     gretl_matrix_print(v->A, "vecm->A");
 #endif
 
-    if (!err && !(flags & BOOTSTRAPPING)) {
+    if (!err && !bootstrap(flags)) {
+	const gretl_matrix *XTX = NULL;
+
+	if (estimate_alpha(flags)) {
+	    XTX = v->XTX;
+	}
 	transcribe_VAR_models(v, Z, pdinfo, XTX);
-	if (flags == NET_OUT_ALPHA) {
+	if (net_out_alpha(flags)) {
 	    transcribe_alpha(v);
 	}
     }
 
-#if 1
-    /* experimental */
-    if (!err &&  flags == NET_OUT_ALPHA) {
-	err = correct_variance(v, rset, Z);
-    }
-#endif
-
  bailout:
     
     gretl_matrix_free(Pi);
-    gretl_matrix_free(XTX);
     gretl_matrix_free(Ai);
     gretl_matrix_array_free(G, order);
 
@@ -996,7 +893,7 @@ print_beta_and_alpha (const GRETL_VAR *jvar, gretl_matrix *evals, int h,
 {
     int i, err = 0;
 
-    pprintf(prn, "\n%s", _("eigenvalue"));
+    pputs(prn, _("eigenvalue"));
     for (i=0; i<h; i++) {
 	pprintf(prn, "%#12.5g ", evals->val[i]);
     }
@@ -1327,6 +1224,7 @@ static int beta_variance (GRETL_VAR *vecm)
 	goto bailout;
     }
 
+    /* note use of df rather than T here */
     gretl_matrix_divide_by_scalar(vecm->jinfo->Bvar, vecm->df);
 
     k = 0;
@@ -1381,18 +1279,11 @@ int johansen_ll_calc (GRETL_VAR *jvar, const gretl_matrix *evals)
 
 static int vecm_ll_stats (GRETL_VAR *vecm)
 {
-    gretl_matrix *S;
     int T = vecm->T;
     int g = vecm->neqns;
     int k = g * (vecm->order + 1); /* FIXME gappy */
 
-    S = gretl_matrix_copy(vecm->S);
-    if (S == NULL) {
-	return E_ALLOC;
-    } 
-
-    vecm->ldet = gretl_vcv_log_determinant(S);
-    gretl_matrix_free(S);
+    vecm->ldet = gretl_vcv_log_determinant(vecm->S);
 
     k += vecm->jinfo->seasonals;
 
@@ -1419,11 +1310,25 @@ static int vecm_ll_stats (GRETL_VAR *vecm)
     return 0;
 }
 
+static void coint_test_print_exog (const int *list, const DATAINFO *pdinfo,
+				   PRN *prn)
+{
+    int i, vi;
+
+    pprintf(prn, "\n%s: ", _("Exogenous regressor(s)"));
+    for (i=1; i<=list[0]; i++) {
+	vi = list[i];
+	pprintf(prn, "%s ", pdinfo->varname[vi]);
+    }
+}
+
 static int 
-compute_coint_test (GRETL_VAR *jvar, const gretl_matrix *evals, PRN *prn)
+compute_coint_test (GRETL_VAR *jvar, const gretl_matrix *evals, 
+		    const DATAINFO *pdinfo, PRN *prn)
 {
     int T = jvar->T;
     int n = jvar->neqns;
+    int nexo = 0;
     double cumeig = 0.0;
     double *lmax = NULL;
     double *trace = NULL;
@@ -1445,8 +1350,15 @@ compute_coint_test (GRETL_VAR *jvar, const gretl_matrix *evals, PRN *prn)
 	trace[i] = cumeig; 
     }
 
-    pputc(prn, '\n');
+    if (jvar->xlist != NULL) {
+	nexo = jvar->xlist[0];
+    }
+
     print_Johansen_test_case(jcode(jvar), prn);
+    if (nexo > 0) {
+	coint_test_print_exog(jvar->xlist, pdinfo, prn);
+    }
+    pputc(prn, '\n');
     pprintf(prn, "\n%s %s %s %s   %s  %s\n", _("Rank"), _("Eigenvalue"), 
 	    _("Trace test"), _("p-value"),
 	    _("Lmax test"), _("p-value"));	
@@ -1458,6 +1370,13 @@ compute_coint_test (GRETL_VAR *jvar, const gretl_matrix *evals, PRN *prn)
     }
     pputc(prn, '\n');
 
+    if (nexo > 0) {
+	pputs(prn, _("Note: in general, the test statistics above "
+		     "are valid only in the\nabsence of additional "
+		     "regressors."));
+	pputs(prn, "\n\n");
+    }
+
     free(lmax);
     free(trace);
 
@@ -1537,7 +1456,7 @@ int johansen_coint_test (GRETL_VAR *jvar, const DATAINFO *pdinfo,
 	pputs(prn, _("Failed to find eigenvalues\n"));
     } else {
 	johansen_ll_calc(jvar, evals);
-	compute_coint_test(jvar, evals, prn);
+	compute_coint_test(jvar, evals, pdinfo, prn);
 
 	if (!(opt & OPT_Q)) {
 	    print_beta_and_alpha(jvar, evals, p, pdinfo, prn);
@@ -1553,9 +1472,10 @@ int johansen_coint_test (GRETL_VAR *jvar, const DATAINFO *pdinfo,
 static void set_beta_test_df (GRETL_VAR *jvar, const gretl_matrix *H)
 {
     int r = jrank(jvar);
-    int nb = jvar->jinfo->Beta->rows;
+    int p1 = jvar->jinfo->Beta->rows;
+    int s = H->cols;
 
-    jvar->jinfo->lrdf = r * (nb - H->cols);
+    jvar->jinfo->lrdf = r * (p1 - s);
 }
 
 /* Likelihood ratio test calculation, for restriction on
@@ -1572,6 +1492,7 @@ johansen_LR_calc (const GRETL_VAR *jvar, const gretl_matrix *evals,
     double T_2 = (double) jvar->T / 2.0;
     int n = jvar->neqns;
     int r = jrank(jvar);
+    int s = H->cols;
     int h = (r > 0)? r : n;
     int i, err = 0;
 
@@ -1602,9 +1523,9 @@ johansen_LR_calc (const GRETL_VAR *jvar, const gretl_matrix *evals,
 	int df;
 
 	if (job == V_BETA) {
-	    df = h * (nb - H->cols);
+	    df = h * (nb - s);
 	} else {
-	    df = h * (n - H->cols);
+	    df = h * (n - s);
 	}
 
 	/* allow for possible prior restriction */
@@ -1613,6 +1534,7 @@ johansen_LR_calc (const GRETL_VAR *jvar, const gretl_matrix *evals,
 	pprintf(prn, _("Unrestricted loglikelihood (lu) = %.8g\n"), jvar->ll);
 	pprintf(prn, _("Restricted loglikelihood (lr) = %.8g\n"), llr);
 	pprintf(prn, "2 * (lu - lr) = %g\n", x);
+
 	if (df > 0) {
 	    double pv = chisq_cdf_comp(df, x);
 
@@ -1627,14 +1549,22 @@ johansen_LR_calc (const GRETL_VAR *jvar, const gretl_matrix *evals,
     return err;
 }
 
-static int johansen_prep_restriction (const GRETL_VAR *jvar, 
-				      const gretl_matrix *R,
-				      gretl_matrix **S01,
-				      gretl_matrix **S11,
-				      gretl_matrix **pH)
+/* see Johansen (1995), section 7.2: compute the p x s
+   direct restriction matrix H = R_\perp and form the 
+   following two intermediate matrices:
+
+   H' * S_{11} * H  in "S11" 
+   S_{01} * H       in "S01"
+*/
+
+static int prep_beta_restriction (const GRETL_VAR *jvar, 
+				  const gretl_matrix *R,
+				  gretl_matrix **S01,
+				  gretl_matrix **S11,
+				  gretl_matrix **pH)
 {
     gretl_matrix *H;
-    int m, n = jvar->neqns;
+    int s, p = jvar->neqns;
     int err = 0;
 
     if (R == NULL) {
@@ -1647,10 +1577,10 @@ static int johansen_prep_restriction (const GRETL_VAR *jvar,
     }
 
     *pH = H;
-    m = gretl_matrix_cols(H);
+    s = gretl_matrix_cols(H);
 
-    *S11 = gretl_matrix_alloc(m, m);
-    *S01 = gretl_matrix_alloc(n, m);
+    *S11 = gretl_matrix_alloc(s, s);
+    *S01 = gretl_matrix_alloc(p, s);
     if (*S11 == NULL || *S01 == NULL) {
 	return E_ALLOC;
     }
@@ -1668,13 +1598,14 @@ static int johansen_prep_restriction (const GRETL_VAR *jvar,
     return err;
 }
 
-/* test for homogeneous restriction, either for a rank-1 system 
-   or in common across the columns of beta (or alpha)
+/* test for presence of a homogeneous restriction, either for a rank-1
+   system or in common across the columns of beta (or alpha),
+   having already screened for the case where both beta and alpha
+   are restricted.
 */
 
-static int 
-simple_restriction (const GRETL_VAR *jvar,
-		    const gretl_restriction *rset)
+static int simple_restriction (const GRETL_VAR *jvar,
+			       const gretl_restriction *rset)
 {
     const gretl_matrix *R, *q;
     int rcols = jvar->neqns;
@@ -1700,20 +1631,28 @@ simple_restriction (const GRETL_VAR *jvar,
     return ret;
 }
 
+/* check whether the restriction on @jvar in @rset is
+   a simple restriction on \beta */
+
 static int simple_beta_restriction (const GRETL_VAR *jvar,
 				    const gretl_restriction *rset)
 {
     if (rset_VECM_acols(rset) > 0) {
+	/* not so if alpha is restricted */
 	return 0;
     } else {
 	return simple_restriction(jvar, rset);
     }
 }
 
+/* check whether the restriction on @jvar in @rset is
+   a simple restriction on \alpha */
+
 static int simple_alpha_restriction (const GRETL_VAR *jvar,
 				     const gretl_restriction *rset)
 {
     if (rset_VECM_bcols(rset) > 0) {
+	/* not so if beta is restricted */
 	return 0;
     } else {
 	return simple_restriction(jvar, rset);
@@ -1736,9 +1675,9 @@ static int j_general_restrict (GRETL_VAR *jvar,
     err = general_vecm_analysis(jvar, rset, pdinfo, prn);
 
     if (!err) {
-	int flag = (acols > 0)? NET_OUT_ALPHA : ESTIMATE_ALPHA;
+	int flags = (acols > 0)? NET_OUT_ALPHA : 0;
 
-	err = VECM_estimate_full(jvar, rset, Z, pdinfo, flag);
+	err = VECM_estimate_full(jvar, rset, Z, pdinfo, flags);
     }
 
     if (!err) {
@@ -1853,19 +1792,21 @@ static int get_unrestricted_ll (GRETL_VAR *jvar)
 
 static int vecm_finalize (GRETL_VAR *jvar, gretl_matrix *H, 
 			  const gretl_matrix *Ra,
-			  const double **Z, const DATAINFO *pdinfo,
-			  int flags)
+			  const double **Z, const DATAINFO *pdinfo)
 {
     int do_stderrs = jrank(jvar) < jvar->neqns;
     int err = 0;
 
-    if (flags & ESTIMATE_ALPHA) {
+    if (Ra == NULL) {
+	/* alpha unrestricted */
 	err = normalize_beta(jvar, H, &do_stderrs);
     } else {
 	do_stderrs = 0;
     }
 
     if (!err) {
+	int flags = (Ra != NULL)? NET_OUT_ALPHA : 0;
+
 	vecm_set_df(jvar, H, Ra);
 	err = VECM_estimate_full(jvar, NULL, Z, pdinfo, flags);
     }
@@ -1916,7 +1857,7 @@ est_simple_alpha_restr (GRETL_VAR *jvar,
     }
 
     if (!err) {
-	err = vecm_finalize(jvar, NULL, R, Z, pdinfo, NET_OUT_ALPHA);
+	err = vecm_finalize(jvar, NULL, R, Z, pdinfo);
     }
 
     if (!err) {
@@ -1954,7 +1895,7 @@ est_simple_beta_restr (GRETL_VAR *jvar,
     
     if (!err) {
 	R = rset_get_R_matrix(rset);
-	err = johansen_prep_restriction(jvar, R, &S01, &S11, &H);
+	err = prep_beta_restriction(jvar, R, &S01, &S11, &H);
     }
     
     if (!err) {
@@ -1983,7 +1924,7 @@ est_simple_beta_restr (GRETL_VAR *jvar,
     }
 
     if (!err) {
-	err = vecm_finalize(jvar, H, NULL, Z, pdinfo, ESTIMATE_ALPHA);
+	err = vecm_finalize(jvar, H, NULL, Z, pdinfo);
     }
 
     if (!err) {
@@ -2029,7 +1970,7 @@ j_estimate_unrestr (GRETL_VAR *jvar,
     }
 
 #if JDEBUG
-    gretl_matrix_print(jvar->jinfo->Beta, "raw eigenvector(s)");
+    gretl_matrix_print(jvar->jinfo->Beta, "raw Beta");
 #endif
 
     if (!err) {
@@ -2037,7 +1978,7 @@ j_estimate_unrestr (GRETL_VAR *jvar,
     }
 
     if (!err) {
-	err = vecm_finalize(jvar, NULL, NULL, Z, pdinfo, ESTIMATE_ALPHA);
+	err = vecm_finalize(jvar, NULL, NULL, Z, pdinfo);
     }
 
     gretl_matrix_free(S00);
@@ -2151,7 +2092,7 @@ johansen_boot_round (GRETL_VAR *jvar, const double **Z,
 	err = normalize_beta(jvar, NULL, NULL); 
 	if (!err) {
 	    err = VECM_estimate_full(jvar, NULL, Z, pdinfo, 
-				     ESTIMATE_ALPHA | BOOTSTRAPPING);
+				     BOOTSTRAPPING);
 	}
 	if (!err) {
 	    err = compute_omega(jvar);
@@ -2230,7 +2171,7 @@ static int vecm_beta_test (GRETL_VAR *jvar,
     gretl_matrix *S00 = NULL;
     gretl_matrix *evals = NULL;
     int verbose = (opt & OPT_V);
-    int m, n, rank;
+    int s, p, r;
     int err = 0;
 
     R = rset_get_R_matrix(rset);
@@ -2240,12 +2181,12 @@ static int vecm_beta_test (GRETL_VAR *jvar,
 	return err;
     }
 
-    n = jvar->neqns;
-    rank = jrank(jvar);
-    m = gretl_matrix_cols(H);
+    p = jvar->neqns;
+    r = jrank(jvar);
+    s = gretl_matrix_cols(H);
 
-    S11 = gretl_matrix_alloc(m, m);
-    S01 = gretl_matrix_alloc(n, m);
+    S11 = gretl_matrix_alloc(s, s);
+    S01 = gretl_matrix_alloc(p, s);
     S00 = gretl_matrix_copy(jvar->jinfo->S00);
 
     if (S11 == NULL || S01 == NULL || S00 == NULL) {
@@ -2279,7 +2220,7 @@ static int vecm_beta_test (GRETL_VAR *jvar,
 
     if (!err) {
 	err = johansen_get_eigenvalues(S00, S01, S11, 
-				       &M, &evals, rank);
+				       &M, &evals, r);
     }
 
     if (!err) {
@@ -2324,6 +2265,7 @@ int vecm_test_restriction (GRETL_VAR *jvar,
 
     B0 = gretl_matrix_copy(jvar->jinfo->Beta);
     A0 = gretl_matrix_copy(jvar->jinfo->Alpha);
+
     if (B0 == NULL || A0 == NULL) {
 	return E_ALLOC;
     }
