@@ -914,17 +914,14 @@ static void restricted_vecm_set_df (Jwrap *J, GRETL_VAR *v)
 
     K *= J->p;
 
-    /* free beta terms */
-    K += J->blen;
+    /* add number of free beta and alpha terms */
+    K += J->blen + J->alen;
 
-    /* free alpha terms */
-    K += J->alen;
-
-    c = floor(K / J->p);
-    v->df = v->T - c;
+    c = K / (double) J->p;
+    v->df = v->T - floor(c);
 
 #if JDEBUG
-    fprintf(stderr, "global K = %d, c = %g\n", K, c);
+    fprintf(stderr, "T = %d, K = %d, c = %g, df = %d\n", v->T, K, c, v->df);
 #endif
 }
 
@@ -1732,6 +1729,11 @@ static int check_jacobian (Jwrap *J)
     if (J->H != NULL) {
 	gretl_matrix *phi = gretl_column_vector_alloc(J->H->cols);
 
+	if (phi == NULL) {
+	    err = E_ALLOC;
+	    goto bailout;
+	}
+
 	gretl_matrix_random_fill(phi, D_NORMAL);
 	gretl_matrix_reuse(J->beta, J->p1 * J->r, 1);
 	gretl_matrix_multiply(J->H, phi, J->beta);
@@ -1748,6 +1750,11 @@ static int check_jacobian (Jwrap *J)
 	gretl_matrix *psi = gretl_column_vector_alloc(J->G->cols);
 	gretl_matrix *avec = gretl_column_vector_alloc(J->p * J->r);
 
+	if (psi == NULL || avec == NULL) {
+	    err = E_ALLOC;
+	    goto bailout;
+	}	
+
 	gretl_matrix_random_fill(psi, D_NORMAL);
 	gretl_matrix_multiply(J->G, psi, avec);
 	unvec_transpose(J->alpha, avec);
@@ -1796,6 +1803,8 @@ static int check_jacobian (Jwrap *J)
 #endif
     }
 
+ bailout:
+
     gretl_matrix_free(A);
     gretl_matrix_free(B);
     gretl_matrix_free(Jac);
@@ -1805,8 +1814,7 @@ static int check_jacobian (Jwrap *J)
 
 /* Doornik's approach to checking identification */
 
-static int 
-vecm_id_check (Jwrap *J, GRETL_VAR *jvar, PRN *prn)
+static int vecm_id_check (Jwrap *J, GRETL_VAR *jvar, PRN *prn)
 {
     int npar = J->alen + J->blen;
     int err = 0;
@@ -3073,18 +3081,16 @@ static void transcribe_to_jvar (Jwrap *J, GRETL_VAR *jvar)
 
 static int do_bfgs (Jwrap *J, gretlopt opt, PRN *prn)
 {
-    int maxit = 4000;
-    double reltol = 1.0e-11;
-    int fncount = 0;
-    int grcount = 0;
-    int nn = J->theta->rows;
-    int err = 0;
-
-    pputs(prn, "LBFGS: using analytical derivatives\n\n");
-    err = add_gradhelper(J);
+    int err = add_gradhelper(J);
 
     if (!err) {
-	err = LBFGS_max(J->theta->val, nn, maxit, reltol, 
+	int maxit = 4000, fncount = 0, grcount = 0;
+	int nn = J->theta->rows;
+	double toler = 1.0e-11;
+
+	pputs(prn, "LBFGS: using analytical derivatives\n\n");
+
+	err = LBFGS_max(J->theta->val, nn, maxit, toler, 
 			&fncount, &grcount, Jloglik, C_LOGLIK,
 			Jgradient, J, opt, prn);
     }
