@@ -234,10 +234,9 @@ static void rq_transcribe_results (MODEL *pmod,
     /* sum of absolute residuals */
     gretl_model_set_double(pmod, "ladsum", SAR);
 
-    /* invalidate ess-based stats */
-    pmod->rsq = NADBL;
-    pmod->adjrsq = NADBL;
-    pmod->fstt = NADBL;
+    /* invalidate unused stats */
+    pmod->rsq = pmod->adjrsq = NADBL;
+    pmod->fstt = pmod->chisq = NADBL;
 
     /* LaPlace errors: equivalent of standard error is sum of
        absolute residuals over nobs */
@@ -1423,7 +1422,10 @@ int rq_driver (const char *parm, MODEL *pmod,
 	gretl_model_set_int(pmod, "rq", 1);
 	if (opt & OPT_R) {
 	    gretl_model_set_int(pmod, "rq_nid", 1);
+	    pmod->opt |= OPT_R;
 	}
+	gretl_model_set_vcv_info(pmod, VCV_RQ, (opt & OPT_R)?
+				 RQ_NID : RQ_ASY);
     }
 
     gretl_matrix_free(y);
