@@ -1,10 +1,10 @@
 /*
-   Reader for Stata .dta files, versions 8.0, 7.0, 7/SE, 6.0 and 5.0.
+   Reader for Stata .dta files, versions 5.0 to 11.
 
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.34 2008/12/08 19:13:09 allin Exp $
+     * $Id: stata_import.c,v 1.39 2009/09/30 17:27:44 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -21,7 +21,9 @@
      Versions of Stata before 4.0 used different file formats.
 
   This version was fairly substantially modified for gretl 
-  by Allin Cottrell, July 2005.
+  by Allin Cottrell, July 2005, then modified again in August
+  2009 to support "format 114" .dta files as written by Stata
+  10 and 11.
 */
 
 #include <stdio.h>
@@ -46,6 +48,7 @@
 #define VERSION_7   0x6e
 #define VERSION_7SE  111
 #define VERSION_8    113
+#define VERSION_10   114
 
 /* Stata format constants */
 #define STATA_STRINGOFFSET 0x7f
@@ -89,6 +92,7 @@
 
 /* it's convenient to have these as file-scope globals */
 static int stata_version;
+static int stata_SE;
 static int stata_endian;
 static int swapends;
 
@@ -127,12 +131,8 @@ static int stata_read_signed_byte (FILE *fp, int naok, int *err)
     } else {
 	ret = (int) b;
 
-	if (!naok) {
-	    int v = abs(stata_version);
-
-	    if (STATA_BYTE_NA(b, v)) {
-		ret = NA_INT;
-	    }
+	if (!naok && STATA_BYTE_NA(b, stata_version)) {
+	    ret = NA_INT;
 	}
     }
 
@@ -217,8 +217,6 @@ stata_get_version_and_namelen (unsigned char u, int *vnamelen)
 {
     int err = 0;
 
-    /* a negative value of stata_version indicates "SE" */
-
     switch (u) {
     case VERSION_5:
         stata_version = 5;
@@ -233,11 +231,18 @@ stata_get_version_and_namelen (unsigned char u, int *vnamelen)
 	*vnamelen = 32;
 	break;
     case VERSION_7SE:
-	stata_version = -7;
+	stata_version = 7;
+	stata_SE = 1;
 	*vnamelen = 32; 
 	break;
     case VERSION_8:
-	stata_version = -8;  /* version 8 automatically uses SE format */
+	stata_version = 8; /* versions >= 8 automatically use 'SE' format */
+	stata_SE = 1;
+	*vnamelen = 32; 
+	break;
+    case VERSION_10:
+	stata_version = 10;
+	stata_SE = 1;
 	*vnamelen = 32; 
 	break;
     default:
@@ -254,12 +259,12 @@ static int stata_get_endianness (FILE *fp, int *err)
     return (i == 0x01)? G_BIG_ENDIAN : G_LITTLE_ENDIAN;
 }
 
-#define stata_type_float(t)  ((stata_version > 0 && t == STATA_FLOAT) || t == STATA_SE_FLOAT)
-#define stata_type_double(t) ((stata_version > 0 && t == STATA_DOUBLE) || t == STATA_SE_DOUBLE)
-#define stata_type_long(t)   ((stata_version > 0 && t == STATA_LONG) || t == STATA_SE_LONG)
-#define stata_type_int(t)    ((stata_version > 0 && t == STATA_INT) || t == STATA_SE_INT)
-#define stata_type_byte(t)   ((stata_version > 0 && t == STATA_BYTE) || t == STATA_SE_BYTE)
-#define stata_type_string(t) ((stata_version > 0 && t >= STATA_STRINGOFFSET) || t <= 244)
+#define stata_type_float(t)  ((stata_SE && t == STATA_SE_FLOAT)  || t == STATA_FLOAT)
+#define stata_type_double(t) ((stata_SE && t == STATA_SE_DOUBLE) || t == STATA_DOUBLE)
+#define stata_type_long(t)   ((stata_SE && t == STATA_SE_LONG)   || t == STATA_LONG)
+#define stata_type_int(t)    ((stata_SE && t == STATA_SE_INT)    || t == STATA_INT)
+#define stata_type_byte(t)   ((stata_SE && t == STATA_SE_BYTE)   || t == STATA_BYTE)
+#define stata_type_string(t) ((stata_SE && t <= 244) || t >= STATA_STRINGOFFSET)
 
 static int check_variable_types (FILE *fp, int *types, int nvar, int *nsv)
 {
@@ -324,23 +329,57 @@ dta_make_string_table (int *types, int nvar, int ncols)
     return st;
 }
 
-static int 
-save_dataset_info (DATAINFO *dinfo, const char *s1, const char *s2)
+static gchar *recode_stata_string (const char *s)
 {
-    int len = strlen(s1) + strlen(s2) + 2;
-    int err = 0;
+    const gchar *cset;
+    gchar *tr = NULL;
+    gsize bw;
 
-    dinfo->descrip = malloc(len);
-    if (dinfo->descrip != NULL) {
-	*dinfo->descrip = '\0';
-	strcat(dinfo->descrip, s1);
-	strcat(dinfo->descrip, "\n");
-	strcat(dinfo->descrip, s2);
-    } else {
-	err = 1;
+    if (!g_get_charset(&cset)) {
+	/* try recoding from current locale */
+	tr = g_locale_to_utf8(s, -1, NULL, &bw, NULL);
     }
 
-    return err;
+    if (tr == NULL) {
+	/* wild guess: try Windows CP 1252? */
+	tr = g_convert(s, -1, "UTF-8", "CP1252", NULL, &bw, NULL);
+    }
+
+    return tr;
+}
+
+static void
+save_dataset_info (DATAINFO *dinfo, const char *label, const char *stamp)
+{
+    int dlen = strlen(stamp);
+    gchar *tr = NULL;
+
+    if (*label != '\0') {
+	if (g_utf8_validate(label, -1, NULL)) {
+	    tr = g_strdup(label);
+	} else {
+	    tr = recode_stata_string(label);
+	}
+    }
+
+    if (tr != NULL) {
+	dlen += strlen(tr);
+    }
+
+    if (dlen > 0) {
+	dinfo->descrip = malloc(dlen + 2);
+    }
+
+    if (dinfo->descrip != NULL) {
+	*dinfo->descrip = '\0';
+	if (tr != NULL) {
+	    strcat(dinfo->descrip, tr);
+	    strcat(dinfo->descrip, "\n");
+	} 
+	strcat(dinfo->descrip, stamp);
+    } 
+
+    g_free(tr);
 }
 
 static int try_fix_varname (char *name)
@@ -474,9 +513,10 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 {
     int i, j, t, clen;
     int labellen, nlabels, totlen;
+    int fmtlen;
     int nvar = dinfo->v - 1, nsv = 0;
     int soffset, pd = 0, tnum = -1;
-    char datalabel[81], c18[18], aname[33];
+    char label[81], c50[50], aname[33];
     int *types = NULL;
     int *lvars = NULL;
     char **lnames = NULL;
@@ -487,21 +527,22 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
     int err = 0;
 
     labellen = (stata_version == 5)? 32 : 81;
-    soffset = (stata_version > 0)? STATA_STRINGOFFSET : STATA_SE_STRINGOFFSET;
+    fmtlen = (stata_version < 10)? 12 : 49;
+    soffset = (stata_SE)? STATA_SE_STRINGOFFSET : STATA_STRINGOFFSET;
     *nvread = nvar;
 
     printf("Max length of labels = %d\n", labellen);
 
-    /* data label - zero terminated string */
-    stata_read_string(fp, labellen, datalabel, &err);
-    printf("datalabel: '%s'\n", datalabel);
+    /* dataset label - zero terminated string */
+    stata_read_string(fp, labellen, label, &err);
+    printf("dataset label: '%s'\n", label);
 
     /* file creation time - zero terminated string */
-    stata_read_string(fp, 18, c18, &err);  
-    printf("timestamp: '%s'\n", c18);
+    stata_read_string(fp, 18, c50, &err);  
+    printf("timestamp: '%s'\n", c50);
 
-    if (*datalabel != '\0' || *c18 != '\0') {
-	save_dataset_info(dinfo, datalabel, c18);
+    if (*label != '\0' || *c50 != '\0') {
+	save_dataset_info(dinfo, label, c50);
     }
   
     /** read variable descriptors **/
@@ -542,16 +583,16 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
     
     /* format list (use it to identify date variables?) */
     for (i=0; i<nvar && !err; i++){
-        stata_read_string(fp, 12, c18, &err);
-	if (*c18 != '\0' && c18[strlen(c18)-1] != 'g') {
-	    printf("variable %d: format = '%s'\n", i+1, c18);
-	    if (!strcmp(c18, "%tm")) {
+        stata_read_string(fp, fmtlen, c50, &err);
+	if (*c50 != '\0' && c50[strlen(c50)-1] != 'g') {
+	    printf("variable %d: format = '%s'\n", i+1, c50);
+	    if (!strcmp(c50, "%tm")) {
 		pd = 12;
 		tnum = i;
-	    } else if (!strcmp(c18, "%tq")) {
+	    } else if (!strcmp(c50, "%tq")) {
 		pd = 4;
 		tnum = i;
-	    } else if (!strcmp(c18, "%ty")) {
+	    } else if (!strcmp(c50, "%ty")) {
 		pd = 1;
 		tnum = i;
 	    }
@@ -570,28 +611,26 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 
     /* variable descriptive labels */
     for (i=0; i<nvar && !err; i++) {
-	stata_read_string(fp, labellen, datalabel, &err);
-	if (*datalabel != '\0') {
-	    printf("variable %d: label = '%s'\n", i+1, datalabel);
-	    if (!g_utf8_validate(datalabel, -1, NULL)) {
-		gsize b;
-		gchar *tr = g_locale_to_utf8(datalabel, -1, NULL,
-					     &b, NULL); 
+	stata_read_string(fp, labellen, label, &err);
+	if (*label != '\0') {
+	    printf("variable %d: label = '%s'\n", i+1, label);
+	    if (g_utf8_validate(label, -1, NULL)) {
+		strncat(VARLABEL(dinfo, i+1), label, MAXLABEL - 1);
+	    } else {
+		gchar *tr = recode_stata_string(label);
 
 		if (tr != NULL) {
 		    strncat(VARLABEL(dinfo, i+1), tr, MAXLABEL - 1);
 		    g_free(tr);
 		}
-	    } else {
-		strncat(VARLABEL(dinfo, i+1), datalabel, MAXLABEL - 1);
-	    }
+	    } 
 	}
     }
 
     /* variable 'characteristics' -- not handled */
     if (!err) {
 	while (stata_read_byte(fp, &err)) {
-	    if (abs(stata_version) >= 7) { /* manual is wrong here */
+	    if (stata_version >= 7) { /* manual is wrong here */
 		clen = stata_read_long(fp, 1, &err);
 	    } else {
 		clen = stata_read_int(fp, 1, &err);
@@ -600,7 +639,7 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 		stata_read_signed_byte(fp, 1, &err);
 	    }
 	}
-	if (abs(stata_version) >= 7) {
+	if (stata_version >= 7) {
 	    clen = stata_read_long(fp, 1, &err);
 	} else {
 	    clen = stata_read_int(fp, 1, &err);
@@ -658,8 +697,9 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 
     /* value labels */
 
-    if (!err && !st_err && lvars != NULL && abs(stata_version) > 5) {
+    if (!err && !st_err && lvars != NULL && stata_version > 5) {
 	PRN *st_prn = NULL;
+	const char *vlabel;
 	double *level;
 	
 	for (j=0; j<nvar; j++) {
@@ -725,9 +765,22 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 	    }	    
 
 	    stata_read_string(fp, totlen, txt, &err);
+
 	    for (i=0; i<nlabels; i++) {
-		printf(" label %d = '%s'\n", i, txt + off[i]);
-		pprintf(st_prn, "%10g -> '%s'\n", level[i], txt + off[i]);
+		vlabel = txt + off[i];
+		printf(" label %d = '%s'\n", i, vlabel);
+		if (g_utf8_validate(vlabel, -1, NULL)) {
+		    pprintf(st_prn, "%10g -> '%s'\n", level[i], vlabel);
+		} else {
+		    gchar *tr = recode_stata_string(vlabel);
+
+		    if (tr != NULL) {
+			pprintf(st_prn, "%10g -> '%s'\n", level[i], tr);
+			g_free(tr);
+		    } else {
+			pprintf(st_prn, "%10g -> 'unknown'\n", level[i]);
+		    }
+		}
 	    }
 
 	    free(off);
@@ -758,18 +811,18 @@ static int parse_dta_header (FILE *fp, int *namelen, int *nvar, int *nobs)
     unsigned char u;
     int err = 0;
     
-    u = stata_read_byte(fp, &err);   /* release version */
+    u = stata_read_byte(fp, &err); /* release version */
 
     if (!err) {
 	err = stata_get_version_and_namelen(u, namelen);
     }
 
     if (err) {
-	fputs("not a Stata version 5-8 .dta file\n", stderr);
+	fputs("not a Stata version 5-11 .dta file\n", stderr);
 	return err;
     } 
 
-    printf("Stata file version %d\n", abs(stata_version));
+    printf("Stata file version %d\n", stata_version);
 
     /* these are file-scope globals */
     stata_endian = stata_get_endianness(fp, &err);
@@ -798,9 +851,8 @@ int dta_get_data (const char *fname,
 		  double ***pZ, DATAINFO *pdinfo,
 		  gretlopt opt, PRN *prn)
 {
-    int namelen = 0;
-    int nvar = 0, nobs = 0;
-    int nvread = 0;
+    int namelen = 0, nobs = 0;
+    int nvar = 0, nvread = 0;
     FILE *fp;
     double **newZ = NULL;
     DATAINFO *newinfo = NULL;
@@ -833,7 +885,6 @@ int dta_get_data (const char *fname,
 
     newinfo->v = nvar + 1;
     newinfo->n = nobs;
-    /* time-series info?? */
 
     err = start_new_Z(&newZ, newinfo, 0);
     if (err) {
@@ -851,6 +902,7 @@ int dta_get_data (const char *fname,
 	    gretl_string_table_destroy(st);
 	}	
     } else {
+	int merge = (*pZ != NULL);
 	int nvtarg = newinfo->v - 1;
 
 	if (nvread < nvtarg) {
@@ -867,14 +919,13 @@ int dta_get_data (const char *fname,
 	}
 
 	err = merge_or_replace_data(pZ, pdinfo, &newZ, &newinfo, opt, prn);
+    
+	if (!err && !merge) {
+	    dataset_add_import_info(pdinfo, fname, GRETL_DTA);
+	}
     }
 
     fclose(fp);
 
     return err;
 }  
-
-
-
-
-
